<?php

namespace application\personal_calendar;

use common\libraries\CommonDataManager;
use common\libraries\Translation;
use repository\content_object\event\Appointment;

/**
 * This personal calendar connector allows the personal calendar to retrieve the
 * calendar events for appointments (as a subscriber to an event) from the appointment tool/application
 */
class PersonalCalendarAppointmentsSubscriberConnector implements PersonalCalendarConnector
{
    public function is_active()
    {
            $registration = CommonDataManager :: get_registration('\repository\content_object\event\Event');
            if (! $registration || ! $registration->is_active())
            {
                return false;
            }
            else
            {
                return true;
            }

    }

    public function get_events($user, $from_date, $to_date)
    {
         $appointments = Appointment::retrieve_appointments_for_user_as_subscriber($user->get_id());

        $result = array();
        while ($appointment = $appointments->next_result())
        {
            $event = new PersonalCalendarEvent();
            $event->set_start_date($appointment->get_appointment_start_date());
            $event->set_end_date($appointment->get_appointment_end_date());

//            $url = $parent->get_url(array(Application :: PARAM_ACTION => PersonalCalendarManager::ACTION_VIEW_APPOINTMENT, PersonalCalendarManager :: PARAM_APPOINTMENT_ID => $appointment->get_id()));
//            $event->set_url($url);

            $title = $appointment->get_event()->get_title();

            $event->set_title($title);
            $event->set_content($appointment->get_calendar_description());

            $event->set_id($appointment->get_id());
            $courses = $appointment->get_courses_where_published();

            //user appointments in an event published in different courses are displayed separately under the course name as a legend
            for ($i = 0; $i < sizeof($courses); $i++)
            {
                if ($i > 0)
                {
                    $new_event = clone $event;
                    $new_event->set_source(Translation::get('AppointmentsAsSubscriber') . $courses[$i]->get_title());
                    $result[] = $new_event;
                }
                else
                {
                     $event->set_source(Translation::get('AppointmentsAsSubscriber'). ' ' . $courses[$i]->get_title());
                     $result[] = $event;
                }
            }

        }
        return $result;

    }



//    /**
//     * @see PersonalCalendarConnector
//     */
//    public function get_events($user, $from_date, $to_date, $parent)
//    {
//
//        $appointments = Appointment::retrieve_appointments_for_user_as_subscriber($user->get_id());
//
//        $result = array();
//        while ($appointment = $appointments->next_result())
//        {
//            $event = new PersonalCalendarEvent();
//            $event->set_start_date($appointment->get_appointment_start_date());
//            $event->set_end_date($appointment->get_appointment_end_date());
//
//            $url = $parent->get_url(array(Application :: PARAM_ACTION => PersonalCalendarManager::ACTION_VIEW_APPOINTMENT, PersonalCalendarManager :: PARAM_APPOINTMENT_ID => $appointment->get_id()));
//            $event->set_url($url);
//
//            $title = $appointment->get_event()->get_title();
//
//            $event->set_title($title);
//            $event->set_content($appointment->get_calendar_description());
//
//            $event->set_id($appointment->get_id());
//            $courses = $appointment->get_courses_where_published();
//
//            //user appointments in an event published in different courses are displayed separately under the course name as a legend
//            for ($i = 0; $i < sizeof($courses); $i++)
//            {
//                if ($i > 0)
//                {
//                    $new_event = clone $event;
//                    $new_event->set_source(Translation::get('AppointmentsAsSubscriber') . $courses[$i]->get_name());
//                    $result[] = $new_event;
//                }
//                else
//                {
//                     $event->set_source(Translation::get('AppointmentsAsSubscriber') . $courses[$i]->get_name());
//                     $result[] = $event;
//                }
//            }
//
//        }
//        return $result;
//    }

}
?>