<?php

namespace application\personal_calendar;

interface PersonalCalendarDataManagerInterface
{

    /**
     * Initializes the data manager.
     */
    public function initialize();

    /**
     * Creates a storage unit in the personal calendar storage system
     *
     * @param $name string
     * @param $properties array
     * @param $indexes array
     */
    public function create_storage_unit($name, $properties, $indexes);

    /**
     *
     * @see Application::content_object_is_published()
     */
    public function content_object_is_published($object_id);

    /**
     *
     * @see Application::any_content_object_is_published()
     */
    public function any_content_object_is_published($object_ids);

    /**
     *
     * @see Application::get_content_object_publication_attributes()
     */
    public function get_content_object_publication_attributes(
            $object_id, $type = null, $condition = null,
            $count = null, $offset = null, $order_properties = null);

    /**
     *
     * @see Application::get_content_object_publication_attribute()
     */
    public function get_content_object_publication_attribute($publication_id);

    /**
     *
     * @see Application::count_publication_attributes()
     */
    public function count_publication_attributes($user = null, $object_id = null, $condition = null);

    /**
     *
     * @see Application::delete_content_object_publications()
     */
    public function delete_content_object_publications($object_id);

    /**
     *
     * @see Application::update_content_object_publication_id()
     */
    public function update_content_object_publication_id($publication_attr);

    /**
     * Retrieve a profile publication
     *
     * @param $id int
     * @return ProfilerPublication
     */
    public function retrieve_personal_calendar_publication($id);

    /**
     * Retrieve a series of profile publications
     *
     * @param $condition Condition
     * @param $order_by array
     * @param $offset int
     * @param $max_objects int
     * @return ProfilePublicationResultSet
     */
    public function retrieve_personal_calendar_publications($condition = null, $order_by = array (), $offset = 0, $max_objects = -1);

    /**
     * Update the publication
     *
     * @param $profile_publication ProfilerPublication
     * @return boolean
     */
    public function update_personal_calendar_publication($publication);

    /**
     * Delete the publication
     *
     * @param $profile_publication ProfilerPublication
     * @return boolean
     */
    public function delete_personal_calendar_publication($publication);

    /**
     * Delete the publications
     *
     * @param $object_id Array
     *            An array of publication ids
     * @return boolean
     */
    public function delete_personal_calendar_publications($object_id);

    /**
     * Update the publication id
     *
     * @param $publication_attr ContentObjectPublicationAttribure
     * @return boolean
     */
    public function update_personal_calendar_publication_id($publication_attr);

    /**
     * Create a publication
     *
     * @param $publication PersonalMessagePublication
     * @return boolean
     */
    public function create_personal_calendar_publication($publication);

    public function retrieve_personal_calendar_publication_target_groups($calendar_event_publication);

    public function retrieve_personal_calendar_publication_target_users($calendar_event_publication);

}
