<?php
namespace application\personal_calendar;

use common\libraries\Display;
use repository\ContentObjectRendition;
use repository\ContentObjectRenditionImplementation;
use common\libraries\Request;
use common\libraries\Application;
use common\libraries\Translation;
use common\libraries\ParameterNotDefinedException;
use common\libraries\ObjectNotExistException;
use common\libraries\NotAllowedException;

/**
 * $Id: attachment_viewer.class.php 201 2009-11-13 12:34:51Z chellee $
 * 
 * @package application.personal_calendar.personal_calendar_manager.component
 */
class PersonalCalendarManagerAttachmentViewerComponent extends PersonalCalendarManager
{

    public function run()
    {
        // retrieve the attachment id
        $attachment_id = Request :: get('object');
        if (is_null($attachment_id))
        {
            throw new ParameterNotDefinedException('object');
        }
        
        // retrieve the calendar publication
        $calendar_publication_id = Request :: get(PersonalCalendarManager :: PARAM_PERSONAL_CALENDAR_ID);
        if (is_null($calendar_publication_id))
        {
            throw new ParameterNotDefinedException(PersonalCalendarManager :: PARAM_PERSONAL_CALENDAR_ID);
        }
        
        $calendar_publication = PersonalCalendarDataManager :: get_instance()->retrieve_personal_calendar_publication(
                $calendar_publication_id);
        if (! $calendar_publication)
        {
            throw new ObjectNotExistException(Translation :: get('PersonalCalendarPublication'), 
                    $calendar_publication_id);
        }
        
        /* are you allowed to view the publication? */
        
        $user = $this->get_user();
        
        $is_target = $calendar_publication->is_target($user);
        $is_publisher = ($calendar_publication->get_publisher() == $user->get_id());
        $is_platform_admin = $user->is_platform_admin();
        
        if (! $is_target && ! $is_publisher && ! $is_platform_admin)
        {
            throw new NotAllowedException();
        }
        
        // is the attachment actually attached to the publication
        if (! $calendar_publication->get_publication_object()->is_attached_to_or_included_in($attachment_id))
        {
            throw new NotAllowedException();
        }
        
        $object = \repository\DataManager :: retrieve_content_object($attachment_id);
        if (! $object)
        {
            throw new ObjectNotExistException(Translation :: get('ContentObject'), $attachment_id);
        }
        
        Display :: small_header();
        
        echo ContentObjectRenditionImplementation :: launch($object, ContentObjectRendition :: FORMAT_HTML, 
                ContentObjectRendition :: VIEW_FULL, $this);
        
        Display :: small_footer();
    }

    public function get_additional_parameters()
    {
        return array('object', PersonalCalendarManager :: PARAM_PERSONAL_CALENDAR_ID);
    }

    public function get_content_object_display_attachment_url($attachment)
    {
        return $this->get_url(
                array(Application :: PARAM_ACTION => PersonalCalendarManager :: ACTION_VIEW_ATTACHMENT, 
                        'object' => $attachment->get_id()));
    }
}
