<?php
namespace application\personal_calendar;

use common\libraries\BreadcrumbTrail;

use common\libraries\DelegateComponent;

use repository\ContentObjectRendition;
use repository\ContentObjectRenditionImplementation;
use group\DataManager as GroupDataManager;
use group\Group;
use common\libraries\Request;
use common\libraries\Application;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\DatetimeUtilities;
use common\libraries\Breadcrumb;
use common\libraries\ActionBarRenderer;
use common\libraries\PlatformSetting;

/**
 * $Id: viewer.class.php 201 2009-11-13 12:34:51Z chellee $
 *
 * @package application.personal_calendar.personal_calendar_manager.component
 */

class PersonalCalendarManagerViewerComponent extends PersonalCalendarManager implements DelegateComponent
{
    private $event;
    private $action_bar;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $id = Request :: get(PersonalCalendarManager :: PARAM_PERSONAL_CALENDAR_ID);
        
        if ($id)
        {
            $this->event = $this->retrieve_personal_calendar_publication($id);
            if (! $this->can_view())
            {
                $this->display_header();
                $this->display_error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
                $this->display_footer();
                exit();
            }
            
            $this->action_bar = $this->get_action_bar();
            $output = $this->get_publication_as_html();
            
            $this->display_header();
            echo '<br /><a name="top"></a>';
            echo $this->action_bar->as_html() . '<br />';
            echo '<div id="action_bar_browser">';
            echo $output;
            echo '</div>';
            
            $this->display_footer();
        }
        else
        {
            $this->display_error_page(htmlentities(Translation :: get('NoProfileSelected')));
        }
    }

    public function can_view()
    {
        $event = $this->event;
        $user = $this->get_user();
        
        $is_target = $event->is_target($user);
        $is_publisher = ($event->get_publisher() == $user->get_id());
        $is_platform_admin = $user->is_platform_admin();
        
        if (! $is_target && ! $is_publisher && ! $is_platform_admin)
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    public function get_publication_as_html()
    {
        $event = $this->event;
        $action_bar = $this->action_bar;
        
        $content_object = $event->get_publication_object();
        $content_object_properties = $content_object->get_properties();
        BreadcrumbTrail :: get_instance()->add(new Breadcrumb(null, $content_object_properties['default_properties']['title']));
        
        $html = array();
        
        $time = Request :: get('time') ? intval(Request :: get('time')) : time();
        $view = Request :: get('view') ? Request :: get('view') : 'month';
        $this->set_parameter('time', $time);
        $this->set_parameter('view', $view);
        $this->set_parameter(Application :: PARAM_ACTION, PersonalCalendarManager :: ACTION_BROWSE_CALENDAR);
        
        $minimonthcalendar = new PersonalCalendarMiniMonthRenderer($this, $time);
        $html[] = '<div style="float: left; width: 19%; padding-right: 1%;">';
        $html[] = $minimonthcalendar->render();
        $html[] = '</div>';
        $html[] = '<div style="float: left; width: 80%;">';
        
        $html[] = ContentObjectRenditionImplementation :: launch($content_object, ContentObjectRendition :: FORMAT_HTML, ContentObjectRendition :: VIEW_FULL, $this);
        $html[] = '<div class="event_publication_info">';
        $html[] = htmlentities(Translation :: get('PublishedOn', null, Utilities :: COMMON_LIBRARIES)) . ' ' . $this->render_publication_date();
        $html[] = htmlentities(Translation :: get('By', null, Utilities :: COMMON_LIBRARIES)) . ' ' . $this->render_repository_viewer($event);
        $html[] = htmlentities(Translation :: get('SharedWith', null, Utilities :: COMMON_LIBRARIES)) . ' ' . $this->render_publication_targets();
        $html[] = '</div>';
        
        $back = $this->get_url();
        $edit_url = $this->get_publication_editing_url($event);
        $delete_url = $this->get_publication_deleting_url($event);
        $ical_url = $this->get_ical_export_url($event);
        
        $action_bar->add_common_action(new ToolbarItem(Translation :: get('Back', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_prev.png', $back));
        $action_bar->add_tool_action(new ToolbarItem(Translation :: get('ExportIcal'), Theme :: get_common_image_path() . 'export_csv.png', $ical_url));
        
        $user = $this->get_user();
        
        if ($user->is_platform_admin() || $event->get_publisher() == $user->get_id())
        {
            $action_bar->add_common_action(new ToolbarItem(Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_edit.png', $edit_url));
            $action_bar->add_common_action(new ToolbarItem(Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_delete.png', $delete_url));
        }
        
        $html[] = '</div>';
        
        return implode("\n", $html);
    }

    public function render_repository_viewer()
    {
        $user = $this->event->get_publication_publisher();
        return $user->get_firstname() . ' ' . $user->get_lastname();
    }

    public function render_publication_date()
    {
        $date_format = Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES);
        return DatetimeUtilities :: format_locale_date($date_format, $this->event->get_published());
    }

    public function render_publication_targets()
    {
        $event = $this->event;
        
        if ($event->is_for_nobody())
        {
            return htmlentities(Translation :: get('Nobody', null, 'user'));
        }
        else
        {
            $users = $event->get_target_users();
            $groups = $event->get_target_groups();
            if (count($users) + count($groups) == 1)
            {
                if (count($users) == 1)
                {
                    $user = $this->get_user_info($users[0]);
                    return $user->get_firstname() . ' ' . $user->get_lastname();
                }
                else
                {
                    $group = GroupDataManager :: retrieve_by_id(Group :: class_name(), $groups[0]);
                    return $group->get_name();
                }
            }
            $target_list = array();
            $target_list[] = '<select>';
            foreach ($users as $index => $user_id)
            {
                $user = $this->get_user_info($user_id);
                $target_list[] = '<option>' . $user->get_firstname() . ' ' . $user->get_lastname() . '</option>';
            }
            foreach ($groups as $index => $group_id)
            {
                // Todo: make this more efficient. Get all course_groups using a
                // single query
                $group = GroupDataManager :: retrieve_by_id(Group :: class_name(), $group_id);
                $target_list[] = '<option>' . $group->get_name() . '</option>';
            }
            $target_list[] = '</select>';
            return implode("\n", $target_list);
        }
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        
        if (PlatformSetting :: get('allow_personal_agenda', 'user') && ! Request :: get('calendar_event'))
        {
            $action_bar->add_common_action(new ToolbarItem(Translation :: get('Publish', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_publish.png', $this->get_url(array(
                    Application :: PARAM_ACTION => PersonalCalendarManager :: ACTION_CREATE_PUBLICATION))));
        }
        
        $action_bar->add_tool_action(new ToolbarItem(Translation :: get('ListView', null, Utilities :: COMMON_LIBRARIES), Theme :: get_image_path() . 'tool_calendar_down.png', $this->get_url(array(
                Application :: PARAM_ACTION => PersonalCalendarManager :: ACTION_BROWSE_CALENDAR, 'view' => 'list'))));
        $action_bar->add_tool_action(new ToolbarItem(Translation :: get('MonthView', null, Utilities :: COMMON_LIBRARIES), Theme :: get_image_path() . 'tool_calendar_month.png', $this->get_url(array(
                Application :: PARAM_ACTION => PersonalCalendarManager :: ACTION_BROWSE_CALENDAR, 'view' => 'month'))));
        $action_bar->add_tool_action(new ToolbarItem(Translation :: get('WeekView', null, Utilities :: COMMON_LIBRARIES), Theme :: get_image_path() . 'tool_calendar_week.png', $this->get_url(array(
                Application :: PARAM_ACTION => PersonalCalendarManager :: ACTION_BROWSE_CALENDAR, 'view' => 'week'))));
        $action_bar->add_tool_action(new ToolbarItem(Translation :: get('DayView', null, Utilities :: COMMON_LIBRARIES), Theme :: get_image_path() . 'tool_calendar_day.png', $this->get_url(array(
                Application :: PARAM_ACTION => PersonalCalendarManager :: ACTION_BROWSE_CALENDAR, 'view' => 'day'))));
        $action_bar->add_tool_action(new ToolbarItem(Translation :: get('Today', null, Utilities :: COMMON_LIBRARIES), Theme :: get_image_path() . 'tool_calendar_today.png', $this->get_url(array(
                Application :: PARAM_ACTION => PersonalCalendarManager :: ACTION_BROWSE_CALENDAR, 
                'view' => (Request :: get('view') ? Request :: get('view') : 'month'), 'time' => time()))));
        return $action_bar;
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('personal_calendar_viewer');
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_PERSONAL_CALENDAR_ID);
    }

    public function get_content_object_display_attachment_url($attachment)
    {
        return $this->get_url(array(Application :: PARAM_ACTION => PersonalCalendarManager :: ACTION_VIEW_ATTACHMENT, 
                'object' => $attachment->get_id()));
    }

}
