<?php
namespace application\personal_calendar;

use user\User;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\WebApplication;
use common\libraries\Application;
use repository\content_object\calendar_event\CalendarEvent;
use common\libraries\package\PackageList;
use common\libraries\CommonDataManager;
use common\libraries\Theme;

/**
 * $Id: personal_calendar_manager.class.php 205 2009-11-13 12:57:33Z vanpouckesven $
 *
 * @package application.personal_calendar.personal_calendar_manager
 */

/**
 * This application gives each user the possibility to maintain a personal calendar.
 */
class PersonalCalendarManager extends WebApplication
{
    const APPLICATION_NAME = 'personal_calendar';
    const PARAM_PERSONAL_CALENDAR_ID = 'personal_calendar';
    const ACTION_BROWSE_CALENDAR = 'browser';
    const ACTION_VIEW_PUBLICATION = 'viewer';
    const ACTION_CREATE_PUBLICATION = 'publisher';
    const ACTION_DELETE_PUBLICATION = 'deleter';
    const ACTION_EDIT_PUBLICATION = 'editor';
    const ACTION_VIEW_ATTACHMENT = 'attachment_viewer';
    const ACTION_EXPORT_ICAL = 'ical_exporter';
    const ACTION_IMPORT_ICAL = 'ical_importer';
    const ACTION_RIGHT_EDITS = 'rights_editor';
    const DEFAULT_ACTION = self :: ACTION_BROWSE_CALENDAR;
    const ACTION_RENDER_BLOCK = 'block';
    const PARAM_TIME = 'time';
    const PARAM_VIEW = 'view';

    /**
     * Constructor
     *
     * @param $user_id int
     */
    public function __construct($user)
    {
        parent :: __construct($user);
    }

    /**
     * Gets the events
     *
     * @param $from_date int
     * @param $to_date int
     */
    public function get_events($from_date, $to_date)
    {
        return PersonalCalendarDataManager :: get_events($this, $from_date, $to_date);
    }

    public function get_user_events($from_date, $to_date)
    {
        return PersonalCalendarDataManager :: get_user_events($this, $from_date, $to_date);
    }

    public function get_connector_events($from_date, $to_date)
    {
        return PersonalCalendarDataManager :: get_connector_events($this, $from_date, $to_date);
    }

    public function get_user_shared_events($from_date, $to_date)
    {
        return PersonalCalendarDataManager :: get_user_shared_events($this, $from_date, $to_date);
    }

    /**
     *
     * @see Application::content_object_is_published()
     */
    public static function content_object_is_published($object_id)
    {
        $dm = PersonalCalendarDataManager :: get_instance();
        return $dm->content_object_is_published($object_id);
    }

    /**
     *
     * @see Application::any_content_object_is_published()
     */
    public static function any_content_object_is_published($object_ids)
    {
        $dm = PersonalCalendarDataManager :: get_instance();
        return $dm->any_content_object_is_published($object_ids);
    }

    /**
     *
     * @see Application::get_content_object_publication_attributes()
     */
    public static function get_content_object_publication_attributes(
            $object_id, $type = self :: ATTRIBUTES_TYPE_OBJECT, $condition = null,
            $count = null, $offset = null, $order_properties = null)
    {
        $dm = PersonalCalendarDataManager :: get_instance();
        return $dm->get_content_object_publication_attributes(
            $object_id,
            $type,
            $condition,
            $count,
            $offset,
            $order_properties);
    }

    /**
     *
     * @see Application::get_content_object_publication_attribute()
     */
    public static function get_content_object_publication_attribute($publication_id)
    {
        $dm = PersonalCalendarDataManager :: get_instance();
        return $dm->get_content_object_publication_attribute($publication_id);
    }

    public static function count_publication_attributes($user = null, $object_id = null, $condition = null)
    {
        return PersonalCalendarDataManager :: get_instance()->count_publication_attributes(
            $user,
            $object_id,
            $condition);
    }

    /**
     *
     * @see Application::delete_content_object_publications()
     */
    public static function delete_content_object_publications($object_id)
    {
        $dm = PersonalCalendarDataManager :: get_instance();
        return $dm->delete_content_object_publications($object_id);
    }

    public static function delete_content_object_publication($publication_id)
    {
        $dm = PersonalCalendarDataManager :: get_instance();
        return $dm->delete_content_object_publication($publication_id);
    }

    /**
     *
     * @see Application::update_content_object_publication_id()
     */
    public static function update_content_object_publication_id($publication_attr)
    {
        return PersonalCalendarDataManager :: get_instance()->update_content_object_publication_id($publication_attr);
    }

    /**
     * Inherited
     */
    public static function get_content_object_publication_locations($content_object)
    {
        $allowed_types = array(CalendarEvent :: class_name());

        $type = $content_object->get_type();
        if (in_array($type, $allowed_types))
        {
            $locations = array(Translation :: get('TypeName'));
            return $locations;
        }

        return array();
    }

    public static function publish_content_object($content_object, $location)
    {
        $pub = new PersonalCalendarPublication();
        $pub->set_content_object_id($content_object->get_id());
        $pub->set_publisher($content_object->get_owner_id());
        $pub->create();

        return Translation :: get(
            'ObjectCreated',
            array('OBJECT' => Translation :: get('PersonalCalendarPublication')),
            Utilities :: COMMON_LIBRARIES);
    }

    public function retrieve_personal_calendar_publication($publication_id)
    {
        $pcdm = PersonalCalendarDataManager :: get_instance();
        return $pcdm->retrieve_personal_calendar_publication($publication_id);
    }

    public function retrieve_task_publication($publication_id)
    {
        $pcdm = PersonalCalendarDataManager :: get_instance();
        return $pcdm->retrieve_personal_calendar_publication($publication_id);
    }

    public function get_publication_deleting_url($publication)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_DELETE_PUBLICATION,
                self :: PARAM_PERSONAL_CALENDAR_ID => $publication->get_id()));
    }

    public function get_publication_editing_url($publication)
    {
        return $this->get_url(
            array(
                self :: PARAM_ACTION => self :: ACTION_EDIT_PUBLICATION,
                self :: PARAM_PERSONAL_CALENDAR_ID => $publication->get_id()));
    }

    public function get_publication_viewing_url($publication)
    {
        return $this->get_link($parameters);
    }

    public function get_ical_export_url($publication)
    {
        $parameters = array();
        $parameters[self :: PARAM_PERSONAL_CALENDAR_ID] = $publication->get_id();
        $parameters[self :: PARAM_ACTION] = self :: ACTION_EXPORT_ICAL;

        return $this->get_url($parameters);
    }

    public function get_ical_import_url()
    {
        $parameters = array();
        $parameters[self :: PARAM_ACTION] = self :: ACTION_IMPORT_ICAL;

        return $this->get_url($parameters);
    }

    public function get_user_info($user_id)
    {
        return \user\DataManager :: retrieve(User :: class_name(), (int) $user_id);
    }

    /**
     * Helper function for the Application class, pending access to class constants via variables in PHP 5.3 e.g. $name
     * = $class :: DEFAULT_ACTION DO NOT USE IN THIS APPLICATION'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourApplicationManager :: DEFAULT_ACTION in all other application classes
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    public static function get_installable_application_packages($include_installed = false)
    {
        $package_list = new PackageList(
            self :: context(),
            Translation :: get('TypeName', null, __NAMESPACE__),
            Theme :: get_image_path() . 'logo/16.png');

        $integration_list = new \common\libraries\package\PackageList(
            self :: context() . '\integration',
            Translation :: get('Integrations', null, __NAMESPACE__));

        $core_list = new \common\libraries\package\PackageList(
            self :: context() . '\integration\core',
            Translation :: get('Core', null, __NAMESPACE__));

        if (! CommonDataManager :: get_registration(self :: context() . '\integration\core\home') || $include_installed)
        {
            $core_list->add_package(self :: context() . '\integration\core\home');
        }

        $integration_list->add_child($core_list);
        $package_list->add_child($integration_list);

        return $package_list;
    }
}
