<?php
namespace application\personal_messenger;

use common\libraries\FormValidator;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\Utilities;

require_once Path :: get_plugin_path() . 'html2text/class.html2text.inc';

/**
 * This form offers the functionality to quickly reply on a message.
 *
 * @name Chamilo Personal Messenger > QuickReplyForm
 * @version 1.0.0
 * @author Stijn Van Hoecke
 */
class QuickReplyForm extends FormValidator
{

    // **************************************************************************
    // VARIABLES
    // **************************************************************************
    /**
     * The original message on which we reply.
     *
     * @var Message
     */
    private $original_message;
    /**
     * The new message.
     *
     * @var Message
     */
    private $message;
    /**
     * The user for further use.
     *
     * @var User
     */
    private $user;

    // **************************************************************************
    // CONSTRUCTOR
    // **************************************************************************
    /**
     * Constructor.
     */
    public function __construct($original_message, $action, $user)
    {
        parent :: __construct('quick_reply_personal_message', 'post', $action);

        $this->original_message = $original_message;
        $this->user = $user;

        // create new message
        $this->message = new Message();

        // preset ids
        $this->message->set_author_id($this->user->get_id());
        $this->message->set_reply_id($this->original_message->get_id());
        $this->message->set_parent_id($this->original_message->get_id());

        // preset the recipients of the message
        $this->message->set_target_users($this->original_message->get_target_users());
        $this->message->set_target_groups($this->original_message->get_target_groups());

        $this->build_quick_form();
        $this->setDefaults();
    }

    // **************************************************************************
    // FORM FUNCTIONS
    // **************************************************************************
    /**
     * Builds the quick reply form.
     */
    public function build_quick_form()
    {
        /*
         * HTML EDITOR DOESN'T LOAD => to fix // html_editor for quick message $this->add_html_editor(Message ::
         * PROPERTY_MESSAGE_CONTENT, null, true);
         */

        $this->add_textfield(Message :: PROPERTY_MESSAGE_CONTENT, null, true);

        // send - clear buttons
        $buttons[] = $this->createElement('style_submit_button', 'submit',
                Translation :: get('Send', null, Utilities :: COMMON_LIBRARIES), array('class' => 'positive send'));
        // $buttons[] = $this->createElement(
        // 'style_reset_button',
        // 'reset',
        // Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES),
        // array('class' => 'normal empty')
        // );

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    // **************************************************************************
    // OBJECT FUNCTIONS
    // **************************************************************************
    /**
     * Creates the reply message.
     *
     * @return Message
     */
    public function create_quick_reply()
    {
        $message = $this->message;
        $values = $this->exportValues();

        // content
        $message->set_content($values[Message :: PROPERTY_MESSAGE_CONTENT]);

        // create message
        return $message->create();
    }

    // **************************************************************************
    // DEFAULTS
    // **************************************************************************
    /**
     * Sets default values.
     *
     * @param $defaults array Default values for this form's parameters.
     */
    public function setDefaults($defaults = array())
    {
        parent :: setDefaults($defaults);
    }

}
