<?php

namespace application\personal_messenger;

use common\libraries\FormValidator;
use common\libraries\Path;
use common\libraries\Session;
use common\libraries\Translation;
use common\libraries\Utilities;
use group\DataManager as GroupDataManager;
use group\Group;

/** * ***************************************************************************
 * This form offers the functionality to select users and groups.
 *
 * @name Chamilo Personal Messenger > SelectUserForm
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class SelectUserForm extends FormValidator
{
    //**************************************************************************
    // CONSTANTS
    //**************************************************************************
    const SELECT_USERS = 'select_users';

    const SUBARRAY_USER_GROUP_FINDER_USER = 'user';
    const SUBARRAY_USER_GROUP_FINDER_GROUP = 'group';

    //**************************************************************************
    // VARIABLES
    //**************************************************************************
    /**
     * The text to display on the submit button.
     * @var string
     */
    private $submit_button_text;
    /**
     * Indicates if the selector should be collapsed by default.
     * @var boolean
     */
    private $collapse;
    /**
     * Holds the hidden users.
     * @var array
     */
    private $hidden_users;

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor.
     */
    public function __construct($manager, $action, $submit_button_text = 'Submit', $hidden_ids = array(), $default_selected = array(), $collapse = false)
    {
        parent :: __construct('select_and_add_user', 'post', $action);
        $this->submit_button_text = $submit_button_text;
        $this->collapse = $collapse;
        $this->set_default_hidden($hidden_ids);
        $this->build_select_user_form();
        $this->set_default_selected($default_selected);
    }

    //**************************************************************************
    // FORM FUNCTIONS
    //**************************************************************************
    /**
     * Builds the move form.
     */
    public function build_select_user_form()
    {
        // user - group finder
        $url = Path :: get(WEB_PATH) . 'group/php/xml_feeds/xml_user_group_feed.php';
        $locale = array();
        $locale['Display'] = Translation :: get('SelectUsers');
        $locale['Searching'] = Translation :: get('Searching', null, Utilities :: COMMON_LIBRARIES);
        $locale['NoResults'] = Translation :: get('NoResults', null, Utilities :: COMMON_LIBRARIES);
        $locale['Error'] = Translation :: get('Error', null, Utilities :: COMMON_LIBRARIES);

        $elem = $this->addElement(
                        'user_group_finder',
                        self :: SELECT_USERS,
                        Translation :: get('Users', null, Utilities :: COMMON_LIBRARIES),
                        $url,
                        $locale,
                        array()
        );
        $elem->excludeElements($this->hidden_users);
        $elem->setDefaultCollapsed($this->collapse);

        // send - clear buttons
        $buttons[] = $this->createElement(
                        'style_submit_button',
                        'submit',
                        Translation :: get($this->submit_button_text),
                        array('class' => 'positive send')
        );
        $buttons[] = $this->createElement(
                        'style_reset_button',
                        'reset',
                        Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES),
                        array('class' => 'normal empty')
        );

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    //**************************************************************************
    // OBJECT FUNCTIONS
    //**************************************************************************
    /**
     * Returns the selected user_ids.
     * @return array Array of user_ids
     */
    public function get_selected_users($include_group_users = false)
    {
        $values = $this->exportValues();
        $user_ids = array();

        // users (target)
        if ($values[self :: SELECT_USERS][self :: SUBARRAY_USER_GROUP_FINDER_USER])
        {
            $user_ids = array_merge(
                            $user_ids,
                            $values[self :: SELECT_USERS][self :: SUBARRAY_USER_GROUP_FINDER_USER]
            );
        }

        if ($include_group_users)
        {
            // groups (target)
            if ($values[self :: SELECT_USERS][self :: SUBARRAY_USER_GROUP_FINDER_GROUP])
            {
                $groups = $this->get_selected_groups();
                foreach ($groups as $group)
                {
                    array_merge(
                            $user_ids,
                            GroupDataManager :: retrieve_by_id(Group :: Class_name(), $group)->get_users(true, true)
                    );
                }
            }
        }

        return $user_ids;
    }

    /**
     * Returns the selected group_ids.
     * @return array Array of group_ids
     */
    public function get_selected_groups()
    {
        $values = $this->exportValues();
        $group_ids = array();

        if ($values[self :: SELECT_USERS][self :: SUBARRAY_USER_GROUP_FINDER_GROUP])
        {
            $group_ids = array_merge(
                            $group_ids,
                            $values[self :: SELECT_USERS][self :: SUBARRAY_USER_GROUP_FINDER_GROUP]
            );
        }

        return $group_ids;
    }

    //**************************************************************************
    // DEFAULTS
    //**************************************************************************
    /**
     * Sets default selected values.
     * @param array $selected Array of user_group_finder elements
     */
    public function set_default_selected($selected)
    {
        if (!empty($selected))
        {
            $active = $this->getElement(self :: SELECT_USERS);
            if ($active)
            {
                if ($active->_elements[0])
                {
                    $active->_elements[0]->setValue(serialize($selected));
                }
            }
        }
    }

    /**
     * Sets default hidden values.
     * @param array $hidden Array of user_ids
     */
    public function set_default_hidden($hidden_ids)
    {
        $this->hidden_users = array();
        $this->hidden_users[] = 'user_' . Session :: get_user_id();

        if (!is_array($hidden_ids))
        {
            $hidden_ids = array($hidden_ids);
        }

        foreach ($hidden_ids as $id)
        {
            $this->hidden_users[] = 'user_' . $id;
        }
    }

}
