<?php

namespace application\personal_messenger;

use common\libraries\Application;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * This component offers the functionality to create a template based on a message.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerTemplatorComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class PersonalMessengerManagerTemplatorComponent extends PersonalMessengerManager
{

    //**************************************************************************
    // RUN
    //**************************************************************************
    /**
     * Runs this component.
     */
    public function run()
    {
        // retrieve the message to convert to a template
        $dm = PersonalMessengerDataManager :: get_instance();
        $message_condition = new EqualityCondition(Message :: PROPERTY_ID, Request :: get(self :: PARAM_MESSAGE_ID));
        $message = $dm->retrieve_message($message_condition);

        $success = $this->create_template($message);

        // go back
        $go_back_parameters = array();
        if (Request :: get(self :: PARAM_REFERER))
        {
            $go_back_parameters[Application :: PARAM_ACTION] = Request :: get(self :: PARAM_REFERER);
            // in case of template, if you have a referer it is the (viewer) of the message -> return to the message
            $go_back_parameters[self :: PARAM_MESSAGE_ID] = Request :: get(self :: PARAM_MESSAGE_ID);
        }
        else
        {
            $go_back_parameters[Application :: PARAM_ACTION] = self :: DEFAULT_ACTION;
        }

        $this->redirect(
                Translation :: get(
                        $success ? 'ObjectCreated' : 'ObjectNotCreated',
                        array('OBJECT' => Translation :: get('Template')),
                        Utilities :: COMMON_LIBRARIES
                ),
                ($success ? false : true),
                $go_back_parameters
        );
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Adds the additional parameters to the URL.
     * @return array
     */
    public function get_additional_parameters()
    {
        return array(self :: PARAM_REFERER, self :: PARAM_MESSAGE_ID, self :: PARAM_REPLY_ID);
    }

}
