<?php

namespace application\personal_messenger;

use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;

/** * ***************************************************************************
 * Actual implementation of the UserManager for ignored users.
 *
 * @name Chamilo Personal Messenger > PersonalMessengerManagerUserManagerIgnoredComponent
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * @see PersonalMessengerManagerUserManagerComponent
 * **************************************************************************** */
class PersonalMessengerManagerUserManagerIgnoredComponent extends PersonalMessengerManagerUserManagerComponent
{

    //**************************************************************************
    // CONSTRUCTOR
    //**************************************************************************
    /**
     * Constructor.
     */
    public function __construct($user)
    {
        parent :: __construct($user);
        Request :: set_get(self :: PARAM_REFERER, self :: ACTION_MANAGE_USERS_IGNORED);
    }

    //**************************************************************************
    // INHERITED FUNCTIONS
    //**************************************************************************
    /**
     * Returns the base condition for ignored user managers.
     * @return Condition
     */
    public function get_base_condition()
    {
        $dm = PersonalMessengerDataManager :: get_instance();
        return new EqualityCondition(
                UserIgnore :: PROPERTY_USER_IGNORE_USER_ID,
                $this->get_user_id(),
                $dm->get_alias(UserIgnore:: get_table_name()),
                true
        );
    }

    /**
     * Returns an IgnoredUserTable.
     * @param array $parameters
     * @param Condition $condition
     * @return table
     */
    public function get_table($parameters, $condition)
    {
        return new IgnoredUserTable($this, $parameters, $condition);
    }

    /**
     * Returns the default selected values in the user selector
     * @return array
     */
    public function get_default_selected()
    {
        return array();
    }

    /**
     * Retuns the default hidden values in the user selector.
     * @return array
     */
    public function get_default_hidden_users()
    {
        // construct condition
        $condition = new EqualityCondition(
                        UserIgnore :: PROPERTY_USER_IGNORE_USER_ID,
                        $this->get_user_id()
        );

        // retrieve users
        $ignored_users = PersonalMessengerDataManager :: get_instance()->retrieve_all_user_ignores($condition);

        // construct user_group_finder elements from users
        $default_hidden = array();
        while ($ignored_user = $ignored_users->next_result())
        {
            $default_hidden[] = $ignored_user->get_ignored_user_id();
        }

        // return the ids
        return $default_hidden;
    }

    /**
     * Returns the text for the table category division.
     * @return string The translation key for the table category text.
     */
    public function get_table_category_text()
    {
        return 'IgnoredUsers';
    }

    /**
     * Returns the text for the form category division.
     * @return string The translation key for the form category text.
     */
    public function get_form_category_text()
    {
        return 'IgnoreUsers';
    }

    /**
     * Returns the text for the submit button.
     * @return string The translation key for the button text.
     */
    public function get_submit_button_text()
    {
        return 'IgnoreUsers';
    }

    /**
     * Ignores the given users.
     * @param array $user_ids
     * @return boolean
     */
    public function add($form)
    {
        return ($this->ignore_users($form->get_selected_users(true)) == 0);
    }

    /**
     * Restores the given users.
     * @param array $user_ids
     * @return boolean
     */
    public function delete($form)
    {
        return ($this->unignore_users($form->get_selected_users(true)) == 0);
    }

    //**************************************************************************
    // ADDITIONAL FUNCTIONS
    //**************************************************************************
    /**
     * Adds the additional breadcrumbs for navigation.
     * @param BreadcrumbTrail $breadcrumbtrail
     */
    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $browser_folder_url = $this->get_browser_url($this->get_folder(), $this->get_category_id());

        // browser
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get('PersonalMessengerManagerBrowserComponent'))
        );

        // folder
        $breadcrumbtrail->add(new Breadcrumb(
                        $browser_folder_url,
                        Translation :: get(Utilities:: underscores_to_camelcase(
                                        PersonalMessengerFolder :: get_folder_name($this->get_folder())
                        ))));
    }

}
