<?php

namespace application\personal_messenger;

use common\libraries\Utilities;

/** * ***************************************************************************
 * This class represents a user ignoring anothe user (IGNORE).
 *
 * IGNORE objects only have 2 default properties, because they link two people
 * exluding each other from their inbox:
 * - user_id: the numeric ID identifying the user;
 * - ignored_user_id: the numeric ID identifying the ignored user;
 *
 * @name Chamilo Personal Messenger > UserIgnore
 * @version 1.0.0
 * @author Stijn Van Hoecke
 * **************************************************************************** */
class UserIgnore extends PersonalMessengerDataClass
{
    //**************************************************************************
    // GENERAL CONSTANTS
    //**************************************************************************
    const CLASS_NAME = __CLASS__;

    //**************************************************************************
    // PROPERTY CONSTANTS
    //**************************************************************************
    const PROPERTY_USER_IGNORE_USER_ID = 'user_id';
    const PROPERTY_USER_IGNORE_IGNORED_USER_ID = 'ignored_user_id';

    //**************************************************************************
    // GENERAL GETTERS
    //**************************************************************************
    /**
     * Get the default properties of all ignored users.
     * @return array The property names
     */
    public static function get_default_property_names()
    {
        return array(
            self :: PROPERTY_USER_IGNORE_USER_ID,
            self :: PROPERTY_USER_IGNORE_IGNORED_USER_ID);
    }

    //**************************************************************************
    // PROPERTY GETTERS
    //**************************************************************************
    /**
     * Returns the numeric ID of the user (who ignores).
     * @return int The ID of the user (who ignores)
     */
    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_IGNORE_USER_ID);
    }

    /**
     * Returns the numeric ID of the ignored user (who gets ignored).
     * @return int The ID of the ignored user (who gets ignored)
     */
    public function get_ignored_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_IGNORE_IGNORED_USER_ID);
    }

    //**************************************************************************
    // PROPERTY SETTERS
    //**************************************************************************
    /**
     * Sets the numeric ID of the user (who ignores).
     * @param int $user_id The ID of the user (who ignores)
     */
    public function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_IGNORE_USER_ID, $user_id);
    }

    /**
     * Sets the numeric ID of the ignored user (who gets ignored).
     * @param int $user_id The ID of the ignored user (who gets ignored)
     */
    public function set_ignored_user_id($ignored_user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_IGNORE_IGNORED_USER_ID, $ignored_user_id);
    }

    //**************************************************************************
    // DATABASE FUNCTIONS
    //**************************************************************************
    /**
     * Gets the table name for this class.
     */
    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

}
