<?php
namespace application\weblcms\course_type\upgrade\v4_0_0;

use application\weblcms\course_type\CourseType;
use application\weblcms\course_type\CourseTypeRelCourseSettingValue;
use application\weblcms\course\DataManager;
use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 *
 * @author Pieterjan Broekaert
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\application\weblcms\course_type\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                $success = $upgrader->upgrade_db($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->update_db_content($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpdateDatabaseContentSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpdateDatabaseContentFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if(!$success)
                {
                    return false;
                }

                $success = $upgrader->upgrade_registration($data_manager);

                if($success)
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_NORMAL, Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(\common\libraries\package\Upgrader :: TYPE_ERROR, Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                return $success;

            });

        if ($success)
        {
            $this->add_message(self :: TYPE_NORMAL, Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
        }
        else
        {
            $this->add_message(self :: TYPE_ERROR, Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
        }

        return $success;
    }

    /**
     * Upgrade the table structures
     * @param $data_manager
     * @return bool
     */
    public function upgrade_db($data_manager)
    {
        // Change some property types for the settings storage unit
        $attributes = array('type' => 'text');
        $success = $data_manager->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
            CourseType :: get_table_name(),
            CourseType :: PROPERTY_DESCRIPTION,
            $attributes);

        if (! $success)
        {
            return false;
        }

        // Change some property types for the settings storage unit
        $attributes = array('type' => 'text');
        $success = $data_manager->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
            CourseTypeRelCourseSettingValue :: get_table_name(),
            CourseTypeRelCourseSettingValue :: PROPERTY_VALUE,
            $attributes);

        if (! $success)
        {
            return false;
        }

        return true;
    }

    /**
     * Performs the database content upgrade.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function update_db_content($data_manager)
    {
        return true;
    }

    /**
     * Upgrades the package registration.
     *
     * @return boolean
     */
    public function upgrade_registration()
    {
        $data_manager = $this->get_data_manager();
        $package = Package :: get( $data_manager :: context());
        $registration = CommonDataManager :: get_registration($data_manager :: context());
        $registration->set_version($package->get_version());
        if (! $registration->update())
        {
            return false;
        }
        return true;
    }
}
