<?php
namespace application\weblcms\integration\core\home;

use application\weblcms\ContentObjectPublication;
use repository\content_object\document\Document;
use repository\content_object\webpage\Webpage;
use repository\content_object\file\File;
use repository\content_object\announcement\Announcement;
use repository\content_object\assignment\Assignment;
use application\weblcms\course\CourseGroupRelation;
use common\libraries\AndCondition;
use common\libraries\InCondition;
use common\libraries\InequalityCondition;
use common\libraries\ObjectTableOrder;
use group\DataManager as GroupDataManager;
use rights\TemplateEntity;
use common\libraries\Translation;
use common\libraries\Utilities;

use common\libraries\PlatformSetting;
use common\libraries\Request;

use application\weblcms\course\DataManager as CourseDataManager;
use application\weblcms\DataManager as WeblcmsDataManager;

/**
 * Description of weblcms_new_block
 *
 * @author Anthony Hurst (Hogeschool Gent)
 */
class NewBlock extends Block
{
    const TOOL_ANNOUNCEMENT = 'announcement';
    const TOOL_ASSIGNMENT = 'assignment';
    const TOOL_DOCUMENT = 'document';
    
    const OVERSIZED_SETTING = 'oversized_new_list_threshold';
    const FORCE_OVERSIZED = 'force_oversized_newblocks';
    const DO_FORCE_OVERSIZED = '1';
    const OVERSIZED_WARNING = 'oversized';

    private $courses;

    public function get_content($tool)
    {
        // All user courses
        $condition = CourseDataManager :: get_user_courses_condition($this->get_user());
        $user_courses = CourseDataManager :: retrieve_user_courses($condition);

        $threshold = intval(PlatformSetting :: get(self :: OVERSIZED_SETTING, 'application\weblcms'));
        
        if ($threshold !== 0 &&
            Request :: get (self :: FORCE_OVERSIZED) != self :: DO_FORCE_OVERSIZED &&
            $user_courses->size() > $threshold)
        {
            $this->courses = array();
            return self :: OVERSIZED_WARNING;
        }
        
        /*
        Courses by group subscription already retrieved by retrieve_user_courses.
        
         * // Courses active through group membership.
        $groups = GroupDataManager :: retrieve_all_subscribed_groups_array($this->get_user_id());

        $group_ids = array();
        foreach ($groups as $group)
        {
            $group_ids[$group->get_id()] = $group->get_id();
        }

        $condition = new InCondition(CourseGroupRelation :: PROPERTY_GROUP_ID, $group_ids,
                CourseGroupRelation :: get_table_name());

        array_splice($user_courses, 0, 0, CourseDataManager ::retrieve_user_courses($condition)->as_array());*/
        
        $this->courses = array();

        $course_settings_controller = \application\weblcms\CourseSettingsController :: get_instance();
        $unique_publications = array();
        while ($course = $user_courses->next_result())
        {
            $this->courses[$course->get_id()] = $course;
            
            if ($course_settings_controller->get_course_setting($course->get_id(),
                    \application\weblcms\CourseSettingsConnector :: VISIBILITY) == 1)
            {
                $condition = $this->get_publication_conditions($course, $tool);
                $course_module_id = WeblcmsDataManager :: retrieve_course_tool_by_name($tool)->get_id();
                $location = \application\weblcms\WeblcmsRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree(
                        \application\weblcms\WeblcmsRights :: TYPE_COURSE_MODULE, $course_module_id, $course->get_id());

                $entities = array();
                $entities[\application\weblcms\CourseGroupEntity :: ENTITY_TYPE] = \application\weblcms\CourseGroupEntity :: get_instance(
                        $course->get_id());
                $entities[\application\weblcms\CourseUserEntity :: ENTITY_TYPE] = \application\weblcms\CourseUserEntity :: get_instance();
                $entities[\application\weblcms\CoursePlatformGroupEntity :: ENTITY_TYPE] = \application\weblcms\CoursePlatformGroupEntity :: get_instance();
                $entities[TemplateEntity :: ENTITY_TYPE] = TemplateEntity :: get_instance();

                $publications = WeblcmsDataManager :: retrieve_content_object_publications_with_view_right_granted_in_category_location(
                        $location, $entities, $condition,
                        new ObjectTableOrder(
                                \application\weblcms\ContentObjectPublication :: PROPERTY_DISPLAY_ORDER_INDEX, SORT_DESC));

                if ($publications == 0)
                {
                    continue;
                }
                foreach ($publications->as_array() as $publication)
                {
                    $unique_publications[
                        $course->get_id() . '.' . $publication[ContentObjectPublication :: PROPERTY_ID]
                    ] = $publication;
                }
            }
        }
        return $unique_publications;
    }

    private function get_publication_conditions($course, $tool)
    {
        $type = null;
        switch ($tool)
        {
            case self :: TOOL_ANNOUNCEMENT :
                $type = Announcement :: class_name();
                break;
            case self :: TOOL_ASSIGNMENT :
                $type = Assignment :: class_name();
                break;
            case self :: TOOL_DOCUMENT :
                $type = array(Document :: class_name(), File :: class_name(), Webpage :: class_name());
                break;
        }
        $last_visit_date = \application\weblcms\DataManager :: get_last_visit_date($course->get_id(),
                $this->get_user_id(), $tool, 0);

        $conditions = array();
        $conditions[] = WeblcmsDataManager :: get_publications_condition($course,
                $this->get_user(), $tool, $type);
        $conditions[] = new InequalityCondition(
                \application\weblcms\ContentObjectPublication :: PROPERTY_PUBLICATION_DATE,
                InequalityCondition :: GREATER_THAN_OR_EQUAL, $last_visit_date);
        return new AndCondition($conditions);
    }

    protected function get_course_by_id($course_id)
    {
        return $this->courses[$course_id];
    }
    
    public function get_oversized_warning() {
        return '<div class="warning-message" style="width: auto; margin: 0 0 1em 0; position: static;">' .
            Translation :: get('OversizedWarning', null, Utilities :: COMMON_LIBRARIES) .
            ' <a href="?' . Utilities :: get_current_query_string(array(self :: FORCE_OVERSIZED => self :: DO_FORCE_OVERSIZED)) . '">' .
            Translation :: get('ForceOversized', null, Utilities :: COMMON_LIBRARIES) . '</a></div>';
    }
}
