<?php
namespace application\weblcms\integration\core\home;

use common\libraries\Translation;
use common\libraries\StringUtilities;
use common\libraries\Theme;
use common\libraries\PlatformSetting;
use common\libraries\RssIconGenerator;
use common\libraries\Redirect;
use common\libraries\SimpleTemplate;
use application\weblcms\course_type\DataManager as CourseTypeDataManager;
use application\weblcms\course\CourseManager;

/**
 * Block that displays main course's actions available in the main course menu. That is create course,
 * register/unregister to course, etc. Do not display less common actions such as manage categories.
 *
 * @copyright (c) 2011 University of Geneva
 * @license GNU General Public License - http://www.gnu.org/copyleft/gpl.html
 * @author lopprecht
 */
class CourseMenu extends Block
{

    public function is_teacher()
    {
        return $this->get_user()->is_teacher();
    }

    public function display_content()
    {
        $html = array();
        $html[] = '<div class="tool_menu">';
        $html[] = '<ul>';
        $html[] = '{$ADMIN_MENU}';
        $html[] = '{$USER_MENU}';
        $html[] = '</ul>';
        $html[] = '</div>';

        $target = $this->get_link_target();

        $template = '<li class="tool_list_menu" style="background-image: url({$IMG})">
        <a style="top: -3px; position: relative;" href="{$HREF}" target="' . $target . '">{$TEXT}</a></li>';

        $ADMIN_MENU = $this->display_admin_menu($template);
        $USER_MENU = SimpleTemplate :: all($template, $this->get_edit_course_menu());

        $this->display_admin_menu($template);
        SimpleTemplate :: all($template, $this->get_edit_course_menu());

        return SimpleTemplate :: ex($html, array('ADMIN_MENU' => $ADMIN_MENU, 'USER_MENU' => $USER_MENU));
    }

    public function display_admin_menu($template)
    {
        $result = array();
        if ($this->get_user()->is_platform_admin())
        {
            $menu = $this->get_platform_admin_menu();
            $result[] = SimpleTemplate :: all($template, $menu);
            $result[] = '<div style="margin: 10px 0 10px 0; border-bottom: 1px dotted #4271B5; height: 0px;"></div>';
        }
        else
        {
            if ($menu = $this->get_create_course_menu())
            {
                $result[] = SimpleTemplate :: all($template, $menu);
            }
        }
        return implode(StringUtilities :: NEW_LINE, $result);
    }

    public function get_create_course_menu()
    {
        if (! $this->is_teacher())
        {
            return '';
        }

        $result = array();

        $course_management_rights = \application\weblcms\CourseManagementRights :: get_instance();

        $count_direct = $count_request = 0;

        $course_types = CourseTypeDataManager :: retrieve_active_course_types();
        while ($course_type = $course_types->next_result())
        {
            if ($course_management_rights->is_allowed(
                    \application\weblcms\CourseManagementRights :: CREATE_COURSE_RIGHT, $course_type->get_id(),
                    \application\weblcms\CourseManagementRights :: TYPE_COURSE_TYPE))
            {
                $count_direct ++;
            }
            elseif ($course_management_rights->is_allowed(
                    \application\weblcms\CourseManagementRights :: REQUEST_COURSE_RIGHT, $course_type->get_id(),
                    \application\weblcms\CourseManagementRights :: TYPE_COURSE_TYPE))
            {
                $count_request ++;
            }
        }

        if (PlatformSetting :: get('allow_course_creation_without_coursetype', __NAMESPACE__))
        {
            $count_direct ++;
        }

        if ($count_direct)
        {
            $HREF = $this->get_course_action_url(\application\weblcms\WeblcmsManager :: ACTION_COURSE_MANAGER,
                    array(CourseManager :: PARAM_ACTION => CourseManager :: ACTION_QUICK_CREATE));
            $TEXT = htmlspecialchars(Translation :: get('CourseCreate'));
            $IMG = Theme :: get_common_image_path() . 'action_create.png';
            $result[] = compact('HREF', 'TEXT', 'IMG');
        }

        if ($count_request)
        {
            $HREF = $this->get_course_action_url(
                    \application\weblcms\WeblcmsManager :: ACTION_COURSE_CREATE_REQUEST_CREATOR);
            $TEXT = htmlspecialchars(Translation :: get('CourseRequest'));
            $IMG = Theme :: get_common_image_path() . 'action_create.png';
            $result[] = compact('HREF', 'TEXT', 'IMG');
        }

        return $result;
    }

    public function get_edit_course_menu()
    {
        $result = array();

        $HREF = $this->get_course_action_url(\application\weblcms\WeblcmsManager :: ACTION_COURSE_MANAGER,
                array(CourseManager :: PARAM_ACTION => CourseManager :: ACTION_BROWSE_UNSUBSCRIBED_COURSES));

        $TEXT = htmlspecialchars(Translation :: get('CourseSubscribe'));
        $IMG = Theme :: get_common_image_path() . 'action_subscribe.png';
        $result[] = compact('HREF', 'TEXT', 'IMG');

        $HREF = $this->get_course_action_url(\application\weblcms\WeblcmsManager :: ACTION_COURSE_MANAGER,
                array(CourseManager :: PARAM_ACTION => CourseManager :: ACTION_BROWSE_SUBSCRIBED_COURSES));

        $TEXT = htmlspecialchars(Translation :: get('CourseUnsubscribe'));
        $IMG = Theme :: get_common_image_path() . 'action_unsubscribe.png';
        $result[] = compact('HREF', 'TEXT', 'IMG');

        $HREF = htmlspecialchars(
                RssIconGenerator :: generate_rss_url(\application\weblcms\WeblcmsManager :: APPLICATION_NAME,
                        'publication', $this->get_user()));
        $TEXT = htmlspecialchars(Translation :: get('RssFeed'));
        $IMG = Theme :: get_common_image_path() . 'rss_feed.png';
        $result[] = compact('HREF', 'TEXT', 'IMG');

        return $result;
    }

    public function get_platform_admin_menu()
    {
        $result = array();

        $HREF = $this->get_course_action_url(\application\weblcms\WeblcmsManager :: ACTION_COURSE_MANAGER,
                array(CourseManager :: PARAM_ACTION => CourseManager :: ACTION_QUICK_CREATE));
        $TEXT = htmlspecialchars(Translation :: get('CourseCreate'));
        $IMG = Theme :: get_common_image_path() . 'action_create.png';
        $result[] = compact('HREF', 'TEXT', 'IMG');

        $HREF = $this->get_course_action_url(\application\weblcms\WeblcmsManager :: ACTION_COURSE_MANAGER);
        $TEXT = htmlspecialchars(Translation :: get('CourseList'));
        $IMG = Theme :: get_common_image_path() . 'action_browser.png';
        $result[] = compact('HREF', 'TEXT', 'IMG');

        $HREF = $this->get_course_action_url(\application\weblcms\WeblcmsManager :: ACTION_ADMIN_REQUEST_BROWSER);
        $TEXT = htmlspecialchars(
            Translation :: get('UserRequestList', null, \application\weblcms\WeblcmsManager :: context())
        );
        $IMG = Theme :: get_common_image_path() . 'action_browser.png';
        $result[] = compact('HREF', 'TEXT', 'IMG');

        $HREF = $this->get_course_action_url(\application\weblcms\WeblcmsManager :: ACTION_REQUEST);
        $TEXT = htmlspecialchars(
            Translation :: get('RequestList', null, \application\weblcms\WeblcmsManager :: context())
        );
        $IMG = Theme :: get_common_image_path() . 'action_browser.png';
        $result[] = compact('HREF', 'TEXT', 'IMG');

        return $result;
    }

    public function get_course_action_url($action, $params = array())
    {
        $params[\application\weblcms\WeblcmsManager :: PARAM_APPLICATION] = \application\weblcms\WeblcmsManager :: APPLICATION_NAME;
        $params[\application\weblcms\WeblcmsManager :: PARAM_ACTION] = $action;
        return htmlspecialchars(Redirect :: get_link(\application\weblcms\WeblcmsManager :: APPLICATION_NAME, $params));
    }
}
