<?php
namespace application\weblcms\integration\core\home;

use application\weblcms\ContentObjectPublication;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use repository\content_object\assignment\Assignment;
use common\libraries\AndCondition;
use common\libraries\InCondition;
use common\libraries\Translation;
use repository\ContentObjectRenderer;
use application\weblcms\tool\assignment\AssignmentTool;
use common\libraries\Application;
use common\libraries\Redirect;
use repository\ContentObject;
use common\libraries\SubselectCondition;
use common\libraries\Utilities;
use common\libraries\DatetimeUtilities;
use application\weblcms\course\CourseUserRelation;
use group\DataManager as GroupDataManager;
use application\weblcms\course\CourseGroupRelation;

use application\weblcms\course\DataManager as CourseDataManager;

/**
 * A notificationblock for new assignment submissions (assignmenttool)
 */
class EndingAssignments extends Block
{

    public function display_content()
    {
        // deadline min 1 week (60 * 60 * 24 * 7)
        $deadline = time() + 604800;


        $condition = new EqualityCondition(CourseUserRelation :: PROPERTY_USER_ID, $this->get_user_id(),
                CourseUserRelation :: get_table_name());
        $courses = CourseDataManager :: retrieve_user_courses($condition)->as_array();

        $groups = GroupDataManager :: retrieve_all_subscribed_groups_array($this->get_user_id());

        $group_ids = array();
        foreach ($groups as $group)
        {
            $group_ids[$group->get_id()] = $group->get_id();
        }

        $condition = new InCondition(CourseGroupRelation :: PROPERTY_GROUP_ID, $group_ids,
                CourseGroupRelation :: get_table_name());
        array_splice($courses, 0, 0, CourseDataManager :: retrieve_user_courses($condition)->as_array());

        $course_ids = array();
        foreach ($courses as $course)
        {
            $course_ids[$course->get_id()] = $course->get_id();
        }

        $conditions = array();
        $conditions[] = new InCondition(\application\weblcms\ContentObjectPublication :: PROPERTY_COURSE_ID, $course_ids);
        $conditions[] = new EqualityCondition(\application\weblcms\ContentObjectPublication :: PROPERTY_TOOL,
                'assignment');

        $subselect_condition = new EqualityCondition(
            new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_TYPE),
            new StaticConditionVariable(Assignment :: class_name())
        );

        $conditions[] = new SubselectCondition(
                new PropertyConditionVariable(
                    ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID
                ),
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID),
                null, $subselect_condition
        );
        $condition = new AndCondition($conditions);

        $publications = \application\weblcms\DataManager :: retrieves(
            ContentObjectPublication :: class_name(),
            new DataClassRetrievesParameters($condition, null, null, array(
                new ObjectTableOrder(
                    \application\weblcms\ContentObjectPublication :: PROPERTY_DISPLAY_ORDER_INDEX, SORT_DESC)
                )
            )
        )->as_array();

        $ending_assignments = array();
        foreach ($publications as $publication)
        {
            $assignment = $publication->get_content_object();
            if ($assignment->get_end_time() > time() && $assignment->get_end_time() < $deadline)
            {
                $parameters = array(
                        \application\weblcms\WeblcmsManager :: PARAM_COURSE => $publication->get_course_id(),
                        Application :: PARAM_ACTION => \application\weblcms\WeblcmsManager :: ACTION_VIEW_COURSE,
                        \application\weblcms\WeblcmsManager :: PARAM_TOOL => Assignment :: class_name(),
                        \application\weblcms\WeblcmsManager :: PARAM_TOOL_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS,
                        \application\weblcms\Tool :: PARAM_BROWSER_TYPE => ContentObjectRenderer :: TYPE_TABLE,
                        \application\weblcms\Tool :: PARAM_PUBLICATION_ID => $publication->get_id());
                $link = Redirect :: get_link(\application\weblcms\WeblcmsManager :: APPLICATION_NAME, $parameters);
                $ending_assignments[$assignment->get_end_time() . ' ' . $publication->get_id()] = array(
                        'title' => $assignment->get_title(), 'link' => $link, 'end_time' => $assignment->get_end_time());
            }
        }

        ksort($ending_assignments);
        $html = $this->display_new_items($ending_assignments);

        if (count($html) == 0)
        {
            return Translation :: get('NoAssignmentsEndComingWeek');
        }

        return implode("\n", $html);
    }

    public function display_new_items($items)
    {
        $html = array();
        foreach ($items as $item)
        {
            $end_date = DatetimeUtilities :: format_locale_date(
                    Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' . Translation :: get(
                            'TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES), $item['end_time']);

            $html[] = '<a href="' . $item[link] . '">' . $item[title] . '</a>: ' . Translation :: get('Until') . ' ' . $end_date . '<br />';
        }
        return $html;
    }
}
