<?php
namespace application\weblcms\integration\core\home;

use application\weblcms\ContentObjectPublication;
use repository\content_object\assignment\Assignment;
use repository\ContentObjectRenderer;
use application\weblcms\tool\assignment\AssignmentTool;
use common\libraries\Application;
use common\libraries\Redirect;
use common\libraries\Utilities;
use common\libraries\DatetimeUtilities;
use common\libraries\Theme;
use common\libraries\Translation;
use repository\ContentObject;

/**
 * A notificationblock for new assignment submissions (assignmenttool)
 */
class NewAssignments extends NewBlock
{

    public function display_content()
    {
        $publications = $this->get_content(self :: TOOL_ASSIGNMENT);
        $html = $this->display_new_items($publications);

        if (count($html) < 3)
        {
            return Translation :: get('NoNewAssignmentsSinceLastVisit');
        }
        return implode("\n", $html);
    }

    public function display_new_items($publications)
    {
        ksort($publications);
        $icon = '<img src="' . $this->get_new_assignments_icon() . '"/>';

        $html = array();
        $html[] = '<ul style="padding: 0px; margin: 0px 0px 0px 15px;">';
        $current_course_id = - 1;
        foreach ($publications as $publication)
        {
            $course_id = $publication[ContentObjectPublication :: PROPERTY_COURSE_ID];
            $id = $publication[ContentObjectPublication :: PROPERTY_ID];

            if($publication[ContentObject::PROPERTY_TYPE] != Assignment :: class_name())
                continue;

            $content_object = \repository\DataManager :: retrieve_content_object(
                $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID], Assignment :: class_name()
            );

            if ($course_id != $current_course_id)
            {
                $current_course_id = $course_id;
                $html[] = '<li>' . $this->get_course_by_id($current_course_id)->get_title() . '</li>';
            }

            $parameters = array(
                \application\weblcms\WeblcmsManager :: PARAM_COURSE => $course_id,
                Application :: PARAM_ACTION => \application\weblcms\WeblcmsManager :: ACTION_VIEW_COURSE,
                \application\weblcms\WeblcmsManager :: PARAM_TOOL => Assignment :: class_name(),
                \application\weblcms\WeblcmsManager :: PARAM_TOOL_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS,
                \application\weblcms\Tool :: PARAM_BROWSER_TYPE => ContentObjectRenderer :: TYPE_TABLE,
                \application\weblcms\Tool :: PARAM_PUBLICATION_ID => $id
            );

            $link = Redirect :: get_link(\application\weblcms\WeblcmsManager :: APPLICATION_NAME, $parameters);

            $start_date = DatetimeUtilities :: format_locale_date(
                Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES),
                $content_object->get_start_time()
            );
            $end_date = DatetimeUtilities :: format_locale_date(
                Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES),
                $content_object->get_end_time()
            );
            $html[] =
                '<a href="' . $link . '">' . $icon . ' ' . $content_object->get_title() . '</a>: ' . Translation :: get(
                    'From'
                ) . ' ' . $start_date . ' ' . Translation :: get('Until') . ' ' . $end_date . '<br />';
        }
        $html[] = '</ul>';
        return $html;
    }

    private function get_new_assignments_icon()
    {
        return Theme :: get_image_path(\application\weblcms\Tool :: get_tool_type_namespace(self :: TOOL_ASSIGNMENT)) .
            'logo/' . Theme :: ICON_MINI . '_new.png';
    }
}
