<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\Request;
use common\libraries\Translation;
use reporting\ReportingData;
use user\User;

/**
 * Shows the information block for a single user with a single question of a single assessment
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class AssessmentQuestionUserInformationBlock extends AssessmentQuestionUsersBlock
{
    public function count_data()
    {
        $question_cid = Request :: get(ReportingTool :: PARAM_QUESTION);
        $complex_question = \repository\DataManager :: retrieve_complex_content_object_item($question_cid);

        $reporting_data = new ReportingData();

        $categories = $this->get_assessment_information_headers();
        $categories = array_merge($categories, $this->get_question_information_headers());
        $categories = array_merge($categories, $this->get_question_user_reporting_info_headers());

        $reporting_data->set_categories($categories);

        $publication = \application\weblcms\DataManager :: retrieve_by_id(
            ContentObjectPublication :: class_name(),
            $this->get_publication_id()
        );

        $this->add_category_from_array(
            Translation :: get('Details'), $this->get_assessment_information($publication), $reporting_data
        );

        $this->add_category_from_array(
            Translation :: get('Details'), $this->get_question_information($complex_question->get_ref_object()),
            $reporting_data
        );

        $user = \user\DataManager :: retrieve(User :: class_name(), $this->get_user_id());
        $user_attempts = $this->get_question_attempts_from_publication_and_question(
            $this->get_publication_id(), $question_cid, $this->get_user_id()
        );

        $this->add_category_from_array(
            Translation :: get('Details'), $this->get_question_user_reporting_info(
                $complex_question, $user, $user_attempts->as_array()
            ),
            $reporting_data
        );

        $reporting_data->set_rows(array(Translation :: get('Details')));

        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
