<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\integration\tracking\AssessmentAttempt;
use common\libraries\DatetimeUtilities;
use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\Theme;
use common\libraries\Redirect;
use common\libraries\Utilities;
use reporting\ReportingManager;
use reporting\Reporting;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use repository\content_object\assessment\Assessment;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overiew of the assessments the user has
 *          attempted
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class CourseUserExerciseInformationBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_rows(
            array(
                Translation :: get('Title'),
                Translation :: get('NumberOfAttempts'),
                Translation :: get('LastAttempt'),
                Translation :: get('TotalTime'),
                Translation :: get('AverageScore'),
                Translation :: get('Attempts')));
        $course_id = $this->get_course_id();
        $user_id = $this->get_user_id();
        $tracker = new \application\weblcms\integration\tracking\AssessmentAttempt();
        $img = '<img src="' . Theme :: get_common_image_path() . 'action_reporting.png" title="' .
             Translation :: get('Details') . '" />';

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_COURSE_ID,
            $course_id);
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_USER_ID,
            $user_id);
        $condition = new AndCondition($conditions);
        $trackerdata = $tracker->retrieve_tracker_items($condition);

        $exercises = array();
        // $score_count = 0;
        foreach ($trackerdata as $key => $value)
        {
            if ($value->get_status() == AssessmentAttempt :: STATUS_COMPLETED)
            {
                $exercises[$value->get_assessment_id()]['score'] += $value->get_total_score();
                $exercises[$value->get_assessment_id()]['score_count'] ++;
            }
            $exercises[$value->get_assessment_id()]['count'] ++;
            $exercises[$value->get_assessment_id()]['total_time'] += $value->get_total_time();

            if ($exercises[$value->get_assessment_id()]['last'] == null ||
                 $value->get_start_time() > $exercises[$value->get_assessment_id()]['last'])
            {
                $exercises[$value->get_assessment_id()]['last'] = $value->get_start_time();
            }
        }

        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $course_id;
        $params[WeblcmsManager :: PARAM_TOOL] = Assessment :: get_type_name();
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = Tool :: ACTION_VIEW;

        $params_detail = $this->get_parent()->get_parameters();
        $params_detail[ReportingManager :: PARAM_TEMPLATE_ID] = AssessmentAttemptsUserTemplate :: class_name();

        $conditions = array();
        $conditions[] = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_TOOL,
            Utilities :: get_classname_from_namespace(Assessment :: class_name(), true));
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course_id);
        $condition = new AndCondition($conditions);
        $publications_resultset = \application\weblcms\DataManager :: retrieve_content_object_publications($condition);

        $key = 0;
        while ($publication = $publications_resultset->next_result())
        {
            if (! \application\weblcms\DataManager :: is_publication_target_user(
                $user_id,
                $publication[ContentObjectPublication :: PROPERTY_ID]))
            {
                continue;
            }
            ++ $key;
            $title = $score = $last = $last = $link = $time = null;
            $count = 0;

            if ($exercises[$publication[ContentObjectPublication :: PROPERTY_ID]])
            {
                $value = $exercises[$publication[ContentObjectPublication :: PROPERTY_ID]];
                $time = mktime(0, 0, $value[total_time], 0, 0, 0);
                $time = date('G:i:s', $time);
                $score = $this->get_score_bar($value['score'] / $value['score_count']);
                $last = DatetimeUtilities :: format_locale_date(
                    Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' .
                         Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                        $value['last']);
                $params_detail[WeblcmsManager :: PARAM_PUBLICATION] = $publication[ContentObjectPublication :: PROPERTY_ID];
                $link = '<a href="' . $this->get_parent()->get_url($params_detail) . '">' . $img . '</a>';
                $count = $value['count'];
            }

            $params[WeblcmsManager :: PARAM_PUBLICATION] = $publication[ContentObjectPublication :: PROPERTY_ID];

            $content_object = \repository\DataManager :: retrieve_content_object(
                $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);

            $title = '<a href="' . Redirect :: get_url($params) . '">' . $content_object->get_title() . '</a>';

            $reporting_data->add_category($key);
            $reporting_data->add_data_category_row($key, Translation :: get('Title'), $title);
            $reporting_data->add_data_category_row($key, Translation :: get('NumberOfAttempts'), $count);
            $reporting_data->add_data_category_row($key, Translation :: get('AverageScore'), $score);
            $reporting_data->add_data_category_row($key, Translation :: get('LastAttempt'), $last);
            $reporting_data->add_data_category_row($key, Translation :: get('TotalTime'), $time);
            $reporting_data->add_data_category_row($key, Translation :: get('Attempts'), $link);
        }
        $reporting_data->hide_categories();
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
