<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\WeblcmsManager;
use application\weblcms\ContentObjectPublication;
use application\weblcms\tool\assignment\AssignmentTool;
use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Utilities;
use common\libraries\DatetimeUtilities;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\Theme;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use reporting\Reporting;
use reporting\ReportingManager;
use repository\content_object\assignment\Assignment;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block displaying all assigments within a course and their
 *          submission stats
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssignmentBlock extends CourseBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_rows(
            array(
                Translation :: get('Title'),
                Translation :: get('NumberOfSubmissions'),
                Translation :: get('LastSubmission'),
                Translation :: get('AverageScore'),
                Translation :: get('AssignmentDetails')));

        $course_id = $this->get_course_id();
        $tool = Utilities :: get_classname_from_namespace(Assignment :: class_name(), true);
        $submissions_tracker = new \application\weblcms\integration\tracking\AssignmentSubmission();
        $score_tracker = new \application\weblcms\integration\tracking\SubmissionScore();
        $img = '<img src="' . Theme :: get_common_image_path() . 'action_statistics.png" title="' .
             Translation :: get('Details') . '" />';
        $count = 1;

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course_id);
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_TOOL, $tool);
        $condition = new AndCondition($conditions);

        $pub_resultset = \application\weblcms\DataManager :: retrieve_content_object_publications($condition);

        while ($pub = $pub_resultset->next_result())
        {
            $condition = new EqualityCondition(
                \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_PUBLICATION_ID,
                $pub[ContentObjectPublication :: PROPERTY_ID]);
            $submissions = $submissions_tracker->retrieve_tracker_items($condition);

            $score = $score_count = $last_submission = null;
            foreach ($submissions as $submission)
            {
                if ($submission->get_date_submitted() > $last_submission)
                {
                    $last_submission = $submission->get_date_submitted();
                }

                $condition = new EqualityCondition(
                    \application\weblcms\integration\tracking\SubmissionScore :: PROPERTY_SUBMISSION_ID,
                    $submission->get_id());
                $result = $score_tracker->retrieve_tracker_items($condition);
                if ($result[0] != null)
                {
                    $score += $result[0]->get_score();
                    $score_count ++;
                }
            }
            if ($last_submission != null)
            {
                $last_submission = DatetimeUtilities :: format_locale_date(
                    Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES) . ', ' .
                         Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                        $last_submission);
            }
            if ($score != null)
            {
                $score = $this->get_score_bar($score / $score_count);
            }

            $params = $this->get_parent()->get_parameters();
            $params[ReportingManager :: PARAM_TEMPLATE_ID] = CourseAssignmentSubmittersTemplate :: class_name();
            $params[WeblcmsManager :: PARAM_PUBLICATION] = $pub[ContentObjectPublication :: PROPERTY_ID];
            $link = '<a href="' . $this->get_parent()->get_url($params) . '">' . $img . '</a>';

            $params = array();
            $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
            $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
            $params[WeblcmsManager :: PARAM_COURSE] = $course_id;
            $params[WeblcmsManager :: PARAM_TOOL] = $tool;
            $params[WeblcmsManager :: PARAM_PUBLICATION] = $pub[ContentObjectPublication :: PROPERTY_ID];
            $params[WeblcmsManager :: PARAM_TOOL_ACTION] = AssignmentTool :: ACTION_BROWSE_SUBMITTERS;
            $url_title = Redirect :: get_url($params);

            $content_object = \repository\DataManager :: retrieve_content_object(
                $pub[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);

            $reporting_data->add_category($count);
            $reporting_data->add_data_category_row(
                $count,
                Translation :: get('Title'),
                '<a href="' . $url_title . '">' . $content_object->get_title() . '</a>');
            $reporting_data->add_data_category_row(
                $count,
                Translation :: get('NumberOfSubmissions'),
                count($submissions));
            $reporting_data->add_data_category_row($count, Translation :: get('LastSubmission'), $last_submission);
            $reporting_data->add_data_category_row($count, Translation :: get('AverageScore'), $score);
            $reporting_data->add_data_category_row($count, Translation :: get('AssignmentDetails'), $link);

            $count ++;
        }

        $reporting_data->hide_categories();

        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
