<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\CourseGroupEntity;
use common\libraries\EqualityCondition;
use application\weblcms\tool\course_group\CourseGroup;
use application\weblcms\tool\course_group\DataManager as CourseGroupDataManager;

/**
 * Reporting block with scores for the course groups.
 *
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssignmentCourseGroupScoresBlock extends AssignmentGroupScoresBlock
{

    /**
     * Returns the submitter type for this reporting block.
     *
     * @return int The submitter type
     */
    public function get_current_submitter_type()
    {
        return \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP;
    }

    /**
     * Returns the groups for this reporting block.
     *
     * @return array The groups
     */
    public function get_groups()
    {
        $condition = new EqualityCondition(CourseGroup :: PROPERTY_PARENT_ID, $this->course_id);

        return CourseGroupDataManager :: retrieves(CourseGroup :: class_name(), $condition)->as_array();
    }

    /**
     * Returns true if the group with the given group id belongs to the given target entities and false otherwise.
     *
     * @param $target_entities array The target entities
     * @param $group_id int The group id
     * @return boolean True if the given group id belongs to the given target entities
     */
    public function is_target_entity_group($target_entities, $group_id)
    {
        foreach ($target_entities[CourseGroupEntity :: ENTITY_TYPE] as $course_group_id)
        {
            if ($group_id == $course_group_id)
            {
                return true;
            }
        }

        return false;
    }
}
