<?php
namespace application\weblcms\integration\reporting;

use common\libraries\Utilities;
use application\weblcms\WeblcmsRights;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\ContentObjectPublication;
use reporting\Reporting;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Request;
use application\weblcms\tool\assignment\AssignmentTool;
use repository\ContentObjectRenderer;
use repository\content_object\assignment\Assignment;
use common\libraries\ObjectTableOrder;
use common\libraries\InCondition;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overview of scores of each assigment per
 *          group
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Bert De Clercq (Hogeschool Gent)
 */
abstract class AssignmentGroupScoresBlock extends AssignmentReportingManager
{

    private $reporting_data;

    public function count_data()
    {
        if (! isset($this->reporting_data))
        {
            $this->reporting_data = new ReportingData();

            $this->course_id = $this->get_course_id();

            $conditions = array();
            $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $this->course_id);

            $conditions[] = new EqualityCondition(
                ContentObjectPublication :: PROPERTY_TOOL,
                Utilities :: get_classname_from_namespace(Assignment :: class_name()));

            $condition = new AndCondition($conditions);
            $order_by = new ObjectTableOrder(ContentObjectPublication :: PROPERTY_MODIFIED_DATE, SORT_DESC);
            $publication_resultset = \application\weblcms\DataManager :: retrieve_content_object_publications(
                $condition,
                $order_by);

            $publications = array();
            // fill publications with group assignments
            while ($publication = $publication_resultset->next_result())
            {
                $content_object = \repository\DataManager :: retrieve_content_object(
                    $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);

                $method = 'get_allow_group_submissions';
                if (method_exists($content_object, $method) && $content_object->$method())
                {
                    $publications[] = $publication;
                }
            }

            // set the table headers
            $headings = array();
            $headings[] = Translation :: get('Name');

            foreach ($publications as $publication)
            {
                $content_object = \repository\DataManager :: retrieve_content_object(
                    $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);

                if (count($publications) > 5)
                {
                    $headings[] = '<div id="' . $publication[ContentObjectPublication :: PROPERTY_ID] . '">' .
                         substr($content_object->get_title(), 0, 14) . '</div>';
                }
                else
                {
                    $headings[] = '<div id="' . $publication[ContentObjectPublication :: PROPERTY_ID] . '">' . $content_object->get_title() . '</div>';
                }
            }
            $this->reporting_data->set_rows($headings);

            $groups = $this->get_groups();
            // traverse groups
            foreach ($groups as $key => $group)
            {
                $this->reporting_data->add_category($key);
                $this->reporting_data->add_data_category_row($key, Translation :: get('Name'), $group->get_name());

                foreach ($publications as $publication)
                {
                    $content_object = \repository\DataManager :: retrieve_content_object(
                        $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);

                    if (count($publications) > 5)
                    {
                        $title = '<div id="' . $publication[ContentObjectPublication :: PROPERTY_ID] . '">' .
                             substr($content_object->get_title(), 0, 14) . '</div>';
                    }
                    else
                    {
                        $title = '<div id="' . $publication[ContentObjectPublication :: PROPERTY_ID] . '">' .
                             $content_object->get_title() . '</div>';
                    }

                    $submission_tracker = new \application\weblcms\integration\tracking\AssignmentSubmission();
                    $conditions = array();

                    $conditions[] = new EqualityCondition(
                        \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_PUBLICATION_ID,
                        $publication[ContentObjectPublication :: PROPERTY_ID]);

                    $conditions[] = new EqualityCondition(
                        \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_TYPE,
                        $this->get_current_submitter_type());

                    $conditions[] = new EqualityCondition(
                        \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_SUBMITTER_ID,
                        $group->get_id());

                    $condition = new AndCondition($conditions);
                    $submissions_by_group = $submission_tracker->retrieve_tracker_items($condition);

                    $submission_ids = array();
                    foreach ($submissions_by_group as $submission)
                    {
                        $submission_ids[] = $submission->get_id();
                    }

                    if (count($submission_ids) == 0)
                    {
                        if ($this->is_publication_target_group($group->get_id(), $publication[ContentObjectPublication :: PROPERTY_ID]))
                        {
                            $this->reporting_data->add_data_category_row($key, $title, null);
                            continue;
                        }

                        $this->reporting_data->add_data_category_row($key, $title, 'X');
                        continue;
                    }

                    $score_tracker = new \application\weblcms\integration\tracking\SubmissionScore();
                    $condition = new InCondition(
                        \application\weblcms\integration\tracking\SubmissionScore :: PROPERTY_SUBMISSION_ID,
                        $submission_ids);
                    $score_trackers = $score_tracker->retrieve_tracker_items($condition);

                    if (count($score_trackers) > 0)
                    {
                        $score = $this->format_score_html($this->get_score($score_trackers));

                        $this->reporting_data->add_data_category_row($key, $title, $score);
                    }
                    else
                    {
                        $params = array();
                        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
                        $params[WeblcmsManager :: PARAM_COURSE] = $this->course_id;
                        $params[WeblcmsManager :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
                        $params[WeblcmsManager :: PARAM_TOOL] = Utilities :: camelcase_to_underscores(
                            Utilities :: get_classname_from_namespace(Assignment :: class_name())
                        );
                        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = AssignmentTool :: ACTION_BROWSE_SUBMISSIONS;
                        $params[Tool :: ACTION_BROWSE] = ContentObjectRenderer :: TYPE_TABLE;
                        $params[WeblcmsManager :: PARAM_PUBLICATION] = $publication[ContentObjectPublication :: PROPERTY_ID];
                        $params[AssignmentTool :: PARAM_TARGET_ID] = $group->get_id();
                        $params[AssignmentTool :: PARAM_SUBMITTER_TYPE] = $this->get_current_submitter_type();

                        $link = Redirect :: get_url($params);
                        $this->reporting_data->add_data_category_row(
                            $key,
                            $title,
                            '<span style="text-decoration: blink;"><b><a href="' . $link . '">?</a></b></span>');
                    }
                }
            }

            $this->reporting_data->hide_categories();
        }

        return $this->reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }

    /**
     * This function should return the submitter type as an int.
     */
    abstract public function get_current_submitter_type();

    /**
     * This function should return the groups as an array.
     */
    abstract public function get_groups();

    /**
     * This function check whether the group with the given group id belongs to the given target entities.
     */
    abstract public function is_target_entity_group($target_entities, $group_id);

    private function is_publication_target_group($group_id, $pub_id)
    {
        $target_entities = WeblcmsRights :: get_instance()->get_target_entities(
            WeblcmsRights :: VIEW_RIGHT,
            WeblcmsManager :: context(),
            $pub_id,
            WeblcmsRights :: TYPE_PUBLICATION,
            $this->course_id,
            WeblcmsRights :: TREE_TYPE_COURSE);

        if ($target_entities[0] || count($target_entities) == 0)
        {
            return true;
        }

        return $this->is_target_entity_group($target_entities, $group_id);
    }

    /**
     * Returns the score from the given score trackers. The type of score is determined by the request. If no score type
     * is found, it will return the average score.
     *
     * @param $score_trackers \application\weblcms\integration\tracking\SubmissionScore The score trackers used to
     *            determine the score
     * @return int The score
     */
    private function get_score($score_trackers)
    {
        $score_type = (Request :: post('sel')) ? Request :: post('sel') : Request :: get('sel');

        return $this->get_score_by_type($score_trackers, $score_type);
    }
}
