<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\tool\learning_path\LearningPathTool;
use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\Redirect;
use common\libraries\Theme;
use common\libraries\Utilities;
use reporting\ReportingManager;
use reporting\Reporting;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use repository\content_object\learning_path\LearningPath;

/**
 *
 * @package application.weblcms.php.reporting.blocks Reporting block with an overiew of the learning paths the user has
 *          attempted
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class CourseUserLearningPathInformationBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_rows(
            array(
                Translation :: get('Title'),
                Translation :: get('Progress'),
                Translation :: get('LearningPathDetails')));
        $course_id = $this->get_course_id();
        $user_id = $this->get_user_id();
        $tracker = new \application\weblcms\integration\tracking\LpAttempt();
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_COURSE_ID,
            $course_id);
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_USER_ID,
            $user_id);
        $condition = new AndCondition($conditions);
        $trackerdata = $tracker->retrieve_tracker_items($condition);

        foreach ($trackerdata as $tracker)
        {
            $learning_paths[$tracker->get_lp_id()] = $tracker;
        }

        $params = array();
        $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
        $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
        $params[WeblcmsManager :: PARAM_COURSE] = $course_id;
        $params[WeblcmsManager :: PARAM_TOOL] = LearningPath :: get_type_name();
        $params[WeblcmsManager :: PARAM_TOOL_ACTION] = Tool :: ACTION_VIEW;

        $params_detail = $this->get_parent()->get_parameters();
        $params_detail[ReportingManager :: PARAM_TEMPLATE_ID] = LearningPathAttemptProgressTemplate :: class_name();
        $img = '<img src="' . Theme :: get_common_image_path() . 'action_reporting.png" title="' .
             Translation :: get('Details') . '" />';

        $conditions = array();
        $conditions[] = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_TOOL,
            Utilities :: get_classname_from_namespace(LearningPath :: class_name(), true));
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $course_id);
        $condition = new AndCondition($conditions);
        $publications_resultset = \application\weblcms\DataManager :: retrieve_content_object_publications($condition);
        /*
         * $publications_arr = $publications_resultset->as_array();
         */
        $key = 0;
        while ($publication = $publications_resultset->next_result())
        {
            $progress = $url = $link = null;
            if (! \application\weblcms\DataManager :: is_publication_target_user(
                $user_id,
                $publication[ContentObjectPublication :: PROPERTY_ID]))
            {
                continue;
            }
            ++ $key;

            if ($learning_paths[$publication[ContentObjectPublication :: PROPERTY_ID]])
            {
                $params_detail[WeblcmsManager :: PARAM_PUBLICATION] = $publication[ContentObjectPublication :: PROPERTY_ID];
                $params_detail[LearningPathTool :: PARAM_ATTEMPT_ID] = $learning_paths[$publication[ContentObjectPublication :: PROPERTY_ID]];
                $link = '<a href="' . $this->get_parent()->get_url($params_detail) . '">' . $img . '</a>';
                $progress = $this->get_progress_bar(
                    $learning_paths[$publication[ContentObjectPublication :: PROPERTY_ID]]->get_progress());
            }

            $params[WeblcmsManager :: PARAM_PUBLICATION] = $publication[ContentObjectPublication :: PROPERTY_ID];
            $learning_path = \repository\DataManager :: retrieve_content_object(
                $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);
            $url = '<a href="' . Redirect :: get_url($params) . '">' . $learning_path->get_title() . '</a>';

            $reporting_data->add_category($key);
            $reporting_data->add_data_category_row($key, Translation :: get('Title'), $url);
            $reporting_data->add_data_category_row($key, Translation :: get('Progress'), $progress);
            $reporting_data->add_data_category_row($key, Translation :: get('LearningPathDetails'), $link);
        }

        $reporting_data->hide_categories();
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
