<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\WeblcmsManager;
use application\weblcms\tool\learning_path\LearningPathTool;
use application\weblcms\tool\learning_path\LearningPathToolStatisticsViewerComponent;
use common\libraries\ToolbarItem;
use reporting\Reporting;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use reporting\ReportingManager;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Redirect;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Text;
use common\libraries\Utilities;

class LearningPathAttemptsBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();
        $reporting_data->set_rows(
            array(Translation :: get('Name'), Translation :: get('Progress'), Translation :: get('Details')));

        $pid = $this->get_publication_id();
        $course_id = $this->get_course_id();
        $tool = $this->get_tool();

        $users_resultset = \application\weblcms\DataManager :: retrieve_publication_target_users($pid, $course_id);

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_COURSE_ID,
            $course_id);
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_LP_ID,
            $pid);
        $condition = new AndCondition($conditions);

        $dummy = new \application\weblcms\integration\tracking\LpAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);
        $user_progress = array();
        foreach ($trackers as $tracker)
        {
            $user_progress[$tracker->get_user_id()] = $tracker;
        }

        $count = 1;
        $params = $this->get_parent()->get_parameters();
        $params[ReportingManager :: PARAM_TEMPLATE_ID] = LearningPathAttemptProgressTemplate :: class_name();
        $img = '<img src="' . Theme :: get_common_image_path() . 'action_reporting.png" title="' .
             Translation :: get('Details') . '" />';

        while ($user = $users_resultset->next_result())
        {
            $reporting_data->add_category($count);
            $reporting_data->add_data_category_row($count, Translation :: get('Name'), $user->get_fullname());

            $tracker = $user_progress[$user->get_id()];
            if ($tracker)
            {
                $params[LearningPathTool :: PARAM_ATTEMPT_ID] = $tracker->get_id();
                $params[WeblcmsManager :: PARAM_USERS] = $user->get_id();
                $params[LearningPathToolStatisticsViewerComponent :: PARAM_STAT] = null;
                $url = Redirect :: get_url($params);

                $params[LearningPathToolStatisticsViewerComponent :: PARAM_STAT] = LearningPathToolStatisticsViewerComponent :: ACTION_DELETE_LP_ATTEMPT;
                $delete_url = Redirect :: get_url($params);

                if ($tool == 'reporting')
                {
                    $action = '<a href="' . $url . '">' . $img . '</a>';
                }
                else
                {
                    $action = Text :: create_link(
                        $url, Theme :: get_common_image(
                            'action_reporting', 'png',
                            Translation :: get('Details'), null,
                            ToolbarItem::DISPLAY_ICON
                        )
                    );

                    $action .= ' ' . Text :: create_link(
                        $delete_url, Theme :: get_common_image(
                            'action_delete', 'png', Translation :: get('Delete', null, Utilities::COMMON_LIBRARIES),
                            null, ToolbarItem::DISPLAY_ICON
                        )
                    );
                }

                $progress = $this->get_progress_bar($tracker->get_progress());

                $reporting_data->add_data_category_row($count, Translation :: get('Progress'), $progress);
                $reporting_data->add_data_category_row($count, Translation :: get('Details'), $action);
            }
            $count ++;
        }

        $reporting_data->hide_categories();
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
