<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\ContentObjectPublication;
use application\weblcms\integration\tracking\CourseVisit;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\PropertyConditionVariable;
use common\libraries\Redirect;
use common\libraries\Application;
use common\libraries\StaticConditionVariable;
use common\libraries\Utilities;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\Request;
use common\libraries\DatetimeUtilities;
use reporting\Reporting;
use reporting\ReportingManager;
use reporting\ReportingData;
use reporting\ReportingFormatter;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;

class ToolPublicationsBlock extends ToolBlock
{

    public function count_data()
    {
        $reporting_data = new ReportingData();

        $course_id = $this->get_course_id();
        $user_id = $this->get_user_id();
        $tool = Request :: get(ReportingTool :: PARAM_REPORTING_TOOL);

        $reporting_data->set_rows(
            array(
                Translation :: get('Title'),
                Translation :: get('Description'),
            )
        );

        $this->add_reporting_data_rows_for_course_visit_data($reporting_data);

        if (is_null($user_id))
        {
            $reporting_data->add_row(Translation :: get('AccessDetails'));
        }

        $conditions = array();

        if ($user_id)
        {
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_PUBLISHER_ID),
                new StaticConditionVariable($user_id)
            );
        }

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_COURSE_ID),
            new StaticConditionVariable($course_id)
        );

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_TOOL),
            new StaticConditionVariable($tool)
        );

        $condition = new AndCondition($conditions);

        $content_object_publications = \application\weblcms\DataManager :: retrieves(
            ContentObjectPublication :: class_name(), $condition
        );

        $img = '<img src="' . Theme :: get_common_image_path() . 'action_reporting.png" title="' .
            Translation :: get('Details') . '" />';

        $index = 1;
        while ($content_object_publication = $content_object_publications->next_result())
        {
            $params = array();

            $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
            $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
            $params[WeblcmsManager :: PARAM_COURSE] = $course_id;
            $params[WeblcmsManager :: PARAM_TOOL] = $tool;
            $params[WeblcmsManager :: PARAM_PUBLICATION] = $content_object_publication->get_id();
            $params[WeblcmsManager :: PARAM_TOOL_ACTION] = Tool :: ACTION_VIEW;

            $url = Redirect :: get_url($params);

            $content_object = \repository\DataManager :: retrieve_content_object(
                $content_object_publication->get_content_object_id()
            );

            $des = $content_object->get_description();
            $this->get_parent()->set_parameter($content_object_publication->get_id());
            $this->set_params($course_id, $user_id, $tool, $this->get_publication_id());

            $reporting_data->add_category($index);

            $reporting_data->add_data_category_row(
                $index,
                Translation :: get('Title'),
                '<a href="' . $url . '">' . $content_object->get_title() . '</a>'
            );

            $reporting_data->add_data_category_row(
                $index,
                Translation :: get('Description'),
                Utilities :: truncate_string($des, 50)
            );

            $course_visit_data = $this->get_course_visit_summary_from_publication($content_object_publication);
            $this->add_reporting_data_from_course_visit_as_row($index, $reporting_data, $course_visit_data);

            if (is_null($user_id))
            {
                $params = $this->get_parent()->get_parameters();

                $params[ReportingManager :: PARAM_TEMPLATE_ID] = PublicationDetailTemplate :: class_name();
                $params[WeblcmsManager :: PARAM_COURSE] = $course_id;
                $params[WeblcmsManager :: PARAM_USERS] = $user_id;
                $params[WeblcmsManager :: PARAM_PUBLICATION] = $content_object_publication->get_id();

                $link = '<a href="' . $this->get_parent()->get_url($params) . '">' . $img . '</a>';

                $reporting_data->add_data_category_row($index, Translation :: get('AccessDetails'), $link);
            }

            $index ++;
        }

        $reporting_data->hide_categories();
        return $reporting_data;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }
}
