<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\CourseTool;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\integration\tracking\CourseVisit;
use common\libraries\AndCondition;
use common\libraries\Application;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\Redirect;
use common\libraries\StaticConditionVariable;
use common\libraries\Translation;
use reporting\ReportingData;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;

/**
 * Base class to display the access to the tools ToolAccessBlock
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 * @package application\weblcms\integration\reporting
 */
abstract class ToolAccessBlock extends ToolBlock
{
    public function count_data()
    {
        $reporting_data = new ReportingData();

        $reporting_data->set_rows(
            array(
                Translation :: get('Tool'),
                Translation :: get('FirstAccess'),
                Translation :: get('LastAccess'),
                Translation :: get('TotalVisits'),
                Translation :: get('TotalTime'),
                Translation :: get('TotalPublications')
            )
        );

        $course_id = $this->get_course_id();

        $course_tools_summary_data = $this->retrieve_course_summary_data();

        while($course_tool_summary_data = $course_tools_summary_data->next_result())
        {
            $tool_name = $course_tool_summary_data[CourseTool :: PROPERTY_NAME];

            $tool_translation = Translation :: get('TypeName', null, Tool :: get_tool_type_namespace($tool_name));

            $params = array();

            $params[Application :: PARAM_ACTION] = WeblcmsManager :: ACTION_VIEW_COURSE;
            $params[Application :: PARAM_APPLICATION] = WeblcmsManager :: APPLICATION_NAME;
            $params[WeblcmsManager :: PARAM_COURSE] = $this->get_course_id();
            $params[WeblcmsManager :: PARAM_TOOL] = $tool_name;

            $link = ' <a href="' . Redirect :: get_url($params) . '">' . $tool_translation . '</a>';

            $reporting_data->add_category($tool_name);
            $reporting_data->add_data_category_row($tool_name, Translation :: get('Tool'), $link);


            $reporting_data->add_data_category_row(
                $tool_name,
                Translation :: get('FirstAccess'),
                $this->format_date($course_tool_summary_data[CourseVisit :: PROPERTY_FIRST_ACCESS_DATE])
            );

            $reporting_data->add_data_category_row(
                $tool_name,
                Translation :: get('LastAccess'),
                $this->format_date($course_tool_summary_data[CourseVisit :: PROPERTY_LAST_ACCESS_DATE])
            );

            $reporting_data->add_data_category_row(
                $tool_name,
                Translation :: get('TotalVisits'),
                $course_tool_summary_data[CourseVisit :: PROPERTY_TOTAL_NUMBER_OF_ACCESS] ?
                    $course_tool_summary_data[CourseVisit :: PROPERTY_TOTAL_NUMBER_OF_ACCESS] : 0
            );

            $reporting_data->add_data_category_row(
                $tool_name,
                Translation :: get('TotalTime'),
                $this->convert_seconds_to_hours($course_tool_summary_data[CourseVisit :: PROPERTY_TOTAL_TIME])
            );

            $reporting_data->add_data_category_row(
                $tool_name, Translation :: get('TotalPublications'),
                $this->count_tool_publications($tool_name)
            );
        }

        $reporting_data->hide_categories();
        return $reporting_data;
    }

    /**
     * Counts the publications of a tool
     *
     * @param int $course_id
     * @param string $tool_name
     *
     * @return int
     */
    public function count_tool_publications($tool_name)
    {
        return \application\weblcms\DataManager :: count_content_object_publications(
            $this->get_tool_publications_condition($tool_name)
        );
    }

    /**
     * Returns the condition for the tools publication count
     *
     * @param string $tool_name
     *
     * @return AndCondition
     */
    public function get_tool_publications_condition($tool_name)
    {
        return new EqualityCondition(
            new PropertyConditionVariable(
                ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_TOOL
            ),
            new StaticConditionVariable($tool_name)
        );
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_views()
    {
        return array(\reporting\viewer\HtmlBlockRendition :: VIEW_TABLE);
    }

    /**
     * Returns the summary data for this course
     *
     * @return RecordResultSet
     */
    abstract public function retrieve_course_summary_data();
}
