<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\WeblcmsManager;
use application\weblcms\ContentObjectPublication;
use common\libraries\BreadcrumbTrail;
use common\libraries\Request;
use common\libraries\Translation;
use reporting\ReportingTemplate;
use common\libraries\Breadcrumb;
use user\UserDataManager;
use reporting\ReportingManager;
use reporting\Reporting;
use common\libraries\Utilities;

/**
 *
 * @package application.weblcms.php.reporting.templates Reporting template with an overview of the assessment attempts
 *          of one user
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssessmentAttemptsUserTemplate extends ReportingTemplate
{

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $this->initialize_parameters();
        $this->add_reporting_block($this->get_user_assessment_information());
        $this->add_reporting_block($this->get_user_assessment_attempts());

        $this->add_breadcrumbs();
    }

    public function initialize_parameters()
    {
        $this->publication_id = Request :: get(WeblcmsManager :: PARAM_PUBLICATION);
        $this->user_id = request :: get(WeblcmsManager :: PARAM_USERS);

        if ($this->publication_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_PUBLICATION, $this->publication_id);
        }

        if ($this->user_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_USERS, $this->user_id);
        }

        $course_id = Request :: get(WeblcmsManager :: PARAM_COURSE);
        if ($course_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_COURSE, $course_id);
        }
    }

    public function display_context()
    {
    }

    public function get_application()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    public function get_user_assessment_attempts()
    {
        $assessment_block = new AssessmentAttemptsUserBlock($this);
        $assessment_block->set_sorting(0, SORT_DESC);
        return $assessment_block;
    }

    public function get_user_assessment_information()
    {
        $assessment_block = new AssessmentUserInformationBlock($this);
        return $assessment_block;
    }

    /**
     * Adds the breadcrumbs to the breadcrumbtrail
     */
    protected function add_breadcrumbs()
    {
        $assessment = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id
        )->get_content_object();

        $trail = BreadcrumbTrail :: get_instance();

        $trail->add(
            new Breadcrumb(
                $this->get_url(
                    array(\reporting\Viewer\Manager :: PARAM_BLOCK_ID => 2),
                    array(ReportingManager :: PARAM_TEMPLATE_ID)
                ),
                Translation :: get('Assessments')
            )
        );

        $filters = array(WeblcmsManager :: PARAM_USERS);

        $params = array();
        $params[ReportingManager :: PARAM_TEMPLATE_ID] = AssessmentAttemptsTemplate :: class_name();
        $params[WeblcmsManager :: PARAM_PUBLICATION] = $this->publication_id;

        $trail->add(
            new Breadcrumb($this->get_url($params, $filters), $assessment->get_title())
        );

        $params[\reporting\Viewer\Manager :: PARAM_BLOCK_ID] = 1;

        $trail->add(
            new Breadcrumb($this->get_url($params, $filters), Translation :: get('Users'))
        );

        $trail->add(
            new Breadcrumb($this->get_url(), \user\DataManager :: get_fullname_from_user($this->user_id))
        );
    }
}
