<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsManager;
use common\libraries\Request;
use application\weblcms\tool\course_group\CourseGroup;
use reporting\ReportingTemplate;
use application\weblcms\tool\assignment\AssignmentTool;
use common\libraries\Breadcrumb;
use user\UserDataManager;
use group\DataManager as GroupDataManager;
use group\Group;
use reporting\Reporting;
use common\libraries\Utilities;
use reporting\ReportingManager;
use application\weblcms\tool\course_group\DataManager as CourseGroupDataManager;

/**
 *
 * @package application.weblcms.php.reporting.templates Reporting template with an overview of the assignment
 *          submissions from a user/group
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class CourseSubmitterSubmissionsTemplate extends ReportingTemplate
{

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $this->init_parameters();
        $this->add_reporting_block($this->get_submitter_submissions_information());
        $this->add_reporting_block($this->get_submitter_submissions_detail());

        $assignment = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id)->get_content_object();

        $custom_breadcrumbs = array();
        $params = array();
        $params[AssignmentTool :: PARAM_SUBMITTER_TYPE] = null;
        $params[AssignmentTool :: PARAM_TARGET_ID] = null;
        $params[ReportingManager :: PARAM_TEMPLATE_ID] = CourseAssignmentSubmittersTemplate :: class_name();
        $custom_breadcrumbs[] = new Breadcrumb($this->get_url($params), $assignment->get_title());
        if ($this->submitter_type ==
             \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER)
        {
            $user_data_manager = UserDataManager :: get_instance();
            $user = $user_data_manager->retrieve_user($this->target_id);
            $custom_breadcrumbs[] = new Breadcrumb($this->get_url(), $user->get_fullname());
        }
        if ($this->submitter_type ==
             \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP)
        {
            $course_group = CourseGroupDataManager :: retrieve_by_id(CourseGroup :: class_name(), $this->target_id);
            $custom_breadcrumbs[] = new Breadcrumb($this->get_url(), $course_group->get_name());
        }
        if ($this->submitter_type ==
             \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP)
        {
            $group = GroupDataManager :: retrieve_by_id(Group :: class_name(), $this->target_id);
            $custom_breadcrumbs[] = new Breadcrumb($this->get_url(), $group->get_name());
        }
        $this->set_custom_breadcrumb_trail($custom_breadcrumbs);
    }

    public function display_context()
    {
        // publicatie, content_object, application ...
    }

    private function init_parameters()
    {
        $this->publication_id = Request :: get(WeblcmsManager :: PARAM_PUBLICATION);
        if ($this->publication_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_PUBLICATION, $this->publication_id);
        }

        $this->target_id = Request :: get(AssignmentTool :: PARAM_TARGET_ID);
        if ($this->target_id)
        {
            $this->set_parameter(AssignmentTool :: PARAM_TARGET_ID, $this->target_id);
        }

        $this->submitter_type = Request :: get(AssignmentTool :: PARAM_SUBMITTER_TYPE);
        if (isset($this->submitter_type))
        {
            $this->set_parameter(AssignmentTool :: PARAM_SUBMITTER_TYPE, $this->submitter_type);
        }
    }

    public function get_application()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    public function get_submitter_submissions_detail()
    {
        $submitter_submissions_block = new AssignmentSubmitterSubmissionsBlock($this);
        if ($this->submitter_type ==
             \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER)
        {
            $submitter_submissions_block->set_sorting(2, SORT_DESC);
        }
        else
        {
            $submitter_submissions_block->set_sorting(3, SORT_DESC);
        }
        return $submitter_submissions_block;
    }

    public function get_submitter_submissions_information()
    {
        switch ($this->submitter_type)
        {
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                return new AssignmentCourseGroupInformationBlock($this);
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                return new AssignmentPlatformGroupInformationBlock($this);
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER :
                return new AssignmentUserInformationBlock($this);
        }
    }
}
