<?php
namespace application\weblcms\integration\reporting;

use repository\DataManager;
use common\libraries\Utilities;
use application\weblcms\ContentObjectPublication;
use application\weblcms\WeblcmsManager;
use common\libraries\Request;
use reporting\ReportingTemplate;
use common\libraries\Breadcrumb;
use common\libraries\Translation;
use reporting\Reporting;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\ObjectTableOrder;
use repository\content_object\learning_path\LearningPath;

/**
 *
 * @package application.weblcms.php.reporting.templates Reporting template with an overview of the progress of each
 *          learning path per user
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class LearningPathProgressTemplate extends ReportingTemplate
{

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $this->course_id = Request :: get(WeblcmsManager :: PARAM_COURSE);
        $this->set_parameter(WeblcmsManager :: PARAM_COURSE, $this->course_id);

        $custom_breadcrumbs = array();
        $custom_breadcrumbs[] = new Breadcrumb($this->get_url(), Translation :: get('LearningPathProgress'));
        $this->set_custom_breadcrumb_trail($custom_breadcrumbs);

        // learning path titles for tooltips
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $this->course_id);
        $conditions[] = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_TOOL,
            Utilities :: get_classname_from_namespace(LearningPath :: class_name(), true));
        $condition = new AndCondition($conditions);
        $order_by = new ObjectTableOrder(ContentObjectPublication :: PROPERTY_MODIFIED_DATE, SORT_DESC);
        $publications = \application\weblcms\DataManager :: retrieve_content_object_publications($condition, $order_by);

        while ($publication = $publications->next_result())
        {
            $content_object = \repository\DataManager :: retrieve_content_object(
                $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID]);
            $this->th_titles[] = $content_object->get_title();
        }

        $this->add_reporting_block($this->get_learning_path_progress_users());
    }

    public function display_context()
    {
    }

    public function get_application()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    public function get_learning_path_progress_users()
    {
        $lp_block = new LearningPathProgressUsersBlock($this, true);
        $lp_block->set_sorting(0);

        $legend[] = 'X : ' . Translation :: get('NotPublishedForUser') . '<br />';
        $legend[] = '- : ' . Translation :: get('NotAttemptedByUser') . '<br />';
        $lp_block->set_legend($legend);

        return $lp_block;
    }
}
