<?php
namespace application\weblcms\integration\reporting;

use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\ObjectNotExistException;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use reporting\ReportingTemplate;
use reporting\ReportingManager;
use reporting\Reporting;

/**
 * $Id: publication_detail_reporting_template.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.reporting.templates
 */
/**
 *
 * @author Michael Kyndt
 */
class PublicationDetailTemplate extends ReportingTemplate
{

    public function __construct($parent)
    {
        parent :: __construct($parent);

        $this->tool = Request :: get(ReportingTool :: PARAM_REPORTING_TOOL);
        $this->pid = Request :: get(WeblcmsManager :: PARAM_PUBLICATION);

        $this->add_reporting_block($this->get_publication_access());

        $this->add_breadcrumbs();
    }

    public function display_context()
    {
    }

    public function get_application()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    public function get_publication_access()
    {
        $course_weblcms_block = new PublicationAccessBlock($this);

        $course_id = Request :: get(WeblcmsManager :: PARAM_COURSE);
        $user_id = Request :: get(WeblcmsManager :: PARAM_USERS);
        $reporting_tool = Request :: get(ReportingTool :: PARAM_REPORTING_TOOL);

        if ($course_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_COURSE, $course_id);
        }
        if ($user_id)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_USERS, $user_id);
        }
        if ($this->tool)
        {
            $this->set_parameter(ReportingTool :: PARAM_REPORTING_TOOL, $reporting_tool);
        }
        if ($this->pid)
        {
            $this->set_parameter(WeblcmsManager :: PARAM_PUBLICATION, $this->pid);
        }

        return $course_weblcms_block;
    }

    /**
     * Adds the breadcrumbs to the breadcrumbtrail
     */
    protected function add_breadcrumbs()
    {
        $trail = BreadcrumbTrail :: get_instance();

        $trail->add(
            new Breadcrumb(
                $this->get_url(
                    array(\reporting\Viewer\Manager :: PARAM_BLOCK_ID => 4),
                    array(ReportingManager :: PARAM_TEMPLATE_ID)
                ),
                Translation :: get('LastAccessToToolsBlock')
            )
        );

        $trail->add(
            new Breadcrumb(
                $this->get_url(
                    array(
                        ReportingManager :: PARAM_TEMPLATE_ID => ToolPublicationsDetailTemplate :: class_name()
                    ),
                    array(WeblcmsManager :: PARAM_PUBLICATION)
                ),
                Translation :: get('TypeName', null, Tool :: get_tool_type_namespace($this->tool ? $this->tool : Request :: get ('tool')))
            )
        );

        $publication = \application\weblcms\DataManager :: retrieve_by_id(
            ContentObjectPublication :: class_name(), $this->pid
        );

        if(! isset($publication))
            throw new ObjectNotExistException(Translation::get('ContentObjectPublication'), $this->pid);

        $trail->add(
            new Breadcrumb(
                $this->get_url(), $publication->get_content_object()->get_title()
            )
        );

    }
}
