<?php
namespace application\weblcms\integration\webservice;


use common\libraries\Translation;
use common\libraries\CreateObjectWebserviceHandler;
use common\libraries\EqualityCondition;
use group\Group;
use group\DataManager as GroupDataManager;
use application\weblcms\course\CourseGroupRelation;
use application\weblcms\course\Course;

use application\weblcms\course\DataManager as CourseDataManager;

class CourseGroupRelationCreateWebserviceHandler extends CreateObjectWebserviceHandler
{

    public function get_object()
    {
        return new CourseGroupRelation();
    }

    public function get_object_translation()
    {
        return Translation :: get('CourseGroupRelation');
    }

    public function manipulate_data($object, $extra_properties = null)
    {
        if ($extra_properties['course_code'])
        {
            $condition = new EqualityCondition(Course :: PROPERTY_VISUAL_CODE, $extra_properties['course_code']);
            $course = CourseDataManager :: retrieve(Course :: class_name(), $condition);
            if (! $course)
            {
                return false;
            }
            $object->set_course_id($course->get_id());
        }
        if ($extra_properties['group_code'])
        {
            $condition = new EqualityCondition(Group :: PROPERTY_CODE, $extra_properties['group_code']);
            $group = GroupDataManager :: retrieve(Group :: class_name(), $condition);
            if (! $group)
            {
                return false;
            }
            $object->set_group_id($group->get_id());
        }
        if (! $object->get_course_id() || ! $object->get_group_id())
        {
            return false;
        }
        return $object;
    }

    public function default_properties_required()
    {
        return false;
    }

    public function is_allowed_to_create_object($object)
    {
        if ($this->get_user()->is_platform_admin())
        {
            return true;
        }

        $course = \application\weblcms\course\DataManager :: retrieve_by_id(
            Course :: class_name(), $object->get_course_id()
        );
        return $course->is_course_admin($this->get_user());
    }
}
