<?php

namespace application\weblcms;

use application\weblcms\WeblcmsBlock;
use application\weblcms\course\DataManager as CourseDataManager;
use common\libraries\EqualityCondition;
use common\libraries\ObjectTableOrder;
use repository\content_object\assignment\Assignment;
use common\libraries\AndCondition;
use common\libraries\InCondition;
use common\libraries\Translation;
use repository\ContentObjectRenderer;
use application\weblcms\tool\assignment\AssignmentTool;
use common\libraries\Application;
use common\libraries\Redirect;
use repository\ContentObject;
use common\libraries\SubselectCondition;
use common\libraries\Utilities;
use common\libraries\DatetimeUtilities;
use application\weblcms\course\CourseUserRelation;
use group\DataManager as GroupDataManager;
use application\weblcms\course\CourseGroupRelation;


/**
 * A notificationblock for new assignment submissions (assignmenttool)
 */
class WeblcmsEndingAssignments extends WeblcmsBlock
{

    public function display_content()
    {
        // deadline min 1 week (60 * 60 * 24 * 7)
        $deadline = time() + 604800;
        
        $condition = new EqualityCondition(
            CourseUserRelation :: PROPERTY_USER_ID, $this->get_user_id(), CourseUserRelation :: get_table_name()
        );
        $courses = CourseDataManager :: retrieve_user_courses($condition)->as_array();
        
        $groups = GroupDataManager :: retrieve_all_subscribed_groups_array($this->get_user_id());
        
        $group_ids = array();
        foreach($groups as $group)
        {
            $group_ids[$group->get_id()] = $group->get_id();
        }
        
        $condition = new InCondition(
            CourseGroupRelation :: PROPERTY_GROUP_ID, $group_ids, CourseGroupRelation :: get_table_name()
        );
        array_splice($courses, 0, 0, CourseDataManager :: retrieve_user_courses($condition)->as_array());
        
        $course_ids = array();
        foreach($courses as $course)
        {
            $course_ids[$course->get_id()] = $course->get_id();
        }
        
        $conditions = array();
        $conditions[] = new InCondition(ContentObjectPublication::PROPERTY_COURSE_ID, $course_ids);
        $conditions[] = new EqualityCondition(ContentObjectPublication::PROPERTY_TOOL, 'assignment');
        $subselect_condition = new EqualityCondition(ContentObject::PROPERTY_TYPE, Assignment::class_name());
        $conditions[] = new SubselectCondition(
            ContentObjectPublication::PROPERTY_CONTENT_OBJECT_ID,
            ContentObject::PROPERTY_ID, ContentObject::get_table_name(),
            $subselect_condition, null, \repository\DataManager::get_instance()
        );
        $condition = new AndCondition($conditions);
        $publications = DataManager :: retrieve_content_object_publications(
            $condition, new ObjectTableOrder(ContentObjectPublication :: PROPERTY_DISPLAY_ORDER_INDEX, SORT_DESC)
        )
            ->as_array();
        
        $ending_assignments = array();
        foreach($publications as $publication)
        {
            $assignment = $publication->get_content_object();
            if($assignment->get_end_time() > time() && $assignment->get_end_time() < $deadline)
            {
                $parameters = array(WeblcmsManager :: PARAM_COURSE => $publication->get_course_id(), 
                    Application :: PARAM_ACTION => WeblcmsManager :: ACTION_VIEW_COURSE, 
                    WeblcmsManager :: PARAM_TOOL => Assignment :: class_name(),
                    WeblcmsManager :: PARAM_TOOL_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS, 
                    Tool :: PARAM_BROWSER_TYPE => ContentObjectRenderer :: TYPE_TABLE, 
                    Tool :: PARAM_PUBLICATION_ID => $publication->get_id());
                $link = Redirect :: get_link(WeblcmsManager :: APPLICATION_NAME, $parameters);
                $ending_assignments[$assignment->get_end_time() . ' ' . $publication->get_id()] = 
                    array(
                        'title' => $assignment->get_title(), 'link' => $link,
                        'end_time' => $assignment->get_end_time()
                    );
            }
        }
        
        ksort($ending_assignments);
        $html = $this->display_new_items($ending_assignments);
        
        if (count($html) == 0)
        {
            return Translation :: get('NoAssignmentsEndComingWeek');
        }
        
        return implode("\n", $html);
    }

    public function display_new_items($items)
    {
        $html = array();
        foreach ($items as $item)
        {
            $end_date = DatetimeUtilities :: format_locale_date(
                Translation :: get('DateFormatShort', null, Utilities :: COMMON_LIBRARIES)
                . ', ' . Translation :: get('TimeNoSecFormat', null, Utilities :: COMMON_LIBRARIES),
                $item['end_time']
            );
            
            $html[] = '<a href="' . $item[link] . '">' . $item[title] . '</a>: '
                . Translation :: get('Until') . ' ' . $end_date . '<br />';
        }
        return $html;
    }
}
