<?php
namespace application\weblcms;

use common\extensions\category_manager\CategoryManager;
use common\extensions\category_manager\ProhibitGeneralCategories;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\InCondition;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: content_object_publication_category_manager.class.php 216 2009-11-13
 * 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.category_manager
 */
class ContentObjectPublicationCategoryManager extends CategoryManager implements ProhibitGeneralCategories
{

    public function __construct($parent, $trail = null, $is_subcategories_allowed = true)
    {
        $trail = BreadcrumbTrail :: get_instance();
        $trail->add(
            new Breadcrumb(
                $parent->get_url(
                    array(Tool :: PARAM_ACTION => Tool :: ACTION_MANAGE_CATEGORIES)
                ),
                Translation :: get('ManageCategories', null, Utilities :: COMMON_LIBRARIES)
            )
        );
        parent :: __construct($parent, $trail, $is_subcategories_allowed);
    }

    public function get_category()
    {
        $category = new ContentObjectPublicationCategory();
        $category->set_tool($this->get_parent()->get_tool_id());
        $category->set_course($this->get_parent()->get_course_id());
        $category->set_allow_change(1);
        return $category;
    }

    public function allowed_to_delete_category($category_id)
    {
        $category = DataManager :: retrieves(
            ContentObjectPublicationCategory :: class_name(),
            new EqualityCondition('id', $category_id)
        )
            ->next_result();

        if ($category)
        {
            if ($category->get_tool() == 'document' && !$category->get_allow_change())
            {
                return false;
            }
        }

        $count = $this->count_category_publications($category_id);
        if ($count > 0)
        {
            return false;
        }

        return !$this->have_subcategories_publications($category_id);
    }

    public function allowed_to_edit_category($category_id)
    {
        $category = DataManager :: retrieves(
            ContentObjectPublicationCategory :: class_name(),
            new EqualityCondition('id', $category_id)
        )
            ->next_result();

        if ($category)
        {
            if ($category->get_tool() == 'document' && !$category->get_allow_change())
            {
                return false;
            }
        }

        return true;
    }

    public function count_categories($condition)
    {
        if ($condition)
        {
            $conditions[] = $condition;
        }

        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_COURSE, $this->get_parent()->get_course_id()
        );

        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_TOOL, $this->get_parent()->get_tool_id()
        );

        $condition = new AndCondition($conditions);

        return DataManager :: count(ContentObjectPublicationCategory :: class_name(), $condition);
    }

    public function retrieve_categories($condition, $offset, $count, $order_property)
    {
        if ($condition)
        {
            $conditions[] = $condition;
        }

        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_COURSE, $this->get_parent()->get_course_id()
        );
        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_TOOL, $this->get_parent()->get_tool_id()
        );
        $condition = new AndCondition($conditions);

        return DataManager :: retrieves(
            ContentObjectPublicationCategory :: class_name(),
            new DataClassRetrievesParameters($condition, $count, $offset, $order_property)
        );
    }

    public function get_next_category_display_order($parent_id)
    {
        return null;
    }

    public function count_category_publications($category_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_COURSE_ID, $this->get_parent()->get_course_id()
        );
        $conditions[] = new InCondition(ContentObjectPublication :: PROPERTY_CATEGORY_ID, $category_id);
        $conditions[] = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_TOOL, $this->get_parent()->get_tool_id()
        );
        $condition = new AndCondition($conditions);
        return DataManager :: count_content_object_publications($condition);
    }

    private function have_subcategories_publications($category_id)
    {
        $condition = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_PARENT, $category_id);
        $subcategries = DataManager :: retrieves(ContentObjectPublicationCategory :: class_name(), $condition);

        while ($cat = $subcategries->next_result())
        {
            $count = $this->count_category_publications($cat->get_id());
            if ($count > 0 || $this->have_subcategories_publications($cat->get_id()))
            {
                return true;
            }
        }
        return false;
    }
}
