<?php
namespace application\weblcms;

use common\libraries\FormValidator;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: course_request_form.class.php 2 2010-02-25 11:43:06Z Yannick & Tristan $
 *
 * @package application.lib.weblcms.course_type
 */
class CourseCodeForm extends FormValidator
{
    const TEMP_CODE = 'temp_code';
    private $course;
    private $parent;
    private $user;

    public function __construct($action, $course, $parent, $user)
    {
        parent :: __construct('course_code', 'post', $action);
        $this->parent = $parent;
        $this->course = $course;
        $this->user = $user;

        $this->build_creating_form();

        $this->setDefaults();
        $this->add_progress_bar(2);
    }

    public function build_creating_form()
    {
        $this->build_code_form();

        $buttons[] = $this->createElement(
            'style_submit_button', 'submit', Translation :: get('Subscribe'),
            array('class' => 'positive update')
        );
        $buttons[] = $this->createElement(
            'style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'normal empty')
        );

        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function build_code_form()
    {
        $this->addElement('category', Translation :: get('CourseCodeProperties'));

        $course_name = $this->course->get_name();
        $this->addElement('static', 'course', Translation :: get('Course'), $course_name);

        $user_name = $this->user->get_fullname();
        $this->addElement('static', 'user', Translation :: get('User', null, 'user'), $user_name);

        $this->add_textfield(self :: TEMP_CODE, Translation :: get('Code'));

        $this->addElement('category');
    }

    public function check_code()
    {
        $temp_code = $this->exportValue(self :: TEMP_CODE);
        $code = $this->course->get_code();

        if ($temp_code == $code)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
}
