<?php
namespace application\weblcms;

use common\libraries\Breadcrumb;

use common\libraries\FormValidator;
use common\libraries\BreadcrumbTrail;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: move_selected_to_category.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.component
 */
class ToolComponentCategoryMoverComponent extends ToolComponent
{

    public function run()
    {
        if ($this->is_allowed(WeblcmsRights :: ADD_RIGHT))
        {
            $form = $this->build_move_to_category_form();
            if (!$form)
            {
                $this->display_header();
                $this->display_error_message('CategoryFormCouldNotBeBuild');
                $this->display_footer();
            }

            $publication_ids = Request :: get(Tool :: PARAM_PUBLICATION_ID);
            if (!is_array($publication_ids))
            {
                $publication_ids = array($publication_ids);
            }
            $form->addElement('hidden', 'pids', implode('-', $publication_ids));
            if ($form->validate())
            {
                $values = $form->exportValues();
                $publication_ids = explode('-', $values['pids']);
                // TODO: update all publications in a single action/query

                foreach ($publication_ids as $publication_id)
                {
                    $publication = DataManager :: retrieve(ContentObjectPublication :: class_name(), $publication_id);
                    $publication->set_category_id($form->exportValue('category'));
                    $publication->update();

                    if ($publication->get_category_id())
                    {
                        $new_parent_id =
                            WeblcmsRights :: get_instance()->get_weblcms_location_id_by_identifier_from_courses_subtree(
                                WeblcmsRights :: TYPE_COURSE_CATEGORY, $publication->get_category_id(),
                                $publication->get_course_id()
                            );
                    }
                    else
                    {
                        $condition = new EqualityCondition(CourseTool::PROPERTY_NAME, $publication->get_tool());
                        $course_modules = DataManager :: retrieves(CourseTool :: class_name(), $condition)->as_array();

                        $course_module_id = $course_modules[0]->get_id();
                        $new_parent_id =
                            WeblcmsRights :: get_instance()->get_weblcms_location_id_by_identifier_from_courses_subtree(
                                WeblcmsRights :: TYPE_COURSE_MODULE, $course_module_id, $publication->get_course_id()
                            );
                    }

                    $location =
                        WeblcmsRights :: get_instance()->get_weblcms_location_by_identifier_from_courses_subtree(
                            WeblcmsRights :: TYPE_PUBLICATION, $publication->get_id(), $publication->get_course_id()
                        );

                    if ($location)
                    {
                        $location->move($new_parent_id);
                    }

                }
                if (count($publication_ids) == 1)
                {
                    $message = Translation :: get(
                        'ObjectMoved', array('OBJECT' => Translation :: get('Publication')),
                        Utilities :: COMMON_LIBRARIES
                    );
                }
                else
                {
                    $message = Translation :: get(
                        'ObjectsMoved', array(
                            'OBJECTS' => Translation :: get('Publications')
                        ), Utilities :: COMMON_LIBRARIES
                    );
                }
                $this->redirect($message, false, array('tool_action' => null, Tool :: PARAM_PUBLICATION_ID => null));
            }
            else
            {
                // $message = $form->toHtml();
                $trail = BreadcrumbTrail :: get_instance();

                if (count($publication_ids) > 1)
                {
                    $trail->add(new Breadcrumb($this->get_url(), Translation :: get('PublicationsMover')));
                }
                else
                {
                    $publication =
                        DataManager :: retrieve(ContentObjectPublication :: class_name(), $publication_ids[0]);

                    $trail->add(
                        new Breadcrumb(
                            $this->get_url(),
                            Translation :: get(
                                'PublicationMover',
                                array('PUBLICATION' => $publication->get_content_object()->get_title())
                            )
                        )
                    );
                }
                $this->display_header();
                $form->display();
                $this->display_footer();
            }
        }
    }

    private $tree;

    public function build_move_to_category_form()
    {
        $publication_ids = Request :: get(Tool :: PARAM_PUBLICATION_ID);
        if (!is_array($publication_ids))
        {
            $publication_ids = array($publication_ids);
        }

        if (count($publication_ids) > 0)
        {
            $pub = DataManager :: retrieve(ContentObjectPublication :: class_name(), $publication_ids[0]);
            if ($pub)
            {
                $cat = $pub->get_category_id();
                $course = $this->get_course();
                $is_course_admin = $course->is_course_admin($this->get_user());

                if ($cat != 0)
                {
                    $module = DataManager :: retrieve_course_tool_by_name($this->get_tool_id());

                    if ($is_course_admin || WeblcmsRights :: get_instance()->is_allowed_in_courses_subtree(
                        WeblcmsRights :: ADD_RIGHT, $module->get_id(), WeblcmsRights :: TYPE_COURSE_MODULE,
                        $course->get_id()
                    )
                    )
                    {
                        $this->tree[0] = Translation :: get('Root');
                    }
                }
                $this->build_category_tree(0, $cat, $is_course_admin);

                if (count($this->tree) < 1)
                {
                    $this->redirect(
                        Translation :: get('NoCategoriesAvailable'), true, null, array(
                            Tool :: PARAM_ACTION
                        )
                    );
                }

                $form = new FormValidator(
                    'select_category',
                    'post',
                    $this->get_url(
                        array(
                            Tool :: PARAM_ACTION         => Tool :: ACTION_MOVE_TO_CATEGORY,
                            Tool :: PARAM_PUBLICATION_ID => Request :: get(Tool :: PARAM_PUBLICATION_ID)
                        )
                    )
                );
                foreach ($publication_ids as $publication_id)
                {
                    $publications[] =
                        DataManager :: retrieve(ContentObjectPublication :: class_name(), $publication_id)
                            ->get_content_object()->get_title();
                }
                $form->addElement(
                    'static', null, Translation :: get(
                        'ObjectSelected', array(
                            'OBJECT' => Translation :: get(count($publication_ids) > 1 ? 'Publications' : 'Publication')
                        ), Utilities :: COMMON_LIBRARIES
                    ), implode('<br>', $publications)
                );

                $select = $form->addElement(
                    'select',
                    'category',
                    Translation :: get('Category', null, Utilities :: COMMON_LIBRARIES)/*,
                    $this->tree*/);

                foreach($this->tree as $key => $value)
                {
                    if ($cat == $key)
                    {
                        $select->addOption($value, $key, array('disabled'));
                    }
                    else
                    {
                        $select->addOption($value, $key);
                    }
                }	
				
                // $form->addElement('submit', 'submit', Translation ::
                // get('Ok', null ,Utilities:: COMMON_LIBRARIES));
                $buttons[] = $form->createElement(
                    'style_submit_button', 'submit', Translation :: get('Move', null, Utilities :: COMMON_LIBRARIES),
                    array(
                        'class' => 'positive move'
                    )
                );
                $buttons[] = $form->createElement(
                    'style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES),
                    array(
                        'class' => 'normal empty'
                    )
                );

                $form->addGroup($buttons, 'buttons', null, '&nbsp;', false);
                return $form;

            }
        }
    }

    private $level = 1;

    public function build_category_tree($parent_id, $exclude, $is_course_admin)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_PARENT, $parent_id);

        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_COURSE, $this->get_course_id()
        );

        $conditions[] = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_TOOL, $this->get_tool_id());

        $condition = new AndCondition($conditions);

        $categories = DataManager :: retrieves(ContentObjectPublicationCategory :: class_name(), $condition);

        while ($cat = $categories->next_result())
        {
            if ($is_course_admin || WeblcmsRights :: get_instance()->is_allowed_in_courses_subtree(
                WeblcmsRights :: ADD_RIGHT, $cat->get_id(), WeblcmsRights :: TYPE_COURSE_CATEGORY,
                $this->get_course_id()
            )
            )
            {
                //if ($cat->get_id() != $exclude)
                //{
                    $this->tree[$cat->get_id()] = str_repeat('--', $this->level) . ' ' . $cat->get_name();
                //}
                $this->level++;
                $this->build_category_tree($cat->get_id(), $exclude, $is_course_admin);
                $this->level--;
            }
        }
    }

}
