<?php
namespace application\weblcms;

use common\libraries\Display;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\PlatformSetting;
use common\libraries\Request;

/**
 * $Id: announcement_publisher.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.announcement.component
 */
class ToolComponentPublisherComponent extends ToolComponent implements \repository\viewer\ViewerInterface
{

    public function run()
    {
        if (! ($this->get_course()->is_course_admin($this->get_user()) || $this->is_allowed(WeblcmsRights :: ADD_RIGHT)))
        {
            Display :: error_page(Translation :: get("NotAllowed"));
            return;
        }

        if (! \repository\viewer\Manager :: is_ready_to_be_published())
        {
            $repository_viewer = \repository\viewer\Manager :: construct($this);
            $repository_viewer->run();
        }
        else
        {
            $objects = \repository\viewer\Manager :: get_selected_objects();

            $mode = Request :: get(Tool :: PARAM_PUBLISH_MODE);
            $publish_type = PlatformSetting :: get('display_publication_screen', __NAMESPACE__);
            $show_form = (($publish_type == Tool :: PUBLISH_TYPE_FORM) ||
                 ($publish_type == Tool :: PUBLISH_TYPE_BOTH && $mode != Tool :: PUBLISH_MODE_QUICK));

            $publisher = new ContentObjectPublisher($this, $objects, $show_form);

            if ($publisher->ready_to_publish())
            {
                $success = $publisher->publish();

                $message = Translation :: get(
                    ($success ? 'ObjectPublished' : 'ObjectNotPublished'),
                    array('OBJECT' => Translation :: get('Object')),
                    Utilities :: COMMON_LIBRARIES);

                $parameters = array(Tool :: PARAM_ACTION => Tool :: ACTION_BROWSE);

                if ($publisher->is_publish_and_build_submit())
                {
                    $parameters[Tool :: PARAM_ACTION] = Tool :: ACTION_BUILD_COMPLEX_CONTENT_OBJECT;

                    $publications = $publisher->get_publications();
                    $parameters[Tool :: PARAM_PUBLICATION_ID] = $publications[0]->get_id();
                }
                
                if ($publisher->is_publish_and_view_submit())
                {
                    $parameters[Tool :: PARAM_ACTION] = Tool :: ACTION_DISPLAY_COMPLEX_CONTENT_OBJECT;

                    $publications = $publisher->get_publications();
                    $parameters[Tool :: PARAM_PUBLICATION_ID] = $publications[0]->get_id();
                }

                $this->redirect($message, ! $success, $parameters);
            }
            else
            {
                $this->display_header();
                echo $publisher->toHtml();
                $this->display_footer();
            }
        }
    }

    public function get_additional_parameters()
    {
        return array(Tool :: PARAM_PUBLISH_MODE);
    }
}
