<?php
namespace application\weblcms;

use application\weblcms\tool\search\SearchTool;
use common\libraries\FormValidator;
use common\libraries\Application;
use common\libraries\Theme;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: menu_tool_list_renderer.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool_list_renderer
 */
/**
 * Tool list renderer to display a navigation menu.
 */
class MenuToolListRenderer extends ToolListRenderer
{
    private $is_course_admin;
    private $menu_properties;

    /**
     * Constructor
     *
     * @param $parent WebLcms
     *            The parent application
     */
    public function __construct($parent, $visible_tools)
    {
        parent :: __construct($parent, $visible_tools);
        $this->is_course_admin = $this->get_parent()->is_allowed(WeblcmsRights :: EDIT_RIGHT);
        $this->menu_properties = $this->load_menu_properties();
    }

    // Inherited
    public function display()
    {
        //$parent = $this->get_parent();
        $this->show_tools($this->get_visible_tools());
    }

    /**
     * Show the tools of a given section
     *
     * @param $tools array
     */
    private function show_tools($tools)
    {
        $parent = $this->get_parent();
        //$course = $parent->get_course();

        $menu_style = $this->get_menu_style();

        $html[] = '<div id="tool_bar" class="tool_bar tool_bar_' .
            ($this->display_menu_icons() && ! $this->display_menu_text() ? 'icon_' : '') . $menu_style . '">';

        if ($this->get_menu_style() == 'right')
        {
            $html[] = '<div id="tool_bar_hide_container" class="hide">';
            $html[] =
                '<a id="tool_bar_hide" href="#"><img src="' . Theme :: get_common_image_path() . 'action_action_bar_' .
                    $menu_style . '_hide.png" /></a>';
            $html[] =
                '<a id="tool_bar_show" href="#"><img src="' . Theme :: get_common_image_path() . 'action_action_bar_' .
                    $menu_style . '_show.png" /></a>';
            $html[] = '</div>';
        }

        $html[] = '<div class="tool_menu">';
        $html[] = '<ul>';

        $show_search = false;

        foreach ($tools as $tool)
        {
            if ($tool->get_section_type() == CourseSection :: TYPE_ADMIN)
            {
                $admin_tools[] = $tool;
                continue;
            }

            if ($tool->get_name() == 'search')
            {
                $show_search = true;
            }

            $html[] = $this->display_tool($tool);
        }

        if (count($admin_tools) && $this->is_course_admin)
        {
            $html[] = '<div style="margin: 10px 0 10px 0; border-bottom: 1px dotted #4271B5; height: 0px;"></div>';
            foreach ($admin_tools as $tool)
            {
                $html[] = $this->display_tool($tool);
            }
        }
        $html[] = '</ul>';

        if ($this->display_menu_text() && $show_search)
        {
            $html[] =
                '<div style="margin: 10px 0 10px 0; border-bottom:
                1px dotted #4271B5; height: 0px; text-align: center;"></div>';

            $form = new FormValidator(
                'search_simple', 'post', $parent->get_url(
                    array(
                        WeblcmsManager :: PARAM_TOOL => 'search',
                        Tool :: PARAM_ACTION => SearchTool :: ACTION_SEARCH
                    )
                ), '', array('style' => 'text-align: center;'), false
            );

            $renderer = clone $form->defaultRenderer();
            $renderer->setFormTemplate('<form {attributes}>{content}</form>');
            $renderer->setElementTemplate('{element}<br />');

            $form->addElement(
                'text', 'query', '',
                'size="18" class="search_query_no_icon" style="background-color: white;
                border: 1px solid grey; height: 18px; margin-bottom: 10px;"'
            );
            $form->addElement(
                'style_submit_button', 'submit', Translation :: get('Search', null, Utilities :: COMMON_LIBRARIES),
                array(
                    'class' => 'normal search'
                )
            );
            $form->accept($renderer);

            $html[] = $renderer->toHtml();
        }

        $html[] = '</div>';
        $html[] = '<div class="clear">&nbsp;</div>';

        if ($this->get_menu_style() == 'left')
        {
            $html[] = '<div id="tool_bar_hide_container" class="hide">';
            $html[] =
                '<a id="tool_bar_hide" href="#"><img src="' . Theme :: get_common_image_path() . 'action_action_bar_' .
                    $menu_style . '_hide.png" /></a>';
            $html[] =
                '<a id="tool_bar_show" href="#"><img src="' . Theme :: get_common_image_path() . 'action_action_bar_' .
                    $menu_style . '_show.png" /></a>';
            $html[] = '</div>';
        }

        $html[] = '</div>';
        $html[] = '<script type="text/javascript" src="' . Path :: get(WEB_LIB_PATH) .
            'libraries/resources/javascript/tool_bar.js' . '"></script>';

        if ($_SESSION['toolbar_state'] == 'hide')
        {
            $html[] = '<script type="text/javascript">var hide = "true";</script>';
        }
        else
        {
            $html[] = '<script type="text/javascript">var hide = "false";</script>';
        }

        $html[] = '<div class="clear">&nbsp;</div>';

        echo implode("\n", $html);
    }

    public function display_tool($tool)
    {
        $parent = $this->get_parent();
        //$course = $parent->get_course();

        $new = '';
        if ($parent->tool_has_new_publications($tool->get_name()))
        {
            $new = '_new';
        }

        $tool_image = Theme :: ICON_MINI . $new . '.png';

        $title =
            htmlspecialchars(Translation :: get('TypeName', null, Tool :: get_tool_type_namespace($tool->get_name())));

        $html[] = '<li class="tool_list_menu" style="padding: 0px 0px 2px 0px;">';

        $html[] = '<a href="' . $parent->get_url(
            array(
                Application :: PARAM_ACTION => WeblcmsManager :: ACTION_VIEW_COURSE,
                WeblcmsManager :: PARAM_TOOL => $tool->get_name()
            ),
            array(
                Tool :: PARAM_ACTION,
                Tool :: PARAM_PUBLICATION_ID, WeblcmsManager :: PARAM_CATEGORY
            ), true
        ) . '" title="' . $title . '">';

        if ($this->display_menu_icons())
        {
            $html[] =
                '<img src="' . Theme :: get_image_path(Tool :: get_tool_type_namespace($tool->get_name())) . 'logo/' .
                    $tool_image . '" style="vertical-align: middle;" alt="' . $title . '"/> ';
        }

        if ($this->display_menu_text())
        {
            $html[] = $title;
        }

        $html[] = '</a>';
        $html[] = '</li>';

        return implode("\n", $html);
    }

    public function load_menu_properties()
    {
        $course_settings_controller = CourseSettingsController :: get_instance();
        $menu_style = $course_settings_controller->get_course_setting(
            $this->get_parent()->get_course()->get_id(), CourseSettingsConnector :: MENU_LAYOUT
        );

        $properties = array();

        switch ($menu_style)
        {
            case CourseSettingsConnector :: MENU_LAYOUT_LEFT_WITH_ICONS :
                $properties['style'] = 'left';
                $properties['icons'] = true;
                $properties['text'] = false;
                break;
            case CourseSettingsConnector :: MENU_LAYOUT_LEFT_BOTH :
                $properties['style'] = 'left';
                $properties['icons'] = true;
                $properties['text'] = true;
                break;
            case CourseSettingsConnector :: MENU_LAYOUT_LEFT_WITH_TEXT :
                $properties['style'] = 'left';
                $properties['icons'] = false;
                $properties['text'] = true;
                break;

            case CourseSettingsConnector :: MENU_LAYOUT_RIGHT_WITH_ICONS :
                $properties['style'] = 'right';
                $properties['icons'] = true;
                $properties['text'] = false;
                break;
            case CourseSettingsConnector :: MENU_LAYOUT_RIGHT_BOTH :
                $properties['style'] = 'right';
                $properties['icons'] = true;
                $properties['text'] = true;
                break;
            case CourseSettingsConnector :: MENU_LAYOUT_RIGHT_WITH_TEXT :
                $properties['style'] = 'right';
                $properties['icons'] = false;
                $properties['text'] = true;
                break;

            default :
                $properties['style'] = 'left';
                $properties['icons'] = true;
                $properties['text'] = true;
                break;
        }

        return $properties;
    }

    public function get_menu_properties()
    {
        return $this->menu_properties;
    }

    public function get_menu_style()
    {
        $properties = $this->get_menu_properties();
        return $properties['style'];
    }

    public function display_menu_icons()
    {
        $properties = $this->get_menu_properties();
        return $properties['icons'];
    }

    public function display_menu_text()
    {
        $properties = $this->get_menu_properties();
        return $properties['text'];
    }
}
