<?php
namespace application\weblcms;

use common\libraries\Application;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Redirect;

/**
 * $Id: shortcut_tool_list_renderer.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool_list_renderer
 */
/**
 * Tool list renderer to display a navigation menu.
 */
class ShortcutToolListRenderer extends ToolListRenderer
{

    // Inherited
    public function display()
    {
        $this->is_course_admin = $this->get_parent()->get_parent()->is_teacher();
        $this->show_tools($this->get_visible_tools());
    }

    /**
     * Show the tools of a given section
     *
     * @param $tools array
     */
    private function show_tools($tools)
    {
        $parent = $this->get_parent();
        $course = $parent->get_course();

        foreach ($tools as $tool)
        {
            if ($tool->get_section_type() == CourseSection :: TYPE_ADMIN && !$this->is_course_admin)
            {
                continue;
            }

            $new = '';
            if ($parent->tool_has_new_publications($tool->get_name()))
            {
                $new = '_new';
            }

            $tool_image = Theme :: ICON_MINI . $new . '.png';

            $title = htmlspecialchars(
                Translation :: get('TypeName', null, Tool :: get_tool_type_namespace($tool->get_name()))
            );

            $params = array(Application :: PARAM_APPLICATION => WeblcmsManager :: APPLICATION_NAME,
                WeblcmsManager :: PARAM_COURSE => $course->get_id(),
                Application :: PARAM_ACTION => WeblcmsManager :: ACTION_VIEW_COURSE,
                WeblcmsManager :: PARAM_TOOL => $tool->get_name(),
                array(WeblcmsManager :: PARAM_CATEGORY)
            );

            $html[] = '<a href="' . Redirect :: get_url($params, array(), true) . '" title="' . $title . '">';
            $html[] =
                '<img src="' . Theme :: get_image_path(Tool :: get_tool_type_namespace($tool->get_name())) . 'logo/' .
                    $tool_image . '" style="vertical-align: middle;" alt="' . $title . '"/> ';
            $html[] = '</a>';
        }

        echo implode("\n", $html);
    }
}
