<?php
namespace application\weblcms;

use application\weblcms\course\DataManager as CourseDataManager;
use common\libraries\Application;
use common\libraries\BreadcrumbTrail;
use common\libraries\Request;
use common\libraries\Translation;

/**
 * $Id: course_deleter.class.php 218 2009-11-13 14:21:26Z kariboe $
 *
 * @package application.lib.weblcms.weblcms_manager.component
 */

/**
 * Repository manager component which provides functionality to delete a course
 */
class WeblcmsManagerCourseDeleterComponent extends WeblcmsManager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $course_codes = Request :: get(WeblcmsManager :: PARAM_COURSE);
        $failures = 0;

        if (!$this->get_user()->is_platform_admin())
        {
            throw new \common\libraries\NotAllowedException();
        }

        if (!empty($course_codes))
        {
            if (!is_array($course_codes))
            {
                $course_codes = array($course_codes);
            }

            foreach ($course_codes as $course_code)
            {
                $course = CourseDataManager :: retrieve(Course :: class_name(), $course_code);

                if (!$course->delete())
                {
                    $failures++;
                }
            }

            if ($failures)
            {
                if (count($course_codes) == 1)
                {
                    $message = 'SelectedCourseNotDeleted';
                }
                else
                {
                    $message = 'SelectedCoursesNotDeleted';
                }
            }
            else
            {
                if (count($course_codes) == 1)
                {
                    $message = 'SelectedCourseDeleted';
                }
                else
                {
                    $message = 'SelectedCoursesDeleted';
                }
            }

            $this->redirect(
                Translation :: get($message), ($failures ? true : false), array(
                    Application :: PARAM_ACTION    => WeblcmsManager :: ACTION_ADMIN_COURSE_BROWSER,
                    WeblcmsManager :: PARAM_COURSE => null
                )
            );
        }
        else
        {
            $this->display_error_page(htmlentities(Translation :: get('NoCourseSelected')));
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {

        $breadcrumbtrail->add_help('weblcms_course_deleter');
    }

    public function get_additional_parameters()
    {
        return array();
    }

}
