<?php
namespace application\weblcms;

use common\libraries\RssIconGenerator;
use common\libraries\PlatformSetting;
use common\libraries\Application;
use common\libraries\Theme;
use common\libraries\Session;
use common\libraries\BreadcrumbTrail;
use common\libraries\Request;
use common\libraries\Path;
use common\libraries\DelegateComponent;
use common\libraries\Translation;
use common\libraries\LocalSetting;
use common\libraries\Utilities;
use application\weblcms\course\CourseManager;

/**
 * $Id: home.class.php 218 2009-11-13 14:21:26Z kariboe $
 *
 * @package application.lib.weblcms.weblcms_manager.component
 */
/**
 * Weblcms component which provides the user with a list of all courses he or she has subscribed to.
 */
class WeblcmsManagerHomeComponent extends WeblcmsManager implements DelegateComponent
{
    const VIEW_MIXED = 0;
    const VIEW_OPEN_CLOSED = 1;
    const VIEW_OPEN = 2;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        // $trail = BreadcrumbTrail :: get_instance();

        // $this->message = Request :: get('message');
        // Request :: set_get('message', null);
        $view_state = LocalSetting :: get('view_state', WeblcmsManager :: APPLICATION_NAME);

        switch ($view_state)
        {
            case self :: VIEW_MIXED :
                $renderer = new CourseTypeCourseListRenderer($this);
                break;
            case self :: VIEW_OPEN_CLOSED :
                $renderer = new OpenClosedCourseTypeCourseListRenderer($this);
                break;
            case self :: VIEW_OPEN :
                $renderer = new OpenCourseTypeCourseListRenderer($this);
                break;
        }

        $renderer->show_new_publication_icons();
        $html[] = $renderer->as_html();
        $html[] = '<script type="text/javascript" src="' . htmlspecialchars(Path :: get(WEB_LIB_PATH)) .
             'libraries/resources/javascript/home_ajax.js' . '"></script>';
        $toolbar_state = Session :: retrieve('toolbar_state');
        if ($toolbar_state == 'hide')
        {
            $html[] = '<script type="text/javascript">var hide = "true";</script>';
        }
        else
        {
            $html[] = '<script type="text/javascript">var hide = "false";</script>';
        }

        $this->display_header();
        echo '<div class="clear"></div>';

        echo $this->display_menu();

        echo '<div id="tool_browser_right">';
        echo implode("\n", $html);
        echo '</div>';

        $this->display_footer();
    }

    public function display_menu()
    {
        $html = array();

        $html[] = '<div id="tool_bar" class="tool_bar tool_bar_right">';

        $img_path = htmlspecialchars(Theme :: get_common_image_path());
        $html[] = '<div id="tool_bar_hide_container" class="hide">';
        $html[] = '<a id="tool_bar_hide" href="#"><img src="' . $img_path . 'action_action_bar_right_hide.png" /></a>';
        $html[] = '<a id="tool_bar_show" href="#"><img src="' . $img_path . 'action_action_bar_right_show.png" /></a>';
        $html[] = '</div>';

        $html[] = '<div class="tool_menu">';
        $html[] = '<ul>';

        if ($this->get_user()->is_platform_admin())
        {
            $html[] = '<li class="tool_list_menu title" style="font-weight: bold">' . htmlspecialchars(
                Translation :: get('CourseManagement')) . '</li><br />';
            $html[] = $this->display_platform_admin_course_list_links();
            $html[] = '<div style="margin: 10px 0 10px 0; border-bottom: 1px dotted #4271B5; height: 0px;"></div>';
        }
        else
        {

            $display_add_course_link = $this->get_user()->is_teacher() && ($_SESSION["studentview"] != "studentenview");
            if ($display_add_course_link)
            {
                $display = $this->display_create_course_link();

                if ($display)
                {
                    $html[] = '<li class="tool_list_menu" style="font-weight: bold">' . htmlspecialchars(
                        Translation :: get('MenuUser')) . '</li><br />';
                    $html[] = $display;
                }
            }
        }

        $html[] = '<li class="tool_list_menu title" style="font-weight: bold">' . htmlspecialchars(
            Translation :: get('UserCourseManagement')) . '</li><br />';
        $html[] = $this->display_edit_course_list_links();
        $html[] = '</ul>';
        $html[] = '</div>';

        $html[] = '</div>';
        $html[] = '<script type="text/javascript" src="' . htmlspecialchars(Path :: get(WEB_LIB_PATH)) .
             'libraries/resources/javascript/tool_bar.js' . '"></script>';
        $html[] = '<div class="clear"></div>';
        return implode($html, "\n");
    }

    public function display_create_course_link()
    {
        $html = array();

        $img_path = htmlspecialchars(Theme :: get_common_image_path());

        $course_management_rights = CourseManagementRights :: get_instance();

        $count_direct = $count_request = 0;

        $course_types = course_type\DataManager :: retrieve_active_course_types();
        while ($course_type = $course_types->next_result())
        {
            if ($course_management_rights->is_allowed(
                CourseManagementRights :: CREATE_COURSE_RIGHT,
                $course_type->get_id(),
                CourseManagementRights :: TYPE_COURSE_TYPE))
            {
                $count_direct ++;
            }
            elseif ($course_management_rights->is_allowed(
                CourseManagementRights :: REQUEST_COURSE_RIGHT,
                $course_type->get_id(),
                CourseManagementRights :: TYPE_COURSE_TYPE))
            {
                $count_request ++;
            }
        }

        if (PlatformSetting :: get('allow_course_creation_without_coursetype', __NAMESPACE__))
        {
            $count_direct ++;
        }

        if ($count_direct)
        {
            $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
                 'action_create.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                    $this->get_url(
                        array(
                            Application :: PARAM_ACTION => WeblcmsManager :: ACTION_COURSE_MANAGER,
                            CourseManager :: PARAM_ACTION => CourseManager :: ACTION_QUICK_CREATE))) . '">' .
                 htmlspecialchars(Translation :: get('CourseCreate')) . '</a></li>';
        }

        if ($count_request)
        {
            $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
                 'action_create.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                    $this->get_url(
                        array(Application :: PARAM_ACTION => WeblcmsManager :: ACTION_REQUEST,
                            \application\weblcms\request\Manager :: PARAM_ACTION => \application\weblcms\request\Manager :: ACTION_CREATE))) .
                 '">' . Utilities :: htmlentities(Translation :: get('CourseRequest')) . '</a></li>';
        }
        
        if (\application\weblcms\request\rights\Rights :: get_instance()->request_is_allowed())
        {
            $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
            'action_browser.png)"><a style="top: -3px; position: relative;" href="' .
            htmlspecialchars(
                $this->get_url(array(Application :: PARAM_ACTION => WeblcmsManager :: ACTION_REQUEST))) . '">' . htmlspecialchars(
                    Translation :: get('RequestList')) . '</a></li>';
        }

        return implode("\n", $html);
    }

    public function display_edit_course_list_links()
    {
        $html = array();
        $img_path = htmlspecialchars(Theme :: get_common_image_path());
        $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
             'action_reset.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                $this->get_url(
                    array(Application :: PARAM_ACTION => WeblcmsManager :: ACTION_MANAGER_SORT))) . '">' .
             htmlspecialchars(Translation :: get('SortMyCourses')) . '</a></li>';

        if (PlatformSetting :: get('show_subscribe_button_on_course_home', __NAMESPACE__))
        {
            $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path . 'action_subscribe.png)">' .
                 '<a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                    $this->get_url(
                        array(
                            Application :: PARAM_ACTION => WeblcmsManager :: ACTION_COURSE_MANAGER,
                            CourseManager :: PARAM_ACTION => CourseManager :: ACTION_BROWSE_UNSUBSCRIBED_COURSES))) . '">' .
                 htmlspecialchars(Translation :: get('CourseSubscribe')) . '</a></li>';
            $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
                 'action_unsubscribe.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                    $this->get_url(
                        array(
                            Application :: PARAM_ACTION => WeblcmsManager :: ACTION_COURSE_MANAGER,
                            CourseManager :: PARAM_ACTION => CourseManager :: ACTION_BROWSE_SUBSCRIBED_COURSES))) . '">' .
                 htmlspecialchars(Translation :: get('CourseUnsubscribe')) . '</a></li>';
        }

        $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
             'rss_feed.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                RssIconGenerator :: generate_rss_url(
                    WeblcmsManager :: APPLICATION_NAME,
                    'publication',
                    $this->get_user())) . '">' . htmlspecialchars(Translation :: get('RssFeed')) . '</a></li>';

        return implode($html, "\n");
    }

    public function display_platform_admin_course_list_links()
    {
        $html = array();
        $img_path = htmlspecialchars(Theme :: get_common_image_path());
        $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
             'action_create.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                $this->get_url(
                    array(
                        Application :: PARAM_ACTION => WeblcmsManager :: ACTION_COURSE_MANAGER,
                        CourseManager :: PARAM_ACTION => CourseManager :: ACTION_QUICK_CREATE))) . '">' .
             htmlspecialchars(Translation :: get('CourseCreate')) . '</a></li>';
        $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
             'action_browser.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                $this->get_url(
                    array(Application :: PARAM_ACTION => WeblcmsManager :: ACTION_COURSE_MANAGER))) . '">' .
             htmlspecialchars(Translation :: get('CourseList')) . '</a></li>';

        $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
             'action_browser.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                $this->get_url(
                    array(Application :: PARAM_ACTION => WeblcmsManager :: ACTION_REQUEST))) . '">' .
             htmlspecialchars(Translation :: get('RequestList')) . '</a></li>';

        $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
            'action_browser.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
            $this->get_url(
                array(Application :: PARAM_ACTION => WeblcmsManager :: ACTION_ADMIN_REQUEST_BROWSER))) . '">' .
            htmlspecialchars(Translation :: get('UserRequestList')) . '</a></li>';

        $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
             'action_move.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                $this->get_url(
                    array(Application :: PARAM_ACTION => WeblcmsManager :: ACTION_COURSE_CATEGORY_MANAGER))) . '">' .
             htmlspecialchars(Translation :: get('CourseCategoryManagement')) . '</a></li>';
        $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
             'action_add.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                $this->get_url(
                    array(Application :: PARAM_ACTION => WeblcmsManager :: ACTION_IMPORT_COURSES))) . '">' .
             htmlspecialchars(Translation :: get('ImportCourseCSV')) . '</a></li>';
        $html[] = '<li class="tool_list_menu" style="background-image: url(' . $img_path .
             'action_add.png)"><a style="top: -3px; position: relative;" href="' . htmlspecialchars(
                $this->get_url(
                    array(Application :: PARAM_ACTION => WeblcmsManager :: ACTION_IMPORT_COURSE_USERS))) . '">' .
             htmlspecialchars(Translation :: get('ImportUsersForCourseCSV')) . '</a></li>';

        return implode($html, "\n");
    }

    public function get_course_user_category_actions(CourseUserCategory $course_user_category, CourseType $course_type,
        $offset, $count)
    {
        $img_path = htmlspecialchars(Theme :: get_common_image_path());
        return '<a href="#" class="closeEl"><img class="visible" src="' . $img_path .
             'action_visible.png"/><img class="invisible" style="display: none;" src="' . $img_path .
             'action_invisible.png" /></a>';
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('weblcms_course_home');
    }

    public function get_additional_parameters()
    {
        return array();
    }

    public function show_empty_courses()
    {
        return false;
    }
}
