<?php
namespace application\weblcms;

use application\weblcms\course_type\CourseType;
use common\libraries\ActionBarRenderer;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\BreadcrumbTrail;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\NoObjectSelectedException;
use common\libraries\ObjectNotExistException;

/**
 * $Id: sorter.class.php 218 2009-11-13 14:21:26Z kariboe $
 *
 * @package application.lib.weblcms.weblcms_manager.component
 */
/**
 * Weblcms component which allows the user to manage his or her course
 * subscriptions
 */
class WeblcmsManagerSorterComponent extends WeblcmsManager
{
    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        course_type\DataManager :: fix_course_tab_user_orders_for_user($this->get_user_id());
        DataManager :: fix_course_type_user_category_rel_course_for_user($this->get_user_id());

        $component_action = Request :: get(WeblcmsManager :: PARAM_COMPONENT_ACTION);
        $this->set_parameter(WeblcmsManager :: PARAM_COMPONENT_ACTION, $component_action);

        switch ($component_action)
        {
            case 'add' :
                $this->add_course_user_category();
                break;
            case 'move' :
                $this->move_course_list();
                break;
            case 'movecat' :
                $this->move_category_list();
                break;
            case 'assign' :
                $this->assign_course_category();
                break;
            case 'edit' :
                $this->edit_course_user_category();
                break;
            case 'delete' :
                $this->delete_course_type_user_category();
                break;
            case 'view' :
                $this->show_course_list();
                break;
            case 'move_course_type_up' :
                $this->move_course_type(-1);
                break;
            case 'move_course_type_down' :
                $this->move_course_type(1);
                break;
            default :
                $this->show_course_list();
        }
    }

    /**
     * Moves a course type for a given direction
     *
     * @param $direction int
     */
    public function move_course_type($direction)
    {
        $selected_course_type_id = Request :: get(CourseTypeCourseListRenderer :: PARAM_SELECTED_COURSE_TYPE);

        $course_type_translation = Translation :: get('CourseType');

        if (isset($selected_course_type_id) && $selected_course_type_id != 0)
        {
            $selected_course_type = course_type\DataManager :: retrieve(
                CourseType :: class_name(), $selected_course_type_id
            );

            if (!$selected_course_type)
            {
                throw new ObjectNotExistException($course_type_translation, $selected_course_type_id);
            }

            $selected_course_type_user_order = course_type\DataManager :: retrieve_user_order_for_course_type(
                $selected_course_type_id, $this->get_user_id()
            );

            if (!$selected_course_type_user_order)
            {
                course_type\DataManager :: create_course_type_user_orders_for_user($this->get_user_id());

                $selected_course_type_user_order = course_type\DataManager :: retrieve_user_order_for_course_type(
                    $selected_course_type_id, $this->get_user_id()
                );
            }

            $selected_course_type_user_order->change_display_order_with_count($direction);
            $succes = $selected_course_type_user_order->update();

            $this->redirect(
                Translation :: get($succes ? 'CourseTypeMoved' : 'CourseTypeNotMoved'), !$succes, array(
                    CourseTypeCourseListRenderer :: PARAM_SELECTED_COURSE_TYPE => $selected_course_type_id
                ), array(
                    WeblcmsManager :: PARAM_COMPONENT_ACTION
                )
            );
        }
        else
        {
            throw new NoObjectSelectedException($course_type_translation);
        }
    }

    public function move_course_list()
    {
        $direction = Request :: get(WeblcmsManager :: PARAM_DIRECTION);
        $course = Request :: get(WeblcmsManager :: PARAM_COURSE);
        $course_type_user_category_id = Request :: get(WeblcmsManager :: PARAM_COURSE_TYPE_USER_CATEGORY_ID);

        if (isset($direction) && isset($course))
        {
            $success = $this->move_course($course, $course_type_user_category_id, $direction);
            $this->redirect(
                Translation :: get($success ? 'CourseUserMoved' : 'CourseUserNotMoved'), ($success ? false : true),
                array(
                    WeblcmsManager :: PARAM_COMPONENT_ACTION => WeblcmsManager :: ACTION_MANAGER_SORT
                )
            );
        }
        else
        {
            $this->show_course_list();
        }
    }

    public function move_category_list()
    {
        $direction = Request :: get(WeblcmsManager :: PARAM_DIRECTION);
        $course_type_user_category_id = Request :: get(WeblcmsManager :: PARAM_COURSE_TYPE_USER_CATEGORY_ID);

        if (isset($direction) && isset($course_type_user_category_id))
        {
            $success = $this->move_category($course_type_user_category_id, $direction);
            $this->redirect(
                Translation :: get($success ? 'CourseUserCategoryMoved' : 'CourseUserCategoryNotMoved'),
                ($success ? false : true), array(
                    WeblcmsManager :: PARAM_COMPONENT_ACTION => WeblcmsManager :: ACTION_MANAGER_SORT
                )
            );
        }
        else
        {
            $this->show_course_list();
        }
    }

    public function assign_course_category()
    {
        $course_id = Request :: get(WeblcmsManager :: PARAM_COURSE);
        $course_type_user_category_id = Request :: get(WeblcmsManager :: PARAM_COURSE_TYPE_USER_CATEGORY_ID);

        if (!$course_type_user_category_id)
        {
            $course_type_user_category_rel_course = new CourseTypeUserCategoryRelCourse();
            $course_type_user_category_rel_course->set_course_id($course_id);
            $course_type_user_category_rel_course->set_user_id($this->get_user_id());
        }
        else
        {
            $conditions = array();
            $conditions[] = new EqualityCondition(CourseTypeUserCategoryRelCourse :: PROPERTY_COURSE_ID, $course_id);

            $conditions[] = new EqualityCondition(
                CourseTypeUserCategoryRelCourse :: PROPERTY_COURSE_TYPE_USER_CATEGORY_ID,
                $course_type_user_category_id
            );

            $condition = new AndCondition($conditions);

            $course_type_user_category_rel_course = DataManager :: retrieves(
                CourseTypeUserCategoryRelCourse :: class_name(),
                $condition
            )->next_result();
        }

        $form = new CourseTypeUserCategoryRelCourseForm(
            $course_type_user_category_rel_course,
            $this->get_user(),
            $this->get_url(
                array(
                    WeblcmsManager :: PARAM_COURSE                       => $course_id,
                    WeblcmsManager :: PARAM_COURSE_TYPE_USER_CATEGORY_ID => $course_type_user_category_id
                )
            )
        );

        if ($form->validate())
        {
            $success = $form->update_course_type_user_category_rel_course();
            $this->redirect(
                Translation :: get($success ? 'CourseUserCategoryUpdated' : 'CourseUserCategoryNotUpdated'),
                ($success ? false : true),
                array(
                    WeblcmsManager :: PARAM_COMPONENT_ACTION => WeblcmsManager :: ACTION_MANAGER_SORT
                )
            );
        }
        else
        {
            $this->display_page_header();
            $form->display();
            $this->display_footer();
        }
    }

    public function move_course($course_id, $course_type_user_category_id, $direction)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                CourseTypeUserCategoryRelCourse :: class_name(),
                CourseTypeUserCategoryRelCourse :: PROPERTY_COURSE_ID
            ),
            new StaticConditionVariable($course_id)
        );

        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                CourseTypeUserCategoryRelCourse :: class_name(),
                CourseTypeUserCategoryRelCourse :: PROPERTY_COURSE_TYPE_USER_CATEGORY_ID
            ),
            new StaticConditionVariable($course_type_user_category_id)
        );

        $condition = new AndCondition($conditions);

        $course_type_user_category_rel_course = DataManager :: retrieve(
            CourseTypeUserCategoryRelCourse :: class_name(),
            $condition
        );

        $sort_factor = $direction == 'up' ? -1 : 1;
        $sort = $course_type_user_category_rel_course->get_sort();
        $new_sort = $sort + $sort_factor;


        $course_type_user_category_rel_course->set_sort($new_sort);

        return $course_type_user_category_rel_course->update();
    }

    public function move_category($course_type_user_category_id, $direction)
    {
        $condition = new EqualityCondition(CourseTypeUserCategory :: PROPERTY_ID, $course_type_user_category_id);

        $move_category = DataManager :: retrieve(CourseTypeUserCategory :: class_name(), $condition);

        $sort = $move_category->get_sort();

        $next_category = $this->retrieve_course_type_user_category_at_sort(
            $this->get_user_id(), $move_category->get_course_type_id(), $sort, $direction
        );

        if ($direction == 'up')
        {
            $move_category->set_sort($sort - 1);
            $next_category->set_sort($sort);
        }
        elseif ($direction == 'down')
        {
            $move_category->set_sort($sort + 1);
            $next_category->set_sort($sort);
        }

        if ($move_category->update() && $next_category->update())
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function add_course_user_category()
    {
        $courseusercategory = new CourseUserCategory();

        $form = new CourseUserCategoryForm(
            CourseUserCategoryForm :: TYPE_CREATE,
            $courseusercategory,
            $this->get_user(),
            $this->get_url(),
            $this
        );

        if ($form->validate())
        {
            $success = $form->create_course_user_category();
            $this->redirect(
                Translation :: get($success ? 'CourseUserCategoryAdded' : 'CourseUserCategoryNotAdded'),
                ($success ? false : true),
                array(
                    WeblcmsManager :: PARAM_COMPONENT_ACTION => 'view'
                )
            );
        }
        else
        {
            $this->display_page_header();
            $form->display();
            $this->display_footer();
        }
    }

    public function edit_course_user_category()
    {
        $course_type_user_category_id = Request :: get(WeblcmsManager :: PARAM_COURSE_TYPE_USER_CATEGORY_ID);
        $condition = new EqualityCondition(CourseTypeUserCategory :: PROPERTY_ID, $course_type_user_category_id);

        $course_type_user_category = DataManager :: retrieve(CourseTypeUserCategory :: class_name(), $condition);

        $condition = new EqualityCondition(
            CourseUserCategory :: PROPERTY_ID,
            $course_type_user_category->get_course_user_category_id()
        );

        $course_user_category = DataManager :: retrieve(CourseUserCategory :: class_name(), $condition);

        $form = new CourseUserCategoryForm(
            CourseUserCategoryForm :: TYPE_EDIT,
            $course_user_category,
            $this->get_user(),
            $this->get_url(
                array(
                    WeblcmsManager :: PARAM_COURSE_TYPE_USER_CATEGORY_ID => $course_type_user_category_id
                )
            ),
            $this
        );

        if ($form->validate())
        {
            $success = $form->update_course_user_category();
            $this->redirect(
                Translation :: get($success ? 'CourseUserCategoryUpdated' : 'CourseUserCategoryNotUpdated'),
                ($success ? false : true),
                array(
                    WeblcmsManager :: PARAM_COMPONENT_ACTION => 'view'
                )
            );
        }
        else
        {
            $this->display_page_header();
            $form->display();
            $this->display_footer();
        }
    }

    public function delete_course_type_user_category()
    {
        $course_type_user_category_id = Request :: get(WeblcmsManager :: PARAM_COURSE_TYPE_USER_CATEGORY_ID);

        $conditions = array();

        $conditions[] = new EqualityCondition(CourseTypeUserCategory :: PROPERTY_ID, $course_type_user_category_id);
        $condition = new AndCondition($conditions);

        $course_type_user_category = DataManager :: retrieve(CourseTypeUserCategory :: class_name(), $condition);

        $success = $course_type_user_category->delete();
        $this->redirect(
            Translation :: get($success ? 'CourseUserCategoryDeleted' : 'CourseUserCategoryNotDeleted'),
            ($success ? false : true),
            array(
                WeblcmsManager :: PARAM_COMPONENT_ACTION => 'view'
            )
        );
    }

    public function display_page_header()
    {
        $trail = BreadcrumbTrail :: get_instance();

        $this->display_header($trail, false, true);
        echo '<div class="clear"></div><br />';
    }

    /**
     * Shows the tabs of the course types
     * For each of the tabs show the course list
     */
    public function show_course_list()
    {
        $renderer = new CourseTypeCourseListRenderer($this);
        $html = $renderer->as_html();

        $this->display_page_header();
        echo $this->get_actionbar($renderer)->as_html();
        echo '<div class="clear"></div><br />';
        echo $html;
        $this->display_footer();
    }

    /**
     * Returns the action bar for the given course list renderer
     *
     * @param $renderer CourseTypeCourseListRenderer
     *
     * @return ActionBarRenderer
     */
    public function get_actionbar(CourseTypeCourseListRenderer $renderer)
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $action_bar->add_common_action(
            new ToolbarItem(
                Translation :: get('CreateCourseUserCategory'),
                Theme :: get_common_image_path() . 'action_create.png',
                $this->get_course_user_category_add_url(),
                ToolbarItem :: DISPLAY_ICON_AND_LABEL
            )
        );

        /**
         * Retrieve the user order and if not existing, fall back on the default
         * order
         */
        $course_type_id = $renderer->get_selected_course_type_id();

        $course_type_user_order = course_type\DataManager :: retrieve_user_order_for_course_type(
            $course_type_id, $this->get_user_id()
        );

        if ($course_type_user_order)
        {
            $display_order = $course_type_user_order->get_display_order();
            $max = course_type\DataManager :: get_max_display_order_for_course_type_user_orders($this->get_user_id());
        }
        else
        {
            $selected_course_type = $renderer->get_selected_course_type();

            if ($selected_course_type)
            {
                $display_order = $selected_course_type->get_display_order();
                $max = course_type\DataManager :: get_max_display_order_for_course_types();
            }
            else
            {
                $display_order = $max = 1;
            }
        }

        if ($display_order > 1)
        {
            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('MoveCourseType'),
                    Theme :: get_common_image_path() . 'action_left.png',
                    $this->get_url(
                        array(
                            self :: PARAM_COMPONENT_ACTION                             => 'move_course_type_up',
                            CourseTypeCourseListRenderer :: PARAM_SELECTED_COURSE_TYPE => $course_type_id
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }
        else
        {
            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('MoveCourseTypeNA'),
                    Theme :: get_common_image_path() . 'action_left_na.png',
                    null,
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }

        if ($display_order < $max)
        {
            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('MoveCourseType'),
                    Theme :: get_common_image_path() . 'action_right.png',
                    $this->get_url(
                        array(
                            self :: PARAM_COMPONENT_ACTION                             => 'move_course_type_down',
                            CourseTypeCourseListRenderer :: PARAM_SELECTED_COURSE_TYPE => $course_type_id
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }
        else
        {
            $action_bar->add_tool_action(
                new ToolbarItem(
                    Translation :: get('MoveCourseTypeNA'),
                    Theme :: get_common_image_path() . 'action_right_na.png',
                    null,
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }

        return $action_bar;
    }

    public function get_course_actions($course_type_user_category = array(), $course = null, $offset = 1, $count = 1)
    {
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        if ($course_type_user_category)
        {
            if ($offset > 0 && $count > 1)
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('MoveUp', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_up.png',
                        $this->get_course_user_move_url($course_type_user_category, $course, 'up'),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
            else
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('MoveUpNA', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_up_na.png',
                        null,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }

            if ($offset < ($count - 1) && $count > 1)
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('MoveDown', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_down.png',
                        $this->get_course_user_move_url($course_type_user_category, $course, 'down'),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
            else
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('MoveDownNA', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_down_na.png',
                        null,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
        }
        $toolbar->add_item(
            new ToolbarItem(
                Translation :: get('Move', null, Utilities :: COMMON_LIBRARIES),
                Theme :: get_common_image_path() . 'action_move.png',
                $this->get_course_user_edit_url($course_type_user_category, $course),
                ToolbarItem :: DISPLAY_ICON
            )
        );

        return $toolbar->as_html();
    }

    public function get_course_type_user_category_actions($course_type_user_category = array(), $offset = 1, $count = 1)
    {
        if (!$course_type_user_category)
        {
            return;
        }

        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        if ($offset > 0 && $count > 1)
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('MoveUp', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_up.png',
                    $this->get_course_user_category_move_url($course_type_user_category, 'up'),
                    ToolbarItem :: DISPLAY_ICON
                )
            );
        }
        else
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('MoveUpNA', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_up_na.png',
                    null,
                    ToolbarItem :: DISPLAY_ICON
                )
            );
        }

        if ($offset < ($count - 1) && $count > 1)
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('MoveDown', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_down.png',
                    $this->get_course_user_category_move_url($course_type_user_category, 'down'),
                    ToolbarItem :: DISPLAY_ICON
                )
            );
        }
        else
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('MoveDownNA', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_down_na.png',
                    null,
                    ToolbarItem :: DISPLAY_ICON
                )
            );
        }

        $toolbar->add_item(
            new ToolbarItem(
                Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES),
                Theme :: get_common_image_path() . 'action_edit.png',
                $this->get_course_user_category_edit_url($course_type_user_category),
                ToolbarItem :: DISPLAY_ICON
            )
        );

        $toolbar->add_item(
            new ToolbarItem(
                Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                Theme :: get_common_image_path() . 'action_delete.png',
                $this->get_course_user_category_delete_url($course_type_user_category),
                ToolbarItem :: DISPLAY_ICON, true
            )
        );

        return '<div style="float:right;">' . $toolbar->as_html() . '</div>';
    }

    public function get_additional_parameters()
    {
        return array(CourseTypeCourseListRenderer :: PARAM_SELECTED_COURSE_TYPE);
    }

    public function show_empty_courses()
    {
        return true;
    }
}
