<?php
namespace application\weblcms\request\upgrade\v4_0_0;

use common\libraries\package\Package;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\CommonDataManager;

/**
 *
 * @author Pieterjan Broekaert
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\application\weblcms\request\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                $success = $upgrader->upgrade_registration($data_manager);

                if ($success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_NORMAL,
                        Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR,
                        Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                return $success;
            });

        if ($success)
        {
            $this->add_message(
                self :: TYPE_NORMAL,
                Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
        }
        else
        {
            $this->add_message(
                self :: TYPE_ERROR,
                Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
        }

        return $success;
    }

    /**
     * Upgrades the package registration.
     *
     * @return boolean
     */
    public function upgrade_registration()
    {
        $package = Package :: get(\application\weblcms\request\DataManager :: context());
        $registration = CommonDataManager :: get_registration(\application\weblcms\request\DataManager :: context());
        $registration->set_version($package->get_version());

        return $registration->update();
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::get_additional_packages()
     */
    public function get_additional_packages()
    {
        $packages = array();
        $packages[] = 'application\weblcms\request\rights';
        return $packages;
    }
}
