<?php
namespace application\weblcms\tool\assessment;

use application\weblcms\WeblcmsRights;
use application\weblcms\ContentObjectPublication;
use application\weblcms\integration\tracking\AssessmentAttempt;
use repository\ContentObject;
use application\weblcms\ContentObjectPublicationListRenderer;
use application\weblcms\Tool;

use common\libraries\Categorizable;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Utilities;

use repository\content_object\hotpotatoes\Hotpotatoes;
use repository\content_object\assessment\Assessment;

/**
 * $Id: assessment_tool.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.assessment
 */

/**
 * This tool allows a user to publish assessments in his or her course.
 */
class AssessmentTool extends Tool implements Categorizable
{
    const ACTION_VIEW_RESULTS = 'results_viewer';
    const ACTION_ATTEMPT_RESULT_VIEWER = 'attempt_result_viewer';
    const ACTION_SAVE_DOCUMENTS = 'document_saver';
    const ACTION_EXPORT_RESULTS = 'results_export';
    const ACTION_RAW_EXPORT_RESULTS = 'raw_export_results';
    const ACTION_DELETE_RESULTS = 'results_deleter';
    const ACTION_EXPORT_QTI = 'qti_exporter';
    const ACTION_IMPORT_QTI = 'qti_importer';
    const ACTION_TAKE_ASSESSMENT = 'complex_display';

    const PARAM_USER_ASSESSMENT = 'uaid';
    const PARAM_QUESTION_ATTEMPT = 'qaid';
    const PARAM_ASSESSMENT = 'aid';
    const PARAM_ANONYMOUS = 'anonymous';
    const PARAM_INVITATION_ID = 'invitation_id';
    const PARAM_PUBLICATION_ACTION = 'publication_action';

    public static function get_allowed_types()
    {
        return array(Assessment :: CLASS_NAME, Hotpotatoes :: CLASS_NAME);
    }

    public function get_available_browser_types()
    {
        $browser_types = array();
        $browser_types[] = ContentObjectPublicationListRenderer :: TYPE_TABLE;
        $browser_types[] = ContentObjectPublicationListRenderer :: TYPE_LIST;
        $browser_types[] = ContentObjectPublicationListRenderer :: TYPE_CALENDAR;
        return $browser_types;
    }

    public function add_content_object_publication_actions($toolbar, $publication)
    {
        $publication_id = $publication[ContentObjectPublication :: PROPERTY_ID];

        if ($publication[ContentObject :: PROPERTY_TYPE] == Assessment :: class_name())
        {
            $complex_display_item = $toolbar->get_item(1);
            $complex_display_item->set_image(Theme :: get_common_image_path() . 'action_next.png');
            $complex_display_item->set_label(Translation :: get('Take'));
        }

        if ($publication[ContentObject :: PROPERTY_TYPE] == Hotpotatoes :: class_name())
        {
            $toolbar->insert_item(
                new ToolbarItem(
                    Translation :: get('Take'),
                    Theme :: get_common_image_path() . 'action_next.png',
                    $this->get_complex_display_url($publication_id), ToolbarItem :: DISPLAY_ICON
                ), 1
            );
        }

        $toolbar->add_item(
            new ToolbarItem(
                Translation :: get('ManageAttempts'),
                Theme :: get_image_path(__NAMESPACE__) . 'manage_attempts.png',
                $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => AssessmentTool :: ACTION_VIEW_RESULTS,
                        AssessmentTool :: PARAM_ASSESSMENT => $publication_id
                    )
                ), ToolbarItem :: DISPLAY_ICON
            )
        );

        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Export', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_export.png', $this->get_url(
                        array(
                            AssessmentTool :: PARAM_ACTION => AssessmentTool :: ACTION_EXPORT_QTI,
                            Tool :: PARAM_PUBLICATION_ID => $publication_id
                        )
                    ), ToolbarItem :: DISPLAY_ICON
                )
            );
        }
    }

    private static $checked_publications = array();

    public function is_content_object_attempt_possible($publication)
    {
        if (!array_key_exists($publication->get_id(), self :: $checked_publications))
        {
            $assessment = $publication->get_content_object();
            $track = new \application\weblcms\integration\tracking\AssessmentAttempt();
            $condition_t = new EqualityCondition(
                \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_ASSESSMENT_ID, $publication->get_id()
            );
            $condition_u = new EqualityCondition(
                \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_USER_ID, $this->get_user_id()
            );
            $condition = new AndCondition(array($condition_t, $condition_u));
            $trackers = $track->retrieve_tracker_items($condition);

            $count = count($trackers);

            foreach ($trackers as $tracker)
            {
                if ($tracker->get_status() == AssessmentAttempt :: STATUS_NOT_COMPLETED)
                {
                    $this->active_tracker = $tracker;
                    $count--;
                    break;
                }
            }

            self :: $checked_publications[$publication->get_id()] = (
                $assessment->get_maximum_attempts() == 0 || $count < $assessment->get_maximum_attempts()
            );
        }

        return self :: $checked_publications[$publication->get_id()];
    }

    /**
     * Helper function for the SubManager class,
     * pending access to class constants via variables in PHP 5.3
     * e.g.
     * $name = $class :: DEFAULT_ACTION
     *
     * DO NOT USE IN THIS SUBMANAGER'S CONTEXT
     * Instead use:
     * - self :: DEFAULT_ACTION in the context of this class
     * - YourSubManager :: DEFAULT_ACTION in all other application classes
     */
    public static function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    /**
     * Helper function for the SubManager class,
     * pending access to class constants via variables in PHP 5.3
     * e.g.
     * $name = $class :: PARAM_ACTION
     *
     * DO NOT USE IN THIS SUBMANAGER'S CONTEXT
     * Instead use:
     * - self :: PARAM_ACTION in the context of this class
     * - YourSubManager :: PARAM_ACTION in all other application classes
     */
    public static function get_action_parameter()
    {
        return self :: PARAM_ACTION;
    }
}
