<?php
namespace application\weblcms\tool\assessment;

use application\weblcms\WeblcmsManager;
use application\weblcms\tool\reporting\ReportingTool;
use common\libraries\DelegateComponent;
use application\weblcms\ContentObjectPublication;

use application\weblcms\WeblcmsRights;
use application\weblcms\ToolComponent;
use application\weblcms\Tool;

use repository\content_object\calendar_event\CalendarEvent;
use repository\ContentObject;

use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\Translation;

class AssessmentToolBrowserComponent extends AssessmentTool implements DelegateComponent
{

    public function run()
    {
        ToolComponent :: launch($this);
    }

    public function get_tool_actions()
    {
        $tool_actions = array();

        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $tool_actions[] = new ToolbarItem(
                Translation :: get('ImportQti'),
                Theme :: get_common_image_path() . 'action_import.png', $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => AssessmentTool :: ACTION_IMPORT_QTI
                    )
                ), ToolbarItem :: DISPLAY_ICON_AND_LABEL
            );

            $tool_actions[] = new ToolbarItem(
                Translation :: get('Reporting'),
                Theme :: get_common_image_path() . 'action_view_results.png', $this->get_url(
                    array(
                        WeblcmsManager :: PARAM_TOOL => 'reporting',
                        Tool :: PARAM_ACTION => TOOL :: ACTION_VIEW,
                        \reporting\viewer\Manager :: PARAM_BLOCK_ID => 2
                    )
                ), ToolbarItem :: DISPLAY_ICON_AND_LABEL
            );

        }

        return $tool_actions;
    }

    public function convert_content_object_publication_to_calendar_event($publication, $from_time, $to_time)
    {
        $calendar_event = ContentObject :: factory(CalendarEvent :: class_name());

        $calendar_event->set_title($publication[ContentObject :: PROPERTY_TITLE]);
        $calendar_event->set_description($publication[ContentObject :: PROPERTY_DESCRIPTION]);
        $calendar_event->set_start_date($publication[ContentObjectPublication :: PROPERTY_MODIFIED_DATE]);
        $calendar_event->set_end_date($publication[ContentObjectPublication :: PROPERTY_MODIFIED_DATE]);
        $calendar_event->set_frequency(CalendarEvent :: FREQ_NONE);

        return $calendar_event;
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_BROWSE_PUBLICATION_TYPE);
    }
}
