<?php
namespace application\weblcms\tool\assessment;

use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use common\libraries\EqualityCondition;
use common\libraries\DataClassRetrieveParameters;
use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsManager;
use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use application\weblcms\ToolComponent;
use common\libraries\WebApplication;
use repository\content_object\introduction\Introduction;
use common\libraries\Request;
use common\libraries\Translation;
use application\gradebook\GradebookUtilities;

class AssessmentToolDeleterComponent extends AssessmentTool
{

    /**
     * Modified version of the default Deleter to allow for the feedback-functionality
     */
    public function run()
    {
        if (Request :: get(Tool :: PARAM_PUBLICATION_ID))
        {
            $publication_ids = Request :: get(Tool :: PARAM_PUBLICATION_ID);
        }
        else
        {
            $publication_ids = $_POST[Tool :: PARAM_PUBLICATION_ID];
        }

        if (! is_array($publication_ids))
        {
            $publication_ids = array($publication_ids);
        }

        $failures = 0;

        foreach ($publication_ids as $pid)
        {
            $publication = \application\weblcms\DataManager :: retrieve(ContentObjectPublication :: class_name(), $pid);

            $content_object = $publication->get_content_object();

            if ($content_object->get_type() == Introduction :: class_name())
            {
                $publication->ignore_display_order();
            }

            if (WebApplication :: is_active('gradebook'))
            {
                GradebookUtilities :: move_internal_item_to_external_item(
                    WeblcmsManager :: APPLICATION_NAME,
                    $publication->get_id());
            }
            if ($this->is_allowed(WeblcmsRights :: DELETE_RIGHT, $publication))
            {
                if ($publication->delete())
                {
                    $parameters = new DataClassRetrieveParameters(
                        new EqualityCondition(
                            new PropertyConditionVariable(
                                Publication :: class_name(),
                                Publication :: PROPERTY_PUBLICATION_ID),
                            new StaticConditionVariable($publication->get_id())));
                    $assessment_publication = DataManager :: retrieve(Publication :: class_name(), $parameters);
                    if (! $assessment_publication->delete())
                    {
                        $failures ++;
                    }
                }
                else
                {
                    $failures ++;
                }
            }
            else
            {
                $failures ++;
            }
        }

        if ($failures == 0)
        {
            if (count($publication_ids) > 1)
            {
                $message = htmlentities(Translation :: get('ContentObjectPublicationsDeleted'));
            }
            else
            {
                $message = htmlentities(Translation :: get('ContentObjectPublicationDeleted'));
            }
        }
        else
        {
            $message = htmlentities(Translation :: get('ContentObjectPublicationsNotDeleted'));
        }

        $this->redirect($message, $failures !== 0, array(Tool :: PARAM_PUBLICATION_ID => null, 'tool_action' => null));
    }
}
