<?php
namespace application\weblcms\tool\assessment;

use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use application\weblcms\Tool;
use application\weblcms\ToolComponent;
use common\libraries\Display;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\PlatformSetting;
use common\libraries\Request;

class AssessmentToolPublisherComponent extends AssessmentTool
{

    /**
     * Modified version of the default Publisher to allow for the feedback-functionality
     */
    public function run()
    {
        if (! ($this->get_course()->is_course_admin($this->get_user()) || $this->is_allowed(WeblcmsRights :: ADD_RIGHT)))
        {
            Display :: error_page(Translation :: get("NotAllowed"));
            return;
        }

        if (! \repository\viewer\Manager :: is_ready_to_be_published())
        {
            $repository_viewer = \repository\viewer\Manager :: construct($this);
            $repository_viewer->run();
        }
        else
        {
            $objects = \repository\viewer\Manager :: get_selected_objects();

            $mode = Request :: get(Tool :: PARAM_PUBLISH_MODE);
            $publish_type = PlatformSetting :: get('display_publication_screen', WeblcmsManager :: context());
            $show_form = (($publish_type == Tool :: PUBLISH_TYPE_FORM) ||
                 ($publish_type == Tool :: PUBLISH_TYPE_BOTH && $mode != Tool :: PUBLISH_MODE_QUICK));

            $publisher = new Publisher($this, $objects, $show_form);

            if ($publisher->ready_to_publish())
            {
                $success = $publisher->publish();

                $message = Translation :: get(
                    ($success ? 'ObjectPublished' : 'ObjectNotPublished'),
                    array('OBJECT' => Translation :: get('Object')),
                    Utilities :: COMMON_LIBRARIES);

                $parameters = array(Tool :: PARAM_ACTION => Tool :: ACTION_BROWSE);

                if ($publisher->is_publish_and_build_submit())
                {
                    $parameters[Tool :: PARAM_ACTION] = Tool :: ACTION_BUILD_COMPLEX_CONTENT_OBJECT;

                    $publications = $publisher->get_publications();
                    $parameters[Tool :: PARAM_PUBLICATION_ID] = $publications[0]->get_id();
                }

                $this->redirect($message, ! $success, $parameters);
            }
            else
            {
                $this->display_header();
                echo $publisher->toHtml();
                $this->display_footer();
            }
        }
    }

    public function get_allowed_content_object_types()
    {
        return $this->get_allowed_types();
    }

    public function get_additional_parameters()
    {
        return array(\repository\viewer\Manager :: PARAM_ID, \repository\viewer\Manager :: PARAM_ACTION);
    }
}
