<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\ContentObjectPublication;





use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsManager;
use application\weblcms\Tool;
use common\libraries\Request;
use common\libraries\EqualityCondition;
use common\libraries\Translation;

/**
 *
 * @package application.weblcms.tool.assignment.php.component
 *
 *          Deleter for assignments.
 *
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssignmentToolDeleterComponent extends AssignmentTool
{

    public function run()
    {
        if (Request :: get(Tool :: PARAM_PUBLICATION_ID))
            $publication_ids = Request :: get(Tool :: PARAM_PUBLICATION_ID);
        else
            $publication_ids = $_POST[Tool :: PARAM_PUBLICATION_ID];

        if (!is_array($publication_ids))
        {
            $publication_ids = array($publication_ids);
        }

        $failures = 0;

        foreach ($publication_ids as $pid)
        {
            $publication = \application\weblcms\DataManager :: retrieve(
                ContentObjectPublication :: class_name(),
                $pid
            );

            if ($this->is_allowed(WeblcmsRights :: DELETE_RIGHT, $publication))
            {
                $publication->delete();
                $condition = new EqualityCondition(
                    \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_PUBLICATION_ID, $pid
                );
                $submissions = \application\weblcms\integration\tracking\AssignmentSubmission :: get_data(
                    \application\weblcms\integration\tracking\AssignmentSubmission :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition
                );
                while ($submission = $submissions->next_result())
                {
                    //$id = $submission->get_id();
                    $subm_condition = new EqualityCondition(
                        \application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_SUBMISSION_ID, $submission->get_id()
                    );
                    \application\weblcms\integration\tracking\SubmissionFeedback :: remove_data(
                        \application\weblcms\integration\tracking\SubmissionFeedback :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME,
                        $subm_condition
                    );
                    \application\weblcms\integration\tracking\SubmissionScore :: remove_data(
                        \application\weblcms\integration\tracking\SubmissionScore :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $subm_condition
                    );
                    \application\weblcms\integration\tracking\SubmissionNote :: remove_data(
                        \application\weblcms\integration\tracking\SubmissionNote :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $subm_condition
                    );
                    $submission->delete();
                }
            }
            else
            {
                $failures++;
            }
        }
        if ($failures == 0)
        {
            if (count($publication_ids) > 1)
            {
                $message = htmlentities(Translation :: get('ContentObjectPublicationsDeleted'));
            }
            else
            {
                $message = htmlentities(Translation :: get('ContentObjectPublicationDeleted'));
            }
        }
        else
        {
            $message = htmlentities(Translation :: get('ContentObjectPublicationsNotDeleted'));
        }

        $this->redirect($message, $failures > 0, array(Tool :: PARAM_PUBLICATION_ID => null, 'tool_action' => null));
    }

}
