<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\WeblcmsRights;
use application\weblcms\ContentObjectPublication;
use common\libraries\Display;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Request;
use application\weblcms\Tool;



/**
 *
 * @package application.weblcms.tool.assignment.php.component
 *
 *          Deleter for submission feedback.
 *
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 */
class AssignmentToolFeedbackDeleterComponent extends AssignmentTool
{

    public function run()
    {
        $pub = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            Request :: get(Tool :: PARAM_PUBLICATION_ID)
        );

        if (!$this->is_allowed(WeblcmsRights :: DELETE_RIGHT, $pub))
        {
            Display :: not_allowed();
            return;
        }

        $ids = Request :: get(AssignmentTool :: PARAM_FEEDBACK_ID);
        if (!is_array($ids))
        {
            $feedback_ids[] = $ids;
        }
        else
        {
            $feedback_ids = $ids;
        }
        $tracker = new \application\weblcms\integration\tracking\SubmissionFeedback();

        foreach ($feedback_ids as $fid)
        {
            $condition = new EqualityCondition(\application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_ID, $fid);
            $feedbacks = $tracker->retrieve_tracker_items($condition);
            foreach ($feedbacks as $feedback)
            {
                $submission_id = $feedback->get_submission_id();
                $feedback->delete();
            }
        }

        $publication_id = Request :: get(AssignmentTool :: PARAM_PUBLICATION_ID);
        $target_id = Request :: get(AssignmentTool :: PARAM_TARGET_ID);
        $submitter_type = Request :: get(AssignmentTool :: PARAM_SUBMITTER_TYPE);

        $params = array(
            Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION,
            Tool :: PARAM_PUBLICATION_ID => $publication_id, AssignmentTool :: PARAM_TARGET_ID => $target_id,
            AssignmentTool :: PARAM_SUBMITTER_TYPE => $submitter_type,
            AssignmentTool :: PARAM_SUBMISSION => $submission_id
        );

        $this->redirect(Translation :: get('FeedbackDeleted'), false, $params);
    }

}
