<?php

namespace application\weblcms\tool\assignment;

use group\DataManager as GroupDataManager;
use group\Group;

use application\weblcms\tool\course_group\CourseGroup;
use application\weblcms\ContentObjectPublication;

use common\libraries\Display;

use application\weblcms\WeblcmsManager;

use common\libraries\EqualityCondition;
use common\libraries\Request;
use application\weblcms\Tool;
use repository\ContentObjectForm;
use common\libraries\Translation;
use common\libraries\Breadcrumb;

use application\weblcms\WeblcmsRights;

use user\UserDataManager;

/**
 *
 * @package application.weblcms.tool.assignment.php.component
 *
 *          Updater for submission feedback.
 *
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssignmentToolFeedbackUpdaterComponent extends AssignmentTool
{

    public function run()
    {
        $feedback_id = Request :: get(AssignmentTool :: PARAM_FEEDBACK_ID);
        $tracker = new \application\weblcms\integration\tracking\SubmissionFeedback();
        $condition = new EqualityCondition(\application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_ID, $feedback_id);
        $feedbacks = $tracker->retrieve_tracker_items($condition);
        $feedback = $feedbacks[0];
        $feedback_content_object = $feedback->get_content_object();

        $object_form = ContentObjectForm :: factory(
            ContentObjectForm :: TYPE_EDIT, $feedback_content_object, 'edit', 'post', $this->get_url(
                array(
                    AssignmentTool :: PARAM_FEEDBACK_ID => $feedback_id
                )
            ), null, null, false
        );

        $publication_id = Request :: get(AssignmentTool :: PARAM_PUBLICATION_ID);
        $target_id = Request :: get(AssignmentTool :: PARAM_TARGET_ID);
        $submitter_type = Request :: get(AssignmentTool :: PARAM_SUBMITTER_TYPE);

        if ($object_form->validate())
        {
            $success = $object_form->update_content_object();
            $this->redirect(
                Translation :: get($success ? 'FeedbackUpdated' : 'UpdateFailed'), !$success, array(
                    Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION,
                    Tool :: PARAM_PUBLICATION_ID => $publication_id, AssignmentTool :: PARAM_TARGET_ID => $target_id,
                    AssignmentTool :: PARAM_SUBMITTER_TYPE => $submitter_type,
                    AssignmentTool :: PARAM_SUBMISSION => $feedback->get_submission_id()
                )
            );
        }
        else
        {
            $this->display_header();
            $object_form->display();
            $this->display_footer();
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $submission_id = Request :: get(AssignmentTool :: PARAM_SUBMISSION);
        $tracker = new \application\weblcms\integration\tracking\AssignmentSubmission();
        $condition = new EqualityCondition(\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_ID, $submission_id);
        $submission_trackers = $tracker->retrieve_tracker_items($condition);
        $submission_tracker = $submission_trackers[0];

        $submissions = \application\weblcms\integration\tracking\AssignmentSubmission :: get_data(
            \application\weblcms\integration\tracking\AssignmentSubmission :: CLASS_NAME, WeblcmsManager :: APPLICATION_NAME, $condition
        )->as_array();
        $submission = $submissions[0]->get_content_object();

        $pub = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $submission_tracker->get_publication_id()
        );

        if (!$this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $pub))
        {
            Display :: not_allowed();
            return;
        }
        $assignment = $pub->get_content_object();

        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_BROWSE)),
                Translation :: get('AssignmentToolBrowserComponent')
            )
        );
        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMITTERS,
                        Tool :: PARAM_PUBLICATION_ID => $submission_tracker->get_publication_id()
                    )
                ), $assignment->get_title()
            )
        );
        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS,
                        Tool :: PARAM_PUBLICATION_ID => $submission_tracker->get_publication_id(),
                        AssignmentTool :: PARAM_TARGET_ID => $submission_tracker->get_submitter_id(),
                        AssignmentTool :: PARAM_SUBMITTER_TYPE => $submission_tracker->get_submitter_type()
                    )
                ), $this->get_submitter_name($submission_tracker)
            )
        );
        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION,
                        Tool :: PARAM_PUBLICATION_ID => $submission_tracker->get_publication_id(),
                        AssignmentTool :: PARAM_TARGET_ID => $submission_tracker->get_submitter_id(),
                        AssignmentTool :: PARAM_SUBMITTER_TYPE => $submission_tracker->get_submitter_type(),
                        AssignmentTool :: PARAM_SUBMISSION => $submission_id
                    )
                ), $submission->get_title() . ' - ' . Translation :: get('Detail')
            )
        );
    }

    public function get_additional_parameters()
    {
        return array(
            Tool :: PARAM_PUBLICATION_ID, AssignmentTool :: PARAM_FEEDBACK_ID,
            AssignmentTool :: PARAM_SUBMISSION
        );
    }

    /**
     * Returns the name of the submitter as a string.
     * When submitted as a group, it will return the name of the user who
     * submitted followed by the group name.
     *
     * @return string The name of the submitter
     */
    private function get_submitter_name($submission_tracker)
    {
        // name of the user who submitted
        $user_name = UserDataManager :: get_instance()->retrieve_user($submission_tracker->get_user_id());

        switch ($submission_tracker->get_submitter_type())
        {
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER :
                return $user_name->get_fullname();
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_COURSE_GROUP :
                return $user_name->get_fullname() . ' - ' .
                    \application\weblcms\DataManager :: retrieve(
                        CourseGroup :: class_name(),
                        $submission_tracker->get_submitter_id()
                    )->get_name();
            case \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_PLATFORM_GROUP :
                return $user_name->get_fullname() . ' - ' .
                    GroupDataManager :: retrieve_by_id(Group :: class_name(), $submission_tracker->get_submitter_id())
                        ->get_name();
        }
    }

}
