<?php

namespace application\weblcms\tool\assignment;

use application\weblcms\WeblcmsRights;
use application\weblcms\ContentObjectPublication;
use common\libraries\Display;
use common\libraries\EqualityCondition;

use common\libraries\Translation;
use common\libraries\Request;
use application\weblcms\Tool;





/**
 *
 * @package application.weblcms.tool.assignment.php.component
 *
 *          Deleter for submissions.
 *
 * @author Joris Willems <joris.willems@gmail.com>
 * @author Alexander Van Paemel
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssignmentToolSubmissionDeleterComponent extends AssignmentTool
{

    public function run()
    {
        $pub = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            Request :: get(Tool :: PARAM_PUBLICATION_ID)
        );

        if (!$this->is_allowed(WeblcmsRights :: DELETE_RIGHT, $pub))
        {
            Display :: not_allowed();
            return;
        }

        if (Request :: get(AssignmentTool :: PARAM_SUBMISSION))
        {
            $submission_ids = Request :: get(AssignmentTool :: PARAM_SUBMISSION);
        }
        else
        {
            $table_name = Request :: post('table_name');
            $submission_ids = Request :: post($table_name . '_id');
        }

        if (!is_array($submission_ids))
        {
            $submission_ids = array($submission_ids);
        }

        $submission_tracker = new \application\weblcms\integration\tracking\AssignmentSubmission();

        foreach ($submission_ids as $sid)
        {
            $condition = new EqualityCondition(\application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_ID, $sid);
            $submissions = $submission_tracker->retrieve_tracker_items($condition);
            $submission = $submissions[0];
            $submission_id = $submission->get_id();
            $publication_id = $submission->get_publication_id();
            $submitter_type = $submission->get_submitter_type();
            $submitter_id = $submission->get_submitter_id();
            $submission->delete();

            // delete the feedbacks of the submission
            $feedback_tracker = new \application\weblcms\integration\tracking\SubmissionFeedback();
            $condition = new EqualityCondition(
                \application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_SUBMISSION_ID, $submission_id
            );
            $feedback_tracker->remove($condition);

            // delete the score of the submission
            $score_tracker = new \application\weblcms\integration\tracking\SubmissionScore();
            $score_tracker->remove($condition);

            // delete the note of the submission
            $note_tracker = new \application\weblcms\integration\tracking\SubmissionNote();
            $note_tracker->remove($condition);
        }

        $params = array(
            Tool :: PARAM_ACTION => AssignmentTool :: ACTION_BROWSE_SUBMISSIONS,
            AssignmentTool :: PARAM_PUBLICATION_ID => $publication_id,
            AssignmentTool :: PARAM_SUBMITTER_TYPE => $submitter_type,
            AssignmentTool :: PARAM_TARGET_ID => $submitter_id
        );

        $this->redirect(Translation :: get('SubmissionDeleted'), false, $params);
    }

}
