<?php
namespace application\weblcms\tool\assignment;

use application\weblcms\Tool;

use application\weblcms\WeblcmsRights;


use common\libraries\DatetimeUtilities;
use common\libraries\EqualityCondition;
use common\libraries\Theme;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\DefaultContentObjectTableCellRenderer;

/**
 * Description of submitter_submissions_user_browser_table_cell_renderer
 *
 * @author Anthony Hurst (Hogeschool Gent)
 * @author Bert De Clercq (Hogeschool Gent)
 */
class SubmitterUserSubmissionsBrowserTableCellRenderer extends DefaultContentObjectTableCellRenderer
{
    private $table_actions;
    private $browser;

    public function __construct($browser)
    {
        $this->table_actions = array();
        $this->browser = $browser;
    }

    public function render_cell($column, $submission)
    {
        if ($column === SubmitterSubmissionsTableColumnModel :: get_action_column())
        {
            return $this->get_actions($submission);
        }

        switch ($column->get_name())
        {
            case SubmitterSubmissionsTableColumnModel :: PROPERTY_PUBLICATION_TITLE :
                $content_object = $submission->get_content_object();
                $title = $content_object ? $content_object->get_title() : Translation :: get('ContentObjectUnknown');

                if (
                    $this->browser->get_assignment()->get_visibility_submissions() ||
                    $this->browser->is_allowed(WeblcmsRights :: EDIT_RIGHT) ||
                    $submission->get_submitter_id() == $this->browser->get_user_id()
                )
                {
                    $url = $this->browser->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION,
                            Tool :: PARAM_PUBLICATION_ID => $submission->get_publication_id(),
                            AssignmentTool :: PARAM_TARGET_ID => $submission->get_submitter_id(),
                            AssignmentTool :: PARAM_SUBMITTER_TYPE =>
                            \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER,
                            AssignmentTool :: PARAM_SUBMISSION => $submission->get_id()
                        )
                    );



                    return '<a href=\'' . $url . '\'>' . $title . '</a>';
                }
                return $title;

            case SubmitterSubmissionsTableColumnModel :: PROPERTY_CONTENT_OBJECT_DESCRIPTION :
                $content_object = $submission->get_content_object();
                return $content_object ? $content_object->get_description() : Translation :: get('ContentObjectUnknown');

            case \application\weblcms\integration\tracking\AssignmentSubmission :: PROPERTY_DATE_SUBMITTED :
                return $this->format_date($submission->get_date_submitted());
            case \application\weblcms\integration\tracking\SubmissionScore :: PROPERTY_SCORE :
                return $this->get_score_from_submission($submission->get_id());
            case AssignmentTool :: PROPERTY_NUMBER_OF_FEEDBACKS :
                return $this->get_number_of_feedback($submission->get_id());
        }
    }

    /**
     * Creates a toolbar with the appropriate actions
     *
     * @param $submission type
     * @return string The HTML code that represents the actions.
     */
    private function get_actions($submission)
    {
        $toolbar = new Toolbar();

        if ($submission->get_user_id() == $this->browser->get_user_id() ||
            $this->browser->get_assignment()->get_visibility_submissions() ||
            $this->browser->is_allowed(WeblcmsRights :: EDIT_RIGHT)
        )
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('ViewSubmission'),
                    Theme :: get_common_image_path() . 'action_browser.png', $this->browser->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssignmentTool :: ACTION_VIEW_SUBMISSION,
                            Tool :: PARAM_PUBLICATION_ID => $submission->get_publication_id(),
                            AssignmentTool :: PARAM_TARGET_ID => $submission->get_submitter_id(),
                            AssignmentTool :: PARAM_SUBMITTER_TYPE =>
                            \application\weblcms\integration\tracking\AssignmentSubmission :: SUBMITTER_TYPE_USER,
                            AssignmentTool :: PARAM_SUBMISSION => $submission->get_id()
                        )
                    ), ToolbarItem :: DISPLAY_ICON
                )
            );
        }
        if ($this->browser->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if ($submission->get_content_object() &&
                SubmissionsManager :: is_downloadable($submission->get_content_object()))
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('DownloadSubmission'),
                        Theme :: get_common_image_path() . 'action_download.png', $this->browser->get_url(
                            array(
                                Tool :: PARAM_ACTION => AssignmentTool :: ACTION_DOWNLOAD_SUBMISSIONS,
                                AssignmentTool :: PARAM_SUBMISSION => $submission->get_id()
                            )
                        ), ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
            else
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('DownloadNotPossible'),
                        Theme :: get_common_image_path() . 'action_download_na.png', null, ToolbarItem :: DISPLAY_ICON
                    )
                );
            }

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('DeleteSubmission'),
                    Theme :: get_common_image_path() . 'action_delete.png', $this->browser->get_url(
                        array(
                            Tool :: PARAM_ACTION => AssignmentTool :: ACTION_DELETE_SUBMISSION,
                            AssignmentTool :: PARAM_SUBMISSION => $submission->get_id()
                        )
                    ), ToolbarItem :: DISPLAY_ICON, true
                )
            );
        }
        return $toolbar->as_html();
    }

    /**
     * Formats a date.
     *
     * @param $date type
     *            the date to be formatted.
     * @return the formatted representation of the date.
     */
    private function format_date($date)
    {
        $formatted_date = DatetimeUtilities :: format_locale_date(
            Translation :: get('DateTimeFormatLong', null, Utilities :: COMMON_LIBRARIES), $date
        );

        if ($date > $this->browser->get_assignment()->get_end_time())
        {
            return '<span style="color:red">' . $formatted_date . '</span>';
        }

        return $formatted_date;
    }

    /**
     * Retrieves the score of the submission.
     *
     * @param $submission_id type
     *            the identity of the submission.
     * @return type the score of the submission.
     */
    private function get_score_from_submission($submission_id)
    {
        $tracker = new \application\weblcms\integration\tracking\SubmissionScore();
        $condition = new EqualityCondition(\application\weblcms\integration\tracking\SubmissionScore :: PROPERTY_SUBMISSION_ID, $submission_id);
        $trackers = $tracker->retrieve_tracker_items($condition);
        // $trackers = $tracker->get_data()

        if (count($trackers) > 0)
        {
            return $trackers[0]->get_score() . '%';
        }
        return null;
    }

    /**
     * Return the number of feedback a submission has.
     *
     * @param $submission_id int
     *            The id of the submission
     * @return int The number of feedback
     */
    private function get_number_of_feedback($submission_id)
    {
        $tracker = new \application\weblcms\integration\tracking\SubmissionFeedback();
        $condition = new EqualityCondition(\application\weblcms\integration\tracking\SubmissionFeedback :: PROPERTY_SUBMISSION_ID, $submission_id);
        return $tracker->count_tracker_items($condition);
    }

}
