<?php
namespace application\weblcms\tool\calendar;

use repository\content_object\calendar_event\CalendarEvent;
use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;

use application\weblcms\ContentObjectPublicationCourseGroup;

use application\weblcms\ContentObjectPublicationUser;

use application\weblcms\WeekCalendarContentObjectPublicationListRenderer;

use application\weblcms\ContentObjectPublication;
use application\weblcms\tool\course_group\CourseGroup;

use application\weblcms\WeblcmsRights;
use application\weblcms\Tool;
use repository\ContentObject;
use common\libraries\ObjectTableFormAction;
use common\libraries\SubselectCondition;

use user\User;
use user\UserDataManager;
use common\libraries\FormValidator;
use common\libraries\OrCondition;
use common\libraries\InCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use application\weblcms\ContentObjectPublicationBrowser;
use application\weblcms\MiniMonthCalendarContentObjectPublicationListRenderer;
use application\weblcms\MonthCalendarContentObjectPublicationListRenderer;
use application\weblcms\DayCalendarContentObjectPublicationListRenderer;
use application\weblcms\ContentObjectPublicationDetailsRenderer;
use application\weblcms\course\CourseUserRelation;

/**
 * $Id: calendar_browser.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.calendar.component.calendar_viewer
 */
class CalendarBrowser extends ContentObjectPublicationBrowser
{
    const CALENDAR_MONTH_VIEW = 'month';
    const CALENDAR_WEEK_VIEW = 'week';
    const CALENDAR_DAY_VIEW = 'day';
    const CALENDAR_LIST_VIEW = 'list';
    private $publications;
    private $time;

    public function __construct($parent)
    {
        parent :: __construct($parent, 'calendar');
        if (Request :: get(Tool :: PARAM_PUBLICATION_ID))
        {
            $this->set_publication_id(Request :: get(Tool :: PARAM_PUBLICATION_ID));
            // $renderer = new ContentObjectPublicationDetailsRenderer($this);
            $renderer = new CalendarDetailsRenderer($this);
        }
        else
        {
            $time = Request :: get('time') ? intval(Request :: get('time')) : time();
            $this->time = $time;
            // $this->set_parameter('time',$time);

            switch (Request :: get('view'))
            {
                case CalendarBrowser :: CALENDAR_DAY_VIEW :
                    {
                    $renderer = new DayCalendarContentObjectPublicationListRenderer($this);
                    $renderer->set_display_time($time);
                    break;
                    }
                case CalendarBrowser :: CALENDAR_WEEK_VIEW :
                    {
                    $renderer = new WeekCalendarContentObjectPublicationListRenderer($this);
                    $renderer->set_display_time($time);
                    break;
                    }
                case CalendarBrowser :: CALENDAR_MONTH_VIEW :
                    {
                    $renderer = new MonthCalendarContentObjectPublicationListRenderer($this);
                    $renderer->set_display_time($time);
                    break;
                    }
                case CalendarBrowser :: CALENDAR_LIST_VIEW :
                    {
                    $renderer = new CalendarListRenderer($this);
                    // $actions = array(Tool :: ACTION_DELETE => Translation
                    // :: get('DeleteSelected', null , Utilities ::
                    // COMMON_LIBRARIES), Tool :: ACTION_HIDE => Translation
                    // :: get('Hide', null , Utilities :: COMMON_LIBRARIES),
                    // Tool :: ACTION_SHOW => Translation :: get('Show',
                    // null , Utilities :: COMMON_LIBRARIES));

                    $actions[] = new ObjectTableFormAction(
                        Tool :: ACTION_DELETE,
                        Translation :: get('DeleteSelected', null, Utilities :: COMMON_LIBRARIES)
                    );
                    $actions[] = new ObjectTableFormAction(
                        Tool :: ACTION_HIDE,
                        Translation :: get('Hide', null, Utilities :: COMMON_LIBRARIES), false
                    );
                    $actions[] = new ObjectTableFormAction(
                        Tool :: ACTION_SHOW,
                        Translation :: get('Show', null, Utilities :: COMMON_LIBRARIES), false
                    );

                    $renderer->set_actions($actions);
                    break;
                    }
                default :
                    {
                    $renderer = new MonthCalendarContentObjectPublicationListRenderer($this);
                    $renderer->set_display_time($time);
                    break;
                    }
            }
        }

        $this->set_publication_list_renderer($renderer);
    }

    public function get_publications($from, $count, $column, $direction)
    {
        if (isset($this->publications))
        {
            return $this->publications;
        }
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $user_id = array();
            $course_group_ids = array();

            $filter = Request :: post('filter');

            if ($filter)
            {
                if (strpos($filter, 'user') !== false)
                {
                    $user_id = substr($filter, 5);
                }

                /*if (strpos($filter, 'group') !== false)
                {
                    $course_groups = array(substr($filter, 6));
                }*/
            }
        }
        else
        {
            $user_id = $this->get_user_id();
            $course_groups = $this->get_course_groups();

            $course_group_ids = array();

            foreach ($course_groups as $course_group)
            {
                $course_group_ids[] = $course_group->get_id();
            }
        }

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $this->get_course_id());
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_TOOL, 'calendar');

        $access = array();
        $access[] = new InCondition('user_id', $user_id, ContentObjectPublicationUser :: get_table_name());
        $access[] = new InCondition(
            'course_group_id', $course_group_ids, ContentObjectPublicationCourseGroup :: get_table_name()
        );
        if (!empty($user_id) || !empty($course_group_ids))
        {
            $access[] = new AndCondition(
                array(
                    new EqualityCondition('user_id', null, ContentObjectPublicationUser :: get_table_name()),
                    new EqualityCondition(
                        'course_group_id', null, ContentObjectPublicationCourseGroup :: get_table_name()
                    )
                )
            );
        }
        $conditions[] = new OrCondition($access);

        $subselect_conditions = array();
        $subselect_conditions[] = new EqualityCondition(
            ContentObject :: PROPERTY_TYPE, CalendarEvent :: class_name()
        );
        if ($this->get_parent()->get_condition())
        {
            $subselect_conditions[] = $this->get_parent()->get_condition();
        }
        $subselect_condition = new AndCondition($subselect_conditions);

        $conditions[] = new SubselectCondition(
            new PropertyConditionVariable(
                ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID
            ),
            new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID),
            ContentObject :: get_table_name(), $subselect_condition
        );

        $condition = new AndCondition($conditions);

        if ($column)
        {
            $order = new ObjectTableOrder($column, $direction);
        }
        $this->publications = \application\weblcms\DataManager :: retrieve_content_object_publications($condition, $order)->as_array();
        return $this->publications;
    }

    public function get_publication_count()
    {
        return count($this->get_publications());
    }

    /**
     * Get calendar events in a certain time range
     *
     * @param $from_time int
     * @param $to_time int
     * @return array A set of publications of calendar_events
     */
    public function get_calendar_events($from_time, $to_time)
    {
        $publications = $this->get_publications();

        $events = array();
        foreach ($publications as $publication)
        {
            $object = $publication->get_content_object();

            if ($object->repeats())
            {
                $repeats = $object->get_repeats($from_time, $to_time);

                foreach ($repeats as $repeat)
                {
                    $the_publication = clone $publication;
                    $the_publication->set_content_object($repeat);

                    $events[] = $the_publication;
                }
            }

            elseif (
                $from_time <= $object->get_start_date() && $object->get_start_date() <= $to_time ||
                $from_time <= $object->get_end_date() && $object->get_end_date() <= $to_time ||
                $object->get_start_date() <= $from_time && $to_time <= $object->get_end_date()
            )
            {
                $events[] = $publication;
            }
        }

        return $events;
    }

    public function as_html()
    {
        if (!Request :: get(Tool :: PARAM_PUBLICATION_ID))
        {
            $minimonthcalendar = new MiniMonthCalendarContentObjectPublicationListRenderer($this);
            $minimonthcalendar->set_display_time($this->time);
            $html[] = '<div class="mini_calendar">';
            $html[] = $minimonthcalendar->as_html();
            $html[] = '<br />';
            $html[] = $this->render_upcomming_events();
            $html[] = '</div>';
            // style="margin-left: 0px; float: right; width: 70%;"
            $html[] = '<div class="normal_calendar">';

            if (
                $this->get_parent()->is_allowed(WeblcmsRights :: EDIT_RIGHT) &&
                parent :: get_publication_list_renderer() instanceof CalendarListRenderer
            )
            {
                $html[] = '<div style="float: right;">';

                $form = new FormValidator('user_filter', 'post', $this->get_parent()->get_url());
                $renderer = $form->defaultRenderer();
                $renderer->setElementTemplate('{element}');
                $form->addElement(
                    'select', 'filter', Translation :: get('Filter', null, Utilities :: COMMON_LIBRARIES),
                    $this->get_filter_targets()
                );
                $form->addElement('submit', 'submit', Translation :: get('Ok', null, Utilities :: COMMON_LIBRARIES));

                $html[] = $form->toHtml();
                $html[] = '<div class="clear"></div></div>';
                $html[] = '<br />';
            }

            $html[] = parent :: as_html();
            $html[] = '</div>';
        }
        else
        {
            $html[] = parent :: as_html();
        }
        return implode("\n", $html);
    }

    public function get_filter_targets()
    {
        $course = $this->get_parent()->get_course_id();

        $targets = array();
        $targets[] = Translation :: get('Users', null, 'user');
        $targets[] = '----------';

        $users = \application\weblcms\course\DataManager :: retrieves(
            CourseUserRelation :: class_name(),
            new EqualityCondition(CourseUserRelation :: PROPERTY_COURSE_ID, $course)
        );

        while ($user = $users->next_result())
        {
            if ($user->get_user() == $this->get_parent()->get_user_id())
                continue;

            $targets['user|' . $user->get_user()] = UserDataManager :: get_instance()->retrieve_user($user->get_user())
                ->get_username();
        }

        $targets[] = '';
        $targets[] = Translation :: get('Groups', null, 'group');
        $targets[] = '----------';

        $groups = \application\weblcms\DataManager :: retrieves(
            CourseGroup :: class_name(),
            new EqualityCondition(CourseGroup :: PROPERTY_COURSE_CODE, $course)
        );

        while ($group = $groups->next_result())
        {
            $targets['group|' . $group->get_id()] = $group->get_name();
        }

        return $targets;
    }

    public function get_upcomming_events($amount)
    {
        $from_time = time();
        $publications = $this->get_publications();

        $events = array();
        foreach ($publications as $publication)
        {
            $object = $publication->get_content_object();

            if ($object->repeats())
            {
                $repeats = $object->get_repeats($from_time, 9999999999);

                foreach ($repeats as $repeat)
                {
                    $the_publication = clone $publication;
                    $the_publication->set_content_object($repeat);

                    $events[] = $the_publication;
                }
            }
            elseif ($from_time <= $object->get_start_date())
            {
                $events[] = $publication;
            }

            if (count($events) > $amount)
                return $events;
        }

        return $events;
    }

    public function render_upcomming_events()
    {
        $html = array();
        $html[] = '<b>' . Translation :: get('UpcomingEvents') . '</b><br /><br />';

        $amount_to_show = 5;
        $publications = $this->get_upcomming_events($amount_to_show);
        $count = count($publications);
        $total = $count < $amount_to_show ? $count : $amount_to_show;

        for ($i = 0; $i < $total; $i++)
        {
            $html[] = $this->render_small_publication($publications[$i]);
        }

        return implode("\n", $html);
    }

    public function render_small_publication($publication)
    {
        $feedback_url = $this->get_parent()->get_url(
            array(
                Tool :: PARAM_PUBLICATION_ID => $publication->get_id(),
                Tool :: PARAM_ACTION => 'view'
            ), array(), true
        );

        return '<a href="' . $feedback_url . '">' .
            date('d/m/y H:i:s -', $publication->get_content_object()->get_start_date()) . ' ' .
            $publication->get_content_object()->get_title() . '</a><br />';
    }

}
