<?php
namespace application\weblcms\tool\course_group;

use common\libraries\DelegateComponent;
use application\weblcms\tool\course_group\CourseGroup;

use application\weblcms\WeblcmsRights;
use application\weblcms\Tool;
use common\libraries\Application;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Display;
use common\libraries\NotAllowedException;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: course_group_editor.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.course_group.component
 */

class CourseGroupToolEditorComponent extends CourseGroupTool implements DelegateComponent
{

    public function run()
    {
        if (!$this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            throw new NotAllowedException();
        }

        $course_group_id = Request :: get(CourseGroupTool :: PARAM_COURSE_GROUP);

        $course_group = DataManager :: retrieve(
            CourseGroup :: class_name(),
            $course_group_id
        );

        BreadcrumbTrail :: get_instance()->add(
            new Breadcrumb(
                $this->get_url(), Translation :: get(
                    'EditorComponent', array(
                        'GROUPNAME' => $course_group->get_name()
                    )
                )
            )
        );

        $form = new CourseGroupForm(
            CourseGroupForm :: TYPE_EDIT, $course_group, $this->get_url(
                array(
                    Tool :: PARAM_ACTION => CourseGroupTool :: ACTION_EDIT_COURSE_GROUP,
                    CourseGroupTool :: PARAM_COURSE_GROUP => $course_group_id
                )
            )
        );
        if ($form->validate())
        {
            $succes = $form->update_course_group();

            if ($succes)
            {
                $message = Translation :: get(
                    'ObjectUpdated', array('OBJECT' => Translation :: get('CourseGroup')), Utilities :: COMMON_LIBRARIES
                );
            }
            else
            {
                $message = Translation :: get(
                    'ObjectNotUpdated', array('OBJECT' => Translation :: get('CourseGroup')),
                    Utilities :: COMMON_LIBRARIES
                ) . '<br />' . implode('<br />', $course_group->get_errors());
            }

            $this->redirect(
                $message, !$succes, array(
                    Tool :: PARAM_ACTION => CourseGroupTool :: ACTION_VIEW_GROUPS,
                    CourseGroupTool :: PARAM_COURSE_GROUP => $course_group->get_parent_id()
                )
            );
        }
        else
        {
            $this->display_header();
            $form->display();
            $this->display_footer();
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('weblcms_course_group_editor');
    }

    public function get_additional_parameters()
    {
        return array(CourseGroupTool :: PARAM_COURSE_GROUP);
    }

}
