<?php
namespace application\weblcms\tool\course_group;


use application\weblcms\tool\course_group\CourseGroup;
use common\libraries\DataClass;
use application\weblcms\WeblcmsRights;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\Path;
use common\libraries\BreadcrumbTrail;
use common\libraries\Filesystem;
use common\libraries\NotCondition;
use application\weblcms\Tool;
use PHPExcel;
use PHPExcel_Style_Alignment;
use PHPExcel_IOFactory;
use PHPExcel_Style_Font;
use PHPExcel_Style_Color;
use user\User;
use common\libraries\NotAllowedException;
use application\weblcms\course\DataManager as CourseDataManager;

require_once Path :: get_plugin_path() . 'phpexcel/PHPExcel.php';

/**
 * Class To export users and course groups from a course
 *
 * @author VUB - Original author
 * @author Sven Vanpoucke - Bugfixes, comments
 */
class CourseGroupToolExporterComponent extends CourseGroupTool
{

    private $current_tab;

    private $course_group;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            throw new NotAllowedException();
        }

        // $this->display_header();

        //$trail = BreadcrumbTrail :: get_instance();
        // $this->display_footer();

        $this->current_tab = CourseGroupToolSubscriptionsOverviewerComponent :: TAB_USERS;
        if (Request :: get(CourseGroupToolSubscriptionsOverviewerComponent :: PARAM_TAB))
        {
            $this->current_tab = Request :: get(CourseGroupToolSubscriptionsOverviewerComponent :: PARAM_TAB);
        }

        if (Request :: get(CourseGroupTool :: PARAM_COURSE_GROUP))
        {
            $course_group_id = Request :: get(CourseGroupTool :: PARAM_COURSE_GROUP);

            $course_group = DataManager :: retrieve(
                CourseGroup :: class_name(),
                $course_group_id
            );

            $this->course_group = $course_group;
        }

        $file_path = $this->render();
        $this->send_as_download($file_path);

        Filesystem :: remove($file_path);
    }

    /**
     * Sends the exported file as download
     *
     * @param $file_path string
     */
    public function send_as_download($file_path)
    {
        // Determines the filename that the user will see when they download the
        // file
        $course = $this->get_course();

        if ($this->course_group)
        {
            $filename = $course->get_title() . '_' . $this->course_group->get_name() . "_" . date('Ymd') . '.xlsx';
        }
        else
        {
            $filename = $course->get_title() . date('Ymd') . '.xlsx';
        }

        // Make safe name
        $filename = Filesystem :: create_safe_name($filename);
        $filename = preg_replace('/[\s]+/', '_', $filename);

        $type = 'application/vnd.openxmlformats';

        // Send file for download
        Filesystem :: file_send_for_download($file_path, true, $filename, $type);
    }

    /**
     * Returns the conditions when the course groups need to be exported
     *
     * @return AndCondition
     */
    public function get_condition()
    {
        $conditions = array();
        //$course_id = $this->get_course_id();

        if ($this->current_tab == CourseGroupToolSubscriptionsOverviewerComponent :: TAB_COURSE_GROUPS)
        {
            $conditions[] = new EqualityCondition(CourseGroup :: PROPERTY_COURSE_CODE, $this->get_course_id());
            $conditions[] = new NotCondition(new EqualityCondition(CourseGroup :: PROPERTY_PARENT_ID, 0));
        }
        if ($conditions)
        {
            return new AndCondition($conditions);
        }
        return null;
    }

    /**
     * Gets the export data for the users
     *
     * @param $data User
     * @return array
     */
    private function get_users_table($data)
    {
        $table = array();
        $table[0][User :: PROPERTY_OFFICIAL_CODE] = User :: PROPERTY_OFFICIAL_CODE;
        $table[0][User :: PROPERTY_USERNAME] = User :: PROPERTY_USERNAME;
        $table[0][User :: PROPERTY_LASTNAME] = User :: PROPERTY_LASTNAME;
        $table[0][User :: PROPERTY_FIRSTNAME] = User :: PROPERTY_FIRSTNAME;
        $table[0][User :: PROPERTY_EMAIL] = User :: PROPERTY_EMAIL;
        $table[0]['Course Groups'] = Translation :: get('CourseGroups');
        $table[0]['Type'] = Translation :: get('SubscriptionType');

        $individual_subscriptions = \application\weblcms\course\DataManager ::
            retrieve_users_directly_subscribed_to_course_by_id($this->get_course()->get_id());

        while($individual_subscription = $individual_subscriptions->next_result())
        {
            $individual_subscriptions_array[$individual_subscription['id']] = true;
        }

        $index = 0;
        while ($block_data = $data->next_result())
        {
            if(!$block_data instanceof User)
            {
                $block_data = DataClass :: factory(User :: class_name(), $block_data);
            }

            $index++;
            // get the list of course_groups the user is subscribed
            $course_groups = DataManager :: retrieve_course_groups_from_user(
                $block_data->get_id(), $this->get_course_id()
            );
            $course_groups_subscribed = array();
            while ($course_group = $course_groups->next_result())
            {
                $course_groups_subscribed[] = $course_group->get_name();
            }
            $course_groups_string = null;
            foreach ($course_groups_subscribed as $item)
            {
                if ($course_groups_string)
                {
                    $course_groups_string = $course_groups_string . ', ' . $item;
                }
                else
                    $course_groups_string = $item;
            }

            //check if user is subscribed individually or via platform group
            if(isset($individual_subscriptions_array[$block_data->get_id()]))
            {
                $type = Translation :: get('SubscribedDirectly');
            }
            else
            {
                $type = Translation :: get('SubscribedGroup');
            }

            $table[$index][User :: PROPERTY_OFFICIAL_CODE] = $block_data->get_official_code();
            $table[$index][User :: PROPERTY_USERNAME] = $block_data->get_username();
            $table[$index][User :: PROPERTY_LASTNAME] = $block_data->get_lastname();
            $table[$index][User :: PROPERTY_FIRSTNAME] = $block_data->get_firstname();
            $table[$index][User :: PROPERTY_EMAIL] = $block_data->get_email();
            $table[$index]['Course Groups'] = $course_groups_string;
            $table[$index]['Type'] = $type;
        }
        return $table;
    }

    /**
     * Retrieves the users when exporting users
     *
     * @param $worksheet Worksheet
     */
    private function get_users_tab($worksheet)
    {
        $data = CourseDataManager :: get_instance()->retrieve_all_course_users(
            $this->get_course_id(), $this->get_condition(), null, null, null
        );
        $table = $this->get_users_table($data);
        $title = 'Subscription List';
        $rowcount = 0;
        $this->render_table($worksheet, $title, $table, $rowcount);
    }

    /**
     * Retrieves the course groups when exporting all the course groups
     *
     * @param $worksheet <type>
     */
    private function get_course_groups_tab($worksheet)
    {
        $course_groups = DataManager :: retrieves(
            CourseGroup :: class_name(), $this->get_condition()
        );

        $rowcount = 0;
        while ($course_group = $course_groups->next_result())
        {
            $course_group_users = DataManager :: retrieve_course_group_users(
                $course_group->get_id(), null, null, null, null
            );

            $users_table = $this->get_users_table($course_group_users);
            $title = $course_group->get_name();
            $rowcount = $this->render_table($worksheet, $title, $users_table, $rowcount);
        }
    }

    /**
     * Renders the worksheet
     *
     * @return string - the path to the rendered worksheet
     */
    public function render()
    {

        $excel = new PHPExcel();
        $worksheet = $excel->getSheet(0)->setTitle('Export');

        $this->get_data($worksheet);

        $objWriter = PHPExcel_IOFactory :: createWriter($excel, 'Excel2007');

        $temp_dir = Path :: get(SYS_TEMP_PATH) . 'excel/';

        if (!is_dir($temp_dir))
        {
            mkdir($temp_dir, 0777, true);
        }

        $filename = 'export_course_group_' . $this->get_course_id();

        if ($this->course_group)
        {
            $filename .= '_' . $this->course_group->get_id();
        }

        $filename .= '_' . time();

        $file_path = $temp_dir . $filename;

        $objWriter->save($file_path);
        return $file_path;
    }

    /**
     * Gets the data for the worksheet
     *
     * @param $worksheet Worksheet
     */
    public function get_data($worksheet)
    {
        if ($this->course_group)
        {
            // subscription overview for one course group
            $this->get_course_group_subscription($worksheet);
        }
        else
        {
            switch ($this->current_tab)
            {
                case CourseGroupToolSubscriptionsOverviewerComponent :: TAB_USERS :
                    $this->get_users_tab($worksheet);
                    break;
                case CourseGroupToolSubscriptionsOverviewerComponent :: TAB_COURSE_GROUPS :
                    $this->get_course_groups_tab($worksheet);
                    break;
            }
        }
    }

    /**
     * Gets the course group subscriptions for the given course group
     *
     * @param $worksheet <type>
     */
    private function get_course_group_subscription($worksheet)
    {
        // $data = WeblcmsDataManager ::
        // get_instance()->retrieve_all_course_users($this->get_course_id(),
        // $this->get_condition(), null, null, null);
        $data = DataManager :: retrieve_course_group_users(
            $this->course_group->get_id(), null, null, null, null
        );

        $table = $this->get_users_table($data);
        $title = Translation :: get('SubscriptionList');
        $rowcount = 0;
        $this->render_table($worksheet, $title, $table, $rowcount);
    }

    /**
     * Renders the data
     *
     * @param $worksheet Worksheet
     * @param $title String
     * @param $table String[]
     * @param $block_row Integer
     * @return Integer
     */
    private function render_table($worksheet, $title, $table, $block_row)
    {

        $block_row++;
        $block_row++;

        $worksheet->setCellValueByColumnAndRow($column, $block_row, $title);
        // $this->wrap_text($worksheet, $column, $block_row);
        $worksheet->mergeCells('A' . $block_row . ':G' . $block_row);
        $worksheet->getStyleByColumnAndRow($column, $block_row)->getAlignment()->setHorizontal(
            PHPExcel_Style_Alignment :: HORIZONTAL_CENTER
        );
        $worksheet->getStyleByColumnAndRow($column, $block_row)->getFont()->setBold(true);

        $block_row++;

        $column = 0;
        $column1 = 1;
        $column2 = 2;
        $color = PHPExcel_Style_Color :: COLOR_BLUE;

        $styleArray = array(
            'font' => array(
                'underline' => PHPExcel_Style_Font :: UNDERLINE_SINGLE,
                'color' => array('argb' => $color)
            )
        );

        //moved this block outside the loop, since it has nothing to do with the data
        {
            $worksheet->getColumnDimension('A')->setWidth(20);
            $worksheet->getColumnDimension('B')->setWidth(30);
            $worksheet->getColumnDimension('C')->setWidth(30);
            $worksheet->getColumnDimension('D')->setWidth(30);
            $worksheet->getColumnDimension('E')->setWidth(50);
            $worksheet->getColumnDimension('F')->setWidth(50);
            $worksheet->getColumnDimension('G')->setWidth(50);

            $worksheet->getStyleByColumnAndRow($column, $block_row + 1)->applyFromArray($styleArray);
            $worksheet->getStyleByColumnAndRow($column1, $block_row + 1)->applyFromArray($styleArray);
            $worksheet->getStyleByColumnAndRow($column2, $block_row + 1)->applyFromArray($styleArray);
            $worksheet->getStyleByColumnAndRow($column2 + 1, $block_row + 1)->applyFromArray($styleArray);
            $worksheet->getStyleByColumnAndRow($column2 + 2, $block_row + 1)->applyFromArray($styleArray);
            $worksheet->getStyleByColumnAndRow($column2 + 3, $block_row + 1)->applyFromArray($styleArray);
            $worksheet->getStyleByColumnAndRow($column2 + 4, $block_row + 1)->applyFromArray($styleArray);
        }

        //$i = 0;

        foreach ($table as $entry)
        {
            //$i++;
            $block_row++;
            $worksheet->setCellValueByColumnAndRow($column, $block_row, $entry[User :: PROPERTY_OFFICIAL_CODE]);
            $worksheet->setCellValueByColumnAndRow($column1, $block_row, $entry[User :: PROPERTY_USERNAME]);
            $worksheet->setCellValueByColumnAndRow($column2, $block_row, $entry[User :: PROPERTY_LASTNAME]);
            $worksheet->setCellValueByColumnAndRow($column2 + 1, $block_row, $entry[User :: PROPERTY_FIRSTNAME]);
            $worksheet->setCellValueByColumnAndRow($column2 + 2, $block_row, $entry[User :: PROPERTY_EMAIL]);
            $worksheet->setCellValueByColumnAndRow($column2 + 3, $block_row, $entry['Course Groups']);
            $worksheet->setCellValueByColumnAndRow($column2 + 4, $block_row, $entry['Type']);

            //if ($i == 1)

        }
        // $block_row++;
        return $block_row;
    }
}
