<?php

namespace application\weblcms\tool\course_group;

use application\weblcms\tool\course_group\CourseGroup;
use application\weblcms\tool\course_group\CourseGroupTableCellRenderer;
use application\weblcms\tool\course_group\CourseGroupTool;
use application\weblcms\tool\course_group\DefaultCourseGroupTableColumnModel;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use common\libraries\Theme;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * $Id: default_course_group_table_cell_renderer.class.php 216 2009-11-13
 * 14:08:06Z kariboe $
 *
 * @package
 *          application.lib.weblcms.tool.course_group.component.course_group_table
 */
class DefaultCourseGroupTableCellRenderer implements CourseGroupTableCellRenderer
{

    private $course_group_tool;

    /**
     * Constructor
     */
    public function __construct($course_group_tool)
    {
        $this->course_group_tool = $course_group_tool;
    }

    /**
     * Renders a table cell
     *
     * @param $column CourseGroupTableColumnModel
     *            The column which should be rendered
     * @param $course_group CourseGroup
     *            The User to render
     * @return string A HTML representation of the rendered table cell
     */
    public function render_cell($column, $course_group)
    {
        if ($column === DefaultCourseGroupTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($course_group);
        }
        if ($column === DefaultCourseGroupTableColumnModel :: get_number_of_members_column())
        {
            $members = $course_group->get_members();
            if (!is_null($members))
            {
                return sizeof($members);
            }
            else
            {
                return '0';
            }
        }
        if ($column === DefaultCourseGroupTableColumnModel :: get_max_number_of_members_column())
        {
            return $course_group->get_max_number_of_members();
        }
        if ($property = $column->get_course_group_property())
        {
            switch ($property)
            {
                case CourseGroup :: PROPERTY_ID :
                    return $course_group->get_id();
                case CourseGroup :: PROPERTY_NAME :
                    if (
                        $this->course_group_tool->is_allowed(WeblcmsRights :: EDIT_RIGHT) ||
                        $course_group->is_member($this->course_group_tool->get_user())
                    )
                    {
                        $url = $this->course_group_tool->get_url(
                            array(
                                CourseGroupTool :: PARAM_ACTION => CourseGroupTool :: ACTION_UNSUBSCRIBE,
                                WeblcmsManager :: PARAM_COURSE_GROUP => $course_group->get_id()
                            )
                        );
                        return '<a href="' . $url . '">' . $course_group->get_name() . '</a>';
                    }
                    else
                    {
                        return $course_group->get_name();
                    }
                case CourseGroup :: PROPERTY_DESCRIPTION :
                    return strip_tags($course_group->get_description());
                case CourseGroup :: PROPERTY_MAX_NUMBER_OF_MEMBERS :
                    // // case DefaultCourseGroupTableColumnModel ::
                    // get_max_number_of_members_column():
                    return $course_group->get_max_number_of_members();

                case CourseGroup :: PROPERTY_SELF_REG :
                    return $course_group->is_self_registration_allowed() ? Translation :: get(
                        'ConfirmYes', null, Utilities :: COMMON_LIBRARIES
                    ) : Translation :: get('ConfirmNo', null, Utilities :: COMMON_LIBRARIES);
                case CourseGroup :: PROPERTY_SELF_UNREG :
                    return $course_group->is_self_unregistration_allowed() ? Translation :: get(
                        'ConfirmYes', null, Utilities :: COMMON_LIBRARIES
                    ) : Translation :: get('ConfirmNo', null, Utilities :: COMMON_LIBRARIES);
                case CourseGroup :: PROPERTY_DOCUMENT_CATEGORY_ID :
                    return $course_group->get_document_category_id();
                case CourseGroup :: PROPERTY_FORUM_CATEGORY_ID :
                    return $course_group->get_forum_category_id();
                case CourseGroup :: PROPERTY_RANDOM_REG :
                    return $course_group->is_random_registration_done() ? Translation :: get(
                        'ConfirmYes', null, Utilities :: COMMON_LIBRARIES
                    ) : Translation :: get('ConfirmNo', null, Utilities :: COMMON_LIBRARIES);
            }
        }
        return '&nbsp;';
    }

    /**
     * Gets the action links to display
     *
     * @param $course_group CourseGroup
     *            The course_group for which the action links should be
     *            returned
     * @return string A HTML representation of the action links
     */
    private function get_modification_links($course_group)
    {
        $toolbar = new Toolbar();
        $parameters = array();
        $parameters[WeblcmsManager :: PARAM_COURSE_GROUP] = $course_group->get_id();
        //$details_url = $this->course_group_tool->get_url($parameters);
        // Default functionity achieved by clicking the course_group name, why
        // add it as an icon ?
        // $toolbar_data[] = array ('href' => $details_url, 'label' =>
        // Translation :: get('Details'), 'img' => Theme ::
        // get_common_image_path().'description.png');
        if ($this->course_group_tool->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $parameters = array();
            $parameters[CourseGroupTool :: PARAM_COURSE_GROUP] = $course_group->get_id();
            $parameters[CourseGroupTool :: PARAM_COURSE_GROUP_ACTION] = CourseGroupTool :: ACTION_EDIT_COURSE_GROUP;
            $edit_url = $this->course_group_tool->get_url($parameters);
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_edit.png', $edit_url, ToolbarItem :: DISPLAY_ICON
                )
            );

            $parameters = array();
            $parameters[WeblcmsManager :: PARAM_COURSE_GROUP] = $course_group->get_id();
            $parameters[CourseGroupTool :: PARAM_COURSE_GROUP_ACTION] = CourseGroupTool :: ACTION_DELETE_COURSE_GROUP;
            $delete_url = $this->course_group_tool->get_url($parameters);

            $confirm_messages = array();
            $confirm_messages[] = Translation :: get('DeleteConfirm', array('NAME' => $course_group->geT_name())); // TODO :: Better
            if($course_group->has_children()) {
                $confirm_messages[] = Translation :: get('DeleteConfirmChildren');
            }
            if($course_group->get_document_category_id()) {
                $confirm_messages[] = Translation :: get('DeleteConfirmDocument');
            }
            if($course_group->get_forum_category_id()) {
                $confirm_messages[] = Translation :: get('DeleteConfirmForum');
            }
            $confirm_message = implode('\\n\\n', $confirm_messages);

            $toolbar->add_item(
                new NoEscapeToolbarItem(
                    Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() .
                    'action_delete.png', $delete_url, ToolbarItem :: DISPLAY_ICON, true, null, null, $confirm_message
                )
            );
        }

        $user = $this->course_group_tool->get_user();

        if (!$this->course_group_tool->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if (
                $course_group->is_self_registration_allowed() && (
                    $course_group->count_members() < $course_group->get_max_number_of_members() ||
                    $course_group->get_max_number_of_members() == 0
                )
            )
            {
                if (
                    ! $course_group->is_member($user) &&
                    DataManager :: more_subscriptions_allowed_for_user_in_group(
                        $course_group->get_parent_id(), $user->get_id()
                    )
                )
                {
                    $parameters = array();
                    $parameters[WeblcmsManager :: PARAM_COURSE_GROUP] = $course_group->get_id();
                    $parameters[CourseGroupTool :: PARAM_COURSE_GROUP_ACTION] =
                        CourseGroupTool :: ACTION_USER_SELF_SUBSCRIBE;
                    $subscribe_url = $this->course_group_tool->get_url($parameters);
                    $toolbar->add_item(
                        new ToolbarItem(
                            Translation :: get('Subscribe'), Theme :: get_common_image_path() .
                            'action_subscribe.png', $subscribe_url, ToolbarItem :: DISPLAY_ICON
                        )
                    );
                }
            }
        }
        else
        {
            $parameters = array();
            $parameters[WeblcmsManager :: PARAM_COURSE_GROUP] = $course_group->get_id();
            $parameters[CourseGroupTool :: PARAM_COURSE_GROUP_ACTION] = CourseGroupTool :: ACTION_MANAGE_SUBSCRIPTIONS;
            $subscribe_url = $this->course_group_tool->get_url($parameters);
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Subscribe'), Theme :: get_common_image_path() .
                    'action_subscribe.png', $subscribe_url, ToolbarItem :: DISPLAY_ICON
                )
            );
        }

        if (!$this->course_group_tool->is_allowed(WeblcmsRights :: EDIT_RIGHT) &&
            $course_group->is_self_unregistration_allowed() && $course_group->is_member($user)
        )
        {
            $parameters = array();
            $parameters[WeblcmsManager :: PARAM_COURSE_GROUP] = $course_group->get_id();
            $parameters[CourseGroupTool :: PARAM_COURSE_GROUP_ACTION] = CourseGroupTool :: ACTION_USER_SELF_UNSUBSCRIBE;
            $unsubscribe_url = $this->course_group_tool->get_url($parameters);
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Unsubscribe'), Theme :: get_common_image_path() .
                    'action_unsubscribe.png', $unsubscribe_url, ToolbarItem :: DISPLAY_ICON
                )
            );
        }
        /*
         * else { $parameters = array (); $parameters[WeblcmsManager ::
         * PARAM_COURSE_GROUP] = $course_group->get_id();
         * $parameters[CourseGroupTool::PARAM_COURSE_GROUP_ACTION] =
         * CourseGroupTool::ACTION_UNSUBSCRIBE; $unsubscribe_url =
         * $this->course_group_tool->get_url($parameters); $toolbar_data[] =
         * array ('href' => $unsubscribe_url, 'label' => Translation ::
         * get('UnsubscribeUsers'), 'img' => Theme ::
         * get_common_image_path().'action_unsubscribe.png'); }
         */

        return $toolbar->as_html();
    }

}

/**
 * The ToolbarItem class' as_html() method takes the confirmation message
 * through addslashes() and htmlentities(). This means that we cannot use
 * newlines and special characters (ë, ...) in the confirmation labels.
 *
 * Because it is unclear if anything depends on the fact that those functions
 * are called, I don't want to make the fix in ToolbarItem. So I'll use this 
 * specialized class, which overrides that method to remove those function
 * calls, instead.
 *
 * @author Bram Thielemans
 */
class NoEscapeToolBarItem extends ToolbarItem
{
    public function as_html()
    {
        // Note: replaced private instance variable accesses with getter/setter
        // method calls.
        
        $label = ($this->get_label() ? htmlspecialchars($this->get_label()) : null);
        if (! $this->get_display())
        {
            $this->set_display(self :: DISPLAY_ICON);
        }
        $display_label = ($this->get_display() & self :: DISPLAY_LABEL) == self :: DISPLAY_LABEL && ! empty($label);

        $button = '';
        if (($this->get_display() & self :: DISPLAY_ICON) == self :: DISPLAY_ICON && $this->get_image())
        {
            $button .= '<img src="' . htmlentities($this->get_image()) . '" alt="' . $label . '" title="' . $label . '"' . ($display_label ? ' class="labeled"' : '') . '/>';
        }

        /*if ($this->get_class())
        {
            $class = ' class="' . $this->get_class() . '"';
        }
        else
        {
            $class = '';
        }*/
        $class = ''; // no public getter for class ...

        if ($display_label)
        {
            if ($this->get_href())
            {
                $button .= '<span>' . $label . '</span>';
            }
            else
            {
                $button .= '<span' . $class . '>' . $label . '</span>';
            }
        }

        if ($this->get_href())
        {
            if ($this->get_confirmation() === true)
            {
                $this->set_confirmation(Translation :: get($this->get_confirm_message()));
            }

            if ($this->get_target())
            {
                $target = ' target="' . $this->get_target() . '"';
                $button = '<a' . $class . $target . ' href="' . htmlentities($this->get_href()) . '" title="' . $label . '"' . ($this->needs_confirmation() ? ' onclick="return confirm(\'' . $this->get_confirmation() . '\');"' : '') . '>' . $button . '</a>';
            }
            else
            {
                $button = '<a' . $class . ' href="' . htmlentities($this->get_href()) . '" title="' . $label . '"' . ($this->needs_confirmation() ? ' onclick="return confirm(\'' . $this->get_confirm_message() . '\');"' : '') . '>' . $button . '</a>';
            }
        }

        return $button;
    }
}