<?php
namespace application\weblcms\tool\course_sections;


use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use application\weblcms\DefaultCourseSectionsTableCellRenderer;
use application\weblcms\CourseSection;

/**
 * $Id: course_sections_browser_table_cell_renderer.class.php 216 2009-11-13
 * 14:08:06Z kariboe $
 *
 * @package
 *          application.lib.weblcms.tool.course_sections.component.course_sections_browser
 */
/**
 * Cell rendere for the learning object browser table
 */
class CourseSectionsBrowserTableCellRenderer extends DefaultCourseSectionsTableCellRenderer
{
    /**
     * The repository browser component
     */
    private $browser;
    private $count;

    /**
     * Constructor
     *
     * @param $browser WeblcmsBrowserComponent
     */
    public function __construct($browser)
    {
        parent :: __construct();

        $this->browser = $browser;
        $this->count = \application\weblcms\DataManager :: count(
            CourseSection :: class_name(), $browser->get_condition()
        );
    }

    // Inherited
    public function render_cell($column, $course_section)
    {
        if ($column === CourseSectionsBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($course_section);
        }

        // Add special features here
        /*switch ($column->get_name())
        {
            // Exceptions that need post-processing go here ...
        }*/
        return parent :: render_cell($column, $course_section);
    }

    /**
     * Gets the action links to display
     *
     * @param $course Course
     *            The course for which the
     *            action links should be returned
     * @return string A HTML representation of the action links
     */
    private function get_modification_links($course_section)
    {
        $toolbar = new Toolbar();

        $filter = array(
            CourseSection :: TYPE_DISABLED, CourseSection :: TYPE_TOOL, CourseSection :: TYPE_LINK,
            CourseSection :: TYPE_ADMIN
        );

        if (!in_array($course_section->get_type(), $filter))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_edit.png', $this->browser->get_url(
                        array(
                            CourseSectionsTool :: PARAM_ACTION => CourseSectionsTool :: ACTION_UPDATE_COURSE_SECTION,
                            CourseSectionsTool :: PARAM_COURSE_SECTION_ID => $course_section->get_id()
                        )
                    ), ToolbarItem :: DISPLAY_ICON
                )
            );

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_delete.png', $this->browser->get_url(
                        array(
                            CourseSectionsTool :: PARAM_ACTION => CourseSectionsTool :: ACTION_REMOVE_COURSE_SECTION,
                            CourseSectionsTool :: PARAM_COURSE_SECTION_ID => $course_section->get_id()
                        )
                    ), ToolbarItem :: DISPLAY_ICON, true
                )
            );

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('SelectTools'),
                    Theme :: get_common_image_path() . 'action_move.png', $this->browser->get_url(
                        array(
                            CourseSectionsTool :: PARAM_ACTION =>
                            CourseSectionsTool :: ACTION_SELECT_TOOLS_COURSE_SECTION,
                            CourseSectionsTool :: PARAM_COURSE_SECTION_ID => $course_section->get_id()
                        )
                    ), ToolbarItem :: DISPLAY_ICON
                )
            );

        }
        else
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('EditNA', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_edit_na.png', null, ToolbarItem :: DISPLAY_ICON
                )
            );
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('DeleteNA', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_delete_na.png', null, ToolbarItem :: DISPLAY_ICON
                )
            );
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('SelectToolsNA'),
                    Theme :: get_common_image_path() . 'action_move_na.png', null, ToolbarItem :: DISPLAY_ICON
                )
            );
        }

        $order = $course_section->get_display_order();

        if ($order == 1)
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('MoveUpNA', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_up_na.png', null, ToolbarItem :: DISPLAY_ICON
                )
            );
        }
        else
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('MoveUp', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_up.png', $this->browser->get_url(
                        array(
                            CourseSectionsTool :: PARAM_ACTION => CourseSectionsTool :: ACTION_MOVE_COURSE_SECTION,
                            CourseSectionsTool :: PARAM_COURSE_SECTION_ID => $course_section->get_id(),
                            CourseSectionsTool :: PARAM_DIRECTION => -1
                        )
                    ), ToolbarItem :: DISPLAY_ICON
                )
            );
        }

        if ($order == $this->count)
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('MoveDownNA', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_down_na.png', null, ToolbarItem :: DISPLAY_ICON
                )
            );
        }
        else
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('MoveDown', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_down.png', $this->browser->get_url(
                        array(
                            CourseSectionsTool :: PARAM_ACTION => CourseSectionsTool :: ACTION_MOVE_COURSE_SECTION,
                            CourseSectionsTool :: PARAM_COURSE_SECTION_ID => $course_section->get_id(),
                            CourseSectionsTool :: PARAM_DIRECTION => 1
                        )
                    ), ToolbarItem :: DISPLAY_ICON
                )
            );
        }

        if ($course_section->get_type() != CourseSection :: TYPE_ADMIN)
        {
            if ($course_section->is_visible())
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('ChangeVisible'),
                        Theme :: get_common_image_path() . 'action_visible.png', $this->browser->get_url(
                            array(
                                CourseSectionsTool :: PARAM_ACTION =>
                                CourseSectionsTool :: ACTION_CHANGE_COURSE_SECTION_VISIBILITY,
                                CourseSectionsTool :: PARAM_COURSE_SECTION_ID => $course_section->get_id()
                            )
                        ), ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
            else
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('ChangeVisible'),
                        Theme :: get_common_image_path() . 'action_invisible.png', $this->browser->get_url(
                            array(
                                CourseSectionsTool :: PARAM_ACTION =>
                                CourseSectionsTool :: ACTION_CHANGE_COURSE_SECTION_VISIBILITY,
                                CourseSectionsTool :: PARAM_COURSE_SECTION_ID => $course_section->get_id()
                            )
                        ), ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
        }

        return $toolbar->as_html();

    }
}
