<?php
namespace application\weblcms\tool\course_truncater;

use common\libraries\DelegateComponent;

use common\libraries\Display;
use common\libraries\InCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\AndCondition;

use application\weblcms\ContentObjectPublicationCategory;
use application\weblcms\ContentObjectPublication;

use application\weblcms\CourseSection;

use application\weblcms\WeblcmsManager;

/*
 * Component for emptying the course publication,publication categories and sections
 * 
 * @author Maarten Volckaert - Hogeschool Gent
 * @author Mattias De Pauw - Hogeschool Gent
 */
class CourseTruncaterToolBrowserComponent extends CourseTruncaterTool implements DelegateComponent
{
    public $course_emptier_form;

    /**
     * checks whether the user has the rights if so create empty form if
     * validate delete selected publications section.
     */
    public function run()
    {
        if (!$this->get_course()->is_course_admin($this->get_parent()->get_user()))
        {
            throw new \common\libraries\NotAllowedException();
        }

        $sections_founded = false;
        $condition = new EqualityCondition(CourseSection :: PROPERTY_COURSE_ID, $this->get_course_id());

        $course_sections = \application\weblcms\DataManager :: retrieves(
            CourseSection :: class_name(), $condition
        );

        $common_sections = array(
            CourseSection :: TYPE_TOOL, CourseSection :: TYPE_DISABLED, CourseSection :: TYPE_LINK,
            CourseSection :: TYPE_ADMIN
        );

        while ($course_section = $course_sections->next_result())
        {
            if (!in_array($course_section->get_type(), $common_sections))
            {
                $sections_founded = true;
            }
        }
        $condition = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $this->get_course_id());
        if (\application\weblcms\DataManager :: count_content_object_publications($condition) == 0 && !$sections_founded)
        {
            throw new \Exception(Translation :: get('NoPublications'));
        }

        $this->course_emptier_form = new CourseTruncaterForm($this);
        $this->course_emptier_form->buildForm();

        if ($this->course_emptier_form->validate())
        {
            $values = $this->course_emptier_form->exportValues();

            if (
                isset($values['publications']) || isset($values["course_sections"])
                || $values['content_object_categories'] == 1
            )
            {
                //always first delete the publications before deleting the categories

                $succes_publications = $this->empty_publications($values);
                $succes_sections = $this->empy_sections($values);
                $succes_categories = $this->empty_categories($values);

                if ($succes_publications && $succes_sections && $succes_categories)
                {
                    $this->redirect(
                        Translation :: get('AllSelectedObjectsRemoved'), false, array(
                            WeblcmsManager :: PARAM_ACTION => WeblcmsManager :: ACTION_VIEW_WEBLCMS_HOME
                        )
                    );
                }
                else
                {
                    throw new \Exception(Translation :: get('NotAllSelectedObjectsRemoved'));
                }
            }
            else
            {
                $this->display_header();
                Display :: error_message(Translation :: get('SelectAItem'));
                $this->course_emptier_form->display();
                $this->display_footer();
            }
        }
        else
        {
            $this->display_header();
            $this->course_emptier_form->display();
            $this->display_footer();
        }
    }

    /**
     * delete the selected publications
     *
     * @param type $values
     * @return succes
     */
    public function empty_publications($values)
    {
        $publication_ids = array_keys($values['publications']);

        $succes = true;

        foreach ($publication_ids as $id)
        {
            $publication = \application\weblcms\DataManager :: retrieve(
                ContentObjectPublication :: class_name(),
                $id
            );

            if (!$publication->delete())
            {
                $succes = false;
            }
        }
        return $succes;
    }

    /**
     * delete the selected sections
     *
     * @param type $values
     * @return succes
     */
    public function empy_sections($values)
    {
        $succes = true;

        $course_section_ids = array_keys($values['course_sections']);
        $condition = new InCondition(CourseSection :: PROPERTY_ID, $course_section_ids);

        $course_sections = \application\weblcms\DataManager :: retrieves(
            CourseSection :: class_name(), $condition
        );

        while ($course_section = $course_sections->next_result())
        {
            if (!$course_section->delete())
            {
                $succes = false;
            }
        }
        return $succes;
    }

    /**
     * delete the selected categories
     *
     * @param type $values
     * @return succes
     */
    public function empty_categories($values)
    {
        $succes = true;

        if ($values['content_object_categories'] == 1)
        {
            $condition = new EqualityCondition(
                ContentObjectPublicationCategory :: PROPERTY_COURSE, $this->get_course_id()
            );

            $categories = \application\weblcms\DataManager :: retrieves(
                ContentObjectPublicationCategory :: class_name(),
                $condition
            );

            while ($category = $categories->next_result())
            {
                if (!$category->get_allow_change() || !$this->allowed_to_delete_category($category->get_id()))
                {
                    continue;
                }

                if (!$category->delete())
                {
                    $succes = false;
                }
            }
        }

        return $succes;
    }

    /**
     * checks whether a category can be deleted
     *
     * @param int $category_id
     * @return boolean
     */
    private function allowed_to_delete_category($category_id)
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $this->get_course_id());
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_CATEGORY_ID, $category_id);
        $condition = new AndCondition($conditions);
        $count = \application\weblcms\DataManager :: count_content_object_publications($condition);

        if ($count > 0)
        {
            return false;
        }

        return !$this->have_subcategories_publications($category_id);
    }

    /**
     * checks if a category has subcategory's
     *
     * @param int $category_id
     *
     * @return boolean
     */
    private function have_subcategories_publications($category_id)
    {
        $condition = new EqualityCondition(ContentObjectPublicationCategory :: PROPERTY_PARENT, $category_id);

        $subcategries = \application\weblcms\DataManager :: retrieves(
            ContentObjectPublicationCategory :: class_name(),
            $condition
        );

        while ($cat = $subcategries->next_result())
        {
            $conditions = array();
            $conditions[] = new EqualityCondition(
                ContentObjectPublication :: PROPERTY_COURSE_ID, $this->get_course_id()
            );
            $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_CATEGORY_ID, $cat->get_id());
            $condition = new AndCondition($conditions);
            $count = \application\weblcms\DataManager :: count_content_object_publications($condition);

            if ($count > 0 || $this->have_subcategories_publications($cat->get_id()))
            {
                return true;
            }
        }
        return false;
    }
}
