<?php
namespace application\weblcms\tool\course_truncater;


use application\weblcms\ContentObjectPublication;
use common\libraries\DataClassRetrievesParameters;
use application\weblcms\CourseSection;

use application\weblcms\Tool;

use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\FormValidator;

use common\libraries\ObjectTableOrder;
use common\libraries\EqualityCondition;

/**
 * This form can let the user select the sections and publications of a course
 * for deleting them.
 *
 * @author Mattias De Pauw - Hogeschool Gent
 * @author Maarten Volckaert - Hogeschool Gent
 */
class CourseTruncaterForm extends FormValidator
{

    private $parent;

    /**
     * Constructor
     *
     * @param type $parent
     */
    public function __construct($parent)
    {
        parent :: __construct($parent);
        $this->parent = $parent;
    }

    /**
     *Builds the form
     */
    public function buildForm()
    {
        $defaults = array();

        $condition = new EqualityCondition(
            ContentObjectPublication :: PROPERTY_COURSE_ID, $this->parent->get_course_id()
        );

        $publications_set = \application\weblcms\DataManager :: retrieves(
            ContentObjectPublication :: class_name(),
            new DataClassRetrievesParameters(
                $condition, null, null,
                array(new ObjectTableOrder(ContentObjectPublication :: PROPERTY_DISPLAY_ORDER_INDEX, SORT_DESC))
            )
        );

        while ($publication = $publications_set->next_result())
        {
            $publications[$publication->get_tool()][] = $publication;
        }

        $this->addElement('html', '<h3>' . Translation :: get('Publications') . '</h3>');

        foreach ($publications as $tool => $tool_publications)
        {
            foreach ($tool_publications as $index => $publication)
            {
                $label =
                    $index == 0 ? Translation :: get('TypeName', null, Tool :: get_tool_type_namespace($tool)) : '';
                $content_object = $publication->get_content_object();
                $id = 'publications[' . $publication->get_id() . ']';
                $this->addElement('checkbox', $id, $label, $content_object->get_title());
                $defaults[$id] = true;
            }
        }

        $this->addFormRule(array('PublicationSelectionMaintenanceWizardPage', 'count_selected_publications'));

        $this->addElement('html', '<h3>' . Translation :: get('CourseSections') . '</h3>');

        $condition = new EqualityCondition(CourseSection :: PROPERTY_COURSE_ID, $this->parent->get_course_id());

        $course_sections = \application\weblcms\DataManager :: retrieves(
            CourseSection :: class_name(), $condition
        );

        $common_sections = array(
            CourseSection :: TYPE_TOOL, CourseSection :: TYPE_DISABLED, CourseSection :: TYPE_LINK,
            CourseSection :: TYPE_ADMIN
        );

        while ($course_section = $course_sections->next_result())
        {
            if (!in_array($course_section->get_type(), $common_sections))
            {
                $id = 'course_sections[' . $course_section->get_id() . ']';
                $this->addElement('checkbox', $id, $course_section->get_name());
                $defaults[$id] = true;
            }
        }

        $this->addElement('html', '<h3>' . Translation :: get('Other') . '</h3>');
        $this->addElement('checkbox', 'content_object_categories', Translation :: get('PublicationCategories'));
        $defaults['content_object_categories'] = true;

        $this->setDefaults($defaults);

        $this->addElement('html', '<h3>' . Translation :: get('EmptyThisCourseInformation') . '</h3>');
        $this->addElement('checkbox', 'confirm', Translation :: get('Confirm', null, Utilities :: COMMON_LIBRARIES));
        $this->addRule(
            'confirm', Translation :: get('ThisFieldIsRequired', null, Utilities :: COMMON_LIBRARIES), 'required'
        );
        $prevnext = array();
        $prevnext[] = $this->createElement(
            'submit', $this->parent->get_url($trail), Translation :: get('Submit', null, Utilities :: COMMON_LIBRARIES)
        );
        $this->addGroup($prevnext, 'buttons', '', '&nbsp;', false);
        $this->updateAttributes(array('action' => $this->parent->get_url()));
    }
}
