<?php

namespace application\weblcms\tool\forum;

use common\libraries\DelegateComponent;
use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use common\libraries\DataClassRetrievesParameters;

use repository\content_object\forum\ForumComplexDisplay;

use repository\content_object\forum\Forum;
use repository\content_object\forum\DataManager as ForumDataManager;
use repository\content_object\introduction\Introduction;
use application\weblcms\ContentObjectPublicationCategory;
use application\weblcms\ContentObjectPublication;

use application\weblcms\DataManager as WeblcmsDataManager;
use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsManager;
use application\weblcms\Tool;
use common\libraries\DatetimeUtilities;
use repository\ContentObject;
use common\libraries\ActionBarRenderer;
use HTML_Table;
use common\libraries\SubselectCondition;

use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use user\User;
use user\UserDataManager;
use common\libraries\WebApplication;
use common\libraries\Theme;
use common\libraries\InCondition;
use common\libraries\ObjectTableOrder;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\ComplexDisplay;
use application\gradebook\EvaluationManager;
use application\weblcms\CourseSettingsController;
use application\weblcms\CourseSettingsConnector;
use repository\content_object\forum_topic\DataManager as ForumTopicDataManager;
use common\libraries\NotAllowedException;

//added for tool dependent publish button
use common\libraries\PlatformSetting;
use common\libraries\Request;

/**
 * $Id: forum_browser.class.php 216 2009-11-13 14:08:06Z kariboe $
 * @package application.lib.weblcms.tool.forum.component
 */
class ForumToolBrowserComponent extends ForumTool implements DelegateComponent
{

    private $action_bar;
    private $introduction_text;
    private $size; //Number of published forums

    public function run()
    {
        if (!$this->is_allowed(WeblcmsRights :: VIEW_RIGHT))
        {
            throw new NotAllowedException();
        }

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $this->get_course_id());
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_TOOL, 'forum');

        $subselect_condition = new EqualityCondition(ContentObject :: PROPERTY_TYPE, Introduction :: class_name());

        $conditions[] = new SubselectCondition(
            new PropertyConditionVariable(
                ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID
            ),
            new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID),
            ContentObject :: get_table_name(), $subselect_condition
        );

        $condition = new AndCondition($conditions);

        $this->introduction_text = \application\weblcms\DataManager :: retrieve(ContentObjectPublication :: class_name(),
            $condition);

        $this->size = 0;
        $this->allowed =
            $this->is_allowed(WeblcmsRights :: DELETE_RIGHT) || $this->is_allowed(WeblcmsRights :: EDIT_RIGHT);
        $this->action_bar = $this->get_action_bar();

        $table = $this->get_table_html();

        $this->display_header();

        $intro_text_allowed = CourseSettingsController :: get_instance()->get_course_setting(
            $this->get_course_id(),
            CourseSettingsConnector :: ALLOW_INTRODUCTION_TEXT
        );

        if ($intro_text_allowed)
        {
            echo $this->display_introduction_text($this->introduction_text);
        }

        echo $this->action_bar->as_html();
        echo $table->toHtml();

        if ($this->size == 0)
        {
            echo'<br><div style="text-align: center"><h3>' .
                Translation :: get('NoPublications', null, Utilities :: COMMON_LIBRARIES) . '</h3></div>';
        }

        $this->display_footer();
    }

    public function get_table_html()
    {
        $table = new HTML_Table(
            array('class' => 'forum', 'cellspacing' => 1)
        );

        $this->create_table_header($table);
        $row = 2;
        $this->create_table_forums($table, $row, 0);
        $this->create_table_categories($table, $row);

        return $table;
    }

    public function create_table_header($table)
    {
        $table->setCellContents(0, 0, '');

        $table->setCellAttributes(
            0, 0, array(
                'colspan' => 6,
                'class' => 'category'
            )
        );


        $table->setHeaderContents(
            1, 0, Translation :: get("Forum", null, ContentObject :: get_content_object_type_namespace('forum'))
        );
        $table->setCellAttributes(
            1, 0, array(
                'colspan' => 2
            )
        );
        $table->setHeaderContents(
            1, 2, Translation :: get("Topic", null, ContentObject :: get_content_object_type_namespace('forum'))
        );
        $table->setCellAttributes(
            1, 2, array(
                'width' => 50
            )
        );
        $table->setHeaderContents(
            1, 3, Translation :: get("Posts", null, ContentObject :: get_content_object_type_namespace('forum'))
        );
        $table->setCellAttributes(
            1, 3, array(
                'width' => 50
            )
        );
        $table->setHeaderContents(
            1, 4, Translation :: get("LastPostForum", null, ContentObject :: get_content_object_type_namespace('forum'))
        );
        $table->setCellAttributes(
            1, 4, array(
                'width' => 130
            )
        );


        $table->setHeaderContents(1, 5, '');
        $table->setCellAttributes(
            1, 5, array(
                'width' => '130'
            )
        );

    }

    public function create_table_categories($table, &$row)
    {
        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_COURSE,
            $this->get_parent()->get_course_id()
        );

        $conditions[] = new EqualityCondition(
            ContentObjectPublicationCategory :: PROPERTY_TOOL,
            $this->get_parent()->get_tool_id()
        );

        $condition = new AndCondition($conditions);
        $order = new ObjectTableOrder(ContentObjectPublicationCategory :: PROPERTY_DISPLAY_ORDER);

        $categories = \application\weblcms\DataManager :: retrieves(
            ContentObjectPublicationCategory :: class_name(),
            new DataClassRetrievesParameters($condition, null, null, $order)
        );

        while ($category = $categories->next_result())
        {
            if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
            {
                $item = new ToolbarItem(
                    Translation :: get('ManageRights', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_rights.png',
                    $this->get_url(
                        array(
                            WeblcmsManager :: PARAM_CATEGORY => $category->get_id(),
                            Tool :: PARAM_ACTION => Tool :: ACTION_EDIT_RIGHTS
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON
                );

                $actions = $item->as_html();
            }

            $table->setCellContents(
                $row, 0, '<a href="javascript:void();">' . $category->get_name() . '</a> ' . $actions
            );
            $table->setCellAttributes(
                $row, 0, array(
                    'colspan' => 2,
                    'class' => 'category'
                )
            );
            $table->setCellContents($row, 2, '');

            if ($this->allowed)
            {
                $table->setCellAttributes(
                    $row, 2, array(
                        'colspan' => 4,
                        'class' => 'category_right'
                    )
                );
            }
            else
            {
                $table->setCellAttributes(
                    $row, 2, array(
                        'colspan' => 3,
                        'class' => 'category_right'
                    )
                );
            }

            $row++;
            $this->create_table_forums($table, $row, $category->get_id());
        }
    }

    public function create_table_forums($table, &$row, $parent)
    {
        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $user_id = array();
            $course_group_ids = array();
        }
        else
        {
            $user_id = $this->get_user_id();
            $course_groups = $this->get_course_groups();

            $course_group_ids = array();

            foreach ($course_groups as $course_group)
            {
                $course_group_ids[] = $course_group->get_id();
            }
        }

        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_COURSE_ID, $this->get_course_id());
        $conditions[] = new EqualityCondition(ContentObjectPublication :: PROPERTY_TOOL, 'forum');
        $conditions[] = new InCondition(ContentObjectPublication :: PROPERTY_CATEGORY_ID, $parent);


        $subselect_condition = new EqualityCondition(ContentObject :: PROPERTY_TYPE, Forum :: class_name());

        $conditions[] = new SubselectCondition(
            new PropertyConditionVariable(
                ContentObjectPublication :: class_name(), ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID
            ),
            new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_ID),
            ContentObject :: get_table_name(), $subselect_condition
        );

        $condition = new AndCondition($conditions);

        $location = $this->get_location($parent);
        $order[] = new ObjectTableOrder(ContentObjectPublication :: PROPERTY_DISPLAY_ORDER_INDEX);
        if (!$this->get_course()->is_course_admin($this->get_user()))
        {
            $publications =
                WeblcmsDataManager :: retrieve_content_object_publications_with_view_right_granted_in_category_location(
                    $location, $this->get_entities(), $condition,
                    $order, 0, -1, $this->get_user_id()
                );
        }
        else
        {
            $publications = \application\weblcms\DataManager :: retrieve_content_object_publications(
                $condition, $order
            );
        }

        if ($publications)
        {

            $size = $publications->size();
            $this->size += $size;


            $counter = 0;
            while ($publication = $publications->next_result())
            {
                if (!($publication[ContentObjectPublication :: PROPERTY_HIDDEN] &&
                    !$this->is_allowed(WeblcmsRights :: EDIT_RIGHT)))
                {
                    $first = $counter == 0 ? true : false;
                    $last = $counter == ($size - 1) ? true : false;

                    $forum = \repository\DataManager :: retrieve_content_object(
                        $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID], Forum :: class_name()
                    );

                    if ($forum->get_locked() && !$this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
                    {
                        $title = $forum->get_title() . '<br/>' . strip_tags($forum->get_description());
                    }
                    else
                    {
                        $title = '<a href="' . $this->get_url(
                            array(
                                Tool :: PARAM_ACTION => ForumTool :: ACTION_VIEW_FORUM,
                                ComplexDisplay :: PARAM_DISPLAY_ACTION => ForumComplexDisplay:: ACTION_VIEW_FORUM,
                                Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID]
                            )
                        ) . '">' . $forum->get_title() . '</a><br />' . strip_tags($forum->get_description());
                    }
                    if ($publication[ContentObjectPublication :: PROPERTY_HIDDEN])
                    {
                        $title = '<span style="color: grey;">' . $title . '</span>';
                    }

                    $last_post = \repository\content_object\forum_topic\DataManager :: retrieve_forum_post(
                        $forum->get_last_post()
                    );

                    $src = Theme :: get_image_path() . 'forum/forum_read.png';
                    if ($forum->get_locked())
                    {
                        $src = Theme :: get_common_image_path() . 'action_lock.png';
                    }

                    $table->setCellContents(
                        $row, 0, '<img title="' . Translation :: get('NoNewPosts') . '" src="' . $src . '" />'
                    );
                    $table->setCellAttributes(
                        $row, 0, array(
                            'width' => 50,
                            'class' => 'row1',
                            'style' => 'height:50px; text-align: center;'
                        )
                    );
                    $table->setCellContents($row, 1, $title);
                    $table->setCellAttributes(
                        $row, 1, array(
                            'width' => '0%',
                            'class' => 'row1'
                        )
                    );
                    $table->setCellContents($row, 2, $forum->get_total_topics());
                    $table->setCellAttributes(
                        $row, 2, array(
                            'class' => 'row2',
                            'align' => 'center'
                        )
                    );
                    $table->setCellContents($row, 3, $forum->get_total_posts());
                    $table->setCellAttributes(
                        $row, 3, array(
                            'class' => 'row2',
                            'align' => 'center'
                        )
                    );

                    if ($last_post)
                    {
                        $link = $this->get_url(
                            array(
                                self :: PARAM_ACTION => self :: ACTION_VIEW,
                                self :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                                ComplexDisplay :: PARAM_DISPLAY_ACTION => ForumComplexDisplay :: ACTION_VIEW_TOPIC,
                                'pid' => $forum->get_id(),
                                ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID =>
                                $forum->get_last_topic_changed_cloi(),
                                ForumComplexDisplay :: PARAM_LAST_POST => $last_post->get_id()
                            )
                        );

                        $udm = UserDataManager :: get_instance();
                        $user = $udm->retrieve_user($last_post->get_user_id());

                        if (!$user)
                        {
                            $name = Translation :: get(
                                'UserNotFound', null, ContentObject :: get_content_object_type_namespace('forum')
                            );
                        }
                        else
                        {
                            $name = $user->get_fullname();
                        }

                        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
                        {
                            $content = DatetimeUtilities :: format_locale_date(null, $last_post->get_creation_date()) .
                                '<br />' . $name . ' <a href="' . $link . '"><img title="' .
                                Translation :: get('ViewLastPost', null, 'repository\content_object\forum') .
                                '" src="' . Theme :: get_image_path() . 'forum/icon_topic_latest.gif" /></a>';

                            $table->setCellContents($row, 4, $content);
                        }
                        else
                        {
                            if ($forum->get_locked())
                            {
                                $content =  DatetimeUtilities :: format_locale_date(
                                    null,
                                    $last_post->get_creation_date()
                                );

                                $content .= '<br />' . $name;

                                $table->setCellContents($row, 4, $content);
                            }
                            else
                            {
                                $content = DatetimeUtilities :: format_locale_date(
                                    null,
                                    $last_post->get_creation_date()
                                );

                                $content .= '<br />' . $name . ' <a href="' . $link . '"><img title="' .
                                    Translation :: get('ViewLastPost', null, 'repository\content_object\forum') .
                                    '" src="' . Theme :: get_image_path() . 'forum/icon_topic_latest.gif" /></a>';

                                $table->setCellContents($row, 4, $content);
                            }
                        }

                    }
                    else
                    {
                        $table->setCellContents($row, 4, '-');
                    }

                    $table->setCellAttributes(
                        $row, 4, array(
                            'align' => 'center',
                            'class' => 'row2'
                        )
                    );


                    $table->setCellContents($row, 5, $this->get_forum_actions($publication, $first, $last));
                    $table->setCellAttributes(
                        $row, 5, array(
                            'class' => 'row2'
                        )
                    );


                    $row++;
                    $counter++;
                }
            }
        }
    }

    public function get_forum_actions($publication, $first, $last)
    {
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if ($publication[ContentObjectPublication :: PROPERTY_HIDDEN])
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Show', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_invisible.png',
                        $this->get_url(
                            array(
                                Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                                Tool :: PARAM_ACTION => Tool :: ACTION_TOGGLE_VISIBILITY
                            )
                        ),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
            else
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Hide', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_visible.png',
                        $this->get_url(
                            array(
                                Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                                Tool :: PARAM_ACTION => Tool :: ACTION_TOGGLE_VISIBILITY
                            )
                        ),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }

            if ($first)
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('MoveUpNA', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_up_na.png',
                        null,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
            else
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('MoveUp', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_up.png',
                        $this->get_url(
                            array(
                                Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                                Tool :: PARAM_ACTION => Tool :: ACTION_MOVE,
                                Tool :: PARAM_MOVE_DIRECTION => Tool :: PARAM_MOVE_DIRECTION_UP
                            )
                        ),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }

            if ($last)
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('MoveDownNA', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_down_na.png',
                        null,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
            else
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('MoveDown', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_down.png',
                        $this->get_url(
                            array(
                                Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                                Tool :: PARAM_ACTION => Tool :: ACTION_MOVE,
                                Tool :: PARAM_MOVE_DIRECTION => Tool :: PARAM_MOVE_DIRECTION_DOWN
                            )
                        ),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Move', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_move.png',
                    $this->get_url(
                        array(
                            Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                            Tool :: PARAM_ACTION => Tool :: ACTION_MOVE_TO_CATEGORY
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON
                )
            );

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('EditContentObject', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_edit.png', $this->get_url(
                        array(
                            Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                            Tool :: PARAM_ACTION => Tool :: ACTION_UPDATE_CONTENT_OBJECT
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON
                )
            );

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('EditPublicationDetails', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_image_path(WeblcmsManager::context()) . 'action_edit_publication.png',
                    $this->get_url(
                        array(
                            Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                            Tool :: PARAM_ACTION => Tool :: ACTION_UPDATE_PUBLICATION
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON
                )
            );

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('BuildComplexObject', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_build.png',
                    $this->get_url(
                        array(
                            Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                            Tool :: PARAM_ACTION => Tool :: ACTION_BUILD_COMPLEX_CONTENT_OBJECT
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON
                )
            );

            $forum = \repository\DataManager :: retrieve_content_object(
                $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID], Forum :: class_name()
            );

            if ($forum->get_locked())
            {
                $parameters[ForumTool :: PARAM_ACTION] = ForumTool :: ACTION_CHANGE_LOCK;
                $parameters[ForumTool :: PARAM_PUBLICATION_ID] = $publication[ContentObjectPublication :: PROPERTY_ID];
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Unlock'),
                        Theme :: get_common_image_path() . 'action_unlock.png',
                        $this->get_url($parameters),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
            else
            {
                $parameters[ForumTool :: PARAM_ACTION] = ForumTool :: ACTION_CHANGE_LOCK;
                $parameters[ForumTool :: PARAM_PUBLICATION_ID] = $publication[ContentObjectPublication :: PROPERTY_ID];

                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Lock'),
                        Theme :: get_common_image_path() . 'action_lock.png',
                        $this->get_url($parameters),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
        }
        else
        {
            $forum = \repository\DataManager :: retrieve_content_object(
                $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID], Forum :: class_name()
            );
        }

        if ($this->is_allowed(WeblcmsRights :: DELETE_RIGHT))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_delete.png',
                    $this->get_url(
                        array(
                            Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                            Tool :: PARAM_ACTION => Tool :: ACTION_DELETE
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON,
                    true
                )
            );
        }
        if (WebApplication :: is_active('gradebook'))
        {
            $internal_item = EvaluationManager :: retrieve_internal_item_by_publication(
                WeblcmsManager :: APPLICATION_NAME, $publication[ContentObjectPublication :: PROPERTY_ID]
            );
            if ($internal_item && $internal_item->get_calculated() != 1)
            {
                $evaluate_url = $this->get_url(
                    array(
                        Tool :: PARAM_ACTION => Tool :: ACTION_EVALUATE_TOOL_PUBLICATION,
                        Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID]
                    )
                );

                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Evaluate', null, 'application/gradebook'),
                        Theme :: get_common_image_path() . 'action_evaluation.png',
                        $evaluate_url,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
        }

        $forum_namespace = ContentObject :: get_content_object_type_namespace('forum');

        if (!$forum->get_locked())
        {
            $subscribed = ForumDataManager :: retrieve_subscribe($forum->get_id(), $this->get_user_id());

            if (!$subscribed)
            {
                $parameters[Tool :: PARAM_ACTION] = ForumTool :: ACTION_FORUM_SUBSCRIBE;
                $parameters[ForumTool :: PARAM_FORUM_ID] = $forum->get_id();
                $parameters[Tool::PARAM_PUBLICATION_ID] = $publication[ContentObjectPublication :: PROPERTY_ID];

                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Subscribe', null, $forum_namespace),
                        Theme :: get_image_path($forum_namespace) . 'action_mail.png',
                        $this->get_url($parameters),
                        ToolbarItem :: DISPLAY_ICON,
                        true
                    )
                );
            }
            else
            {

                $parameters[ForumTool :: PARAM_SUBSCRIBE_ID] = $forum->get_id();
                $parameters[Tool :: PARAM_ACTION] = ForumTool :: ACTION_FORUM_UNSUBSCRIBE;
                $parameters[Tool::PARAM_PUBLICATION_ID] = $publication[ContentObjectPublication :: PROPERTY_ID];

                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('UnSubscribe', null, $forum_namespace),
                        Theme :: get_image_path($forum_namespace) . 'action_unmail.png',
                        $this->get_url($parameters),
                        ToolbarItem :: DISPLAY_ICON,
                        true
                    )
                );
            }

        }
        return '<div style="float: right;">' . $toolbar->as_html() . '</div>';
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        if ($this->is_allowed(WeblcmsRights :: ADD_RIGHT))
        {
            //added tool dependent publish button
            $tool_dependent_publish = PlatformSetting :: get('tool_dependent_publish_button', 'application\weblcms');
            
            if ($tool_dependent_publish == Tool :: PUBLISH_INDEPENDENT)
            {
                $action_bar->add_common_action(
                    new ToolbarItem(Translation :: get('Publish', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_publish.png', $this->get_url(
                            array(
                                Tool :: PARAM_ACTION => Tool :: ACTION_PUBLISH
                            )
                        ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
                );
            }
            else {
                $tool = Request :: get('tool');
                $action_bar->add_common_action(
                    new ToolbarItem(Translation :: get('PublishToolDependent', array (
                        'TYPE' => Translation :: get ('TypeNameSingle', null, 'application\weblcms\tool\\' . $tool)
                    ), Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_publish.png', $this->get_url(
                            array(
                                Tool :: PARAM_ACTION => Tool :: ACTION_PUBLISH
                            )
                        ), ToolbarItem :: DISPLAY_ICON_AND_LABEL)
                );
            }
        }

        if ($this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('ManageCategories', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_category.png',
                    $this->get_url(array(Tool :: PARAM_ACTION => ForumTool :: ACTION_MANAGE_CATEGORIES)),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }

        if ($this->get_course()->is_course_admin($this->get_user()) || $this->get_user()->is_platform_admin())
        {
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('ManageRights', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_rights.png',
                    $this->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_EDIT_RIGHTS)),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }

        $intro_text_allowed = CourseSettingsController :: get_instance()->get_course_setting(
            $this->get_course_id(),
            CourseSettingsConnector :: ALLOW_INTRODUCTION_TEXT
        );

        if (!$this->introduction_text && $intro_text_allowed && $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('PublishIntroductionText', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_introduce.png',
                    $this->get_url(array(Tool :: PARAM_ACTION => Tool :: ACTION_PUBLISH_INTRODUCTION)),
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL
                )
            );
        }

        return $action_bar;
    }

}
