<?php
namespace application\weblcms\tool\forum;

use common\libraries\Display;
use application\weblcms\ContentObjectPublication;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\DelegateComponent;

use repository\ComplexDisplay;
use repository\content_object\forum\Forum;
use repository\content_object\forum\ForumComplexDisplay;
use repository\content_object\forum\ForumComplexDisplaySupport;

use application\weblcms\WeblcmsRights;
use application\weblcms\WeblcmsManager;
use application\weblcms\Tool;

use user\User;

use tracking\Event;
use common\libraries\NotAllowedException;

/**
 * $Id: forum_viewer.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.forum.component
 */
class ForumToolViewerComponent extends ForumTool implements DelegateComponent, ForumComplexDisplaySupport
{

    private $root_content_object;
    private $publication_id;

    public function run()
    {
        $this->publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);

        $publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $this->publication_id
        );

        if (! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $publication))
        {
            throw new NotAllowedException();
        }

        $this->set_parameter(Tool :: PARAM_PUBLICATION_ID, $this->publication_id);

        if (is_null(Request :: get(ComplexDisplay :: PARAM_DISPLAY_ACTION)))
        {
            Request :: set_get(ComplexDisplay :: PARAM_DISPLAY_ACTION, ForumComplexDisplay :: ACTION_VIEW_FORUM);
        }

        $this->root_content_object = $publication->get_content_object();

        ComplexDisplay :: launch(Forum :: class_name(), $this);
    }

    public function get_root_content_object()
    {
        return $this->root_content_object;
    }

    public function forum_topic_viewed($complex_topic_id)
    {
        $parameters = array();
        $parameters[\application\weblcms\integration\tracking\ForumTopicView :: PROPERTY_USER_ID] =
            $this->get_user_id();
        $parameters[\application\weblcms\integration\tracking\ForumTopicView :: PROPERTY_PUBLICATION_ID] =
            $this->publication_id;
        $parameters[\application\weblcms\integration\tracking\ForumTopicView :: PROPERTY_FORUM_TOPIC_ID] =
            $complex_topic_id;

        Event :: trigger('view_forum_topic', WeblcmsManager :: context(), $parameters);
    }

    public function forum_count_topic_views($complex_topic_id)
    {
        $conditions[] =
            new EqualityCondition(\application\weblcms\integration\tracking\ForumTopicView :: PROPERTY_PUBLICATION_ID, $this->publication_id);
        $conditions[] =
            new EqualityCondition(\application\weblcms\integration\tracking\ForumTopicView :: PROPERTY_FORUM_TOPIC_ID, $complex_topic_id);
        $condition = new AndCondition($conditions);

        $dummy = new \application\weblcms\integration\tracking\ForumTopicView();
        return $dummy->count_tracker_items($condition);
    }

    public function get_additional_parameters()
    {
        return array(Tool :: PARAM_PUBLICATION_ID);
    }

    /**
     * function checks wether the user is forum manager
     *
     * @param $user type
     *            return bool;
     */
    public function is_forum_manager($user)
    {
        $is_forum_manager = false;

        if ($user->is_teacher())
        {

            $course = $this->get_course();

            $is_forum_manager = ($course->get_titular_id() == $user->get_id());

            if (! $is_forum_manager)
            {
                $is_forum_manager = $course->is_course_admin($user);
            }
        }

        return $is_forum_manager;
    }

    // METHODS FOR COMPLEX DISPLAY RIGHTS
    public function is_allowed_to_edit_content_object()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_view_content_object()
    {
        return $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->publication);
    }

    public function is_allowed_to_add_child()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_delete_child()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_delete_feedback()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_edit_feedback()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }
}
