<?php
namespace application\weblcms\tool\learning_path;

use application\weblcms\integration\reporting\LearningPathAttemptProgressTemplate;
use application\weblcms\integration\reporting\LearningPathAttemptProgressDetailsTemplate;
use application\weblcms\integration\tracking\LearningPathQuestionAttempt;
use common\libraries\DelegateComponent;
use application\weblcms\ContentObjectPublication;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use int;
use repository\content_object\learning_path\LearningPathComplexDisplay;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Path;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Display;
use common\libraries\Utilities;
use repository\ComplexDisplay;
use repository\content_object\learning_path\LearningPathComplexDisplaySupport;
use repository\content_object\assessment\AssessmentComplexDisplaySupport;
use repository\content_object\forum\ForumComplexDisplaySupport;
use repository\content_object\wiki\WikiComplexDisplaySupport;
use repository\content_object\blog\BlogComplexDisplaySupport;
use repository\content_object\glossary\GlossaryComplexDisplaySupport;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use tracking\Event;
use repository\content_object\learning_path\LearningPathContentObjectDisplay;
use application\weblcms\integration\tracking\DataManager as WeblcmsTrackingDataManager;

class LearningPathToolComplexDisplayComponent extends LearningPathTool implements LearningPathComplexDisplaySupport,
    AssessmentComplexDisplaySupport, ForumComplexDisplaySupport, GlossaryComplexDisplaySupport,
    BlogComplexDisplaySupport, WikiComplexDisplaySupport, DelegateComponent
{

    private $publication;

    /*
     * The question attempts
     *
     * @var QuestionAttempt[]
     */
    private $question_attempts;

    public function run()
    {
        $publication_id = Request :: get(Tool :: PARAM_PUBLICATION_ID);
        $this->set_parameter(Tool :: PARAM_PUBLICATION_ID, $publication_id);

        $this->publication = \application\weblcms\DataManager :: retrieve(
            ContentObjectPublication :: class_name(),
            $publication_id);

        if (! $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->publication))
        {
            $this->redirect(
                Translation :: get("NotAllowed", null, Utilities :: COMMON_LIBRARIES),
                true,
                array(),
                array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }
        BreadcrumbTrail :: get_instance()->add(new Breadcrumb(null, $this->get_root_content_object()->get_title()));

        ComplexDisplay :: launch($this->get_root_content_object()->get_type(), $this);
    }

    public function get_root_content_object()
    {
        $embedded_content_object_id = LearningPathContentObjectDisplay :: get_embedded_content_object_id();

        if ($embedded_content_object_id)
        {
            $this->set_parameter(
                LearningPathContentObjectDisplay :: PARAM_EMBEDDED_CONTENT_OBJECT_ID,
                $embedded_content_object_id);
            return \repository\DataManager :: retrieve_content_object($embedded_content_object_id);
        }
        else
        {
            $this->set_parameter(
                LearningPathComplexDisplay :: PARAM_LEARNING_PATH_ITEM_ID,
                Request :: get(LearningPathComplexDisplay :: PARAM_LEARNING_PATH_ITEM_ID));
            $this->set_parameter(
                ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID,
                Request :: get(ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID));
            return $this->publication->get_content_object();
        }
    }

    public function display_header()
    {
        $embedded_content_object_id = LearningPathContentObjectDisplay :: get_embedded_content_object_id();

        if ($embedded_content_object_id)
        {
            Display :: small_header();
        }
        else
        {
            parent :: display_header();
        }
    }

    public function display_footer()
    {
        $embedded_content_object_id = LearningPathContentObjectDisplay :: get_embedded_content_object_id();

        if ($embedded_content_object_id)
        {
            Display :: small_footer();
        }
        else
        {
            parent :: display_footer();
        }
    }

    public function get_publication()
    {
        return $this->publication;
    }

    public function get_additional_parameters()
    {
        return array(Tool :: PARAM_PUBLICATION_ID, LearningPathComplexDisplay :: PARAM_STEP);
    }

    public function retrieve_learning_path_tracker()
    {
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_COURSE_ID,
            $this->get_course_id());
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_LP_ID,
            $this->get_publication()->get_id());
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_USER_ID,
            $this->get_user_id());
        $condition = new AndCondition($conditions);

        $dummy = new \application\weblcms\integration\tracking\LpAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);
        $learning_path_tracker = $trackers[0];

        if (! $learning_path_tracker)
        {
            $parameters = array();
            $parameters[\application\weblcms\integration\tracking\LpAttempt :: PROPERTY_USER_ID] = $this->get_user_id();
            $parameters[\application\weblcms\integration\tracking\LpAttempt :: PROPERTY_COURSE_ID] = $this->get_course_id();
            $parameters[\application\weblcms\integration\tracking\LpAttempt :: PROPERTY_LP_ID] = $this->get_publication()->get_id();
            $parameters[\application\weblcms\integration\tracking\LpAttempt :: PROPERTY_PROGRESS] = 0;

            $return = Event :: trigger('attempt_learning_path', WeblcmsManager :: context(), $parameters);
            $learning_path_tracker = $return[0];
        }

        return $learning_path_tracker;
    }

    public function retrieve_learning_path_tracker_items($learning_path_tracker)
    {
        $learning_path_item_attempt_data = array();

        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_LP_VIEW_ID,
            $learning_path_tracker->get_id());

        $dummy = new \application\weblcms\integration\tracking\LpiAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);

        foreach ($trackers as $tracker)
        {
            $item_id = $tracker->get_lp_item_id();

            if (! $learning_path_item_attempt_data[$item_id])
            {
                $learning_path_item_attempt_data[$item_id]['score'] = 0;
                $learning_path_item_attempt_data[$item_id]['time'] = 0;
            }

            $learning_path_item_attempt_data[$item_id]['trackers'][] = $tracker;
            $learning_path_item_attempt_data[$item_id]['size'] ++;
            $learning_path_item_attempt_data[$item_id]['score'] += $tracker->get_score();

            if ($tracker->get_total_time())
            {
                $learning_path_item_attempt_data[$item_id]['time'] += $tracker->get_total_time();
            }

            if ($tracker->get_status() == 'completed' || $tracker->get_status() == 'passed')
            {
                $learning_path_item_attempt_data[$item_id]['completed'] = 1;
            }
            else
            {
                $learning_path_item_attempt_data[$item_id]['active_tracker'] = $tracker;
            }
        }

        return $learning_path_item_attempt_data;
    }

    public function get_learning_path_tree_menu_url()
    {
        return Path :: get(WEB_PATH) . 'index.php?go=course_viewer&course=' . Request :: get('course') .
             '&application=weblcms&tool=learning_path&tool_action=complex_display&publication=' .
             $this->publication->get_id() . '&' . LearningPathComplexDisplay :: PARAM_STEP . '=%s';
    }

    /**
     *
     * @param $total_steps int
     */
    public function get_learning_path_previous_url($total_steps)
    {
        return $this->get_url(
            array(
                Tool :: PARAM_ACTION => LearningPathTool :: ACTION_DISPLAY_COMPLEX_CONTENT_OBJECT,
                LearningPathTool :: PARAM_PUBLICATION_ID => Request :: get(Tool :: PARAM_PUBLICATION_ID),
                'step' => $total_steps));
    }

    /**
     * Creates a learning path item tracker
     *
     * @param $learning_path_tracker \application\weblcms\integration\tracking\LearningPathAttempt
     * @param $current_complex_content_object_item ComplexContentObjectItem
     */
    public function create_learning_path_item_tracker($learning_path_tracker, $current_complex_content_object_item)
    {
        $parameters = array();
        $parameters[\application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_LP_VIEW_ID] = $learning_path_tracker->get_id();
        $parameters[\application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_LP_ITEM_ID] = $current_complex_content_object_item->get_id();
        $parameters[\application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_START_TIME] = time();
        $parameters[\application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_TOTAL_TIME] = 0;
        $parameters[\application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_SCORE] = 0;
        $parameters[\application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_MIN_SCORE] = 0;
        $parameters[\application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_MAX_SCORE] = 0;
        $parameters[\application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_STATUS] = 'not attempted';

        $result = Event :: trigger('attempt_learning_path_item', WeblcmsManager :: context(), $parameters);
        return $result[0];
    }

    /**
     *
     * @param $complex_content_object_id int
     */
    public function get_learning_path_content_object_item_details_url($complex_content_object_id)
    {
        return $this->get_url(
            array(
                Tool :: PARAM_ACTION => LearningPathTool :: ACTION_DISPLAY_COMPLEX_CONTENT_OBJECT,
                Tool :: PARAM_PUBLICATION_ID => $this->publication->get_id(),
                LearningPathComplexDisplay :: PARAM_SHOW_PROGRESS => 'true',
                ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $complex_content_object_id,
                'attempt_id' => Request :: get('attempt_id')));
    }

    /**
     * Get the url of the assessment result
     *
     * @param $complex_content_object_id int
     * @param $details unknown_type
     */
    public function get_learning_path_content_object_assessment_result_url($complex_content_object_id, $details)
    {
        return $this->get_url(
            array(
                Tool :: PARAM_ACTION => LearningPathTool :: ACTION_DISPLAY_COMPLEX_CONTENT_OBJECT,
                Tool :: PARAM_PUBLICATION_ID => $this->publication->get_id(),
                LearningPathComplexDisplay :: PARAM_SHOW_PROGRESS => 'true',
                ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $complex_content_object_id,
                LearningPathComplexDisplay :: PARAM_DETAILS => $details));
    }

    /**
     *
     * @return array
     */
    public function get_learning_path_attempt_progress_details_reporting_template_name()
    {
        return LearningPathAttemptProgressDetailsTemplate :: class_name();
    }

    /**
     *
     * @return array
     */
    public function get_learning_path_attempt_progress_reporting_template_name()
    {
        return LearningPathAttemptProgressTemplate :: class_name();
    }

    public function get_learning_path_template_application_name()
    {
        return WeblcmsManager :: APPLICATION_NAME;
    }

    public function save_assessment_answer($complex_question_id, $answer, $score, $hint)
    {
        $question_attempt = $this->get_assessment_question_attempt($complex_question_id);
        $parameters[\application\weblcms\integration\tracking\LearningPathQuestionAttempt :: PROPERTY_HINT] = $hint;

        $question_attempt->set_answer($answer);
        $question_attempt->set_score($score);
        $question_attempt->set_hint($hint);

        $question_attempt->update();
    }

    public function save_assessment_result($total_score)
    {
        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_ID,
            $this->get_parameter(LearningPathComplexDisplay :: PARAM_LEARNING_PATH_ITEM_ID));

        $dummy = new \application\weblcms\integration\tracking\LpiAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);
        $lpi_tracker = $trackers[0];

        if (! $lpi_tracker)
        {
            return;
        }

        $lpi_tracker->set_score($total_score);
        $lpi_tracker->set_total_time($lpi_tracker->get_total_time() + (time() - $lpi_tracker->get_start_time()));

        $cloi = \repository\DataManager :: retrieve_complex_content_object_item(
            Request :: get(ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID));
        $lp_item = \repository\DataManager :: retrieve_content_object($cloi->get_ref());
        $mastery_score = $lp_item->get_mastery_score();

        if ($mastery_score)
        {
            $status = ($total_score >= $mastery_score) ? 'passed' : 'failed';
        }
        else
        {
            $status = 'completed';
        }

        $lpi_tracker->set_status($status);
        $lpi_tracker->update();
    }

    public function get_assessment_current_attempt_id()
    {
        return $this->get_parameter(LearningPathComplexDisplay :: PARAM_LEARNING_PATH_ITEM_ID);
    }

    public function get_assessment_feedback_configuration($complex_content_object_item)
    {
        return $complex_content_object_item->get_ref_object()->get_feedback_configuration();
    }

    public function get_assessment_parameters()
    {
        return array(
            LearningPathComplexDisplay :: PARAM_LEARNING_PATH_ITEM_ID,
            ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID,
            LearningPathComplexDisplay :: PARAM_STEP);
    }

    /**
     * Returns the assessment question attempts
     *
     * @return QuestionAttempt[]
     */
    public function get_assessment_question_attempts()
    {
        if(is_null($this->question_attempts))
        {
            $this->question_attempts = $this->retrieve_question_attempts();
        }

        return $this->question_attempts;
    }

    /**
     * Retrieves the question attempts for the selected assessment attempt
     *
     * @return QuestionAttempt[]
     */
    protected function retrieve_question_attempts()
    {
        $question_attempts = array();

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                LearningPathQuestionAttempt :: class_name(), LearningPathQuestionAttempt :: PROPERTY_LPI_ATTEMPT_ID
            ),
            new StaticConditionVariable($this->get_parameter(LearningPathComplexDisplay :: PARAM_LEARNING_PATH_ITEM_ID))
        );

        $question_attempts_result_set = WeblcmsTrackingDataManager :: retrieves(
            LearningPathQuestionAttempt :: class_name(), $condition
        );

        while($question_attempt = $question_attempts_result_set->next_result())
        {
            $question_attempts[$question_attempt->get_question_cid()] = $question_attempt;
        }

        return $question_attempts;
    }

    /**
     * Registers the question ids
     *
     * @param int[] $question_ids
     */
    public function register_question_ids($question_ids)
    {
        foreach($question_ids as $complex_question_id)
        {
            $parameters = array();

            $parameters[LearningPathQuestionAttempt :: PROPERTY_LPI_ATTEMPT_ID] = $this->get_parameter(
                LearningPathComplexDisplay :: PARAM_LEARNING_PATH_ITEM_ID
            );

            $parameters[LearningPathQuestionAttempt :: PROPERTY_QUESTION_CID] = $complex_question_id;
            $parameters[LearningPathQuestionAttempt :: PROPERTY_ANSWER] = '';
            $parameters[LearningPathQuestionAttempt :: PROPERTY_SCORE] = 0;
            $parameters[LearningPathQuestionAttempt :: PROPERTY_FEEDBACK] = '';
            $parameters[LearningPathQuestionAttempt :: PROPERTY_HINT] = 0;

            $trackers = Event :: trigger('attempt_learning_path_question', WeblcmsManager :: context(), $parameters);

            $this->question_attempts[$complex_question_id] = $trackers[0];
        }
    }

    /**
     * Returns the registered question ids
     *
     * @return int[] $question_ids
     */
    public function get_registered_question_ids()
    {
        $question_ids = array();

        $question_attempts = $this->get_assessment_question_attempts();
        foreach($question_attempts as $question_attempt)
        {
            $question_ids[] = $question_attempt->get_question_cid();
        }

        return $question_ids;
    }

    public function get_assessment_question_attempt($complex_question_id)
    {
        return $this->question_attempts[$complex_question_id];
    }

    public function forum_topic_viewed($complex_topic_id)
    {
        $parameters = array();
        $parameters[\application\weblcms\integration\tracking\ForumTopicView :: PROPERTY_USER_ID] = $this->get_user_id();
        $parameters[\application\weblcms\integration\tracking\ForumTopicView :: PROPERTY_PUBLICATION_ID] = $this->get_publication()->get_id();
        $parameters[\application\weblcms\integration\tracking\ForumTopicView :: PROPERTY_FORUM_TOPIC_ID] = $complex_topic_id;

        Event :: trigger('view_forum_topic', WeblcmsManager :: context(), $parameters);
    }

    public function forum_count_topic_views($complex_topic_id)
    {
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\ForumTopicView :: PROPERTY_PUBLICATION_ID,
            $this->get_publication()->get_id());

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\ForumTopicView :: PROPERTY_FORUM_TOPIC_ID,
            $complex_topic_id);
        $condition = new AndCondition($conditions);

        $dummy = new \application\weblcms\integration\tracking\ForumTopicView();
        return $dummy->count_tracker_items($condition);
    }

    /**
     * Returns whether or not the logged in user is a forum manager
     *
     * @return boolean
     */
    public function is_forum_manager($user)
    {
        return $this->get_course()->is_course_admin($user);
    }

    public function get_wiki_page_statistics_reporting_template_name()
    {
        return \application\weblcms\integration\reporting\WikiPageTemplate :: class_name();
    }

    public function get_wiki_statistics_reporting_template_name()
    {
        return \application\weblcms\integration\reporting\WikiTemplate :: class_name();
    }

    public function get_wiki_publication()
    {
        throw new \Exception("Unimplemented method : " . __CLASS__ . ':' . __METHOD__);
    }

    public function get_assessment_continue_url()
    {
    }

    public function get_assessment_back_url()
    {
    }

    public function get_assessment_current_url()
    {
        return sprintf(
            $this->get_learning_path_tree_menu_url(),
            Request :: get(LearningPathComplexDisplay :: PARAM_STEP));
    }

    // METHODS FOR COMPLEX DISPLAY RIGHTS
    public function is_allowed_to_edit_content_object()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_view_content_object()
    {
        return $this->is_allowed(WeblcmsRights :: VIEW_RIGHT, $this->publication);
    }

    public function is_allowed_to_add_child()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_delete_child()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_delete_feedback()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }

    public function is_allowed_to_edit_feedback()
    {
        return $this->is_allowed(WeblcmsRights :: EDIT_RIGHT, $this->publication);
    }
}
