<?php
namespace application\weblcms\tool\learning_path;

use application\weblcms\ContentObjectPublisher;
use application\weblcms\Tool;

use common\libraries\Application;
use common\libraries\FileProperties;
use common\libraries\Request;
use common\libraries\Session;
use common\libraries\StringUtilities;
use common\libraries\Translation;
use common\libraries\Utilities;

use repository\ContentObject;
use repository\ContentObjectImportController;
use repository\ContentObjectImportForm;
use repository\ImportParameters;
use repository\RepositoryCategory;

/**
 * $Id: learning_path_scorm_importer.class.php 216 2009-11-13 14:08:06Z kariboe
 * $
 *
 * @package application.lib.weblcms.tool.learning_path.component
 */

class LearningPathToolScormImporterComponent extends LearningPathTool
{

    public function run()
    {
        $parameters = array(Tool :: PARAM_ACTION => LearningPathTool :: ACTION_IMPORT_SCORM);
        $import_form = ContentObjectImportForm :: factory('scorm', $this, 'post', $this->get_url($parameters));

        $objects = Request :: get('objects');

        if ($import_form->validate() || $objects)
        {
            if (!$objects)
            {
                $values = $import_form->exportValues();
                $parent_id = $values[ContentObject :: PROPERTY_PARENT_ID];
                $new_category_name = $values[ContentObjectImportForm :: NEW_CATEGORY];

                if (!StringUtilities :: is_null_or_empty($new_category_name, true))
                {
                    $new_category = new RepositoryCategory();
                    $new_category->set_name($new_category_name);
                    $new_category->set_parent($parent_id);
                    $new_category->set_user_id($this->get_user_id());
                    $new_category->set_type(RepositoryCategory :: TYPE_NORMAL);
                    if (!$new_category->create())
                    {
                        throw new \Exception(Translation :: get('CategoryCreationFailed'));
                    }
                    else
                    {
                        $category_id = $new_category->get_id();
                    }
                }
                else
                {
                    $category_id = $parent_id;
                }

                if (isset($_FILES[ContentObjectImportForm :: IMPORT_FILE_NAME]))
                {
                    $file = FileProperties :: from_upload($_FILES[ContentObjectImportForm :: IMPORT_FILE_NAME]);
                }
                else
                {
                    $file = null;
                }

                $parameters = ImportParameters :: factory(
                    $import_form->exportValue(ContentObjectImportForm :: PROPERTY_TYPE), $this->get_user_id(),
                    $category_id, $file, $values
                );
                $controller = ContentObjectImportController :: factory($parameters);
                $lo_ids = $controller->run();

                $messages = $controller->get_messages_for_url();

                Session :: register(Application :: PARAM_MESSAGES, $messages);
            }
            else
            {
                $lo_ids = $objects;
            }

            $this->set_parameter('objects', $lo_ids);
            $publisher = new ContentObjectPublisher($this, $lo_ids);

            if ($publisher->ready_to_publish())
            {
                $success = $publisher->publish();

                $message = Translation :: get(
                    ($success ? 'ObjectPublished' : 'ObjectNotPublished'), array(
                        'OBJECT' => Translation :: get('Object')
                    ), Utilities :: COMMON_LIBRARIES
                );

                $parameters = array(Tool :: PARAM_ACTION => Tool :: ACTION_BROWSE);

                if ($publisher->is_publish_and_build_submit())
                {
                    $parameters[Tool :: PARAM_ACTION] = Tool :: ACTION_BUILD_COMPLEX_CONTENT_OBJECT;

                    $publications = $publisher->get_publications();
                    $parameters[Tool :: PARAM_PUBLICATION_ID] = $publications[0]->get_id();
                }

                $this->redirect($message, !$success, $parameters);
            }
            else
            {
                $this->display_header();
                echo $publisher->toHtml();
                $this->display_footer();
            }
        }
        else
        {
            $this->display_header();
            $import_form->display();
            $this->display_footer();
        }

    }

    public function with_mail_option()
    {
        return false;
    }
}
