<?php
namespace application\weblcms\tool\learning_path;

use application\weblcms\integration\reporting\LearningPathAttemptProgressTemplate;
use application\weblcms\integration\reporting\LearningPathAttemptProgressDetailsTemplate;
use application\weblcms\integration\reporting\LearningPathAttemptsTemplate;
use application\weblcms\ContentObjectPublication;
use repository\ComplexDisplay;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use application\weblcms\Tool;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Session;
use common\libraries\DelegateComponent;
use common\libraries\Translation;
use common\libraries\Utilities;
use user\UserDataManager;

/**
 * $Id: learning_path_statistics_viewer.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.learning_path.component
 */
class LearningPathToolStatisticsViewerComponent extends LearningPathTool implements DelegateComponent
{
    const PARAM_STAT = 'stats_action';
    const ACTION_DELETE_LP_ATTEMPT = 'delete_lp_attempt';
    const ACTION_DELETE_LPI_ATTEMPT = 'delete_lpi_attempt';
    const ACTION_DELETE_LPI_ATTEMPTS = 'delete_lpi_attempts';
    const PARAM_ITEM_ID = 'item_id';
    const PARAM_DELETE_ID = 'delete_id';

    private $root_content_object;

    public function run()
    {
        $trail = BreadcrumbTrail :: get_instance();

        $pid = Request :: get(Tool :: PARAM_PUBLICATION_ID);

        if (! $pid)
        {
            throw new \common\libraries\NoObjectSelectedException(Translation :: get('Publication'));
        }

        $stats_action = Request :: get(self :: PARAM_STAT);
        switch ($stats_action)
        {
            case self :: ACTION_DELETE_LP_ATTEMPT :
                $this->delete_lp_attempt(Request :: get(LearningPathTool :: PARAM_ATTEMPT_ID));
                return;
            case self :: ACTION_DELETE_LPI_ATTEMPTS :
                $this->delete_lpi_attempts_from_item(Request :: get('item_id'));
                return;
            case self :: ACTION_DELETE_LPI_ATTEMPT :
                $this->delete_lpi_attempt(Request :: get('delete_id'));
                return;
        }

        $publication = \application\weblcms\DataManager :: retrieve(ContentObjectPublication :: class_name(), $pid);

        $root_object = $publication->get_content_object();

        $parameters = array(
            Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_STATISTICS,
            Tool :: PARAM_PUBLICATION_ID => $pid);
        $url = $this->get_url($parameters);

        $attempt_id = Request :: get(LearningPathTool :: PARAM_ATTEMPT_ID);

        if ($attempt_id)
        {
            $tracker = $this->retrieve_tracker($attempt_id);
            $attempt_data = $this->retrieve_tracker_items($tracker);
            $menu = $this->get_menu($root_object->get_id(), null, $pid, $attempt_data);

            $cid = Request :: get('cid');
            if ($cid)
            {
                $parameters['cid'] = $cid;
                $url = $this->get_url($parameters);
                $trail->add(new Breadcrumb($url, Translation :: get('ItemDetails')));
            }

            $objects = $menu->get_objects();
            $details = Request :: get('details');

            if ($details)
            {
                $trail->add(
                    new Breadcrumb(
                        $this->get_url($parameters),
                        Translation :: get('AssessmentResult', null, 'application/assessment')));

                $this->set_parameter('tool_action', 'stats');
                $this->set_parameter(Tool :: PARAM_PUBLICATION_ID, $pid);
                $this->set_parameter(LearningPathTool :: PARAM_ATTEMPT_ID, $attempt_id);
                $this->set_parameter('cid', $cid);
                $this->set_parameter('details', $details);
                $_GET['display_action'] = 'view_result';

                $object = $objects[$cid];

                $this->root_content_object = $object;
                ComplexDisplay :: launch($object->get_type(), $this);
            }
            else
            {
                if ($cid)
                {
                    \reporting\viewer\Manager :: launch(
                        $this,
                        LearningPathAttemptProgressDetailsTemplate :: class_name());
                }
                else
                {
                    if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT) &&
                         Session :: get_user_id() != Request :: get(WeblcmsManager :: PARAM_USERS))
                    {
                        $this->redirect(
                            Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES),
                            true,
                            array(),
                            array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
                    }

                    if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
                    {
                        $breadcrumb_trail = BreadcrumbTrail :: get_instance();
                        $breadcrumbs = $breadcrumb_trail->get_breadcrumbs();

                        $params = array();
                        $params[LearningPathTool :: PARAM_ATTEMPT_ID] = Request :: get(
                            LearningPathTool :: PARAM_ATTEMPT_ID);
                        $params[WeblcmsManager :: PARAM_USERS] = Request :: get(WeblcmsManager :: PARAM_USERS);

                        $breadcrumbs[$breadcrumb_trail->size() - 1] = new Breadcrumb(
                            $this->get_url($params),
                            $root_object->get_title());

                        $breadcrumb_trail->set_breadcrumbtrail($breadcrumbs);
                    }
                    else
                    {
                        $breadcrumb_trail = BreadcrumbTrail :: get_instance();
                        $breadcrumbs = $breadcrumb_trail->get_breadcrumbs();

                        $breadcrumbs[$breadcrumb_trail->size() - 1] = new Breadcrumb(
                            $this->get_url(),
                            $root_object->get_title());

                        $params = array();
                        $params[LearningPathTool :: PARAM_ATTEMPT_ID] = Request :: get(
                            LearningPathTool :: PARAM_ATTEMPT_ID);
                        $params[WeblcmsManager :: PARAM_USERS] = Request :: get(WeblcmsManager :: PARAM_USERS);

                        $user_id = Request :: get(WeblcmsManager :: PARAM_USERS);
                        if ($user_id)
                        {
                            $user = UserDataManager :: get_instance()->retrieve_user($user_id);

                            $breadcrumbs[] = new Breadcrumb($this->get_url($params), $user->get_fullname());
                        }

                        $breadcrumb_trail->set_breadcrumbtrail($breadcrumbs);
                    }

                    \reporting\viewer\Manager :: launch($this, LearningPathAttemptProgressTemplate :: class_name());
                }
            }
        }
        else
        {
            if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
            {
                $this->redirect(
                    Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES),
                    true,
                    array(),
                    array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
            }

            \reporting\viewer\Manager :: launch($this, LearningPathAttemptsTemplate :: class_name());
        }
    }

    private function get_menu($root_object_id, $selected_object_id, $pid, $lpi_tracker_data)
    {
        $url = '?go=courseviewer&course=' . Request :: get('course') .
             '&application=weblcms&tool=learning_path&tool_action=view&publication=' . $pid . '&' .
             LearningPathTool :: PARAM_LP_STEP . '=%s';

        $menu = new LearningPathTree($root_object_id, $selected_object_id, $url, $lpi_tracker_data);

        return $menu;
    }

    // Statistics
    private function retrieve_tracker($attempt_id)
    {
        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_ID,
            $attempt_id);
        $dummy = new \application\weblcms\integration\tracking\LpAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);
        return $trackers[0];
    }

    private function retrieve_tracker_items($lp_tracker)
    {
        $lpi_attempt_data = array();

        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_LP_VIEW_ID,
            $lp_tracker->get_id());

        $dummy = new \application\weblcms\integration\tracking\LpiAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);

        foreach ($trackers as $tracker)
        {
            $item_id = $tracker->get_lp_item_id();
            if (! $lpi_attempt_data[$item_id])
            {
                $lpi_attempt_data[$item_id]['score'] = 0;
                $lpi_attempt_data[$item_id]['time'] = 0;
            }

            $lpi_attempt_data[$item_id]['trackers'][] = $tracker;
            $lpi_attempt_data[$item_id]['size'] ++;
            $lpi_attempt_data[$item_id]['score'] += $tracker->get_score();
            if ($tracker->get_total_time())
            {
                $lpi_attempt_data[$item_id]['time'] += $tracker->get_total_time();
            }

            if ($tracker->get_status() == 'completed')
            {
                $lpi_attempt_data[$item_id]['completed'] = 1;
            }
            else
            {
                $lpi_attempt_data[$item_id]['active_tracker'] = $tracker;
            }
        }
        return $lpi_attempt_data;
    }

    // Deleter functions
    private function delete_lp_attempt($lp_attempt_id)
    {
        if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $this->redirect(
                Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES),
                true,
                array(),
                array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }

        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_ID,
            $lp_attempt_id);
        $dummy = new \application\weblcms\integration\tracking\LpAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);
        foreach ($trackers as $tracker)
        {
            $tracker->delete();
        }

        $params = array(
            Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_STATISTICS,
            Tool :: PARAM_PUBLICATION_ID => Request :: get(Tool :: PARAM_PUBLICATION_ID));

        $this->redirect(Translation :: get('LpAttemptDeleted'), false, $params, array());
    }

    private function delete_lpi_attempt($lpi_attempt_id)
    {
        if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $this->redirect(
                Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES),
                true,
                array(),
                array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }

        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_ID,
            $lpi_attempt_id);
        $dummy = new \application\weblcms\integration\tracking\LpiAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);
        foreach ($trackers as $tracker)
        {
            $tracker->delete();
        }

        $params = array(
            Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_STATISTICS,
            Tool :: PARAM_PUBLICATION_ID => Request :: get(Tool :: PARAM_PUBLICATION_ID),
            LearningPathTool :: PARAM_ATTEMPT_ID => Request :: get(LearningPathTool :: PARAM_ATTEMPT_ID),
            'cid' => Request :: get('cid'));

        $this->redirect(Translation :: get('LpiAttemptDeleted'), false, $params, array());
    }

    private function delete_lpi_attempts_from_item($item_id)
    {
        if (! $this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $this->redirect(
                Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES),
                true,
                array(),
                array(Tool :: PARAM_ACTION, Tool :: PARAM_PUBLICATION_ID));
        }

        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_LP_ITEM_ID,
            $item_id);
        $dummy = new \application\weblcms\integration\tracking\LpiAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);
        foreach ($trackers as $tracker)
        {
            $tracker->delete();
        }

        $params = array(
            Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_STATISTICS,
            Tool :: PARAM_PUBLICATION_ID => Request :: get(Tool :: PARAM_PUBLICATION_ID),
            LearningPathTool :: PARAM_ATTEMPT_ID => Request :: get(LearningPathTool :: PARAM_ATTEMPT_ID),
            WeblcmsManager :: PARAM_USERS => Request :: get(WeblcmsManager :: PARAM_USERS));

        $this->redirect(Translation :: get('LpiAttemptsDeleted'), false, $params, array());
    }

    public function can_change_answer_data()
    {
        return true;
    }

    public function retrieve_assessment_results()
    {
        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\LearningPathQuestionAttempt :: PROPERTY_LPI_ATTEMPT_ID,
            Request :: get('details'));

        $dummy = new \application\weblcms\integration\tracking\LearningPathQuestionAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);

        $results = array();

        foreach ($trackers as $tracker)
        {
            $results[$tracker->get_question_cid()] = array(
                'answer' => $tracker->get_answer(),
                'feedback' => $tracker->get_feedback(),
                'score' => $tracker->get_score(),
                'hint' => $tracker->get_hint());
        }

        return $results;
    }

    public function change_answer_data($question_cid, $score, $feedback)
    {
        $conditions = array();

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LearningPathQuestionAttempt :: PROPERTY_LPI_ATTEMPT_ID,
            Request :: get('details'));

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LearningPathQuestionAttempt :: PROPERTY_QUESTION_CID,
            $question_cid);

        $condition = new AndCondition($conditions);

        $dummy = new \application\weblcms\integration\tracking\LearningPathQuestionAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);
        $tracker = $trackers[0];
        $tracker->set_score($score);
        $tracker->set_feedback($feedback);
        $tracker->update();
    }

    public function change_total_score($total_score)
    {
        $condition = new EqualityCondition(
            \application\weblcms\integration\tracking\LpiAttempt :: PROPERTY_ID,
            Request :: get('details'));

        $dummy = new \application\weblcms\integration\tracking\LpiAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);
        $lpi_tracker = $trackers[0];

        $lpi_tracker->set_score($total_score);
        $lpi_tracker->update();
    }

    public function get_root_content_object()
    {
        return $this->root_content_object;
    }

    public function get_additional_parameters()
    {
        return array(Tool :: PARAM_PUBLICATION_ID);
    }
}
