<?php
namespace application\weblcms\tool\learning_path;

use application\weblcms\WeblcmsRights;
use application\weblcms\ContentObjectPublication;
use repository\content_object\learning_path\LearningPath;
use common\libraries\Categorizable;
use application\weblcms\ContentObjectPublicationListRenderer;
use application\weblcms\Tool;

use repository\ComplexContentObjectItem;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\EqualityCondition;
use common\libraries\Translation;

/**
 * $Id: learning_path_tool.class.php 216 2009-11-13 14:08:06Z kariboe $
 *
 * @package application.lib.weblcms.tool.learning_path
 */
/**
 * This tool allows a user to publish learning paths in his or her course.
 */
class LearningPathTool extends Tool implements Categorizable
{

    const ACTION_DOWNLOAD_DOCUMENTS = 'document_saver';
    const ACTION_EXPORT_SCORM = 'scorm_exporter';
    const ACTION_IMPORT_SCORM = 'scorm_importer';
    const ACTION_VIEW_ASSESSMENT_RESULTS = 'assessment_results_viewer';
    const ACTION_EXPORT_RAW_RESULTS = 'assessment_raw_results_exporter';
    const ACTION_VIEW_STATISTICS = 'statistics_viewer';
    const ACTION_VIEW_CLO = 'clo_viewer';
    const ACTION_VIEW_ASSESSMENT_CLO = 'assessment_clo_viewer';
    const ACTION_VIEW_DOCUMENT = 'document_viewer';

    const PARAM_ASSESSMENT_ID = 'assessment';
    const PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID = 'lpi_attempt';
    const PARAM_OBJECT_ID = 'object_id';
    const PARAM_LEARNING_PATH = 'lp';
    const PARAM_LP_STEP = 'step';
    const PARAM_LEARNING_PATH_ID = 'lpid';
    const PARAM_ATTEMPT_ID = 'attempt_id';

    public function get_available_browser_types()
    {
        $browser_types = array();
        $browser_types[] = ContentObjectPublicationListRenderer :: TYPE_TABLE;
        $browser_types[] = ContentObjectPublicationListRenderer :: TYPE_LIST;
        return $browser_types;
    }

    public static function get_allowed_types()
    {
        return array(LearningPath :: CLASS_NAME);
    }

    public function add_content_object_publication_actions($toolbar, $publication)
    {
        $allowed = $this->is_allowed(WeblcmsRights :: EDIT_RIGHT);

        if (!$this->is_empty_learning_path($publication))
        {
            if ($allowed)
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Statistics'),
                        Theme :: get_common_image_path() . 'action_statistics.png',
                        $this->get_url(
                            array(
                                Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_STATISTICS,
                                Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID]
                            )
                        ),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );

                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('ExportRawResults'),
                        Theme :: get_common_image_path() . 'action_export.png',
                        $this->get_url(
                            array(
                                Tool :: PARAM_ACTION => LearningPathTool :: ACTION_EXPORT_RAW_RESULTS,
                                Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID]
                            )
                        ),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
        }
        else
        {
            if ($allowed)
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('StatisticsNA'),
                        Theme :: get_common_image_path() . 'action_statistics_na.png',
                        null,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
        }
    }

    private static $checked_publications = array();

    public function is_empty_learning_path($publication)
    {
        if (!array_key_exists($publication[ContentObjectPublication :: PROPERTY_ID], $this->checked_publications))
        {
            $object = $publication[ContentObjectPublication :: PROPERTY_CONTENT_OBJECT_ID];
            $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $object);
            $count = \repository\DataManager :: count_complex_content_object_items(
                ComplexContentObjectItem :: class_name(), $condition
            );

            $this->checked_publications[$publication[ContentObjectPublication :: PROPERTY_ID]] = $count == 0;
        }

        return $this->checked_publications[$publication[ContentObjectPublication :: PROPERTY_ID]];
    }

    /**
     * Helper function for the SubManager class,
     * pending access to class constants via variables in PHP 5.3
     * e.g.
     * $name = $class :: DEFAULT_ACTION
     *
     * DO NOT USE IN THIS SUBMANAGER'S CONTEXT
     * Instead use:
     * - self :: DEFAULT_ACTION in the context of this class
     * - YourSubManager :: DEFAULT_ACTION in all other application classes
     */
    public static function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    /**
     * Helper function for the SubManager class,
     * pending access to class constants via variables in PHP 5.3
     * e.g.
     * $name = $class :: PARAM_ACTION
     *
     * DO NOT USE IN THIS SUBMANAGER'S CONTEXT
     * Instead use:
     * - self :: PARAM_ACTION in the context of this class
     * - YourSubManager :: PARAM_ACTION in all other application classes
     */
    public static function get_action_parameter()
    {
        return self :: PARAM_ACTION;
    }
}
