<?php

namespace application\weblcms\tool\learning_path;

use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use common\libraries\DatetimeUtilities;
use common\libraries\Theme;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\ComplexBuilder;
use repository\DefaultContentObjectTableCellRenderer;
use user\UserDataManager;

/**
 * Renders the cells of the table
 *
 * @author Bert De Clercq (Hogeschool Gent)
 */
class AssessmentResultsTableCellRenderer extends DefaultContentObjectTableCellRenderer
{

    private $parent;
    private $table_actions;

    public function __construct($parent)
    {
        $this->parent = $parent;
        $this->table_actions = array();
    }

    /**
     * Renders the cells with data from the assessment attempts.
     *
     * @param type $column
     * @param type $assessment_attempt
     *
     * @return string
     */
    public function render_cell($column, $assessment_attempt)
    {
        if ($column === AssessmentResultsTableColumnModel :: get_action_column())
        {
            return $this->get_actions($assessment_attempt);
        }
        else
        {
            switch ($column->get_name())
            {
                case Translation :: get('User') :
                    return UserDataManager :: get_instance()->retrieve_user($this->parent->get_user_id())->get_fullname(
                    );
                case Translation :: get('Date') :
                    return DatetimeUtilities :: format_locale_date(null, $assessment_attempt->get_start_time());
                case Translation :: get('Score') :
                    return $assessment_attempt->get_score() . '%';
                case Translation :: get('Time') :
                    return Utilities :: format_seconds_to_hours($assessment_attempt->get_total_time());
                default :
                    return '';
            }
        }
    }

    /**
     * Returns an action bar
     *
     * @param type $assessment_attempt
     *
     * @return Toolbar The toolbar with the actions.
     */
    public function get_actions($assessment_attempt)
    {
        $toolbar = new Toolbar();

        $toolbar->add_item(
            new ToolbarItem(
                Translation :: get('ViewResults'),
                Theme :: get_common_image_path() . 'action_view_results.png',
                $this->parent->get_url(
                    array(
                        Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_ASSESSMENT_RESULTS,
                        Tool :: PARAM_PUBLICATION_ID => $this->parent->get_publication_id(),
                        WeblcmsManager :: PARAM_USERS => $this->parent->get_user_id(),
                        LearningPathTool :: PARAM_ATTEMPT_ID => $this->parent->get_attempt_id(),
                        ComplexBuilder :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->parent->get_ccoi_id(),
                        LearningPathTool :: PARAM_ASSESSMENT_ID => $this->parent->get_assessment_id(),
                        LearningPathTool :: PARAM_LEARNING_PATH_ITEM_ATTEMPT_ID => $assessment_attempt->get_id()
                    )
                ),
                ToolbarItem :: DISPLAY_ICON
            )
        );

        if ($this->parent->is_allowed(WeblcmsRights :: DELETE_RIGHT))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('DeleteResult'),
                    Theme :: get_common_image_path() . 'action_delete.png',
                    $this->parent->get_url(
                        array(
                            Tool :: PARAM_ACTION => LearningPathTool :: ACTION_VIEW_ASSESSMENT_RESULTS,
                            WeblcmsManager :: PARAM_PUBLICATION => $this->parent->get_publication_id(),
                            WeblcmsManager :: PARAM_USERS => $this->parent->get_user_id(),
                            LearningPathTool :: PARAM_ATTEMPT_ID => $this->parent->get_attempt_id(),
                            LearningPathToolStatisticsViewerComponent :: PARAM_DELETE_ID =>
                            $assessment_attempt->get_id()
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON,
                    true
                )
            );
        }

        return $toolbar->as_html();
    }

}
