<?php

namespace application\weblcms\tool\learning_path;

use application\weblcms\ObjectPublicationTableCellRenderer;
use common\libraries\Text;
use application\weblcms\ContentObjectPublication;
use application\weblcms\Tool;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;

/**
 * Extension on the content object publication table cell renderer for this tool
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class PublicationTableCellRenderer extends ObjectPublicationTableCellRenderer
{
    /****************************************************************************************************************
     * Inherited Functionality                                                                                      *
     ****************************************************************************************************************/

    /**
     * Renders a cell for a given object
     *
     * @param $column \common\libraries\ObjectTableColumn
     *
     * @param mixed $publication
     *
     * @return String
     */
    public function render_cell($column, $publication)
    {
        switch ($column->get_name())
        {
            case PublicationTableColumnModel :: COLUMN_PROGRESS :
            {
                if (!$this->get_component()->get_tool_browser()->get_parent()->is_empty_learning_path($publication))
                {
                    return $this->get_progress($publication);
                }
                else
                {
                    return Translation :: get('EmptyLearningPath');
                }
            }
        }

        return parent :: render_cell($column, $publication);
    }

    /****************************************************************************************************************
     * Helper Functionality                                                                                         *
     ****************************************************************************************************************/

    /**
     * Returns the progress of a given publication
     *
     * @param mixed[] $publication
     *
     * @return string
     */
    public function get_progress($publication)
    {
        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_COURSE_ID,
            $this->get_component()->get_course_id()
        );

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_LP_ID,
            $publication[ContentObjectPublication :: PROPERTY_ID]
        );

        $conditions[] = new EqualityCondition(
            \application\weblcms\integration\tracking\LpAttempt :: PROPERTY_USER_ID,
            $this->get_component()->get_user_id()
        );

        $condition = new AndCondition($conditions);

        $dummy = new \application\weblcms\integration\tracking\LpAttempt();
        $trackers = $dummy->retrieve_tracker_items($condition);
        $lp_tracker = $trackers[0];

        if ($lp_tracker)
        {
            $progress = $lp_tracker->get_progress();
        }
        else
        {
            $progress = 0;
        }

        $bar = $this->get_progress_bar($progress);
        $url = $this->get_component()->get_url(
            array(
                LearningPathTool :: PARAM_ACTION => LearningPathTool :: ACTION_DISPLAY_COMPLEX_CONTENT_OBJECT,
                Tool :: PARAM_PUBLICATION_ID => $publication[ContentObjectPublication :: PROPERTY_ID],
                'lp_action' => 'view_progress'
            )
        );

        return Text :: create_link($url, $bar);
    }

    /**
     * Returns a progress bar
     *
     * @param number $progress
     *
     * @return string
     */
    private function get_progress_bar($progress)
    {
        $html[] = '<div style="position: relative; border: 1px solid black; height: 14px; width:100px;">';
        $html[] =
            '<div style="background-color: lightblue; height: 14px; width:' . $progress . 'px; text-align: center;">';
        $html[] = '</div>';
        $html[] = '<div style="width: 100px; text-align: center; position: absolute; top: 0px;">' . round($progress) .
            '%</div></div>';

        return implode("\n", $html);
    }
}