<?php
namespace application\weblcms\tool\user;

use common\libraries\TableFormAction;
use common\libraries\TableFormActions;
use common\libraries\TableFormActionsSupport;
use common\libraries\RecordTable;
use common\libraries\Translation;
use application\weblcms\Tool;

use application\weblcms\WeblcmsRights;

/**
 * Table to display a list of groups directly subscribed to a course.
 *
 * @author Stijn Van Hoecke
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring from ObjectTable to RecordTable
 */
class DirectSubscribedPlatformGroupBrowserTable extends RecordTable implements TableFormActionsSupport
{
    const TABLE_IDENTIFIER = UserTool :: PARAM_OBJECTS;

    /****************************************************************************************************************
     * Inherited Functionality                                                                                      *
     ****************************************************************************************************************/

    /**
     * Returns the implemented form actions
     *
     * @return TableFormActions
     */
    public function get_implemented_form_actions()
    {
        if ($this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            $actions = new TableFormActions(__NAMESPACE__, Tool :: PARAM_ACTION);

            // unsubscribe
            $actions->add_form_action(
                new TableFormAction(
                    UserTool :: ACTION_UNSUBSCRIBE_GROUPS,
                    Translation :: get('UnsubscribeSelectedGroups'),
                    false
                )
            );

            // make teacher
            $actions->add_form_action(
                new TableFormAction(
                    UserTool :: ACTION_CHANGE_PLATFORMGROUP_STATUS_TEACHER,
                    Translation :: get('MakeTeacher'),
                    false
                )
            );

            // make student
            $actions->add_form_action(
                new TableFormAction(
                    UserTool :: ACTION_CHANGE_PLATFORMGROUP_STATUS_STUDENT,
                    Translation :: get('MakeStudent'),
                    false
                )
            );

            return $actions;
        }
    }
}
