<?php
namespace application\weblcms\tool\user;

use common\libraries\WebApplication;
use common\libraries\TableCellRendererActionsColumnSupport;
use common\libraries\RecordTableCellRenderer;
use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\Theme;
use common\libraries\Request;

use group\Group;

use application\weblcms\WeblcmsManager;
use application\weblcms\WeblcmsRights;
use application\weblcms\Tool;
use application\weblcms\course\CourseGroupRelation;
use application\weblcms\CourseManagementRights;

/**
 * Cell renderer for a direct subscribed course group browser table.
 *
 * @author Stijn Van Hoecke
 * @author Sven Vanpoucke - Hogeschool Gent - Refactoring from ObjectTable to RecordTable
 */
class DirectSubscribedPlatformGroupBrowserTableCellRenderer extends RecordTableCellRenderer implements
    TableCellRendererActionsColumnSupport
{
    /****************************************************************************************************************
     * Inherited Functionality                                                                                      *
     ****************************************************************************************************************/

    /**
     * Renders a single cell
     *
     * @param RecordTableColumn $column
     * @param string[] $group_with_subscription_status
     *
     * @return String
     */
    public function render_cell($column, $group_with_subscription_status)
    {
        switch ($column->get_name())
        {
            // Exceptions that need post-processing go here ...
            case Group :: PROPERTY_DESCRIPTION :
                $description = strip_tags(parent :: render_cell($column, $group_with_subscription_status));
                return Utilities :: truncate_string($description);
            case CourseGroupRelation :: PROPERTY_STATUS :
                switch ($group_with_subscription_status[CourseGroupRelation :: PROPERTY_STATUS])
                {
                    case CourseGroupRelation :: STATUS_TEACHER :
                        return Translation :: get('CourseAdmin');
                    case CourseGroupRelation :: STATUS_STUDENT :
                        return Translation :: get('Student');
                    default :
                        return Translation :: get('Unknown');
                }
        }

        return parent :: render_cell($column, $group_with_subscription_status);
    }

    /**
     * Gets the action links to display
     *
     * @param string[] $group_with_subscription_status
     *
     * @return string
     */
    public function get_actions($group_with_subscription_status)
    {
        $group_id = $group_with_subscription_status[Group :: PROPERTY_ID];

        $toolbar = new Toolbar();

        if ($this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            if ($this->get_component()->get_user()->is_platform_admin() || (
                $this->get_component()->is_allowed(WeblcmsRights :: EDIT_RIGHT) &&
                    CourseManagementRights :: get_instance()->is_allowed_for_platform_group(
                        CourseManagementRights :: TEACHER_UNSUBSCRIBE_RIGHT, $group_id,
                        $this->get_component()->get_course_id()
                    ))
            )
            {
                // unsubscribe group
                $parameters[Tool :: PARAM_ACTION] = UserTool :: ACTION_UNSUBSCRIBE_GROUPS;
                $parameters[UserTool :: PARAM_TAB] = Request :: get(UserTool :: PARAM_TAB);
                $parameters[UserTool :: PARAM_OBJECTS] = $group_id;

                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('UnsubscribeGroup'),
                        Theme :: get_common_image_path() . 'action_unsubscribe.png',
                        $this->get_component()->get_url($parameters),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }

            $weblcms_manager_namespace = WebApplication :: determine_namespace(WeblcmsManager :: APPLICATION_NAME);
                
            // change status
            switch ($group_with_subscription_status[CourseGroupRelation :: PROPERTY_STATUS])
            {
                case CourseGroupRelation :: STATUS_TEACHER :
                    $status_change_url = $this->get_component()->get_platformgroup_status_changer_url(
                        $group_id, CourseGroupRelation :: STATUS_STUDENT
                    );

                    $toolbar->add_item(
                        new ToolbarItem(
                            Translation :: get('MakeStudent'),
                            Theme :: get_image_path($weblcms_manager_namespace) . 'action_subscribe_student.png',
                            $status_change_url,
                            ToolbarItem :: DISPLAY_ICON
                        )
                    );

                    break;
                case CourseGroupRelation :: STATUS_STUDENT :
                    $status_change_url = $this->get_component()->get_platformgroup_status_changer_url(
                        $group_id, CourseGroupRelation :: STATUS_TEACHER
                    );

                    $toolbar->add_item(
                        new ToolbarItem(
                            Translation :: get('MakeTeacher'),
                            Theme :: get_image_path($weblcms_manager_namespace) . 'action_subscribe_teacher.png',
                            $status_change_url,
                            ToolbarItem :: DISPLAY_ICON
                        )
                    );

                    break;
            }
        }

        return $toolbar->as_html();
    }

}
