<?php
namespace application\weblcms\tool\user;

use application\weblcms\CourseManagementRights;
use application\weblcms\Tool;
use application\weblcms\WeblcmsManager;
use common\libraries\Request;
use common\libraries\Translation;
use application\weblcms\WeblcmsRights;
use application\weblcms\course\CourseGroupRelation;

/**
 * $Id: subscribe.class.php 218 2009-11-13 14:21:26Z kariboe $
 *
 * @package application.lib.weblcms.weblcms_manager.component
 */
/**
 * Weblcms component which allows the user to manage his or her course
 * subscriptions
 */
class UserToolGroupSubscribeComponent extends UserTool
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        if (!$this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            throw new \common\libraries\NotAllowedException();
        }

        $course = $this->get_course();
        $group_ids = Request :: get(UserTool :: PARAM_OBJECTS);

        if (isset($group_ids) && !is_array($group_ids))
        {
            $group_ids = array($group_ids);
        }

        if (isset($course))
        {
            if (isset($group_ids) && count($group_ids) > 0)
            {
                $failures = 0;

                $course_management_rights = CourseManagementRights :: get_instance();

                $parent_group_id = null;

                foreach ($group_ids as $group_id)
                {
                    if (!$this->get_user()->is_platform_admin() &&
                        !$course_management_rights->is_allowed_for_platform_group(
                            CourseManagementRights :: TEACHER_DIRECT_SUBSCRIBE_RIGHT, $group_id, $course->get_id()
                        )
                    )
                    {
                        $failures++;
                        continue;
                    }

                    if (!$this->get_parent()->subscribe_group_to_course(
                        $course, $group_id, CourseGroupRelation :: STATUS_STUDENT
                    )
                    )
                    {
                        $failures++;
                    }

                    if(!$parent_group_id)
                    {
                        $group = \group\DataManager :: retrieve_by_id(\group\Group::class_name(), $group_id);
                        $parent_group_id = $group->get_parent_id();
                    }
                }

                if ($failures == 0)
                {
                    $success = true;

                    if (count($group_ids) == 1)
                    {
                        $message = 'GroupSubscribedToCourse';
                    }
                    else
                    {
                        $message = 'GroupsSubscribedToCourse';
                    }
                }
                elseif ($failures == count($group_ids))
                {
                    $success = false;

                    if (count($group_ids) == 1)
                    {
                        $message = 'GroupNotSubscribedToCourse';
                    }
                    else
                    {
                        $message = 'GroupsNotSubscribedToCourse';
                    }
                }
                else
                {
                    $success = false;
                    $message = 'PartialGroupsNotSubscribedToCourse';
                }

                $this->redirect(
                    Translation :: get($message), ($success ? false : true), array(
                        Tool :: PARAM_ACTION => UserTool :: ACTION_SUBSCRIBE_GROUP_BROWSER,
                        WeblcmsManager :: PARAM_GROUP => $parent_group_id
                    )
                );
            }
            else
            {
                $this->display_error_page(htmlentities(Translation :: get('NoGroupsSelected')));
            }
        }
        else
        {
            $this->display_error_page(htmlentities(Translation :: get('NoCourseSelected')));
        }
    }
}
