<?php
namespace application\weblcms\tool\user;

use application\weblcms\Tool;
use application\weblcms\CourseManagementRights;
use common\libraries\Display;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use application\weblcms\WeblcmsRights;

/**
 * $Id: unsubscribe.class.php 218 2009-11-13 14:21:26Z kariboe $
 *
 * @package application.lib.weblcms.weblcms_manager.component
 */
/**
 * Weblcms component which allows the user to manage his or her course
 * subscriptions
 */
class UserToolGroupUnsubscribeComponent extends UserTool
{
    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        if (!$this->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            Display :: not_allowed();
        }

        $course = $this->get_course();
        $group_ids = Request :: get(UserTool :: PARAM_OBJECTS);
        if (!is_array($group_ids))
        {
            $group_ids = array($group_ids);
        }
        if (isset($course))
        {
            if (isset($group_ids))
            {
                $failures = 0;

                $course_management_rights = CourseManagementRights :: get_instance();

                foreach ($group_ids as $group_id)
                {
                    if(!$this->get_user()->is_platform_admin() &&
                        !$course_management_rights->is_allowed_for_platform_group(
                            CourseManagementRights :: TEACHER_UNSUBSCRIBE_RIGHT, $group_id, $course->get_id()
                        )
                    )
                    {
                        $failures++;
                        continue;
                    }

                    if (!$this->get_parent()->unsubscribe_group_from_course($course, $group_id))
                    {
                        $failures++;
                    }
                }

                if ($failures == 0)
                {
                    $success = true;

                    if (count($group_ids) == 1)
                    {
                        $message = 'GroupUnsubscribedFromCourse';
                    }
                    else
                    {
                        $message = 'GroupsUnsubscribedFromCourse';
                    }
                }
                elseif ($failures == count($group_ids))
                {
                    $success = false;

                    if (count($group_ids) == 1)
                    {
                        $message = 'GroupNotUnsubscribedFromCourse';
                    }
                    else
                    {
                        $message = 'GroupsNotUnsubscribedFromCourse';
                    }
                }
                else
                {
                    $success = false;
                    $message = 'PartialGroupsNotUnsubscribedFromCourse';
                }

                $this->redirect(
                    Translation :: get($message), ($success ? false : true), array(
                        Tool :: PARAM_ACTION  => UserTool :: ACTION_UNSUBSCRIBE_BROWSER,
                        UserTool :: PARAM_TAB => Request :: get(UserTool :: PARAM_TAB)
                    )
                );
            }
            else
            {
                $this->display_error_page(
                    htmlentities(
                        Translation :: get(
                            'NoObjectsSelected', array(
                                'OBJECT' => Translation :: get('Group')
                            ), Utilities :: COMMON_LIBRARIES
                        )
                    )
                );
            }
        }
        else
        {
            $this->display_error_page(
                htmlentities(
                    Translation :: get(
                        'NoObjectSelected', array(
                            'OBJECT' => Translation :: get('Course')
                        ), Utilities :: COMMON_LIBRARIES
                    )
                )
            );
        }
    }
}
