<?php
namespace application\weblcms\tool\user;

use application\weblcms\Tool;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Translation;
use common\libraries\Request;
use application\weblcms\CourseRequest;
use application\weblcms\CourseRequestForm;
use application\weblcms\CourseManagementRights;
use common\libraries\NotAllowedException;

/**
 * $Id: request.class.php 224 2010-04-06 14:40:30Z Yannick $
 *
 * @package applicatie.lib.weblcms.weblcms_manager.component
 */
class UserToolRequestSubscribeUserComponent extends UserTool
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $course = $this->get_course();
        $request = new CourseRequest();

        $course_management_rights = CourseManagementRights :: get_instance();

        if (!$this->get_user()->is_platform_admin() && !$course_management_rights->is_allowed(
            CourseManagementRights :: TEACHER_REQUEST_SUBSCRIBE_RIGHT, $this->get_course_id(),
            CourseManagementRights :: TYPE_COURSE, Request :: get(self :: PARAM_OBJECTS)
        )
        )
        {
            throw new NotAllowedException();
        }

        $form = new CourseRequestForm(
            CourseRequestForm :: TYPE_CREATE,
            $this->get_url(),
            $course,
            $this,
            $request,
            false,
            Request :: get(self :: PARAM_OBJECTS)
        );

        if ($form->validate())
        {
            $success_request = $form->create_request();

            $this->redirect(
                Translation :: get($success_request ? 'RequestSent' : 'RequestNotSent'),
                ($success_request ? false : true),
                array(
                    Tool :: PARAM_ACTION => UserTool :: ACTION_SUBSCRIBE_USER_BROWSER
                )
            );
        }
        else
        {
            $this->display_header();
            $form->display();
            $this->display_footer();
        }
    }

    /**
     * Returns the available parameters for registration
     *
     * @return string[]
     */
    public function get_additional_parameters()
    {
        return array(self :: PARAM_OBJECTS);
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(
                    array(
                        Tool :: PARAM_ACTION  => UserTool :: ACTION_UNSUBSCRIBE_BROWSER,
                        UserTool :: PARAM_TAB => Request :: get(UserTool :: PARAM_TAB)
                    )
                ),
                Translation :: get('UserToolUnsubscribeBrowserComponent')
            )
        );

        $breadcrumbtrail->add(
            new Breadcrumb(
                $this->get_url(array(Tool :: PARAM_ACTION => UserTool :: ACTION_SUBSCRIBE_USER_BROWSER)),
                Translation :: get('UserToolSubscribeBrowserComponent')
            )
        );
    }

}
