<?php
namespace application\weblcms\tool\user;

use application\weblcms\Tool;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\ObjectTable;
use common\libraries\Translation;
use common\libraries\ObjectTableFormActions;
use common\libraries\ObjectTableFormAction;
use application\weblcms\WeblcmsRights;

/**
 * * ***************************************************************************
 * Table to display a list of groups not in a course.
 *
 * @author Stijn Van Hoecke
 *
 *         ****************************************************************************
 */
class UnsubscribedGroupBrowserTable extends ObjectTable
{
    // **************************************************************************
    // GENERAL CONSTANTS
    // **************************************************************************
    const DEFAULT_NAME = 'unsubscribed_group_browser_table';
    const DEFAULT_ROW_COUNT = 20;

    // **************************************************************************
    // CONSTRUCTOR
    // **************************************************************************
    /**
     * Constructor
     */
    public function __construct($browser, $parameters = array(), $condition = null)
    {
        // model - renderer - dataprovider
        $model = new UnsubscribedGroupBrowserTableColumnModel();
        $renderer = new UnsubscribedGroupBrowserTableCellRenderer($browser);
        $data_provider = new UnsubscribedGroupBrowserTableDataProvider($browser, $condition);

        // parent construct
        parent :: __construct(
            $data_provider, Utilities :: get_classname_from_namespace(__CLASS__, true), $model, $renderer
        );

        // default settings
        $this->set_default_row_count(self :: DEFAULT_ROW_COUNT);
        $this->set_additional_parameters($parameters);

        // TABLE ACTIONS
        if ($browser->is_allowed(WeblcmsRights :: EDIT_RIGHT))
        {
            // add subscribe options
            $actions = new ObjectTableFormActions(__NAMESPACE__, Tool :: PARAM_ACTION);

            $actions->add_form_action(
                new ObjectTableFormAction(
                    UserTool :: ACTION_SUBSCRIBE_GROUPS,
                    Translation :: get('SubscribeSelectedGroups'),
                    false
                )
            );

            $this->set_form_actions($actions);
        }
    }

    public function handle_table_action()
    {
        $class = Utilities :: get_classname_from_namespace(__CLASS__, true);
        $ids = self :: get_selected_ids($class);
        Request :: set_get(UserTool :: PARAM_TAB, Request :: get(UserTool :: PARAM_TAB));
        Request :: set_get(UserTool :: PARAM_OBJECTS, $ids);
    }
}
