<?php
namespace application\weblcms\upgrade\v4_0_0;

use application\weblcms\CourseModuleLastAccess;
use application\weblcms\CourseSettingDefaultValue;
use application\weblcms\CourseTool;
use application\weblcms\CourseToolRelCourseSection;
use application\weblcms\DataManager;
use application\weblcms\integration\tracking\CourseVisit;
use common\libraries\AndCondition;
use common\libraries\CommonDataManager;
use common\libraries\ComparisonCondition;
use common\libraries\DataClassCache;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\DataClassResultSetCache;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\FunctionConditionVariable;
use common\libraries\InequalityCondition;
use common\libraries\OperationConditionVariable;
use common\libraries\PatternMatchCondition;
use common\libraries\package\Package;
use common\libraries\Path;
use common\libraries\PropertyConditionVariable;
use common\libraries\Registration;
use common\libraries\Setting;
use common\libraries\StaticConditionVariable;
use common\libraries\Translation;
use common\libraries\Utilities;
use core\home\HomeBlockRegistration;
use core\lynx\manager\PackageInstaller;
use tracking\Event;
use tracking\TrackerRegistration;
use tracking\EventRelTracker;
use user\integration\tracking\Visit;

/**
 *
 * @author Pieterjan Broekaert
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\application\weblcms\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                $success = $upgrader->upgrade_db($data_manager);

                if ($success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_NORMAL,
                        Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR,
                        Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if (! $success)
                {
                    return false;
                }

                $success = $upgrader->update_db_content($data_manager);

                if ($success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_NORMAL,
                        Translation :: get('UpdateDatabaseContentSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR,
                        Translation :: get('UpdateDatabaseContentFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                if (! $success)
                {
                    return false;
                }

                $success = $upgrader->upgrade_registration($data_manager);

                if ($success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_NORMAL,
                        Translation :: get('UpgradeRegistrationSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR,
                        Translation :: get('UpgradeRegistrationFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                $success = $upgrader->upgrade_indexes($data_manager);

                if ($success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_NORMAL,
                        Translation :: get('UpgradeIndexesSuccessful', null, Utilities :: COMMON_LIBRARIES));
                }
                else
                {
                    $upgrader->add_message(
                        \common\libraries\package\Upgrader :: TYPE_ERROR,
                        Translation :: get('UpgradeIndexesFailed', null, Utilities :: COMMON_LIBRARIES));
                }

                // Install the requests package if it wasn't installed seperately
                $registration = CommonDataManager :: get_registration('application\weblcms\request');

                if (! $registration instanceof Registration)
                {
                    $installer = new PackageInstaller('application\weblcms\request');
                    if (! $installer->run())
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_ERROR,
                            $installer->get_result(true));
                        return false;
                    }
                    else
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Upgrader :: TYPE_NORMAL,
                            $installer->get_result(true));
                    }
                }

                return $success;
            });

        if ($success)
        {
            $this->add_message(
                self :: TYPE_NORMAL,
                Translation :: get('UpgradeDatabaseSuccessful', null, Utilities :: COMMON_LIBRARIES));
        }
        else
        {
            $this->add_message(
                self :: TYPE_ERROR,
                Translation :: get('UpgradeDatabaseFailed', null, Utilities :: COMMON_LIBRARIES));
        }

        return $success;
    }

    /**
     * Upgrade the table structures
     *
     * @param $data_manager
     * @return bool
     */
    public function upgrade_db($data_manager)
    {
        // Change some property types for the settings storage unit
        $attributes = array('type' => 'integer', 'length' => 10);
        $success = $data_manager->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
            CourseTool :: get_table_name(),
            CourseTool :: PROPERTY_SECTION_TYPE,
            $attributes);

        if (! $success)
        {
            return false;
        }

        $attributes = array('type' => 'text');
        $success = $data_manager->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
            CourseSettingDefaultValue :: get_table_name(),
            CourseSettingDefaultValue :: PROPERTY_VALUE,
            $attributes);

        // Create course visit table
        $path = Path :: namespace_to_full_path('application\weblcms') .
             'integration/tracking/php/package/install/course_visit.xml';
        $storage_unit_info = self :: parse_xml_file($path);

        $tracking_integration_data_manager = \application\weblcms\integration\tracking\DataManager :: get_instance();

        if (! $tracking_integration_data_manager->storage_unit_exist($storage_unit_info['name']))
        {
            if (! $tracking_integration_data_manager->create_storage_unit(
                $storage_unit_info['name'],
                $storage_unit_info['properties'],
                $storage_unit_info['indexes']))
            {
                return false;
            }
        }

        // create user status change table
        $path = Path :: namespace_to_full_path('application\weblcms') .
             'integration/tracking/php/package/install/user_status_change.xml';
        $storage_unit_info = self :: parse_xml_file($path);

        if (! $tracking_integration_data_manager->storage_unit_exist($storage_unit_info['name']))
        {
            if (! $tracking_integration_data_manager->create_storage_unit(
                $storage_unit_info['name'],
                $storage_unit_info['properties'],
                $storage_unit_info['indexes']))
            {
                return false;
            }
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_assignment_submissions_tracker',
            'weblcms_assignment_submission'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_course_changes_tracker',
            'weblcms_course_change'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_forum_topic_views_tracker',
            'weblcms_forum_topic_view'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_learning_path_question_attempts_tracker',
            'weblcms_learning_path_question_attempt'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_lp_attempt_tracker',
            'weblcms_lp_attempt'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_lpi_attempt_interaction_tracker',
            'weblcms_lpi_attempt_interaction'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_lpi_attempt_objective_tracker',
            'weblcms_lpi_attempt_objective'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_lpi_attempt_tracker',
            'weblcms_lpi_attempt'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_publication_changes_tracker',
            'weblcms_publication_change'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_question_attempts_tracker',
            'weblcms_question_attempt'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_submission_feedback_tracker',
            'weblcms_submission_feedback'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_submission_note_tracker',
            'weblcms_submission_note'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_submission_score_tracker',
            'weblcms_submission_score'))
        {
            return false;
        }

        if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
            'weblcms_assessment_attempts_tracker',
            'weblcms_assessment_attempt'))
        {
            return false;
        }

        // Change field content before changing the field type
        $properties = new DataClassProperties();
        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
                    \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_STATUS),
                new StaticConditionVariable(1)));

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
                \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_STATUS),
            new StaticConditionVariable('not attempted'));

        if (! \tracking\TrackingDataManager :: get_instance()->updates(
            \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
            $properties,
            $condition))
        {
            return false;
        }

        $properties = new DataClassProperties();
        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
                    \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_STATUS),
                new StaticConditionVariable(2)));

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
                \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_STATUS),
            new StaticConditionVariable('completed'));

        if (! \tracking\TrackingDataManager :: get_instance()->updates(
            \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
            $properties,
            $condition))
        {
            return false;
        }

        // Now update the field type itself
        $attributes = array('type' => 'integer', 'notnull' => 1, 'unsigned' => 1, 'default' => 1);
        $success = \tracking\TrackingDataManager :: get_instance()->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
            \application\weblcms\integration\tracking\AssessmentAttempt :: get_table_name(),
            \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_STATUS,
            $attributes);

        if (! $success)
        {
            return false;
        }

        $attributes = array('type' => 'integer', 'notnull' => 1, 'unsigned' => 1, 'default' => 0);
        $success = \tracking\TrackingDataManager :: get_instance()->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
            \application\weblcms\integration\tracking\AssessmentAttempt :: get_table_name(),
            \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_END_TIME,
            $attributes);

        if (! $success)
        {
            return false;
        }

        $success = \tracking\TrackingDataManager :: get_instance()->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_DROP,
            \application\weblcms\integration\tracking\AssessmentAttempt :: get_table_name(),
            'date'
        );

        if (! $success)
        {
            return false;
        }

        $properties = new DataClassProperties();
        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
                    \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_END_TIME
                ),
                new OperationConditionVariable(
                    new PropertyConditionVariable(
                        \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
                        \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_START_TIME
                    ),
                    OperationConditionVariable::ADDITION,
                    new PropertyConditionVariable(
                        \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
                        \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_TOTAL_TIME
                    )
                )
            )
        );

        $condition = new InEqualityCondition(
            new PropertyConditionVariable(
                \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
                \application\weblcms\integration\tracking\AssessmentAttempt :: PROPERTY_TOTAL_TIME),
            InEqualityCondition::GREATER_THAN,
            new StaticConditionVariable(0)
        );

        if (! \tracking\TrackingDataManager :: get_instance()->updates(
            \application\weblcms\integration\tracking\AssessmentAttempt :: class_name(),
            $properties,
            $condition))
        {
            return false;
        }


        $attributes = array('type' => 'integer', 'default' => 0);
        $success = \tracking\TrackingDataManager :: get_instance()->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
            \application\weblcms\integration\tracking\QuestionAttempt :: get_table_name(),
            \application\weblcms\integration\tracking\QuestionAttempt :: PROPERTY_HINT,
            $attributes);

        if (! $success)
        {
            return false;
        }

        $attributes = array('type' => 'integer', 'default' => 0);
        $success = \tracking\TrackingDataManager :: get_instance()->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
            \application\weblcms\integration\tracking\LearningPathQuestionAttempt :: get_table_name(),
            \application\weblcms\integration\tracking\LearningPathQuestionAttempt :: PROPERTY_HINT,
            $attributes);

        if (! $success)
        {
            return false;
        }

        return true;
    }

    /**
     * Performs the database content upgrade.
     *
     * @param type $data_manager
     * @return boolean
     */
    public function update_db_content($data_manager)
    {
        $tracker = new TrackerRegistration();
        $tracker->set_context('application\weblcms\integration\tracking');
        $tracker->set_tracker('course_visit');

        if (! $tracker->create())
        {
            return false;
        }

        // Visit course
        $event = new Event();
        $event->set_context('application\weblcms\integration\tracking');
        $event->set_name('visit_course');
        $event->set_active(1);

        if (! $event->create())
        {
            return false;
        }

        $tracker_event = new EventRelTracker();
        $tracker_event->set_event_id($event->get_id());
        $tracker_event->set_tracker_id($tracker->get_id());
        $tracker_event->set_active(1);

        if (! $tracker_event->create())
        {
            return false;
        }

        // Leave course
        $event = new Event();
        $event->set_context('application\weblcms\integration\tracking');
        $event->set_name('leave_course');
        $event->set_active(1);

        if (! $event->create())
        {
            return false;
        }

        $tracker_event = new EventRelTracker();
        $tracker_event->set_event_id($event->get_id());
        $tracker_event->set_tracker_id($tracker->get_id());
        $tracker_event->set_active(1);

        if (! $tracker_event->create())
        {
            return false;
        }

        // User status change
        $tracker = new TrackerRegistration();
        $tracker->set_context('application\weblcms\integration\tracking');
        $tracker->set_tracker('user_status_change');

        if (! $tracker->create())
        {
            return false;
        }

        $event = new Event();
        $event->set_context('application\weblcms\integration\tracking');
        $event->set_name('user_status_change');
        $event->set_active(1);

        if (! $event->create())
        {
            return false;
        }

        $tracker_event = new EventRelTracker();
        $tracker_event->set_event_id($event->get_id());
        $tracker_event->set_tracker_id($tracker->get_id());
        $tracker_event->set_active(1);

        if (! $tracker_event->create())
        {
            return false;
        }

        return true;
    }

    /**
     * Upgrades the package registration.
     *
     * @return boolean
     */
    public function upgrade_registration()
    {
        $package = Package :: get(DataManager :: context());
        $registration = CommonDataManager :: get_registration(DataManager :: context());
        $registration->set_version($package->get_version());

        if (! $registration->update())
        {
            return false;
        }

        // New subpackage registrations
        $subpackages = array(
            'application\weblcms\integration\core\home',
            'application\weblcms\integration\webservice',
            'application\weblcms\integration\tracking',
            'application\weblcms\integration\reporting');

        foreach ($subpackages as $subpackage)
        {
            $package_info = Package :: get($subpackage);
            $application_registration = new Registration();

            $parent_namespace = Utilities :: get_namespace_parent($subpackage);
            $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));

            $application_registration->set_name($package_info->get_code());
            $application_registration->set_category($package_info->get_category());
            $application_registration->set_version($package_info->get_version());
            $application_registration->set_status(Registration :: STATUS_ACTIVE);
            $application_registration->set_context($subpackage);

            if (! $application_registration->create())
            {
                return false;
            }
        }

        // Tracking
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \tracking\TrackerRegistration :: class_name(),
                \tracking\TrackerRegistration :: PROPERTY_CONTEXT),
            new StaticConditionVariable('weblcms'));
        $tracker_registrations = \tracking\TrackingDataManager :: get_instance()->retrieves(
            \tracking\TrackerRegistration :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($tracker_registration = $tracker_registrations->next_result())
        {
            $tracker_registration->set_context('application\weblcms\integration\tracking');
            $tracker_registration->update();
        }

        $condition = new EqualityCondition(
            new PropertyConditionVariable(\tracking\Event :: class_name(), \tracking\Event :: PROPERTY_CONTEXT),
            new StaticConditionVariable('weblcms'));
        $event_registrations = \tracking\TrackingDataManager :: get_instance()->retrieves(
            \tracking\Event :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($event_registration = $event_registrations->next_result())
        {
            $event_registration->set_context('application\weblcms\integration\tracking');
            $event_registration->update();
        }

        $trackers = array(
            'weblcms_assignment_submissions_tracker' => 'assignment_submission',
            'weblcms_forum_topic_views_tracker' => 'forum_topic_view',
            'weblcms_learning_path_question_attempts_tracker' => 'learning_path_question_attempt',
            'weblcms_lp_attempt_tracker' => 'lp_attempt',
            'weblcms_lpi_attempt_interaction_tracker' => 'lpi_attempt_interaction',
            'weblcms_lpi_attempt_objective_tracker' => 'lpi_attempt_objective',
            'weblcms_lpi_attempt_tracker' => 'lpi_attempt',
            'weblcms_publication_changes_tracker' => 'publication_change',
            'weblcms_question_attempts_tracker' => 'question_attempt',
            'weblcms_submission_feedback_tracker' => 'submission_feedback',
            'weblcms_submission_note_tracker' => 'submission_note',
            'weblcms_submission_score_tracker' => 'submission_score',
            'weblcms_assessment_attempts_tracker' => 'assessment_attempt',
            'weblcms_course_changes_tracker' => 'course_change');

        foreach ($trackers as $old_tracker => $new_tracker)
        {

            $properties = new DataClassProperties();
            $properties->add(
                new DataClassProperty(
                    new PropertyConditionVariable(
                        \tracking\TrackerRegistration :: class_name(),
                        \tracking\TrackerRegistration :: PROPERTY_TRACKER),
                    new StaticConditionVariable($new_tracker)));

            $conditions = array();
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    \tracking\TrackerRegistration :: class_name(),
                    \tracking\TrackerRegistration :: PROPERTY_TRACKER),
                new StaticConditionVariable($old_tracker));
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(
                    \tracking\TrackerRegistration :: class_name(),
                    \tracking\TrackerRegistration :: PROPERTY_CONTEXT),
                new StaticConditionVariable('application\weblcms\integration\tracking'));
            $condition = new AndCondition($conditions);

            if (! \tracking\TrackingDataManager :: get_instance()->updates(
                \tracking\TrackerRegistration :: class_name(),
                $properties,
                $condition))
            {
                return false;
            }
        }

        // Home blocks
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                HomeBlockRegistration :: class_name(),
                HomeBlockRegistration :: PROPERTY_CONTEXT),
            new StaticConditionVariable('application\weblcms'));
        $home_block_registrations = \core\home\DataManager :: retrieves(
            HomeBlockRegistration :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($home_block_registration = $home_block_registrations->next_result())
        {
            $home_block_registration->set_context('application\weblcms\integration\core\home');
            if (! $home_block_registration->update())
            {
                return false;
            }
        }

        return true;
    }

    /**
     * Upgrades the indexes
     *
     * @param $data_manager
     * @return bool
     */
    public function upgrade_indexes(\common\libraries\DataManager $data_manager)
    {

        // course_module_last_access table
        if (! $data_manager->alter_storage_unit_index(
            DataManager :: ALTER_STORAGE_UNIT_DROP_PRIMARY_KEY,
            CourseModuleLastAccess :: get_table_name()))
        {
            return false;
        }

        // add primary key column to course_module_last_access
        $attributes = array(
            'name' => 'id',
            'type' => 'integer',
            'unsigned' => '1',
            'notnull' => '1',
            'autoincrement' => 'true');
        $column = CourseModuleLastAccess :: PROPERTY_ID;

        $success = $data_manager->alter_storage_unit(
            \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
            CourseModuleLastAccess :: get_table_name(),
            $column,
            $attributes);

        if (! $success)
        {
            return false;
        }

        if (! $data_manager->alter_storage_unit_index(
            DataManager :: ALTER_STORAGE_UNIT_ADD_INDEX,
            CourseModuleLastAccess :: get_table_name(),
            'wcmla_course_module',
            array(
                CourseModuleLastAccess :: PROPERTY_COURSE_CODE,
                CourseModuleLastAccess :: PROPERTY_MODULE_NAME,
                CourseModuleLastAccess :: PROPERTY_USER_ID,
                CourseModuleLastAccess :: PROPERTY_CATEGORY_ID)))
        {
            return false;
        }

        // course_tool_rel_course_section
        if (! $data_manager->alter_storage_unit_index(
            DataManager :: ALTER_STORAGE_UNIT_DROP_INDEX,
            CourseToolRelCourseSection :: get_table_name(),
            'wctrcs_course_tool_rel_course_section_idx'))
        {
            return false;
        }

        return true;
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::get_additional_packages()
     */
    public function get_additional_packages()
    {
        $packages = array();

        $registration = CommonDataManager :: get_registration('application\weblcms\request');
        if ($registration instanceof Registration && $registration->get_version() == '3.0.0')
        {
            $packages[] = 'application\weblcms\request';
        }

        $packages[] = 'application\weblcms\course';
        $packages[] = 'application\weblcms\course_type';

        $tool_registrations = CommonDataManager :: get_registrations_by_type('application\weblcms\tool');

        foreach ($tool_registrations as $tool_registration)
        {
            $packages[] = $tool_registration->get_context();
        }

        return $packages;
    }
}
