<?php
namespace common\extensions\external_repository_manager\implementation\vimeo;


use common\libraries\Path;
use common\libraries\Request;
use common\libraries\Redirect;
use common\libraries\ActionBarSearchForm;
use common\libraries\ArrayResultSet;
use common\libraries\Session;
use repository\ExternalUserSetting;
use repository\ExternalSetting;
use phpVimeo;

require_once Path :: get_plugin_path(__NAMESPACE__) . 'phpvimeo/vimeo.php';

/**
 * Consumer Key: Consumer Secret:
 */
class DataConnector extends \common\extensions\external_repository_manager\DataConnector
{
    const SORT_DATE_POSTED = 'date-posted';
    const SORT_DATE_TAKEN = 'date-taken';
    const SORT_INTERESTINGNESS = 'interestingness';
    const SORT_RELEVANCE = 'relevance';
    //
    private $vimeo;

    private $consumer_key;

    private $consumer_secret;

    private $token;

    private $user;

    /**
     *
     * @param $external_repository_instance ExternalRepository
     */
    public function __construct($external_repository_instance)
    {
        parent :: __construct($external_repository_instance);

        $this->consumer_key = ExternalSetting :: get('consumer_key', $this->get_external_repository_instance_id());
        $this->consumer_secret = ExternalSetting :: get('consumer_secret', $this->get_external_repository_instance_id());

        $this->vimeo = new phpVimeo($this->consumer_key, $this->consumer_secret);
        $oauth_token = ExternalUserSetting :: get('oauth_token', $this->get_external_repository_instance_id());
        $oauth_token_secret = ExternalUserSetting :: get('oauth_token_secret',
                $this->get_external_repository_instance_id());

        if (! $oauth_token || ! $oauth_token_secret)
        {
            if (! $_SESSION['request_token'])
            {
                $this->vimeo->auth('delete', Redirect :: current_url());
            }
            else
            {
                $this->vimeo->setToken($_SESSION['request_token'], $_SESSION['request_token_secret'], 'access', true);

                $this->token = $this->vimeo->getAccessToken($_GET['oauth_verifier']);
                $setting = \repository\DataManager :: retrieve_external_setting_from_variable_name('oauth_token',
                        $this->get_external_repository_instance_id());
                $user_setting = new ExternalUserSetting();
                $user_setting->set_setting_id($setting->get_id());
                $user_setting->set_user_id(Session :: get_user_id());
                $user_setting->set_value($this->token['oauth_token']);
                $user_setting->create();

                $setting = \repository\DataManager :: retrieve_external_setting_from_variable_name('oauth_token_secret',
                        $this->get_external_repository_instance_id());
                $user_setting = new ExternalUserSetting();
                $user_setting->set_setting_id($setting->get_id());
                $user_setting->set_user_id(Session :: get_user_id());
                $user_setting->set_value($this->token['oauth_token_secret']);
                $user_setting->create();
            }
        }
        else
        {
            $this->vimeo->setToken($oauth_token, $oauth_token_secret, 'access');
        }
    }

    /**
     *
     * @param $instance_id int
     * @return DataConnector:
     */
    public static function get_instance($instance_id)
    {
        if (! isset(self :: $instance[$instance_id]))
        {
            self :: $instance[$instance_id] = new self($instance_id);
        }
        return self :: $instance[$instance_id];
    }

    /**
     *
     * @param $condition mixed
     * @param $order_property ObjectTableOrder
     * @param $offset int
     * @param $count int
     * @return array
     */
    public function retrieve_videos($condition = null, $order_property, $offset, $count)
    {
        $feed_type = Request :: get(Manager :: PARAM_FEED_TYPE);

        $offset = (($offset - ($offset % $count)) / $count) + 1;
        $attributes = 'description,upload_date,modified_date,owner';

        $search_parameters = array();
        $search_parameters['per_page'] = $count;
        $search_parameters['page'] = $offset;

        if ($order_property)
        {

            $order_direction = $this->convert_order_property($order_property);

            if ($order_direction)
            {
                $search_parameters['sort'] = $order_direction;
            }
        }
        // videos for the current user.
        switch ($feed_type)
        {
            case Manager :: FEED_TYPE_MY_PHOTOS :

                if ($condition)
                {
                    $search_parameters['query'] = $condition;
                    $search_parameters['user_id'] = $this->get_user_info()->id;

                    $videos = $this->vimeo->call('vimeo.videos.search', $search_parameters);
                }
                else
                {
                    $videos = $this->vimeo->call('vimeo.videos.getAll');
                }
                break;
            case Manager :: FEED_TYPE_GENERAL :
                $search_parameters['query'] = $condition ? $condition : 'chamilo';

                $videos = $this->vimeo->call('vimeo.videos.search', $search_parameters);
                break;
            default :
                if ($condition)
                {
                    $search_parameters['query'] = $condition;
                    $search_parameters['user_id'] = $this->get_user_info()->id;

                    $videos = $this->vimeo->call('vimeo.videos.search', $search_parameters);
                }
                else
                {
                    $videos = $this->vimeo->call('vimeo.videos.getAll');
                }
                break;
        }
        return $videos;
    }

    public function get_user_info()
    {
        if (! isset($this->user))
        {
            $token = $this->vimeo->getToken();
            $response = $this->vimeo->call('vimeo.people.getInfo', array('user_id' => $token[0]));
            $this->user = $response->person;
        }
        return $this->user;
    }

    /**
     *
     * @param $condition mixed
     * @param $order_property ObjectTableOrder
     * @param $offset int
     * @param $count int
     * @return ArrayResultSet
     */
    public function retrieve_external_repository_objects($condition = null, $order_property, $offset, $count)
    {
        $videos = $this->retrieve_videos($condition, $order_property, $offset, $count);
        $videos_id = array();
        foreach ($videos->videos->video as $video)
        {
            if ($video->title && $video->upload_date)
            {
                $videos_id[] = $video->id;
            }
        }
        $videos_info = array();
        foreach ($videos_id as $video_id)
        {
            $videos_info[] = $this->vimeo->call('vimeo.videos.getInfo', array('video_id' => $video_id));
        }

        $objects = array();
        foreach ($videos_info as $video_info)
        {
            $video = $video_info->video[0];
            $object = new ExternalObject();
            $object->set_external_repository_id($this->get_external_repository_instance_id());
            $object->set_id($video->id);
            $object->set_title($video->title);
            $object->set_description($video->description);
            $object->set_created(strtotime($video->upload_date));
            $object->set_modified(strtotime($video->modified_date));
            $object->set_duration($video->duration);
            $object->set_owner_id($video->owner->id);
            $object->set_urls($video->urls->url[0]->_content);
            $object->set_tags($video->tags->tag);
            $object->set_type('video');

            $object->set_thumbnail($video->thumbnails->thumbnail[1]->_content);

            $object->set_rights($this->determine_rights($video));
            $objects[] = $object;
        }
        return new ArrayResultSet($objects);
    }

    /**
     *
     * @param $condition mixed
     * @return int
     */
    public function count_external_repository_objects($condition)
    {
        $videos = $this->retrieve_videos($condition, $order_property, 1, 1);
        return $videos->videos->total;
    }

    /**
     *
     * @param $query string
     * @return string
     */
    public static function translate_search_query($query)
    {
        return $query;
    }

    /**
     *
     * @param $order_properties ObjectTableOrder
     * @return string null
     */
    public function convert_order_property($order_properties)
    {
        if (count($order_properties) > 0)
        {
            $order_property = $order_properties[0]->get_property();
            if ($order_property == self :: SORT_RELEVANCE)
            {
                return $order_property;
            }
            else
            {
                $sorting_direction = $order_properties[0]->get_direction();

                if ($sorting_direction == SORT_ASC)
                {
                    return $order_property . '-asc';
                }
                elseif ($sorting_direction == SORT_DESC)
                {
                    return $order_property . '-desc';
                }
            }
        }

        return null;
    }

    /**
     *
     * @return array
     */
    public static function get_sort_properties()
    {
        $feed_type = Request :: get(Manager :: PARAM_FEED_TYPE);
        $query = ActionBarSearchForm :: get_query();

        if ($feed_type == Manager :: FEED_TYPE_MY_PHOTOS)
        {
            return array(self :: SORT_DATE_POSTED, self :: SORT_DATE_TAKEN, self :: SORT_INTERESTINGNESS,
                    self :: SORT_RELEVANCE);
        }
        else
        {
            return array();
        }
    }

    /*
     * (non-PHPdoc) @see
     * common/extensions/external_repository_manager/ManagerConnector#retrieve_external_repository_object()
     */
    public function retrieve_external_repository_object($id)
    {
        $video = $this->vimeo->call('vimeo.videos.getInfo', array('video_id' => $id));
        $video = $video->video[0];

        $object = new ExternalObject();
        $object->set_external_repository_id($this->get_external_repository_instance_id());
        $object->set_id($video->id);
        $object->set_title($video->title);
        $object->set_description($video->description);
        $object->set_created(strtotime($video->upload_date));
        $object->set_modified(strtotime($video->modified_date));
        $object->set_duration($video->duration);
        $object->set_owner_id($video->owner->id);
        $object->set_urls($video->urls->url[0]->_content);
        $object->set_tags($video->tags->tag);

        $object->set_thumbnail($video->thumbnails->thumbnail[2]->_content);

        $object->set_rights($this->determine_rights($video));

        return $object;
    }

    /**
     *
     * @param $values array
     * @return boolean
     */
    public function update_external_repository_object($values)
    {
        $response = $this->vimeo->call('vimeo.videos.setDescription',
                array('description' => $values['description'], 'video_id' => $values['id']));
        if (! $response->stat == 'ok')
        {
            return false;
        }
        else
        {
            $response = $this->vimeo->call('vimeo.videos.setTitle',
                    array('title' => $values['title'], 'video_id' => $values['id']));
            if (! $response->stat == 'ok')
            {
                return false;
            }
            else
            {
                $response = $this->vimeo->call('vimeo.videos.clearTags', array('video_id' => $values['id']));
                if ($response->stat == 'ok')
                {
                    $response = $this->vimeo->call('vimeo.videos.addTags',
                            array('video_id' => $values['id'], 'tags' => $values['tags']));
                    if (! $response->stat == 'ok')
                    {
                        return false;
                    }
                }
                return true;
            }
        }
    }

    /**
     *
     * @param $values array
     * @param $photo_path string
     * @return mixed
     */
    public function create_external_repository_object($values, $video_path)
    {

        $video_id = $this->vimeo->upload($video_path);
        var_dump($video_id);
        $response = $this->vimeo->call('vimeo.videos.setDescription',
                array('description' => $values['description'], 'video_id' => $video_id));
 
        if (! $response->stat == 'ok')
        {
            return false;
        }
        else
        {
            $response = $this->vimeo->call('vimeo.videos.setTitle',
                    array('title' => $values['title'], 'video_id' => $video_id));
            if (! $response->stat == 'ok')
            {
                return false;
            }
            else
            {
                $response = $this->vimeo->call('vimeo.videos.clearTags', array('video_id' => $video_id));
                if ($response->stat == 'ok')
                {
                    $response = $this->vimeo->call('vimeo.videos.addTags',
                            array('video_id' => $video_id, 'tags' => $values['tags']));
                    if (! $response->stat == 'ok')
                    {
                        return false;
                    }
                }

                return true;
            }
        }
    }

    /**
     *
     * @param $content_object ContentObject
     * @return mixed
     */
    public function export_external_repository_object($content_object)
    {
        $video_id = $this->vimeo->upload($content_object->get_full_path());

        $response = $this->vimeo->call('vimeo.videos.setDescription',
                array('description' => $content_object->get_description(), 'video_id' => $video_id));
        if (! $response->stat == 'ok')
        {
            return false;
        }
        else
        {
            $response = $this->vimeo->call('vimeo.videos.setTitle',
                    array('title' => $content_object->get_title(), 'video_id' => $video_id));
            if (! $response->stat == 'ok')
            {
                return false;
            }
        }
        return true;
    }

    /**
     *
     * @param $license int
     * @param $photo_user_id string
     * @return boolean
     */
    public function determine_rights($video_entry)
    {
        $rights = array();
        $rights[ExternalObject :: RIGHT_USE] = true;
        $rights[ExternalObject :: RIGHT_EDIT] = true;
        $rights[ExternalObject :: RIGHT_DELETE] = true;
        $rights[ExternalObject :: RIGHT_DOWNLOAD] = false;
        return $rights;
    }

    /**
     *
     * @param $id string
     * @return mixed
     */
    public function delete_external_repository_object($id)
    {
        return $this->vimeo->call('vimeo.videos.delete', array('video_id' => $id));
    }
}
