<?php
namespace common\extensions\rights_editor_manager;

use common\libraries\Toolbar;
use common\libraries\Translation;
use common\libraries\Utilities;

use user\DefaultUserTableCellRenderer;

use rights\RightsUtilities;

/**
 * Cell renderer for the user object browser table
 */
class LocationUserBrowserTableCellRenderer extends DefaultUserTableCellRenderer
{

    // Inherited
    public function render_cell($column, $user)
    {
        if ($column === LocationUserBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($user);
        }

        if (LocationUserBrowserTableColumnModel :: is_rights_column($column))
        {
            return $this->get_rights_column_value($column, $user);
        }

        // Add special features here
        switch ($column->get_name())
        {
        }
        return parent :: render_cell($column, $user);
    }


    private function get_rights_column_value($column, $user)
    {
        $component = $this->get_component();
        $locations = $component->get_locations();
        $locked_parent = $locations[0]->get_locked_parent();
        $rights = $component->get_available_rights();
        $user_id = $user->get_id();

        $location_url = $component->get_url(array('application' => $this->application,
                'location' => ($locked_parent ? $locked_parent->get_id() : $locations[0]->get_id())));

        foreach ($rights as $right_name => $right_id)
        {
            $column_name = Translation :: get(Utilities :: underscores_to_camelcase(strtolower($right_name)));
            if ($column->get_name() == $column_name)
            {
                $rights_url = $component->get_url(array(
                        RightsEditorManager :: PARAM_RIGHTS_EDITOR_ACTION => RightsEditorManager :: ACTION_SET_USER_RIGHTS,
                        'user_id' => $user_id, 'right_id' => $right_id));
                return RightsUtilities :: get_rights_icon($location_url, $rights_url, $locked_parent, $right_id, $user, $locations[0]);
            }
        }
        return '&nbsp;';
    }
    
    /**
     * Returns the actions toolbar
     *
     * @param Course $course
     *
     * @return String
     */
    public function get_object_actions($course)
    {
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);
        return $toolbar->as_html();
    }
}
