<?php
namespace common\extensions\rights_editor_manager;

use common\libraries\ObjectTableColumn;
use common\libraries\StaticTableColumn;
use common\libraries\Utilities;
use common\libraries\Translation;

use user\DefaultUserTableColumnModel;
use user\User;
/**
 * $Id: location_user_browser_table_column_model.class.php 191 2009-11-13
 * 11:50:28Z chellee $
 *
 * @package
 *          application.common.rights_editor_manager.component.location_user_bowser
 */
/**
 * Table column model for the user browser table
 */
class LocationUserBrowserTableColumnModel extends DefaultUserTableColumnModel
{
    /**
     * The tables modification column
     */
    private static $modification_column;
    private static $rights_columns;

    /**
     * Constructor
     */
    public function __construct($object_table)
    {
        parent :: __construct($object_table);
        $this->add_column(new ObjectTableColumn(User :: PROPERTY_USERNAME));
        $this->set_default_order_column(1);
        $this->add_rights_columns();
        $this->set_columns(array_splice($this->get_columns(), 1));
    }

    /**
     * Gets the modification column
     *
     * @return ContentObjectTableColumn
     */
    public static function get_modification_column()
    {
        if (! isset(self :: $modification_column))
        {
            self :: $modification_column = new StaticTableColumn('');
        }
        return self :: $modification_column;
    }

    public static function is_rights_column($column)
    {
        return in_array($column, self :: $rights_columns);
    }

    public function add_rights_columns()
    {
        $rights = $this->get_component()->get_available_rights();

        foreach ($rights as $right_name => $right_id)
        {
            $column_name = Utilities :: underscores_to_camelcase(strtolower($right_name));
            $column = new StaticTableColumn(Translation :: get($column_name));
            $this->add_column($column);
            self :: $rights_columns[] = $column;
        }
    }
}
