<?php
namespace common\extensions\rights_editor_manager;

use common\libraries\SubManager;
use common\libraries\Session;
use common\libraries\Request;
use common\libraries\Path;
use common\libraries\Utilities;

/**
 * $Id: rights_editor_manager.class.php 191 2009-11-13 11:50:28Z chellee $
 * @package application.common.rights_editor_manager
 */
class RightsEditorManager extends SubManager
{
    const PARAM_RIGHTS_EDITOR_ACTION = 'rights_action';

    const ACTION_SET_USER_RIGHTS = 'set_user_rights';
    const ACTION_SET_GROUP_RIGHTS = 'set_group_rights';
    const ACTION_SET_TEMPLATE_RIGHTS = 'set_template_rights';
    const ACTION_CHANGE_INHERIT = 'change_inherit';
    const ACTION_EDIT_SIMPLE_RIGHTS = 'simple_rights';
    const ACTION_EDIT_ADVANCED_RIGHTS_RIGHTS = 'advanced_rights_editor';

    const PARAM_GROUP = 'group_id';

    const TYPE_USER = 'user';
    const TYPE_GROUP = 'group';
    const TYPE_TEMPLATE = 'template';

    private $locations;
    private $excluded_groups;
    private $excluded_users;
    private $limited_groups;
    private $limited_users;
    private $limited_templates;

    private $types;

    /**
     * @param unknown_type $parent
     * @param array $locations
     */
    public function __construct($parent, $locations)
    {
        parent :: __construct($parent);

        $this->locations = $locations;
        $this->excluded_users = array(Session :: get_user_id());
        $this->excluded_groups = array();

        $this->included_users = array();
        $this->included_groups = array();

        $this->types = array(self :: TYPE_USER, self :: TYPE_GROUP, self :: TYPE_TEMPLATE);

        $rights_editor_action = Request :: get(self :: PARAM_RIGHTS_EDITOR_ACTION);
        if ($rights_editor_action)
        {
            $this->set_parameter(self :: PARAM_RIGHTS_EDITOR_ACTION, $rights_editor_action);
        }
    }

    public function factory($content_object, $parent, $locations)
    {
        if ($content_object)
        {
            $type = $content_object->get_type_name();
            $file = Path :: get_repository_path() . 'content_object/' . $type . '/php/rights/' . $type . '_rights_editor_manager.class.php';
            if (file_exists($file))
            {
                $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'RightsEditorManager';
                $manager = new $class($parent, $locations);

                return $manager;
            }
        }
        return new RightsEditorManager($parent, $locations);
    }

    public function run()
    {
        $parent = $this->get_parameter(self :: PARAM_RIGHTS_EDITOR_ACTION);

        switch ($parent)
        {
            case self :: ACTION_EDIT_ADVANCED_RIGHTS_RIGHTS :
                $component = $this->create_component('AdvancedRightsEditor');
                break;
            case self :: ACTION_SET_USER_RIGHTS :
                $component = $this->create_component('UserRightsSetter');
                break;
            case self :: ACTION_SET_GROUP_RIGHTS :
                $component = $this->create_component('GroupRightsSetter');
                break;
            case self :: ACTION_SET_TEMPLATE_RIGHTS :
                $component = $this->create_component('TemplateRightsSetter');
                break;
            case self :: ACTION_CHANGE_INHERIT :
                $component = $this->create_component('InheritChanger');
                break;
            default :
                $component = $this->create_component('AdvancedRightsEditor');
                break;
        }
        $component->run();
    }

    public function get_locations()
    {
        return $this->locations;
    }

    public function set_locations($locations)
    {
        $this->locations = $locations;
    }

    public function get_available_rights()
    {
        return $this->get_parent()->get_available_rights();
    }

    public function exclude_users($users)
    {
        $this->excluded_users = $users;
    }

    public function exclude_groups($groups)
    {
        $this->excluded_groups = $groups;
    }

    public function exclude_templates($templates)
    {
        $this->excluded_templates = $templates;
    }

    public function get_excluded_templates()
    {
        return $this->excluded_templates;
    }

    public function get_excluded_users()
    {
        return $this->excluded_users;
    }

    public function get_excluded_groups()
    {
        return $this->excluded_groups;
    }

    /**
     * @param Array $users An array of user ids
     */
    public function limit_users(array $users)
    {
        $this->limited_users = $users;
    }

    /**
     * @param Array $templates An array of template ids
     */
    public function limit_templates(array $templates)
    {
        $this->limited_templates = $templates;
    }

    /**
     * @param Array $groups An array of group ids
     */
    public function limit_groups(array $groups)
    {
        $this->limited_groups = $groups;
    }

    public function get_limited_users()
    {
        return $this->limited_users;
    }

    public function get_limited_templates()
    {
        return $this->limited_templates;
    }

    public function get_limited_groups()
    {
        if (is_null($this->limited_groups))
        {

            return array();
        }
        else
        {
            return $this->limited_groups;
        }
    }

    public function set_limited_groups($groups)
    {
        $this->limited_groups = $groups;
    }

    public function set_types(array $types)
    {
        $this->types = $types;
    }

    public function get_types()
    {
        return $this->types;
    }

    public function create_component($type, $application)
    {
        $component = parent :: create_component($type, $application);

        if (is_subclass_of($component, __CLASS__))
        {
            $component->set_locations($this->locations);
            $component->exclude_users($this->get_excluded_users());
            $component->exclude_groups($this->get_excluded_groups());
            $component->limit_users($this->get_limited_users());
            $component->limit_groups($this->get_limited_groups());
            $component->set_types($this->get_types());
        }

        return $component;
    }

}
