<?php
namespace common\extensions\video_conferencing_manager;

use repository\ExternalInstance;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Session;
use repository\ContentObject;
use repository\ExternalSync;

abstract class ConferenceObject
{

    /**
     *
     * @var array
     */
    private $default_properties;

    /**
     *
     * @var VideoConferencingSync
     */
    private $synchronization_data;
    const PROPERTY_ID = 'id';
    const PROPERTY_VIDEO_CONFERENCING_ID = 'video_conferencing_id';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_OWNER_ID = 'owner_id';

    /**
     *
     * @param $default_properties array
     */
    public function __construct($default_properties = array ())
    {
        $this->default_properties = $default_properties;
    }

    /**
     * Get the default properties of all data classes.
     * 
     * @return array The property names.
     */
    public static function get_default_property_names($extended_property_names = array())
    {
        $extended_property_names[] = self :: PROPERTY_ID;
        $extended_property_names[] = self :: PROPERTY_VIDEO_CONFERENCING_ID;
        $extended_property_names[] = self :: PROPERTY_TITLE;
        $extended_property_names[] = self :: PROPERTY_OWNER_ID;
        return $extended_property_names;
    }

    /**
     * Gets a default property of this data class object by name.
     * 
     * @param $name string The name of the property.
     * @param mixed
     */
    public function get_default_property($name)
    {
        return (isset($this->default_properties) && array_key_exists($name, $this->default_properties)) ? $this->default_properties[$name] : null;
    }

    /**
     *
     * @param $default_properties the $default_properties to set
     */
    public function set_default_properties($default_properties)
    {
        $this->default_properties = $default_properties;
    }

    /**
     * Sets a default property of this data class by name.
     * 
     * @param $name string The name of the property.
     * @param $value mixed The new value for the property.
     */
    public function set_default_property($name, $value)
    {
        $this->default_properties[$name] = $value;
    }

    public function get_default_properties()
    {
        return $this->default_properties;
    }

    /**
     *
     * @return string
     */
    public function get_modified()
    {
        return time();
    }

    /**
     *
     * @return string
     */
    public function get_id()
    {
        return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     *
     * @return int
     */
    public function get_video_conferencing_id()
    {
        return $this->get_default_property(self :: PROPERTY_VIDEO_CONFERENCING_ID);
    }

    /**
     *
     * @return string
     */
    public function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    /**
     *
     * @param $title string
     */
    public function set_title($title)
    {
        $this->set_default_property(self :: PROPERTY_TITLE, $title);
    }

    /**
     *
     * @param $id string
     */
    public function set_id($id)
    {
        $this->set_default_property(self :: PROPERTY_ID, $id);
    }

    /**
     *
     * @param $video_conferencing_id int
     */
    public function set_video_conferencing_id($video_conferencing_id)
    {
        $this->set_default_property(self :: PROPERTY_VIDEO_CONFERENCING_ID, $video_conferencing_id);
    }

    /**
     *
     * @return string
     */
    public function get_owner_id()
    {
        return $this->get_default_property(self :: PROPERTY_OWNER_ID);
    }

    /**
     *
     * @param $owner_id string
     */
    public function set_owner_id($owner_id)
    {
        $this->set_default_property(self :: PROPERTY_OWNER_ID, $owner_id);
    }

    /**
     *
     * @return VideoConferencingSync
     */
    public function get_synchronization_data()
    {
        if (! isset($this->synchronization_data))
        {
            $sync_conditions = array();
            $sync_conditions[] = new EqualityCondition(ExternalSync :: PROPERTY_EXTERNAL_OBJECT_ID, $this->get_id());
            $sync_conditions[] = new EqualityCondition(ExternalSync :: PROPERTY_EXTERNAL_ID, 
                    $this->get_video_conferencing_id());
            $sync_conditions[] = new EqualityCondition(ContentObject :: PROPERTY_OWNER_ID, Session :: get_user_id(), 
                    ContentObject :: get_table_name());
            $sync_condition = new AndCondition($sync_conditions);
            $this->synchronization_data = \repository\DataManager :: retrieve_external_sync($sync_condition);
        }
        
        return $this->synchronization_data;
    }

    /**
     *
     * @return int
     */
    public function get_synchronization_status()
    {
        return $this->get_synchronization_data()->get_synchronization_status(null, $this->get_modified());
    }

    /**
     *
     * @return boolean
     */
    public function is_importable()
    {
        return ! $this->get_synchronization_data() instanceof ExternalSync;
    }

    public function get_connector()
    {
        $external_instance = \repository\DataManager :: retrieve_by_id(ExternalInstance :: class_name(), 
                $this->get_video_conferencing_id());
        return DataConnector :: get_instance($external_instance);
    }

    /**
     * Get the fully qualified class name of the ExternalObject
     * 
     * @return string
     */
    public static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the ExternalObject
     * 
     * @return string
     */
    public static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }

    /**
     * Get the type of the ExternalObject
     * 
     * @return string
     * @deprecated Use context() to get the namespace (= type)
     */
    public static function get_object_type()
    {
        return self :: OBJECT_TYPE;
    }
}
