<?php
namespace common\extensions\video_conferencing_manager;


use repository\ExternalSync;

abstract class DataConnector
{

    private static $instances = array();

    private $video_conferencing_instance;

    /**
     *
     * @param $video_conferencing_instance VideoConferencing
     */
    public function __construct($video_conferencing_instance)
    {
        $this->video_conferencing_instance = $video_conferencing_instance;
    }

    /**
     *
     * @return VideoConferencing
     */
    public function get_video_conferencing_instance()
    {
        return $this->video_conferencing_instance;
    }

    /**
     *
     * @param $video_conferencing_instance VideoConferencing
     */
    public function set_video_conferencing_instance($video_conferencing_instance)
    {
        $this->video_conferencing_instance = $video_conferencing_instance;
    }

    /**
     *
     * @return int
     */
    public function get_video_conferencing_instance_id()
    {
        return $this->get_video_conferencing_instance()->get_id();
    }

    /**
     *
     * @param $external_instance ExternalInstance
     * @return DataConnector
     */
    public static function factory($external_instance)
    {
        $class = $external_instance->get_type() . '\DataConnector';
        return new $class($external_instance);
    }

    /**
     *
     * @param $video_conferencing_instance VideoConferencing
     * @return DataConnector
     */
    public static function get_instance($video_conferencing_instance)
    {
        if (! isset(self :: $instances[$video_conferencing_instance->get_id()]))
        {
            self :: $instances[$video_conferencing_instance->get_id()] = self :: factory($video_conferencing_instance);
        }
        return self :: $instances[$video_conferencing_instance->get_id()];
    }

    /**
     *
     * @param $id string
     */
    abstract public function retrieve_video_conferencing_object($external_sync);

    public function retrieve_external_object(ExternalSync $external_sync)
    {
        return $this->retrieve_video_conferencing_object($external_sync);
    }

    /**
     *
     * @param $condition mixed
     * @param $order_property ObjectTableOrder
     * @param $offset int
     * @param $count int
     */
    abstract public function retrieve_video_conferencing_objects($condition, $order_property, $offset, $count);

    abstract public function create_video_conferencing_object(ConferenceObject $video_conferencing_object);

    abstract public function join_video_conferencing_object(ExternalSync $external_sync, ConferenceRights $rights);

    /**
     *
     * @param $condition mixed
     */
    abstract public function count_video_conferencing_objects($condition);

    /**
     *
     * @param $id string
     */
    abstract public function delete_video_conferencing_object($id);

    /**
     *
     * @param $id string
     */
    abstract public function export_video_conferencing_object($id);

    /**
     *
     * @param $query string
     */
    abstract public static function translate_search_query($query);
}
