<?php
namespace common\libraries\package;

use common\libraries\Path;
use common\libraries\Utilities;
use common\libraries\Translation;

abstract class Action
{
    const TYPE_NORMAL = '1';
    const TYPE_CONFIRM = '2';
    const TYPE_WARNING = '3';
    const TYPE_ERROR = '4';

    private $message;

    public function __construct()
    {
        $this->message = array();
    }

    public function add_message($type = self :: TYPE_NORMAL, $message)
    {
        switch ($type)
        {
            case self :: TYPE_NORMAL :
                $this->message[] = $message;
                break;
            case self :: TYPE_CONFIRM :
                $this->message[] = '<span style="color: green; font-weight: bold;">' . $message . '</span>';
                break;
            case self :: TYPE_WARNING :
                $this->message[] = '<span style="color: orange; font-weight: bold;">' . $message . '</span>';
                break;
            case self :: TYPE_ERROR :
                $this->message[] = '<span style="color: red; font-weight: bold;">' . $message . '</span>';
                break;
            default :
                $this->message[] = $message;
                break;
        }
    }

    public function set_message($message)
    {
        $this->message = $message;
    }

    /**
     *
     * @return multitype:string
     */
    public function get_message()
    {
        return $this->message;
    }

    public function failed($error_message)
    {
        $this->add_message(self :: TYPE_ERROR, $error_message);
        $this->add_message(self :: TYPE_ERROR, Translation :: get($this->get_type() . 'Failed'));
        return false;
    }

    public function successful()
    {
        $this->add_message(self :: TYPE_CONFIRM, Translation :: get($this->get_type() . 'Successful'));
        return true;
    }

    /**
     *
     * @return string
     */
    public function retrieve_message()
    {
        return implode('<br />' . "\n", $this->get_message());
    }

    public function get_type()
    {
        return Utilities :: get_classname_from_object($this);
    }

    public function get_path()
    {
        return Path :: namespace_to_full_path(static :: context());
    }

    /**
     * Get the fully qualified class name of the Action
     * 
     * @return string
     */
    public static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get the namespace of the Action
     * 
     * @return string
     */
    public static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }
}
