<?php
namespace common\libraries;

/**
 * A case element condition variable that describes a single element of a case in a select query
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 *
 * @package common.libraries
 */
class CaseElementConditionVariable extends ConditionVariable
{
    /**
     * The condition used after the WHEN statement. If empty the case element is an ELSE statement.
     *
     * @var Condition
     */
    private $condition;

    /**
     * The Statement
     *
     * @var string
     */
    private $statement;

    /**
     * Constructor
     *
     * @param string $statement
     * @param Condition $condition
     */
    public function __construct($statement, $condition)
    {
        $this->statement = $statement;
        $this->condition = $condition;
    }

    /**
     * Get the condition
     *
     * @return string
     */
    public function get_condition()
    {
        return $this->condition;
    }

    /**
     * Set the condition
     *
     * @param Condition $condition
     */
    public function set_condition($condition)
    {
        $this->condition = $condition;
    }

    /**
     * Get the statement
     *
     * @return Condition
     */
    public function get_statement()
    {
        return $this->statement;
    }

    /**
     * Set the statement
     *
     * @param string $statement
     */
    public function set_statement($statement)
    {
        $this->statement = $statement;
    }

    /**
     * Get an md5 representation of this object for identification purposes
     *
     * @param string[] $hash_parts
     *
     * @return string
     */
    public function hash($hash_parts = array())
    {
        if($this->condition instanceof Condition)
        {
            $hash_parts[] = $this->condition->hash();
        }

        $hash_parts[] = $this->statement;

        return parent :: hash($hash_parts);
    }
}
