<?php
namespace common\libraries;

/**
 * A ConditionVariable that describes a regular DataClass property with a fixed alias
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class FixedPropertyConditionVariable extends PropertyConditionVariable
{

    /**
     * Constructor
     *
     * @param $class string
     * @param $property string
     * @param $alias string
     */
    public function __construct($class, $property, $alias)
    {
        parent :: __construct($class, $property);
        $this->alias = $alias;
    }

    /**
     * Get the alias of the DataClass object the property belongs to
     *
     * @return string
     */
    public function get_alias()
    {
        return $this->alias;
    }

    /**
     * Set the alias of the DataClass object the property belongs to
     *
     * @param $alias string
     */
    public function set_alias($alias)
    {
        $this->alias = $alias;
    }

    /**
     * Get an md5 representation of this object for identification purposes
     *
     * @param string[] $hash_parts
     *
     * @return string
     */
    public function hash($hash_parts = array())
    {
        $hash_parts[] = $this->alias;
        
        return parent :: hash($hash_parts);
    }
}
