<?php
namespace common\libraries;

/**
 * A ConditionVariable that describes an operation on two other ConditionVariables
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class OperationConditionVariable extends ConditionVariable
{
    /**
     * A constant defining an addition
     *
     * @var int
     */
    CONST ADDITION = 1;
    
    /**
     * A constant defining a subtraction
     *
     * @var int
     */
    CONST MINUS = 2;
    
    /**
     * A constant defining a multiplication
     *
     * @var int
     */
    CONST MULTIPLICATION = 3;
    
    /**
     * A constant defining a division
     *
     * @var int
     */
    CONST DIVISION = 4;
    
    /**
     * The ConditionVariable on the left side of the operation
     *
     * @var \common\libraries\ConditionVariable
     */
    private $left;
    
    /**
     * The operator that connects both ConditionVariables
     *
     * @var int
     */
    private $operator;
    
    /**
     * The ConditionVariable on the right side of the operation
     *
     * @var \common\libraries\ConditionVariable
     */
    private $right;

    /**
     * Constructor
     *
     * @param $context string
     * @param $left \common\libraries\ConditionVariable
     * @param $operator int
     * @param $right \common\libraries\ConditionVariable
     */
    public function __construct($left, $operator, $right)
    {
        $this->left = $left;
        $this->operator = $operator;
        $this->right = $right;
    }

    /**
     * Get the ConditionVariable on the left side of the operation
     *
     * @return \common\libraries\ConditionVariable
     */
    public function get_left()
    {
        return $this->left;
    }

    /**
     * Set the ConditionVariable on the left side of the operation
     *
     * @param $left \common\libraries\ConditionVariable
     */
    public function set_left($left)
    {
        $this->left = $left;
    }

    /**
     * Get the operator that connects both ConditionVariables
     *
     * @return int
     */
    public function get_operator()
    {
        return $this->operator;
    }

    /**
     * Set the operator that connects both ConditionVariables
     *
     * @param $operator int
     */
    public function set_operator($operator)
    {
        $this->operator = $operator;
    }

    /**
     * Get the ConditionVariable on the right side of the operation
     *
     * @return \common\libraries\ConditionVariable
     */
    public function get_right()
    {
        return $this->right;
    }

    /**
     * Set the ConditionVariable on the right side of the operation
     *
     * @param $right \common\libraries\ConditionVariable
     */
    public function set_right($right)
    {
        $this->right = $right;
    }

    /**
     * Get an md5 representation of this object for identification purposes
     *
     * @param $hash_parts multitype:string
     * @return string
     */
    public function hash($hash_parts = array())
    {
        $parts = array();
        $parts[] = $this->left->hash();
        $parts[] = $this->right->hash();
        
        if ($this->operator != self :: DIVISION)
        {
            sort($parts);
        }
        foreach ($parts as $part)
        {
            $hash_parts[] = $part;
        }
        
        $hash_parts[] = $this->operator;
        
        return parent :: hash($hash_parts);
    }
}
