<?php
namespace common\libraries;

/**
 * A ConditionVariable that describes all the properties of a DataClass
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 *
 * @package common.libraries
 */
class PropertiesConditionVariable extends ConditionVariable
{
    /**
     * The fully qualified class name of the DataClass object the property belongs to
     *
     * @var string
     */
    private $class;

    /**
     * Constructor
     *
     * @param $class string
     */
    public function __construct($class)
    {
        $this->class = $class;
    }

    /**
     * Get the fully qualified class name of the DataClass object the property belongs to
     *
     * @return string
     */
    public function get_class()
    {
        return $this->class;
    }

    /**
     * Set the fully qualified class name of the DataClass object the property belongs to
     *
     * @param $class string
     */
    public function set_class($class)
    {
        $this->class = $class;
    }

    /**
     * Get an md5 representation of this object for identification purposes
     *
     * @param string[] $hash_parts
     *
     * @return string
     */
    public function hash($hash_parts = array())
    {
        $hash_parts[] = $this->class;

        return parent :: hash($hash_parts);
    }

    /****************************************************************************************************************
     * Helper Functionality                                                                                         *
     ****************************************************************************************************************/

    /**
     * Determines the datamanager class of this dataclass
     *
     * @return string
     */
    public function determine_data_manager()
    {
        $class = $this->get_class();

        $namespace = $class :: context();
        $data_manager = $namespace . '\DataManager';
        if (! class_exists($data_manager))
        {
            $data_manager = $namespace . '\\' . Utilities :: get_package_name_from_namespace($namespace, true) .
                'DataManager';
        }

        return $data_manager;
    }

    /**
     * Determines the alias of the dataclass
     *
     * @return string
     */
    public function get_alias()
    {
        $data_manager = $this->determine_data_manager();
        $class_name = $this->get_class();

        return $data_manager :: get_instance()->get_alias($class_name :: get_table_name());
    }

}
