<?php
namespace common\libraries;

/**
 * A ConditionVariable that describes a static value
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class StaticConditionVariable extends ConditionVariable
{
    /**
     * A static value that should remain unchanged in the Condition
     *
     * @var string
     */
    private $value;

    /**
     * Whether or not the variable should be quoted
     * 
     * @var bool
     */
    private $quote;

    /**
     *
     * @param mixed $value
     * @param bool $quote
     */
    public function __construct($value, $quote = true)
    {
        $this->value = $value;
        $this->quote = $quote;
    }

    /**
     * Get the static value
     *
     * @return string
     */
    public function get_value()
    {
        return $this->value;
    }

    /**
     * Set the static value
     *
     * @param $value string
     */
    public function set_value($value)
    {
        $this->value = $value;
    }

    /**
     * Get the static quote
     *
     * @return string
     */
    public function get_quote()
    {
        return $this->quote;
    }

    /**
     * Set the static quote
     *
     * @param $quote string
     */
    public function set_quote($quote)
    {
        $this->quote = $quote;
    }


    /**
     * Get an md5 representation of this object for identification purposes
     *
     * @param array $hash_parts
     * @return string
     */
    public function hash($hash_parts = array())
    {
        $hash_parts[] = $this->value;
        $hash_parts[] = $this->quote;

        return parent :: hash($hash_parts);
    }
}
