<?php

namespace common\libraries;

/**
 * Description of css
 *
 * @author laurent
 */
class CssUtilities
{
    const DEFAULT_CHARSET = 'iso-8859-1';

    /**
     * Minify a CSS string
     *
     * @param string $css
     * @return string
     */
    public static function Minify($css)
    {

        require_once Path :: get_plugin_path() . '/minify/css/Compressor.php';

        $result = \Minify_CSS_Compressor :: process($css);
        $result = str_ireplace('@CHARSET "' . self :: DEFAULT_CHARSET . '";', '', $result); //we feed the charset in the http header
        return $result;
    }

    /**
     * Read a css path, process it and returns its content.
     * Includes css imports into the file.
     * Resolve relative urls from the page's base.
     *
     * @param string $path  file path
     * @return string       the css content
     *
     * @todo: transform urls to absolute path instead of relative??
     */
    public static function get($path)
    {
        if (! is_readable($path))
        {
            return false;
        }
        $result = file_get_contents($path);

        //includes imports
        $pattern = '/^\.*@import.*url\(.*\);.*$/mi';
        $result = preg_replace_callback($pattern, function ($import) use($path)
        {
            $import = reset($import);
            $import = trim($import);
            $import = str_replace('  ', ' ', $import);
            $import = str_ireplace('@import url(', '', $import);
            $import = str_ireplace(');', '', $import);
            $import = str_replace("'", '', $import);
            $rel = dirname($path) . '/' . $import;
            $r = CssUtilities :: get($rel);
            return $r;
        }, $result);

        //rewrite relative urls
        $urls = array();
        $pattern = '#url\(.?\.\./.*.?\)#i';
        $result = preg_replace_callback($pattern, function ($url) use($path)
        {
            $url = reset($url);
            $url = trim($url);
            $url = str_ireplace('url(', '', $url);
            $url = str_ireplace(')', '', $url);
            $url = str_ireplace("'", '', $url);
            $dir = dirname($path) . '/';
            $file_path = $dir . $url;
            $file_path = realpath($file_path);
            $sys_path = Path :: get(SYS_PATH);
            $sys_path = realpath($sys_path);
            $new_url = str_ireplace($sys_path . DIRECTORY_SEPARATOR, '', $file_path);
            $new_url = str_replace('\\', '/', $new_url); //for windows

            return "url('" . Path :: get(WEB_PATH) . $new_url. "')";
        }, $result);

        return $result;
    }

}
