<?php
namespace common\libraries;


/**
 * $Id: header.class.php 128 2009-11-09 13:13:20Z vanpouckesven $
 * @package common.html
 */
/**
 * Class to display the header of a HTML-page
 */
class Header
{
    /**
     * Singleton
     *
     * @var Header
     */
    private static $instance;

    /**
     * The http headers which will be send to the browser using php's header
     * (...) function.
     */
    private $http_headers;
    /**
     * The html headers which will be added in the <head> tag of the html
     * document.
     */
    private $html_headers;
    /**
     * The language code
     */
    private $language_code;

    /**
     * Constructor
     */
    public function __construct($language_code = 'en')
    {
        $this->http_headers = array();
        $this->html_headers = array();
        $this->set_language_code($language_code);
    }

    /**
     *
     * @return Header
     */
    public static function get_instance()
    {
        if (self :: $instance == null)
        {
            self :: set_instance(new Header());
        }

        return self :: $instance;
    }

    public static function set_instance($instance)
    {
        self :: $instance = $instance;
    }

    public function set_language_code($language_code)
    {
        $this->language_code = $language_code;
    }

    /**
     * Adds some default headers to the output
     */
    public function add_default_headers()
    {
        $server_type = PlatformSetting :: get('server_type');
        $theme = Theme :: get_theme();
        $time = ClientCache :: get_start_time();
        $application = Theme :: get_application();
        $context = "theme=$theme&amp;time=$time&amp;server_type=$server_type";

        $this->add_http_header('Content-Type: text/html; charset=UTF-8');
        $this->add_http_header('X-Powered-By: Chamilo ' . $this->get_setting('version', 'core\admin'));

        $this->add_html_header('<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />');

        $this->add_css_file_header(Path :: get(WEB_PATH) . 'css.inc.php?' . $context);
        $this->add_css_file_header(Path :: get(WEB_PATH) . 'css.inc.php?' . $context . '&amp;application=' . $application);

        $this->add_link_header($this->get_path(WEB_PATH) . 'index.php', 'top');
        $this->add_link_header('http://help.chamilo.org/', 'help');
        $this->add_link_header(Theme :: get_common_image_path() . 'favicon.ico', 'shortcut icon', null, 'image/x-icon');
        $this->add_html_header('<script type="text/javascript">var rootWebPath="' . Path :: get(WEB_PATH) . '";</script>');
        $this->add_javascript_file_header(Path :: get(WEB_PATH) . 'javascript.inc.php?' . $context);
//         $this->add_javascript_file_header(Path :: get(WEB_PATH) . 'javascript.inc.php?' . $context . '&amp;application=' . $application);

    }

    /**
     * Adds a http header
     */
    public function add_http_header($http_header)
    {
        $this->http_headers[] = $http_header;
    }

    /**
     * Adds a html header
     */
    public function add_html_header($html_header)
    {
        $this->html_headers[] = $html_header;
    }

    /**
     * Sets the page title
     */
    public function set_page_title($title)
    {
        $this->add_html_header('<title>' . $title . '</title>');
    }

    /**
     * Adds a css file
     */
    public function add_css_file_header($file, $media = 'screen,projection')
    {
        $header = '<link rel="stylesheet" type="text/css" media="' . $media . '" href="' . $file . '" />';
        $this->add_html_header($header);
    }

    /**
     * Adds a javascript file
     */
    public function add_javascript_file_header($file)
    {
        $header[] = '<script type="text/javascript" src="' . $file . '"></script>';
        $this->add_html_header(implode(' ', $header));
    }

    /**
     * Adds a link
     */
    public function add_link_header($url, $rel = null, $title = null, $type = null)
    {
        $type = $type ? ' type="' . $type . '"' : '';
        $title = $title ? ' title="' . $title . '"' : '';
        $rel = $rel ? ' rel="' . $rel . '"' : '';
        $href = ' href="' . $url . '"';
        $header = '<link' . $href . $rel . $title . $type . '/>';
        $this->add_html_header($header);
    }

    /**
     * Displays the header. This function will send all http headers to the
     * browser and display the head-tag of the html document.
     */
    public function display()
    {
        echo $this->toHtml();
    }

    /**
     * Creates the HTML output for the header. This function will send all http
     * headers to the browser and return the head-tag of the html document
     */
    public function toHtml()
    {
        foreach ($this->http_headers as $index => & $http_header)
        {
            header($http_header);
        }
        $output[] = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';
        $output[] = '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="' . $this->language_code . '" lang="' . $this->language_code . '">';
        $output[] = ' <head>';
        foreach ($this->html_headers as $index => & $html_header)
        {
            $output[] = '  ' . $html_header;
        }
        $output[] = ' </head>';
        return implode("\n", $output);
    }

    public function get_path($path_type)
    {
        return Path :: get($path_type);
    }

    public static function get_section()
    {
        global $this_section;
        return $this_section;
    }

    public static function set_section($section)
    {
        global $this_section;
        $this_section = $section;
    }

    public function get_setting($variable, $application)
    {
        return PlatformSetting :: get($variable, $application);
    }
}
