<?php
namespace common\libraries;

/**
 * This class represents a column for a  table
 *
 * Refactoring from ObjectTable to split between a table based on a record and based on an object
 * (@TODO: Used New in the name because of the fact that there is currently a class with the name TableColumn)
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class NewTableColumn
{
    /****************************************************************************************************************
     * Properties                                                                                                   *
     ****************************************************************************************************************/

    /**
     * The name of the column
     *
     * @var string
     */
    private $name;

    /**
     * The visible title of the column in the layout
     */
    private $title;

    /**
     * Whether or not the column is sortable
     *
     * @var bool
     */
    private $sortable;

    /**
     * The alias of the storage unit for the column
     *
     * @var string
     */
    private $storage_unit_alias;

    /****************************************************************************************************************
     * Constructor                                                                                                  *
     ****************************************************************************************************************/

    /**
     * Constructor
     *
     * @param string $name
     * @param string $title - [OPTIONAL] default null - translation of the column name
     * @param bool $sortable - [OPTIONAL] default null
     * @param string $storage_unit_alias - [OPTIONAL] default null
     */
    public function __construct($name, $title = null, $sortable = true, $storage_unit_alias = null)
    {
        $this->set_name($name);

        if(is_null($title))
        {
            $backtrace = debug_backtrace();

            $index = 1;

            do
            {
                $called_class = $backtrace[$index]['class'];
                $context = Utilities :: get_namespace_from_classname($called_class);
                $index++;
            }
            while($context == Utilities :: COMMON_LIBRARIES);


            $title = Translation :: get(Utilities :: underscores_to_camelcase($name), array(), $context);
        }

        $this->set_title($title);
        $this->set_sortable($sortable);
        $this->set_storage_unit_alias($storage_unit_alias);
    }

    /****************************************************************************************************************
     * Getters & Setters                                                                                            *
     ****************************************************************************************************************/

    /**
     * Returns the name of this column
     *
     * @return string
     */
    public function get_name()
    {
        return $this->name;
    }

    /**
     * Sets the name of this column
     *
     * @param string $name
     */
    public function set_name($name)
    {
        $this->name = $name;
    }
    
    /**
     * Returns the title of this column
     * 
     * @return string
     */
    public function get_title()
    {
        return $this->title;
    }

    /**
     * Sets the title of this column
     * 
     * @param string $title
     */
    public function set_title($title)
    {
        $this->title = $title;
    }

    /**
     * Returns if this column is sortable
     *
     * @return boolean
     */
    public function is_sortable()
    {
        return $this->sortable;
    }

    /**
     * Sets if this column is sortable
     *
     * @param boolean $sortable
     */
    public function set_sortable($sortable)
    {
        $this->sortable = $sortable;
    }

    /**
     * Returns the storage unit alias
     *
     * @return string
     */
    public function get_storage_unit_alias()
    {
        return $this->storage_unit_alias;
    }

    /**
     * Sets the storage unit alias
     *
     * @param string $storage_unit_alias
     */
    public function set_storage_unit_alias($storage_unit_alias)
    {
        $this->storage_unit_alias = $storage_unit_alias;
    }
}
