<?php
namespace common\libraries;
/**
 * This class allows you to wrap an array in a result set. It does not offer any performance increase, but in select
 * cases, you will need it.
 *
 * @author Tim De Pauw
 */
class ArrayResultSet extends ResultSet
{
    /**
     * The data in this set
     */
    private $data;
    /**
     * A pointer to the current element in the set
     */
    private $pointer;

    /**
     * Constructor
     *
     * @param $array array
     */
    public function __construct($array)
    {
        $this->data = $array;
        $this->pointer = 0;
    }

    // Inherited
    public function next_result($map_to_object = false)
    {
        if ($this->pointer <= count($this->data))
        {
            $this->pointer ++;
            return $this->data[$this->pointer - 1];
        }
        return null;
    }

    // Inherited
    public function as_array()
    {
        return $this->data;
    }

    // Inherited
    public function size()
    {
        return count($this->data);
    }

    public function current()
    {
        return $this->pointer;
    }

    // Inherited
    public function skip($count)
    {
        $this->pointer += $count;
    }

    public function reset()
    {
        $this->pointer = 0;
    }
}
