<?php
namespace common\libraries;

use Exception;

/**
 * A class to allow caching of DataClass object grouped counts
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class DataClassCountGroupedCache extends DataClassCache
{

    /**
     * Add a DataClass object grouped count to the cache
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassCountGroupedParameters
     * @param $count multitype:int
     * @return boolean
     */
    public static function add($class, $parameters, $counts)
    {
        if (! $parameters instanceof DataClassCountGroupedParameters)
        {
            throw new Exception('Illegal parameters passed to the DataClassCountGroupedCache');
        }

        if (! is_array($counts))
        {
            $type = is_object($counts) ? get_class($counts) : gettype($counts);
            throw new Exception(
                'The DataClassCountGroupedCache cache only allows for caching of integer arrays. Currently trying to add: ' .
                     $type . '.');
        }

        if (! DataClassCache :: exists($class, $parameters))
        {
            DataClassCache :: set_cache($class, $parameters->hash(), $counts);
        }

        return true;
    }
}
