<?php
namespace common\libraries;

use Exception;

/**
 * A class to allow caching of distinct property values of DataClass objects
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
class DataClassDistinctCache extends DataClassCache
{

    /**
     * Add a set of distinct property values of a DataClass object to the cache
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassDistinctParameters
     * @param $property_values multitype:string
     * @return boolean
     */
    public static function add($class, $parameters, $property_values)
    {
        if (! $parameters instanceof DataClassDistinctParameters)
        {
            throw new Exception('Illegal parameters passed to the DataClassDistinctCache');
        }

        if (! is_array($property_values))
        {
            $type = is_object($property_values) ? get_class($property_values) : gettype($property_values);
            throw new Exception(
                'The DataClassDistinctCache cache only allows for caching of string arrays. Currently trying to add: ' .
                     $type . '.');
        }

        if (! DataClassCache :: exists($class, $parameters))
        {
            DataClassCache :: set_cache($class, $parameters->hash(), $property_values);
        }

        return true;
    }
}
