<?php
namespace common\libraries;


/**
 * The optional parameters for a DataClass storage layer action
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
abstract class DataClassParameters implements Hashable
{

    /**
     * The condition to be applied to the action
     *
     * @var \common\libraries\Condition
     */
    private $condition;
    /**
     * The joins to be applied to the action
     *
     * @var  \common\libraries\Joins
     */
    private $joins;

    /**
     * Constructor
     *
     * @param $condition \common\libraries\Condition
     */
    public function __construct($condition = null,\common\libraries\Joins $joins = null)
    {
        $this->condition = $condition;
        $this->joins = $joins;
    }

    /**
     * Get the condition to be applied to the action
     *
     * @return \common\libraries\Condition
     */
    public function get_condition()
    {
        return $this->condition;
    }

    /**
     * Set the condition to be applied to the action
     *
     * @param $condition \common\libraries\Condition
     */
    public function set_condition($condition)
    {
        $this->condition = $condition;
    }

    /**
     * Get the join data classes to be applied to the action
     *
     * @return  \common\libraries\Joins
     */
    public function get_joins()
    {
        return $this->joins;
    }

    /**
     * Set the join data classes to be applied to the action
     *
     * @param $joins \common\libraries\Joins
     */
    public function set_joins($joins)
    {
        $this->joins = $joins;
    }

    /**
     * Get the fully qualified class name
     *
     * @return string
     */
    public static function class_name()
    {
        return get_called_class();
    }

    /**
     * Get an md5 representation of this object for identification purposes
     *
     * @param $hash_parts multitype:string
     * @return string
     */
    public function hash($hash_parts = array())
    {
        $hash_parts[] = self :: class_name();
        $hash_parts[] = ($this->get_condition() instanceof Condition ? $this->get_condition()->hash() : null);
        $hash_parts[] = ($this->get_joins() instanceof Joins ? $this->get_joins()->hash() : null);

        return md5(serialize($hash_parts));
    }
}
