<?php

namespace common\libraries;

/**
 * DataClassListener which can be extended to listen to the crud functionality of a dataclass
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
abstract class DataClassListener
{
    /***************************************************************************************************************
     * Dummy functionality                                                                                         *
     ***************************************************************************************************************/

    /**
     * Calls this function before the creation of a dataclass in the database
     */
    public function on_before_create() {}

    /**
     * Calls this function after the creation of a dataclass in the database
     *
     * @param bool $success
     */
    public function on_after_create($success) {}

    /**
     * Calls this function before the update of a dataclass in the database
     */
    public function on_before_update() {}

    /**
     * Calls this function after the update of a dataclass in the database
     *
     * @param bool $success
     */
    public function on_after_update($success) {}

    /**
     * Calls this function before the deletion of a dataclass in the database
     */
    public function on_before_delete() {}

    /**
     * Calls this function after the deletion of a dataclass in the database
     *
     * @param bool $success
     */
    public function on_after_delete($success) {}

    /**
     * Calls this function before a property is set
     */
    public function on_before_set_property($name, $value) {}

    /**
     * Calls this function after a property is set
     */
    public function on_after_set_property($name, $value) {}
}
