<?php
namespace common\libraries;

/**
 * The optional parameters for a DataClass storage layer property-based action
 *
 * @author Hans De Bisschop
 * @package common.libraries
 */
abstract class DataClassPropertyParameters extends DataClassParameters
{

    /**
     * The property of the DataClass object to be used as a parameter
     *
     * @var string
     */
    private $property;

    /**
     * Constructor
     *
     * @param $condition \common\libraries\Condition
     * @param $property string
     */
    public function __construct($condition = null, $property = array(), Joins $joins = null)
    {
        parent :: __construct($condition, $joins);
        $this->property = $property;
    }

    /**
     * Get the property of the DataClass object to be used as a parameter
     *
     * @return string
     */
    public function get_property()
    {
        return $this->property;
    }

    /**
     * Set the property of the DataClass object to be used as a parameter
     *
     * @param $property string
     */
    public function set_property($property)
    {
        $this->property = $property;
    }

    /**
     * Get an md5 representation of this object for identification purposes
     *
     * @param $hash_parts multitype:string
     * @return string
     */
    public function hash($hash_parts = array())
    {
        $hash_parts[] = $this->get_property();

        return parent :: hash($hash_parts);
    }

    /**
     * Dynamically generate a valid DataClassCountDistinctParameters object based on the type of the input parameter
     *
     * @param $parameter string|\common\libraries\Condition|\common\libraries\DataClassCountDistinctParameters
     * @return \common\libraries\DataClassCountDistinctParameters boolean
     */
    public static function generate($parameter)
    {
        // So you think you're being funny, eh? Right back at you ... you dog-blasted, ornery, no-account, long-eared
        // varmint!
        $class = self :: class_name();
        if (is_object($parameter) && $parameter instanceof $class)
        {
            return $parameter;
        }

        // If the parameter is a Condition, generate a new DataClassPropertyParameters instance using the Condition
        // provided by the context
        elseif (is_object($parameter) && $parameter instanceof Condition)
        {
            return new $class($parameter);
        }
        elseif (is_object($parameter) && $parameter instanceof Joins)
        {
            return new self(null, null, $parameter);
        }
        // If it's a string, generate a new DataClassPropertyParameters instance using the property
        // provided by the context
        elseif (is_string($parameter))
        {
            return new $class(null, $parameter);
        }
        else
        {
            static :: invalid();
        }
    }

    /**
     * Throw an exception if the DataClassPropertyParameters object is invalid
     *
     * @throws \Exception
     */
    abstract public static function invalid();
}
