<?php
namespace common\libraries;

/**
 *
 * @package common.libraries
 * @author Pieterjan Broekaert Hogent
 */
class DoctrineComparisonConditionTranslator extends DoctrineConditionTranslator
{

    public function translate()
    {
        $operator_string = $this->translate_operator($this->get_condition()->get_operator());

        // if the operator is an equality operator and no value is given, translate to 'is null' equality condition
        if ($this->get_condition()->get_operator() == ComparisonCondition :: EQUAL &&
             is_null($this->get_condition()->get_value()))
        {
            return $this->translate_equality_condition_with_empty_value($this->get_condition());
        }
        else
        {
            return $this->translate_condition($this->get_condition(), $operator_string);
        }
    }

    /**
     * Translates the operator to the correct string
     *
     * @param $condition_operator
     * @return string
     */
    private function translate_operator($condition_operator)
    {
        switch ($condition_operator)
        {
            case ComparisonCondition :: GREATER_THAN :
                $translated_operator = '>';
                break;
            case ComparisonCondition :: GREATER_THAN_OR_EQUAL :
                $translated_operator = '>=';
                break;
            case ComparisonCondition :: LESS_THAN :
                $translated_operator = '<';
                break;
            case ComparisonCondition :: LESS_THAN_OR_EQUAL :
                $translated_operator = '<=';
                break;
            case ComparisonCondition :: EQUAL :
                $translated_operator = '=';
                break;
            default :
                die('Unknown operator for Comparison condition');
        }

        return $translated_operator;
    }

    /**
     * Translates an equality condition with an empty value
     *
     * @param $condition
     * @return string
     */
    private function translate_equality_condition_with_empty_value($condition)
    {
        if ($condition->get_name() instanceof ConditionVariable)
        {
            return DoctrineConditionVariableTranslator :: render($condition->get_name(), $this->get_connection()) .
                 ' IS NULL';
        }
        else
        {
            return DoctrineDatabase :: escape_column_name(
                $condition->get_name(),
                $this->get_deprecated_storage_unit(),
                $this->get_connection()) . ' IS NULL';
        }
    }

    /**
     * Translates the (in)equalitycondition with the given operator_string
     *
     * @param $condition
     * @param $operator_string string
     * @return string
     */
    private function translate_condition($condition, $operator_string)
    {
        if ($condition->get_name() instanceof ConditionVariable)
        {
            return DoctrineConditionVariableTranslator :: render($condition->get_name(), $this->get_connection()) . ' ' .
                 $operator_string . ' ' .
                 DoctrineConditionVariableTranslator :: render($condition->get_value(), $this->get_connection());
        }
        else
        {
            $storage_unit = $this->get_deprecated_storage_unit();

            $name = $condition->get_name();
            $condition_storage_unit = $condition->get_storage_unit();

            if (! is_null($condition_storage_unit))
            {
                if ($condition->is_alias())
                {
                    $storage_unit = $condition_storage_unit;
                }
                else
                {
                    $namespace = self :: $source;
                    $data_manager = $namespace . '\DataManager';
                    if (! class_exists($data_manager))
                    {
                        $data_manager = $namespace . '\\' .
                             Utilities :: get_package_name_from_namespace($namespace, true) . 'DataManager';
                    }
                    $storage_unit = $data_manager :: get_instance()->get_alias($condition_storage_unit);
                }
            }
            return DoctrineDatabase :: escape_column_name($name, $storage_unit, $this->get_connection()) . ' ' .
                 $operator_string . ' ' .
                 DoctrineDatabase :: quote($condition->get_value(), null, true, false, $this->get_connection());
        }
    }
}
