<?php
namespace common\libraries;

/**
 *
 * @package common.libraries.storage.doctrine
 * @author Hans De Bisschop
 */
class DoctrineSubselectConditionTranslator extends DoctrineConditionTranslator
{

    public function translate()
    {
        $string = array();
        if ($this->get_condition()->get_name() instanceof ConditionVariable)
        {
            $string[] = DoctrineConditionVariableTranslator :: render(
                $this->get_condition()->get_name(),
                $this->get_connection());
        }
        else
        {
            $name_table = $this->get_condition()->get_storage_unit_name();
            $alias_name = null;
            if ($name_table)
            {
                $namespace = self :: $source;
                $data_manager = $namespace . '\DataManager';
                if (! class_exists($data_manager))
                {
                    $data_manager = $namespace . '\\' . Utilities :: get_package_name_from_namespace($namespace, true) .
                         'DataManager';
                }
                $alias_name = $data_manager :: get_instance()->get_alias($name_table);
            }
            $string[] = DoctrineDatabase :: escape_column_name(
                $this->get_condition()->get_name(),
                $alias_name,
                $this->get_connection());
        }
        $string[] = 'IN (';

        $string[] = 'SELECT';

        if ($this->get_condition()->get_value() instanceof ConditionVariable)
        {
            $string[] = DoctrineConditionVariableTranslator :: render(
                $this->get_condition()->get_value(),
                $this->get_connection());
        }
        else
        {
            $table = $this->get_condition()->get_storage_unit_value();

            $namespace = self :: $source;
            $data_manager = $namespace . '\DataManager';
            if (! class_exists($data_manager))
            {
                $data_manager = $namespace . '\\' . Utilities :: get_package_name_from_namespace($namespace, true) .
                     'DataManager';
            }
            $alias = $data_manager :: get_instance()->get_alias($table);
            $string[] = DoctrineDatabase :: escape_column_name(
                $this->get_condition()->get_value(),
                $alias,
                $this->get_connection());
        }

        $string[] = 'FROM';

        if ($this->get_condition()->get_value() instanceof ConditionVariable)
        {
            $class = $this->get_condition()->get_value()->get_class();
            $table = $class :: get_table_name();

            $namespace = Utilities :: get_namespace_from_classname($class);
            $datamanager_class = $namespace . '\\DataManager';
            $alias = $datamanager_class :: get_instance()->get_alias($table);

            $string[] = $datamanager_class :: get_instance()->escape_table_name($table);
        }
        else
        {
            $namespace = self :: $source;
            $data_manager = $namespace . '\DataManager';
            if (! class_exists($data_manager))
            {
                $data_manager = $namespace . '\\' . Utilities :: get_package_name_from_namespace($namespace, true) .
                     'DataManager';
            }

            $table = $this->get_condition()->get_storage_unit_value();

            if ($this->get_condition()->get_data_manager())
            {
                $etable = $this->get_condition()->get_data_manager()->escape_table_name($table);
            }
            else
            {
                $etable = $data_manager :: get_instance()->escape_table_name($table);
            }

            $alias = $data_manager :: get_instance()->get_alias($table);
            $string[] = $etable;
        }

        $string[] = 'AS';
        $string[] = $alias;

        if ($this->get_condition()->get_condition())
        {
            $string[] = 'WHERE ';
            $string[] = DoctrineConditionTranslator :: render(
                $this->get_condition()->get_condition(),
                $alias,
                $this->get_connection());
        }

        $string[] = ')';

        return implode(' ', $string);
    }
}
