<?php
namespace common\libraries;

use Doctrine\DBAL\DriverManager;
use Doctrine\Common\ClassLoader;

/**
 * $Id: connection.class.php 128 2009-11-09 13:13:20Z vanpouckesven $
 *
 * @package common.database
 */

/**
 * This class represents the current database connection.
 *
 * @author Hans De Bisschop
 */
class DoctrineConnection
{

    /**
     * Instance of this class for the singleton pattern.
     */
    private static $instance;

    /**
     *
     * @var Doctrine\DBAL\Connection
     */
    protected $connection;

    /**
     * Use $config = Configuration :: get_instance(); to retrieve DB settings
     *
     * @param $connection Doctrine\DBAL\Connection
     */
    private function __construct($connection = null)
    {
        $classLoader = new ClassLoader('Doctrine', Path :: get_plugin_path());
        $classLoader->register();

        if (is_null($connection))
        {
            $data_source_name = DataSourceName :: get_from_config('doctrine', Configuration :: get_instance());
            $configuration = new \Doctrine\DBAL\Configuration();
            $connection_parameters = array('dbname' => $data_source_name->get_database(),
                    'user' => $data_source_name->get_username(), 'password' => $data_source_name->get_password(),
                    'host' => $data_source_name->get_host(), 'driver' => $data_source_name->get_driver(true));
            $this->connection = DriverManager :: getConnection($connection_parameters, $configuration);
        }
        else
        {
            $this->connection = $connection;
        }

        $this->connection->setCharset('utf8');
    }

    /**
     * Returns the instance of this class.
     *
     * @return Connection The instance.
     */
    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: $instance = new self();
        }
        return self :: $instance;
    }

    public static function set_instance($connection)
    {
        self :: $instance = new self($connection);
    }

    /**
     * Gets the database connection.
     *
     * @return Doctrine\DBAL\Connection
     */
    public function get_connection()
    {
        return $this->connection;
    }

    /**
     *
     * @param $connection Doctrine\DBAL\Connection
     */
    public function set_connection($connection)
    {
        $this->connection = $connection;
    }

    public function set_option($option, $value)
    {
        $this->connection->setOption($option, $value);
    }
}
