<?php
namespace common\libraries;

/**
 *
 * @package common.libraries.storage.mdb2
 * @author Hans De Bisschop
 */
class Mdb2InConditionTranslator extends Mdb2ConditionTranslator
{

    public function translate()
    {
        $values = $this->get_condition()->get_values();

        if (! is_array($values))
        {
            if (is_scalar($values))
            {
                $values = array($values);
            }
            elseif (is_null($values))
            {
                $values = array();
            }
            else
            {
                throw new \InvalidArgumentException(
                    'An InCondition only accepts an array or a scalar as input for the values');
            }
        }

        if (count($values) > 0)
        {
            $where_clause = array();
            if ($this->get_condition()->get_name() instanceof ConditionVariable)
            {
                $where_clause[] = Mdb2ConditionVariableTranslator :: render(
                    $this->get_condition()->get_name(),
                    $this->get_connection()) . ' IN (';
            }
            else
            {
                $storage_unit = $this->get_deprecated_storage_unit();
                $condition_storage_unit = $this->get_condition()->get_storage_unit();

                if (! is_null($condition_storage_unit))
                {
                    if ($this->get_condition()->is_alias())
                    {
                        $storage_unit = $condition_storage_unit;
                    }
                    else
                    {
                        $namespace = self :: $source;
                        $data_manager = $namespace . '\DataManager';
                        if (! class_exists($data_manager))
                        {
                            $data_manager = $namespace . '\\' .
                                 Utilities :: get_package_name_from_namespace($namespace, true) . 'DataManager';
                        }
                        $storage_unit = $data_manager :: get_instance()->get_alias($condition_storage_unit);
                    }
                }
                $where_clause[] = Mdb2Database :: escape_column_name(
                    $this->get_condition()->get_name(),
                    $storage_unit,
                    $this->get_connection()) . ' IN (';
            }
            $placeholders = array();
            foreach ($values as $value)
            {
                $placeholders[] = Mdb2Database :: quote($value, null, true, false, $this->get_connection());
            }

            $where_clause[] = implode(',', $placeholders);
            $where_clause[] = ')';
            return implode('', $where_clause);
        }
        else
        {
            return 'true = false';
        }
    }
}
