<?php
namespace common\libraries;

use MDB2;
use Exception;

/**
 * $Id: connection.class.php 128 2009-11-09 13:13:20Z vanpouckesven $
 *
 * @package common.database
 */

/**
 * This class represents the current database connection.
 *
 * @author Hans De Bisschop
 */
class Mdb2Connection
{

    /**
     * Instance of this class for the singleton pattern.
     */
    private static $instance;

    /**
     * The MDB2 Connection object.
     */
    protected $connection;

    // 'debug_handler'=>array('DatabaseRepositoryDataManager','debug')

    /**
     * Constructor.
     */
    private function __construct($connection = null)
    {
        if (is_null($connection))
        {
            $this->connection = self :: connection_from_config();
        }
        else
        {
            $this->connection = $connection;
        }

        $this->connection->setOption('portability', MDB2_PORTABILITY_NONE);
        $this->connection->setCharset('utf8');
    }

    public static function connection_from_config()
    {
        $data_source_name = DataSourceName :: get_from_config('mdb2', Configuration :: get_instance());

        // The following line is for software under development, to be disabled, see below:
        $connection = MDB2 :: connect($data_source_name->get_connection_string(), array('debug' => 3));
        if (MDB2 :: isError($connection))
        {
            throw new Exception(
                    'The system can not connect to the database. If you are installing, please remove the configuration file.');
        }
        // TODO: The following line is for production systems, debugging feature is disabled:
        // $this->connection = MDB2 :: connect($configuration->get_parameter('database', 'connection_string'),
        // array('debug' => 0));

        return $connection;
    }

    /**
     * Returns the instance of this class.
     *
     * @return Connection The instance.
     */
    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: $instance = new self();
        }
        return self :: $instance;
    }

    public static function set_instance($connection)
    {
        self :: $instance = new self($connection);
    }

    /**
     * Gets the database connection.
     *
     * @return mixed MDB2 DB Conenction.
     */
    public function get_connection()
    {
        return $this->connection;
    }

    public function set_connection($connection)
    {
        $this->connection = $connection;
    }

    public function set_option($option, $value)
    {
        $this->connection->setOption($option, $value);
    }
}
