<?php
namespace common\libraries;


class RecordCache
{
    /**
     * The instance of the RecordCache
     *
     * @var \common\libraries\RecordCache
     */
    private static $instance;
    
    /**
     * The cache
     *
     * @var multitype:multitype:mixed
     */
    private $cache;

    /**
     * Get an instance of the RecordCache
     *
     * @return \common\libraries\RecordCache
     */
    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            self :: $instance = new self();
        }
        return self :: $instance;
    }

    /**
     * Get a DataClass object from the cache
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassResultParameters
     * @return boolean
     */
    public static function get(DataClassParameters $parameters)
    {
        $instance = self :: get_instance();
        
        if (self :: exists($parameters))
        {
            return $instance->cache[$parameters->hash()];
        }
        else
        {
            return false;
        }
    }

    /**
     * Returns whether a DataClass object exists in the cache
     *
     * @param $class string
     * @param $parameters \common\libraries\DataClassParameters
     * @return boolean
     */
    public static function exists(DataClassParameters $parameters)
    {
        $instance = self :: get_instance();
        
        if (isset($instance->cache[$parameters->hash()]))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Clear the cache for a specific DataClass type
     *
     * @param $class string
     * @return boolean
     */
    public static function truncate()
    {
        $instance = self :: get_instance();
        
        if (isset($instance->cache))
        {
            unset($instance->cache);
        }
        
        return true;
    }

    /**
     * Set the cache value for a specific DataClass object type, hash
     *
     * @param $class string
     * @param $hash string
     * @param $value mixed
     */
    public static function set_cache($hash, $value)
    {
        $instance = self :: get_instance();
        $instance->cache[$hash] = $value;
    }

    public static function reset()
    {
        $instance = self :: get_instance();
        $instance->cache = array();
    }
}
