<?php
namespace common\libraries;

/**
 * $Id: sub_manager.class.php 128 2009-11-09 13:13:20Z vanpouckesven $
 *
 * @package common
 */
abstract class SubManager
{

    private $parent;

    public function __construct($parent)
    {
        $this->parent = $parent;
    }

    public function get_parent()
    {
        return $this->parent;
    }

    public function get_url($parameters = array (), $filter = array(), $encode_entities = false)
    {
        return $this->get_parent()->get_url($parameters, $filter, $encode_entities);
    }

    public function get_link($parameters = array (), $filter = array(), $encode_entities = false,
        $application_type = Redirect :: TYPE_APPLICATION)
    {
        return $this->get_parent()->get_link($parameters, $filter, $encode_entities, $application_type);
    }

    public function simple_redirect($parameters = array (), $filter = array(), $encode_entities = false,
        $redirect_type = Redirect :: TYPE_URL, $application_type = Redirect :: TYPE_APPLICATION)
    {
        $this->get_parent()->simple_redirect($parameters, $filter, $encode_entities, $redirect_type, $application_type);
    }

    public function redirect($message = '', $error_message = false, $parameters = array (), $filter = array(), $encode_entities = false,
        $redirect_type = Redirect :: TYPE_URL, $application_type = Redirect :: TYPE_APPLICATION)
    {
        $this->get_parent()->redirect(
            $message,
            $error_message,
            $parameters,
            $filter,
            $encode_entities,
            $redirect_type,
            $application_type);
    }

    public function get_parameters()
    {
        return $this->get_parent()->get_parameters();
    }

    public function get_parameter($name)
    {
        return $this->get_parent()->get_parameter($name);
    }

    public function set_parameter($name, $value)
    {
        $this->get_parent()->set_parameter($name, $value);
    }

    public function display_header()
    {
        $this->get_parent()->display_header();
    }

    public function display_footer()
    {
        $this->get_parent()->display_footer();
    }

    public function small_header()
    {
        $this->get_parent()->small_header();
    }

    public function small_footer()
    {
        $this->get_parent()->small_footer();
    }

    public function display_message($message)
    {
        $this->get_parent()->display_message($message);
    }

    public function display_error_message($message)
    {
        $this->get_parent()->display_error_message($message);
    }

    public function display_warning_message($message)
    {
        $this->get_parent()->display_warning_message($message);
    }

    public function display_error_page($message)
    {
        $this->get_parent()->display_error_page($message);
    }

    public function display_warning_page($message)
    {
        $this->get_parent()->display_warning_page($message);
    }

    public function display_messages($messages, $types)
    {
        $this->get_parent()->display_messages($messages, $types);
    }

    public function get_result($failures, $count, $fail_message_single, $fail_message_multiple, $succes_message_single,
        $succes_message_multiple)
    {
        return $this->get_parent()->get_result(
            $failures,
            $count,
            $fail_message_single,
            $fail_message_multiple,
            $succes_message_single,
            $succes_message_multiple);
    }

    public function get_general_result($failures, $count, $single_object, $multiple_object,
        $type = Application :: RESULT_TYPE_CREATED)
    {
        return $this->get_parent()->get_general_result($failures, $count, $single_object, $multiple_object, $type);
    }

    public function not_allowed($show_login_form = true)
    {
        $this->get_parent()->not_allowed($show_login_form);
    }

    /**
     *
     * @return int
     */
    public function get_user_id()
    {
        return $this->get_parent()->get_user_id();
    }

    /**
     *
     * @return User
     */
    public function get_user()
    {
        return $this->get_parent()->get_user();
    }

    public function get_action()
    {
        return $this->get_parent()->get_action();
    }

    public function set_action($action)
    {
        return $this->get_parent()->set_action($action);
    }

    public function get_application_name()
    {
        return $this->get_parent()->get_application_name();
    }

    public function create_component($type, $application = null)
    {
        if ($application == null)
        {
            $application = $this;
        }

        return $this->get_parent()->create_component($type, $application);
    }

    /**
     *
     * @return string NULL
     */
    public function process_table_action($action_parameter)
    {
        $table_name = Request :: post('table_name');
        if (isset($table_name))
        {
            $namespace = Request :: post($table_name . '_namespace');
            $class = $namespace . '\\' . Utilities :: underscores_to_camelcase($table_name);
            if (class_exists($class))
            {
                call_user_func(array($class, 'handle_table_action'));

                $table_action_name = Request :: post($table_name . '_action_name');
                $table_action_value = Request :: post($table_name . '_action_value');
                Request :: set_get($table_action_name, $table_action_value);

                if ($table_action_name == $action_parameter)
                {
                    return $table_action_value;
                }
            }
        }

        return null;
    }

    /**
     *
     * @param $sub_manager_class string
     * @param $action string
     * @return string
     */
    private static function load_class($action)
    {
        $component_class = static :: context() . '\\' . Utilities :: underscores_to_camelcase($action) . 'Component';

        if (! class_exists($component_class))
        {
            $component_class = static :: context() . '\\' .
                 Utilities :: get_classname_from_namespace(static :: class_name()) .
                 Utilities :: underscores_to_camelcase($action) . 'Component';
            if (! class_exists($component_class))
            {
                $message = array();
                $message[] = Translation :: get('ComponentFailedToLoad') . '<br /><br />';
                $message[] = '<b>' . Translation :: get('ComponentClass') . ':</b><br />';
                $message[] = $component_class . '<br /><br />';
                $message[] = '<b>' . Translation :: get('Stacktrace') . ':</b>';
                $message[] = '<ul>';
                $message[] = '<li>' . static :: class_name() . '</li>';
                $message[] = '<li>' . $action . '</li>';
                $message[] = '</ul>';

                BreadcrumbTrail :: get_instance()->add(new Breadcrumb('#', Translation :: get(static :: class_name())));

                Display :: header();
                Display :: error_message(implode("\n", $message));
                Display :: footer();
                exit();
            }
        }
        return $component_class;
    }

    /**
     *
     * @param $sub_manager_class string
     * @param $action string
     * @param $application Application
     * @return SubManager
     */
    private static function component($action, $application)
    {
        $class = static :: load_class($action);
        return new $class($application);
    }

    /**
     *
     * @param $sub_manager_class string
     * @param $action_parameter string
     * @return string
     */
    public static function get_component_action($action_parameter)
    {
        $default_action = call_user_func(array(static :: class_name(), 'get_default_action'));

        $action = Request :: get($action_parameter);
        $action = ! isset($action) ? $default_action : $action;
        $table_action = self :: process_table_action($action_parameter);
        if ($table_action)
        {
            $action = $table_action;
        }

        return $action;
    }

    /**
     *
     * @param $sub_manager_class string
     * @param $application Application
     */
    public static function construct($application, $add_breadcrumb = true)
    {
        $action_parameter = call_user_func(array(static :: class_name(), 'get_action_parameter'));
        $action = static :: get_component_action($action_parameter);

        $component = static :: component($action, $application);
        $component->set_parameter($action_parameter, $action);

        $context = Utilities :: get_namespace_from_object($component);

        $trail = BreadcrumbTrail :: get_instance();

        $component->add_additional_breadcrumbs($trail);

        $parameters = $component->get_additional_parameters();
        foreach ($parameters as $parameter)
        {
            $component->set_parameter($parameter, Request :: get($parameter));
        }

        if ($add_breadcrumb && ! $component instanceof DelegateComponent)
        {
            $trail->add_help($context, Utilities :: get_classname_from_object($component, true));
            $trail->add(
                new Breadcrumb(
                    $component->get_url(),
                    Translation :: get(Utilities :: get_classname_from_object($component), null, $context)));
        }

        return $component;
    }

    // Dummy functions for breadcrumbtrail
    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
    }

    public function get_additional_parameters()
    {
        return array();
    }

    /**
     *
     * @param $sub_manager_class string
     * @param $application Application
     */
    public static function launch($sub_manager_class, $application, $add_breadcrumb = true)
    {
        static :: construct($application, $add_breadcrumb)->run();
    }

    public static function class_name()
    {
        return get_called_class();
    }

    public static function context()
    {
        return Utilities :: get_namespace_from_classname(get_called_class());
    }
}
