<?php

namespace common\libraries;

/**
 * Basic cache using the file system to store values.
 *
 */
class FileCache
{

    private $root = '';
    private $serialize = false;

    /**
     *
     * @param string $root      the root of the cache, i.e. the relative path where content will be stored
     * @param bool $serialize   if true content is serialized, if false only string content is expected
     */
    public function __construct($root = '', $serialize = false)
    {
        $this->root = Path :: get(SYS_PATH) . '/files/cache/' . $root . '/';
        $this->serialize = $serialize;
    }

    /**
     * If the resource is available in the cache send it and exit.
     *
     * @param string $key
     * @param int $time
     * @param bool $caching
     */
    public function prolog($key, $time = null, $caching = true)
    {
        if (! $caching)
        {
            return;
        }
        if (! $this->has($key))
        {
            return;
        }

        $this->read($key);
        exit();
    }

    public function is_fresh($key, $modification_time)
    {
        if(!file_exists($this->path($key)))
        {
            return false;
        }

        return filemtime($this->path($key)) >= $modification_time;
    }

    public function is_stale($key, $modification_time)
    {
        return ! $this->is_fresh($key, $modification_time);
    }

    public function get_root()
    {
        return $this->root;
    }

    public function path($key)
    {
        return $this->get_root() . $key;
    }

    public function reset()
    {
        $this->clear();
    }

    public function clear()
    {
        Filesystem :: remove($this->get_root());
    }

    public function remove($key)
    {
        $path = $this->path($key);
        if (is_readable($path))
        {
            Filesystem :: remove($this->path($key));
        }
    }

    public function has($key)
    {
        return is_readable($this->path($key));
    }

    public function get($key)
    {
        if (! $this->has($key))
        {
            return false;
        }

        $value = file_get_contents($this->path($key));
        if ($this->serialize)
        {
            $value = unserialize($value);
        }
        return $value;
    }

    public function __invoke($key)
    {
        return $this->get($key);
    }

    /**
     * Read the file and output it directly.
     * @param string $key
     * @return bool
     */
    public function read($key)
    {
        if (! $this->has($key))
        {
            return false;
        }

        if (! $this->serialize)
        {
            readfile($this->path($key));
            return true;
        }
        echo $this->get($key);
        return true;
    }

    public function set($key, $value)
    {
        if ($this->serialize)
        {
            $value = serialize($value);
        }
        Filesystem :: write_to_file($this->path($key), $value);
    }

}
