<?php

namespace common\libraries;

/**
 * Abstract class that tests the creation / update and deletion of a data class extension
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
abstract class DataClassCRUDTest extends Test
{
    /****************************************************************************************************************
    * Test functionality                                                                                            *
    ****************************************************************************************************************/
    
    /**
     * Tests the creation of an object
     */
    public function test_create()
    {
        $object = $this->get_object();
        
        $this->fill_object_with_fake_data($object);
        $this->assertTrue($object->create());
        
        return $object;
    }
    
    /**
     * Tests the update of an object
     * 
     * @param DataClass $object
     * 
     * @depends test_create
     */
    public function test_update(Dataclass $object)
    {
        $this->fill_object_with_fake_data($object);
        $this->assertTrue($object->update());
    }
    
    /**
     * Tests the delete of a request object
     * 
     * @param DataClass $object
     * 
     * @depends test_create
     */
    public function test_delete(Dataclass $object)
    {
        $this->assertTrue($object->delete());
    }
    
    /****************************************************************************************************************
    * Helper functionality                                                                                          *
    ****************************************************************************************************************/
    
    /**
     * Fills an object with fake data which will be generated from the defined properties / types in the given xml
     * file
     * 
     * @param DataClass $object
     */
    protected function fill_object_with_fake_data(DataClass $object)
    {
        $filename = $object->get_table_name() . '.xml';
        $xml_path = Path :: namespace_to_full_path(Utilities :: get_namespace_from_classname(get_class($this)));
        $xml_path .= '/../php/package/install/' . $filename;

        $dom_document = new \DOMDocument();
        $dom_document->load($xml_path);
        
        $dom_xpath = new \DOMXPath($dom_document);
        
        $properties = $dom_xpath->query('/object/properties/property');
        foreach($properties as $property)
        {
            $type = $property->getAttribute('type');
            $name = $property->getAttribute('name');
            
            if($name == 'id')
            {
                continue;
            }
            
            $object->set_default_property($name, $this->get_fake_data_for_type($type));
        }
        
        return $object;
    }
    
    /**
     * Generates fake data for a given type
     * 
     * @param string $type
     */
    protected function get_fake_data_for_type($type)
    {
        switch($type)
        {
            case 'text': return md5(uniqid());
            case 'integer': return rand(0, 100);
            case 'boolean': return (rand(0, 1) == 0);
        }
    }
    
    /****************************************************************************************************************
    * Abstract functionality                                                                                        *
    ****************************************************************************************************************/
    
    /**
     * Returns an instance of the dataclass extension
     * 
     * @return DataClass
     */
    abstract protected function get_object();
}
