<?php
namespace common\libraries;

/**
 * This abstract test case is used for all the testcases of dataclasses. It tests the default property names function.
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
abstract class DataClassTest extends Test
{
    /****************************************************************************************************************
    * Test functionality                                                                                            *
    ****************************************************************************************************************/

    /**
     * Tests the get default property names method
     */
    public function test_get_default_property_names()
    {
        $data_class_object = $this->get_data_class_object();

        $this->assertEquals(
            $this->count_constants($data_class_object, 'PROPERTY'), 
            count($data_class_object->get_default_property_names())
        );
    }
    
    /**
     * Tests the getters for each property in the dataclass
     * 
     * @string $constant
     * 
     * @dataProvider property_tests_data_provider
     */
    public function test_property_getter($constant)
    {
        $data_class_object = $this->get_data_class_object();
        $method = $this->prepare_property_test($constant, 'get');
        
        $test_value = uniqid();
        
        $data_class_object->set_default_property($constant, $test_value);
        $this->assert_property_test($constant, $test_value, call_user_method($method, $data_class_object), 'get');
    }
    
    /**
     * Tests the setters for each property in the dataclass
     * 
     * @string $constant
     * 
     * @dataProvider property_tests_data_provider
     */
    public function test_property_setter($constant)
    {
        $data_class_object = $this->get_data_class_object();
        $method = $this->prepare_property_test($constant, 'set');
                
        $test_value = uniqid();
        
        call_user_method($method, $data_class_object, $test_value);
        $this->assert_property_test($constant, $test_value, $data_class_object->get_default_property($constant), 'set');
    }
    
    /****************************************************************************************************************
    * DataProvider functionality                                                                                    *
    ****************************************************************************************************************/
    
    /**
     * Dataprovider for the property tests
     */
    public function property_tests_data_provider()
    {
        $data = array();
        
        $constants = $this->get_constants($this->get_data_class_object(), 'PROPERTY');
        foreach($constants as $constant)
        {
            $data[] = array($constant);
        }
        
        return $data;
    }
    
    /****************************************************************************************************************
    * Helper functionality                                                                                          *
    ****************************************************************************************************************/
    
    /**
     * Prepares a property test (getter or setter) and returns the method to call
     * 
     * @param string $constant
     * @param string $type
     * 
     * @return string
     */
    protected function prepare_property_test($constant, $type)
    {
        $data_class_object = $this->get_data_class_object();
        $method = $type . '_' . $constant;
        
        if(!method_exists($data_class_object, $method))
        {
            $this->fail('Could not find ' . $type . 'ter method for property ' . $constant);
        }
        
        return $method;
    }
    
    /**
     * Create an assert for the property tests
     * 
     * @param string $constant
     * @param string $test_value
     * @param string $result
     * @param string $type
     */
    protected function assert_property_test($constant, $test_value, $result, $type)
    {
        $this->assertEquals(
            $test_value, 
            $result,
            'Testing ' . $type . 'ter for ' . get_class($this->get_data_class_object()) . ':' . $constant  
        );
    }
    
    /****************************************************************************************************************
    * Abstract functionality                                                                                        *
    ****************************************************************************************************************/

    /**
     * Returns the object for the current tested dataclass
     *
     * @return DataClass
     */
    abstract protected function get_data_class_object();
}
