<?php
namespace common\libraries;

/**
 *
 * @package common.libraries
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
abstract class WebApplication extends Application
{
    const CLASS_NAME = __CLASS__;
    const ATTRIBUTES_TYPE_USER = 1;
    const ATTRIBUTES_TYPE_OBJECT = 2;

    public static $application_path_cache = array();

    /**
     * Determines whether the given learning object cab be edited in this application.
     *
     * @param int $object_id
     * @return boolean
     */
    public static function is_content_object_editable($object_id)
    {
        return true;
    }

    /**
     * Determines whether the given learning object has been published in this application.
     *
     * @param int $object_id
     * @return boolean
     */
    public static function content_object_is_published($object_id)
    {
        return false;
    }

    /**
     * Determines whether any of the given learning objects has been published in this application.
     *
     * @param multitype:int $object_ids
     * @return boolean
     */
    public static function any_content_object_is_published($object_ids)
    {
        return false;
    }

    /**
     * Returns attributes for content object publications
     *
     * @param int $type
     * @param int $object_id
     * @param Condition $condition
     * @return multitype:mixed
     */
    public static function get_content_object_publication_attributes(
                $object_id, $type = self :: ATTRIBUTES_TYPE_OBJECT, $condition = null,
                $count = null, $offset = null, $order_properties = null)
    {
        return array();
    }

    /**
     * Determines where in this application the given learning object publication is published.
     *
     * @param int $publication_id
     * @return ContentObjectPublicationAttributes
     */
    public static function get_content_object_publication_attribute($publication_id)
    {
        return null;
    }

    /**
     * Counts the number of content object publications
     *
     * @param int $attributes_type
     * @param int $identifier
     * @param \common\libraries\Condition $condition
     * @return int
     */
    public static function count_publication_attributes($attributes_type = self :: ATTRIBUTES_TYPE_OBJECT, $identifier, $condition = null)
    {
        return 0;
    }

    /**
     * Deletes all publications of a given learning object
     *
     * @param int $object_id
     * @return boolean
     */
    public static function delete_content_object_publications($object_id)
    {
        return true;
    }

    /**
     *
     * @param int $publication_id
     * @return boolean
     */
    public static function delete_content_object_publication($publication_id)
    {
        return true;
    }

    /**
     *
     * @param \repository\ContentObject $content_object
     * @return multitype:mixed
     */
    public static function get_content_object_publication_locations($content_object)
    {
        return array();
    }

    /**
     *
     * @param \repository\ContentObject $content_object
     * @param mixed $location
     * @param multitype:mixed $attributes
     * @return NULL
     */
    public static function publish_content_object($content_object, $location, $attributes)
    {
        return null;
    }

    /**
     *
     * @param mixed $form
     */
    public static function add_publication_attributes_elements($form)
    {
    }

    /**
     *
     * @param boolean $include_application_classes
     * @return multitype: string
     */
    public static function load_all_from_filesystem($include_application_classes = true,
        $only_registered_applications = false)
    {
        $applications = array();
        $path = Path :: get_application_path();

        $directories = Filesystem :: get_directory_content($path, Filesystem :: LIST_DIRECTORIES, false);

        foreach ($directories as $directory)
        {
            $application_name = basename($directory);
            if ($only_registered_applications && ! CommonDataManager :: is_registered(
                'application\\' . $application_name))
            {
                continue;
            }

            if (Application :: is_application_name($application_name))
            {
                if (! in_array($application_name, $applications))
                {
                    $applications[] = $application_name;
                }
            }
        }
        return $applications;
    }

    /**
     *
     * @return multitype:string
     */
    public static function load_all()
    {
        $applications = CommonDataManager :: get_registrations_by_type(Registration :: TYPE_APPLICATION);

        $active_applications = array();

        foreach ($applications as $application)
        {
            if (! $application->is_active())
            {
                continue;
            }

            $active_applications[] = $application->get_name();
        }
        return $active_applications;
    }

    /**
     *
     * @param string $application_name
     * @return string
     */
    public static function get_application_namespace($application_name)
    {
        return 'application\\' . $application_name;
    }

    /**
     *
     * @param \user\User $user
     * @return NULL
     */
    public static function get_additional_user_information($user)
    {
        return null;
    }

    /**
     *
     * @param string $application
     * @return boolean
     */
    public static function exists($application)
    {
        if (! isset(self :: $application_path_cache[$application]))
        {
            $application_path = Path :: namespace_to_full_path(self :: get_application_namespace($application));
            self :: $application_path_cache[$application] = is_dir($application_path);
        }
        return self :: $application_path_cache[$application];
    }

    /**
     *
     * @return multitype:string
     */
    public static function get_packages_from_filesystem()
    {
        $applications = array();

        $directories = Filesystem :: get_directory_content(
            Path :: get_application_path(),
            Filesystem :: LIST_DIRECTORIES,
            false);

        foreach ($directories as $directory)
        {
            $application_name = basename($directory);

            if (Application :: is_application_name($application_name))
            {
                $applications[] = self :: determine_namespace($application_name);
            }
        }

        return $applications;
    }

    /**
     *
     * @param boolean $include_installed
     * @return \common\libraries\package\PackageList
     */
    public static function get_installable_packages($include_installed = false)
    {
        $package_list = new \common\libraries\package\PackageList(
            'application',
            Translation :: get('Application', null, Utilities :: COMMON_LIBRARIES));

        foreach (self :: get_packages_from_filesystem() as $application)
        {
            $class = $application . '\Manager';

            if (! CommonDataManager :: get_registration($application) || $include_installed)
            {
                $package_list->add_package($application);
            }

            if (! class_exists($class))
            {
                $class = $application . '\\' . Utilities :: underscores_to_camelcase(
                    Utilities :: get_package_name_from_namespace($application)) . 'Manager';
            }

            if (class_exists($class))
            {
                $application_package_list = $class :: get_installable_application_packages($include_installed);
                if ($application_package_list instanceof \common\libraries\package\PackageList)
                {
                    $package_list->add_child($application_package_list);
                }
            }
        }

        return $package_list;
    }

    /**
     *
     * @param boolean $include_installed
     * @return boolean
     */
    public static function get_installable_application_packages($include_installed = false)
    {
        return false;
    }
}
