<?php
namespace common\libraries;

/**
 * Base class for specific installation extensions of web applications
 *
 * @author Hans De Bisschop
 */
class WebApplicationInstaller extends \common\libraries\package\Installer
{

    /**
     * Always add a menu-item for a web application. Don't forget to call this method via parent :: extra() in
     * application installers that implement additional logic for this method.
     *
     * @return boolean
     */
    public function extra()
    {
        $context = $this->context();

        $registration = CommonDataManager :: get_registration('core\menu');
        if (! $registration instanceof Registration || ! $registration->is_active())
        {
            return true;
        }

        // Determine whether the category already exists, if not create it
        $info = \common\libraries\package\Package :: get($context);

        $category_name = Translation :: get(
            Utilities :: underscores_to_camelcase($info->get_category()),
            null,
            self :: context(),
            Translation :: get_instance()->get_language());

        $conditions = array();
        $conditions[] = new EqualityCondition(\core\menu\ItemTitle :: PROPERTY_TITLE, $category_name);
        $conditions[] = new EqualityCondition(
            \core\menu\ItemTitle :: PROPERTY_ISOCODE,
            Translation :: get_instance()->get_language());

        $parameters = new DataClassDistinctParameters(
            new AndCondition($conditions),
            \core\menu\ItemTitle :: PROPERTY_ITEM_ID);
        DataClassResultCache :: truncate(\core\menu\ItemTitle :: class_name());
        $titles = \core\menu\DataManager :: distinct(\core\menu\ItemTitle :: class_name(), $parameters);

        if (count($titles) > 0)
        {
            $conditions = array();
            $conditions[] = new EqualityCondition(
                \core\menu\Item :: PROPERTY_TYPE,
                \core\menu\CategoryItem :: class_name());
            $conditions[] = new InCondition(\core\menu\Item :: PROPERTY_ID, $titles);

            $category = \core\menu\DataManager :: retrieve(
                \core\menu\CategoryItem :: class_name(),
                new AndCondition($conditions));
        }
        else
        {

            $languages = CommonDataManager :: get_languages();
            $category = new \core\menu\CategoryItem();
            $category->set_parent(0);

            $item_titles = new \core\menu\ItemTitles();
            foreach ($languages as $isocode => $language)
            {
                $item_title = new \core\menu\ItemTitle();
                $item_title->set_title(
                    Translation :: get(
                        Utilities :: underscores_to_camelcase($info->get_category()),
                        null,
                        self :: context(),
                        $isocode));
                $item_title->set_isocode($isocode);
                $item_titles->add($item_title);
            }

            $category->set_titles($item_titles);
            if (! $category->create())
            {
                return false;
            }
        }

        // Create the actual menu item
        $item = new \core\menu\ApplicationItem();
        $item_title = new \core\menu\ItemTitle();
        $item_title->set_title(Translation :: get('TypeName', null, $context));
        $item_title->set_isocode(Translation :: get_instance()->get_language());
        $item_titles = new \core\menu\ItemTitles(new ArrayResultSet(array($item_title)));

        $item->set_titles($item_titles);
        $item->set_application(Utilities :: get_package_name_from_namespace($context));
        $item->set_parent($category->get_id());
        $item->set_use_translation(1);

        // DataClassResultCache :: truncate(Item :: class_name());

        if (! $item->create())
        {
            return false;
        }

        return true;
    }
}
