<?php

namespace common\libraries;

/**
 * Abstract class to define the business logic of the retrieval of an object (data class) in the database
 * @author Sven Vanpoucke
 */
abstract class GetObjectWebserviceHandler extends WebserviceHandler
{

    /**
     * Retrieves one object
     */
    public function run()
    {
        $object = $this->get_object();
        $id = $this->get_id();
        $translation = $this->get_object_translation();
        
        if (! $object)
        {
            return new SuccessRestMessage(false, Translation :: get('ObjectNotFoundWithId', array(
                    'OBJECT' => $translation, 'ID' => $id), Utilities :: COMMON_LIBRARIES));
        }
        
        if (! $this->is_allowed_to_retrieve_object($object))
        {
            return new SuccessRestMessage(false, Translation :: get('NoRightsForObject', array(
                    'OBJECT' => $translation, 'ID' => $id), Utilities :: COMMON_LIBRARIES));
        }
        
        return $object;
    }

    abstract public function get_object();

    abstract public function get_object_translation();

    abstract public function is_allowed_to_retrieve_object($object);

}
