﻿/*
Copyright (c) 2003-2010, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.html or http://ckeditor.com/license
*/

(function()
{
	// Load image preview.
	var IMAGE = 1,
		CLEANUP = 8,
		regexGetSize = /^\s*(\d+)((px)|\%)?\s*$/i,
		regexGetSizeOrEmpty = /(^\s*(\d+)((px)|\%)?\s*$)|^$/i,
		pxLengthRegex = /^\d+px$/;

	

	// Custom commit dialog logic, where we're intended to give inline style
	// field (txtdlgGenStyle) higher priority to avoid overwriting styles contribute
	// by other fields.
	function commitContent()
	{
		var args = arguments;
		var inlineStyleField = this.getContentElement( 'advanced', 'txtdlgGenStyle' );
		inlineStyleField && inlineStyleField.commit.apply( inlineStyleField, args );

		this.foreach( function( widget )
		{
			if ( widget.commit &&  widget.id != 'txtdlgGenStyle' )
				widget.commit.apply( widget, args );
		});
	}

	// Avoid recursions.
	var incommit;

	// Synchronous field values to other impacted fields is required, e.g. border
	// size change should alter inline-style text as well.
	function commitInternally( targetFields )
	{
		if ( incommit )
			return;

		incommit = 1;

		var dialog = this.getDialog(),
			element = dialog.imageElement;
		if ( element )
		{
			// Commit this field and broadcast to target fields.
			this.commit( IMAGE, element );

			targetFields = [].concat( targetFields );
			var length = targetFields.length,
				field;
			for ( var i = 0; i < length; i++ )
			{
				field = dialog.getContentElement.apply( dialog, targetFields[ i ].split( ':' ) );
				// May cause recursion.
				field && field.setup( IMAGE, element );
			}
		}

		incommit = 0;
	}

	var imageDialog = function( editor )
	{
		return {
			title : editor.lang.image.title,
			minWidth : 420,
			minHeight : 310,
			onShow : function()
			{
				this.imageElement = false;

				// Default: create a new element.
				this.imageEditMode = false;

				this.firstLoad = true;

				var editor = this.getParentEditor(),
					sel = this.getParentEditor().getSelection(),
					element = sel.getSelectedElement();

				// Copy of the image
				this.originalElement = editor.document.createElement( 'img' );
				this.originalElement.setCustomData( 'isReady', 'false' );

				
			},
			onOk : function()
			{
				
				// Image dialog -> create IMG element.
				this.imageElement = editor.document.createElement( 'resource' );
				
			
				// Set attributes.
				this.commitContent( IMAGE, this.imageElement );

				// Remove empty style attribute.
				if ( !this.imageElement.getAttribute( 'style' ) )
					this.imageElement.removeAttribute( 'style' );

				// Insert a new Image.
				if ( !this.imageEditMode )
				{
					editor.insertElement( this.imageElement );
				}
				else		// Image already exists.
				{
					editor.insertElement( this.imageElement );
				}
			},
			onLoad : function()
			{
				var doc = this._.element.getDocument();

				this.commitContent = commitContent;
			},
			contents : [
				{
					id : 'info',
					label : editor.lang.image.infoTab,
					accessKey : 'I',
					elements :
					[
						{
							type : 'vbox',
							padding : 0,
							children :
							[
								{
									type : 'hbox',
									widths : [ '280px', '110px' ],
									align : 'right',
									children :
									[
										{
											id : 'txtUrl',
											type : 'text',
											label : editor.lang.common.url,
											hidden : true,
											required: true,
											
											setup : function( type, element )
											{
												if ( type == IMAGE )
												{
													var url = element.getAttribute( '_cke_saved_source' ) || element.getAttribute( 'source' );
													var field = this;

													this.getDialog().dontResetSize = true;

													field.setValue( url );		// And call this.onChange()
													// Manually set the initial value.(#4191)
													field.setInitValue();
													field.focus();
												}
											},
											commit : function( type, element )
											{
												if ( type == IMAGE && ( this.getValue() || this.isChanged() ) )
												{
													element.setAttribute( '_cke_saved_source', decodeURI( this.getValue() ) );
													element.setAttribute( 'source', decodeURI( this.getValue() ) );
												}
												else if ( type == CLEANUP )
												{
													element.setAttribute( 'source', '' );	// If removeAttribute doesn't work.
													element.removeAttribute( 'source' );
												}
											},
											validate : CKEDITOR.dialog.validate.notEmpty( editor.lang.image.urlMissing )
										},
										{
											type : 'button',
											id : 'browse',
											// v-align with the 'txtUrl' field.
											// TODO: We need something better than a fixed size here.
											style : 'display:inline-block;margin-top:10px;',
											align : 'center',
											label : editor.lang.common.browseServer,
											hidden : true,
//											filebrowser : 'info:txtUrl'
										}
									]
								}
							]
						}
					]
				}
			]
		};
	};

	function addQueryString( url, params )
	{
		var queryString = [];

		if ( !params )
			return url;
		else
		{
			for ( var i in params )
				queryString.push( i + "=" + encodeURIComponent( params[ i ] ) );
		}

		return url + ( ( url.indexOf( "?" ) != -1 ) ? "&" : "?" ) + queryString.join( "&" );
	}
	
	CKEDITOR.dialog.add( 'chamilo', function( editor )
		{	
			var width = editor.config[ 'filebrowserChamiloWindowWidth' ]
					|| editor.config.filebrowserWindowWidth || '80%';
			var height = editor.config[ 'filebrowserChamiloWindowHeight' ]
					|| editor.config.filebrowserWindowHeight || '70%';
	
			var params = {};
			params.CKEditor = editor.name;
			params.CKEditorFuncNum = editor._.filebrowserFn;
			if ( !params.langCode )
				params.langCode = editor.langCode;
	
			var url = addQueryString( editor.config['filebrowserChamiloBrowseUrl'], params );
			editor.popup( url, width, height );
			return imageDialog( editor );
		});

})();
