/**
 * Copyright (c) 2010, Sven Vanpoucke, Chamilo tree menu in jQuery plugin
 */

(function($) {
    $.fn.extend({
	tree_menu : function(options) {

	    // Settings list and the default values
	    var defaults = {
		search : '',
		item_url : '#',
		collapsed : true
	    };

	    var settings = $.extend(defaults, options), self = $(this);

	    function collapseItem(e) {
		$("ul:first", $(this).parent()).hide();
		if ($(this).hasClass("lastCollapse")) {
		    $(this).removeClass("lastCollapse");
		    $(this).addClass("lastExpand");
		} else if ($(this).hasClass("collapse")) {
		    $(this).removeClass("collapse");
		    $(this).addClass("expand");
		}
	    }

	    function expandItem(e) {
		$("ul:first", $(this).parent()).show();
		changeExpandItemIcon($(this));
	    }

	    function changeExpandItemIcon(item) {
		if (item.hasClass("lastExpand")) {
		    item.removeClass("lastExpand");
		    item.addClass("lastCollapse");
		} else if (item.hasClass("expand")) {
		    item.removeClass("expand");
		    item.addClass("collapse");
		}
	    }

	    function expandItemAndLoadChildren(e) {
		var id = $('a', $(this)).attr("id");
		var parent = $(this).parent();
		var children = getChildren(id);

		if (children) {
		    parent.append(children);
		    changeExpandItemIcon($(this));
		    $(this).unbind('click');
		    processTree($(this).parent().parent().parent());
		}
	    }

	    function getChildren(parent_id) {
		if (settings.search == '') {
		    return '';
		}

		var ul = $('<ul></ul>');
		var response = loadChildren(parent_id);
		var tree = $.xml2json(response, true);

		if ((tree.leaf && $(tree.leaf).size() > 0)) {
		    $.each(tree.leaf, function(i, the_leaf) {
			var expand = '';
			if (the_leaf.has_children == '1') {
			    expand = ' class="expand"';
			}
			item_url = sprintf(settings.item_url, the_leaf.id);
			var li = $('<li><div' + expand + '><a href="' + item_url + '" id="' + the_leaf.id + '" class="'
				+ the_leaf.classes + '" title="' + the_leaf.title + '">' + the_leaf.title
				+ '</a></div></li>');
			$(ul).append(li);
		    });

		    return ul;
		}
	    }

	    function loadChildren(parent_id) {
		var response = $.ajax({
		    type : "GET",
		    dataType : "xml",
		    url : settings.search,
		    data : {
			parent_id : parent_id
		    },
		    async : false
		}).responseText;

		return response;
	    }

	    function processTree(parent_element) {
		$("ul li:last-child > div", parent_element).addClass("last");
		$("ul li:last-child > div.expand", parent_element).addClass("lastExpand");
		$("ul li:last-child > div.expand", parent_element).removeClass("expand");
		$("ul li:last-child > ul", parent_element).css("background-image", "none");

		$("ul li:not(:last-child):has(ul) > div", parent_element).addClass("collapse");
		$("ul li:last-child:has(ul) > div", parent_element).addClass("lastCollapse");

		// $("ul li:has(ul) > div", parent).toggle(collapseItem,
		// expandItem);
		$("ul li > div > a", parent_element).click(function(e) {
		    e.stopPropagation();
		});
	    }

	    // Apply some default collapsing in case we're not using an AJAX
	    // postback.
	    function collapseDefault() {
		if (settings.search == '') {
		    var id = self.attr('id');

		    $("#" + id + " ul li:has(ul) > div.collapse").trigger('click');
		    $("#" + id + " ul li:has(ul) > div.lastCollapse").trigger('click');

		    $("#" + id + " ul li.current > div.expand").trigger('click');
		    $("#" + id + " ul li.current > div.lastExpand").trigger('click');

		    $("#" + id + " ul li.current").parents('li.current_path').each(function(index) {
			$("div:first", $(this)).trigger('click');
		    });
		}
	    }

	    function init() {
		processTree(self);

		// We need to select through the id of the container because of
		// a bug in jQuery where it is not possible to do DOM traversel
		// with the .live() function.
		var id = self.attr('id');
		$("#" + id + " ul li:not(:has(ul)) > div.expand").live('click', expandItemAndLoadChildren);
		$("#" + id + " ul li:not(:has(ul)) > div.lastExpand").live('click', expandItemAndLoadChildren);
		$("#" + id + " ul li:has(ul) > div.expand").live('click', expandItem);
		$("#" + id + " ul li:has(ul) > div.lastExpand").live('click', expandItem);
		$("#" + id + " ul li:has(ul) > div.collapse").live('click', collapseItem);
		$("#" + id + " ul li:has(ul) > div.lastCollapse").live('click', collapseItem);

		if (settings.collapsed)
		{
		    collapseDefault();
		}
	    }

	    return this.each(init);
	}
    });
})(jQuery);
