<?php
namespace core\admin\announcement;

use common\libraries\Display;
use common\libraries\ActionBarRenderer;
use repository\ContentObjectRendition;
use repository\ContentObjectRenditionImplementation;
use common\libraries\Theme;
use common\libraries\Utilities;
use common\libraries\Breadcrumb;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\NoContextComponent;

class ViewerComponent extends Manager implements NoContextComponent
{

    private $action_bar;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $id = Request :: get(self :: PARAM_SYSTEM_ANNOUNCEMENT_ID);
        
        if (! Rights :: get_instance()->is_allowed_in_publciation($id, $this->get_user()->get_id()))
        {
            $this->display_header();
            echo Display :: error_message(Translation :: get('NotAllowed'), true);
            $this->display_footer();
            exit();
        }
        
        if ($id)
        {
            $publication = DataManager :: retrieve_by_id(Publication :: class_name(), (int) $id);
            $object = $publication->get_content_object();
            
            $this->display_header();
            echo $this->get_action_bar($publication)->as_html();
            echo ContentObjectRenditionImplementation :: launch(
                $object, 
                ContentObjectRendition :: FORMAT_HTML, 
                ContentObjectRendition :: VIEW_FULL, 
                $this);
            $this->display_footer();
        }
        else
        {
            $this->display_error_page(
                htmlentities(
                    Translation :: get(
                        'NoObjectSelected', 
                        array('OBJECT' => Translation :: get('SystemAnnouncement')), 
                        Utilities :: COMMON_LIBRARIES)));
        }
    }

    public function get_action_bar($publication)
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        
        if ($this->get_user()->is_platform_admin() || $publication->get_publisher_id() == $this->get_user()->get_id())
        {
            
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('Edit', array(), Utilities :: COMMON_LIBRARIES), 
                    Theme :: get_common_image_path() . 'action_edit.png', 
                    $this->get_url(
                        array(
                            self :: PARAM_ACTION => self :: ACTION_EDIT, 
                            self :: PARAM_SYSTEM_ANNOUNCEMENT_ID => $publication->get_id())), 
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('Delete', array(), Utilities :: COMMON_LIBRARIES), 
                    Theme :: get_common_image_path() . 'action_delete.png', 
                    $this->get_url(
                        array(
                            self :: PARAM_ACTION => self :: ACTION_DELETE, 
                            self :: PARAM_SYSTEM_ANNOUNCEMENT_ID => $publication->get_id())), 
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));
            
            if ($publication->is_hidden())
            {
                $visibility_img = 'action_invisible.png';
            }
            elseif ($publication->is_forever())
            {
                $visibility_img = 'action_visible.png';
            }
            else
            {
                $visibility_img = 'action_period.png';
            }
            
            $action_bar->add_common_action(
                new ToolbarItem(
                    Translation :: get('Hide', array(), Utilities :: COMMON_LIBRARIES), 
                    Theme :: get_common_image_path() . $visibility_img, 
                    $this->get_url(
                        array(
                            self :: PARAM_ACTION => self :: ACTION_HIDE, 
                            self :: PARAM_SYSTEM_ANNOUNCEMENT_ID => $publication->get_id())), 
                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));
        }
        
        return $action_bar;
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_SYSTEM_ANNOUNCEMENT_ID);
    }
}
