<?php
namespace core\admin;

use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Utilities;
use common\libraries\Configuration;

/**
 *
 * @package admin.lib
 * @author Hans De Bisschop
 * @author Dieter De Neef
 */
class AdminDataManager
{

    /**
     * Instance of this class for the singleton pattern.
     */
    private static $instance;

    /**
     * Uses a singleton pattern and a factory pattern to return the data manager. The configuration determines which
     * data manager class is to be instantiated.
     *
     * @return AdminDataManagerInterface The data manager.
     */
    public static function get_instance()
    {
        if (! isset(self :: $instance))
        {
            $type = Configuration :: get_instance()->get_parameter('general', 'data_manager');
            $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'AdminDataManager';
            self :: $instance = new $class();
        }
        return self :: $instance;
    }

    public static function set_instance($instance)
    {
        self :: $instance = $instance;
    }

    /**
     * Retrieves a remote package with given context
     *
     * @param $context string
     * @return RemotePackage
     */
    public static function retrieve_remote_package_by_context($context)
    {
        $condition = new EqualityCondition(RemotePackage :: PROPERTY_CONTEXT, $context);
        return self :: get_instance()->retrieve_remote_packages($condition)->next_result();
    }

}
