<?php
namespace core\admin;

use common\libraries\CommonDataManager;
use common\libraries\CoreApplication;
use common\libraries\Translation;
use common\libraries\DynamicAction;
use common\libraries\Theme;
use common\libraries\Redirect;

/**
 *
 * @package admin.lib.admin_manager
 * @author Hans De Bisschop
 * @author Dieter De Neef
 */

/**
 * The admin allows the platform admin to configure certain aspects of his platform
 */
class AdminManager extends CoreApplication
{
    const APPLICATION_NAME = 'admin';
    const PARAM_WEB_APPLICATION = 'web_application';
    const PARAM_CONTEXT = 'context';
    const PARAM_DELETE_SELECTED = 'delete_selected';
    const PARAM_EDIT_SELECTED = 'edit_selected';
    const ACTION_ADMIN_BROWSER = 'browser';
    const ACTION_LANGUAGE = 'language';
    const ACTION_CONFIGURE_PLATFORM = 'configurer';
    const ACTION_MANAGE_CATEGORIES = 'category_manager';
    const ACTION_WHOIS_ONLINE = 'whois_online';
    const ACTION_DIAGNOSE = 'diagnoser';
    const ACTION_VIEW_LOGS = 'log_viewer';
    const ACTION_IMPORTER = 'importer';
    const ACTION_MANAGE_RESOURCE_CACHE = 'resource_cache_manager';
    const ACTION_SYSTEM_ANNOUNCEMENTS = 'announcer';
    const DEFAULT_ACTION = self :: ACTION_ADMIN_BROWSER;
    const ACTION_EDIT_EMAIL_RIGHTS = 'email_rights_editor';

    public static function get_application_platform_admin_links($application = self :: APPLICATION_NAME)
    {
        $links = array();

        $settings_url = Redirect :: get_link(
            self :: APPLICATION_NAME,
            array(self :: PARAM_ACTION => self :: ACTION_CONFIGURE_PLATFORM));

        $links[] = new DynamicAction(
            Translation :: get('Settings'),
            Translation :: get('SettingsDescription'),
            Theme :: get_image_path() . 'admin/settings.png',
            $settings_url);

        $links[] = new DynamicAction(
            Translation :: get('Importer'),
            Translation :: get('ImporterDescription'),
            Theme :: get_image_path() . 'admin/import.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(self :: PARAM_ACTION => self :: ACTION_IMPORTER),
                array(),
                false,
                Redirect :: TYPE_CORE));

        $links[] = new DynamicAction(
            Translation :: get('SystemAnnouncements'),
            Translation :: get('SystemAnnouncementsDescription'),
            Theme :: get_image_path() . 'admin/list.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(self :: PARAM_ACTION => self :: ACTION_SYSTEM_ANNOUNCEMENTS),
                array(),
                false,
                Redirect :: TYPE_CORE));

        $links[] = new DynamicAction(
            Translation :: get('TranslationsImport'),
            Translation :: get('TranslationsImportDescription'),
            Theme :: get_image_path(\core\admin\language\Manager :: context()) . 'logo/32.png',
            Redirect :: get_link(self :: APPLICATION_NAME, array(self :: PARAM_ACTION => self :: ACTION_LANGUAGE)));

        $links[] = new DynamicAction(
            Translation :: get('ManageCategories'),
            Translation :: get('ManageCategoriesDescription'),
            Theme :: get_image_path() . 'admin/list.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(self :: PARAM_ACTION => self :: ACTION_MANAGE_CATEGORIES),
                array(),
                false,
                Redirect :: TYPE_CORE));

        $links[] = new DynamicAction(
            Translation :: get('CacheManagement'),
            Translation :: get('CacheManagementDescription'),
            Theme :: get_image_path() . 'admin/cache.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(self :: PARAM_ACTION => self :: ACTION_MANAGE_RESOURCE_CACHE),
                array(),
                false,
                Redirect :: TYPE_CORE));

        $links[] = new DynamicAction(
            Translation :: get('Diagnose'),
            Translation :: get('DiagnoseDescription'),
            Theme :: get_image_path() . 'admin/information.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(self :: PARAM_ACTION => self :: ACTION_DIAGNOSE),
                array(),
                false,
                Redirect :: TYPE_CORE));

        $links[] = new DynamicAction(
            Translation :: get('LogsViewer'),
            Translation :: get('LogsViewerDescription'),
            Theme :: get_image_path() . 'admin/information.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(self :: PARAM_ACTION => self :: ACTION_VIEW_LOGS),
                array(),
                false,
                Redirect :: TYPE_CORE));
        $links[] = new DynamicAction(
            Translation :: get('EmailRights'),
            Translation :: get('EmailRightsDescription'),
            Theme :: get_image_path() . 'admin/information.png',
            Redirect :: get_link(
                self :: APPLICATION_NAME,
                array(self :: PARAM_ACTION => self :: ACTION_EDIT_EMAIL_RIGHTS),
                array(),
                false,
                Redirect :: TYPE_CORE));

        $admin_info = parent :: get_application_platform_admin_links(self :: APPLICATION_NAME);
        $admin_info['links'] = $links;

        return $admin_info;
    }

    /**
     * Helper function for the Application class, pending access to class constants via variables in PHP 5.3 e.g. $name
     * = $class :: DEFAULT_ACTION DO NOT USE IN THIS APPLICATION'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourApplicationManager :: DEFAULT_ACTION in all other application classes
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    public static function get_installable_application_packages($include_installed = false)
    {
        $package_list = new \common\libraries\package\PackageList(
            self :: context(),
            Translation :: get('TypeName', null, __NAMESPACE__),
            Theme :: get_image_path() . 'logo/16.png');

        if (! CommonDataManager :: get_registration(\core\admin\language\Manager :: context()) || $include_installed)
        {

            $package_list->add_package(\core\admin\language\Manager :: context());
        }

        if (! CommonDataManager :: get_registration(\core\admin\announcement\Manager :: context()) || $include_installed)
        {

            $package_list->add_package(\core\admin\announcement\Manager :: context());
        }

        $announcement_list = new \common\libraries\package\PackageList(
            self :: context() . '\announcement',
            Translation :: get('Announcement', null, __NAMESPACE__));

        $integration_list = new \common\libraries\package\PackageList(
            self :: context() . '\announcement\integration',
            Translation :: get('Integrations', null, __NAMESPACE__));

        $core_list = new \common\libraries\package\PackageList(
            self :: context() . '\announcement\integration\core',
            Translation :: get('Core', null, __NAMESPACE__));

        if (! CommonDataManager :: get_registration(self :: context() . '\announcement\integration\core\home') ||
             $include_installed)
        {
            $core_list->add_package(self :: context() . '\announcement\integration\core\home');
        }

        $integration_list->add_child($core_list);
        $announcement_list->add_child($integration_list);
        $package_list->add_child($announcement_list);

        $integration_list = new \common\libraries\package\PackageList(
            self :: context() . '\integration',
            Translation :: get('Integrations', null, __NAMESPACE__));

        if (! CommonDataManager :: get_registration(self :: context() . '\integration\tracking') || $include_installed)
        {
            $integration_list->add_package(self :: context() . '\integration\tracking');
        }

        if (! CommonDataManager :: get_registration(self :: context() . '\integration\webservice') || $include_installed)
        {
            $integration_list->add_package(self :: context() . '\integration\webservice');
        }

        $core_list = new \common\libraries\package\PackageList(
            self :: context() . '\integration\core',
            Translation :: get('Core', null, __NAMESPACE__));

        if (! CommonDataManager :: get_registration(self :: context() . '\integration\core\home') || $include_installed)
        {
            $core_list->add_package(self :: context() . '\integration\core\home');
        }

        $integration_list->add_child($core_list);
        $package_list->add_child($integration_list);

        return $package_list;
    }
}
