<?php
namespace core\admin;

use common\libraries\DoctrineConditionTranslator;
use common\libraries\Condition;
use user\User;
use common\libraries\Session;
use common\libraries\DoctrineDatabase;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\SubselectCondition;
use common\libraries\InCondition;
use common\libraries\ObjectTableOrder;
use common\extensions\dynamic_form_manager;
use common\extensions\dynamic_form_manager\DynamicForm;
use common\extensions\dynamic_form_manager\DynamicFormElement;
use common\extensions\dynamic_form_manager\DynamicFormElementOption;
use common\extensions\dynamic_form_manager\DynamicFormElementValue;
use common\extensions\invitation_manager\Invitation;

/**
 * $Id: database_admin_data_manager.class.php 231 2009-11-16 09:53:00Z vanpouckesven $
 *
 * @package admin.lib.data_manager
 */
class DoctrineAdminDataManager extends DoctrineDatabase implements AdminDataManagerInterface
{

    public function initialize()
    {
        parent :: initialize();
        $this->set_prefix('admin_');
    }

    public function delete_category($category)
    {
        $condition = new EqualityCondition(AdminCategory :: PROPERTY_ID, $category->get_id());
        $succes = $this->delete('admin_category', $condition);

        $query = 'UPDATE ' . $this->escape_table_name('admin_category') . ' SET ' .
             parent :: escape_column_name(AdminCategory :: PROPERTY_DISPLAY_ORDER) . '=' .
             parent :: escape_column_name(AdminCategory :: PROPERTY_DISPLAY_ORDER) . '-1 WHERE ' .
             parent :: escape_column_name(AdminCategory :: PROPERTY_DISPLAY_ORDER) . '>' .
             $this->quote($category->get_display_order()) . ' AND ' .
             parent :: escape_column_name(AdminCategory :: PROPERTY_PARENT) . '=' . $this->quote($category->get_parent());
        $res = $this->exec($query);

        return $succes;
    }

    public function update_category($category)
    {
        $condition = new EqualityCondition(AdminCategory :: PROPERTY_ID, $category->get_id());
        return $this->update($category, $condition);
    }

    public function create_category($category)
    {
        return $this->create($category);
    }

    public function count_categories($conditions = null)
    {
        return $this->count_objects('admin_category', $conditions);
    }

    public function count_feedback_publications($pid, $cid, $application)
    {
        $conditions[] = new EqualityCondition(FeedbackPublication :: PROPERTY_PID, $pid);
        $conditions[] = new EqualityCondition(FeedbackPublication :: PROPERTY_CID, $cid);
        $conditions[] = new EqualityCondition(FeedbackPublication :: PROPERTY_APPLICATION, $application);
        $condition = new AndCondition($conditions);

        return $this->count_objects('feedback_publication', $condition);
    }

    public function retrieve_categories($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return $this->retrieve_objects('admin_category', $condition, $offset, $count, $order_property);
    }

    public function select_next_display_order($parent_category_id)
    {
        $query = 'SELECT MAX(' . AdminCategory :: PROPERTY_DISPLAY_ORDER . ') AS do FROM ' .
             $this->escape_table_name('admin_category');

        $condition = new EqualityCondition(AdminCategory :: PROPERTY_PARENT, $parent_category_id);

        if (isset($condition))
        {
            $query .= ' WHERE ' . DoctrineConditionTranslator :: render($condition);
        }

        $res = $this->get_connection()->query($query);
        $record = $res->fetchRow(MDB2_FETCHMODE_ORDERED);

        return $record[0] + 1;
    }

    public function retrieve_feedback_publications($pid, $cid, $application)
    {
        $conditions[] = new EqualityCondition(FeedbackPublication :: PROPERTY_PID, $pid);
        $conditions[] = new EqualityCondition(FeedbackPublication :: PROPERTY_CID, $cid);
        $conditions[] = new EqualityCondition(FeedbackPublication :: PROPERTY_APPLICATION, $application);
        $condition = new AndCondition($conditions);
        $order_by[] = new ObjectTableOrder(FeedbackPublication :: PROPERTY_ID, SORT_DESC);

        return $this->retrieve_objects(
            FeedbackPublication :: get_table_name(),
            $condition,
            null,
            null,
            $order_by,
            FeedbackPublication :: CLASS_NAME);
    }

    /*
     * function retrieve_validations($pid,$cid,$application) { $conditions[] = new EqualityCondition(Validation ::
     * PROPERTY_PID, $pid); $conditions[] = new EqualityCondition(Validation :: PROPERTY_CID, $cid); $conditions[] = new
     * EqualityCondition(Validation :: PROPERTY_APPLICATION, $application); $condition = new AndCondition($conditions);
     * //$order_by[] = new ObjectTableOrder(FeedbackPublication::PROPERTY_ID,SORT_DESC); return
     * $this->retrieve_objects(Validation :: get_table_name(),$condition); }
     */
    public function retrieve_feedback_publication($id)
    {
        $condition = new EqualityCondition(FeedbackPublication :: PROPERTY_ID, $id);
        return $this->retrieve_object(
            FeedbackPublication :: get_table_name(),
            $condition,
            array(),
            FeedbackPublication :: CLASS_NAME);
    }

    public function retrieve_validation($id)
    {
        $condition = new EqualityCondition(Validation :: PROPERTY_ID, $id);
        return $this->retrieve_object(Validation :: get_table_name(), $condition);
    }

    public function update_feedback_publication($feedback_publication)
    {
        $condition = new EqualityCondition(FeedbackPublication :: PROPERTY_ID, $feedback_publication->get_id());
        return $this->update($feedback_publication, $condition);
    }

    public function update_validation($validation)
    {
        $condition = new EqualityCondition(Validation :: PROPERTY_ID, $validation->get_id());
        return $this->update($validation, $condition);
    }

    public function delete_feedback_publication($feedback_publication)
    {
        $condition = new EqualityCondition(FeedbackPublication :: PROPERTY_ID, $feedback_publication->get_id());
        return $this->delete($feedback_publication->get_table_name(), $condition);
    }

    /**
     * delete all the publications for a certain feedback content item
     *
     * @param $feedback_co_id int
     * @return boolean success?
     */
    public function delete_feedback_publications($feedback_co_id)
    {
        $condition = new EqualityCondition(FeedbackPublication :: PROPERTY_FID, $feedback_co_id);
        return $this->delete(FeedbackPublication :: get_table_name(), $condition);
    }

    public function delete_validation($validation)
    {
        $condition = new EqualityCondition(FeedbackPublication :: PROPERTY_ID, $validation->get_id());
        return $this->delete($validation->get_table_name(), $condition);
    }

    public function create_feedback_publication($feedback_publication)
    {
        return $this->create($feedback_publication);
    }

    public function create_validation($validation)
    {
        return $this->create($validation);
    }

    public function retrieve_validations($condition = null, $order_by = array (), $offset = 0, $max_objects = -1)
    {
        $val_table = $this->escape_table_name(Validation :: get_table_name());
        $val_table_alias = $this->get_alias(Validation :: get_table_name());
        $user_table = \user\DataManager :: escape_table_name(User :: get_table_name());
        $user_table_alias = \user\DataManager :: get_alias(User :: get_table_name());

        $query = 'SELECT * FROM ' . $val_table . ' AS ' . $val_table_alias . ' JOIN ' . $user_table . ' AS ' .
             $user_table_alias;

        return $this->retrieve_object_set(
            $query,
            Validation :: get_table_name(),
            $condition = null,
            $offset,
            $max_objects,
            $order_by);
    }

    public function count_validations($condition = null)
    {
        return $this->count_objects(Validation :: get_table_name(), $condition);
    }

    // Dynamic Forms
    public function delete_dynamic_form($dynamic_form)
    {
        $condition = new EqualityCondition(DynamicForm :: PROPERTY_ID, $dynamic_form->get_id());
        return $this->delete($dynamic_form->get_table_name(), $condition);
    }

    public function update_dynamic_form($dynamic_form)
    {
        $condition = new EqualityCondition(DynamicForm :: PROPERTY_ID, $dynamic_form->get_id());
        return $this->update($dynamic_form, $condition);
    }

    public function create_dynamic_form($dynamic_form)
    {
        return $this->create($dynamic_form);
    }

    public function count_dynamic_forms($conditions = null)
    {
        return $this->count_objects(DynamicForm :: get_table_name(), $conditions);
    }

    public function retrieve_dynamic_forms($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return $this->retrieve_objects(
            DynamicForm :: get_table_name(),
            $condition,
            $offset,
            $count,
            $order_property,
            DynamicForm :: CLASS_NAME);
    }

    public function delete_dynamic_form_element($dynamic_form_element)
    {
        $condition = new EqualityCondition(DynamicFormElement :: PROPERTY_ID, $dynamic_form_element->get_id());
        return $this->delete($dynamic_form_element->get_table_name(), $condition);
    }

    public function update_dynamic_form_element($dynamic_form_element)
    {
        $condition = new EqualityCondition(DynamicFormElement :: PROPERTY_ID, $dynamic_form_element->get_id());
        return $this->update($dynamic_form_element, $condition);
    }

    public function create_dynamic_form_element($dynamic_form_element)
    {
        return $this->create($dynamic_form_element);
    }

    public function count_dynamic_form_elements($conditions = null)
    {
        return $this->count_objects(DynamicFormElement :: get_table_name(), $conditions);
    }

    public function retrieve_dynamic_form_elements($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return $this->retrieve_objects(
            DynamicFormElement :: get_table_name(),
            $condition,
            $offset,
            $count,
            $order_property,
            DynamicFormElement :: CLASS_NAME);
    }

    public function select_next_dynamic_form_element_order($dynamic_form_id)
    {
        $condition = new EqualityCondition(DynamicFormElement :: PROPERTY_DYNAMIC_FORM_ID, $dynamic_form_id);
        return $this->retrieve_next_sort_value(
            DynamicFormElement :: get_table_name(),
            DynamicFormElement :: PROPERTY_DISPLAY_ORDER,
            $condition);
    }

    public function delete_dynamic_form_element_option($dynamic_form_element_option)
    {
        $condition = new EqualityCondition(
            DynamicFormElementOption :: PROPERTY_ID,
            $dynamic_form_element_option->get_id());
        return $this->delete($dynamic_form_element_option->get_table_name(), $condition);
    }

    public function update_dynamic_form_element_option($dynamic_form_element_option)
    {
        $condition = new EqualityCondition(
            DynamicFormElementOption :: PROPERTY_ID,
            $dynamic_form_element_option->get_id());
        return $this->update($dynamic_form_element_option, $condition);
    }

    public function create_dynamic_form_element_option($dynamic_form_element_option)
    {
        return $this->create($dynamic_form_element_option);
    }

    public function count_dynamic_form_element_options($conditions = null)
    {
        return $this->count_objects(DynamicFormElementOption :: get_table_name(), $conditions);
    }

    public function retrieve_dynamic_form_element_options($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return $this->retrieve_objects(
            DynamicFormElementOption :: get_table_name(),
            $condition,
            $offset,
            $count,
            $order_property,
            DynamicFormElementOption :: CLASS_NAME);
    }

    public function select_next_dynamic_form_element_option_order($dynamic_form_element_id)
    {
        $condition = new EqualityCondition(
            DynamicFormElementOption :: PROPERTY_DYNAMIC_FORM_ELEMENT_ID,
            $dynamic_form_element_id);
        return $this->retrieve_next_sort_value(
            DynamicFormElementOption :: get_table_name(),
            DynamicFormElementOption :: PROPERTY_DISPLAY_ORDER,
            $condition);
    }

    public function delete_all_options_from_form_element($dynamic_form_element_id)
    {
        $condition = new EqualityCondition(
            DynamicFormElementOption :: PROPERTY_DYNAMIC_FORM_ELEMENT_ID,
            $dynamic_form_element_id);
        return $this->delete(DynamicFormElementOption :: get_table_name(), $condition);
    }

    public function delete_dynamic_form_element_value($dynamic_form_element_value)
    {
        $condition = new EqualityCondition(DynamicFormElementValue :: PROPERTY_ID, $dynamic_form_element_value->get_id());
        return $this->delete($dynamic_form_element_value->get_table_name(), $condition);
    }

    public function update_dynamic_form_element_value($dynamic_form_element_value)
    {
        $condition = new EqualityCondition(DynamicFormElementValue :: PROPERTY_ID, $dynamic_form_element_value->get_id());
        return $this->update($dynamic_form_element_value, $condition);
    }

    public function create_dynamic_form_element_value($dynamic_form_element_value)
    {
        return $this->create($dynamic_form_element_value);
    }

    public function count_dynamic_form_element_values($conditions = null)
    {
        return $this->count_objects(DynamicFormElementValue :: get_table_name(), $conditions);
    }

    public function retrieve_dynamic_form_element_values($condition = null, $offset = null, $count = null, $order_property = null)
    {
        return $this->retrieve_objects(
            DynamicFormElementValue :: get_table_name(),
            $condition,
            $offset,
            $count,
            $order_property);
    }

    public function delete_dynamic_form_element_values_from_form($dynamic_form_id)
    {
        $subcondition = new EqualityCondition(DynamicFormElement :: PROPERTY_DYNAMIC_FORM_ID, $dynamic_form_id);
        $subselect = new SubselectCondition(
            DynamicFormElementValue :: PROPERTY_DYNAMIC_FORM_ELEMENT_ID,
            DynamicFormElement :: PROPERTY_ID,
            DynamicFormElement :: get_table_name(),
            $subcondition);

        return $this->delete(DynamicFormElementValue :: get_table_name(), $subselect);
    }

    public function retrieve_invitation($id)
    {
        $condition = new EqualityCondition(Invitation :: PROPERTY_ID, $id);
        return $this->retrieve_object(Invitation :: get_table_name(), $condition, array(), Invitation :: CLASS_NAME);
    }

    public function retrieve_invitation_by_code($code)
    {
        $condition = new EqualityCondition(Invitation :: PROPERTY_CODE, $code);
        return $this->retrieve_object(Invitation :: get_table_name(), $condition, array(), Invitation :: CLASS_NAME);
    }

    public function retrieve_invitations($condition = null, $order_by = array (), $offset = 0, $max_objects = -1)
    {
        return $this->retrieve_objects(
            Invitation :: get_table_name(),
            $condition,
            $offset,
            $max_objects,
            $order_by,
            Invitation :: CLASS_NAME);
    }

    public function create_invitation($invitation)
    {
        return $this->create($invitation);
    }

    public function delete_invitation($invitation)
    {
        $condition = new EqualityCondition(Invitation :: PROPERTY_ID, $invitation->get_id());
        return $this->delete_objects(Invitation :: get_table_name(), $invitation);
    }

    public function update_invitation($invitation)
    {
        $condition = new EqualityCondition(Invitation :: PROPERTY_ID, $invitation->get_id());
        return $this->update($invitation, $condition);
    }

    public function delete_feedback_from_publication($application, $publication_id)
    {
        $conditions[] = new EqualityCondition(FeedbackPublication :: PROPERTY_APPLICATION, $application);
        $conditions[] = new EqualityCondition(FeedbackPublication :: PROPERTY_PID, $publication_id);
        $condition = new AndCondition($conditions);

        return $this->delete(FeedbackPublication :: get_table_name(), $condition);
    }
}
