<?php
namespace core\admin;

use common\libraries\DataClassCache;
use common\libraries\CommonDataManager;
use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\Filesystem;
use common\libraries\Path;

/**
 * $Id: admin_installer.class.php 168 2009-11-12 11:53:23Z vanpouckesven $
 *
 * @package admin.install
 */
/**
 * This installer can be used to create the storage structure for the users application.
 */
class Installer extends \common\libraries\package\Installer
{

    /**
     * Constructor
     */
    public function __construct($values)
    {
        parent :: __construct($values, AdminDataManager :: get_instance());
    }

    /**
     * Runs the install-script.
     */
    public function extra()
    {
        // Update the default settings to the database
        if (! $this->update_settings())
        {
            return false;
        }
        else
        {
            $this->add_message(
                self :: TYPE_NORMAL,
                Translation :: get(
                    'ObjectsAdded',
                    array('OBJECTS' => Translation :: get('DefaultSettings')),
                    Utilities :: COMMON_LIBRARIES));
        }

        return true;
    }

    public function update_settings()
    {
        $values = $this->get_form_values();

        $settings = array();
        $settings[] = array('core\admin', 'site_name', $values['platform_name']);
        $settings[] = array('core\admin', 'server_type', 'production');
        $settings[] = array('core\admin', 'platform_language', $values['platform_language']);
        $settings[] = array('core\admin', 'version', '4.0');
        $settings[] = array('core\admin', 'theme', 'aqua');

        $settings[] = array('core\admin', 'institution', $values['organization_name']);
        $settings[] = array('core\admin', 'institution_url', $values['organization_url']);

        $settings[] = array('core\admin', 'show_administrator_data', 'true');
        $settings[] = array('core\admin', 'administrator_firstname', $values['admin_firstname']);
        $settings[] = array('core\admin', 'administrator_surname', $values['admin_surname']);
        $settings[] = array('core\admin', 'administrator_email', $values['admin_email']);
        $settings[] = array('core\admin', 'administrator_telephone', $values['admin_phone']);

        DataClassCache :: truncate(\common\libraries\Setting :: class_name());

        foreach ($settings as $setting)
        {
            $setting_object = CommonDataManager :: retrieve_setting_from_variable_name($setting[1], $setting[0]);
            $setting_object->set_application($setting[0]);
            $setting_object->set_variable($setting[1]);
            $setting_object->set_value($setting[2]);

            if (! $setting_object->update())
            {
                return false;
            }
        }

        return true;
    }

    public function get_additional_packages()
    {
        $installers = array();
        $installers[] = 'core\admin\announcement';
        $installers[] = 'core\admin\language';
        $installers[] = 'core\admin\integration\core\home';
        $installers[] = 'core\admin\integration\tracking';
        $installers[] = 'core\admin\integration\webservice';
        return $installers;
    }
}
