<?php
namespace core\admin\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use core\lynx\manager\PackageInstaller;
use common\libraries\PropertyConditionVariable;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use common\libraries\StaticConditionVariable;
use common\libraries\Registration;
use common\libraries\Utilities;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\AndCondition;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\core\admin\AdminDataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('core\admin');
                $registration = CommonDataManager :: get_registration('admin');
                $registration->set_version($package->get_version());
                $registration->set_context($package->get_context());
                if (! $registration->update())
                {
                    return false;
                }

                if (! \core\admin\AdminDataManager :: get_instance()->drop_storage_unit(
                    'system_announcement_publication_group'))
                {
                    return false;
                }

                if (! \core\admin\AdminDataManager :: get_instance()->drop_storage_unit(
                    'system_announcement_publication_user'))
                {
                    return false;
                }

                if (! \tracking\TrackingDataManager :: get_instance()->drop_storage_unit('admin_changes'))
                {
                    return false;
                }

                if (! \tracking\TrackingDataManager :: get_instance()->rename_storage_unit(
                    'online_tracker',
                    'admin_online'))
                {
                    return false;
                }

                return true;
            });

        if (! $success)
        {
            return false;
        }

        $packages = array();
        $packages[] = 'core\admin\announcement';

        foreach ($packages as $package)
        {
            $installer = new PackageInstaller($package);
            $installer->run();
        }

        if (! \core\admin\announcement\DataManager :: get_instance()->drop_storage_unit('publication'))
        {
            return false;
        }

        if (! \core\admin\AdminDataManager :: get_instance()->rename_storage_unit(
            'system_announcement_publication',
            'announcement_publication'))
        {
            return false;
        }

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \common\libraries\Setting :: class_name(),
                \common\libraries\Setting :: PROPERTY_CONTEXT),
            new StaticConditionVariable('admin'));
        $settings = CommonDataManager :: retrieves(
            \common\libraries\Setting :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($setting = $settings->next_result())
        {
            $setting->set_context('core\admin');
            if (! $setting->update())
            {
                return false;
            }
        }

        // Home
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \core\home\HomeBlockRegistration :: class_name(),
                \core\home\HomeBlockRegistration :: PROPERTY_CONTEXT),
            new StaticConditionVariable('admin'));
        $home_block_registrations = \core\home\DataManager :: retrieves(
            \core\home\HomeBlockRegistration :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($home_block_registration = $home_block_registrations->next_result())
        {
            if ($home_block_registration->get_block() != 'system_announcements')
            {
                $home_block_registration->set_context('core\admin\integration\core\home');
            }
            else
            {
                $home_block_registration->set_context('core\admin\announcement\integration\core\home');
            }

            if (! $home_block_registration->update())
            {
                return false;
            }
        }

        $package_info = Package :: get('core\admin\integration\core\home');
        $application_registration = new Registration();
        $parent_namespace = Utilities :: get_namespace_parent('core\admin\integration\core\home');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));
        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('core\admin\integration\core\home');

        if (! $application_registration->create())
        {
            return false;
        }

        // Tracking
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \tracking\TrackerRegistration :: class_name(),
                \tracking\TrackerRegistration :: PROPERTY_CONTEXT),
            new StaticConditionVariable('admin'));
        $tracker_registrations = \tracking\TrackingDataManager :: get_instance()->retrieves(
            \tracking\TrackerRegistration :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($tracker_registration = $tracker_registrations->next_result())
        {
            $tracker_registration->set_context('core\admin\integration\tracking');
            $tracker_registration->update();
        }

        $condition = new EqualityCondition(
            new PropertyConditionVariable(\tracking\Event :: class_name(), \tracking\Event :: PROPERTY_CONTEXT),
            new StaticConditionVariable('admin'));
        $event_registrations = \tracking\TrackingDataManager :: get_instance()->retrieves(
            \tracking\Event :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($event_registration = $event_registrations->next_result())
        {
            $event_registration->set_context('core\admin\integration\tracking');
            $event_registration->update();
        }

        $package_info = Package :: get('core\admin\integration\tracking');

        $application_registration = new Registration();
        $parent_namespace = Utilities :: get_namespace_parent('core\admin\integration\tracking');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));
        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('core\admin\integration\tracking');

        if (! $application_registration->create())
        {
            return false;
        }

        $properties = new DataClassProperties();
        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    \tracking\TrackerRegistration :: class_name(),
                    \tracking\TrackerRegistration :: PROPERTY_TRACKER),
                new StaticConditionVariable('online')));

        $conditions = array();
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                \tracking\TrackerRegistration :: class_name(),
                \tracking\TrackerRegistration :: PROPERTY_TRACKER),
            new StaticConditionVariable('online_tracker'));
        $conditions[] = new EqualityCondition(
            new PropertyConditionVariable(
                \tracking\TrackerRegistration :: class_name(),
                \tracking\TrackerRegistration :: PROPERTY_CONTEXT),
            new StaticConditionVariable('core\admin\integration\tracking'));
        $condition = new AndCondition($conditions);

        if (! \tracking\TrackingDataManager :: get_instance()->updates(
            \tracking\TrackerRegistration :: class_name(),
            $properties,
            $condition))
        {
            return false;
        }

        // Webservices
        $package_info = Package :: get('core\admin\integration\webservice');

        $application_registration = new Registration();
        $parent_namespace = Utilities :: get_namespace_parent('core\admin\integration\webservice');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));
        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('core\admin\integration\webservice');

        if (! $application_registration->create())
        {
            return false;
        }

        return true;
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::get_additional_packages()
    */
    public function get_additional_packages()
    {
        $packages = array();
        $packages[] = 'core\admin\language';
        return $packages;
    }
}
