<?php
namespace core\context_linker;
use common\libraries\Configuration;
use common\libraries\Utilities;
/**
 *	This is a skeleton for a data manager for the ContextLinker Application.
 *	Data managers must extend this class and implement its abstract methods.
 *
 *  @author Sven Vanpoucke
 *	@author Jens Vanderheyden
 */
class ContextLinkerDataManager
{
    /**
     * Instance of this class for the singleton pattern.
     */
    private static $instance;

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->initialize();
    }

    /**
     * Uses a singleton pattern and a factory pattern to return the data
     * manager. The configuration determines which data manager class is to
     * be instantiated.
     * @return ContextLinkerDataManager The data manager.
     */
    public static function get_instance()
    {
        if (!isset (self :: $instance))
        {
            $type = Configuration :: get_instance()->get_parameter('general', 'data_manager');
            $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'ContextLinkerDataManager';
            self :: $instance = new $class ();
        }
        return self :: $instance;
    }
}
