<?php
namespace core\context_linker;

use repository\ContentObject;
use repository\content_object\document\Document;

use common\libraries\Request;
use common\libraries\Translation;
use repository\content_object\file\File;
use repository\content_object\youtube\Youtube;
use repository\content_object\mediamosa\Mediamosa;
use common\libraries\Utilities;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use repository\content_object\link\Link;
use common\libraries\EqualityCondition;
use common\libraries\OrCondition;

/**
 * Component to create a new context_link object
 *
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class ContextLinkerManagerContextLinkCreatorComponent extends ContextLinkerManager
{

    /**
     * Selects or creates an aletrnative content object then redirects to context_links publisher to create the actual
     * context link
     */
    private $content_object;

    public function run()
    {
        $trail = BreadcrumbTrail :: get_instance();
        $trail->add(
                new Breadcrumb($this->get_url(array(ContextLinkerManager :: PARAM_ACTION => null)),
                        Translation :: get('ContextLinker')));
        $trail->add(
                new Breadcrumb(
                        Translation :: get('CreateObject', array('OBJECT' => Translation :: get('ContextLink')),
                                Utilities :: COMMON_LIBRARIES)));
        $trail->add_help('ContextLinkCreator');

        $redirect_url = Request :: get(ContextLinkerManager :: PARAM_REDIRECT_URL);

        if ($this->content_object = \repository\DataManager :: retrieve_content_object(
                Request :: get(ContextLinkerManager :: PARAM_CONTENT_OBJECT_ID)))
        {
            if (! \repository\viewer\Manager :: is_ready_to_be_published())
            {
                $repository_viewer = \repository\viewer\Manager :: construct($this);

                // get already established links to exclude them from repoviewer
                $cdm = ContextLinkerDataManager :: get_instance();

                $condition1 = new EqualityCondition(ContextLink :: PROPERTY_ORIGINAL_CONTENT_OBJECT_ID,
                        Request :: get(ContextLinkerManager :: PARAM_CONTENT_OBJECT_ID));
                $condition2 = new EqualityCondition(ContextLink :: PROPERTY_ALTERNATIVE_CONTENT_OBJECT_ID,
                        Request :: get(ContextLinkerManager :: PARAM_CONTENT_OBJECT_ID));

                $condition = new OrCondition($condition1, $condition2);

                $result = $cdm->retrieve_full_context_links($condition);

                foreach ($result as $n => $v)
                {
                    $already_linked_content_objects[] = $v[ContextLinkerManager :: PROPERTY_ALT_ID];
                }

                unset($result);

                array_push($already_linked_content_objects, $this->content_object->get_id());

                $repository_viewer->set_excluded_objects($already_linked_content_objects);
                $repository_viewer->set_parameter(ContextLinkerManager :: PARAM_CONTENT_OBJECT_ID,
                        $this->content_object->get_id());
                $repository_viewer->set_parameter(ContextLinkerManager :: PARAM_REDIRECT_URL, $redirect_url);

                $repository_viewer->run();
            }
            else
            {
                $objects = \repository\viewer\Manager :: get_selected_objects();
                if (! is_array($objects))
                {
                    $objects = array($objects);
                }

                $success = true;

                foreach ($objects as $object_id)
                {
                    $this->redirect(
                            Translation :: get('ObjectSelected',
                                    array('OBJECT' => Translation :: get('AlternativeContentObject')),
                                    Utilities :: COMMON_LIBRARIES), false,
                            array(
                                    ContextLinkerManager :: PARAM_ACTION => ContextLinkerManager :: ACTION_PUBLISH_CONTEXT_LINK,
                                    ContextLinkerManager :: PARAM_CONTENT_OBJECT_ID => Request :: get(
                                            ContextLinkerManager :: PARAM_CONTENT_OBJECT_ID),
                                    ContextLinkerManager :: PARAM_ALTERNATIVE_CONTENT_OBJECT_ID => $object_id,
                                    ContextLinkerManager :: PARAM_REDIRECT_URL => $redirect_url));
                }
            }
        }
        else

        {
            $this->display_header();
            echo '<p>' . Translation :: get('NoContentObjectSelected', null, 'repository') . '</p>';
            $this->display_footer();
        }
    }

    public function get_allowed_content_object_types()
    {
        // TODO : different sets should be possible dependent on type of content object
        /*$types = array(File :: get_type_name(), Youtube :: get_type_name(), MediaMosa :: get_type_name(),
                Link :: get_type_name(), $this->content_object->get_type());

        $optional_types = array(Document :: get_type_name(), File :: get_type_name());*/

        $types = array(File :: class_name(), Youtube :: class_name(), MediaMosa :: class_name(),
                Link :: class_name(), $this->content_object->get_type());

        $optional_types = array(
            Document :: get_type_name() => Document :: class_name(),
            File :: get_type_name() => File :: class_name()
        );

        foreach ($optional_types as $optional_type_name => $optional_type_class)
        {
            if (ContentObject :: is_available($optional_type_class))
            {
                $types[] = $optional_type_class;
            }
        }
        return $types;
    }
}
