<?php
namespace core\context_linker;

use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\Request;
use common\libraries\Utilities;

use core\metadata\MetadataDataManager;

/**
 * Component to edit an existing context_link object
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class ContextLinkerManagerContextLinkUpdaterComponent extends ContextLinkerManager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $context_link = $this->retrieve_context_link(Request :: get(ContextLinkerManager :: PARAM_CONTEXT_LINK));

        $mdm = MetadataDataManager :: get_instance();

        $params = array();
        $params[ContextLinkerManager :: PARAM_CONTENT_OBJECT_ID] = $context_link->get_original_content_object_id();
        $params[ContextLinkerManager :: PARAM_ALTERNATIVE_CONTENT_OBJECT_ID] = $context_link->get_alternative_content_object_id();

        $condition = new EqualityCondition(ContextLinkerManager :: PARAM_CONTENT_OBJECT_ID, $context_link->get_original_content_object_id());
        $metadata_element_values = $mdm->retrieve_full_metadata_element_values($condition);

        $form = new ContextLinkForm(ContextLinkForm :: TYPE_EDIT, $context_link, $metadata_element_values, $this->get_url(array_merge($params, array(
                ContextLinkerManager :: PARAM_CONTEXT_LINK => $context_link->get_id())), $this->get_user()));

        if ($form->validate())
        {
            $success = $form->update_context_link();
            $this->redirect($success ? Translation :: get('ObjectUpdated', array(
                    'OBJECT' => Translation :: get('ContextLink')), Utilities :: COMMON_LIBRARIES) : Translation :: get('ObjectNotUpdated', array(
                    'OBJECT' => Translation :: get('ContextLink')), Utilities :: COMMON_LIBRARIES), ! $success, array(
                    ContextLinkerManager :: PARAM_ACTION => ContextLinkerManager :: ACTION_BROWSE_CONTEXT_LINKS,
                    ContextLinkerManager :: PARAM_CONTENT_OBJECT_ID => $context_link->get_original_content_object_id()));
        }
        else
        {
            $this->display_header();
            $form->display();
            $this->display_footer();
        }
    }
}
