<?php
namespace core\help;

use common\libraries\Application;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Theme;

/**
 * $Id: help_item_browser_table_cell_renderer.class.php 226 2009-11-13 14:44:03Z chellee $
 *
 * @package help.lib.help_manager.component.help_item_browser_table
 */

/**
 * Cell rendere for the learning object browser table
 */
class HelpItemBrowserTableCellRenderer extends DefaultHelpItemTableCellRenderer
{

    private $user;

    /**
     * The repository browser component
     */
    private $browser;

    /**
     * Constructor
     *
     * @param RepositoryManagerBrowserComponent $browser
     */
    public function __construct($browser)
    {
        parent :: __construct();
        $this->browser = $browser;
    }

    // Inherited
    public function render_cell($column, $help_item)
    {
        if ($column === HelpItemBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($help_item);
        }

        return parent :: render_cell($column, $help_item);
    }

    /**
     * Gets the action links to display
     *
     * @param ContentObject $content_object The learning object for which the action links should be returned
     * @return string A HTML representation of the action links
     */
    private function get_modification_links($help_item)
    {
        $toolbar = new Toolbar();

        $toolbar->add_item(
                new ToolbarItem(Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_edit.png',
                        $this->browser->get_url(
                                array(Application :: PARAM_ACTION => Manager :: ACTION_UPDATE_HELP_ITEM,
                                        Manager :: PARAM_HELP_ITEM => $help_item->get_id())), ToolbarItem :: DISPLAY_ICON));

        // if($this->browser->get_user()->is_platform_admin())
        // {
        // $toolbar->add_item(new ToolbarItem(
        // Translation :: get('EditRights', null, Utilities :: COMMON_LIBRARIES),
        // Theme :: get_common_image_path() . 'action_rights.png',
        // $this->browser->get_url(array(Application :: PARAM_ACTION => \core\help\Manager :: ACTION_RIGHT_EDITS, \core\help\Manager
        // :: PARAM_HELP_ITEM => $help_item->get_id())),
        // ToolbarItem :: DISPLAY_ICON
        // ));
        // }

        return $toolbar->as_html();
    }
}
