<?php
namespace core\home;

use common\libraries\Theme;
use common\libraries\Session;
use common\libraries\Translation;
use common\libraries\AjaxManager;
use common\libraries\JsonAjaxResult;
use common\libraries\Application;

/**
 *
 * @author Hans De Bisschop
 */
class HomeAjaxTabAdd extends AjaxManager
{
    const PROPERTY_HTML = 'html';
    const PROPERTY_TITLE = 'title';

    /*
     * (non-PHPdoc) @see common\libraries.AjaxManager::required_parameters()
     */
    public function required_parameters()
    {
        return array();
    }

    /*
     * (non-PHPdoc) @see common\libraries.AjaxManager::run()
     */
    public function run()
    {
        $user_id = DataManager :: determine_user_id();

        if ($user_id === false)
        {
            JsonAjaxResult :: not_allowed();
        }

        $tab = new HomeTab();
        $tab->set_title(Translation :: get('NewTab'));
        $tab->set_user($user_id);
        if (! $tab->create())
        {
            JsonAjaxResult :: general_error(Translation :: get('TabNotAdded'));
        }

        $row = new HomeRow();
        $row->set_title(Translation :: get('NewRow'));
        $row->set_tab($tab->get_id());
        $row->set_user($user_id);
        if (! $row->create())
        {
            JsonAjaxResult :: general_error(Translation :: get('TabRowNotAdded'));
        }

        $column = new HomeColumn();
        $column->set_row($row->get_id());
        $column->set_title(Translation :: get('NewColumn'));
        $column->set_sort('1');
        $column->set_width('100');
        $column->set_user($user_id);
        if (! $column->create())
        {
            JsonAjaxResult :: general_error(Translation :: get('TabColumnNotAdded'));
        }

        // $block = new HomeBlock();
        // $block->set_column($column->get_id());
        // $block->set_title(Translation :: get('DummyBlock'));
        // $registration = DataManager ::
        // retrieve_home_block_registration_by_context_and_block('repository\content_object\link', 'linker');
        // $block->set_registration_id($registration->get_id());
        // $block->set_visibility('1');
        // $block->set_user($user_id);
        // if (! $block->create())
        // {
        // JsonAjaxResult :: general_error(Translation :: get('TabBlockNotAdded'));
        // }
        //
        // $user = UserDataManager :: get_instance()->retrieve_user(Session :: get_user_id());
        //
        // $home_renderer = HomeRenderer :: factory(HomeRenderer :: TYPE_DEFAULT, $user);
        // $block_component = Block :: factory($home_renderer, $block);

        $html[] = '<div class="portal_tab" id="portal_tab_' . $tab->get_id() . '" style="display: none;">';
        $html[] = '<div class="portal_row" id="portal_row_' . $row->get_id() . '">';
        $html[] = '<div class="portal_column" id="portal_column_' . $column->get_id() . '" style="width: ' .
             $column->get_width() . '%;">';

        // $html[] = $block_component->as_html();

        $html[] = '<div class="empty_portal_column" style="display:block;">';
        $html[] = htmlspecialchars(Translation :: get('EmptyColumnText'));
        $img = Theme :: get_image_path(Application :: determine_namespace('home')) . 'action_remove_column.png';
        $html[] = '<div class="deleteColumn"><a href="#"><img src="' . $img . '" alt="' .
             Translation :: get('RemoveColumn') . '"/></a></div>';
        $html[] = '</div>';
        $html[] = '</div>';

        $html[] = '</div>';
        $html[] = '</div>';
        $html[] = '</div>';

        $title = array();
        $title[] = '<li class="normal" id="tab_select_' . $tab->get_id() . '">';
        $title[] = '<a class="tabTitle" href="#">' . $tab->get_title() . '</a>';
        $title[] = '<a class="deleteTab"><img src="' . Theme :: get_image_path() . 'action_delete_tab.png" /></a>';
        $title[] = '</li>';

        $result = new JsonAjaxResult(200);
        $result->set_property(self :: PROPERTY_HTML, implode("\n", $html));
        $result->set_property(self :: PROPERTY_TITLE, implode("\n", $title));
        $result->display();
    }
}
