<?php
namespace core\home;

use common\libraries\Application;
use repository\ContentObjectRendition;
use repository\ContentObjectRenditionImplementation;
use common\libraries\Display;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\ObjectNotExistException;
use common\libraries\ParameterNotDefinedException;

/**
 * $Id: attachment_viewer.class.php 216 2009-11-13 14:08:06Z kariboe $
 * 
 * @package application.lib.weblcms.tool.component
 */
class HomeManagerAttachmentViewerComponent extends HomeManager
{

    private $action_bar;

    public function run()
    {
        $trail = BreadcrumbTrail :: get_instance();
        // $trail->add_help('courses general');
        
        $failed = false;
        $error_message = '';
        
        // retrieve the parent content object
        $parent_id = Request :: get(HomeManager :: PARAM_PARENT_ID);
        
        if (is_null($parent_id))
        {
            throw new ParameterNotDefinedException(HomeManager :: PARAM_PARENT_ID);
        }
        $parent = \repository\DataManager :: retrieve_content_object($parent_id);
        
        if (is_null($parent))
        {
            throw new ObjectNotExistException(Translation :: get('Object'), $parent);
        }
        
        // retrieve the attachment
        $object_id = Request :: get(HomeManager :: PARAM_OBJECT_ID);
        
        if (is_null($object_id))
        {
            $failed = true;
            $error_message = Translation :: get('NoAttachmentSelected');
        }
        
        $object = \repository\DataManager :: retrieve_content_object($object_id);
        
        if (is_null($object))
        {
            $failed = true;
            throw new ObjectNotExistException(Translation :: get('Attachment'), $object);
        }
        
        // Default the attachment is attached to the content object of the
        // publication
        if (! $parent->is_attached_to_or_included_in($object_id))
        
        {
            $failed = true;
            $error_message = Translation :: get('WrongObjectSelected');
        }
        
        if (! $failed)
        {
            $trail->add(
                    new Breadcrumb($this->get_url(array('object' => $parent_id)), 
                            Translation :: get('ViewAttachment', null, 'repository')));
            
            Display :: small_header();
            
            // echo '<a href="javascript:history.go(-1)">' . Translation :: get('Back', null, Utilities ::
            // COMMON_LIBRARIES) . '</a><br /><br />';
            
            // $object = \repository\DataManager :: retrieve_content_object($object_id);
            
            echo ContentObjectRenditionImplementation :: launch($object, ContentObjectRendition :: FORMAT_HTML, 
                    ContentObjectRendition :: VIEW_FULL, $this);
            
            Display :: small_footer();
        }
        else
        {
            Display :: small_header();
            $this->display_error_message($error_message);
            Display :: small_footer();
        }
    }

    public function get_additional_parameters()
    {
        $params = array();
        return $params;
    }

    /**
     * Constructs the attachment url for the given attachment and the current object.
     * 
     * @param ContentObject $attachment The attachment for which the url is needed.
     * @return mixed the url, or null if no view right.
     */
    public function get_content_object_display_attachment_url($attachment)
    {
        $object_id = Request :: get(self :: PARAM_OBJECT_ID);
        $object = \repository\DataManager :: retrieve_content_object($object_id);
        
        if (! $this->is_view_attachment_allowed($object))
        {
            return null;
        }
        return $this->get_url(
                array(self :: PARAM_PARENT_ID => $object_id, self :: PARAM_OBJECT_ID => $attachment->get_id()));
    }

    /**
     * Determines whether the object may be viewed by the current user.
     * 
     * @param ContentObject $object The content object to be tested.
     * @return boolean Whether the current user may view the content object.
     */
    public function is_view_attachment_allowed($object)
    {
        // Is the current user the owner?
        return $object->get_owner_id() == $this->get_user_id();
    }
}
