<?php
namespace core\home;

use common\libraries\PlatformSetting;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\Authentication;

/**
 * $Id: editor.class.php 227 2009-11-13 14:45:05Z kariboe $
 *
 * @package home.lib.home_manager.component
 */
/**
 * Repository manager component to edit an existing learning object.
 */
class HomeManagerEditorComponent extends HomeManager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $id = Request :: get(HomeManager :: PARAM_HOME_ID);
        $type = Request :: get(HomeManager :: PARAM_HOME_TYPE);

        $user = $this->get_user();
        $user_home_allowed = PlatformSetting :: get('allow_user_home', 'core\home');

        if ($user_home_allowed && Authentication :: is_valid())
        {
            $user_id = $user->get_id();
        }

        if ($id && $type)
        {
            $url = $this->get_url();
            switch ($type)
            {
                case HomeManager :: TYPE_BLOCK :
                    $object = HomeDataManager :: get_instance()->retrieve_home_block($id);
                    $form = new HomeBlockForm(HomeBlockForm :: TYPE_EDIT, $object, $url);
                    break;
                case HomeManager :: TYPE_COLUMN :
                    $object = $this->retrieve_home_column($id);
                    $form = new HomeColumnForm(HomeColumnForm :: TYPE_EDIT, $object, $url);
                    break;
                case HomeManager :: TYPE_ROW :
                    $object = $this->retrieve_home_row($id);
                    $form = new HomeRowForm(HomeRowForm :: TYPE_EDIT, $object, $url);
                    break;
                case HomeManager :: TYPE_TAB :
                    $object = $this->retrieve_home_tab($id);
                    $form = new HomeTabForm(HomeTabForm :: TYPE_EDIT, $object, $url);
                    break;
            }

            if ($object->get_user() == $user_id || (! $object->get_user() && $user->is_platform_admin()))
            {
                if ($form->validate())
                {
                    $success = $form->update_object();
                    $this->redirect($success);
                }
                else
                {
                    $this->display_header();
                    $form->display();
                    $this->display_footer();
                }
            }
            else
            {
                $this->display_error_page(
                    htmlentities(Translation :: get('NoObjectSelected', null, Utilities :: COMMON_LIBRARIES)));
            }
        }
        else
        {
            $this->display_error_page(
                htmlentities(Translation :: get('NoObjectSelected', null, Utilities :: COMMON_LIBRARIES)));
        }
    }

    public function get_additional_parameters()
    {
        return array(HomeManager :: PARAM_HOME_TYPE, HomeManager :: PARAM_HOME_ID);
    }
}
