<?php
namespace core\home\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\Registration;
use common\libraries\Utilities;
use common\libraries\AndCondition;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\core\home\HomeDataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('core\home');
                $registration = CommonDataManager :: get_registration('home');
                $registration->set_version($package->get_version());
                $registration->set_context($package->get_context());
                if (! $registration->update())
                {
                    return false;
                }

                if (! \tracking\TrackingDataManager :: get_instance()->drop_storage_unit('home_changes'))
                {
                    return false;
                }

                $conditions = array();
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(
                        \tracking\TrackerRegistration :: class_name(),
                        \tracking\TrackerRegistration :: PROPERTY_TRACKER),
                    new StaticConditionVariable('home_changes_tracker'));
                $conditions[] = new EqualityCondition(
                    new PropertyConditionVariable(
                        \tracking\TrackerRegistration :: class_name(),
                        \tracking\TrackerRegistration :: PROPERTY_CONTEXT),
                    new StaticConditionVariable('home'));
                $condition = new AndCondition($conditions);

                if (! \tracking\TrackingDataManager :: get_instance()->new_delete(
                    \tracking\TrackerRegistration :: class_name(),
                    $condition))
                {
                    return false;
                }

                return true;
            });

        if (! $success)
        {
            return false;
        }

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \common\libraries\Setting :: class_name(),
                \common\libraries\Setting :: PROPERTY_CONTEXT),
            new StaticConditionVariable('home'));
        $settings = CommonDataManager :: retrieves(
            \common\libraries\Setting :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($setting = $settings->next_result())
        {
            $setting->set_context('core\home');
            $setting->update();
        }

        // Home
        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \core\home\HomeBlockRegistration :: class_name(),
                \core\home\HomeBlockRegistration :: PROPERTY_CONTEXT),
            new StaticConditionVariable('home'));
        $home_block_registrations = \core\home\DataManager :: retrieves(
            \core\home\HomeBlockRegistration :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($home_block_registration = $home_block_registrations->next_result())
        {
            $home_block_registration->set_context('core\home\integration\core\home');
            if($home_block_registration->get_block() == 'static')
            {
                $home_block_registration->set_block('static_content');
            }
            $home_block_registration->update();
        }

        $package_info = Package :: get('core\home\integration\core\home');

        $application_registration = new Registration();
        $parent_namespace = Utilities :: get_namespace_parent('core\home\integration\core\home');
        $application_registration->set_type((! $parent_namespace ? 'core' : $parent_namespace));
        $application_registration->set_name($package_info->get_code());
        $application_registration->set_category($package_info->get_category());
        $application_registration->set_version($package_info->get_version());
        $application_registration->set_status(Registration :: STATUS_ACTIVE);
        $application_registration->set_context('core\home\integration\core\home');

        if (! $application_registration->create())
        {
            return false;
        }

        return true;
    }
}
