<?php
namespace core\install;

use common\libraries\Utilities;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\Text;

class ExternalInstanceInstallWizardPage extends InstallWizardPage
{

    public function buildForm()
    {
        $this->set_lang($this->controller->exportValue('page_language', 'install_language'));
        $this->_formBuilt = true;

        $packages = $this->get_package_info();
        $this->get_package_tabs($packages);

        $buttons = array();
        $buttons[] = $this->createElement(
            'style_submit_button',
            $this->getButtonName('back'),
            Translation :: get('Previous', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'normal previous'));
        $buttons[] = $this->createElement(
            'style_submit_button',
            $this->getButtonName('next'),
            Translation :: get('Next', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'normal next'));
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
        $this->setDefaultAction($this->getButtonName('next'));
    }

    public function get_package_info()
    {
        $packages = array();
        $external_instances = \common\extensions\external_repository_manager\Manager :: get_packages_from_filesystem();

        foreach ($external_instances as $external_instance)
        {

            $package_info = \common\libraries\package\Package :: get($external_instance);
            $packages[$package_info->get_category()][$package_info->get_name()] = $package_info;
        }

        $external_instances = \common\extensions\video_conferencing_manager\Manager :: get_packages_from_filesystem();

        foreach ($external_instances as $external_instance)
        {

            $package_info = \common\libraries\package\Package :: get($external_instance);
            $packages[$package_info->get_category()][$package_info->get_name()] = $package_info;
        }

        ksort($packages);
        return $packages;
    }

    public function get_package_tabs($categories)
    {
        $html = array();

        $html[] = '<div class="clear"></div>';

        $html[] = '<div id="selectbuttons" style="padding-left: 10px; display: none;"><br />';
        $html[] = '<a href="#" id="selectall">' . Translation :: get('SelectAll', null, Utilities :: COMMON_LIBRARIES) .
             '</a>';
        $html[] = ' - ';
        $html[] = '<a href="#" id="unselectall">' .
             Translation :: get('UnselectAll', null, Utilities :: COMMON_LIBRARIES) . '</a>';
        $html[] = '</div><br />';

        $html[] = '<div id="tabs">';
        $html[] = '<ul>';

        // Render the tabs
        $index = 0;
        foreach ($categories as $category => $packages)
        {
            $index ++;

            $category_name = Translation :: get(
                Utilities :: underscores_to_camelcase($category),
                null,
                'common\extensions\external_repository_manager');

            $html[] = '<li><a href="#tabs-' . $index . '">';
            $html[] = '<span class="category">';
            $html[] = '<img src="../../common/extensions/external_repository_manager/resources/images/aqua/category_' .
                 $category . '.png" border="0" style="vertical-align: middle;" alt="' . $category_name . '" title="' .
                 $category_name . '"/>';
            $html[] = '<span class="title">' . $category_name . '</span>';
            $html[] = '</span>';
            $html[] = '</a></li>';
        }

        $html[] = '</ul>';

        $this->addElement('html', implode("\n", $html));
        $renderer = $this->defaultRenderer();

        $index = 0;
        foreach ($categories as $category => $packages)
        {
            $category_name = Translation :: get(
                Utilities :: underscores_to_camelcase($category),
                null,
                'common\extensions\external_repository_manager');
            $index ++;

            $html = array();
            $html[] = '<h2><img src="../../common/extensions/external_repository_manager/resources/images/aqua/category_' .
                 $category . '.png" border="0" style="vertical-align: middle;" alt="' . $category_name . '" title="' .
                 $category_name . '"/>&nbsp;' . $category_name . '</h2>';
            $html[] = '<div class="tab" id="tabs-' . $index . '">';
            $html[] = '<a class="prev"></a>';
            $html[] = '<div class="scrollable">';
            $html[] = '<div class="items">';
            $this->addElement('html', implode("\n", $html));

            $count = 0;

            ksort($packages);

            foreach ($packages as $package)
            {
                $count ++;

                $html = array();
                $html[] = '<div class="vertical_action"' . ($count == 1 ? ' style="border-top: 0px solid #FAFCFC;"' : '') .
                     '>';
                $html[] = '<div class="icon">';
                $html[] = '<a href="#"><img src="../../' . Path :: namespace_to_path($package->get_context()) .
                     '/resources/images/aqua/logo/32.png" alt="' . $package->get_name() . '" title="' .
                     $package->get_name() . '"/></a>';
                $html[] = '</div>';
                $html[] = '<div class="description">';
                $html[] = '<h4>' . $package->get_name() . '</h4>';
                $html[] = $package->get_description();
                $html[] = '<br />';
                $this->addElement('html', implode("\n", $html));

                $this->addElement(
                    'checkbox',
                    'external_instance[' . $package->get_context() . ']',
                    '',
                    '',
                    array('class' => 'application_check'));
                $renderer->setElementTemplate('{element}', 'external_instance[' . $package->get_context() . ']');

                $this->addElement('html', '</div></div>');
            }

            $html = array();
            $html[] = '</div>';
            $html[] = '</div>';
            $html[] = '<a class="next"></a>';
            $html[] = '<div class="clear"></div>';
            $html[] = '</div>';
            $this->addElement('html', implode("\n", $html));
        }

        $html = array();
        $html[] = '</div>';
        $html[] = '<script type="text/javascript" src="resources/javascript/install.js"></script>';
        $this->addElement('html', implode("\n", $html));

        return implode("\n", $html);
    }
}
