<?php
namespace core\install;

/**
 * This class helps to notify all the observers at the same time
 */
class InstallerObservers implements InstallerObserver
{

    /**
     *
     * @var array list aof all the observers to be notified
     */
    private $observers;

    public function __construct()
    {
        $this->observers = array();
    }

    public function add_observer(InstallerObserver $observer)
    {
        $this->observers[] = $observer;
    }

    private function call_on_each_observer($method_name, $parameters = null)
    {
        if (is_null($parameters))
        {
            $parameters = array();
        }

        if (! is_array($parameters))
        {
            $parameters = array($parameters);
        }
        foreach ($this->observers as $observer)
        {
            call_user_func_array(array($observer, $method_name), $parameters);
        }
    }

    public function before_install()
    {
        $this->call_on_each_observer(__FUNCTION__);
    }

    public function before_preprod()
    {
        $this->call_on_each_observer(__FUNCTION__);
    }

    public function preprod_config_file_written(InstallerStepResult $result)
    {
        $this->call_on_each_observer(__FUNCTION__, $result);
    }

    public function preprod_db_created(InstallerStepResult $result)
    {
        $this->call_on_each_observer(__FUNCTION__, $result);
    }

    public function after_preprod()
    {
        $this->call_on_each_observer(__FUNCTION__);
    }

    public function before_filesystem_prepared()
    {
        $this->call_on_each_observer(__FUNCTION__);
    }

    public function after_filesystem_prepared(InstallerStepResult $result)
    {
        $this->call_on_each_observer(__FUNCTION__, $result);
    }

    public function before_packages_install()
    {
        $this->call_on_each_observer(__FUNCTION__);
    }

    public function before_package_install($context)
    {
        $this->call_on_each_observer(__FUNCTION__, $context);
    }

    public function after_package_install(InstallerStepResult $result)
    {
        $this->call_on_each_observer(__FUNCTION__, $result);
    }

    public function after_packages_install()
    {
        $this->call_on_each_observer(__FUNCTION__);
    }

    public function after_install()
    {
        $this->call_on_each_observer(__FUNCTION__);
    }
}
