<?php
namespace core\lynx\manager;

use common\libraries\DelegateComponent;
use core\admin\AdminManager;
use common\libraries\CommonDataManager;
use common\libraries\Toolbar;
use common\libraries\Registration;
use common\libraries\Application;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\Display;
use common\libraries\DataClassCountParameters;
use common\libraries\SortableTableFromArray;
use common\libraries\DynamicContentTab;
use common\libraries\DynamicTabsRenderer;
use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\ActionBarRenderer;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\Request;
use common\libraries\Redirect;
use common\libraries\EqualityCondition;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\package\Package;

class BrowserComponent extends Manager implements DelegateComponent
{

    private $action_bar;

    private $current_type;
    const STATUS_INSTALLED = 1;
    const STATUS_AVAILABLE = 2;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $this->add_admin_breadcrumb();
        $this->action_bar = $this->get_action_bar();

        $this->current_type = Request :: get(Manager :: PARAM_REGISTRATION_TYPE);
        $this->current_type = $this->current_type ? $this->current_type : 'application';

        $this->set_parameter(Manager :: PARAM_REGISTRATION_TYPE, $this->current_type);

        $menu = new PackageTypeMenu(
            $this->current_type,
            $this->get_url(array(Manager :: PARAM_REGISTRATION_TYPE => '__type__')));

        $this->display_header();

        echo $this->action_bar->as_html();

        echo '<div style="float: left; width: 17%;">';
        echo $menu->render_as_tree();
        echo '</div>';
        echo '<div style="float: right; width: 82%;">';

        echo $this->get_content();

        echo '<div class="clear">&nbsp;</div>';
        echo '</div>';

        $this->display_footer();
    }

    public function get_content()
    {
        $tabs = new DynamicTabsRenderer(Utilities :: get_classname_from_object($this, true));

        $count = CommonDataManager :: count(
            Registration :: class_name(),
            new DataClassCountParameters(new EqualityCondition(Registration :: PROPERTY_TYPE, $this->get_type())));

        if ($count > 0)
        {
            $tabs->add_tab(
                new DynamicContentTab(
                    self :: STATUS_INSTALLED,
                    Translation :: get('InstalledPackages'),
                    Theme :: get_image_path() . 'status/' . self :: STATUS_INSTALLED . '.png',
                    $this->get_registered_packages_table()));
        }

        $packages = \common\libraries\package\PackageList :: get(false)->get_all_packages();

        if (count($packages[$this->get_type()]) > 0)
        {
            $tabs->add_tab(
                new DynamicContentTab(
                    self :: STATUS_AVAILABLE,
                    Translation :: get('AvailablePackages'),
                    Theme :: get_image_path() . 'status/' . self :: STATUS_AVAILABLE . '.png',
                    $this->get_available_packages_table()));
        }

        if ($tabs->size() > 0)
        {
            return $tabs->render();
        }
        else
        {
            return Display :: normal_message(Translation :: get('NoPackagesAvailableInContextGetSomeNow'), true);
        }
    }

    public function get_registered_packages_table()
    {
        $parameters = new DataClassRetrievesParameters(
            new EqualityCondition(Registration :: PROPERTY_TYPE, $this->get_type()));
        $registrations = CommonDataManager :: retrieves(Registration :: class_name(), $parameters);

        $table_data = array();

        while ($registration = $registrations->next_result())
        {
            $toolbar = new Toolbar();

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('ViewPackageDetails'),
                    Theme :: get_image_path() . 'action/view.png',
                    $this->get_url(
                        array(
                            Manager :: PARAM_ACTION => Manager :: ACTION_VIEW,
                            Manager :: PARAM_CONTEXT => $registration->get_context())),
                    ToolbarItem :: DISPLAY_ICON));

            if ($registration->is_active())
            {
                if (! is_subclass_of(
                    $registration->get_context() . '\Deactivator',
                    'common\libraries\package\NotAllowed'))
                {
                    $toolbar->add_item(
                        new ToolbarItem(
                            Translation :: get('Deactivate', array(), Utilities :: COMMON_LIBRARIES),
                            Theme :: get_image_path() . 'action/deactivate.png',
                            $this->get_url(
                                array(
                                    Manager :: PARAM_ACTION => Manager :: ACTION_DEACTIVATE,
                                    Manager :: PARAM_CONTEXT => $registration->get_context())),
                            ToolbarItem :: DISPLAY_ICON));
                }

                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('UpgradeNotAllowed', array(), Utilities :: COMMON_LIBRARIES),
                        Theme :: get_image_path() . 'action/upgrade_na.png',
                        null,
                        ToolbarItem :: DISPLAY_ICON));
            }
            else
            {
                if (! is_subclass_of($registration->get_context() . '\Activator', 'common\libraries\package\NotAllowed'))
                {
                    $toolbar->add_item(
                        new ToolbarItem(
                            Translation :: get('Activate', array(), Utilities :: COMMON_LIBRARIES),
                            Theme :: get_image_path() . 'action/activate.png',
                            $this->get_url(
                                array(
                                    Manager :: PARAM_ACTION => Manager :: ACTION_ACTIVATE,
                                    Manager :: PARAM_CONTEXT => $registration->get_context())),
                            ToolbarItem :: DISPLAY_ICON));
                }

                $package = Package :: get($registration->get_context());

                if ($package && version_compare($package->get_version(), $registration->get_version(), '>'))
                {
                    $toolbar->add_item(
                        new ToolbarItem(
                            Translation :: get('Upgrade', array(), Utilities :: COMMON_LIBRARIES),
                            Theme :: get_image_path() . 'action/upgrade.png',
                            $this->get_url(
                                array(
                                    Manager :: PARAM_ACTION => Manager :: ACTION_UPGRADE,
                                    Manager :: PARAM_CONTEXT => $registration->get_context())),
                            ToolbarItem :: DISPLAY_ICON));
                }
                else
                {
                    $toolbar->add_item(
                        new ToolbarItem(
                            Translation :: get('UpgradeNotAllowed', array(), Utilities :: COMMON_LIBRARIES),
                            Theme :: get_image_path() . 'action/upgrade_na.png',
                            null,
                            ToolbarItem :: DISPLAY_ICON));
                }
            }

            if (! is_subclass_of($registration->get_context() . '\Remover', 'common\libraries\package\NotAllowed'))
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Remove', array(), Utilities :: COMMON_LIBRARIES),
                        Theme :: get_image_path() . 'action/remove.png',
                        $this->get_url(
                            array(
                                Manager :: PARAM_ACTION => Manager :: ACTION_REMOVE,
                                Manager :: PARAM_CONTEXT => $registration->get_context())),
                        ToolbarItem :: DISPLAY_ICON));
            }

            $row = array();
            $row[] = Translation :: get('TypeName', null, $registration->get_context());
            $row[] = $toolbar->as_html();

            $table_data[] = $row;
        }

        $table = new SortableTableFromArray($table_data, 0, 20, 'registered_packages');
        $table->set_additional_parameters($this->get_parameters());
        $table->set_header(0, Translation :: get('Package'));
        $table->set_header(1, '', false);
        return $table->as_html();
    }

    public function get_available_packages_table()
    {
        $packages = \common\libraries\package\PackageList :: get(false)->get_all_packages();

        $table_data = array();

        foreach ($packages[$this->current_type] as $package)
        {
            $package_info = \common\libraries\package\Package :: get($package);

            $toolbar = new Toolbar();
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('ViewPackageDetails'),
                    Theme :: get_image_path() . 'action/view.png',
                    $this->get_url(
                        array(
                            self :: PARAM_ACTION => self :: ACTION_VIEW,
                            self :: PARAM_CONTEXT => $package_info->get_context())),
                    ToolbarItem :: DISPLAY_ICON));
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Install'),
                    Theme :: get_image_path() . 'action/install.png',
                    $this->get_url(
                        array(
                            self :: PARAM_ACTION => self :: ACTION_INSTALL,
                            self :: PARAM_CONTEXT => $package_info->get_context())),
                    ToolbarItem :: DISPLAY_ICON));

            $row = array();
            $row[] = Translation :: get('TypeName', null, $package);
            $row[] = $toolbar->as_html();

            $table_data[] = $row;
        }

        $table = new SortableTableFromArray($table_data, 0, 20, 'available_packages');
        $table->set_additional_parameters($this->get_parameters());
        $table->set_header(0, Translation :: get('Package'));
        $table->set_header(1, '', false);
        return $table->as_html();
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
        // $action_bar->set_search_url($this->get_url());

        $action_bar->add_common_action(
            new ToolbarItem(
                Translation :: get('ManageSources'),
                Theme :: get_image_path(\core\lynx\source\Manager :: context()) . 'logo/16.png',
                $this->get_url(
                    array(
                        \core\lynx\Manager :: PARAM_ACTION => \core\lynx\Manager :: ACTION_SOURCE,
                        Manager :: PARAM_ACTION => null)),
                ToolbarItem :: DISPLAY_ICON_AND_LABEL));

        $action_bar->add_common_action(
            new ToolbarItem(
                Translation :: get('BrowseRemote'),
                Theme :: get_image_path(\core\lynx\remote\Manager :: context()) . 'logo/16.png',
                $this->get_url(
                    array(
                        \core\lynx\Manager :: PARAM_ACTION => \core\lynx\Manager :: ACTION_REMOTE,
                        Manager :: PARAM_ACTION => null)),
                ToolbarItem :: DISPLAY_ICON_AND_LABEL));

        return $action_bar;
    }

    public function get_type()
    {
        return $this->current_type;
    }

    public function add_admin_breadcrumb()
    {
        $breadcrumb_trail = BreadcrumbTrail :: get_instance();
        $breadcrumbs = $breadcrumb_trail->get_breadcrumbs();
        array_splice(
            $breadcrumbs,
            1,
            0,
            array(
                new Breadcrumb(
                    Redirect :: get_url(array(Application :: PARAM_APPLICATION => AdminManager :: APPLICATION_NAME)),
                    Translation :: get('Administration'))));
        $breadcrumb_trail->set_breadcrumbtrail($breadcrumbs);
    }
}
