<?php
namespace core\lynx\manager;

use common\libraries\DelegateComponent;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\CommonDataManager;
use common\libraries\Theme;
use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\ActionBarRenderer;
use common\libraries\Request;
use common\libraries\Registration;

class ViewerComponent extends Manager implements DelegateComponent
{

    private $context;

    private $registration;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $this->context = Request :: get(self :: PARAM_CONTEXT);
        $this->registration = CommonDataManager :: get_registration($this->context);

        BreadcrumbTrail :: get_instance()->add(
                new Breadcrumb(null,
                        Translation :: get('ViewingPackage',
                                array('PACKAGE' => Translation :: get('TypeName', null, $this->context)))));

        $display = new PackageDisplay($this);

        $this->display_header();

        echo $this->get_action_bar()->as_html();
        echo $display->render();

        $this->display_footer();
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        if ($this->get_registration() instanceof Registration)
        {
            if ($this->get_registration()->is_active())
            {
            	if (! is_subclass_of($this->get_registration()->get_context() . '\Deactivator', 'common\libraries\package\NotAllowed'))
            	{
	                $action_bar->add_common_action(
	                        new ToolbarItem(Translation :: get('Deactivate', array(), Utilities :: COMMON_LIBRARIES),
	                                Theme :: get_image_path() . 'action/deactivate.png',
	                                $this->get_url(
	                                        array(self :: PARAM_ACTION => self :: ACTION_DEACTIVATE,
	                                                self :: PARAM_CONTEXT => $this->context))));
            	}
            }
            else
            {
            	if (! is_subclass_of($this->get_registration()->get_context() . '\Activator', 'common\libraries\package\NotAllowed'))
            	{
	                $action_bar->add_common_action(
	                        new ToolbarItem(Translation :: get('Activate', array(), Utilities :: COMMON_LIBRARIES),
	                                Theme :: get_image_path() . 'action/activate.png',
	                                $this->get_url(
	                                        array(self :: PARAM_ACTION => self :: ACTION_ACTIVATE,
	                                                self :: PARAM_CONTEXT => $this->context))));
            	}
            }
        }
        else
        {
            $action_bar->add_common_action(
                    new ToolbarItem(Translation :: get('Install', array(), Utilities :: COMMON_LIBRARIES),
                            Theme :: get_image_path() . 'action/install.png',
                            $this->get_url(
                                    array(self :: PARAM_ACTION => self :: ACTION_INSTALL,
                                            self :: PARAM_CONTEXT => $this->context)), ToolbarItem::DISPLAY_ICON_AND_LABEL, true));
        }

        return $action_bar;
    }

    public function get_context()
    {
        return $this->context;
    }

    public function get_registration()
    {
        return $this->registration;
    }
}
