<?php
namespace core\lynx\manager;

use common\libraries\Registration;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\NewObjectTableCellRenderer;
use common\libraries\NewObjectTableCellRendererActionsColumnSupport;
use common\libraries\package\Package;

class RegistrationTableCellRenderer extends NewObjectTableCellRenderer implements
    NewObjectTableCellRendererActionsColumnSupport
{

    public function render_cell($column, $registration)
    {
        switch ($column->get_name())
        {
            case Registration :: PROPERTY_NAME :
                return Translation :: get('TypeName', null, $registration->get_context());
        }

        return parent :: render_cell($column, $registration);
    }

    /**
     * Returns the actions toolbar
     *
     * @param $course Course
     * @return String
     */
    public function get_object_actions($registration)
    {
        $toolbar = new Toolbar();

        $toolbar->add_item(
            new ToolbarItem(
                Translation :: get('ViewPackageDetails'),
                Theme :: get_image_path() . 'action/view.png',
                $this->get_component()->get_url(
                    array(
                        Manager :: PARAM_ACTION => Manager :: ACTION_VIEW,
                        Manager :: PARAM_CONTEXT => $registration->get_context())),
                ToolbarItem :: DISPLAY_ICON));

        if ($registration->is_active())
        {
            if (! is_subclass_of($registration->get_context() . '\Deactivator', 'common\libraries\package\NotAllowed'))
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Deactivate', array(), Utilities :: COMMON_LIBRARIES),
                        Theme :: get_image_path() . 'action/deactivate.png',
                        $this->get_component()->get_url(
                            array(
                                Manager :: PARAM_ACTION => Manager :: ACTION_DEACTIVATE,
                                Manager :: PARAM_CONTEXT => $registration->get_context())),
                        ToolbarItem :: DISPLAY_ICON));
            }

            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('UpgradeNotAllowed', array(), Utilities :: COMMON_LIBRARIES),
                    Theme :: get_image_path() . 'action/upgrade_na.png',
                    null,
                    ToolbarItem :: DISPLAY_ICON));
        }
        else
        {
            if (! is_subclass_of($registration->get_context() . '\Activator', 'common\libraries\package\NotAllowed'))
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Activate', array(), Utilities :: COMMON_LIBRARIES),
                        Theme :: get_image_path() . 'action/activate.png',
                        $this->get_component()->get_url(
                            array(
                                Manager :: PARAM_ACTION => Manager :: ACTION_ACTIVATE,
                                Manager :: PARAM_CONTEXT => $registration->get_context())),
                        ToolbarItem :: DISPLAY_ICON));
            }

            $package = Package :: get($registration->get_context());

            if ($package && version_compare($package->get_version(), $registration->get_version(), '>'))
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('Upgrade', array(), Utilities :: COMMON_LIBRARIES),
                        Theme :: get_image_path() . 'action/upgrade.png',
                        $this->get_component()->get_url(
                            array(
                                Manager :: PARAM_ACTION => Manager :: ACTION_UPGRADE,
                                Manager :: PARAM_CONTEXT => $registration->get_context())),
                        ToolbarItem :: DISPLAY_ICON));
            }
            else {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('UpgradeNotAllowed', array(), Utilities :: COMMON_LIBRARIES),
                        Theme :: get_image_path() . 'action/upgrade_na.png',
                        null,
                        ToolbarItem :: DISPLAY_ICON));
            }
        }

        if (! is_subclass_of($registration->get_context() . '\Remover', 'common\libraries\package\NotAllowed'))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Remove', array(), Utilities :: COMMON_LIBRARIES),
                    Theme :: get_image_path() . 'action/remove.png',
                    $this->get_component()->get_url(
                        array(
                            Manager :: PARAM_ACTION => Manager :: ACTION_REMOVE,
                            Manager :: PARAM_CONTEXT => $registration->get_context())),
                    ToolbarItem :: DISPLAY_ICON));
        }

        return $toolbar->as_html();
    }
}
