<?php
namespace core\lynx;

use common\libraries\Redirect;
use common\libraries\PlatformSetting;
use common\libraries\DynamicAction;
use common\libraries\Theme;
use common\libraries\CommonDataManager;
use common\libraries\Translation;
use common\libraries\CoreApplication;

class Manager extends CoreApplication
{
    const APPLICATION_NAME = 'lynx';
    const ACTION_BROWSE = 'browser';
    const ACTION_SOURCE = 'source';
    const ACTION_REMOTE = 'remote';
    const ACTION_INSTALL = 'installer';
    const ACTION_UPGRADE = 'upgrader';
    const ACTION_CONTENT_OBJECT_UPGRADE = 'content_object_upgrader';
    const ACTION_APPLICATION_UPGRADE = 'application_upgrader';
    const DEFAULT_ACTION = self :: ACTION_BROWSE;

    /**
     * Helper function for the Application class, pending access to class constants via variables in PHP 5.3 e.g. $name
     * = $class :: DEFAULT_ACTION DO NOT USE IN THIS APPLICATION'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourApplicationManager :: DEFAULT_ACTION in all other application classes
     */
    public function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    public static function get_installable_application_packages($include_installed = false)
    {
        $package_list = new \common\libraries\package\PackageList(
            self :: context(),
            Translation :: get('TypeName', null, __NAMESPACE__),
            Theme :: get_image_path() . 'logo/16.png');

        $submanagers = array('core\lynx\manager', 'core\lynx\remote', 'core\lynx\source');

        foreach ($submanagers as $submanager)
        {
            if (! CommonDataManager :: get_registration($submanager) || $include_installed)
            {
                $package_list->add_package($submanager);
            }
        }

        return $package_list;
    }

    public static function get_application_platform_admin_links($application = self :: APPLICATION_NAME)
    {
        $links = array();

        $package_management = PlatformSetting :: get('enable_package_management');

        if ($package_management == '1')
        {
            $links[] = new DynamicAction(
                Translation :: get('ManagePackages'),
                Translation :: get('ManagePackagesDescription'),
                Theme :: get_image_path() . 'admin/build.png',
                Redirect :: get_link(self :: APPLICATION_NAME));
        }

        $info = parent :: get_application_platform_admin_links(self :: APPLICATION_NAME);
        $info['links'] = $links;

        return $info;
    }
}
