<?php
namespace core\lynx\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use core\lynx\manager\PackageInstaller;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\core\lynx\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('core\lynx');
                $registration = CommonDataManager :: get_registration('core\lynx');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                return true;
            });

        if (! $success)
        {
            return false;
        }

        $installer = new PackageInstaller('core\lynx\source');
        if (! $installer->run())
        {
            $this->add_message(\common\libraries\package\Action :: TYPE_ERROR, $installer->get_result(true));
            return false;
        }

        return true;
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::get_additional_packages()
     */
    public function get_additional_packages()
    {
        $packages = array();
        $packages[] = 'core\lynx\manager';
        $packages[] = 'core\lynx\remote';
        return $packages;
    }
}
