<?php
namespace core\menu;

use common\libraries\DataClassRetrievesParameters;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Utilities;
use common\libraries\Translation;
use common\libraries\ObjectTableOrder;
use common\libraries\OptionsMenuRenderer;
use common\libraries\TreeMenuRenderer;

use HTML_Menu;
use HTML_Menu_ArrayRenderer;

class ItemMenu extends HTML_Menu
{
    const TREE_NAME = __CLASS__;

    /**
     * The string passed to sprintf() to format category URLs
     */
    private $urlFmt;
    /**
     * The array renderer used to determine the breadcrumbs.
     */
    private $array_renderer;

    public function __construct($current_category, $url_format = '?item=%s', $condition = null)
    {
        $this->urlFmt = $url_format;

        $menu = $this->get_items($condition);
        parent :: __construct($menu);
        $this->array_renderer = new HTML_Menu_ArrayRenderer();
        $this->forceCurrentUrl($this->get_category_url($current_category));
    }

    private function get_items($condition)
    {
        $conditions[] = new EqualityCondition(Item :: PROPERTY_PARENT, 0);
        $conditions[] = new EqualityCondition(Item :: PROPERTY_TYPE, CategoryItem::class_name());
        $condition = new AndCondition($conditions);

        $parameters = new DataClassRetrievesParameters($condition, null, null, new ObjectTableOrder(Item :: PROPERTY_SORT));
        $items = DataManager :: retrieves(Item :: class_name(), $parameters);

        $menu_item = array();
        $menu_item['title'] = Translation :: get('Home');
        $menu_item['url'] = $this->get_category_url(0);
        $menu_item['class'] = 'home';
        $menu_item[OptionsMenuRenderer :: KEY_ID] = 0;

        $sub_menu_items = array();
        while ($item = $items->next_result())
        {
            $sub_menu_item = array();
            $sub_menu_item['title'] = $item->get_titles()->get_current_translation();
            $sub_menu_item['url'] = $this->get_category_url($item->get_id());
            $sub_menu_item['class'] = 'category';
            $sub_menu_item[OptionsMenuRenderer :: KEY_ID] = $item->get_id();
            $sub_menu_items[] = $sub_menu_item;
        }

        $menu_item['sub'] = $sub_menu_items;
        $menu[] = $menu_item;
        return $menu;
    }

    /**
     * Gets the URL of a given category
     * @param int $category The id of the category
     * @return string The requested URL
     */
    private function get_category_url($item_id)
    {
        return htmlentities(sprintf($this->urlFmt, $item_id));
    }

    /**
     * Get the breadcrumbs which lead to the current category.
     * @return array The breadcrumbs.
     */
    public function get_breadcrumbs()
    {
        $this->render($this->array_renderer, 'urhere');
        $breadcrumbs = $this->array_renderer->toArray();
        foreach ($breadcrumbs as $crumb)
        {
            $crumb['name'] = $crumb['title'];
            unset($crumb['title']);
        }
        return $breadcrumbs;
    }

    /**
     * Renders the menu as a tree
     * @return string The HTML formatted tree
     */
    public function render_as_tree()
    {
        $renderer = new TreeMenuRenderer($this->get_tree_name());
        $this->render($renderer, 'sitemap');
        return $renderer->toHTML();
    }

    public static function get_tree_name()
    {
        return Utilities :: get_classname_from_namespace(self :: TREE_NAME, true);
    }

    public function render_as_list()
    {
        $renderer = new OptionsMenuRenderer();
        $this->render($renderer, 'sitemap');
        $list = array('0' => Translation :: get('RootCategory')) + $renderer->toArray();
        return $list;
    }
}
