<?php
namespace core\menu;

use common\libraries\DelegateComponent;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\StringUtilities;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Request;

class MenuManagerEditorComponent extends MenuManager implements DelegateComponent
{

    public function run()
    {
        $this->check_allowed();
        $item = DataManager :: retrieve_item((int) Request :: get(MenuManager :: PARAM_ITEM));

        BreadcrumbTrail :: get_instance()->add(
            new Breadcrumb(
                null,
                Translation :: get('MenuManagerEditor', array('NAME' => $item->get_titles()->get_current_translation()))));

        $item_form = ItemForm :: factory(
            ItemForm :: TYPE_EDIT,
            $item,
            $this->get_url(array(MenuManager :: PARAM_ITEM => $item->get_id())));

        if ($item_form->validate())
        {
            $values = $item_form->exportValues();

            foreach ($item->get_default_property_names() as $property)
            {
                $item->set_default_property($property, $values[$property]);
            }

            foreach ($item->get_additional_property_names() as $property)
            {
                $item->set_additional_property($property, $values[$property]);
            }

            $titles = $item->get_titles();
            $item_titles = new ItemTitles();
            foreach ($values[ItemTitle :: PROPERTY_TITLE] as $isocode => $title)
            {
                if (! StringUtilities :: is_null_or_empty($title, true))
                {
                    $item_title = $titles->get_title_by_isocode($isocode);
                    if (! $item_title instanceof ItemTitle)
                    {
                        $item_title = new ItemTitle();
                        $item_title->set_item_id($item->get_id());
                        $item_title->set_isocode($isocode);
                    }
                    $item_title->set_title($title);
                    $item_titles->add($item_title);
                }
            }

            $item->set_titles($item_titles);
            $success = $item->update();

            if ($success)
            {
                $message = Translation :: get(
                    'ObjectCreated',
                    array('OBJECT' => Translation :: get('MenuManagerItem')),
                    Utilities :: COMMON_LIBRARIES);
            }
            else
            {
                $message = Translation :: get(
                    'ObjectNotCreated',
                    array('OBJECT' => Translation :: get('MenuManagerItem')),
                    Utilities :: COMMON_LIBRARIES);
            }

            $this->redirect(
                $message,
                ($success ? false : true),
                array(
                    MenuManager :: PARAM_ACTION => MenuManager :: ACTION_BROWSE,
                    MenuManager :: PARAM_ITEM => $item->get_parent()));
        }
        else
        {
            $this->display_header();
            $item_form->display();
            $this->display_footer();
        }
    }
}
