<?php
namespace core\menu\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\core\menu\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('core\menu');
                $registration = CommonDataManager :: get_registration('menu');
                $registration->set_version($package->get_version());
                $registration->set_context($package->get_context());
                if (! $registration->update())
                {
                    return false;
                }

                if (! \tracking\TrackingDataManager :: get_instance()->drop_storage_unit('menu_changes'))
                {
                    return false;
                }

                return true;
            });

        if (! $success)
        {
            return false;
        }

        $menu_item_types = array(
            'menu\ApplicationItem',
            'menu\CategoryItem',
            'menu\LinkItem',
            'menu\LinkApplicationItem');

        foreach ($menu_item_types as $menu_item_type)
        {
            $properties = new DataClassProperties();
            $properties->add(
                new DataClassProperty(
                    new PropertyConditionVariable(\core\menu\Item :: class_name(), \core\menu\Item :: PROPERTY_TYPE),
                    new StaticConditionVariable('core\\' . $menu_item_type)));

            $condition = new EqualityCondition(
                new PropertyConditionVariable(\core\menu\Item :: class_name(), \core\menu\Item :: PROPERTY_TYPE),
                new StaticConditionVariable($menu_item_type));

            if (! \core\menu\DataManager :: updates(\core\menu\Item :: class_name(), $properties, $condition))
            {
                return false;
            }
        }

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                \common\libraries\Setting :: class_name(),
                \common\libraries\Setting :: PROPERTY_CONTEXT),
            new StaticConditionVariable('menu'));
        $settings = CommonDataManager :: retrieves(
            \common\libraries\Setting :: class_name(),
            new DataClassRetrievesParameters($condition));

        while ($setting = $settings->next_result())
        {
            $setting->set_context('core\menu');
            if (! $setting->update())
            {
                return false;
            }
        }

        return true;
    }
}
