<?php

namespace core\metadata;

use common\libraries\Mdb2ConditionTranslator;

use common\libraries\Mdb2Database;
use common\libraries\EqualityCondition;

use common\libraries\AndCondition;
use common\libraries\ObjectTableOrder;

/**
 * This is a data manager that uses a database for storage. It was written
 * for MySQL, but should be compatible with most SQL flavors.
 *
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class Mdb2MetadataDataManager extends Mdb2Database implements MetadataDataManagerInterface
{

    public function initialize()
    {
        parent :: initialize();
        $this->set_prefix('metadata_');
    }

    public function get_next_metadata_attribute_association_id()
    {
        return $this->get_next_id(MetadataAttributeAssociation :: get_table_name());
    }

    public function create_metadata_attribute_association($metadata_attribute_association)
    {
        return $this->create($metadata_attribute_association);
    }

    public function update_metadata_attribute_association($metadata_attribute_association)
    {
        $condition = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_ID, $metadata_attribute_association->get_id());
        return $this->update($metadata_attribute_association, $condition);
    }

    public function delete_metadata_attribute_associations($table_name, $metadata_element)
    {
        $condition = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_PARENT_ID, $metadata_element->get_id());
        return $this->delete($table_name, $condition);
    }

    public function count_metadata_attribute_associations($condition = null)
    {
        return $this->count_objects(MetadataAttributeAssociation :: get_table_name(), $condition);
    }

    public function retrieve_metadata_attribute_association($id)
    {
        $condition = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_ID, $id);
        return $this->retrieve_object(MetadataAttributeAssociation :: get_table_name(), $condition, MetadataAttributeAssociation :: CLASS_NAME);
    }

    public function retrieve_metadata_attribute_associations($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(MetadataAttributeAssociation :: get_table_name(), $condition, $offset, $max_objects, $order_by, MetadataAttributeAssociation :: CLASS_NAME);
    }

    public function get_next_metadata_namespace_id()
    {
        return $this->get_next_id(MetadataNamespace :: get_table_name());
    }

    public function create_metadata_namespace($metadata_namespace)
    {
        return $this->create($metadata_namespace);
    }

    public function update_metadata_namespace($metadata_namespace)
    {
        $condition = new EqualityCondition(MetadataNamespace :: PROPERTY_ID, $metadata_namespace->get_id());
        return $this->update($metadata_namespace, $condition);
    }

    public function delete_metadata_namespace($metadata_namespace)
    {
        $condition = new EqualityCondition(MetadataNamespace :: PROPERTY_ID, $metadata_namespace->get_id());
        return $this->delete($metadata_namespace->get_table_name(), $condition);
    }

    public function count_metadata_namespaces($condition = null)
    {
        return $this->count_objects(MetadataNamespace :: get_table_name(), $condition);
    }

    public function retrieve_metadata_namespace($id)
    {
        $condition = new EqualityCondition(MetadataNamespace :: PROPERTY_ID, $id);
        return $this->retrieve_object(MetadataNamespace :: get_table_name(), $condition, null, MetadataNamespace :: CLASS_NAME);
    }

    public function retrieve_metadata_namespace_by_prefix($prefix)
    {
        $condition = new EqualityCondition(MetadataNamespace :: PROPERTY_NS_PREFIX, $prefix);
        $namespace_set = $this->retrieve_metadata_namespaces($condition);
        if ($namespace_set)
        {
            return $namespace_set->next_result();
        }
        else
        {
            return false;
        }
    }

    public function retrieve_metadata_namespaces($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(MetadataNamespace :: get_table_name(), $condition, $offset, $max_objects, $order_by, MetadataNamespace :: CLASS_NAME);
    }

    public function get_next_metadata_element_nesting_id()
    {
        return $this->get_next_id(MetadataElementNesting :: get_table_name());
    }

    public function create_metadata_element_nesting($metadata_element_nesting)
    {
        return $this->create($metadata_element_nesting);
    }

    public function update_metadata_element_nesting($metadata_element_nesting)
    {
        $condition = new EqualityCondition(MetadataElementNesting :: PROPERTY_ID, $metadata_element_nesting->get_id());
        return $this->update($metadata_element_nesting, $condition);
    }

    public function delete_metadata_element_nestings($table_name, $metadata_element)
    {
        $condition = new EqualityCondition(MetadataElementNesting :: PROPERTY_PARENT_ID, $metadata_element->get_id());
        return $this->delete($table_name, $condition);
    }

    public function count_metadata_element_nestings($condition = null)
    {
        return $this->count_objects(MetadataElementNesting :: get_table_name(), $condition);
    }

    public function retrieve_metadata_element_nesting($id)
    {
        $condition = new EqualityCondition(MetadataElementNesting :: PROPERTY_ID, $id);
        return $this->retrieve_object(MetadataElementNesting :: get_table_name(), $condition, null, MetadataElementNesting :: CLASS_NAME);
    }

    public function retrieve_metadata_element_nestings($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(MetadataElementNesting :: get_table_name(), $condition, $offset, $max_objects, $order_by, MetadataElementNesting :: CLASS_NAME);
    }

    public function get_next_content_object_property_metadata_id()
    {
        return $this->get_next_id(ContentObjectPropertyMetadata :: get_table_name());
    }

    public function create_content_object_property_metadata($content_object_property_metadata)
    {
        return $this->create($content_object_property_metadata);
    }

    public function update_content_object_property_metadata($content_object_property_metadata)
    {
        $condition = new EqualityCondition(ContentObjectPropertyMetadata :: PROPERTY_ID, $content_object_property_metadata->get_id());
        return $this->update($content_object_property_metadata, $condition);
    }

    public function delete_content_object_property_metadata($content_object_property_metadata)
    {
        $condition = new EqualityCondition(ContentObjectPropertyMetadata :: PROPERTY_ID, $content_object_property_metadata->get_id());
        return $this->delete($content_object_property_metadata->get_table_name(), $condition);
    }

    public function count_content_object_property_metadatas($condition = null)
    {
        return $this->count_objects(ContentObjectPropertyMetadata :: get_table_name(), $condition);
    }

    public function retrieve_content_object_property_metadata($id)
    {
        $condition = new EqualityCondition(ContentObjectPropertyMetadata :: PROPERTY_ID, $id);
        return $this->retrieve_object(ContentObjectPropertyMetadata :: get_table_name(), $condition, null, ContentObjectPropertyMetadata :: CLASS_NAME);
    }

    public function retrieve_content_object_property_metadatas($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(ContentObjectPropertyMetadata :: get_table_name(), $condition, $offset, $max_objects, $order_by, ContentObjectPropertyMetadata :: CLASS_NAME);
    }

    public function get_next_metadata_element_id()
    {
        return $this->get_next_id(MetadataElement :: get_table_name());
    }

    public function create_metadata_element($metadata_element)
    {
        return $this->create($metadata_element);
    }

    public function update_metadata_element($metadata_element)
    {
        $condition = new EqualityCondition(MetadataElement :: PROPERTY_ID, $metadata_element->get_id());
        return $this->update($metadata_element, $condition);
    }

    public function delete_metadata_element($metadata_element)
    {
        $condition = new EqualityCondition(MetadataElement :: PROPERTY_ID, $metadata_element->get_id());
        return $this->delete($metadata_element->get_table_name(), $condition);
    }

    public function count_metadata_elements($condition = null)
    {
        return $this->count_objects(MetadataElement :: get_table_name(), $condition);
    }

    public function retrieve_metadata_element($id)
    {
        $condition = new EqualityCondition(MetadataElement :: PROPERTY_ID, $id);
        return $this->retrieve_object(MetadataElement :: get_table_name(), $condition, null, MetadataElement :: CLASS_NAME);
    }

    public function retrieve_metadata_elements($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        if (is_null($order_by))
            $order_by = new ObjectTableOrder(MetadataElement :: PROPERTY_ORDER);
        return $this->retrieve_objects(MetadataElement :: get_table_name(), $condition, $offset, $max_objects, $order_by, MetadataElement :: CLASS_NAME);
    }

    public function retrieve_metadata_element_by_ns_name($namespace_id, $name)
    {
        $conditions[] = new EqualityCondition(MetadataElement :: PROPERTY_NAMESPACE, $namespace_id);
        $conditions[] = new EqualityCondition(MetadataElement :: PROPERTY_NAME, $name);
        $condition = new AndCondition($conditions);
        
        return $this->retrieve_object(MetadataElement :: get_table_name(), $condition, null, MetadataElement :: CLASS_NAME);
    
    }

    public function get_next_content_object_metadata_element_value_id()
    {
        return $this->get_next_id(ContentObjectMetadataElementValue :: get_table_name());
    }

    public function create_content_object_metadata_element_value($content_object_metadata_element_value)
    {
        return $this->create($content_object_metadata_element_value);
    }

    public function update_content_object_metadata_element_value($content_object_metadata_element_value)
    {
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $content_object_metadata_element_value->get_id());
        return $this->update($content_object_metadata_element_value, $condition);
    }

    public function delete_content_object_metadata_element_value($content_object_metadata_element_value)
    {
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $content_object_metadata_element_value->get_id());
        $success = $this->delete($content_object_metadata_element_value->get_table_name(), $condition);
        
        $this->delete_dependent_attribute_values($content_object_metadata_element_value);
        return $success;
    }

    public function delete_dependent_attribute_values($metadata_element_value)
    {
        $condition1 = new EqualityCondition(MetadataAttributeValue :: PROPERTY_PARENT_ID, $metadata_element_value->get_id());
        $condition2 = new EqualityCondition(MetadataAttributeValue :: PROPERTY_CONTENT_OBJECT_ID, null);
        
        $condition = new AndCondition($condition1, $condition2);
        
        $this->delete(MetadataAttributeValue :: get_table_name(), $condition);
    }

    public function count_content_object_metadata_element_values($condition = null)
    {
        return $this->count_objects(ContentObjectMetadataElementValue :: get_table_name(), $condition);
    }

    public function retrieve_content_object_metadata_element_value($id)
    {
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $id);
        return $this->retrieve_object(ContentObjectMetadataElementValue :: get_table_name(), $condition, null, ContentObjectMetadataElementValue :: CLASS_NAME);
    }

    public function retrieve_content_object_metadata_element_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(ContentObjectMetadataElementValue :: get_table_name(), $condition, $offset, $max_objects, $order_by, ContentObjectMetadataElementValue :: CLASS_NAME);
    }
    
    /*
     * retrieves metadata property values from content object preformatted
     * @return array property values [$n] = prefix : property type => prefix property value
     */
    
    public function retrieve_full_content_object_metadata_element_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        $namespace_alias = $this->get_alias(MetadataNamespace :: get_table_name());
        $type_alias = $this->get_alias(MetadataElement :: get_table_name());
        $value_alias = $this->get_alias(ContentObjectMetadataElementValue :: get_table_name());
        
        $query = 'SELECT ' . $value_alias . '.' . ContentObjectMetadataElementValue :: PROPERTY_ID . ', ' . $value_alias . '.' . MetadataElementValue :: PROPERTY_VALUE . ', ' . $namespace_alias . '.' . MetadataNamespace :: PROPERTY_NS_PREFIX . ', ' . $type_alias . '.' . MetadataElement :: PROPERTY_NAME . ',' . $type_alias . '.' . MetadataElement :: PROPERTY_ID . ' AS element_id';
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectMetadataElementValue :: get_table_name()) . ' AS ' . $value_alias;
        $query .= ' LEFT JOIN ' . $this->escape_table_name(MetadataElement :: get_table_name()) . ' AS ' . $type_alias;
        $query .= ' ON ' . parent :: escape_column_name(MetadataElementValue :: PROPERTY_ELEMENT_ID, $value_alias) . ' = ' . parent :: escape_column_name(MetadataElement :: PROPERTY_ID, $type_alias);
        $query .= ' LEFT JOIN ' . $this->escape_table_name(MetadataNamespace :: get_table_name()) . ' AS ' . $namespace_alias;
        $query .= ' ON ' . parent :: escape_column_name(MetadataNamespace :: PROPERTY_ID, $namespace_alias) . ' = ' . parent :: escape_column_name(MetadataElement :: PROPERTY_NAMESPACE, $type_alias);
        
        $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition);
        
        return $this->retrieve_object_set($query, ContentObjectMetadataElementValue :: get_table_name());
    }
    
    /*
     * retrieves metadata properties and values
     * @return array property values [prefix : property type] => prefix property value
     */
    
    public function retrieve_content_object_metadata_properties_and_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        $namespace_alias = $this->get_alias(MetadataNamespace :: get_table_name());
        $type_alias = $this->get_alias(MetadataElement :: get_table_name());
        $value_alias = $this->get_alias(ContentObjectMetadataElementValue :: get_table_name());
        
        $query = 'SELECT ' . $value_alias . '.' . MetadataElementValue :: PROPERTY_ID . ', ' . $value_alias . '.' . MetadataElementValue :: PROPERTY_VALUE . ', ' . $namespace_alias . '.' . MetadataNamespace :: PROPERTY_NS_PREFIX . ', ' . $type_alias . '.' . MetadataElement :: PROPERTY_NAME;
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectMetadataElementValue :: get_table_name()) . ' AS ' . $value_alias;
        $query .= ' LEFT JOIN ' . $this->escape_table_name(MetadataElement :: get_table_name()) . ' AS ' . $type_alias;
        $query .= ' ON ' . parent :: escape_column_name(MetadataElementValue :: PROPERTY_ELEMENT_ID, $value_alias) . ' = ' . parent :: escape_column_name(MetadataElement :: PROPERTY_ID, $type_alias);
        $query .= ' LEFT JOIN ' . $this->escape_table_name(MetadataNamespace :: get_table_name()) . ' AS ' . $namespace_alias;
        $query .= ' ON ' . parent :: escape_column_name(MetadataNamespace :: PROPERTY_ID, $namespace_alias) . ' = ' . parent :: escape_column_name(MetadataElement :: PROPERTY_NAMESPACE, $type_alias);
        
        $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition);
        
        $res = $this->query($query);
        
        $element_values = array();
        while ($record = $res->fetchRow(MDB2_FETCHMODE_ASSOC))
        {
            $element_values[$record[MetadataElement :: PROPERTY_NS_PREFIX] . ':' . $record[MetadataElement :: PROPERTY_NAME]] = $record[MetadataElementValue :: PROPERTY_VALUE];
        }
        $res->free();
        return $element_values;
    }

    public function get_next_user_metadata_element_value_id()
    {
        return $this->get_next_id(UserMetadataElementValue :: get_table_name());
    }

    public function create_user_metadata_element_value($user_metadata_element_value)
    {
        return $this->create($user_metadata_element_value);
    }

    public function update_user_metadata_element_value($user_metadata_element_value)
    {
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $user_metadata_element_value->get_id());
        return $this->update($user_metadata_element_value, $condition);
    }

    public function delete_user_metadata_element_value($user_metadata_element_value)
    {
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $user_metadata_element_value->get_id());
        $success = $this->delete($user_metadata_element_value->get_table_name(), $condition);
        
        $this->delete_dependent_attribute_values($user_metadata_element_value);
        return $success;
    }

    public function count_user_metadata_element_values($condition = null)
    {
        return $this->count_objects(UserMetadataElementValue :: get_table_name(), $condition);
    }

    public function retrieve_user_metadata_element_value($id)
    {
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $id);
        return $this->retrieve_object(UserMetadataElementValue :: get_table_name(), $condition, null, UserMetadataElementValue :: CLASS_NAME);
    }

    public function retrieve_user_metadata_element_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(UserMetadataElementValue :: get_table_name(), $condition, $offset, $max_objects, $order_by, UserMetadataElementValue :: CLASS_NAME);
    }

    public function retrieve_all_user_metadata_element_values($user_id)
    {
        //TODO: also retrieve metadata for all user's groups!
        $condition = new EqualityCondition(UserMetadataElementValue :: PROPERTY_USER_ID, $user_id);
        return $this->retrieve_objects(UserMetadataElementValue :: get_table_name());
    }

    public function retrieve_all_user_metadata_element_values_as_array($user_id)
    {
        $user_metadata = self :: retrieve_all_user_metadata_element_values($user_id);
        $user_metadata_array = array();
        while ($metadata = $user_metadata->next_result())
        {
            $metadata_property = self :: retrieve_metadata_element($metadata->get_element_id());
            $prefix = $metadata_property->get_ns_prefix();
            $element = $metadata_property->get_name();
            $user_metadata_array[$prefix . ':' . $element] = $metadata->get_value();
        }
        
        return $user_metadata_array;
    }

    public function retrieve_content_object_metadata_element_values_as_array($co_id)
    {
        $condition = new EqualityCondition(ContentObjectMetadataElementValue :: PROPERTY_CONTENT_OBJECT_ID, $co_id);
        //        $object_metadata_array = self::retrieve_full_content_object_metadata_element_values($condition);
        //        return $object_metadata_array;
        $namespace_alias = $this->get_alias(MetadataNamespace :: get_table_name());
        $type_alias = $this->get_alias(MetadataElement :: get_table_name());
        $value_alias = $this->get_alias(ContentObjectMetadataElementValue :: get_table_name());
        
        $query = 'SELECT ' . $value_alias . '.' . MetadataElementValue :: PROPERTY_ID . ', ' . $value_alias . '.' . MetadataElementValue :: PROPERTY_VALUE . ', ' . $namespace_alias . '.' . MetadataNamespace :: PROPERTY_NS_PREFIX . ', ' . $type_alias . '.' . MetadataElement :: PROPERTY_NAME;
        $query .= ' FROM ' . $this->escape_table_name(ContentObjectMetadataElementValue :: get_table_name()) . ' AS ' . $value_alias;
        $query .= ' LEFT JOIN ' . $this->escape_table_name(MetadataElement :: get_table_name()) . ' AS ' . $type_alias;
        $query .= ' ON ' . parent :: escape_column_name(MetadataElementValue :: PROPERTY_ELEMENT_ID, $value_alias) . ' = ' . parent :: escape_column_name(MetadataElement :: PROPERTY_ID, $type_alias);
        $query .= ' LEFT JOIN ' . $this->escape_table_name(MetadataNamespace :: get_table_name()) . ' AS ' . $namespace_alias;
        $query .= ' ON ' . parent :: escape_column_name(MetadataNamespace :: PROPERTY_ID, $namespace_alias) . ' = ' . parent :: escape_column_name(MetadataElement :: PROPERTY_NAMESPACE, $type_alias);
        
        $query .= ' WHERE ' . Mdb2ConditionTranslator :: render($condition);
        
        $res = $this->query($query);
        
        $element_values = array();
        while ($record = $res->fetchRow(MDB2_FETCHMODE_ASSOC))
        {
            $element_values[$record[MetadataElement :: PROPERTY_NS_PREFIX] . ':' . $record[MetadataElement :: PROPERTY_NAME]] = $record[MetadataElementValue :: PROPERTY_VALUE];
        }
        $res->free();
        return $element_values;
    
    }

    public function get_next_group_metadata_element_value_id()
    {
        return $this->get_next_id(GroupMetadataElementValue :: get_table_name());
    }

    public function create_group_metadata_element_value($group_metadata_element_value)
    {
        return $this->create($group_metadata_element_value);
    }

    public function update_group_metadata_element_value($group_metadata_element_value)
    {
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $group_metadata_element_value->get_id());
        return $this->update($group_metadata_element_value, $condition);
    }

    public function delete_group_metadata_element_value($group_metadata_element_value)
    {
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $group_metadata_element_value->get_id());
        $succes = $this->delete($group_metadata_element_value->get_table_name(), $condition);
        
        $this->delete_dependent_attribute_values($group_metadata_element_value);
        
        return $succes;
    }

    public function count_group_metadata_element_values($condition = null)
    {
        return $this->count_objects(GroupMetadataElementValue :: get_table_name(), $condition);
    }

    public function retrieve_group_metadata_element_value($id)
    {
        $condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $id);
        return $this->retrieve_object(GroupMetadataElementValue :: get_table_name(), $condition, null, GroupMetadataElementValue :: CLASS_NAME);
    }

    public function retrieve_group_metadata_element_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(GroupMetadataElementValue :: get_table_name(), $condition, $offset, $max_objects, $order_by, GroupMetadataElementValue :: CLASS_NAME);
    }
    
    //      function get_next_metadata_element_value_id()
    //	{
    //		return $this->get_next_id(MetadataElementValue :: get_table_name());
    //	}
    //
    //	function create_metadata_element_value($metadata_element_value)
    //	{
    //		return $this->create($metadata_element_value);
    //	}
    //
    //	function update_metadata_element_value($metadata_element_value)
    //	{
    //		$condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $metadata_element_value->get_id());
    //		return $this->update($metadata_element_value, $condition);
    //	}
    //
    //	function delete_metadata_element_value($metadata_element_value)
    //	{
    //		$condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $metadata_element_value->get_id());
    //		return $this->delete($metadata_element_value->get_table_name(), $condition);
    //	}
    //
    //	function count_metadata_element_values($condition = null)
    //	{
    //		return $this->count_objects(MetadataElementValue :: get_table_name(), $condition);
    //	}
    //
    //	function retrieve_metadata_element_value($id)
    //	{
    //		$condition = new EqualityCondition(MetadataElementValue :: PROPERTY_ID, $id);
    //		return $this->retrieve_object(MetadataElementValue :: get_table_name(), $condition, null, MetadataElementValue :: CLASS_NAME);
    //	}
    //
    //	function retrieve_metadata_element_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    //	{
    //		return $this->retrieve_objects(MetadataElementValue :: get_table_name(), $condition, $offset, $max_objects, $order_by, MetadataElementValue :: CLASS_NAME);
    //	}
    //
    //        function retrieve_full_metadata_element_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    //        {
    //            $type_alias = $this->get_alias(MetadataElement :: get_table_name());
    //            $value_alias = $this->get_alias(MetadataElementValue :: get_table_name());
    //
    //            $query = 'SELECT ' . $value_alias . '.' . MetadataElementValue :: PROPERTY_ID . ', ' . $value_alias . '.' . MetadataElementValue :: PROPERTY_VALUE . ', ' .$type_alias . '.' . MetadataElement :: PROPERTY_NS_PREFIX . ', ' . $type_alias . '.' . MetadataElement :: PROPERTY_NAME;
    //            $query .= ' FROM ' . $this->escape_table_name(MetadataElementValue :: get_table_name()) . ' AS ' . $value_alias;
    //            $query .= ' LEFT JOIN ' . $this->escape_table_name(MetadataElement :: get_table_name()) . ' AS ' . $type_alias;
    //            $query .= ' ON ' . parent :: escape_column_name(MetadataElementValue :: PROPERTY_ELEMENT_ID, $value_alias) . ' = '. parent :: escape_column_name(MetadataElement :: PROPERTY_ID, $type_alias);
    //
    //            $translator = ConditionTranslator :: factory($this);
    //            $query .= $translator->render_query($condition);
    //
    //            $res = $this->query($query);
    //
    //            $element_values = array();
    //            while ($record = $res->fetchRow(MDB2_FETCHMODE_ASSOC))
    //            {
    //                $element_values[$record[MetadataElementValue :: PROPERTY_ID]] = $record[MetadataElement :: PROPERTY_NS_PREFIX] . ':' . $record[MetadataElement :: PROPERTY_NAME] . '=' .$record[MetadataElementValue :: PROPERTY_VALUE];
    //            }
    //            $res->free();
    //            return $element_values;
    //        }
    

    public function get_next_metadata_attribute_id()
    {
        return $this->get_next_id(MetadataAttribute :: get_table_name());
    }

    public function create_metadata_attribute($metadata_attribute)
    {
        return $this->create($metadata_attribute);
    }

    public function update_metadata_attribute($metadata_attribute)
    {
        $condition = new EqualityCondition(MetadataAttribute :: PROPERTY_ID, $metadata_attribute->get_id());
        return $this->update($metadata_attribute, $condition);
    }

    public function delete_metadata_attribute($metadata_attribute)
    {
        $condition = new EqualityCondition(MetadataAttribute :: PROPERTY_ID, $metadata_attribute->get_id());
        return $this->delete($metadata_attribute->get_table_name(), $condition);
    }

    public function count_metadata_attributes($condition = null)
    {
        return $this->count_objects(MetadataAttribute :: get_table_name(), $condition);
    }

    public function retrieve_metadata_attribute($id)
    {
        $condition = new EqualityCondition(MetadataAttribute :: PROPERTY_ID, $id);
        return $this->retrieve_object(MetadataAttribute :: get_table_name(), $condition, null, MetadataAttribute :: CLASS_NAME);
    }

    public function retrieve_metadata_attributes($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(MetadataAttribute :: get_table_name(), $condition, $offset, $max_objects, $order_by, MetadataAttribute :: CLASS_NAME);
    }

    public function get_next_metadata_attribute_value_id()
    {
        return $this->get_next_id(MetadataAttributeValue :: get_table_name());
    }

    public function create_metadata_attribute_value($metadata_attribute_value)
    {
        return $this->create($metadata_attribute_value);
    }

    public function update_metadata_attribute_value($metadata_attribute_value)
    {
        $condition = new EqualityCondition(MetadataAttributeValue :: PROPERTY_ID, $metadata_attribute_value->get_id());
        return $this->update($metadata_attribute_value, $condition);
    }

    public function delete_metadata_attribute_value($metadata_attribute_value)
    {
        $condition = new EqualityCondition(MetadataAttributeValue :: PROPERTY_ID, $metadata_attribute_value->get_id());
        return $this->delete($metadata_attribute_value->get_table_name(), $condition);
    }

    public function count_metadata_attribute_values($condition = null)
    {
        return $this->count_objects(MetadataAttributeValue :: get_table_name(), $condition);
    }

    public function retrieve_metadata_attribute_value($id)
    {
        $condition = new EqualityCondition(MetadataAttributeValue :: PROPERTY_ID, $id);
        return $this->retrieve_object(MetadataAttributeValue :: get_table_name(), $condition, null, MetadataAttributeValue :: CLASS_NAME);
    }

    public function retrieve_metadata_attribute_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(MetadataAttributeValue :: get_table_name(), $condition, $offset, $max_objects, $order_by, MetadataAttributeValue :: CLASS_NAME);
    }

    public function retrieve_metadata_default_values($condition = null, $offset = null, $max_objects = null, $order_by = null)
    {
        return $this->retrieve_objects(MetadataDefaultValue :: get_table_name(), $condition, $offset, $max_objects, $order_by, MetadataDefaultValue :: CLASS_NAME);
    }

    public function count_metadata_default_values($condition = null)
    {
        return $this->count_objects(MetadataDefaultValue :: get_table_name(), $condition);
    }

    public function retrieve_metadata_default_value($id)
    {
        $condition = new EqualityCondition(MetadataDefaultValue :: PROPERTY_ID, $id);
        return $this->retrieve_object(MetadataDefaultValue :: get_table_name(), $condition, null, MetadataDefaultValue :: CLASS_NAME);
    }

    public function create_metadata_default_value(MetadataDefaultValue $metadata_default_value)
    {
        return $this->create($metadata_default_value);
    }

    public function update_metadata_default_value(MetadataDefaultValue $metadata_default_value)
    {
        $condition = new EqualityCondition(MetadataDefaultValue :: PROPERTY_ID, $metadata_default_value->get_id());
        return $this->update($metadata_default_value, $condition);
    }

    public function delete_metadata_default_value(MetadataDefaultValue $metadata_default_value)
    {
        $condition = new EqualityCondition(MetadataDefaultValue :: PROPERTY_ID, $metadata_default_value->get_id());
        return $this->delete(MetadataDefaultValue :: get_table_name(), $condition);
    }
    
    /*
     * retrieve prefixes of namespaces that have property types set
     */
    
    public function retrieve_prefixes()
    {
        $namespace_alias = $this->get_alias(MetadataNamespace :: get_table_name());
        $type_alias = $this->get_alias(MetadataElement :: get_table_name());
        
        $query = 'SELECT DISTINCT ' . $namespace_alias . '.' . MetadataNamespace :: PROPERTY_NS_PREFIX . ', ' . $namespace_alias . '.' . MetadataNamespace :: PROPERTY_ID;
        $query .= ' FROM ' . $this->escape_table_name(MetadataNamespace :: get_table_name()) . ' AS ' . $namespace_alias;
        $query .= ' INNER JOIN ' . $this->escape_table_name(MetadataElement :: get_table_name()) . ' AS ' . $type_alias;
        $query .= ' ON ' . parent :: escape_column_name(MetadataElement :: PROPERTY_NAMESPACE, $type_alias) . ' = ' . parent :: escape_column_name(MetadataNamespace :: PROPERTY_ID, $namespace_alias);
        
        //$translator = ConditionTranslator :: factory($this);
        //$query .= $translator->render_query(null);
        

        $res = $this->query($query);
        
        $prefixes = array();
        while ($record = $res->fetchRow(MDB2_FETCHMODE_ASSOC))
        {
            $prefixes[$record[MetadataNamespace :: PROPERTY_ID]] = $record[MetadataNamespace :: PROPERTY_NS_PREFIX];
        }
        $res->free();
        return $prefixes;
    }

    public function get_content_object_publication_attributes($object_id, $type = null, $condition = null, $count = null, $offset = null,  $order_properties = null)
    {
    
    }

    public function get_content_object_publication_attribute($publication_id)
    {
    
    }

    public function count_publication_attributes($user = null, $object_id = null, $condition = null)
    {
        return 0;
    }

}
