<?php
/**
 * This class describes the form for a MetadataElementValue object.
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 **/
namespace core\metadata;

use common\libraries\Translation;
use common\libraries\Utilities;

class GroupMetadataEditorForm extends MetadataForm
{
    const TYPE = 'group';
    
    private $metadata_element_values = array();
    private $group;
    private $application;

    public function __construct($group, $metadata_element_values, $action, $application)
    {
        $this->set_parent_type(self :: TYPE);
        
        parent :: __construct('group_metadata_element_value_settings', 'post', $action);
        
        $this->group = $group;
        $this->form_type = $form_type;
        $this->application = $application;
        $this->metadata_element_values = $metadata_element_values;
        
        $this->build_creation_form();
        
        $this->setDefaults();
    }

    public function build_basic_form()
    {
        $this->addElement('hidden', parent :: PARENT_ID);
        
        $this->build_empty_element_value();
        
        $this->addElement('category', Translation :: get('ElementValues'));
        $this->build_metadata_element_values();
        $this->addElement('category');
    }

    public function build_creation_form()
    {
        $this->build_basic_form();
        
        $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('Create', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive'));
        $buttons[] = $this->createElement('style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'normal empty'));
        
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function edit_metadata()
    {
        $values = $this->exportValues();
        
        $fails = 0;
        
        if (! parent :: edit_metadata())
            $fails ++;
        
     //update existing property values
        foreach ($this->metadata_element_values as $metadata_element_value)
        {
            $cond = $values[MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id()] != $this->_defaults[MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id()];
            if ($cond)
            {
                $metadata_element_value->set_value($values[MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id()]);
                
                if (! $metadata_element_value->update())
                    $fails ++;
            }
        }
        return ($fails) ? false : true;
    }

    /**
     * Sets default values.
     * @param array $defaults Default values for this form's parameters.
     */
    public function setDefaults($defaults = array ())
    {
        $defaults[parent :: PARENT_ID] = $this->group->get_id();
        
        //metadata property values
        foreach ($this->metadata_element_values as $metadata_element_value)
        {
            $defaults[MetadataElementValue :: PROPERTY_ELEMENT_ID . '_' . $metadata_element_value->get_id()] = $metadata_element_value->get_element_id();
            $defaults[MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id()] = $metadata_element_value->get_value();
        }
        parent :: setDefaults($defaults);
    }

    public function get_value_types()
    {
        $value_types = array();
        $value_types[MetadataAttributeValue :: VALUE_TYPE_NONE] = '--';
        $value_types[MetadataAttributeValue :: VALUE_TYPE_ID] = Translation :: get('Id', null, Utilities :: COMMON_LIBRARIES);
        $value_types[MetadataAttributeValue :: VALUE_TYPE_VALUE] = Translation :: get('Value', null, Utilities :: COMMON_LIBRARIES);
        
        return $value_types;
    }

    public function build_metadata_element_values()
    {
        foreach ($this->metadata_element_values as $metadata_element_value)
        {
            $group = array();
            
            $group[] = $this->createElement('text', MetadataElementValue :: PROPERTY_VALUE . '_' . $metadata_element_value->get_id(), Translation :: get('MetadataElementValue'));
            $group[] = $this->createElement('static', null, null, '<a href="' . $this->application->get_url(array(
                    MetadataManager :: PARAM_ACTION => MetadataManager :: ACTION_DELETE_GROUP_METADATA_PROPERTY_VALUE, 
                    MetadataManager :: PARAM_METADATA_PROPERTY_VALUE => $metadata_element_value->get_id(), 
                    MetadataManager :: PARAM_GROUP => $this->group->get_id())) . '">delete</a>');
            
            $elements = $this->get_elements();
            
            $this->addGroup($group, '', $elements[$metadata_element_value->get_element_id()]);
        }
    }
}
