<?php
namespace core\metadata;

use common\libraries\FormValidator;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * This class describes the form for a MetadataAttribute object.
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 **/
class MetadataAttributeForm extends FormValidator
{
    const TYPE_CREATE = 1;
    const TYPE_EDIT = 2;
    
    const OPTION_BLANK = 'blank';
    
    private $metadata_attribute;
    private $user;

    public function __construct($form_type, $metadata_attribute, $action, $user)
    {
        parent :: __construct('metadata_attribute_settings', 'post', $action);
        
        $this->metadata_attribute = $metadata_attribute;
        $this->user = $user;
        $this->form_type = $form_type;
        
        if ($this->form_type == self :: TYPE_EDIT)
        {
            $this->build_editing_form();
        }
        elseif ($this->form_type == self :: TYPE_CREATE)
        {
            $this->build_creation_form();
        }
        
        $this->setDefaults();
    }

    public function build_basic_form()
    {
        //$this->addElement('text', MetadataAttribute :: PROPERTY_ID, Translation :: get('Id'));
        //$this->addRule(MetadataAttribute :: PROPERTY_ID, Translation :: get('ThisFieldIsRequired'), 'required');
        

        $mm = MetadataDataManager :: get_instance();
        $prefixes = $mm->retrieve_metadata_namespaces();
        
        $options = array();
        $options[self :: OPTION_BLANK] = '--';
        
        while ($prefix = $prefixes->next_result())
        {
            $options[$prefix->get_id()] = $prefix->get_ns_prefix();
        }
        
        $this->addElement('select', MetadataAttribute :: PROPERTY_NAMESPACE, Translation :: get('NsPrefix'), $options);
        
        $this->addElement('text', MetadataAttribute :: PROPERTY_NAME, Translation :: get('Name', null, Utilities :: COMMON_LIBRARIES));
        $this->addRule(MetadataAttribute :: PROPERTY_NAME, Translation :: get('ThisFieldIsRequired', null, Utilities :: COMMON_LIBRARIES), 'required');
    
     //        $value_types = array();
    //        $value_types[MetadataAttribute :: VALUE_TYPE_NONE] = '--';
    //        $value_types[MetadataAttribute :: VALUE_TYPE_ID] = Translation :: get('id');
    //        $value_types[MetadataAttribute :: VALUE_TYPE_VALUE] = Translation :: get('value');
    //
    //        $this->addElement('select', MetadataAttribute :: PROPERTY_VALUE_TYPE, Translation :: get('ValueType'), $value_types);
    //$this->addRule(MetadataAttribute :: PROPERTY_VALUE_TYPE, Translation :: get('ThisFieldIsRequired'), 'required');
    

    //        $attribute_types = $mm->retrieve_metadata_attributes();
    //
    //        $attribute_options = array();
    //        $attribute_options[self :: OPTION_BLANK] = '--';
    //
    //        while($attribute_type = $attribute_types->next_result())
    //        {
    //            $prefix = (is_null($attribute_type->get_ns_prefix())) ? '' : $attribute_type->get_ns_prefix() . ':';
    //            $value = (is_null($attribute_type->get_value())) ? '' : '=' . $attribute_type->get_value();
    //            if($attribute_type->get_id() != $this->metadata_attribute->get_id()) $attribute_options[$attribute_type->get_id()] = $prefix . $attribute_type->get_name() . $value;
    //
    //        }
    

    //        $this->addElement('select', MetadataAttribute :: PROPERTY_VALUE, Translation :: get('Value'), $attribute_options);
    //        //$this->addRule(MetadataAttribute :: PROPERTY_VALUE, Translation :: get('ThisFieldIsRequired'), 'required');
    //        $this->addElement('text', MetadataAttribute :: PROPERTY_VALUE. '_2', Translation :: get('Value'));
    }

    public function build_editing_form()
    {
        $this->build_basic_form();
        
        $this->addElement('hidden', MetadataAttribute :: PROPERTY_ID);
        
        $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('Update', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive update'));
        $buttons[] = $this->createElement('style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'normal empty'));
        
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function build_creation_form()
    {
        $this->build_basic_form();
        
        $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('Create', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive'));
        $buttons[] = $this->createElement('style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'normal empty'));
        
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function update_metadata_attribute()
    {
        $metadata_attribute = $this->metadata_attribute;
        $values = $this->exportValues();
        
        $metadata_attribute->set_id($values[MetadataAttribute :: PROPERTY_ID]);
        $metadata_attribute->set_namespace(($values[MetadataAttribute :: PROPERTY_NAMESPACE] != self :: OPTION_BLANK) ? $values[MetadataAttribute :: PROPERTY_NAMESPACE] : null);
        $metadata_attribute->set_name($values[MetadataAttribute :: PROPERTY_NAME]);
        //$metadata_attribute->set_value_type($values[MetadataAttribute :: PROPERTY_VALUE_TYPE]);
        

        //        $metadata_attribute->set_value_type($values[MetadataAttribute :: PROPERTY_VALUE_TYPE]);
        //
        //        switch($metadata_attribute->get_value_type())
        //        {
        //            case MetadataAttribute :: VALUE_TYPE_ID:
        //                $metadata_attribute->set_value($values[MetadataAttribute :: PROPERTY_VALUE]);
        //                break;
        //            case MetadataAttribute :: VALUE_TYPE_VALUE:
        //                 $metadata_attribute->set_value($values[MetadataAttribute :: PROPERTY_VALUE . '_2']);
        //                break;
        //            case MetadataAttribute :: VALUE_TYPE_NONE:
        //                 $metadata_attribute->set_value(null);
        //                break;
        //        }
        

        return $metadata_attribute->update();
    }

    public function create_metadata_attribute()
    {
        $metadata_attribute = $this->metadata_attribute;
        $values = $this->exportValues();
        
        //$metadata_attribute->set_id($values[MetadataAttribute :: PROPERTY_ID]);
        $metadata_attribute->set_namespace(($values[MetadataAttribute :: PROPERTY_NAMESPACE] != self :: OPTION_BLANK) ? $values[MetadataAttribute :: PROPERTY_NAMESPACE] : null);
        $metadata_attribute->set_name($values[MetadataAttribute :: PROPERTY_NAME]);
        //$metadata_attribute->set_value_type($values[MetadataAttribute :: PROPERTY_VALUE_TYPE]);
        

        //        $metadata_attribute->set_value_type($values[MetadataAttribute :: PROPERTY_VALUE_TYPE]);
        //
        //        switch($metadata_attribute->get_value_type())
        //        {
        //            case MetadataAttribute :: VALUE_TYPE_ID:
        //                $metadata_attribute->set_value($values[MetadataAttribute :: PROPERTY_VALUE]);
        //                break;
        //            case MetadataAttribute :: VALUE_TYPE_VALUE:
        //                 $metadata_attribute->set_value($values[MetadataAttribute :: PROPERTY_VALUE . '_2']);
        //                break;
        //            case MetadataAttribute :: VALUE_TYPE_NONE:
        //                 $metadata_attribute->set_value(null);
        //                break;
        //        }
        

        return $metadata_attribute->create();
    }

    /**
     * Sets default values.
     * @param array $defaults Default values for this form's parameters.
     */
    public function setDefaults($defaults = array ())
    {
        $metadata_attribute = $this->metadata_attribute;
        
        $defaults[MetadataAttribute :: PROPERTY_ID] = $metadata_attribute->get_id();
        $defaults[MetadataAttribute :: PROPERTY_NAMESPACE] = $metadata_attribute->get_namespace();
        $defaults[MetadataAttribute :: PROPERTY_NAME] = $metadata_attribute->get_name();
        
        //            $defaults[MetadataAttribute :: PROPERTY_VALUE_TYPE] = $metadata_attribute->get_value_type();
        //            if($metadata_attribute->get_value_type() == MetadataAttribute :: VALUE_TYPE_ID)
        //            {
        //               $defaults[MetadataAttribute :: PROPERTY_VALUE] = $metadata_attribute->get_value();
        //            }else
        //            {
        //               $defaults[MetadataAttribute :: PROPERTY_VALUE . '_2'] = $metadata_attribute->get_value();
        //            }
        

        parent :: setDefaults($defaults);
    }
}
