<?php
namespace core\metadata;

use common\libraries\FormValidator;
use common\libraries\Translation;
use common\libraries\Utilities;

/**
 * This class describes the form for a MetadataElement object.
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 **/
class MetadataElementForm extends FormValidator
{
    const TYPE_CREATE = 1;
    const TYPE_EDIT = 2;
    
    const ACTION_ASSOCIATE = 'associate';
    
    private $metadata_element;
    private $user;

    public function __construct($form_type, $metadata_element, $action, $user)
    {
        parent :: __construct('metadata_element_settings', 'post', $action);
        
        $this->metadata_element = $metadata_element;
        $this->user = $user;
        $this->form_type = $form_type;
        
        if ($this->form_type == self :: TYPE_EDIT)
        {
            $this->build_editing_form();
        }
        elseif ($this->form_type == self :: TYPE_CREATE)
        {
            $this->build_creation_form();
        }
        
        $this->setDefaults();
    }

    public function build_basic_form()
    {
        $mm = MetadataDataManager :: get_instance();
        $prefixes = $mm->retrieve_metadata_namespaces();
        
        while ($prefix = $prefixes->next_result())
        {
            $options[$prefix->get_id()] = $prefix->get_ns_prefix();
        }
        
        $this->addElement('select', MetadataElement :: PROPERTY_NAMESPACE, Translation :: get('NsPrefix'), $options);
        $this->addRule(MetadataElement :: PROPERTY_NAMESPACE, Translation :: get('ThisFieldIsRequired', null, Utilities :: COMMON_LIBRARIES), 'required');
        
        $this->addElement('text', MetadataElement :: PROPERTY_NAME, Translation :: get('Name', null, Utilities :: COMMON_LIBRARIES));
        $this->addRule(MetadataElement :: PROPERTY_NAME, Translation :: get('ThisFieldIsRequired', null, Utilities :: COMMON_LIBRARIES), 'required');
        
        $this->addElement('checkbox', self :: ACTION_ASSOCIATE, Translation :: get('Associate', null, Utilities :: COMMON_LIBRARIES));
    
    }

    public function build_editing_form()
    {
        $this->build_basic_form();
        
        $this->addElement('hidden', MetadataElement :: PROPERTY_ID);
        
        $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('Update', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive update'));
        $buttons[] = $this->createElement('style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'normal empty'));
        
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function build_creation_form()
    {
        $this->build_basic_form();
        
        $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('Create', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'positive'));
        $buttons[] = $this->createElement('style_reset_button', 'reset', Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array(
                'class' => 'normal empty'));
        
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function update_metadata_element()
    {
        $metadata_element = $this->metadata_element;
        $values = $this->exportValues();
        
        $metadata_element->set_id($values[MetadataElement :: PROPERTY_ID]);
        $metadata_element->set_namespace($values[MetadataElement :: PROPERTY_NAMESPACE]);
        $metadata_element->set_name($values[MetadataElement :: PROPERTY_NAME]);
        
        return $metadata_element->update();
    }

    public function create_metadata_element()
    {
        $metadata_element = $this->metadata_element;
        $values = $this->exportValues();
        
        $metadata_element->set_namespace($values[MetadataElement :: PROPERTY_NAMESPACE]);
        $metadata_element->set_name($values[MetadataElement :: PROPERTY_NAME]);
        
        if ($metadata_element->create())
        {
            return $metadata_element;
        }
    }

    /**
     * Sets default values.
     * @param array $defaults Default values for this form's parameters.
     */
    public function setDefaults($defaults = array ())
    {
        $metadata_element = $this->metadata_element;
        
        $defaults[MetadataElement :: PROPERTY_ID] = $metadata_element->get_id();
        $defaults[MetadataElement :: PROPERTY_NAMESPACE] = $metadata_element->get_namespace();
        $defaults[MetadataElement :: PROPERTY_NAME] = $metadata_element->get_name();
        
        parent :: setDefaults($defaults);
    }
}
