<?php
namespace core\metadata;
use common\libraries\DataClass;
use common\libraries\Utilities;
/**
 * metadata
 */

/**
 * This class describes a MetadataElementValue data object
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
abstract class MetadataElementValue extends DataClass
{
    const CLASS_NAME = __CLASS__;
    
    /**
     * MetadataElementValue properties
     */
    const PROPERTY_CONTENT_OBJECT_ID = 'content_object_id';
    const PROPERTY_ELEMENT_ID = 'element_id';
    const PROPERTY_VALUE = 'value';

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_ID, self :: PROPERTY_CONTENT_OBJECT_ID, self :: PROPERTY_ELEMENT_ID, 
                self :: PROPERTY_VALUE);
    }

    public function get_data_manager()
    {
        return MetadataDataManager :: get_instance();
    }

    /**
     * Returns the content_object_id of this MetadataElementValue.
     * @return the content_object_id.
     */
    public function get_content_object_id()
    {
        return $this->get_default_property(self :: PROPERTY_CONTENT_OBJECT_ID);
    }

    /**
     * Sets the content_object_id of this MetadataElementValue.
     * @param content_object_id
     */
    public function set_content_object_id($content_object_id)
    {
        $this->set_default_property(self :: PROPERTY_CONTENT_OBJECT_ID, $content_object_id);
    }

    /**
     * Returns the element_id of this MetadataElementValue.
     * @return the element_id.
     */
    public function get_element_id()
    {
        return $this->get_default_property(self :: PROPERTY_ELEMENT_ID);
    }

    /**
     * Sets the element_id of this MetadataElementValue.
     * @param element_id
     */
    public function set_element_id($element_id)
    {
        $this->set_default_property(self :: PROPERTY_ELEMENT_ID, $element_id);
    }

    /**
     * Returns the value of this MetadataElementValue.
     * @return the value.
     */
    public function get_value()
    {
        return $this->get_default_property(self :: PROPERTY_VALUE);
    }

    /**
     * Sets the value of this MetadataElementValue.
     * @param value
     */
    public function set_value($value)
    {
        $this->set_default_property(self :: PROPERTY_VALUE, $value);
    }

    public static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(Utilities :: get_classname_from_namespace(self :: CLASS_NAME));
    }
}
