<?php
namespace core\metadata;

use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\OrCondition;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\Redirect;
use repository\RepositoryManager;

/**
 * Component to edit an existing metadata_element_value object
 * 
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class MetadataManagerContentObjectMetadataEditorComponent extends MetadataManager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $content_object = \repository\DataManager :: retrieve_content_object(
                Request :: get(MetadataManager :: PARAM_CONTENT_OBJECT));
        
        if ($content_object)
        {
            $trail = BreadcrumbTrail :: get_instance();
            
            $trail->add(
                    new BreadCrumb(
                            Redirect :: get_link(RepositoryManager :: APPLICATION_NAME, 
                                    array(
                                            RepositoryManager :: PARAM_ACTION => RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS), 
                                    array(), false, Redirect :: TYPE_CORE), Translation :: get('Repository')));
            $trail->add(
                    new BreadCrumb(
                            Redirect :: get_link(RepositoryManager :: APPLICATION_NAME, 
                                    array(
                                            RepositoryManager :: PARAM_ACTION => RepositoryManager :: ACTION_VIEW_CONTENT_OBJECTS, 
                                            RepositoryManager :: PARAM_CONTENT_OBJECT_ID => Request :: get(
                                                    MetadataManager :: PARAM_CONTENT_OBJECT)), array(), false, 
                                    Redirect :: TYPE_CORE), Translation :: get('ContentObject')));
            $trail->add(new BreadCrumb($this->get_url(), Translation :: get('Metadata')));
            
            // fetches wich properties of the content object should automatically be converted to metadata
            $content_object_property_metadata = $this->get_content_object_property_metadata_values($content_object);
            
            // fetches corresponding values
            $condition = new EqualityCondition(ContentObjectMetadataElementValue :: PROPERTY_CONTENT_OBJECT_ID, 
                    $content_object->get_id());
            $content_object_metadata_element_values = $this->retrieve_content_object_metadata_element_values($condition);
            $content_object_metadata_element_values = $this->format_metadata_element_values(
                    $content_object_metadata_element_values);
            
            // conditions to eventually find all attribute values (of the default values as well as the regular ones)
            // search where the parent_id is teh current content object id and where the relation is contnet object
            // property -> group with and
            $content_object_condition1 = new EqualityCondition(MetadataAttributeValue :: PROPERTY_CONTENT_OBJECT_ID, 
                    Request :: get(MetadataManager :: PARAM_CONTENT_OBJECT));
            $content_object_condition2 = new EqualityCondition(MetadataAttributeValue :: PROPERTY_RELATION, 
                    MetadataAttributeValue :: RELATION_CONTENT_OBJECT_PROPERTY);
            $content_object_condition = new AndCondition($content_object_condition1, $content_object_condition2);
            
            $element_value_conditions = array();
            
            // elaborate
            // wehere parent_id = metadataPropertyvalue
            foreach ($content_object_metadata_element_values as $id => $metadata_element_value)
            {
                $element_value_conditions[] = new EqualityCondition(MetadataAttributeValue :: PROPERTY_PARENT_ID, 
                        $metadata_element_value->get_id());
                $conditions_allowed[] = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_PARENT_ID, 
                        $metadata_element_value->get_element_id());
            }
            
            foreach ($content_object_property_metadata as $id => $object)
            {
                $condition1 = new EqualityCondition(MetadataAttributeValue :: PROPERTY_PARENT_ID, $object->get_id());
                $condition2 = new EqualityCondition(MetadataAttributeValue :: PROPERTY_CONTENT_OBJECT_ID, 
                        $content_object->get_id());
                $element_value_conditions[] = new AndCondition($condition1, $condition2);
                $conditions_allowed[] = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_PARENT_ID, 
                        $object->get_element_id());
            }
            
            $metadata_attribute_values = array();
            
            if (count($element_value_conditions))
            {
                $element_value_condition1 = new OrCondition($element_value_conditions);
                $element_value_condition2 = new EqualityCondition(MetadataAttributeValue :: PROPERTY_RELATION, 
                        MetadataAttributeValue :: RELATION_PROPERTY_VALUE);
                $element_value_condition = new AndCondition($element_value_condition1, $element_value_condition2);
                
                $condition = new OrCondition($content_object_condition, $element_value_condition);
                
                $metadata_attribute_values = $this->retrieve_metadata_attribute_values($condition);
                $metadata_attribute_values = $this->format_metadata_attribute_values($metadata_attribute_values);
            }
            
            // get allowed property attribute types
            $allowed_metadata_attributes = array();
            
            foreach ($content_object_property_metadata as $id => $value)
            {
                if (! isset($content_object_metadata_element_values[$id]))
                {
                    $conditions_allowed[] = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_PARENT_ID, 
                            $id);
                }
            }
            
            // if(count($conditions_allowed))
            // {
            // $or_condition = new OrCondition($conditions_allowed);
            // $type_condition = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_CHILD_TYPE,
            // MetadataManager :: PARAM_METADATA_ATTRIBUTE);
            // $condition = new AndCondition($or_condition, $type_condition);
            //
            // $allowed_metadata_attributes = $this->retrieve_metadata_attribute_associations($condition);
            // $allowed_metadata_attributes = $this->format_allowed_metadata_attributes($allowed_metadata_attributes);
            // }
            $allowed_metadata_attributes = $this->retrieve_allowed_metadata_attributes($conditions_allowed);
            
            $form = new ContentObjectMetadataEditorForm($content_object, $content_object_metadata_element_values, 
                    $content_object_property_metadata, $metadata_attribute_values, $allowed_metadata_attributes, 
                    $this->get_url(
                            array(MetadataManager :: PARAM_CONTENT_OBJECT => $content_object->get_id())), 
                    $this->get_user(), $this);
            
            if ($form->validate())
            {
                $success = $form->edit_metadata();
                $this->redirect(
                        $success ? Translation :: get('ObjectUpdated', 
                                array('OBJECT' => Translation :: get('Metadata')), Utilities :: COMMON_LIBRARIES) : Translation :: get(
                                'ObjectNotUpdated', 
                                array('OBJECT' => Translation :: get('Metadata')), Utilities :: COMMON_LIBRARIES), 
                        ! $success, 
                        array(MetadataManager :: PARAM_ACTION => MetadataManager :: ACTION_EDIT_CONTENT_OBJECT_METADATA, 
                                MetadataManager :: PARAM_CONTENT_OBJECT => $content_object->get_id()));
            }
            else
            {
                $this->display_header();
                $form->display();
                $this->display_footer();
            }
        }
        else
        {
            // no content object id provided
        }
    }

    public function get_content_object_property_metadata_values($content_object)
    {
        $content_object_property_metadatas = $this->retrieve_content_object_property_metadatas();
        $content_object_property_metadata_values = array();
        
        while ($content_object_property_metadata = $content_object_property_metadatas->next_result())
        {
            $content_object_property_metadata_values[$content_object_property_metadata->get_id()] = $content_object_property_metadata;
        }
        return $content_object_property_metadata_values;
    }

    public function format_metadata_element_values($metadata_element_values)
    {
        $metadata_element_value_arr = array();
        
        while ($metadata_porperty_value = $metadata_element_values->next_result())
        {
            $metadata_element_value_arr[$metadata_porperty_value->get_id()] = $metadata_porperty_value;
        }
        return $metadata_element_value_arr;
    }

    public function format_metadata_attribute_values($metadata_attribute_values)
    {
        $metadata_attribute_value_arr = array();
        
        while ($metadata_attribute_value = $metadata_attribute_values->next_result())
        {
            $metadata_attribute_value_arr[$metadata_attribute_value->get_relation()][$metadata_attribute_value->get_parent_id()][$metadata_attribute_value->get_id()] = $metadata_attribute_value;
        }
        return $metadata_attribute_value_arr;
    }

    public function get_additional_parameters()
    {
        return array(MetadataManager :: PARAM_CONTENT_OBJECT);
    }
}
