<?php
namespace core\metadata;
use common\libraries\Toolbar;
use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\Utilities;

/**
 * Cell rendere for the learning object browser table
 *
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */

class ContentObjectPropertyMetadataBrowserTableCellRenderer extends DefaultContentObjectPropertyMetadataTableCellRenderer
{
    /**
     * The browser component
     */
    private $browser;

    /**
     * Constructor
     *
     * @param $browser ApplicationComponent
     */
    public function __construct($browser)
    {
        parent :: __construct();
        $this->browser = $browser;
    }

    // Inherited
    public function render_cell($column, $content_object_property_metadata)
    {

        if ($column->get_name() == DefaultContentObjectPropertyMetadataTableColumnModel :: COLUMN_TYPE_ELEMENT)
        {
            $metadata_element = $this->browser->retrieve_metadata_element(
                    $content_object_property_metadata->get_element_id());
            return $metadata_element->render_name();
        }

        if ($column === ContentObjectPropertyMetadataBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($content_object_property_metadata);
        }

        return parent :: render_cell($column, $content_object_property_metadata);
    }

    /**
     * Gets the action links to display
     *
     * @param $content_object ContentObject The learning object for which the action links should be returned
     * @return string A HTML representation of the action links
     */
    private function get_modification_links($content_object_property_metadata)
    {
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        $toolbar->add_item(
                new ToolbarItem(Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_edit.png',
                        $this->browser->get_update_content_object_property_metadata_url(
                                $content_object_property_metadata), ToolbarItem :: DISPLAY_ICON));

        $toolbar->add_item(
                new ToolbarItem(Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_delete.png',
                        $this->browser->get_delete_content_object_property_metadata_url(
                                $content_object_property_metadata), ToolbarItem :: DISPLAY_ICON, true));

        return $toolbar->as_html();
    }
}
