<?php
namespace core\metadata;

use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Toolbar;
use common\libraries\Theme;
use common\libraries\Utilities;

/**
 * Cell rendere for the learning object browser table
 *
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */

class MetadataDefaultValueBrowserTableCellRenderer extends DefaultMetadataDefaultValueTableCellRenderer
{
    /**
     * The browser component
     */
    private $browser;

    /**
     * Constructor
     *
     * @param $browser ApplicationComponent
     */
    public function __construct($browser)
    {
        parent :: __construct();
        $this->browser = $browser;
    }

    // Inherited
    public function render_cell($column, $metadata_default_value)
    {
        if ($column === MetadataDefaultValueBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($metadata_default_value);
        }
        if ($column->get_name() == MetadataDefaultValue :: PROPERTY_ATTRIBUTE_ID)
        {
            if ($metadata_default_value->get_attribute_id() === '0')
                return Translation :: get('Error', null, Utilities :: COMMON_LIBRARIES);
            if (! is_null($metadata_default_value->get_attribute_id()))
            {
                return $this->browser->retrieve_metadata_attribute($metadata_default_value->get_attribute_id())->render_name();
            }
        }
        return parent :: render_cell($column, $metadata_default_value);
    }

    /**
     * Gets the action links to display
     *
     * @param $content_object ContentObject The learning object for which the action links should be returned
     * @return string A HTML representation of the action links
     */
    private function get_modification_links($metadata_default_value)
    {
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        $toolbar->add_item(
                new ToolbarItem(Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_edit.png',
                        $this->browser->get_update_metadata_default_value_url($metadata_default_value),
                        ToolbarItem :: DISPLAY_ICON));

        $toolbar->add_item(
                new ToolbarItem(Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                        Theme :: get_common_image_path() . 'action_delete.png',
                        $this->browser->get_delete_metadata_default_value_url($metadata_default_value),
                        ToolbarItem :: DISPLAY_ICON, true));

        return $toolbar->as_html();
    }
}
