<?php
namespace core\metadata;

use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\Utilities;
use common\libraries\Redirect;
use common\libraries\DynamicTabsRenderer;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use core\admin\AdminManager;

/**
 * Component to edit an existing metadata_default_value object
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class MetadataManagerMetadataDefaultValueUpdaterComponent extends MetadataManager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $metadata_default_value = $this->retrieve_metadata_default_value(Request :: get(MetadataManager :: PARAM_METADATA_DEFAULT_VALUE));

        $metadata_element = $this->retrieve_metadata_element($metadata_default_value->get_element_id());

        $conditions = array();
        $conditions[] = new EqualityCondition(MetadataAttributeAssociation :: PROPERTY_PARENT_ID, $metadata_element->get_id());
        $allowed_metadata_attributes = $this->retrieve_allowed_metadata_attributes($conditions);

        foreach ($allowed_metadata_attributes[$metadata_element->get_id()] as $id)
        {
            $metadata_attributes[$id] = $this->retrieve_metadata_attribute($id)->render_name();
        }
        $params[MetadataManager :: PARAM_METADATA_DEFAULT_VALUE] = Request :: get(MetadataManager :: PARAM_METADATA_DEFAULT_VALUE);
        $form = new MetadataDefaultValueForm(MetadataDefaultValueForm :: TYPE_EDIT, $metadata_default_value, $metadata_element, $metadata_attributes, $this->get_url($params));

        if ($form->validate())
        {
            $success = $form->update_metadata_default_value();
            $this->redirect(Translation :: get($success ? 'ObjectUpdated' : 'ObjectNotUpdated', array(
                    'OBJECT' => Translation :: get('MetadataElement')), Utilities :: COMMON_LIBRARIES), ! $success, array(
                    MetadataManager :: PARAM_ACTION => MetadataManager :: ACTION_BROWSE_METADATA_DEFAULT_VALUES,
                    MetadataManager :: PARAM_METADATA_ELEMENT => $metadata_default_value->get_element_id()));
        }
        else
        {
            $trail = BreadcrumbTrail :: get_instance();

            $trail->add(new BreadCrumb(Redirect :: get_link(AdminManager :: APPLICATION_NAME, array(
                    AdminManager :: PARAM_ACTION => AdminManager :: ACTION_ADMIN_BROWSER), array(), false, Redirect :: TYPE_CORE), Translation :: get('Admin')));
            $trail->add(new BreadCrumb(Redirect :: get_link(AdminManager :: APPLICATION_NAME, array(
                    AdminManager :: PARAM_ACTION => AdminManager :: ACTION_ADMIN_BROWSER,
                    DynamicTabsRenderer :: PARAM_SELECTED_TAB => MetadataManager :: APPLICATION_NAME), array(), false, Redirect :: TYPE_CORE), Translation :: get('Metadata')));
            $trail->add(new BreadCrumb($this->get_url(array(
                    MetadataManager :: PARAM_ACTION => MetadataManager :: ACTION_BROWSE_METADATA_ELEMENTS)), Translation :: get('BrowseObjects', array(
                    'OBJECTS' => Translation :: get('MetadataElements')), Utilities :: COMMON_LIBRARIES)));
            $trail->add(new BreadCrumb($this->get_url(), Translation :: get('UpdateObject', array(
                    'OBJECT' => Translation :: get('MetadataDefaultValue')), Utilities :: COMMON_LIBRARIES)));

            $this->display_header();

            $form->display();
            $this->display_footer();
        }
    }
}
