<?php
namespace core\metadata;

use common\libraries\ToolbarItem;
use common\libraries\Translation;
use common\libraries\Application;
use common\libraries\ActionBarRenderer;
use common\libraries\Theme;
use common\libraries\Utilities;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\Redirect;
use common\libraries\DynamicTabsRenderer;
use core\admin\AdminManager;
use common\libraries\PatternMatchCondition;
use common\libraries\OrCondition;
use AndCondition;

/**
 * metadata component which allows the user to browse his metadata_elements
 * @author Sven Vanpoucke
 * @author Jens Vanderheyden
 */
class MetadataManagerMetadataElementsBrowserComponent extends MetadataManager
{

    public function run()
    {
        $trail = BreadcrumbTrail :: get_instance();

        $trail->add(new BreadCrumb(Redirect :: get_link(AdminManager :: APPLICATION_NAME, array(
                AdminManager :: PARAM_ACTION => AdminManager :: ACTION_ADMIN_BROWSER), array(), false, Redirect :: TYPE_CORE), Translation :: get('Admin')));
        $trail->add(new BreadCrumb(Redirect :: get_link(AdminManager :: APPLICATION_NAME, array(
                AdminManager :: PARAM_ACTION => AdminManager :: ACTION_ADMIN_BROWSER,
                DynamicTabsRenderer :: PARAM_SELECTED_TAB => MetadataManager :: APPLICATION_NAME), array(), false, Redirect :: TYPE_CORE), Translation :: get('Metadata')));
        $trail->add(new BreadCrumb($this->get_url(), Translation :: get('BrowseObjects', array(
                'OBJECTS' => Translation :: get('MetadataElements')), Utilities :: COMMON_LIBRARIES)));

        $this->display_header();

        $this->action_bar = $this->get_action_bar();

        $html[] = $this->action_bar->as_html();
        $html[] = $this->get_table();

        echo implode("\n", $html);

        $this->display_footer();
    }

    public function get_table()
    {
        $parameters = $this->get_parameters(true);
        $parameters[Application :: PARAM_APPLICATION] = 'metadata';
        //        $parameters['curriculum_courses_browser_column'] = '2';
        $parameters[Application :: PARAM_ACTION] = MetadataManager :: ACTION_BROWSE_METADATA_ELEMENTS;

        $table = new MetadataElementBrowserTable($this, $parameters, $this->get_condition());
        return $table->as_html();
    }

    public function get_action_bar()
    {
        $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);

        $actions = array();
        $actions[] = new ToolbarItem(Translation :: get('CreateObject', array(
                'OBJECT' => Translation :: get('MetadataElement')), Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_create.png', $this->get_url(array(
                MetadataManager :: PARAM_ACTION => MetadataManager :: ACTION_CREATE_METADATA_ELEMENT)));
        $action_bar->set_common_actions($actions);
        $action_bar->set_search_url($this->get_url());

        return $action_bar;
    }

    public function get_condition()
    {
        $query = $this->action_bar->get_query();

        if (isset($query) && $query != '')
        {
            $condition = new PatternMatchCondition(MetadataElement :: PROPERTY_NAME, '*' . $query . '*');
        }

        //        $condition = null;
        //        if (isset($this->firstletter))
        //        {
        //            $conditions = array();
        //            $conditions[] = new PatternMatchCondition(User :: PROPERTY_LASTNAME, $this->firstletter . '*');
        //            $conditions[] = new PatternMatchCondition(User :: PROPERTY_LASTNAME, chr(ord($this->firstletter) + 1) . '*');
        //            $conditions[] = new PatternMatchCondition(User :: PROPERTY_LASTNAME, chr(ord($this->firstletter) + 2) . '*');
        //            $condition = new OrCondition($conditions);
        //            if (count($search_conditions))
        //            {
        //                $condition = new AndCondition($condition, $search_conditions);
        //            }
        //        }
        //        else
        //        {
        //            if (count($search_conditions))
        //            {
        //                $condition = $search_condition;
        //            }
        //        }
        return $condition;
    }

}
