<?php
namespace core\migration;
/**
 * $Id: dokeos185_blog_task.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.lib.platform.dokeos185
 */
use Exception;

/**
 * This class presents a Dokeos185 blog_task
 *
 * @author Sven Vanpoucke
 */
class Dokeos185BlogTask extends Dokeos185MigrationDataClass
{
    private static $mgdm;
    
    /**
     * Dokeos185BlogTask properties
     */
    const PROPERTY_TASK_ID = 'task_id';
    const PROPERTY_BLOG_ID = 'blog_id';
    const PROPERTY_TITLE = 'title';
    const PROPERTY_DESCRIPTION = 'description';
    const PROPERTY_COLOR = 'color';
    const PROPERTY_SYSTEM_TASK = 'system_task';
    
    /**
     * Default properties stored in an associative array.
     */
    private $defaultProperties;

    /**
     * Creates a new Dokeos185BlogTask object
     * @param array $defaultProperties The default properties
     */
    public function __construct($defaultProperties = array ())
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Gets a default property by name.
     * @param string $name The name of the property.
     */
    public function get_default_property($name)
    {
        return $this->defaultProperties[$name];
    }

    /**
     * Gets the default properties
     * @return array An associative array containing the properties.
     */
    public function get_default_properties()
    {
        return $this->defaultProperties;
    }

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_TASK_ID, self :: PROPERTY_BLOG_ID, self :: PROPERTY_TITLE, 
                self :: PROPERTY_DESCRIPTION, self :: PROPERTY_COLOR, self :: PROPERTY_SYSTEM_TASK);
    }

    /**
     * Sets a default property by name.
     * @param string $name The name of the property.
     * @param mixed $value The new value for the property.
     */
    public function set_default_property($name, $value)
    {
        $this->defaultProperties[$name] = $value;
    }

    /**
     * Sets the default properties of this class
     */
    public function set_default_properties($defaultProperties)
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Returns the task_id of this Dokeos185BlogTask.
     * @return the task_id.
     */
    public function get_task_id()
    {
        return $this->get_default_property(self :: PROPERTY_TASK_ID);
    }

    /**
     * Returns the blog_id of this Dokeos185BlogTask.
     * @return the blog_id.
     */
    public function get_blog_id()
    {
        return $this->get_default_property(self :: PROPERTY_BLOG_ID);
    }

    /**
     * Returns the title of this Dokeos185BlogTask.
     * @return the title.
     */
    public function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    /**
     * Returns the description of this Dokeos185BlogTask.
     * @return the description.
     */
    public function get_description()
    {
        return $this->get_default_property(self :: PROPERTY_DESCRIPTION);
    }

    /**
     * Returns the color of this Dokeos185BlogTask.
     * @return the color.
     */
    public function get_color()
    {
        return $this->get_default_property(self :: PROPERTY_COLOR);
    }

    /**
     * Returns the system_task of this Dokeos185BlogTask.
     * @return the system_task.
     */
    public function get_system_task()
    {
        return $this->get_default_property(self :: PROPERTY_SYSTEM_TASK);
    }

    /**
     * Check if the blog task is valid
     * @param array $array the parameters for the validation
     */
    public function is_valid()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public static function get_table_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public static function get_class_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    public function get_database_name()
    {
        throw new Exception("Unimplemented method " . __FILE__ . "#" . __METHOD__ . "(" . __LINE__ . ")");
    }

    /**
     * Convert to new blog task
     * @param array $array the parameters for the conversion
     */
    public function convert_data()
    {
        $course = $array['course'];
    }

    /**
     * Retrieve all blog tasks from the database
     * @param array $parameters parameters for the retrieval
     */
    public static function retrieve_data($parameters)
    {
        self :: $mgdm = $parameters['mgdm'];
        
        $db = $parameters['course']->get_db_name();
        $tablename = 'blog_task';
        $classname = 'Dokeos185BlogTask';
        
        return self :: $mgdm->get_all($db, $tablename, $classname, $tool_name, $parameters['offset'], $parameters['limit']);
    }

    public static function get_database_table($parameters)
    {
        $array = array();
        $array['database'] = $parameters['course']->get_db_name();
        $array['table'] = 'blog_task';
        return $array;
    }
}
