<?php
namespace core\migration;

use common\libraries\Translation;
use common\libraries\Utilities;
use application\weblcms\WeblcmsLpAttemptTracker;
/**
 * $Id: dokeos185_lp_view.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.lib.platform.dokeos185
 */

require_once dirname(__FILE__) . '/../dokeos185_course_data_migration_data_class.class.php';

/**
 * This class presents a Dokeos185 lp_view
 *
 * @author Sven Vanpoucke
 */
class Dokeos185LpView extends Dokeos185CourseDataMigrationDataClass
{
    const CLASS_NAME = __CLASS__;
    const TABLE_NAME = 'lp_view';

    /**
     * Dokeos185LpView properties
     */
    const PROPERTY_ID = 'id';
    const PROPERTY_LP_ID = 'lp_id';
    const PROPERTY_USER_ID = 'user_id';
    const PROPERTY_VIEW_COUNT = 'view_count';
    const PROPERTY_LAST_ITEM = 'last_item';
    const PROPERTY_PROGRESS = 'progress';

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_ID, self :: PROPERTY_LP_ID, self :: PROPERTY_USER_ID, self :: PROPERTY_VIEW_COUNT, self :: PROPERTY_LAST_ITEM, self :: PROPERTY_PROGRESS);
    }

    /**
     * Returns the id of this Dokeos185LpView.
     * @return the id.
     */
    public function get_id()
    {
        return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     * Returns the lp_id of this Dokeos185LpView.
     * @return the lp_id.
     */
    public function get_lp_id()
    {
        return $this->get_default_property(self :: PROPERTY_LP_ID);
    }

    /**
     * Returns the user_id of this Dokeos185LpView.
     * @return the user_id.
     */
    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    /**
     * Returns the view_count of this Dokeos185LpView.
     * @return the view_count.
     */
    public function get_view_count()
    {
        return $this->get_default_property(self :: PROPERTY_VIEW_COUNT);
    }

    /**
     * Returns the last_item of this Dokeos185LpView.
     * @return the last_item.
     */
    public function get_last_item()
    {
        return $this->get_default_property(self :: PROPERTY_LAST_ITEM);
    }

    /**
     * Returns the progress of this Dokeos185LpView.
     * @return the progress.
     */
    public function get_progress()
    {
        return $this->get_default_property(self :: PROPERTY_PROGRESS);
    }

    /**
     * Check if the lp view is valid
     * @return true if the lp view is valid
     */
    public function is_valid()
    {
        $new_lp_id = $this->get_id_reference($this->get_lp_id(), $this->get_database_name() . '.lp');
        $new_user_id = $this->get_id_reference($this->get_user_id(), 'main_database.user');

        if (! $this->get_id() || ! $new_lp_id || ! $new_user_id)
        {
            $this->create_failed_element($this->get_id());
            
            if(!$new_lp_id)
            {                
                $this->set_message(Translation :: get('LearningPathNotMigratedMessage', array('LPID' => $this->get_lp_id(), 'LPVID' => $this->get_id())));
            }
            if(!$new_user_id)
            {
                $this->set_message(Translation :: get('UserNotMigratedMessage', array('UID' => $new_user_id, 'LPVID' => $this->get_id())));
            }   
            
            $this->set_message(Translation :: get('GeneralInvalidMessage', array('TYPE' => 'learning_path_view', 'ID' => $this->get_id())));
            
            return false;
        }
        return true;
    }

    /**
     * Convert to new lp view
     */
    public function convert_data()
    {
        $course = $this->get_course();

    	$new_user_id = $this->get_id_reference($this->get_user_id(), 'main_database.user');
    	$new_course_id = $this->get_id_reference($course->get_code(), 'main_database.course');
    	$new_lp_id = $this->get_id_reference($this->get_lp_id(), $this->get_database_name() . '.lp');

    	$tracker = new WeblcmsLpAttemptTracker();
    	$tracker->set_course_id($new_course_id);
    	$tracker->set_user_id($new_user_id);
    	$tracker->set_lp_id($new_lp_id);
    	$tracker->set_progress($this->get_progress());
    	$tracker->create();

    	$this->create_id_reference($this->get_id(), $tracker->get_id());
        $this->set_message(Translation :: get('GeneralConvertedMessage', array('TYPE' => 'learning_path_view', 'OLD_ID' => $this->get_id(), 'NEW_ID' => $tracker->get_id())));
    }

    public static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(substr(Utilities :: get_classname_from_namespace(__CLASS__), 9));
    }

    public static function get_class_name()
    {
    	return self :: CLASS_NAME;
    }
}
