<?php
namespace core\migration;

use common\libraries\Translation;
use common\libraries\Utilities;
use application\weblcms\course\CourseDataManager;

/**
 * $Id: dokeos185_tool.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.platform.dokeos185
 */
require_once dirname(__FILE__) . '/../dokeos185_course_data_migration_data_class.class.php';

/**
 * This class represents an old Dokeos 1.8.5 Tool
 *
 * @author Van Wayenbergh David
 */
class Dokeos185Tool extends Dokeos185CourseDataMigrationDataClass
{
    const CLASS_NAME = __CLASS__;
    const TABLE_NAME = 'tool';

    /**
     * Announcement properties
     */
    const PROPERTY_ID = 'id';
    const PROPERTY_NAME = 'name';
    const PROPERTY_LINK = 'link';
    const PROPERTY_VISIBILITY = 'visibility';
    const PROPERTY_ADDED_TOOL = 'added_tool';
    
    /**
     * Default properties stored in an associative array.
     */
    private $defaultProperties;

    /**
     * Creates a new dokeos185 Tool object
     * @param array $defaultProperties The default properties
     */
    public function __construct($defaultProperties = array())
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Gets a default property by name.
     * @param string $name The name of the property.
     */
    public function get_default_property($name)
    {
        return $this->defaultProperties[$name];
    }

    /**
     * Gets the default properties
     * @return array An associative array containing the properties.
     */
    public function get_default_properties()
    {
        return $this->defaultProperties;
    }

    /**
     * Get the default properties
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return array(self :: PROPERTY_ID, self :: PROPERTY_NAME, self :: PROPERTY_VISIBILITY);
    }

    /**
     * Sets a default property by name.
     * @param string $name The name of the property.
     * @param mixed $value The new value for the property.
     */
    public function set_default_property($name, $value)
    {
        $this->defaultProperties[$name] = $value;
    }

    /**
     * Sets the default properties of this class
     */
    public function set_default_properties($defaultProperties)
    {
        $this->defaultProperties = $defaultProperties;
    }

    /**
     * Returns the id of this tool.
     * @return int The id.
     */
    public function get_id()
    {
        return $this->get_default_property(self :: PROPERTY_ID);
    }

    /**
     * Returns the name of this tool.
     * @return string the name.
     */
    public function get_name()
    {
        return $this->get_default_property(self :: PROPERTY_NAME);
    }

    /**
     * Returns the visibility of this tool.
     * @return int the visibility.
     */
    public function get_visibility()
    {
        return $this->get_default_property(self :: PROPERTY_VISIBILITY);
    }

    /**
     * Returns the link of this tool.
     * @return string the link.
     */
    public function get_link()
    {
        return $this->get_default_property(self :: PROPERTY_LINK);
    }

    /**
     * Returns the added tool of this tool.
     * @return boolean the added tool.
     */
    public function get_added_tool()
    {
        return $this->get_default_property(self :: PROPERTY_ADDED_TOOL);
    }

    /**
     * Checks if tool is valid
     */
    public function is_valid()
    {
        if (! $this->get_name())
        {
            $this->create_failed_element($this->get_id());
            //
            $this->set_message(Translation :: get('GeneralInvalidMessage', array('TYPE' => 'tool', 
                    'ID' => $this->get_id())));
            
            return false;
        }
        return true;
    }

    /**
     * migrate to new tool
     * @param String $course
     * @return dokeos185tool
     */
    public function convert_data()
    {
        $value = self :: $tool_mapping[$this->get_name()];
        if(!$value){
            $value = $this->get_name();
        }
        $new_course_id = $this->get_id_reference($this->get_course()->get_code(), 'main_database.course');
        CourseDataManager :: set_tool_visibility_by_tool_name($new_course_id,$value,$this->get_visibility());
        $this->set_message(Translation :: get('GeneralConvertedMessage', array('TYPE' => 'tool', 
                'OLD_ID' => $this->get_id(), 'NEW_ID' => $new_course_id)));
        
        return $this;
    }

    public static function get_table_name()
    {
        return Utilities :: camelcase_to_underscores(substr(Utilities :: get_classname_from_namespace(__CLASS__), 9));
    }

    public static function get_class_name()
    {
        return self :: CLASS_NAME;
    }

}
