<?php

namespace core\migration;

use application\weblcms\ContentObjectPublication;
use core\admin\FeedbackPublication;
use repository\ComplexContentObjectItem;
use application\weblcms\WeblcmsRights;
use rights\RightsLocationEntityRight;
use application\weblcms\CourseGroupEntity;
use rights\UserEntity;
use application\weblcms\WeblcmsManager;
use repository\RepositoryDataManager;

/**
 * $Id: import.class.php 221 2009-11-13 14:36:41Z vanpouckesven $
 * @package migration.lib.migration_manager
 */
require_once dirname(__FILE__) . '/dokeos185_migration_data_class.class.php';
require_once dirname(__FILE__) . '/data_class/dokeos185_item_property.class.php';

/**
 * Abstract class to be used by all the tables that are within a course database of dokeos 1.8.5
 * @author Sven Vanpoucke
 */
abstract class Dokeos185CourseDataMigrationDataClass extends Dokeos185MigrationDataClass
{

    /**
     * The current course for this data
     * @var Dokeos185Course
     */
    private $course;
    /**
     * Item property, used for most tools
     * @var Dokeos185ItemProperty
     */
    protected $item_property;

    /**
     * Map from id to correct chamilo tool
     */
    protected static $tool_mapping = array(
        'course_homepage' => 'home',
        'course_description' => 'description', 
        'calendar_event' => 'calendar', 
        'learnpath' => 'learning_path', 
        'quiz' => 'assessment', 
        'blog' => 'blog',
        'calendar' => 'calendar', 
        'document' => 'document', 
        'newscorm' => 'learning_path',
        'link' => 'link', 
        'announcement' => 'announcement', 
        'announcements' => 'announcement', 
        'forum' => 'forum', 
        'dropbox' => 'dropbox', 
        'ephorus' => 'ephorus',
        'user' => 'user', 
        'glossary' => 'glossary',
        'group' => 'course_group', 
        'chat' => 'chat', 
        'survey' => 'survey', 
        'wiki' => 'wiki', 
        'exercice' => 'assessment',
        'peer_assessments' => 'peer_assessments',
        'assignment' => 'assignment',
        'conference' => 'video_conferencing');
    
    /**
     * Sets the course
     * @param Dokeos185Course $course
     */
    
    public function set_course($course)
    {
        $this->course = $course;
    }

    /**
     * Retrieves the course
     * @return Course
     */
    public function get_course()
    {
        return $this->course;
    }

    public function get_item_property()
    {
        return $this->item_property;
    }

    public function set_item_property($item_property)
    {
        $this->item_property = $item_property;
    }

    public function get_email_sent()
    {
        return 0;
    }

    /**
     * Creates a publication from a content object
     * @param ContentObject $object
     * @param int $course_id
     * @param int $user_id
     * @param String $tool
     * @param int $category_id
     * @param array() $target_users
     * @param array() $target_groups
     */
    public function create_publication($object, $course_id, $user_id, $tool, $category_id = 0, $target_users = null, $target_groups = null, $show_on_homepage = 0, $hidden = null)
    {
        //publication
        $publication = new ContentObjectPublication();

        $publication->set_content_object($object);
        $publication->set_content_object_id($object->get_id());
        $publication->set_course_id($course_id);
        $publication->set_publisher_id($user_id);
        $publication->set_tool($tool);
        
        $publication->set_category_id($category_id);

        $publication->set_from_date(0);
        $publication->set_to_date(0);

        $publication->set_display_order_index(0);

        $publication->set_email_sent($this->get_email_sent());
        $publication->set_show_on_homepage($show_on_homepage);

        if ($this->item_property)
        {

            if ($this->item_property->get_visibility() == 2)
            {
                return;
            }

            if($hidden)
            {
                $publication->set_hidden($hidden);
            }
            else
            {
                $publication->set_hidden($this->item_property->get_visibility() == 1 ? 0 : 1);
            }

            $publication->set_publication_date(strtotime($this->item_property->get_insert_date()));
            $publication->set_modified_date(strtotime($this->item_property->get_lastedit_date()));
        }
        else
        {
            if($hidden)
            {
                $publication->set_hidden($hidden);
            }
            else
            {
                $publication->set_hidden(0);
            }
            
            $publication->set_publication_date($object->get_creation_date());
            $publication->set_modified_date($object->get_modification_date());
        }

        //create publication in database
        if( $publication->create())
        {

            //target users, groups
            if (isset($target_groups) || isset($target_users))
            {
                $location = WeblcmsRights :: get_instance()->
                        get_weblcms_location_by_identifier_from_courses_subtree(WeblcmsRights::TYPE_PUBLICATION, 
                                $publication->get_id(), $course_id);                               
                
                if(!$location)
                {
                    return false;
                }
                
                $location->disinherit();
                $location->update();
                
                if(isset($target_groups) && !is_array($target_groups))
                {
                    $target_groups = array($target_groups);
                }
                if(isset($target_users) && !is_array($target_users))
                {
                    $target_users = array($target_users);
                }
                
                foreach($target_groups as $target_group)
                {
                    $entity_right = new RightsLocationEntityRight();
                    $entity_right->set_context(WeblcmsManager :: APPLICATION_NAME);
                    $entity_right->set_entity_id($target_group);
                    $entity_right->set_entity_type(CourseGroupEntity::ENTITY_TYPE);
                    $entity_right->set_location_id($location->get_id());
                    $entity_right->set_right_id(WeblcmsRights::VIEW_RIGHT);
                    $entity_right->create();
                }

                foreach($target_users as $target_user)
                {
                    $entity_right = new RightsLocationEntityRight();
                    $entity_right->set_context(WeblcmsManager :: APPLICATION_NAME);
                    $entity_right->set_entity_id($target_user);
                    $entity_right->set_entity_type(UserEntity::ENTITY_TYPE);
                    $entity_right->set_location_id($location->get_id());
                    $entity_right->set_right_id(WeblcmsRights::VIEW_RIGHT);
                    $entity_right->create();

                }
            }
            
            return $publication;
        }
        else
        {
            return false;
        }
    }

    /**
     * Creates a complex content object item with given parametesr
     * @param ContentObject $object
     * @param int $parent_id
     * @param int $user_id
     * @param int $add_date
     * @param array() $additional_properties
     */
    public function create_complex_content_object_item($object, $parent_id, $user_id, $add_date = null, $display_order = null, $additional_properties = array())
    {
        $complex_content_object_item = ComplexContentObjectItem :: factory($object->get_type());
        $complex_content_object_item->set_user_id($user_id);
        $complex_content_object_item->set_parent($parent_id);
        $complex_content_object_item->set_ref($object->get_id());
        $complex_content_object_item->set_add_date($add_date ? $add_date : time());
        $complex_content_object_item->set_display_order($display_order);
        $complex_content_object_item->set_additional_properties($additional_properties);
        $complex_content_object_item->create();

        return $complex_content_object_item;
    }

    /**
     * Creates a feedback publication
     * @param ContentObject $object
     * @param int $publication_id
     * @param int $complex_content_object_item_id
     */
    public function create_feedback($object, $publication_id, $complex_content_object_item_id, $creation_date = null, $modification_date = null)
    {
        $feedback = new FeedbackPublication();
        $feedback->set_application('weblcms');
        $feedback->set_fid($object->get_id());
        $feedback->set_pid($publication_id);
        $feedback->set_cid($complex_content_object_item_id);
        $feedback->set_creation_date($creation_date ? $creation_date : time());
        $feedback->set_modification_date($modification_date ? $modification_date : time());
        $feedback->create();

        return $feedback;
    }
    
    public function add_resources($object, $type)
    {
        $resources = $this->get_data_manager()->retrieve_resources($this->get_course(), $object->get_id(), $type);
        while($resource = $resources->next_result())
        {
            $new_resource_id = $this->get_id_reference($resource->get_resource_id(), $this->get_database_name() . '.' . $resource->get_resource_type());
            if($new_resource_id)
            {
                $object->attach_content_object($new_resource_id);
            }
        }
    }

    /**
     * Gets the database name of this data class which is offcourse the database name of the course
     */
    public function get_database_name()
    {
        return $this->course->get_db_name();
    }
    
    /**
     * Returns the repository category id of the course (in root->migration) for the given user
     * @param int $user_id
     * @return int repository category id
     */
    public function get_repository_migration_course_category_id($user_id)
    {
        $migration_root_id = $this->get_repository_migration_root_category_id($user_id);
        $chamilo_repository_course_category_id = RepositoryDataManager :: get_repository_category_by_name_or_create_new($user_id, $this->course->get_title(), $migration_root_id);
        if($chamilo_repository_course_category_id)
        {
            return $chamilo_repository_course_category_id;
        }
        else
        {
            return $migration_root_id;
        }
    }
}
