<?php
namespace core\migration;

use repository\content_object\document\Document;
use repository\ContentObject;
use common\libraries\PatternMatchCondition;
use repository\content_object\file\File;
use common\libraries\Translation;
use common\libraries\MessageLogger;

/**
 * Post processing of the images in html files in chamilo because the link to images is still invalid.
 */
class HtmPostProcess extends MigrationPostProcess
{
    const NAME = 'htm';

    public function run()
    {
        if (ContentObject :: is_available('repository\content_object\document\Document'))
        {
            $condition = new PatternMatchCondition(Document :: PROPERTY_FILENAME, '*.htm*', Document :: get_type_name());
            $documents = \repository\DataManager :: retrieve_active_content_objects(Document :: class_name(), 
                    $condition);
        }
        else
        {
            $condition = new PatternMatchCondition(File :: PROPERTY_FILENAME, '*.htm*', File :: get_type_name());
            $documents = \repository\DataManager :: retrieve_active_content_objects(File :: class_name(), $condition);
        }
        
        $image_parser = Dokeos185TextFieldParser :: factory(Dokeos185TextFieldParser :: TYPE_IMAGE);
        $htm_parser = Dokeos185TextFieldParser :: factory(Dokeos185TextFieldParser :: TYPE_FLASH_HTM);
        
        $counter = 0;
        
        while ($document = $documents->next_result())
        {
            $this->get_file_logger()->log_message(
                    Translation :: get('FixedDocument', array('ID' => $document->get_id())));
            
            $file = $document->get_full_path();
            $contents = file_get_contents($file);
            
            $contents = $image_parser->parse($contents);
            $contents = $htm_parser->parse($contents);
            
            file_put_contents($file, $contents);
            
            if ($image_parser->get_included_objects())
            {
                foreach ($image_parser->get_included_objects() as $included_object_id)
                {
                    $document->include_content_object($included_object_id);
                }
                $image_parser->set_included_objects(array());
            }
            
            if ($htm_parser->get_included_objects())
            {
                foreach ($htm_parser->get_included_objects() as $included_object_id)
                {
                    $document->include_content_object($included_object_id);
                }
                $htm_parser->set_included_objects(array());
            }
            
            $counter ++;
        }
        
        $this->log_message_and_file(Translation :: get('DocumentsFixed', array('COUNTER' => $counter)), 
                MessageLogger :: TYPE_NORMAL);
    }

    public function get_name()
    {
        return self :: NAME;
    }
}
