<?php
use common\libraries\Request;
use common\libraries\Path;
use common\libraries\Theme;
use common\libraries\CssUtilities;
use common\libraries\ResourceCache;
use common\libraries\HttpHeader;
use common\libraries\ClientCache;
use repository\RepositoryManager;

require_once dirname(__FILE__) . '/common/init.inc.php';

$theme = Request :: get('theme');
Theme :: set_theme($theme);
$time = Request :: get('time');
$server_type = Request :: get('server_type', 'production');
$application = Request :: get('application', 'common');
$key = 'css_' . $theme . '_' . $application . '_' . $time . '.css';
$is_caching = $server_type == 'production'; // we turn on caching for production
                                            // only

HttpHeader :: content_type(HttpHeader :: CONTENT_TYPE_CSS, CssUtilities :: DEFAULT_CHARSET);
ClientCache :: prolog($key, $time, $is_caching);
ResourceCache :: prolog($key, $time, $is_caching);

/**
 * complete configuration.
 */
require_once dirname(__FILE__) . '/common/conf.inc.php';

// CONTENT

ob_flush();
ob_start();

if ($application == 'common')
{
    // common
    echo CssUtilities :: get(Theme :: get_common_css_path(false));

    // repo manager
    echo CssUtilities :: get(Theme :: get_css_path(RepositoryManager :: APPLICATION_NAME, false));

    // content objects
    require_once Path :: get_repository_path() . 'css.inc.php';

    // external repository
    require_once Path :: get_common_extensions_path() . 'external_repository_manager/php/css.inc.php';

    // video conferencing
    require_once Path :: get_common_extensions_path() . 'video_conferencing_manager/php/css.inc.php';
}
else
{
    echo CssUtilities :: get(Theme :: get_css_path($application, false));
}

$result = $is_caching ? ob_get_contents() : ''; // does not empty the buffer
ob_end_flush();

if ($is_caching)
{
    $result = CssUtilities :: Minify($result);
    ResourceCache :: set($key, $result);
}
