<?php
namespace group;

use common\libraries\Display;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Application;
use common\libraries\Request;
use common\libraries\AdministrationComponent;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;

/**
 * $Id: editor.class.php 224 2009-11-13 14:40:30Z kariboe $
 *
 * @package group.lib.group_manager.component
 */
class GroupManagerEditorComponent extends GroupManager implements AdministrationComponent
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $id = $this->get_parameter(GroupManager :: PARAM_GROUP_ID);
        if ($id)
        {
            $group = $this->retrieve_group($id);

            if (! $this->get_user()->is_platform_admin())
            {
                $this->display_header();
                Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
                $this->display_footer();
                exit();
            }

            $form = new GroupForm(GroupForm :: TYPE_EDIT, $group,
                    $this->get_url(array(GroupManager :: PARAM_GROUP_ID => $id)), $this->get_user());

            if ($form->validate())
            {
                $success = $form->update_group();
                $group = $form->get_group();
                $message = $success ? Translation :: get('ObjectUpdated',
                        array('OBJECT' => Translation :: get('Group')), Utilities :: COMMON_LIBRARIES) : Translation :: get(
                        'ObjectNotUpdated', array('OBJECT' => Translation :: get('Group')),
                        Utilities :: COMMON_LIBRARIES);
                $this->redirect($message, ($success ? false : true),
                        array(Application :: PARAM_ACTION => GroupManager :: ACTION_VIEW_GROUP,
                                GroupManager :: PARAM_GROUP_ID => $group->get_id()));
            }
            else
            {
                $this->display_header();
                $form->display();
                $this->display_footer();
            }
        }
        else
        {
            $this->display_error_page(
                    htmlentities(Translation :: get('NoObjectSelected', null, Utilities :: COMMON_LIBRARIES)));
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(Application :: PARAM_ACTION => GroupManager :: ACTION_BROWSE_GROUPS)),
                        Translation :: get('GroupManagerBrowserComponent')));
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(Application :: PARAM_ACTION => GroupManager :: ACTION_VIEW_GROUP,
                                        GroupManager :: PARAM_GROUP_ID => Request :: get(GroupManager :: PARAM_GROUP_ID))),
                        Translation :: get('GroupManagerViewerComponent')));
        $breadcrumbtrail->add_help('group general');
    }

    public function get_additional_parameters()
    {
        return array(GroupManager :: PARAM_GROUP_ID);
    }
}
