<?php
namespace group;

use common\libraries\Display;
use common\libraries\Application;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\AdministrationComponent;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;

/**
 * $Id: mover.class.php 224 2009-11-13 14:40:30Z kariboe $
 *
 * @package group.lib.group_manager.component
 */
class GroupManagerMoverComponent extends GroupManager implements AdministrationComponent
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        if (! $this->get_user()->is_platform_admin())
        {
            $this->display_header();
            Display :: warning_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
            $this->display_footer();
            exit();
        }

        $group = $this->retrieve_group(intval(Request :: get(GroupManager :: PARAM_GROUP_ID)));

        // TODO: only show groups you can actually move to (where you have create rights)
        $form = new GroupMoveForm($group,
                $this->get_url(
                        array(GroupManager :: PARAM_GROUP_ID => Request :: get(GroupManager :: PARAM_GROUP_ID))),
                $this->get_user());

        if ($form->validate())
        {

            $success = $form->move_group();
            $parent = $form->get_new_parent();
            $message = $success ? Translation :: get('ObjectMoved', array('OBJECT' => Translation :: get('Group')),
                    Utilities :: COMMON_LIBRARIES) : Translation :: get('ObjectNotMoved',
                    array('OBJECT' => Translation :: get('Group')), Utilities :: COMMON_LIBRARIES);
            $this->redirect($message, $success ? (false) : true,
                    array(Application :: PARAM_ACTION => GroupManager :: ACTION_BROWSE_GROUPS,
                            GroupManager :: PARAM_GROUP_ID => $parent));
        }
        else
        {
            $this->display_header();
            echo Translation :: get('Group') . ': ' . $group->get_name();
            $form->display();
            $this->display_footer();
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(Application :: PARAM_ACTION => GroupManager :: ACTION_BROWSE_GROUPS)),
                        Translation :: get('GroupManagerBrowserComponent')));
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(Application :: PARAM_ACTION => GroupManager :: ACTION_VIEW_GROUP,
                                        GroupManager :: PARAM_GROUP_ID => Request :: get(GroupManager :: PARAM_GROUP_ID))),
                        Translation :: get('GroupManagerViewerComponent')));
        $breadcrumbtrail->add_help('group general');
    }

    public function get_additional_parameters()
    {
        return array(GroupManager :: PARAM_GROUP_ID);
    }
}
