<?php
namespace group;

use common\libraries\Display;
use common\libraries\Application;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\AdministrationComponent;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use tracking\Event;

/**
 * $Id: subscriber.class.php 224 2009-11-13 14:40:30Z kariboe $
 *
 * @package group.lib.group_manager.component
 */
class GroupManagerSubscriberComponent extends GroupManager implements AdministrationComponent
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $user = $this->get_user();
        $group_id = Request :: get(GroupManager :: PARAM_GROUP_ID);

        if (! $this->get_user()->is_platform_admin())
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
            $this->display_footer();
            exit();
        }

        $users = Request :: get(GroupManager :: PARAM_USER_ID);

        $failures = 0;

        if (! empty($users))
        {
            if (! is_array($users))
            {
                $users = array($users);
            }

            foreach ($users as $user)
            {
                $existing_groupreluser = $this->retrieve_group_rel_user($user, $group_id);

                if (! is_null($existing_groupreluser))
                {
                    $groupreluser = new GroupRelUser();
                    $groupreluser->set_group_id($group_id);
                    $groupreluser->set_user_id($user);

                    if (! $groupreluser->create())
                    {
                        $failures ++;
                    }
                    else
                    {
                        Event :: trigger('subscribe_user', GroupManager :: APPLICATION_NAME,
                                array(\group\integration\tracking\Change :: PROPERTY_REFERENCE_ID => $groupreluser->get_group_id(),
                                        \group\integration\tracking\Change :: PROPERTY_TARGET_USER_ID => $groupreluser->get_user_id(),
                                        \group\integration\tracking\Change :: PROPERTY_USER_ID => $this->get_user()->get_id()));
                    }
                }
                else
                {
                    $contains_dupes = true;
                }
            }

            if ($failures)
            {
                if (count($users) == 1)
                {
                    $message = 'SelectedUserNotAddedToGroup' . ($contains_dupes ? 'Dupes' : '');
                }
                else
                {
                    $message = 'SelectedUsersNotAddedToGroup' . ($contains_dupes ? 'Dupes' : '');
                }
            }
            else
            {
                if (count($users) == 1)
                {
                    $message = 'SelectedUserAddedToGroup' . ($contains_dupes ? 'Dupes' : '');
                }
                else
                {
                    $message = 'SelectedUsersAddedToGroup' . ($contains_dupes ? 'Dupes' : '');
                }
            }

            $this->redirect(Translation :: get($message), ($failures ? true : false),
                    array(Application :: PARAM_ACTION => GroupManager :: ACTION_VIEW_GROUP,
                            GroupManager :: PARAM_GROUP_ID => $group_id));
            exit();
        }
        else
        {
            $this->display_error_page(
                    htmlentities(Translation :: get('NoObjectSelected', null, Utilities :: COMMON_LIBRARIES)));
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(array(Application :: PARAM_ACTION => GroupManager :: ACTION_BROWSE_GROUPS)),
                        Translation :: get('GroupManagerBrowserComponent')));
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(Application :: PARAM_ACTION => GroupManager :: ACTION_VIEW_GROUP,
                                        GroupManager :: PARAM_GROUP_ID => Request :: get(GroupManager :: PARAM_GROUP_ID))),
                        Translation :: get('GroupManagerViewerComponent')));
        $breadcrumbtrail->add_help('group general');
    }

    public function get_additional_parameters()
    {
        return array(GroupManager :: PARAM_GROUP_ID);
    }
}
