<?php
namespace group;

use common\libraries\Display;
use common\libraries\Application;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\Request;
use common\libraries\AdministrationComponent;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use tracking\Event;

/**
 * $Id: truncater.class.php 224 2009-11-13 14:40:30Z kariboe $
 *
 * @package group.lib.group_manager.component
 */
class GroupManagerTruncaterComponent extends GroupManager implements AdministrationComponent
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $user = $this->get_user();

        if (! $this->get_user()->is_platform_admin())
        {
            $this->display_header();
            Display :: error_message(Translation :: get('NotAllowed', null, Utilities :: COMMON_LIBRARIES));
            $this->display_footer();
            exit();
        }

        $ids = Request :: get(GroupManager :: PARAM_GROUP_ID);
        $failures = 0;

        if (! empty($ids))
        {
            if (! is_array($ids))
            {
                $ids = array($ids);
            }

            foreach ($ids as $id)
            {
                $group = $this->retrieve_group($id);
                if (! $group->truncate())
                {
                    $failures ++;
                }
                else
                {
                    Event :: trigger('empty', GroupManager :: APPLICATION_NAME,
                            array(\group\integration\tracking\Change :: PROPERTY_REFERENCE_ID => $group->get_id(),
                                    \group\integration\tracking\Change :: PROPERTY_USER_ID => $user->get_id()));
                }
            }

            if ($failures)
            {
                if (count($ids) == 1)
                {
                    $message = 'SelectedGroupNotEmptied';
                }
                else
                {
                    $message = 'SelectedGroupsNotEmptied';
                }
            }
            else
            {
                if (count($ids) == 1)
                {
                    $message = 'SelectedGroupEmptied';
                }
                else
                {
                    $message = 'SelectedGroupsEmptied';
                }
            }

            if (count($ids) == 1)
                $this->redirect(Translation :: get($message), ($failures ? true : false),
                        array(Application :: PARAM_ACTION => GroupManager :: ACTION_VIEW_GROUP,
                                GroupManager :: PARAM_GROUP_ID => $ids[0]));
            else
                $this->redirect(Translation :: get($message), ($failures ? true : false),
                        array(Application :: PARAM_ACTION => GroupManager :: ACTION_BROWSE_GROUPS));
        }
        else
        {
            $this->display_error_page(
                    htmlentities(Translation :: get('NoObjectSelected', null, Utilities :: COMMON_LIBRARIES)));
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(Application :: PARAM_ACTION => GroupManager :: ACTION_BROWSE_GROUPS)),
                        Translation :: get('GroupManagerBrowserComponent')));
        $breadcrumbtrail->add(
                new Breadcrumb(
                        $this->get_url(
                                array(Application :: PARAM_ACTION => GroupManager :: ACTION_VIEW_GROUP,
                                        GroupManager :: PARAM_GROUP_ID => Request :: get(GroupManager :: PARAM_GROUP_ID))),
                        Translation :: get('GroupManagerViewerComponent')));
        $breadcrumbtrail->add_help('group general');
    }

    public function get_additional_parameters()
    {
        return array(GroupManager :: PARAM_GROUP_ID);
    }
}
