<?php
use common\libraries\Request;
use common\libraries\Utilities;
use common\libraries\Authentication;
use common\libraries\Session;
use common\libraries\Application;
use common\libraries\Display;
use common\libraries\PlatformSetting;

use user\User;
use user\UserLoginSession;
use common\libraries\Translation;
/**
 * This script will load the requested application and launch it.
 */
try
{
    require_once dirname(__FILE__) . '/common/common.inc.php';

    if (common\libraries\PlatformSetting :: get('maintenance_mode', 'admin'))
    {
        Display :: header();
        Display :: error_message(Translation :: get('MaintenanceMessage'));
        Display :: footer();
        exit();
    }

    if (! Request :: get(Application :: PARAM_APPLICATION))
    {
        Request :: set_get(Application :: PARAM_APPLICATION, 'home');
    }

    $application_name = Request :: get(Application :: PARAM_APPLICATION);
    $this_section = $application_name;

    // If application path doesn't exist, block the user
    if (! Application :: is_active(Application :: determine_namespace($application_name)))
    {
        Display :: not_allowed();
    }

    Utilities :: set_application($this_section);

    if (Authentication :: is_valid())
    {
        $user_id = Session :: get_user_id();
        if ($user_id)
        {
            $user = \user\DataManager :: retrieve_by_id(User :: CLASS_NAME, Session :: get_user_id());
        }

        if (! $user instanceof User)
        {
            $user = Authentication :: as_anonymous_user();
        }
        // block user if maintenance mode is activated
        elseif (! $user->is_platform_admin() && PlatformSetting :: get('maintenance_mode', 'core\admin'))
        {
            Display :: maintenance_mode();
        }
        elseif (PlatformSetting :: get('prevent_double_login', 'user'))
        {
            $exceptions = explode(',', PlatformSetting :: get('double_login_exceptions', 'user'));
            $exception_found = array_search($user->get_auth_source(), $exceptions);

            // logout when user is logged in more than once
            if (UserLoginSession :: $single_login === false && $exception_found === false &&
                 $user->is_platform_admin() === false)
            {
                $udm = \user\DataManager :: logout();
                Display :: not_allowed();
            }
        }
        
        if($user) $user->display_terms_conditions();
    }
    else
    {
        $user = Authentication :: as_anonymous_user();
        if (! $user instanceof User)
        {
            Display :: not_allowed();
        }
    }
    
    

    // Launch application
    try
    {
        Application :: launch($application_name, $user);
    }
    catch (Exception $exception)
    {
        Display :: error_page($exception->getMessage());
    }
}
catch (Exception $exception)
{
    Display :: error_message($exception->getMessage());
}
