<?php
namespace reporting\viewer;

use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\DynamicVisualTab;
use common\libraries\DynamicVisualTabsRenderer;

/**
 *
 * @author Hans De Bisschop & Magali Gillard
 * @package reporting.viewer
 */
class HtmlBlockRendition extends BlockRendition
{
    const FORMAT = 'html';
    const VIEW_TABLE = 'table';
    const VIEW_PIE = 'pie_chart';
    const VIEW_RING = 'ring_chart';
    const VIEW_3D_PIE = 'three_dee_pie_chart';
    const VIEW_BAR = 'bar_chart';
    const VIEW_STACKED_BAR = 'stacked_bar_chart';
    const VIEW_LINE = 'line_chart';
    const VIEW_AREA = 'area_chart';
    const VIEW_STACKED_AREA = 'stacked_area_chart';
    const VIEW_POLAR = 'polar_chart';
    const VIEW_RADAR = 'radar_chart';
    const VIEW_CSV = 'csv';
    const VIEW_XML = 'xml';
    const VIEW_PDF = 'pdf';
    const VIEW_XLSX = 'xlsx';
    const VIEW_ODS = 'ods';

    public function render()
    {
        $rendered_block = $this->get_content();

        if (count($this->get_block()->get_views()) > 1)
        {
            $tabs = new DynamicVisualTabsRenderer(Utilities :: get_classname_from_object($this->get_block(), true),
                    $rendered_block);

            $context_parameters = $this->get_context()->get_context()->get_parameters();

            foreach ($this->get_block()->get_views() as $view)
            {
                $view_parameters = $context_parameters[Manager :: PARAM_VIEWS] ? $context_parameters[Manager :: PARAM_VIEWS] : array();
                $view_parameters[$this->get_block()->get_id()] = $view;

                $view_parameters = array_merge($context_parameters, array(Manager :: PARAM_VIEWS => $view_parameters));

                $is_current_view = $view == $this->get_context()->determine_current_block_view(
                        $this->get_block()->get_id()) ? true : false;

                $tabs->add_tab(
                        new DynamicVisualTab($view,
                                Translation :: get(Utilities :: underscores_to_camelcase(self :: FORMAT . '_' . $view)),
                                Theme :: get_image_path(__NAMESPACE__) . 'rendition/block/' . self :: FORMAT . '/' . $view . '.png',
                                $this->get_context()->get_context()->get_url($view_parameters), $is_current_view));
            }

            return $tabs->render();
        }
        else
        {
            return $rendered_block;
        }
    }
}
