<?php
namespace reporting\viewer;

use common\libraries\Utilities;

/**
 *
 * @author Hans De Bisschop & Magali Gillard
 * @package reporting.viewer
 */
abstract class TemplateRendition
{
    const FORMAT_XML = 'xml';
    const FORMAT_HTML = 'html';
    const FORMAT_JSON = 'json';
    const FORMAT_CSV = 'csv';
    const FORMAT_XLSX = 'xlsx';
    const FORMAT_ODS = 'ods';
    const VIEW_DEFAULT = 'default';

    /**
     *
     * @var \reporting\TemplateRenditionImplementation
     */
    private $rendition_implementation;

    /**
     *
     * @param \reporting\TemplateRenditionImplementation $rendition_implementation
     */
    public function __construct(\reporting\TemplateRenditionImplementation $rendition_implementation)
    {
        $this->rendition_implementation = $rendition_implementation;
    }

    /**
     *
     * @return \reporting\TemplateRenditionImplementation
     */
    public function get_rendition_implementation()
    {
        return $this->rendition_implementation;
    }

    /**
     *
     * @param \reporting\TemplateRenditionImplementation $rendition_implementation
     */
    public function set_rendition_implementation($rendition_implementation)
    {
        $this->rendition_implementation = $rendition_implementation;
    }

    /**
     *
     * @return \common\libraries\Application
     */
    public function get_context()
    {
        return $this->rendition_implementation->get_context();
    }

    /**
     *
     * @param \common\libraries\Application $context
     */
    public function set_context($context)
    {
        $this->rendition_implementation->set_context($context);
    }

    /**
     *
     * @return \reporting\ReportingTemplate
     */
    public function get_template()
    {
        return $this->rendition_implementation->get_template();
    }

    /**
     *
     * @param \reporting\ReportingTemplate $template
     */
    public function set_template($template)
    {
        $this->rendition_implementation->set_template($template);
    }

    /**
     *
     * @param \reporting\TemplateRenditionImplementation $rendition_implementation
     * @return string
     */
    public static function launch($rendition_implementation)
    {
        return self :: factory($rendition_implementation)->render();
    }

    /**
     *
     * @param \reporting\TemplateRenditionImplementation $rendition_implementation
     * @return \reporting\TemplateRendition
     */
    public static function factory($rendition_implementation)
    {
        $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($rendition_implementation->get_format()) . Utilities :: underscores_to_camelcase(
                $rendition_implementation->get_view()) . 'TemplateRendition';
        return new $class($rendition_implementation);
    }

    public static function get_format()
    {
        return static :: FORMAT;
    }
}
