<?php
namespace reporting\viewer;

use common\libraries\Filesystem;
use common\libraries\Path;
use common\libraries\Utilities;
use common\libraries\Translation;
use PHPExcel;
use PHPExcel_IOFactory;

require_once Path :: get_plugin_path() . 'phpexcel/PHPExcel.php';
/**
 *
 * @author Hans De Bisschop & Magali Gillard
 * @package reporting.viewer
 */
class XlsxDefaultTemplateRendition extends XlsxTemplateRendition
{

    private $letters = array(0 => 'A', 1 => 'B', 2 => 'C', 3 => 'D', 4 => 'E', 5 => 'F', 6 => 'G', 7 => 'H', 8 => 'I',
            9 => 'J', 10 => 'K', 11 => 'L', 12 => 'M', 13 => 'N', 14 => 'O', 15 => 'P', 16 => 'Q', 17 => 'R', 18 => 'S',
            19 => 'T', 20 => 'U', 21 => 'V', 22 => 'W', 23 => 'X', 24 => 'Y', 25 => 'Z');

    /**
     *
     * @var \PHPExcel
     */
    private $php_excel;

    /**
     *
     * @return string
     */
    public function render()
    {
        $current_block_id = $this->get_context()->get_current_block();

        $this->php_excel = new PHPExcel();

        if ($this->get_context()->show_all())
        {
            $views = $this->get_context()->get_current_view();
            $specific_views = array();

            foreach ($views as $key => $view)
            {
                if ($view == static :: get_format())
                {
                    $specific_views[] = $key;
                }
            }

            // Render the specific views as called from another view which can show multiple blocks
            if (count($specific_views) > 0)
            {
                if (count($specific_views) == 1)
                {
                    $current_block = $this->get_template()->get_block($specific_views[0]);
                    $file_name = Translation :: get(Utilities :: get_classname_from_object($current_block), null,
                            Utilities :: get_namespace_from_object($current_block)) . date('_Y-m-d_H-i-s') . '.xlsx';

                    BlockRenditionImplementation :: launch($this, $current_block, $this->get_format(),
                            XlsxBlockRendition :: VIEW_DEFAULT);
                }
                else
                {
                    $this->php_excel->removeSheetByIndex(0);
                    $data = array();

                    foreach ($specific_views as $key => $specific_view)
                    {
                        $block = $this->get_template()->get_block($specific_view);
                        $this->php_excel->createSheet($key);
                        $this->php_excel->setActiveSheetIndex($key);

                        BlockRenditionImplementation :: launch($this, $block, $this->get_format(),
                                XlsxBlockRendition :: VIEW_DEFAULT);
                    }

                    $file_name = Translation :: get(Utilities :: get_classname_from_object($this->get_template()), null,
                            Utilities :: get_namespace_from_object($this->get_template())) . date('_Y-m-d_H-i-s') . '.xlsx';
                }
            }
            // No specific view was set and we are rendering everything, so render everything
            else
            {
                $this->php_excel->removeSheetByIndex(0);
                $data = array();

                foreach ($this->get_template()->get_blocks() as $key => $block)
                {
                    $this->php_excel->createSheet($key);
                    $this->php_excel->setActiveSheetIndex($key);

                    BlockRenditionImplementation :: launch($this, $block, $this->get_format(),
                            XlsxBlockRendition :: VIEW_DEFAULT);
                }

                $file_name = Translation :: get(Utilities :: get_classname_from_object($this->get_template()), null,
                        Utilities :: get_namespace_from_object($this->get_template())) . date('_Y-m-d_H-i-s') . '.xlsx';
            }
        }
        else
        {
            $current_block_id = $this->get_context()->get_current_block();
            $current_block = $this->get_template()->get_block($current_block_id);
            $file_name = Translation :: get(Utilities :: get_classname_from_object($current_block), null,
                    Utilities :: get_namespace_from_object($current_block)) . date('_Y-m-d_H-i-s') . '.xlsx';

            BlockRenditionImplementation :: launch($this, $current_block, $this->get_format(),
                    XlsxBlockRendition :: VIEW_DEFAULT);
        }

        $file = Path :: get(SYS_ARCHIVE_PATH) . Filesystem :: create_unique_name(Path :: get(SYS_ARCHIVE_PATH),
                $file_name);

        $php_excel_writer = PHPExcel_IOFactory :: createWriter($this->php_excel, 'Excel2007');
        $php_excel_writer->save($file);

        $this->php_excel->disconnectWorksheets();
        unset($this->php_excel);

        return $file;
    }

    /**
     *
     * @return multitype:string
     */
    public function get_letters()
    {
        return $this->letters;
    }

    /**
     *
     * @return \PHPExcel
     */
    public function get_php_excel()
    {
        return $this->php_excel;
    }
}
