<?php
namespace reporting\viewer;

use common\libraries\Export;
use common\libraries\Utilities;
use common\libraries\Translation;

/**
 *
 * @author Hans De Bisschop & Magali Gillard
 * @package reporting.viewer
 */
class XmlDefaultTemplateRendition extends XmlTemplateRendition
{

    public function render()
    {
        if ($this->get_context()->show_all())
        {
            $views = $this->get_context()->get_current_view();
            $specific_views = array();

            foreach ($views as $key => $view)
            {
                if ($view == static :: get_format())
                {
                    $specific_views[] = $key;
                }
            }

            // Render the specific views as called from another view which can show multiple blocks
            if (count($specific_views) > 0)
            {
                if (count($specific_views) == 1)
                {
                    $current_block = $this->get_template()->get_block($specific_views[0]);
                    $file_name = Translation :: get(Utilities :: get_classname_from_object($current_block), null,
                            Utilities :: get_namespace_from_object($current_block)) . date('_Y-m-d_H-i-s') . '.xml';

                    $data = BlockRenditionImplementation :: launch($this, $current_block, $this->get_format(),
                            CsvBlockRendition :: VIEW_DEFAULT);
                }
                else
                {
                    $data = array();

                    foreach ($specific_views as $specific_view)
                    {
                        $block = $this->get_template()->get_block($specific_view);
                        $data[Utilities :: get_classname_from_object($block)] = BlockRenditionImplementation :: launch(
                                $this, $block, $this->get_format(), CsvBlockRendition :: VIEW_DEFAULT);
                    }

                    $file_name = Translation :: get(Utilities :: get_classname_from_object($this->get_template()), null,
                            Utilities :: get_namespace_from_object($this->get_template())) . date('_Y-m-d_H-i-s') . '.xml';
                }
            }
            // No specific view was set and we are rendering everything, so render everything
            else
            {
                $data = array();

                foreach ($this->get_template()->get_blocks() as $key => $block)
                {
                    $data[Utilities :: get_classname_from_object($block)] = BlockRenditionImplementation :: launch(
                            $this, $block, $this->get_format(), CsvBlockRendition :: VIEW_DEFAULT);
                }

                $file_name = Translation :: get(Utilities :: get_classname_from_object($this->get_template()), null,
                        Utilities :: get_namespace_from_object($this->get_template())) . date('_Y-m-d_H-i-s') . '.xml';
            }
        }
        else
        {
            $current_block_id = $this->get_context()->get_current_block();
            $current_block = $this->get_template()->get_block($current_block_id);
            $file_name = Translation :: get(Utilities :: get_classname_from_object($current_block), null,
                    Utilities :: get_namespace_from_object($current_block)) . date('_Y-m-d_H-i-s') . '.xml';

            $data = BlockRenditionImplementation :: launch($this, $current_block, $this->get_format(),
                    CsvBlockRendition :: VIEW_DEFAULT);
        }

        $export = Export :: factory('xml', $data);
        $export->set_filename($file_name);
        return $export->write_to_file();
    }
}
