<?php
namespace repository\content_object\assessment;

use common\libraries\Translation;
use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use common\libraries\DataClassRetrievesParameters;
use common\libraries\EqualityCondition;
use repository\ComplexContentObjectItem;

class AssessmentBuilderRandomizerComponent extends AssessmentBuilder
{

    public function run()
    {
        $complex_content_object_items = \repository\DataManager :: retrieve_complex_content_object_items(
            ComplexContentObjectItem :: class_name(),
            new DataClassRetrievesParameters(
                new EqualityCondition(
                    new PropertyConditionVariable(
                        ComplexContentObjectItem :: class_name(),
                        ComplexContentObjectItem :: PROPERTY_PARENT),
                    new StaticConditionVariable($this->get_parent_content_object_id()))));

        $supported_types = array(
            \repository\content_object\fill_in_blanks_question\ComplexFillInBlanksQuestion :: class_name(),
            \repository\content_object\hotspot_question\ComplexHotspotQuestion :: class_name(),
            \repository\content_object\assessment_matching_question\ComplexAssessmentMatchingQuestion :: class_name(),
            \repository\content_object\assessment_matrix_question\ComplexAssessmentMatrixQuestion :: class_name(),
            \repository\content_object\assessment_multiple_choice_question\ComplexAssessmentMultipleChoiceQuestion :: class_name(),
            \repository\content_object\ordering_question\ComplexOrderingQuestion :: class_name(),
            \repository\content_object\assessment_select_question\ComplexAssessmentSelectQuestion :: class_name());

        $failures = 0;
        $questions = 0;

        while ($complex_content_object_item = $complex_content_object_items->next_result())
        {
            if (in_array($complex_content_object_item->class_name(), $supported_types))
            {
                $questions ++;

                if (! $complex_content_object_item->get_random())
                {
                    $complex_content_object_item->set_random(1);
                    if (! $complex_content_object_item->update())
                    {
                        $failures ++;
                    }
                }
            }
        }

        if ($failures > 0)
        {
            if ($failures == $questions && $questions == 1)
            {
                $message = 'QuestionNotSuccessfullyRandomized';
            }
            elseif ($failures == $questions && $questions > 1)
            {
                $message = 'QuestionsNotSuccessfullyRandomized';
            }
            else
            {
                $message = 'SomeQuestionsNotSuccessfullyRandomized';
            }
        }
        else
        {
            if ($questions == 0)
            {
                $message = 'NoQuestions';
            }
            elseif ($questions == 1)
            {
                $message = 'QuestionSuccessfullyRandomized';
            }
            else
            {
                $message = 'QuestionsSuccessfullyRandomized';
            }
        }

        $this->redirect(
            Translation :: get($message),
            false,
            array(
                self :: PARAM_BUILDER_ACTION => self :: ACTION_BROWSE,
                self :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id()));

        exit();

        // $trail = BreadcrumbTrail :: get_instance();
        // $trail->add(new Breadcrumb($this->get_url(array(
        // ComplexBuilder :: PARAM_BUILDER_ACTION => ComplexBuilder :: ACTION_BROWSE)),
        // $this->get_root_content_object()->get_title()));
        // $trail->add(new Breadcrumb($this->get_url(array()), Translation :: get('MergeAssessment')));
        // $trail->add_help('repository assessment builder');
        // $assessment = $this->get_root_content_object();

        // if (! \repository\viewer\Manager :: is_ready_to_be_published())
        // {
        // $repository_viewer = \repository\viewer\Manager :: construct($this);
        // $repository_viewer->set_maximum_select(\repository\viewer\Manager :: SELECT_SINGLE);
        // $repository_viewer->set_parameter(\repository\viewer\Manager :: PARAM_ID, Request ::
        // get(\repository\viewer\Manager :: PARAM_ID));

        // $repository_viewer->get_parent()->parse_input_from_table();
        // $repository_viewer->run();
        // }
        // else
        // {
        // $selected_assessment = \repository\DataManager :: retrieve_content_object(\repository\viewer\Manager ::
        // get_selected_objects(), Assessment :: get_type_name());
        // $display = ContentObjectRenditionImplementation :: launch($selected_assessment, ContentObjectRendition ::
        // FORMAT_HTML, ContentObjectRendition :: VIEW_FULL, $this);

        // $bar = $this->get_action_bar($selected_assessment);

        // //$html[] = '<h3>' . Translation :: get('SelectedAssessment') . '</h3>';
        // $html[] = $display;
        // $html[] = '<br />';
        // $html[] = $bar->as_html();
        // $html[] = '<h3>' . Translation :: get('SelectQuestions') . '</h3>';

        // $params = array(\repository\viewer\Manager :: PARAM_ID => Request :: get(\repository\viewer\Manager ::
        // PARAM_ID));
        // $table = new ObjectBrowserTable($this, array_merge($params, $this->get_parameters()),
        // $this->get_condition($selected_assessment));
        // $html[] = $table->as_html();

        // $this->display_header();
        // echo '<br />' . implode("\n", $html);
        // $this->display_footer();
        // }
    }
}
