<?php
namespace repository\content_object\assessment;

use common\libraries\Session;

use repository\ComplexDisplayPreview;
use repository\ComplexDisplay;

class AssessmentComplexDisplayPreview extends ComplexDisplayPreview implements AssessmentComplexDisplaySupport
{
    const TEMPORARY_STORAGE = 'assessment_preview';

    public function run()
    {
        ComplexDisplay :: launch(Assessment :: class_name(), $this);
    }

    /**
     * Preview mode, so always return true.
     *
     * @param $right
     * @return boolean
     */
    public function is_allowed($right)
    {
        return true;
    }

    /**
     * Preview mode, so no actual saving done.
     *
     * @param int $complex_question_id
     * @param mixed $answer
     * @param int $score
     */
    public function save_assessment_answer($complex_question_id, $answer, $score, $hint)
    {
        $parameters = array();
        $parameters[DummyQuestionAttemptsTracker :: PROPERTY_ASSESSMENT_ATTEMPT_ID] = $this->get_root_content_object()->get_id();
        $parameters[DummyQuestionAttemptsTracker :: PROPERTY_QUESTION_CID] = $complex_question_id;
        $parameters[DummyQuestionAttemptsTracker :: PROPERTY_ANSWER] = $answer;
        $parameters[DummyQuestionAttemptsTracker :: PROPERTY_SCORE] = $score;
        $parameters[DummyQuestionAttemptsTracker :: PROPERTY_FEEDBACK] = '';
        $parameters[DummyQuestionAttemptsTracker :: PROPERTY_HINT] = $hint;

        $answers = Session :: retrieve(self :: TEMPORARY_STORAGE);
        $answers[$this->get_root_content_object()->get_id()][$complex_question_id] = new DummyQuestionAttemptsTracker($parameters);
        Session :: register(self :: TEMPORARY_STORAGE, $answers);
    }

    /**
     * Preview mode, so no actual total score will be saved.
     *
     * @param int $total_score
     */
    public function save_assessment_result($total_score)
    {
        $answers = Session :: retrieve(self :: TEMPORARY_STORAGE);
        unset($answers[$this->get_root_content_object()->get_id()]);
        Session :: register(self :: TEMPORARY_STORAGE, $answers);
    }

    /**
     * Preview mode, so there is no acrual attempt.
     */
    public function get_assessment_current_attempt_id()
    {
    }

    public function get_assessment_question_attempts()
    {
        $answers = Session :: retrieve(self :: TEMPORARY_STORAGE);
        return $answers[$this->get_root_content_object()->get_id()];
    }

    public function get_assessment_question_attempt($complex_question_id)
    {
        $answers = $this->get_assessment_question_attempts($complex_question_id);
        return $answers[$complex_question_id];
    }

    /**
     * Preview mode is launched in standalone mode,
     * so there's nothing to go back to.
     *
     * @return void
     */
    public function get_assessment_back_url()
    {
        return null;
    }

    /**
     * Preview mode is launched in standalone mode,
     * so there's nothing to continue to.
     *
     * @return void
     */
    public function get_assessment_continue_url()
    {
        return null;
    }

    public function get_assessment_current_url()
    {
        return null;
    }

    public function get_assessment_feedback_configuration($complex_content_object_item)
    {
        $dummy_configuration = new FeedbackDisplayConfiguration();
        $dummy_configuration->set_feedback_type(FeedbackDisplayConfiguration :: TYPE_BOTH);
        $dummy_configuration->disable_feedback_per_page();
        $dummy_configuration->enable_feedback_summary();
        $dummy_configuration->enable_hints();
        return $dummy_configuration;
    }

    public function get_assessment_parameters()
    {
        return array();
    }

     // FUNCTIONS FOR COMPLEX DISPLAY SUPPORT
     public function is_allowed_to_edit_content_object()
    {
       return true;
    }

    public function is_allowed_to_view_content_object()
    {
        return true;
    }

    public function is_allowed_to_add_child()
    {
        return true;
    }
    public function is_allowed_to_delete_child()
    {
        return true;
    }
    public function is_allowed_to_delete_feedback()
    {
        return true;
    }
    public function is_allowed_to_edit_feedback()
    {
        return true;
    }
	/* (non-PHPdoc)
     * @see \repository\content_object\assessment\AssessmentComplexDisplaySupport::get_registered_question_ids()
     */
    public function get_registered_question_ids()
    {
        // TODO Auto-generated method stub

    }

	/* (non-PHPdoc)
     * @see \repository\content_object\assessment\AssessmentComplexDisplaySupport::register_question_ids()
     */
    public function register_question_ids($question_ids)
    {
        // TODO Auto-generated method stub

    }

}
