<?php
namespace repository\content_object\assessment;

use common\libraries\Translation;
use repository\ContentObjectResourceRenderer;
use common\libraries\Utilities;
use common\libraries\Theme;

/**
 * $Id: question_result_display.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.complex_display.assessment.component.result_viewer
 */
abstract class QuestionResultDisplay
{

    private $complex_content_object_question;

    private $question;

    private $question_nr;

    private $answers;

    private $score;

    private $hints;

    private $feedback;

    private $form;

    private $results_viewer;

    private $can_change;

    public function __construct($results_viewer, &$form, $complex_content_object_question, $question_nr, $answers,
        $score, $hints, $feedback, $can_change)
    {
        $this->complex_content_object_question = $complex_content_object_question;
        $this->question_nr = $question_nr;
        $this->question = $complex_content_object_question->get_ref_object();
        $this->answers = $answers;
        $this->score = $score;
        $this->hints = $hints;
        $this->feedback = $feedback;
        $this->form = $form;
        $this->results_viewer = $results_viewer;
        $this->can_change = $can_change;
    }

    public function get_complex_content_object_question()
    {
        return $this->complex_content_object_question;
    }

    public function get_question()
    {
        return $this->question;
    }

    public function get_question_nr()
    {
        return $this->question_nr;
    }

    public function get_answers()
    {
        return $this->answers;
    }

    public function get_score()
    {
        return $this->score;
    }

    public function get_feedback()
    {
        return $this->feedback;
    }

    public function get_results_viewer()
    {
        return $this->results_viewer;
    }

    public function can_change()
    {
        return $this->can_change;
    }

    public function display()
    {
        $this->display_header();

        if ($this->add_borders())
        {
            $header = array();
            $header[] = '<div class="with_borders">';

            $this->form->addElement('html', implode("\n", $header));
        }

        $this->form->addElement('html', $this->display_question_result());

        if ($this->add_borders())
        {
            $footer = array();
            $footer[] = '<div class="clear"></div>';
            $footer[] = '</div>';
            $this->form->addElement('html', implode("\n", $footer));
        }

        $this->display_feedback();

        $this->form->addElement('html', $this->display_footer());
    }

    public function display_question_result()
    {
        return $this->get_score() . '<br />';
    }

    public function display_header()
    {
        $html = array();

        $html[] = '<div class="question">';
        $html[] = '<div class="title">';
        $html[] = '<div class="number">';
        $html[] = '<div class="bevel">';
        $html[] = $this->question_nr . '.';
        $html[] = '</div>';
        $html[] = '</div>';
        $html[] = '<div class="text">';

        $html[] = '<div class="bevel" style="float: left;">';
        $html[] = $this->question->get_title();
        $html[] = '</div>';
        $html[] = '<div class="bevel" style="text-align: right;">';
        $this->form->addElement('html', implode("\n", $html));
        $html = array();

        if ($this->hints > 0)
        {
            $variable = $this->hints == 1 ? 'HintUsed' : 'HintsUsed';
            $label = Translation :: get($variable, array('COUNT' => $this->hints));

            $html[] = '<img style="float: none; vertical-align: baseline;" src="' .
                 Theme :: get_image_path(__NAMESPACE__) . 'buttons/button_hint.png" alt="' . $label . '" title="' .
                 $label . '" />&nbsp;&nbsp;';
        }

        if (! $this->can_change)
        {
            if ($this->get_results_viewer()->display_numeric_feedback())
            {
                $html[] = $this->get_score() . ' / ' . $this->get_complex_content_object_question()->get_weight();
            }
            else
            {
                $html[] = '&nbsp;';
            }
        }
        else
        {
            for ($i = - $this->get_complex_content_object_question()->get_weight(); $i <=
                 $this->get_complex_content_object_question()->get_weight(); $i ++)
            {
                $score[$i] = $i;
            }

            $renderer = $this->form->defaultRenderer();

            $this->form->addElement('select', $this->complex_content_object_question->get_id() . '_score', '', $score);
            $renderer->setElementTemplate('{element}', $this->complex_content_object_question->get_id() . '_score');
            $defaults[$this->complex_content_object_question->get_id() . '_score'] = $this->get_score();
            $this->form->setDefaults($defaults);
        }

        $html[] = '</div>';

        $html[] = '</div>';
        $html[] = '<div class="clear"></div>';
        $html[] = '</div>';
        $html[] = '<div class="answer">';

        $description = $this->question->get_description();

        if ($this->question->has_description())
        {
            $html[] = '<div class="description">';

            $renderer = new ContentObjectResourceRenderer($this, $description);
            $html[] = $renderer->run();

            $html[] = '<div class="clear"></div>';
            $html[] = '</div>';
        }

        $html[] = '<div class="clear"></div>';

        $this->form->addElement('html', implode("\n", $html));
    }

    public function display_feedback()
    {
        if (! $this->can_change)
        {
            $html[] = '<div class="splitter">';
            $html[] = Translation :: get('CourseAdministratorFeedback');
            $html[] = '</div>';
            $html[] = '<div class="with_borders">';
            if (!$this->feedback) {
                $html[] = '<div class="warning-message">' . Translation :: get('NotYetRatedWarning') . '</div>';
            }
            $this->form->addElement('html', implode("\n", $html));
            $html = array();
            if ($this->feedback)
            {
                $html[] = $this->feedback;
            }
            $html[] = '</div>';
            $this->form->addElement('html', implode("\n", $html));
        }
        else
        {
            $html[] = '<div class="splitter">';
            $html[] = Translation :: get('CourseAdministratorFeedback');
            $html[] = '</div>';
            $html[] = '<div class="with_borders">';
            
            if (!$this->feedback)
            {
                $html[] = '<div class="warning-message">' . Translation :: get('NotYetRatedWarning') . '</div>';
            }

            $this->form->addElement('html', implode("\n", $html));
            $html = array();

            $this->form->add_html_editor($this->complex_content_object_question->get_id() . '_feedback', '', false);
            $defaults[$this->complex_content_object_question->get_id() . '_feedback'] = $this->get_feedback();
            $this->form->setDefaults($defaults);

            $html[] = '</div>';
            $this->form->addElement('html', implode("\n", $html));
        }
    }

    public function display_footer()
    {
        $html[] = '</div>';
        $html[] = '</div>';

        $footer = implode("\n", $html);
        return $footer;
    }

    public function add_borders()
    {
        return false;
    }

    public static function factory($results_viewer, &$form, $complex_content_object_question, $question_nr, $answers,
        $score, $hints, $feedback, $can_change)
    {
        $class = $complex_content_object_question->get_ref_object()->get_type();
        $type = $class :: get_type_name();

        $file = dirname(__FILE__) . '/question_result_display/' . $type . '_result_display.class.php';

        if (! file_exists($file))
        {
            die('file does not exist: ' . $file);
        }

        require_once ($file);

        $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($type) . 'ResultDisplay';

        $question_result_display = new $class(
            $results_viewer,
            $form,
            $complex_content_object_question,
            $question_nr,
            $answers,
            $score,
            $hints,
            $feedback,
            $can_change);

        return $question_result_display;
    }
}
