<?php
namespace repository\content_object\assessment;

use common\libraries\DelegateComponent;
use common\libraries\FormValidator;
use common\libraries\Translation;
use common\libraries\InCondition;
use repository\ComplexContentObjectItem;
use common\libraries\Utilities;

/**
 * $Id: result_viewer.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.complex_display.assessment.component
 */
class AssessmentComplexDisplayResultsViewerComponent extends AssessmentComplexDisplay implements DelegateComponent
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $form = new FormValidator('result_viewer', 'post', $this->get_url());

        $results = $this->get_parent()->retrieve_assessment_results();
        $question_cids = array_keys($results);

        if (count($question_cids) <= 0)
        {
            throw new \Exception(Translation :: get('ThisAssessmentHasNoAnswers'));
        }

        $this->display_header();

        $condition = new InCondition(
            ComplexContentObjectItem :: PROPERTY_ID,
            $question_cids,
            ComplexContentObjectItem :: get_table_name());

        $questions_cloi = \repository\DataManager :: retrieve_complex_content_object_items(
            ComplexContentObjectItem :: class_name(),
            $condition);

        $assessment = $this->get_root_content_object();
        $form->add_information_message('information', $assessment->get_title(), $assessment->get_description(), true);
        $form->addElement('html', '<br />');

        $total_score = 0;
        $total_weight = 0;
        $question_number = 1;

        while ($question_cloi = $questions_cloi->next_result())
        {
            $result = $results[$question_cloi->get_id()];

            $question = \repository\DataManager :: retrieve_content_object($question_cloi->get_ref());
            $answers = unserialize($result['answer']);
            $feedback = $result['feedback'];

            $question_cloi->set_ref_object($question);

            $score = $result['score'];
            $score = round($score * 100) / 100;

            $total_score += $score;
            $total_weight += $question_cloi->get_weight();

            $display = QuestionResultDisplay :: factory(
                $this,
                $form,
                $question_cloi,
                $question_number,
                $answers,
                $score,
                $result['hint'],
                $feedback,
                $this->get_parent()->can_change_answer_data());
            $display->display();

            $question_number ++;
        }

        if ($form->validate())
        {
            $values = $form->exportValues();

            $question_forms = array();
            foreach ($values as $key => $value)
            {
                $split = split('_', $key);
                if (is_numeric($split[0]))
                {
                    $question_forms[$split[0]][$split[1]] = $value;
                }
            }

            $total_score = 0;

            foreach ($question_forms as $question_id => $question_form)
            {
                $score = $question_form['score'];
                $feedback = $question_form['feedback'];

                $this->change_answer_data($question_id, $score, $feedback);

                $total_score += $score;
            }

            $percent = round(($total_score / $total_weight) * 100);
            $this->change_total_score($percent);
        }

        if ($this->display_numeric_feedback())
        {
            $html = array();

            $html[] = '<div class="question">';
            $html[] = '<div class="title">';
            $html[] = '<div class="text">';
            $html[] = '<div class="bevel" style="float: left;">';
            $html[] = Translation :: get('TotalScore');
            $html[] = '</div>';
            $html[] = '<div class="bevel" style="text-align: right;">';

            if ($total_score < 0)
            {
                $total_score = 0;
            }

            $percent = round(($total_score / $total_weight) * 100);

            $html[] = $total_score . ' / ' . $total_weight . ' (' . $percent . '%)';
            $html[] = '</div>';

            $html[] = '</div></div></div>';
            $html[] = '<div class="clear"></div>';

            $form->addElement('html', implode("\n", $html));
        }

        $buttons[] = $form->createElement(
            'style_submit_button',
            'submit',
            Translation :: get('Save', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'positive'));
        $buttons[] = $form->createElement(
            'style_reset_button',
            'reset',
            Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES),
            array('class' => 'normal empty'));

        if ($this->get_parent()->can_change_answer_data())
        {
            $form->addGroup($buttons, 'buttons', null, '&nbsp;', false);
        }

        $form->display();

        $this->display_footer();
    }
}
