<?php
namespace repository\content_object\assessment;

use common\libraries\Request;
use common\libraries\StringUtilities;
use common\libraries\FormValidator;
use common\libraries\Utilities;
use common\libraries\Translation;



class AssessmentResultViewerForm extends FormValidator
{
    const FORM_NAME = 'assessment_result_viewer_form';
    const PAGE_NUMBER = 'assessment_result_page_number';
    
    /**
     * @var AssessmentResultProcessor
     */
    private $assessment_result_processor;

    public function __construct(AssessmentResultProcessor $assessment_result_processor, $method = 'post', $action = null)
    {
        parent :: __construct('assessment_result_viewer_form', $method, $action);
        
        $this->assessment_result_processor = $assessment_result_processor;
        
        $this->add_general();
        $this->add_buttons();
        $this->add_results();
        $this->add_buttons();
    }

    public function get_page_number()
    {
        return $this->assessment_result_processor->get_assessment_viewer()->get_questions_page();
    }

    public function get_total_pages()
    {
        return $this->assessment_result_processor->get_assessment_viewer()->get_total_pages();
    }

    public function add_general()
    {
        
        $current_page = self :: PAGE_NUMBER . '-' . $this->get_page_number();
        $this->addElement('hidden', $current_page, $this->get_page_number());

        $start_time = Request :: post('start_time');
        $start_time = $start_time ? $start_time : 0;

        $this->addElement('hidden', 'start_time', $start_time, array('id' => 'start_time'));
    }

    public function add_buttons()
    {
        $finished = $this->assessment_result_processor->is_finished();
        
        if (! $finished && $this->get_page_number() != ($this->get_total_pages() + 1))
        {
            //$progress = round(($this->get_page_number() / $this->get_total_pages()) * 100);
            //Display::get_progress_bar($progress)
            // TODO: Temporary fix
            $this->get_page_number();
            $this->get_total_pages();
        
     //            $this->addElement('html', '<div style="float: left; padding: 7px; font-weight: bold; line-height: 100%;">' . Translation :: get('PageNumberOfTotal', array(
        //                    'CURRENT' => $this->get_page_number(),
        //                    'TOTAL' => $this->get_total_pages())) . '</div>');
        }
        
        if (! $finished && ($this->get_page_number() < $this->assessment_result_processor->get_assessment_viewer()->get_total_pages()))
        {
            $buttons[] = $this->createElement('style_submit_button', 'next', Translation :: get('Next', null, Utilities :: COMMON_LIBRARIES), array(
                    'class' => 'normal next'));
        }
        elseif (! $finished && $this->get_page_number() == $this->assessment_result_processor->get_assessment_viewer()->get_total_pages())
        {
            
            if ($this->assessment_result_processor->get_assessment_viewer()->display_numeric_feedback())
            {
                $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('SeeResults'), array(
                        'class' => 'positive'));
            }
            else
            {
                $buttons[] = $this->createElement('style_submit_button', 'submit', Translation :: get('ViewResults'), array(
                        'class' => 'positive'));
            }
        }
        elseif ($finished || $this->get_page_number() == ($this->get_total_pages() + 1))
        {
            $back_url = $this->assessment_result_processor->get_assessment_viewer()->get_assessment_back_url();
            $continue_url = $this->assessment_result_processor->get_assessment_viewer()->get_assessment_continue_url();
            $current_url = $this->assessment_result_processor->get_assessment_viewer()->get_assessment_current_url();
            
            if ($this->assessment_result_processor->get_assessment_viewer()->get_root_content_object()->has_unlimited_attempts())
            {
                $buttons[] = $this->createElement('static', null, null, '<a href="' . $current_url . '" class="button refresh_button" target="_parent">' . Translation :: get('DoAssessmentAgain') . '</a>');
            }
            
            if (! StringUtilities :: is_null_or_empty($back_url))
            {
                $buttons[] = $this->createElement('static', null, null, '<a href="' . $back_url . '" class="button negative_button finish_button" target="_parent">' . Translation :: get('Finish') . '</a>');
            }
            
            if (! StringUtilities :: is_null_or_empty($continue_url))
            {
                $buttons[] = $this->createElement('static', null, null, '<a href="' . $continue_url . '" class="button normal_button" target="_parent">' . Translation :: get('ContinueSession') . '</a>');
            }
        }
        
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
        
        $renderer = $this->defaultRenderer();
        $renderer->setElementTemplate('<div style="float: right;">{element}</div><br /><br />', 'buttons');
        $renderer->setGroupElementTemplate('{element}', 'buttons');
    }

    public function add_results()
    {
        $question_results = $this->assessment_result_processor->get_question_results();
        $question_results = implode("\n", $question_results);
        $this->addElement('html', $question_results);
    }
}
