<?php
namespace repository\content_object\assessment;

use common\libraries\StringUtilities;
use repository\ContentObjectResourceRenderer;
use common\libraries\Utilities;
use common\libraries\Theme;
use common\libraries\Translation;

/**
 * $Id: question_result_display.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.complex_display.assessment.component.viewer.wizard.inc
 */
abstract class AssessmentQuestionResultDisplay
{

    /**
     *
     * @var AssessmentResultProcessor
     */
    private $assessment_result_processor;

    private $complex_content_object_question;

    private $question;

    private $question_nr;

    private $answers;

    private $score;

    private $hints;

    public function __construct(AssessmentResultProcessor $assessment_result_processor, $complex_content_object_question,
        $question_nr, $answers, $score, $hints)
    {
        $this->assessment_result_processor = $assessment_result_processor;
        $this->complex_content_object_question = $complex_content_object_question;
        $this->question_nr = $question_nr;
        $this->question = $complex_content_object_question->get_ref_object();
        $this->answers = $answers;
        $this->score = $score;
        $this->hints = $hints;
    }

    public function get_assessment_result_processor()
    {
        return $this->assessment_result_processor;
    }

    public function get_complex_content_object_question()
    {
        return $this->complex_content_object_question;
    }

    public function get_question()
    {
        return $this->question;
    }

    public function get_question_nr()
    {
        return $this->question_nr;
    }

    public function get_answers()
    {
        return $this->answers;
    }

    public function get_score()
    {
        return $this->score;
    }

    public function get_hints()
    {
        return $this->hints;
    }

    public function as_html()
    {
        $html = array();

        $html[] = $this->header();

        if ($this->add_borders())
        {
            $html[] = '<div class="with_borders">';
        }

        $html[] = $this->get_question_result();

        if ($this->add_borders())
        {
            $html[] = '<div class="clear"></div>';
            $html[] = '</div>';
        }

        $html[] = $this->footer();
        return implode("\n", $html);
    }

    public function display()
    {
        echo $this->as_html();
    }

    public function get_question_result()
    {
        return $this->get_score() . '<br />';
    }

    public function header()
    {
        $html = array();

        $html[] = '<div class="question">';
        $html[] = '<div class="title">';
        $html[] = '<div class="number">';
        $html[] = '<div class="bevel">';
        $html[] = $this->question_nr . '.';
        $html[] = '</div>';
        $html[] = '</div>';
        $html[] = '<div class="text">';

        $html[] = '<div class="bevel" style="float: left;">';
        $html[] = $this->question->get_title();
        $html[] = '</div>';
        $html[] = '<div class="bevel" style="text-align: right;">';

        if ($this->hints > 0)
        {
            $variable = $this->hints == 1 ? 'HintUsed' : 'HintsUsed';
            $label = Translation :: get($variable, array('COUNT' => $this->hints));

            $html[] = '<img style="float: none; vertical-align: baseline;" src="' .
                 Theme :: get_image_path(__NAMESPACE__) . 'buttons/button_hint.png" alt="' . $label . '" title="' .
                 $label . '" />&nbsp;&nbsp;';
        }

        if ($this->get_assessment_result_processor()->get_assessment_viewer()->display_numeric_feedback())
        {
            $html[] = $this->get_score() . ' / ' . $this->get_complex_content_object_question()->get_weight();
        }

        $html[] = '</div>';
        $html[] = '<div class="clear"></div>';
        $html[] = '</div>';
        $html[] = '<div class="clear"></div>';
        $html[] = '</div>';
        $html[] = '<div class="answer">';

        $description = $this->question->get_description();
        $has_description = StringUtilities :: has_value($description, true);
        if ($has_description)
        {
            $html[] = '<div class="description">';

            $renderer = new ContentObjectResourceRenderer($this, $description);
            $html[] = $renderer->run();

            $html[] = '<div class="clear"></div>';
            $html[] = '</div>';
        }

        $html[] = '<div class="clear"></div>';

        return implode("\n", $html);
    }

    public function footer()
    {
        $html[] = '</div>';
        $html[] = '</div>';

        return implode("\n", $html);
    }

    public function add_borders()
    {
        return false;
    }

    public static function factory(AssessmentResultProcessor $assessment_result_processor,
        $complex_content_object_question, $question_nr, $answers, $score, $hints)
    {
        $type = $complex_content_object_question->get_ref_object()->get_type();

        $class = __NAMESPACE__ . '\\' . Utilities :: get_classname_from_namespace($type) . 'ResultDisplay';
        $question_result_display = new $class(
            $assessment_result_processor,
            $complex_content_object_question,
            $question_nr,
            $answers,
            $score,
            $hints);
        return $question_result_display;
    }
}
