<?php
namespace repository\content_object\assessment_match_numeric_question\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\EqualityCondition;
use common\libraries\SubselectCondition;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(
            \repository\content_object\assessment_match_numeric_question\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();

        $success = $data_manager->transactional(
            function ($c) use($data_manager)
            {
                $package = Package :: get('repository\content_object\assessment_match_numeric_question');
                $registration = CommonDataManager :: get_registration(
                    'repository\content_object\assessment_match_numeric_question');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                $attributes = array('type' => 'text', 'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion :: get_table_name(),
                    \repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion :: PROPERTY_OPTIONS,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text');
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion :: get_table_name(),
                    \repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion :: PROPERTY_HINT,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'text', 'length' => 10, 'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion :: get_table_name(),
                    \repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion :: PROPERTY_TOLERANCE_TYPE,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $attributes = array('type' => 'boolean', 'notnull' => 1, 'default' => 0);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                    \repository\content_object\assessment_match_numeric_question\ComplexAssessmentMatchNumericQuestion :: get_table_name(),
                    \repository\content_object\assessment_match_numeric_question\ComplexAssessmentMatchNumericQuestion :: PROPERTY_FEEDBACK_ANSWER,
                    $attributes);

                if (! $success)
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable(
                            'repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion')));

                $condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable('assessment_match_numeric_question'));

                if (! \repository\DataManager :: updates(
                    \repository\ContentObject :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            \repository\ComplexContentObjectItem :: class_name(),
                            \repository\ComplexContentObjectItem :: PROPERTY_TYPE),
                        new StaticConditionVariable(
                            'repository\content_object\assessment_match_numeric_question\ComplexAssessmentMatchNumericQuestion')));

                $subselect_condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable(
                        'repository\content_object\assessment_match_numeric_question\AssessmentMatchNumericQuestion'));

                $condition = new SubselectCondition(
                    new PropertyConditionVariable(
                        \repository\ComplexContentObjectItem :: class_name(),
                        \repository\ComplexContentObjectItem :: PROPERTY_REF),
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_ID),
                    null,
                    $subselect_condition);

                if (! \repository\DataManager :: updates(
                    \repository\ComplexContentObjectItem :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                return true;
            });

        return $success;
    }
}
