<?php
namespace repository\content_object\assessment_select_question;

use common\libraries\Translation;
use repository\ComplexContentObjectItemForm;

/**
 * $Id: complex_assessment_select_question_form.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.content_object.select_question
 */

/**
 * This class represents a form to create or update complex assessments
 */
class ComplexAssessmentSelectQuestionForm extends ComplexContentObjectItemForm
{
    const PROPERTY_RECALCULATE_WEIGHT = 'recalculate_weight';

    public function get_elements()
    {
        $elements[] = $this->createElement(
            'checkbox',
            self :: PROPERTY_RECALCULATE_WEIGHT,
            Translation :: get('RecalculateWeight'));

        $elements[] = $this->createElement(
            'text',
            ComplexAssessmentSelectQuestion :: PROPERTY_WEIGHT,
            Translation :: get('Weight'),
            array("size" => "50", 'disabled' => 'disabled'));

        $elements[] = $this->createElement(
            'checkbox',
            ComplexAssessmentSelectQuestion :: PROPERTY_RANDOM,
            Translation :: get('RandomizeOptions'));

        $elements[] = $this->createElement(
            'radio',
            ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_OPTIONS,
            Translation :: get('FeedbackOptions'),
            Translation :: get('FeedbackOptionsAll'),
            ComplexAssessmentSelectQuestion :: FEEDBACK_OPTIONS_ALL);

        $elements[] = $this->createElement(
            'radio',
            ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_OPTIONS,
            '',
            Translation :: get('FeedbackOptionsSelected'),
            ComplexAssessmentSelectQuestion :: FEEDBACK_OPTIONS_SELECTED);

        $elements[] = $this->createElement(
            'checkbox',
            ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_ANSWER,
            Translation :: get('FeedbackAnswer'));

        return $elements;
    }

    public function get_default_values()
    {
        $cloi = $this->get_complex_content_object_item();

        if (isset($cloi))
        {
            $defaults[ComplexAssessmentSelectQuestion :: PROPERTY_WEIGHT] = $cloi->get_weight() ? $cloi->get_weight() : 0;
            $defaults[self :: PROPERTY_RECALCULATE_WEIGHT] = 1;
            $defaults[ComplexAssessmentSelectQuestion :: PROPERTY_RANDOM] = $cloi->get_random() ? $cloi->get_random() : 0;
            $defaults[ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_OPTIONS] = $cloi->get_feedback_options();
            $defaults[ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_ANSWER] = $cloi->get_feedback_answer() ? $cloi->get_feedback_answer() : 0;
        }

        return $defaults;
    }

    // Inherited
    public function create_complex_content_object_item()
    {
        $cloi = $this->get_complex_content_object_item();
        $values = $this->exportValues();
        $cloi->set_weight($values[ComplexAssessmentSelectQuestion :: PROPERTY_WEIGHT]);
        $cloi->set_random($values[ComplexAssessmentSelectQuestion :: PROPERTY_RANDOM]);
        $cloi->set_feedback_options($values[ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_OPTIONS]);
        $cloi->set_feedback_answer($values[ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_ANSWER]);

        return parent :: create_complex_content_object_item();
    }

    public function create_cloi_from_values($values)
    {
        $cloi = $this->get_complex_content_object_item();
        $cloi->set_weight($values[ComplexAssessmentSelectQuestion :: PROPERTY_WEIGHT]);
        $cloi->set_random($values[ComplexAssessmentSelectQuestion :: PROPERTY_RANDOM]);
        $cloi->set_feedback_options($values[ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_OPTIONS]);
        $cloi->set_feedback_answer($values[ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_ANSWER]);

        return parent :: create_complex_content_object_item();
    }

    public function update_cloi_from_values($values)
    {
        $cloi = $this->get_complex_content_object_item();

        if ($values[self :: PROPERTY_RECALCULATE_WEIGHT] == 1)
        {
            $cloi->set_weight($cloi->get_ref_object()->get_default_weight());
        }
        else
        {
            $cloi->set_weight($values[ComplexAssessmentSelectQuestion :: PROPERTY_WEIGHT]);
        }

        $cloi->set_random($values[ComplexAssessmentSelectQuestion :: PROPERTY_RANDOM]);
        $cloi->set_feedback_options($values[ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_OPTIONS]);
        $cloi->set_feedback_answer($values[ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_ANSWER]);

        return parent :: update_complex_content_object_item();
    }

    // Inherited
    public function update_complex_content_object_item()
    {
        $cloi = $this->get_complex_content_object_item();
        $values = $this->exportValues();

        if ($values[self :: PROPERTY_RECALCULATE_WEIGHT] == 1)
        {
            $cloi->set_weight($cloi->get_ref_object()->get_default_weight());
        }
        else
        {
            $cloi->set_weight($values[ComplexAssessmentSelectQuestion :: PROPERTY_WEIGHT]);
        }

        $cloi->set_random($values[ComplexAssessmentSelectQuestion :: PROPERTY_RANDOM]);
        $cloi->set_feedback_options($values[ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_OPTIONS]);
        $cloi->set_feedback_answer($values[ComplexAssessmentSelectQuestion :: PROPERTY_FEEDBACK_ANSWER]);

        return parent :: update_complex_content_object_item();
    }

    protected function build_creation_form()
    {
        parent :: build_creation_form();
        $elements = $this->get_elements();
        foreach ($elements as $element)
        {
            $this->addElement($element);
        }
    }

    // Inherited
    protected function build_editing_form()
    {
        parent :: build_editing_form();
        $elements = $this->get_elements();
        foreach ($elements as $element)
        {
            $this->addElement($element);
        }
    }

    public function setDefaults($defaults = array ())
    {
        $defaults = array_merge($defaults, $this->get_default_values());
        parent :: setDefaults($defaults);
    }
}
