<?php
namespace repository\content_object\forum;

use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\DelegateComponent;
use common\libraries\Display;
use common\libraries\Translation;
use common\libraries\Utilities;


use common\libraries\EqualityCondition;
use repository\ComplexDisplay;
use repository\ComplexContentObjectItem;

/**
 * $Id: forum_subforum_creator.class.php 205 2009-11-13 12:57:33Z vanpouckesven $
 * 
 * @package repository.lib.complex_display.forum.component
 */
class ForumComplexDisplayForumSubforumCreatorComponent extends ForumComplexDisplay implements 
        \repository\viewer\ViewerInterface, 
        DelegateComponent
{

    public function run()
    {
        if ($this->get_parent()->is_allowed(ADD_RIGHT))
        {
            
            if (! \repository\viewer\Manager :: is_ready_to_be_published())
            {
                
                $exclude = $this->retrieve_used_items($this->get_root_content_object()->get_id());
                
                $exclude[] = $this->get_root_content_object()->get_id();
                
                BreadcrumbTrail :: get_instance()->add(
                        new Breadcrumb(
                                $this->get_url(
                                        array(
                                                ComplexDisplay :: PARAM_DISPLAY_ACTION => ForumComplexDisplay :: ACTION_VIEW_FORUM, 
                                                ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => null)), 
                                $this->get_root_content_object()->get_title()));
                
                if ($this->get_complex_content_object_item())
                {
                    
                    $forums_with_key_cloi = array();
                    $forums_with_key_cloi = $this->retrieve_children_from_root_to_cloi(
                            $this->get_root_content_object()->get_id(), 
                            $this->get_complex_content_object_item()->get_id());
                    
                    if ($forums_with_key_cloi)
                    {
                        
                        foreach ($forums_with_key_cloi as $key => $value)
                        {
                            
                            BreadcrumbTrail :: get_instance()->add(
                                    new Breadcrumb(
                                            $this->get_url(
                                                    array(
                                                            ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $key)), 
                                            $value->get_title()));
                        }
                    }
                    else
                    {
                        throw new \Exception('The forum you requested has not been found');
                    }
                }
                
                $repository_viewer = \repository\viewer\Manager :: construct($this);
                $repository_viewer->set_maximum_select(\repository\viewer\Manager :: SELECT_SINGLE);
                $repository_viewer->set_parameter(ComplexDisplay :: PARAM_DISPLAY_ACTION, 
                        ForumComplexDisplay :: ACTION_CREATE_SUBFORUM);
                $repository_viewer->set_parameter(ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID, 
                        $this->get_complex_content_object_item_id());
                $repository_viewer->set_excluded_objects($exclude);
                $repository_viewer->run();
            }
            else
            {
                $cloi = ComplexContentObjectItem :: factory(Forum :: class_name());
                
                if ($this->get_complex_content_object_item())
                {
                    $cloi->set_parent($this->get_complex_content_object_item()->get_ref());
                }
                else
                {
                    $cloi->set_parent($this->get_root_content_object_id());
                }
                
                $cloi->set_ref(\repository\viewer\Manager :: get_selected_objects());
                $cloi->set_user_id($this->get_user_id());
                $cloi->set_display_order(\repository\DataManager :: select_next_display_order($cloi->get_parent()));
                
                $success = $cloi->create();
                
                $this->my_redirect($success);
            }
        }
        else
        {
            Display :: not_allowed();
        }
    }

    private function retrieve_used_items($object)
    {
        $items = array();
        $items = array_merge($items, $this->retrieve_used_items_parents($object));
        $complex_content_object_items = \repository\DataManager :: retrieve_complex_content_object_items(
                ComplexContentObjectItem :: class_name(), 
                new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $object, 
                        ComplexContentObjectItem :: get_table_name()));
        while ($complex_content_object_item = $complex_content_object_items->next_result())
        {
            
            if ($complex_content_object_item->is_complex())
            {
                
                $items[] = $complex_content_object_item->get_ref();
                $items = array_merge($items, $this->retrieve_used_items($complex_content_object_item->get_ref()));
            }
        }
        return $items;
    }

    private function retrieve_used_items_parents($object_id)
    {
        $items = array();
        $items[] = $object_id;
        $complex_content_object_items_parent = \repository\DataManager :: retrieve_complex_content_object_items(
                ComplexContentObjectItem :: class_name(), 
                new EqualityCondition(ComplexContentObjectItem :: PROPERTY_REF, $object_id));
        while ($complex_content_object_item_parent = $complex_content_object_items_parent->next_result())
        {
            if ($complex_content_object_item_parent->is_complex())
            {
                
                $items = array_merge($items, 
                        $this->retrieve_used_items_parents($complex_content_object_item_parent->get_parent()));
            }
        }
        return $items;
    }

    private function my_redirect($success)
    {
        $message = htmlentities(
                Translation :: get(($success ? 'ObjectCreated' : 'ObjectNotCreated'), 
                        array('OBJECT' => Translation :: get('Subforum')), Utilities :: COMMON_LIBRARIES));
        
        $params = array();
        $params[ComplexDisplay :: PARAM_DISPLAY_ACTION] = ForumComplexDisplay :: ACTION_VIEW_FORUM;
        $params[ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID] = $this->get_complex_content_object_item_id();
        
        $this->redirect($message, ($success ? false : true), $params);
    }

    public function get_allowed_content_object_types()
    {
        return array(Forum :: class_name());
    }
}
