<?php
namespace repository\content_object\learning_path;

use common\libraries\Path;
use common\libraries\Request;
use common\libraries\Application;
use common\libraries\Translation;
use repository\ComplexDisplayPreview;
use repository\ComplexDisplay;
use repository\RepositoryManager;
use repository\content_object\assessment\AssessmentComplexDisplaySupport;
use repository\content_object\learning_path\LearningPathComplexDisplaySupport;
use repository\content_object\forum\ForumComplexDisplaySupport;
use repository\content_object\wiki\WikiComplexDisplaySupport;
use repository\content_object\blog\BlogComplexDisplaySupport;
use repository\content_object\glossary\GlossaryComplexDisplaySupport;
use common\libraries\Session;

class LearningPathComplexDisplayPreview extends ComplexDisplayPreview implements 
        LearningPathComplexDisplaySupport, 
        AssessmentComplexDisplaySupport, 
        ForumComplexDisplaySupport, 
        GlossaryComplexDisplaySupport, 
        BlogComplexDisplaySupport, 
        WikiComplexDisplaySupport
{
    const TEMPORARY_STORAGE = 'learning_path_assessment_preview';

    function run()
    {
        ComplexDisplay :: launch($this->get_root_content_object()->get_type(), $this);
    }

    function get_root_content_object()
    {
        $embedded_content_object_id = LearningPathContentObjectDisplay :: get_embedded_content_object_id();

        if ($embedded_content_object_id)
        {
     $this->set_parameter(
                LearningPathContentObjectDisplay :: PARAM_EMBEDDED_CONTENT_OBJECT_ID,
                $embedded_content_object_id);
            return \repository\DataManager :: retrieve_content_object($embedded_content_object_id);
        }
        else
        {
            $this->set_parameter(
                LearningPathComplexDisplay :: PARAM_LEARNING_PATH_ITEM_ID,
                Request :: get(LearningPathComplexDisplay :: PARAM_LEARNING_PATH_ITEM_ID));
            $this->set_parameter(
                ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID,
                Request :: get(ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID));
            return parent :: get_root_content_object();
        }
    }

    /**
     * Preview mode, so always return true.
     *
     * @param $right
     * @return boolean
     */
    function is_allowed($right)
    {
        return true;
    }

    /**
     * Since this is a preview, no actual view event is triggered.
     *
     * @param $complex_topic_id
     */
    function forum_topic_viewed($complex_topic_id)
    {
    }

    /**
     * Since this is a preview, no views are logged and no count can be retrieved.
     *
     * @param $complex_topic_id
     * @return string
     */
    function forum_count_topic_views($complex_topic_id)
    {
        return '-';
    }

    /**
     * Returns whether or not the logged in user is a forum manager
     *
     * @param $user User
     *
     * @return boolean
     */
    function is_forum_manager($user)
    {
        return false;
    }

    /**
     * Functionality is publication dependent, so not available in preview mode.
     */
    function get_wiki_page_statistics_reporting_template_name()
    {
        $this->not_available(Translation :: get('ImpossibleInPreviewMode'));
    }

    /**
     * Functionality is publication dependent, so not available in preview mode.
     */
    function get_wiki_statistics_reporting_template_name()
    {
        $this->not_available(Translation :: get('ImpossibleInPreviewMode'));
    }

    /**
     * Functionality is publication dependent, so not available in preview mode.
     */
    function get_publication()
    {
        $this->not_available(Translation :: get('ImpossibleInPreviewMode'));
    }

    /**
     * Preview mode, so no actual saving done.
     *
     * @param $complex_question_id int
     * @param $answer mixed
     * @param $score int
     */
    function save_assessment_answer($complex_question_id, $answer, $score, $hint)
    {
    }

    /**
     * Preview mode, so no actual total score will be saved.
     *
     * @param $total_score int
     */
    function save_assessment_result($total_score)
    {
    }

    /**
     * Preview mode, so there is no acrual attempt.
     */
    function get_assessment_current_attempt_id()
    {
    }

    function get_assessment_question_attempts()
    {
        $answers = Session :: retrieve(self :: TEMPORARY_STORAGE);
        return $answers[$this->get_root_content_object()->get_id()];
    }

    function get_assessment_question_attempt($complex_question_id)
    {
        $answers = $this->get_assessment_question_attempts($complex_question_id);
        return $answers[$complex_question_id];
    }

    /**
     * Preview mode is launched in standalone mode, so there's nothing to go back to.
     *
     * @return void
     */
    function get_assessment_back_url()
    {
    }

    /**
     * Preview mode is launched in standalone mode, so there's nothing to continue to.
     *
     * @return void
     */
    function get_assessment_continue_url()
    {
    }

    function get_assessment_current_url()
    {
    }

    function get_assessment_feedback_configuration($complex_content_object_item)
    {
        return $complex_content_object_item->get_ref_object()->get_feedback_configuration();
    }

    function get_assessment_parameters()
    {
        return array();
    }

    function retrieve_learning_path_tracker()
    {
        return new DummyLpAttemptTracker();
    }

    function retrieve_learning_path_tracker_items($learning_path_tracker)
    {
    }

    function get_learning_path_tree_menu_url()
    {
        return Path :: get(WEB_PATH) . 'index.php?' . Application :: PARAM_APPLICATION . '=' .
             RepositoryManager :: APPLICATION_NAME . '&' . Application :: PARAM_ACTION . '=' .
             RepositoryManager :: ACTION_PREVIEW . '&' . RepositoryManager :: PARAM_CONTENT_OBJECT_ID . '=' .
             $this->get_root_content_object()->get_id() . '&' . LearningPathComplexDisplay :: PARAM_STEP . '=%s';
    }

    /**
     *
     * @param $total_steps int
     */
    function get_learning_path_previous_url($total_steps)
    {
        return $this->get_url(array('step' => $total_steps));
    }

    /**
     * Creates a learning path item tracker
     *
     * @param $learning_path_tracker LearningPathAttemptTracker
     * @param $current_complex_content_object_item ComplexContentObjectItem
     * @return array LearningPathItemAttemptTracker
     */
    function create_learning_path_item_tracker($learning_path_tracker, $current_complex_content_object_item)
    {
        $item_tracker = new DummyLpiAttemptTracker();
        $item_tracker->set_lp_item_id($learning_path_tracker->get_id());
        return $item_tracker;
    }

    /**
     *
     * @param $complex_content_object_id int
     */
    function get_learning_path_content_object_item_details_url($complex_content_object_id)
    {
    }

    /**
     * Get the url of the assessment result
     *
     * @param $complex_content_object_id int
     * @param $details unknown_type
     */
    function get_learning_path_content_object_assessment_result_url($complex_content_object_id, $details)
    {
    }

    /**
     *
     * @return array
     */
    function get_learning_path_attempt_progress_details_reporting_template_name()
    {
        $this->not_available(Translation :: get('ImpossibleInPreviewMode'));
    }

    /**
     *
     * @return array
     */
    function get_learning_path_attempt_progress_reporting_template_name()
    {
        $this->not_available(Translation :: get('ImpossibleInPreviewMode'));
    }

    function get_learning_path_template_application_name()
    {
        $this->not_available(Translation :: get('ImpossibleInPreviewMode'));
    }

    // FUNCTIONS FOR COMPLEX DISPLAY SUPPORT
    public function is_allowed_to_edit_content_object()
    {
        return true;
    }

    public function is_allowed_to_view_content_object()
    {
        return true;
    }

    function is_allowed_to_add_child()
    {
        return true;
    }

    function is_allowed_to_delete_child()
    {
        return true;
    }

    function is_allowed_to_delete_feedback()
    {
        return true;
    }

    function is_allowed_to_edit_feedback()
    {
        return true;
    }
    /*
     * (non-PHPdoc) @see
     * \repository\content_object\assessment\AssessmentComplexDisplaySupport::get_registered_question_ids()
     */
    public function get_registered_question_ids()
    {
        // TODO Auto-generated method stub
    }

    /*
     * (non-PHPdoc) @see \repository\content_object\assessment\AssessmentComplexDisplaySupport::register_question_ids()
     */
    public function register_question_ids($question_ids)
    {
        // TODO Auto-generated method stub
    }
}
?>