<?php
namespace repository\content_object\match_question\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\PropertiesConditionVariable;
use common\libraries\package\Package;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\EqualityCondition;
use common\libraries\RecordRetrievesParameters;
use repository\content_object\assessment_match_text_question\AssessmentMatchTextQuestion;
use repository\content_object\assessment_match_text_question\ComplexAssessmentMatchTextQuestion;
use repository\content_object\match_question\ComplexMatchQuestion;
use repository\content_object\match_question\MatchQuestion;
use repository\content_object\assessment_match_text_question\AssessmentMatchTextQuestionOption;
use common\libraries\DataClassCache;
use core\lynx\manager\PackageRemover;
use common\libraries\SubselectCondition;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\match_question\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = \repository\DataManager :: get_instance();

        $success = $data_manager->transactional(
            function ($c) use ($data_manager)
            {
                try
                {
                    $package = Package :: get('repository\content_object\match_question');
                    $registration = CommonDataManager :: get_registration('repository\content_object\match_question');
                    $registration->set_version($package->get_version());
                    if (! $registration->update())
                    {
                        return false;
                    }

                    $attributes = array('type' => 'text', 'notnull' => 1);

                    $success = $data_manager->alter_storage_unit(
                        \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                        \repository\content_object\match_question\MatchQuestion :: get_table_name(),
                        \repository\content_object\match_question\MatchQuestion :: PROPERTY_OPTIONS,
                        $attributes
                    );

                    if (! $success)
                    {
                        return false;
                    }

                    $attributes = array('type' => 'text', 'notnull' => 1);
                    $success = $data_manager->alter_storage_unit(
                        \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                        \repository\content_object\match_question\MatchQuestion :: get_table_name(),
                        \repository\content_object\match_question\MatchQuestion :: PROPERTY_HINT,
                        $attributes
                    );

                    if (! $success)
                    {
                        return false;
                    }

                    $attributes = array('type' => 'text', 'length' => 8, 'notnull' => 1, 'fixed' => 1);
                    $success = $data_manager->alter_storage_unit(
                        \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                        \repository\content_object\match_question\MatchQuestion :: get_table_name(),
                        \repository\content_object\match_question\MatchQuestion :: PROPERTY_ANSWER_TYPE,
                        $attributes
                    );

                    if (! $success)
                    {
                        return false;
                    }

                    $attributes = array('type' => 'boolean', 'notnull' => 1, 'default' => 0);
                    $success = $data_manager->alter_storage_unit(
                        \common\libraries\DataManager :: ALTER_STORAGE_UNIT_ADD,
                        \repository\content_object\match_question\ComplexMatchQuestion :: get_table_name(),
                        \repository\content_object\match_question\ComplexMatchQuestion :: PROPERTY_FEEDBACK_ANSWER,
                        $attributes
                    );

                    if (! $success)
                    {
                        return false;
                    }

                    $properties = new DataClassProperties();
                    $properties->add(
                        new DataClassProperty(
                            new PropertyConditionVariable(
                                \repository\ContentObject :: class_name(),
                                \repository\ContentObject :: PROPERTY_TYPE),
                            new StaticConditionVariable('repository\content_object\match_question\MatchQuestion'))
                    );

                    $condition = new EqualityCondition(
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable('match_question'));

                    if (! \repository\DataManager :: updates(
                        \repository\ContentObject :: class_name(),
                        $properties,
                        $condition
                    )
                    )
                    {
                        return false;
                    }

                    $match_questions = \repository\DataManager :: records(
                        \repository\content_object\match_question\MatchQuestion :: class_name(),
                        new RecordRetrievesParameters()
                    );

                    while ($match_question = $match_questions->next_result())
                    {
                        $match_text_question = array();
                        $match_text_question[AssessmentMatchTextQuestion :: PROPERTY_ID] =
                            $match_question[MatchQuestion :: PROPERTY_ID];
                        $match_text_question[AssessmentMatchTextQuestion :: PROPERTY_HINT] =
                            $match_question[MatchQuestion :: PROPERTY_HINT];
                        $match_text_question[AssessmentMatchTextQuestion :: PROPERTY_USE_WILDCARDS] = 0;

                        $options = array();
                        foreach (unserialize($match_question[MatchQuestion :: PROPERTY_OPTIONS]) as $key => $option)
                        {
                            $options[$key] = new AssessmentMatchTextQuestionOption(
                                $option->get_value(),
                                $option->get_weight(),
                                $option->get_comment());
                        }

                        $match_text_question[AssessmentMatchTextQuestion :: PROPERTY_OPTIONS] = serialize($options);

                        if (! \repository\DataManager :: create_record(
                            AssessmentMatchTextQuestion :: class_name(),
                            $match_text_question
                        )
                        )
                        {
                            return false;
                        }

                        $properties = new DataClassProperties();
                        $properties->add(
                            new DataClassProperty(
                                new PropertyConditionVariable(
                                    \repository\ContentObject :: class_name(),
                                    \repository\ContentObject :: PROPERTY_TYPE),
                                new StaticConditionVariable(
                                    'repository\content_object\assessment_match_text_question\AssessmentMatchTextQuestion'))
                        );

                        $condition = new EqualityCondition(
                            new PropertyConditionVariable(
                                \repository\ContentObject :: class_name(),
                                \repository\ContentObject :: PROPERTY_ID),
                            new StaticConditionVariable($match_text_question[AssessmentMatchTextQuestion :: PROPERTY_ID]));

                        if (! \repository\DataManager :: updates(
                            \repository\ContentObject :: class_name(),
                            $properties,
                            $condition
                        )
                        )
                        {
                            return false;
                        }

                        $condition = new EqualityCondition(
                            new PropertyConditionVariable(
                                \repository\content_object\match_question\MatchQuestion :: class_name(),
                                \repository\content_object\match_question\MatchQuestion :: PROPERTY_ID),
                            new StaticConditionVariable($match_text_question[AssessmentMatchTextQuestion :: PROPERTY_ID]));

                        if (! \repository\DataManager :: deletes(
                            \repository\content_object\match_question\MatchQuestion :: class_name(),
                            $condition
                        )
                        )
                        {
                            return false;
                        }
                    }

                    $retrieve_properties = new DataClassProperties();
                    $retrieve_properties->add(new PropertiesConditionVariable(ComplexMatchQuestion :: class_name()));

                    $complex_match_questions = \repository\DataManager :: records(
                        ComplexMatchQuestion :: class_name(), new RecordRetrievesParameters($retrieve_properties)
                    );

                    while ($complex_match_question = $complex_match_questions->next_result())
                    {
                        $complex_match_text_question = array();

                        $complex_match_text_question[ComplexAssessmentMatchTextQuestion::PROPERTY_ID] =
                            $complex_match_question[ComplexMatchQuestion::PROPERTY_ID];

                        $complex_match_text_question[ComplexAssessmentMatchTextQuestion::PROPERTY_WEIGHT] =
                            $complex_match_question[ComplexMatchQuestion::PROPERTY_WEIGHT];

                        $complex_match_text_question[ComplexAssessmentMatchTextQuestion::PROPERTY_FEEDBACK_ANSWER] = 0;

                        if (! \repository\DataManager :: create_record(
                            ComplexAssessmentMatchTextQuestion :: class_name(),
                            $complex_match_text_question
                        )
                        )
                        {
                            return false;
                        }
                    }

                    $properties = new DataClassProperties();
                    $properties->add(
                        new DataClassProperty(
                            new PropertyConditionVariable(
                                \repository\ComplexContentObjectItem :: class_name(),
                                \repository\ComplexContentObjectItem :: PROPERTY_TYPE),
                            new StaticConditionVariable(
                                'repository\content_object\assessment_match_text_question\ComplexAssessmentMatchTextQuestion'
                            )
                        )
                    );

                    $subselect_condition = new EqualityCondition(
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable(
                            'repository\content_object\assessment_match_text_question\AssessmentMatchTextQuestion'));

                    $condition = new SubselectCondition(
                        new PropertyConditionVariable(
                            \repository\ComplexContentObjectItem :: class_name(),
                            \repository\ComplexContentObjectItem :: PROPERTY_REF),
                        new PropertyConditionVariable(
                            \repository\ContentObject :: class_name(),
                            \repository\ContentObject :: PROPERTY_ID),
                        null,
                        $subselect_condition);

                    if (! \repository\DataManager :: updates(
                        \repository\ComplexContentObjectItem :: class_name(),
                        $properties,
                        $condition
                    )
                    )
                    {
                        return false;
                    }

                    return true;
                }
                catch(\Exception $ex)
                {
                    var_dump($ex);
                }
            }
        );

        DataClassCache :: reset();

        $package_remover = new PackageRemover('repository\content_object\match_question');
        $success = $package_remover->run();

        return $success;
    }
}
