<?php

namespace repository\content_object\personal_message\upgrade\v4_0_0;

use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use common\libraries\Translation;
use common\libraries\Utilities;
use repository\ContentObject;
use repository\content_object\personal_message\PersonalMessage;

/**
 * Database schema & content upgrader for Chamilo 3.x to 4.0.0.
 *
 * @author Sven Vanpoucke - Hogeschool Gent
 */
class Upgrader extends \common\libraries\package\upgrade\v4_0_0\Upgrader
{
    protected static $CONTENT_OBJECT_CLASS = PersonalMessage :: CLASS_NAME; // Change this value!

    protected function get_namespace()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS . '::context');
    }

    protected function get_classname()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS . '::class_name');
    }

    protected function get_type_name()
    {
        return call_user_func(self :: $CONTENT_OBJECT_CLASS . '::get_type_name');
    }

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\personal_message\DataManager :: get_instance());
    }

    /**
     * Upgrades the packages
     *
     * @param $data_manager
     *
     * @return bool
     */
    function upgrade($data_manager)
    {
        $properties = new DataClassProperties();
        $properties->add(
            new DataClassProperty(
                new PropertyConditionVariable(
                    ContentObject :: class_name(),
                    ContentObject :: PROPERTY_TYPE),
                new StaticConditionVariable($this->get_classname()))
        );

        $condition = new EqualityCondition(
            new PropertyConditionVariable(
                ContentObject :: class_name(),
                ContentObject :: PROPERTY_TYPE),
            new StaticConditionVariable($this->get_type_name())
        );

        if (! \repository\DataManager :: updates(
            ContentObject :: class_name(),
            $properties,
            $condition
        )
        )
        {
            $this->add_message(
                \common\libraries\package\Upgrader :: TYPE_ERROR,
                Translation :: get('UpdateDatabaseContentFailed', null, Utilities :: COMMON_LIBRARIES)
            );

            return false;
        }

        $this->add_message(
            \common\libraries\package\Upgrader :: TYPE_NORMAL,
            Translation :: get('UpdateDatabaseContentSuccessful', null, Utilities :: COMMON_LIBRARIES)
        );

        return true;
    }
}