<?php
namespace repository\content_object\scorm_item\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\package\Package;
use repository\content_object\scorm_item\ScormItem;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\EqualityCondition;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use repository\ContentObject;
use common\libraries\SubselectCondition;

/**
 * Database schema & content upgrader for Chamilo 3.x to 4.0.0.
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\scorm_item\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {

                $scorm_item_column_attributes = array(
                    ScormItem :: PROPERTY_PARAMETERS => array(
                        'name' => ScormItem :: PROPERTY_PARAMETERS,
                        'type' => 'text',
                        'length' => '1000',),
                    ScormItem :: PROPERTY_TIME_LIMIT_ACTION => array(
                        'name' => ScormItem :: PROPERTY_TIME_LIMIT_ACTION,
                        'type' => 'text',
                        'length' => '500'),
                    ScormItem :: PROPERTY_DATA_FROM_LMS => array(
                        'name' => ScormItem :: PROPERTY_DATA_FROM_LMS,
                        'type' => 'text',
                        'length' => '4000'),
                    ScormItem :: PROPERTY_COMPLETION_TRESHOLD => array(
                        'name' => ScormItem :: PROPERTY_COMPLETION_TRESHOLD,
                        'type' => 'decimal'),
                    ScormItem :: PROPERTY_HIDE_LMS_UI => array(
                        'name' => ScormItem :: PROPERTY_HIDE_LMS_UI,
                        'type' => 'text',
                        'length' => '255',
                        'default' => ''),
                    ScormItem :: PROPERTY_CONTROL_MODE => array(
                        'name' => ScormItem :: PROPERTY_CONTROL_MODE,
                        'type' => 'text',
                        'length' => '255',
                        'default' => ''),
                    ScormItem :: PROPERTY_TIME_LIMIT => array(
                        'name' => ScormItem :: PROPERTY_TIME_LIMIT,
                        'type' => 'text',
                        'length' => '100',
                        'default' => ''),
                    ScormItem :: PROPERTY_OBJECTIVES => array(
                        'name' => ScormItem :: PROPERTY_OBJECTIVES,
                        'type' => 'text',),
                    ScormItem :: PROPERTY_CONDITION_RULES => array(
                        'name' => ScormItem :: PROPERTY_CONDITION_RULES,
                        'type' => 'text'),
                    ScormItem :: PROPERTY_IDENTIFIER => array(
                        'name' => ScormItem :: PROPERTY_IDENTIFIER,
                        'type' => 'text'),
                );

                foreach ($scorm_item_column_attributes as $column => $attributes)
                {
                    $success = $data_manager->alter_storage_unit(
                        \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                        ScormItem :: get_table_name(),
                        $column,
                        $attributes);

                    if (! $success)
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Action :: TYPE_ERROR,
                            "Failed to make changes to column '$column' of table " . ScormItem :: get_table_name());
                        return false;
                    }
                    else
                    {
                        $upgrader->add_message(
                            \common\libraries\package\Action :: TYPE_NORMAL,
                            "Made changes to column '$column' of table " . ScormItem :: get_table_name());
                    }
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            ContentObject :: class_name(),
                            ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable(ScormItem :: class_name()))
                );

                $condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        ContentObject :: class_name(),
                        ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable(ScormItem :: get_type_name())
                );

                if (! \repository\DataManager :: updates(
                    ContentObject :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            \repository\ComplexContentObjectItem :: class_name(),
                            \repository\ComplexContentObjectItem :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\scorm_item\ComplexScormItem')));

                $subselect_condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable('repository\content_object\scorm_item\ScormItem'));

                $condition = new SubselectCondition(
                    new PropertyConditionVariable(
                        \repository\ComplexContentObjectItem :: class_name(),
                        \repository\ComplexContentObjectItem :: PROPERTY_REF),
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_ID),
                    null,
                    $subselect_condition);

                if (! \repository\DataManager :: updates(
                    \repository\ComplexContentObjectItem :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }
                
                $package = Package :: get('repository\content_object\scorm_item');
                $registration = CommonDataManager :: get_registration('repository\content_object\scorm_item');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                return true;
            });

        return $success;
    }
}
