<?php
namespace repository\content_object\system_announcement;

use common\libraries\EqualityCondition;
use common\libraries\ResourceManager;
use common\libraries\Path;
use common\libraries\Session;
use common\libraries\Translation;

use repository\RepositoryDataManager;
use repository\ContentObjectAdvancedFilter;

class AdvancedFilter extends ContentObjectAdvancedFilter
{

    public function get_conditions()
    {
        $session_filter = Session :: retrieve('advanced_filter');
        $conditions = array();
        if ($this->get_form()->validate() || isset($session_filter))
        {
            if ($this->get_form()->validate())
            {
                $values = $this->get_form()->exportValues();
            }
            else
            {
                $values = unserialize(Session :: retrieve('advanced_filter'));
            }

            $filter = serialize($values);
            if ($this->get_form()->validate())
            {
                Session :: register('advanced_filter', $filter);
            }

            if (isset($values[SystemAnnouncement :: PROPERTY_ICON]) &&
                $values[SystemAnnouncement :: PROPERTY_ICON] != -1
            )
            {
                $conditions[] = new EqualityCondition(
                    SystemAnnouncement :: PROPERTY_ICON, $values[SystemAnnouncement :: PROPERTY_ICON],
                    RepositoryDataManager ::get_alias_type_table(), true
                );
            }
        }

        return $conditions;
    }

    public function get_parameters()
    {
        $session_filter = Session :: retrieve('advanced_filter');
        $html = array();
        if ($this->get_form()->validate() || isset($session_filter))
        {
            if ($this->get_form()->validate())
            {
                $values = $this->get_form()->exportValues();
            }
            else
            {
                $values = unserialize(Session :: retrieve('advanced_filter'));
            }

            if (isset($values[SystemAnnouncement :: PROPERTY_ICON]) &&
                $values[SystemAnnouncement :: PROPERTY_ICON] != -1
            )
            {
                $translation = Translation :: get(
                    'IconSearchParameter', array(
                        'ICON' => SystemAnnouncement :: icon_name($values[SystemAnnouncement :: PROPERTY_ICON])
                    )
                );
                $html[] =
                    '<div class="parameter" id="parameter_system_announcment_' . SystemAnnouncement :: PROPERTY_ICON .
                        '">' . $translation . '</div>';
            }

        }

        return implode("\n", $html);
    }

    public function add_elements()
    {
        $form = $this->get_form();

        $form->addElement('category', Translation :: get('Icon'));
        $options = SystemAnnouncement :: get_possible_icons();
        $options[-1] = Translation :: get('SelectIcon');
        ksort($options);
        $form->addElement('select', SystemAnnouncement :: PROPERTY_ICON, Translation :: get('Icon'), $options);
        $form->addElement('category');

        $form->addElement(
            'html', ResourceManager :: get_instance()->get_resource_html(
                Path :: namespace_to_full_path(__NAMESPACE__, true) . 'resources/javascript/search.js'
            )
        );
    }
}
