<?php
namespace repository\content_object\wiki;

use common\libraries\Path;
use repository\ComplexContentObjectItem;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\ResourceManager;
use common\libraries\Application;
use common\libraries\ObjectTableFormActions;
use common\libraries\ObjectTableFormAction;
use common\libraries\NotAllowedException;
use repository\ComplexDisplay;
use repository\RepositoryVersionBrowserTable;
use repository\ContentObject;
use repository\RepositoryManager;
use repository\ContentObjectDifferenceDisplay;

/**
 * $Id: wiki_history.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.complex_display.wiki.component
 */
class WikiComplexDisplayWikiHistoryComponent extends WikiComplexDisplay
{

    private $complex_wiki_page_id;

    public function run()
    {
        if (! $this->is_allowed(VIEW_RIGHT))
        {
            throw new NotAllowedException();
            return;
        }

        $this->complex_wiki_page_id = Request :: get(ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID);

        if ($this->complex_wiki_page_id)
        {
            $complex_wiki_page = \repository\DataManager :: retrieve_complex_content_object_item(
                $this->complex_wiki_page_id,
                ComplexContentObjectItem :: class_name());
            $compare_object_ids = Request :: post(
                RepositoryVersionBrowserTable :: DEFAULT_NAME . RepositoryVersionBrowserTable :: CHECKBOX_NAME_SUFFIX);

            if ($compare_object_ids)
            {
                if (count($compare_object_ids) < 2)
                {
                    $this->redirect(Translation :: get('TooFewItems'), true);
                }
                $compare_object_id = $compare_object_ids[0];
                $compare_version_id = $compare_object_ids[1];

                $compare_object = \repository\DataManager :: retrieve_content_object($compare_object_id);

                $compare_difference = $compare_object->get_difference($compare_version_id);

                $compare_display = ContentObjectDifferenceDisplay :: factory($compare_difference);

                $html = array();
                $html[] = Utilities :: add_block_hider();
                $html[] = Utilities :: build_block_hider('compare_legend');
                $html[] = $compare_display->get_legend();
                $html[] = Utilities :: build_block_hider();
                $html[] = $compare_display->get_diff_as_html();
            }
            else
            {
                $wiki_page = $complex_wiki_page->get_ref_object();
                $version_parameters = $this->get_parameters();
                $version_parameters[ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID] = $this->complex_wiki_page_id;

                $version_browser = new RepositoryVersionBrowserTable(
                    $this,
                    $version_parameters,
                    new EqualityCondition(ContentObject :: PROPERTY_OBJECT_NUMBER, $wiki_page->get_object_number()));
                $actions = new ObjectTableFormActions(__NAMESPACE__, ComplexDisplay :: PARAM_DISPLAY_ACTION);
                $actions->add_form_action(
                    new ObjectTableFormAction(
                        WikiComplexDisplay :: ACTION_COMPARE,
                        Translation :: get('CompareSelected'),
                        false));
                $version_browser->set_form_actions($actions);

                $html = array();
                $html[] = '<div class="wiki-pane-content-title">' . Translation :: get('RevisionHistory') . ': ' .
                     $wiki_page->get_title() . '</div>';
                $html[] = '<div class="wiki-pane-content-subtitle">' .
                     Translation :: get('From', null, Utilities :: COMMON_LIBRARIES) . ' ' .
                     $this->get_root_content_object()->get_title() . '</div>';
                $html[] = '<div class="wiki-pane-content-history">';
                $html[] = $version_browser->as_html();
                $html[] = ResourceManager :: get_instance()->get_resource_html(
                    Path :: namespace_to_full_path(RepositoryManager :: context(), true) .
                         'resources/javascript/repository.js');
                $html[] = '<div class="clear"></div>';
                $html[] = '</div>';
            }

            $this->display_header($complex_wiki_page);
            echo implode("\n", $html);
            $this->display_footer();
        }
        else
        {
            $this->redirect(
                null,
                false,
                array(ComplexDisplay :: PARAM_DISPLAY_ACTION => WikiComplexDisplay :: ACTION_VIEW_WIKI));
        }
    }

    public function count_content_object_versions_resultset($condition = null)
    {
        return \repository\DataManager :: count_content_objects(ContentObject :: class_name(), $condition);
    }

    public function retrieve_content_object_versions_resultset($condition = null, $order_by = array(), $offset = 0,
        $max_objects = -1)
    {
        return \repository\DataManager :: retrieve_content_objects(ContentObject :: class_name(), $condition);
    }

    public function get_content_object_viewing_url($content_object)
    {
        return $this->get_url(
            array(
                ComplexDisplay :: PARAM_DISPLAY_ACTION => WikiComplexDisplay :: ACTION_VIEW_WIKI_PAGE,
                ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->complex_wiki_page_id,
                self :: PARAM_WIKI_VERSION_ID => $content_object->get_id()));
    }

    public function get_content_object_deletion_url($content_object, $type = null)
    {
        $delete_allowed = \repository\DataManager :: content_object_deletion_allowed($content_object, $type);

        if (! $delete_allowed)
        {
            return null;
        }

        return $this->get_url(
            array(
                ComplexDisplay :: PARAM_DISPLAY_ACTION => WikiComplexDisplay :: ACTION_VERSION_DELETE,
                ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->complex_wiki_page_id,
                self :: PARAM_WIKI_VERSION_ID => $content_object->get_id()));
    }

    public function get_content_object_revert_url($content_object)
    {
        $revert_allowed = \repository\DataManager :: content_object_revert_allowed($content_object);

        if (! $revert_allowed)
        {
            return null;
        }

        return $this->get_url(
            array(
                ComplexDisplay :: PARAM_DISPLAY_ACTION => WikiComplexDisplay :: ACTION_VERSION_REVERT,
                ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->complex_wiki_page_id,
                self :: PARAM_WIKI_VERSION_ID => $content_object->get_id()));
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail = $this->get_breadcrumbtrail();
    }
}
