<?php
namespace repository\content_object\wiki;

use common\libraries\Translation;
use common\libraries\Request;
use common\libraries\Utilities;
use common\libraries\Toolbar;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use repository\DefaultContentObjectTableCellRenderer;
use repository\ContentObject;
use repository\ComplexDisplay;

/**
 * $Id: wiki_page_table_cell_renderer.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.complex_display.wiki.component.wiki_page_table
 */

class WikiPageTableCellRenderer extends DefaultContentObjectTableCellRenderer
{
    private $table_actions;
    private $browser;
    private $datamanager;
    private $publication_id;
    private $complex_id;

    public function __construct($browser)
    {
        $this->table_actions = array();
        $this->browser = $browser;
    }

    /*
     * Inherited
     */
    public function render_cell($column, $publication)
    {
        if ($column === WikiPageTableColumnModel :: get_action_column())
        {
            return $this->get_actions($publication);
        }

        $this->publication_id = Request :: get('publication_id');

        $wiki_page = $this->get_publication_from_complex_content_object_item($publication);
        $this->complex_id = $publication->get_id();

        if ($publication->get_additional_property('is_homepage') == 1)
        {
            $homepage = ' (' . Translation :: get('Homepage') . ')';
        }

        if (isset($wiki_page))
        {
            if ($property = $column->get_name())
            {
                switch ($property)
                {
                    case ContentObject :: PROPERTY_TITLE :
                        return '<a href="' . $this->browser->get_url(
                            array(
                                WikiComplexDisplay :: PARAM_DISPLAY_ACTION =>
                                WikiComplexDisplay :: ACTION_VIEW_WIKI_PAGE,
                                ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->complex_id
                            )
                        ) . '">' . htmlspecialchars(
                            $wiki_page->get_title()
                        ) . '</a>' . $homepage;
                    case Translation :: get('Versions') :
                        return $wiki_page->get_version_count();
                    case ContentObject :: PROPERTY_DESCRIPTION :
                        $description = str_ireplace(
                            ']]', '',
                            str_ireplace('[[', '', str_ireplace('=', '', $wiki_page->get_description()))
                        );
                        return Utilities :: truncate_string($description, 50);
                }
            }

            return parent :: render_cell($column, $wiki_page);
        }
    }

    public function get_actions($publication)
    {
        $toolbar = New Toolbar();
        if ($this->browser->get_parent()->is_allowed(DELETE_RIGHT))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_delete.png',
                    $this->browser->get_url(
                        array(
                            WikiComplexDisplay :: PARAM_DISPLAY_ACTION =>
                            WikiComplexDisplay :: ACTION_DELETE_COMPLEX_CONTENT_OBJECT_ITEM,
                            ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $publication->get_id()
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON, true
                )
            );
        }

        if ($this->browser->get_parent()->is_allowed(EDIT_RIGHT))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_edit.png',
                    $this->browser->get_url(
                        array(
                            WikiComplexDisplay :: PARAM_DISPLAY_ACTION =>
                            WikiComplexDisplay :: ACTION_UPDATE_COMPLEX_CONTENT_OBJECT_ITEM,
                            ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $publication->get_id()
                        )
                    ),
                    ToolbarItem :: DISPLAY_ICON
                )
            );

            if (($publication->get_additional_property('is_homepage') == 0))
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('SetAsHomepage'),
                        Theme :: get_common_image_path() . 'action_home.png',
                        $this->browser->get_url(
                            array(
                                WikiComplexDisplay :: PARAM_DISPLAY_ACTION =>
                                WikiComplexDisplay :: ACTION_SET_AS_HOMEPAGE,
                                ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->complex_id
                            )
                        ),
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
            else
            {
                $toolbar->add_item(
                    new ToolbarItem(
                        Translation :: get('SetAsHomepage'),
                        Theme :: get_common_image_path() . 'action_home_na.png', null,
                        ToolbarItem :: DISPLAY_ICON
                    )
                );
            }
        }

        return $toolbar->as_html();
    }

    /*private function get_publish_links($wiki_page)
    {
    }*/

    private function get_publication_from_complex_content_object_item($clo_item)
    {
        $publication = \repository\DataManager :: retrieve_content_object($clo_item->get_ref());
        return $publication;
    }
}
