<?php
namespace repository\content_object\wiki_page\upgrade\v4_0_0;

use common\libraries\CommonDataManager;
use common\libraries\DataClassProperties;
use common\libraries\DataClassProperty;
use common\libraries\EqualityCondition;
use common\libraries\package\Action;
use common\libraries\package\Package;
use common\libraries\PropertyConditionVariable;
use common\libraries\StaticConditionVariable;
use repository\content_object\wiki_page\ComplexWikiPage;
use repository\content_object\wiki_page\WikiPage;
use repository\ContentObject;
use common\libraries\SubselectCondition;

/**
 * Database schema & content upgrader for Chamilo 3.x to 4.0.0.
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\repository\content_object\wiki_page\DataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {

                $complex_wiki_page_attributes = array(
                    ComplexWikiPage :: PROPERTY_IS_HOMEPAGE => array(
                        'name' => ComplexWikiPage :: PROPERTY_IS_HOMEPAGE,
                        'type' => 'boolean',
                        'default' => '0',
                        'notnull' => true),
                    ComplexWikiPage :: PROPERTY_IS_LOCKED => array(
                        'name' => ComplexWikiPage :: PROPERTY_IS_LOCKED,
                        'type' => 'boolean',
                        'default' => '0',
                        'notnull' => true));

                foreach ($complex_wiki_page_attributes as $column => $attributes)
                {
                    $success = $data_manager->alter_storage_unit(
                        \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                        ComplexWikiPage :: get_table_name(),
                        $column,
                        $attributes);

                    if (! $success)
                    {
                        $upgrader->add_message(
                            Action :: TYPE_ERROR,
                            "Failed to make changes to column '$column' of table " . ComplexWikiPage :: get_table_name());
                        return false;
                    }
                    else
                    {
                        $upgrader->add_message(
                            Action :: TYPE_NORMAL,
                            "Made changes to column '$column' of table " . ComplexWikiPage :: get_table_name());
                    }
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            ContentObject :: class_name(),
                            ContentObject :: PROPERTY_TYPE),
                        new StaticConditionVariable(WikiPage :: class_name()))
                );

                $condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        ContentObject :: class_name(),
                        ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable(WikiPage :: get_type_name())
                );

                if (! \repository\DataManager :: updates(
                    ContentObject :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }

                $properties = new DataClassProperties();
                $properties->add(
                    new DataClassProperty(
                        new PropertyConditionVariable(
                            \repository\ComplexContentObjectItem :: class_name(),
                            \repository\ComplexContentObjectItem :: PROPERTY_TYPE),
                        new StaticConditionVariable('repository\content_object\wiki_page\ComplexWikiPage')));

                $subselect_condition = new EqualityCondition(
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_TYPE),
                    new StaticConditionVariable('repository\content_object\wiki_page\WikiPage'));

                $condition = new SubselectCondition(
                    new PropertyConditionVariable(
                        \repository\ComplexContentObjectItem :: class_name(),
                        \repository\ComplexContentObjectItem :: PROPERTY_REF),
                    new PropertyConditionVariable(
                        \repository\ContentObject :: class_name(),
                        \repository\ContentObject :: PROPERTY_ID),
                    null,
                    $subselect_condition);

                if (! \repository\DataManager :: updates(
                    \repository\ComplexContentObjectItem :: class_name(),
                    $properties,
                    $condition))
                {
                    return false;
                }
                
                $package = Package :: get('repository\content_object\wiki_page');
                $registration = CommonDataManager :: get_registration('repository\content_object\wiki_page');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                return true;
            });

        return $success;
    }
}
