<?php
namespace repository\integration\core\home;

use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\Session;
use repository\content_object\rss_feed\RssFeed;
use repository\content_object\link\Link;
use common\libraries\AndCondition;
use common\libraries\OrCondition;

/**
 * $Id: repository_connector.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.block.connectors
 */

/**
 * Simple connector class to facilitate rendering settings forms by preprocessing data from the datamanagers to a simple
 * array format.
 *
 * @author Hans De Bisschop
 */
class Connector
{

    /**
     * Returns a list of objects for the specified types.
     *
     * @param array $types
     * @return array
     */
    public static function get_objects($types)
    {
        $result = array();

        $conditions = array();
        $conditions[] = new EqualityCondition(\repository\ContentObject :: PROPERTY_OWNER_ID, Session :: get_user_id());

        $types_condition = array();
        foreach ($types as $type)
        {
            $types_condition[] = new EqualityCondition(\repository\ContentObject :: PROPERTY_TYPE, $type);
        }
        $conditions[] = new OrCondition($types_condition);
        $condition = new AndCondition($conditions);

        $objects = \repository\DataManager :: retrieve_active_content_objects(\repository\ContentObject :: class_name(), $condition);

        if ($objects->size() == 0)
        {
            $result[0] = Translation :: get('CreateObjectFirst');
        }
        else
        {
            while ($object = $objects->next_result())
            {
                $result[$object->get_id()] = $object->get_title();
            }
        }

        return $result;
    }

    public function get_rss_feed_objects()
    {
        $options = array();
        $condition = new EqualityCondition(\repository\ContentObject :: PROPERTY_OWNER_ID, Session :: get_user_id());
        $objects = \repository\DataManager :: retrieve_active_content_objects(RssFeed :: class_name(), $condition);

        if ($objects->size() == 0)
        {
            $options[0] = Translation :: get('CreateRssFeedFirst');
        }
        else
        {
            while ($object = $objects->next_result())
            {
                $options[$object->get_id()] = $object->get_title();
            }
        }

        return $options;
    }

    public function get_link_objects()
    {
        $options = array();
        $condition = new EqualityCondition(\repository\ContentObject :: PROPERTY_OWNER_ID, Session :: get_user_id());
        $objects = \repository\DataManager :: retrieve_active_content_objects(Link :: class_name(), $condition);

        if ($objects->size() == 0)
        {
            $options[0] = Translation :: get('CreateLinkFirst');
        }
        else
        {
            while ($object = $objects->next_result())
            {
                $options[$object->get_id()] = $object->get_title();
            }
        }

        return $options;
    }

    /**
     * Returns a list of objects that can be linked to a streaming block.
     *
     * @return array
     */
    public function get_streaming_objects()
    {
        return self :: get_objects(Streaming :: get_supported_types());
    }

    public function get_twitter_search_objects()
    {
        return self :: get_objects(TwitterSearch :: get_supported_types());
    }
}
