<?php
namespace repository\integration\webservice;

use common\libraries\Translation;
use common\libraries\WebserviceHandler;
use common\libraries\SuccessRestMessage;
use common\libraries\Utilities;
use common\libraries\Request;

class ContentObjectIncludeDeleteWebserviceHandler extends WebserviceHandler
{

    public function run()
    {
        $content_object_id = Request :: get('content_object_id');
        $include_id = Request :: get('include_id');

        if (empty($content_object_id) && empty($include_id))
        {
            return new SuccessRestMessage(false,
                    Translation :: get('ObjectNotFound', null, Utilities :: COMMON_LIBRARIES));
        }

        if ($content_object_id && $include_id)
        {
            $content_object = \repository\DataManager :: retrieve_content_object($content_object_id);
            if ($this->is_allowed_to_delete_include($content_object))
            {
                $success = $content_object->exclude_content_object($include_id);
            }
        }
        else
        {
            if ($content_object_id)
            {
                $content_object = \repository\DataManager :: retrieve_content_object($content_object_id);
                if ($this->is_allowed_to_delete_include($content_object))
                {
                    $includes = $content_object->get_includes();
                    foreach ($includes as $include)
                    {
                        if (! $include->delete())
                        {
                            return false;
                        }
                    }
                }
            }
            else
            {
                $content_object = \repository\DataManager :: retrieve_content_object($include_id);
                if ($this->is_allowed_to_delete_include($content_object))
                {
                    $includers = $content_object->get_includers();
                    foreach ($includers as $includer)
                    {
                        if (! $includer->delete())
                        {
                            return false;
                        }
                    }
                }
            }
        }

        if (! $success)
        {
            return new SuccessRestMessage(false,
                    Translation :: get('FailedToDeleteObject', null, Utilities :: COMMON_LIBRARIES));
        }

        return new SuccessRestMessage(true, Translation :: get('IncludeDeleted', null, Utilities :: COMMON_LIBRARIES));
    }

    private function is_allowed_to_delete_include($content_object)
    {
        return ($this->get_user()->is_platform_admin() || $content_object->get_owner_id() == $this->get_user_id());
    }
}
