<?php
namespace repository;

use common\libraries\Display;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Utilities;
use common\libraries\ParameterNotDefinedException;
use common\libraries\NotAllowedException;

/**
 *
 * @author Michael Kyndt
 */
class ComplexDisplayComponentAttachmentViewerComponent extends ComplexDisplayComponent
{
    const PROPERTY_REFERENCE_ID = 'reference_id';

    private $action_bar;

    public function run()
    {
        if (! $this->get_parent()->get_parent()->is_allowed_to_view_content_object())
        {
            Display :: not_allowed();
            return;
        }
        
        /*
         * Retrieve data and check if it is a valid attachment
         */
        
        $attachment_id = Request :: get(ComplexDisplay :: PARAM_ATTACHMENT_ID);
        if (is_null($attachment_id))
        {
            throw new ParameterNotDefinedException(ComplexDisplay :: PARAM_ATTACHMENT_ID);
        }
        
        $selected_complex_content_object_item = $this->get_selected_complex_content_object_item();
        $reference_content_object = $selected_complex_content_object_item->get_ref_object();
        $reference_content_object_id = $reference_content_object->get_id();
        
        if (\repository\DataManager :: is_helper_type($reference_content_object->get_type()))
        
        {
            $reference_content_object_id = $reference_content_object->get_additional_property(
                    self :: PROPERTY_REFERENCE_ID);
        }
        
        $attachment = \repository\DataManager :: retrieve_content_object($attachment_id);
        
        if (! $reference_content_object->is_attached_to_or_included_in($attachment_id))
        {
            throw new NotAllowedException();
        }
        
        /*
         * Render the attachment
         */
        $trail = BreadcrumbTrail :: get_instance();
        $trail->add(
                new Breadcrumb($this->get_url(array(ComplexDisplay :: PARAM_ATTACHMENT_ID => $attachment_id)), 
                        Translation :: get('ViewAttachment')));
        Display :: small_header();
        
        echo '<a href="javascript:history.go(-1)">' . Translation :: get('Back', null, Utilities :: COMMON_LIBRARIES) . '</a><br /><br />';
        
        echo ContentObjectRenditionImplementation :: launch($attachment, ContentObjectRendition :: FORMAT_HTML, 
                ContentObjectRendition :: VIEW_FULL, $this);
        
        Display :: small_footer();
    }
}
