<?php
namespace repository;

use common\libraries\DataClassRetrievesParameters;
use common\libraries\Translation;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\EqualityCondition;
use common\libraries\Utilities;
use common\libraries\Display;

class ComplexDisplayComponentUpdaterComponent extends ComplexDisplayComponent
{

    public function run()
    {
        if ($this->get_parent()->get_parent()->is_allowed_to_edit_content_object())
        {
            $selected_complex_content_object_item = $this->get_selected_complex_content_object_item();
            $content_object = DataManager :: retrieve_content_object($selected_complex_content_object_item->get_ref());
            $form = ContentObjectForm :: factory(ContentObjectForm :: TYPE_EDIT, $content_object, 'edit', 'post', 
                    $this->get_url(
                            array(
                                    ComplexDisplay :: PARAM_DISPLAY_ACTION => ComplexDisplay :: ACTION_UPDATE_COMPLEX_CONTENT_OBJECT_ITEM, 
                                    ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_selected_complex_content_object_item_id(), 
                                    ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id())));
            
            if ($form->validate())
            {
                $succes = $form->update_content_object();
                if ($succes && $form->is_version())
                {
                    $old_id = $selected_complex_content_object_item->get_ref();
                    $new_id = $content_object->get_latest_version()->get_id();
                    $selected_complex_content_object_item->set_ref($new_id);
                    $selected_complex_content_object_item->update();
                    $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $old_id, 
                            ComplexContentObjectItem :: get_table_name());
                    $parameters = new DataClassRetrievesParameters($condition);
                    $children = DataManager :: retrieve_complex_content_object_items(
                            ComplexContentObjectItem :: class_name(), $parameters);
                    $failures = 0;
                    while ($child = $children->next_result())
                    {
                        $child->set_parent($new_id);
                        if (! $child->update())
                        {
                            $failures ++;
                        }
                    }
                    
                    $succes = ($succes) && ($failures == 0);
                }
                
                $message = htmlentities(
                        Translation :: get(($succes ? 'ObjectUpdated' : 'ObjectNotUpdated'), 
                                array('OBJECT' => Translation :: get('ContentObject')), Utilities :: COMMON_LIBRARIES));
                
                $params = array();
                $params[ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID] = $this->get_complex_content_object_item_id();
                $params[ComplexDisplay :: PARAM_DISPLAY_ACTION] = ComplexDisplay :: ACTION_VIEW_COMPLEX_CONTENT_OBJECT;
                
                $this->redirect($message, (! $succes), $params);
            }
            else
            {
                $trail = BreadcrumbTrail :: get_instance();
                $trail->add(
                        new Breadcrumb(
                                $this->get_url(
                                        array(
                                                ComplexDisplay :: PARAM_SELECTED_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_selected_complex_content_object_item_id(), 
                                                ComplexDisplay :: PARAM_COMPLEX_CONTENT_OBJECT_ITEM_ID => $this->get_complex_content_object_item_id())), 
                                Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES)));
                
                $this->display_header();
                $form->display();
                $this->display_footer();
            }
        }
        else
        {
            Display :: not_allowed();
            return;
        }
    }
}
