<?php
namespace repository;

use common\libraries\DataClassRetrievesParameters;
use common\libraries\PlatformSetting;
use common\libraries\Utilities;
use common\libraries\Session;
use common\libraries\Path;
use common\libraries\InCondition;
use vcalendar;

require_once Path :: get_plugin_path() . 'icalcreator/iCalcreator.class.php';
class IcalContentObjectExportController extends ContentObjectExportController
{

    private $ical;

    private $file;

    public function __construct(ExportParameters $parameters)
    {
        parent :: __construct($parameters);
        
        $this->ical = new vcalendar();
        $this->ical->setConfig('unique_id', Path :: get(WEB_PATH));
        $this->ical->setProperty('method', 'PUBLISH');
        $this->ical->setProperty('x-wr-calname', PlatformSetting :: get('site_name'));
        $this->ical->setConfig('url', Path :: get(WEB_PATH));
        
        $this->prepare_file_system();
    }

    public function get_ical()
    {
        return $this->ical;
    }

    public function run()
    {
        $content_object_ids = $this->get_parameters()->get_content_object_ids();
        
        if (count($content_object_ids) > 0)
        {
            $condition = new InCondition(ContentObject :: PROPERTY_ID, $content_object_ids, 
                    ContentObject :: get_table_name());
        }
        else
        {
            $condition = null;
        }
        
        $parameters = new DataClassRetrievesParameters($condition);
        $content_objects = DataManager :: retrieve_active_content_objects(ContentObject :: class_name(), $parameters);
        
        while ($content_object = $content_objects->next_result())
        {
            $this->process($content_object);
        }
        $this->ical();
        return $this->file;
    }

    public function process($content_object)
    {
        $export_types = ContentObjectExportImplementation :: get_types_for_object(
                Utilities :: get_namespace_from_object($content_object));
        if (in_array(ContentObjectExport :: FORMAT_ICAL, $export_types))
        {
            ContentObjectExportImplementation :: launch($this, $content_object, ContentObjectExport :: FORMAT_ICAL, 
                    $this->get_parameters()->get_type());
        }
    }

    public function prepare_file_system()
    {
        $user_id = Session :: get_user_id();
        
        $directory = Path :: get(SYS_TEMP_PATH) . $user_id . '/';
        if (! is_dir($directory))
        {
            mkdir($directory, 0777, true);
        }
        $this->file = $directory . 'export_ical.ics';
    }

    public function ical()
    {
        $calendar = $this->ical->createCalendar();
        
        $handle = fopen($this->file, 'w+');
        fwrite($handle, $calendar);
        fclose($handle);
    }

    public function get_filename()
    {
        return 'export_ical.ics';
    }
}
