<?php
namespace repository;

use common\libraries\EqualityCondition;
use common\libraries\Qti;
use repository\content_object\assessment\Assessment;
use common\libraries\ImsQtiWriter;

/**
 *
 * @copyright (c) 2010 University of Geneva
 * @author laurent.opprecht@unige.ch
 */
class QtiAssessmentSerializer extends QtiSerializerBase
{

    public static function factory($object, $target_root, $controller)
    {
        if ($object instanceof Assessment)
        {
            return new self($target_root, $controller);
        }
        else
        {
            return null;
        }
    }

    public function serialize(Assessment $assessment)
    {
        $writer = new ImsQtiWriter();
        $assessment_id = self :: get_identifier($assessment);
        $test = $writer->add_assessmentTest($assessment_id, $assessment->get_title(), self :: get_tool_name(), 
                Qti :: get_tool_version());
        $part = $test->add_testPart(null, Qti :: NAVIGATION_MODE_LINEAR, Qti :: SUBMISSION_MODE_INDIVIDUAL);
        $part->add_itemSessionControl(0, true, true, true, true, true, false);
        $section = $part->add_assessmentSection(null, $assessment->get_title(), false);
        $instruction = $this->translate_text($assessment->get_description());
        
        $section->add_rubricBlock(Qti :: VIEW_ALL)->add_flow($instruction);
        
        $this->add_children($section, $assessment->get_id());
        return $writer->saveXML();
    }

    protected function add_children(ImsQtiWriter $writer, $parent_id)
    {
        $children = $this->retrieve_children($parent_id);
        while ($child = $children->next_result())
        {
            $child_object = $this->retrieve_object($child->get_ref());
            if ($exporter = $this->create_exporter($child_object))
            {
                $path = $exporter->export_content_object();
                $filename = basename($path);
                $ref = $writer->add_assessmentItemRef('ID_' . $child->get_id(), $filename, '', $required = true);
                $ref->add_weight(null, $child->get_weight());
            }
        }
    }

    protected function retrieve_children($object_id)
    {
        $condition = new EqualityCondition(ComplexContentObjectItem :: PROPERTY_PARENT, $object_id, 
                ComplexContentObjectItem :: get_table_name());
        $result = DataManager :: retrieve_complex_content_object_items(ComplexContentObjectItem :: class_name(), $condition);
        return $result;
    }

    protected function retrieve_object($ref)
    {
        $result = DataManager :: retrieve_content_object($ref);
        if ($result instanceof ComplexContentObjectItem)
        {
            $result = DataManager :: retrieve_content_object($result->get_ref());
        }
        return $result;
    }

    protected function create_exporter($object)
    {
        $result = QtiSerializerExport :: factory($this->get_controller(), $object);
        return $result;
    }
}
