<?php
namespace repository;

use common\libraries\StaticConditionVariable;
use common\libraries\PropertyConditionVariable;
use repository\quota\Calculator;
use common\libraries\Filesystem;
use common\libraries\Path;
use common\libraries\FileProperties;
use common\libraries\Application;
use common\libraries\Redirect;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;
use repository\content_object\document\Document;

class DocumentContentObjectImportController extends ContentObjectImportController
{
    const FORMAT = 'repository\content_object\document\Document';

    public function run()
    {
        if (self :: is_available())
        {
            if ($this->get_parameters()->get_document_type() == DocumentContentObjectImportForm :: DOCUMENT_UPLOAD)
            {
                $file = $this->get_parameters()->get_file();
                $calculator = new Calculator(
                    \user\DataManager :: retrieve(\user\User :: class_name(), (int) $this->get_parameters()->get_user()));

                if ($calculator->get_available_user_disk_quota() < $file->get_size())
                {
                    $this->add_message(Translation :: get('InsufficientDiskQuota'), self :: TYPE_ERROR);
                    return array();
                }
            }
            else
            {
                $file = FileProperties :: from_url($this->get_parameters()->get_link());

                if ($file->get_path() && $file->get_name() && $file->get_extension())
                {
                    $calculator = new Calculator(
                        \user\DataManager :: retrieve(
                            \user\User :: class_name(),
                            (int) $this->get_parameters()->get_user()));

                    if ($calculator->get_available_user_disk_quota() > $file->get_size())
                    {
                        $temp_path = Path :: get_temp_path() . 'repository/import/document/' .
                             $file->get_name_extension();

                        if (file_exists($temp_path))
                        {
                            $this->add_message(Translation :: get('ObjectNotImported'), self :: TYPE_ERROR);
                            return array();
                        }
                        else
                        {
                            $destination_dir = dirname($temp_path);
                            if (Filesystem :: create_dir($destination_dir))
                            {
                                if (copy($file->get_path(), $temp_path))
                                {
                                    $file = FileProperties :: from_path($temp_path);
                                }
                                else
                                {
                                    $this->add_message(Translation :: get('ObjectNotImported'), self :: TYPE_ERROR);
                                    return array();
                                }
                            }
                        }
                    }
                    else
                    {
                        $this->add_message(Translation :: get('InsufficientDiskQuota'), self :: TYPE_ERROR);
                        return array();
                    }
                }
                else
                {
                    $this->add_message(Translation :: get('InvalidDocumentLink'), self :: TYPE_ERROR);
                    return array();
                }
            }

            $document = new Document();
            $document->set_title($file->get_name());
            $document->set_description($file->get_name());
            $document->set_owner_id($this->get_parameters()->get_user());
            $document->set_parent_id($this->get_parameters()->get_category());
            $document->set_filename($file->get_name_extension());

            $hash = md5_file($file->get_path());
            $conditions = array();
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(ContentObject :: class_name(), ContentObject :: PROPERTY_OWNER_ID),
                new StaticConditionVariable($this->get_parameters()->get_user()));
            $conditions[] = new EqualityCondition(
                new PropertyConditionVariable(Document :: class_name(), Document :: PROPERTY_HASH),
                new StaticConditionVariable($hash));
            $condtion = new AndCondition($conditions);
            $content_objects = DataManager :: retrieve_active_content_objects(Document :: class_name(), $condtion);

            if ($content_objects->size() > 0)
            {
                if ($content_objects->size() == 1)
                {
                    $content_object = $content_objects->next_result();
                    $link = Redirect :: get_link(
                        RepositoryManager :: APPLICATION_NAME,
                        array(
                            Application :: PARAM_ACTION => RepositoryManager :: ACTION_VIEW_CONTENT_OBJECTS,
                            RepositoryManager :: PARAM_CONTENT_OBJECT_ID => $content_object->get_id()));
                    $this->add_message(
                        Translation :: get('ObjectAlreadyExists', array('LINK' => $link)),
                        self :: TYPE_ERROR);
                }
                else
                {
                    $this->add_message(Translation :: get('ObjectAlreadyExistsMultipleTimes'), self :: TYPE_ERROR);
                }
            }
            else
            {
                $document->set_temporary_file_path($file->get_path());

                if ($document->create())
                {
                    $this->add_message(Translation :: get('ObjectImported'), self :: TYPE_CONFIRM);
                    return array($document->get_id());
                }
                else
                {
                    $this->add_message(Translation :: get('ObjectNotImported'), self :: TYPE_ERROR);
                }
            }
        }
        else
        {
            $this->add_message(Translation :: get('DocumentObjectNotAvailable'), self :: TYPE_WARNING);
        }
    }

    public static function is_available()
    {
        return in_array(self :: FORMAT, DataManager :: get_registered_types(true));
    }
}
