<?php
namespace repository;

use repository\content_object\vimeo\Vimeo;
use common\extensions\external_repository_manager\DataConnector;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;

class VimeoContentObjectImportController extends ContentObjectImportController
{
    const FORMAT = 'repository\content_object\vimeo\Vimeo';

    public function run()
    {
        $url = $this->get_parameters()->get_url();

        if (self :: is_available())
        {
            $url_parts = parse_url($url);

            parse_str($url_parts['query'], $url_query);

            if (strpos($url_parts['host'], 'vimeo.com') !== false)
            {
                $external_id = substr($url_parts['path'], 1);

                if (! is_numeric($external_id))
                {
                    $this->add_message(Translation :: get('ObjectNotImported'), self :: TYPE_ERROR);
                }
            }
            else
            {
                $this->add_message(Translation :: get('ObjectNotImported'), self :: TYPE_ERROR);
            }

            if (! $this->has_messages(self :: TYPE_ERROR))
            {
                $conditions = array();
                $conditions[] = new EqualityCondition(
                    ExternalInstance :: PROPERTY_TYPE,
                    \common\extensions\external_repository_manager\Manager :: get_namespace(self :: FORMAT));
                $conditions[] = new EqualityCondition(ExternalInstance :: PROPERTY_ENABLED, 1);
                $condition = new AndCondition($conditions);

                $external_repositories = \repository\DataManager :: retrieves(ExternalInstance :: class_name(), $condition);

                $external_repository = $external_repositories->next_result();
                $vimeo_connector = DataConnector :: get_instance($external_repository);
                $external_object = $vimeo_connector->retrieve_external_repository_object($external_id);

                $vimeo = ContentObject :: factory(Vimeo :: get_type_name());
                $vimeo->set_title($external_object->get_title());
                $vimeo->set_description($external_object->get_description());
                $vimeo->set_owner_id($this->get_parameters()->get_user());

                if ($vimeo->create())
                {
                    ExternalSync :: quicksave($vimeo, $external_object, $external_repository->get_id());
                    $this->add_message(Translation :: get('ObjectImported'), self :: TYPE_CONFIRM);
                    return array($vimeo->get_id());
                }
                else
                {
                    $this->add_message(Translation :: get('ObjectNotImported'), self :: TYPE_ERROR);
                }
            }
        }
        else
        {
            $this->add_message(Translation :: get('VimeoObjectNotAvailable'), self :: TYPE_WARNING);
        }
    }

    public static function is_available()
    {
        $vimeo_object_available = in_array(self :: FORMAT, DataManager :: get_registered_types(true));

        $conditions = array();
        $conditions[] = new EqualityCondition(
            ExternalInstance :: PROPERTY_TYPE,
            \common\extensions\external_repository_manager\Manager :: get_namespace(self :: FORMAT));
        $conditions[] = new EqualityCondition(ExternalInstance :: PROPERTY_ENABLED, 1);
        $condition = new AndCondition($conditions);

        $external_repositories = \repository\DataManager :: retrieves(ExternalInstance :: class_name(), $condition);
        $vimeo_connector_available = $external_repositories->size() == 1;

        return $vimeo_object_available && $vimeo_connector_available;
    }
}
