<?php
namespace repository;

use repository\content_object\youtube\Youtube;
use common\extensions\external_repository_manager\DataConnector;
use common\libraries\AndCondition;
use common\libraries\EqualityCondition;
use common\libraries\Translation;

class YoutubeContentObjectImportController extends ContentObjectImportController
{
    const FORMAT = 'repository\content_object\youtube\Youtube';
    const REPOSITORY_TYPE = 'common\extensions\external_repository_manager\implementation\youtube';

    public function run()
    {
        $url = $this->get_parameters()->get_url();

        if (self :: is_available())
        {
            $url_parts = parse_url($url);

            parse_str($url_parts['query'], $url_query);

            if (strpos($url_parts['host'], 'youtu.be') !== false)
            {
                $external_id = substr($url_parts['path'], 1);
            }
            elseif (strpos($url_parts['host'], 'youtube.com') !== false)
            {
                $external_id = $url_query['v'];
            }
            else
            {
                $this->add_message(Translation :: get('ObjectNotImported'), self :: TYPE_ERROR);
            }

            if (! $this->has_messages(self :: TYPE_ERROR))
            {
                $conditions = array();
                $conditions[] = new EqualityCondition(ExternalInstance :: PROPERTY_TYPE, self :: REPOSITORY_TYPE);
                $conditions[] = new EqualityCondition(ExternalInstance :: PROPERTY_ENABLED, 1);
                $condition = new AndCondition($conditions);

                $external_repositories = \repository\DataManager :: retrieves(ExternalInstance :: class_name(), $condition);

                $external_repository = $external_repositories->next_result();
                $youtube_connector = DataConnector :: get_instance($external_repository);
                $external_object = $youtube_connector->retrieve_external_repository_object($external_id);

                $youtube = ContentObject :: factory(Youtube :: class_name());
                $youtube->set_title($external_object->get_title());
                $youtube->set_description($external_object->get_description());
                $youtube->set_owner_id($this->get_parameters()->get_user());

                if ($youtube->create())
                {
                    ExternalSync :: quicksave($youtube, $external_object, $external_repository->get_id());
                    $this->add_message(Translation :: get('ObjectImported'), self :: TYPE_CONFIRM);
                    return array($youtube->get_id());
                }
                else
                {
                    $this->add_message(Translation :: get('ObjectNotImported'), self :: TYPE_ERROR);
                }
            }
        }
        else
        {
            $this->add_message(Translation :: get('YouTubeObjectNotAvailable'), self :: TYPE_WARNING);
        }
    }

    public static function is_available()
    {
        $youtube_object_available = in_array(self :: FORMAT, DataManager :: get_registered_types(true));

        $conditions = array();
        $conditions[] = new EqualityCondition(ExternalInstance :: PROPERTY_TYPE, self :: REPOSITORY_TYPE);
        $conditions[] = new EqualityCondition(ExternalInstance :: PROPERTY_ENABLED, 1);
        $condition = new AndCondition($conditions);

        $external_repositories = \repository\DataManager :: retrieves(ExternalInstance :: class_name(), $condition);
        $youtube_connector_available = $external_repositories->size() == 1;

        return $youtube_object_available && $youtube_connector_available;
    }
}
