<?php
namespace repository;

use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\StaticTableColumn;
use common\libraries\ObjectTableColumnModel;
use common\libraries\ObjectTableColumn;

/**
 * $Id: default_content_object_table_column_model.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.content_object_table
 */

/**
 * This is the default column model, used when a ContentObjectTable does not provide its own model. The default model
 * contains the following columns: - The type of the object - The title of the object - The description of the object -
 * The date when the object was last modified Although this model works best in conjunction with the default cell
 * renderer, it can be used with any ContentObjectTableCellRenderer.
 *
 * @see ContentObjectTable
 * @see ContentObjectTableColumnModel
 * @see DefaultContentObjectTableCellRenderer
 * @author Tim De Pauw
 */
class DefaultContentObjectTableColumnModel extends ObjectTableColumnModel
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(self :: get_default_columns(), 1);
    }

    /**
     * Gets the default columns for this model
     *
     * @return ContentObjectTableColumn[]
     */
    private static function get_default_columns()
    {
        $columns = array();
        // $columns[] = new ObjectTableColumn(ContentObject :: PROPERTY_TYPE);
        $columns[] = new StaticTableColumn(
            Theme :: get_common_image(
                'action_category',
                'png',
                Translation :: get('Type'),
                null,
                ToolbarItem :: DISPLAY_ICON));
        $columns[] = new ObjectTableColumn(
            ContentObject :: PROPERTY_TITLE,
            true,
            DataManager :: get_alias(ContentObject :: get_table_name()));
        $columns[] = new ObjectTableColumn(
            ContentObject :: PROPERTY_DESCRIPTION,
            true,
            DataManager :: get_alias(ContentObject :: get_table_name()));
        return $columns;
    }
}
