<?php
namespace repository;

use common\libraries\ResultSet;

/**
 * $Id: database_content_object_result_set.class.php 200 2009-11-13 12:30:04Z kariboe $
 * @package repository.lib.data_manager.database
 *
 * Resultset to hold a set of objects
 */
class DatabaseContentObjectResultSet extends ResultSet
{
    /**
     * The datamanager used to retrieve objects from the repository
     */
    private $data_manager;
    /**
     * An instance of DB_result
     */
    private $handle;
    /**
     * Flag to know if the $handle contains all properties of the course
     * category
     */
    private $single_type;

    /**
     * Create a new resultset for handling a set of objects
     * @param RepositoryDataManager $data_manager The datamanager used to
     * retrieve objects from the repository
     * @param DB_result $handle The handle to retrieve records from a database
     * resultset
     * @param boolean $single_type True if the handle holds all properties of
     * the objects (so when retrieving the learning objects, the
     * datamanager shouldn't perform additional queries)
     */
    public function __construct($data_manager, $handle, $single_type)
    {
        $this->data_manager = $data_manager;
        $this->handle = $handle;
        $this->single_type = $single_type;
    }

    /*
	 * Inherited
	 */
    public function next_result($map_to_object = true)
    {
        //dump($this->handle);
        if ($record = $this->handle->fetchRow(MDB2_FETCHMODE_ASSOC))
        {
            return $this->data_manager->record_to_content_object($record, $this->single_type);
        }
        return null;
    }

    /*
	 * Inherited
	 */
    public function size()
    {
        return $this->handle->numRows();
    }

    /*
	 * Inherited
	 */
    public function skip($count)
    {
        for($i = 0; $i < $count; $i ++)
        {
            $this->handle->fetchRow();
        }
    }
}
