<?php
namespace repository;

use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;

/**
 * $Id: external_repository.class.php 204 2009-11-13 12:51:30Z kariboe $
 *
 * @package repository.lib
 */
class ExternalInstance extends RepositoryDataClass
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_TITLE = 'title';
    const PROPERTY_DESCRIPTION = 'description';
    const PROPERTY_TYPE = 'type';
    const PROPERTY_ENABLED = 'enabled';

    /**
     * Contains a list of already required export types Allow to spare some business logic processing
     *
     * @var array
     */
    private static $already_required_types = array();

    public function __construct($defaultProperties = array ())
    {
        parent :: __construct($defaultProperties);
    }

    public function set_title($title)
    {
        if (isset($title) && strlen($title) > 0)
        {
            $this->set_default_property(self :: PROPERTY_TITLE, $title);
        }
    }

    /**
     *
     * @return string The export title
     */
    public function get_title()
    {
        return $this->get_default_property(self :: PROPERTY_TITLE);
    }

    public function set_description($description)
    {
        if (isset($description) && strlen($description) > 0)
        {
            $this->set_default_property(self :: PROPERTY_DESCRIPTION, $description);
        }
    }

    /**
     *
     * @return string The export description
     */
    public function get_description()
    {
        return $this->get_default_property(self :: PROPERTY_DESCRIPTION);
    }

    public function set_type($type)
    {
        if (isset($type) && strlen($type) > 0)
        {
            $this->set_default_property(self :: PROPERTY_TYPE, $type);
        }
    }

    /**
     *
     * @return string The export type
     */
    public function get_type()
    {
        return $this->get_default_property(self :: PROPERTY_TYPE);
    }

    public function set_enabled($enabled)
    {
        if (isset($enabled) && is_bool($enabled))
        {
            $this->set_default_property(self :: PROPERTY_ENABLED, $enabled);
        }
    }

    /**
     *
     * @return boolean Indicates wether the export is enabled or not
     */
    public function get_enabled()
    {
        return $this->get_default_property(self :: PROPERTY_ENABLED, false);
    }

    public function is_enabled()
    {
        return $this->get_enabled();
    }

    public static function get_default_property_names($extended_property_names = array())
    {
        $extended_property_names[] = self :: PROPERTY_TITLE;
        $extended_property_names[] = self :: PROPERTY_DESCRIPTION;
        $extended_property_names[] = self :: PROPERTY_TYPE;
        $extended_property_names[] = self :: PROPERTY_ENABLED;

        return parent :: get_default_property_names($extended_property_names);
    }

    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: class_name(), true);
    }

    /**
     * *********************************************************************** Fat model methods
     * ***********************************************************************
     */
    public function create()
    {
        if (! parent :: create())
        {
            return false;
        }
        else
        {
            if (! ExternalSetting :: initialize($this))
            {
                return false;
            }
        }

        $succes = RepositoryRights :: get_instance()->create_location_in_external_instances_subtree($this->get_id(),
                RepositoryRights :: get_instance()->get_external_instances_subtree_root_id());
        if (! $succes)
        {
            return false;
        }

        return true;
    }

    public function delete()
    {
        if (! parent :: delete())
        {
            return false;
        }
        else
        {
            $condition = new EqualityCondition(ExternalSetting :: PROPERTY_EXTERNAL_ID, $this->get_id());
            $settings = \repository\DataManager :: retrieves(ExternalSetting :: class_name(), $condition);

            while ($setting = $settings->next_result())
            {
                if (! $setting->delete())
                {
                    return false;
                }
            }
        }

        $location = RepositoryRights :: get_instance()->get_location_by_identifier_from_external_instances_subtree(
                $this->get_id());
        if ($location)
        {
            if (! $location->delete())
            {
                return false;
            }
        }

        return true;
    }

    public function activate()
    {
        $this->set_enabled(true);
    }

    public function deactivate()
    {
        $this->set_enabled(false);
    }

    public function has_user_settings()
    {
        $conditions[] = new EqualityCondition(ExternalSetting :: PROPERTY_EXTERNAL_ID, $this->get_id());
        $conditions[] = new EqualityCondition(ExternalSetting :: PROPERTY_USER_SETTING, 1);
        $condition = new AndCondition($conditions);
        $settings = DataManager :: count(ExternalSetting :: class_name(), $condition);

        return $settings > 0;
    }

    public function get_user_setting($variable)
    {
        return ExternalUserSetting :: get($variable, $this->get_id());
    }

    public function has_settings()
    {
        $condition = new EqualityCondition(ExternalSetting :: PROPERTY_EXTERNAL_ID, $this->get_id());

        $settings = DataManager :: count(ExternalSetting :: class_name(), $condition);

        return $settings > 0;
    }

    public function get_setting($variable)
    {
        return \repository\DataManager :: retrieve_external_setting_from_variable_name($variable, $this->get_id());
    }
}
