<?php
namespace repository;

use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\Theme;
use rights\RightsManager;

/**
 * Cell rendere for the learning object browser table
 */
class ExternalInstanceBrowserTableCellRenderer extends DefaultExternalInstanceTableCellRenderer
{
    private $browser;

    /**
     * Constructor
     * @param ExternalInstanceManager $browser
     */
    public function __construct($browser)
    {
        parent :: __construct();
        $this->browser = $browser;
    }

    // Inherited
    public function render_cell($column, $external_instance)
    {
        if ($column === ExternalInstanceBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($external_instance);
        }
        
        //        switch ($column->get_name())
        //        {
        //            //            case ContentObject :: PROPERTY_TYPE :
        //        //                return '<a href="' . htmlentities($this->browser->get_type_filter_url($external_repository->get_type())) . '">' . parent :: render_cell($column, $external_repository) . '</a>';
        //        //            case ContentObject :: PROPERTY_TITLE :
        //        //                $title = parent :: render_cell($column, $external_repository);
        //        //                $title_short = Utilities :: truncate_string($title, 53, false);
        //        //                return '<a href="' . htmlentities($this->browser->get_content_object_viewing_url($external_repository)) . '" title="' . $title . '">' . $title_short . '</a>';
        //        }
        return parent :: render_cell($column, $external_instance);
    }

    /**
     * Gets the action links to display
     * @param ContentObject $content_object The learning object for which the
     * action links should be returned
     * @return string A HTML representation of the action links
     */
    private function get_modification_links($external_instance)
    {
        $toolbar = new Toolbar();
        
        if ($external_instance->is_enabled())
        {
            $toolbar->add_item(new ToolbarItem(Translation :: get('Deactivate', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_deactivate.png', $this->browser->get_url(array(
                    ExternalInstanceManager :: PARAM_INSTANCE_ACTION => ExternalInstanceManager :: ACTION_DEACTIVATE_INSTANCE, 
                    ExternalInstanceManager :: PARAM_INSTANCE => $external_instance->get_id())), ToolbarItem :: DISPLAY_ICON, true));
        }
        else
        {
            $toolbar->add_item(new ToolbarItem(Translation :: get('Activate', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_activate.png', $this->browser->get_url(array(
                    ExternalInstanceManager :: PARAM_INSTANCE_ACTION => ExternalInstanceManager :: ACTION_ACTIVATE_INSTANCE, 
                    ExternalInstanceManager :: PARAM_INSTANCE => $external_instance->get_id())), ToolbarItem :: DISPLAY_ICON, true));
        }
        
        $toolbar->add_item(new ToolbarItem(Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_edit.png', $this->browser->get_url(array(
                ExternalInstanceManager :: PARAM_INSTANCE_ACTION => ExternalInstanceManager :: ACTION_UPDATE_INSTANCE, 
                ExternalInstanceManager :: PARAM_INSTANCE => $external_instance->get_id())), ToolbarItem :: DISPLAY_ICON));
        $toolbar->add_item(new ToolbarItem(Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES), Theme :: get_common_image_path() . 'action_delete.png', $this->browser->get_url(array(
                ExternalInstanceManager :: PARAM_INSTANCE_ACTION => ExternalInstanceManager :: ACTION_DELETE_INSTANCE, 
                ExternalInstanceManager :: PARAM_INSTANCE => $external_instance->get_id())), ToolbarItem :: DISPLAY_ICON, true));
        $toolbar->add_item(new ToolbarItem(Translation :: get('ManageRights', null, RightsManager :: APPLICATION_NAME), Theme :: get_common_image_path() . 'action_rights.png', $this->browser->get_url(array(
                ExternalInstanceManager :: PARAM_INSTANCE_ACTION => ExternalInstanceManager :: ACTION_MANAGE_INSTANCE_RIGHTS, 
                ExternalInstanceManager :: PARAM_INSTANCE => $external_instance->get_id())), ToolbarItem :: DISPLAY_ICON));
        return $toolbar->as_html();
    }
}
