<?php
namespace repository;

use common\libraries\CommonDataManager;
use common\libraries\Request;
use common\libraries\SubManager;
use common\libraries\EqualityCondition;
use common\libraries\OrCondition;
use common\libraries\AndCondition;
use common\libraries\Utilities;
use common\libraries\Registration;

class ExternalInstanceManager extends SubManager
{
    const PARAM_INSTANCE_ACTION = 'action';
    const PARAM_INSTANCE = 'instance';
    const PARAM_EXTERNAL_TYPE = 'instance_type';
    const PARAM_EXTERNAL_INSTANCE_TYPE = 'type';
    const ACTION_BROWSE_INSTANCES = 'browser';
    const ACTION_ACTIVATE_INSTANCE = 'activator';
    const ACTION_DEACTIVATE_INSTANCE = 'deactivator';
    const ACTION_UPDATE_INSTANCE = 'updater';
    const ACTION_DELETE_INSTANCE = 'deleter';
    const ACTION_CREATE_INSTANCE = 'creator';
    const ACTION_MANAGE_INSTANCE_RIGHTS = 'rights_editor';
    const DEFAULT_ACTION = self :: ACTION_BROWSE_INSTANCES;

    public function __construct($repository_manager)
    {
        parent :: __construct($repository_manager);
        
        $instance_action = Request :: get(self :: PARAM_INSTANCE_ACTION);
        if ($instance_action)
        {
            $this->set_action($instance_action);
        }
    }

    public function set_action($action)
    {
        $this->set_parameter(self :: PARAM_INSTANCE_ACTION, $action);
    }

    public function get_action()
    {
        return $this->get_parameter(self :: PARAM_INSTANCE_ACTION);
    }

    public static function launch($application)
    {
        parent :: launch(__CLASS__, $application);
    }

    /**
     * Helper function for the SubManager class, pending access to class constants via variables in PHP 5.3 e.g. $name =
     * $class :: DEFAULT_ACTION DO NOT USE IN THIS SUBMANAGER'S CONTEXT Instead use: - self :: DEFAULT_ACTION in the
     * context of this class - YourSubManager :: DEFAULT_ACTION in all other application classes
     */
    public static function get_default_action()
    {
        return self :: DEFAULT_ACTION;
    }

    /**
     * Helper function for the SubManager class, pending access to class constants via variables in PHP 5.3 e.g. $name =
     * $class :: PARAM_ACTION DO NOT USE IN THIS SUBMANAGER'S CONTEXT Instead use: - self :: PARAM_ACTION in the context
     * of this class - YourSubManager :: PARAM_ACTION in all other application classes
     */
    public static function get_action_parameter()
    {
        return self :: PARAM_INSTANCE_ACTION;
    }

    public static function get_registered_types($status = Registration :: STATUS_ACTIVE)
    {
        $instance_conditions = array();
        $instance_conditions[] = new EqualityCondition(Registration :: PROPERTY_TYPE, 
                'common\extensions\external_repository_manager\implementation');
        $instance_conditions[] = new EqualityCondition(Registration :: PROPERTY_TYPE, 
                'common\extensions\video_conferencing_manager\implementation');
        
        $conditions = array();
        $conditions[] = new OrCondition($instance_conditions);
        $conditions[] = new EqualityCondition(Registration :: PROPERTY_STATUS, $status);
        
        return CommonDataManager :: retrieves(Registration :: class_name(), new AndCondition($conditions));
    }

    public static function get_namespace($instance_type = null, $type = null)
    {
        if (is_null($instance_type) && is_null($type))
        {
            return __NAMESPACE__;
        }
        elseif (! is_null($instance_type) && is_null($type))
        {
            return 'common\extensions\\' . $instance_type;
        }
        elseif (! is_null($instance_type) && ! is_null($type))
        {
            return 'common\extensions\\' . $instance_type . '\implementation\\' . $type;
        }
    }

    public static function get_instance_identifier($type)
    {
        $class_name = $type . '\Manager';
        return $class_name :: get_instance_identifier();
    }

    public static function get_manager_class($type)
    {
        $parent = Utilities :: get_namespace_parent(Utilities :: get_namespace_parent($type));
        $instance_type = Utilities :: get_package_name_from_namespace($parent, true);
        $package = Utilities :: get_package_name_from_namespace($type, true);
        
        return $type . '\\' . $package . $instance_type;
    }

    public static function get_manager_connector_class($type)
    {
        return self :: get_manager_class($type) . 'Connector';
    }

    public static function exists($type)
    {
        return class_exists($type . '\Manager');
    }
}
