<?php
namespace repository;

use common\libraries\Path;
use common\libraries\CommonDataManager;
use common\libraries\InequalityCondition;
use common\libraries\OrCondition;
use common\libraries\PatternMatchCondition;
use common\libraries\FormValidator;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\Session;
use common\libraries\ResourceManager;
use common\libraries\AndCondition;

/**
 * $Id: repository_filter_form.class.php 200 2009-11-13 12:30:04Z kariboe $
 *
 * @package repository.lib.forms
 */
class AdvancedFilterForm extends FormValidator
{
    const FILTER_TYPE = 'filter_type';
    const FILTER_TEXT = 'text';

    private $manager;

    private $renderer;

    /**
     * Creates a new search form
     *
     * @param $manager RepositoryManager The repository manager in which this search form will be displayed
     * @param $url string The location to which the search request should be posted.
     */
    public function __construct($manager, $url)
    {
        parent :: __construct('advanced_filter_form', 'post', $url);

        $this->renderer = clone $this->defaultRenderer();
        $this->manager = $manager;

        $this->build_form();
        $this->setDefaults();
        $this->accept($this->renderer);
    }

    public function get_renderer()
    {
        return $this->renderer;
    }

    /**
     * Build the simple search form.
     */
    private function build_form()
    {
        $this->renderer->setFormTemplate(
            '<form {attributes}><div class="advanced_filter_form">{content}</div><div class="clear">&nbsp;</div></form>');
        $this->renderer->setElementTemplate('<div class="row">{element}</div>');
        // title
        $this->addElement('category', Translation :: get('TextSearch'));
        $this->addElement('text', self :: FILTER_TEXT, Translation :: get('TextSearch'), 'class="full"');
        $this->addElement('category');

        // creation date
        $this->addElement('category', Translation :: get('CreationDate'));
        $creation_date_from = array();
        $creation_date_from[] = $this->createElement(
            'static',
            '',
            '',
            '<span style="display:inline-block;width:35px">' . Translation :: get('From') . '</span>');
        $creation_date_from[] = $this->createElement(
            'text',
            ContentObject :: PROPERTY_CREATION_DATE,
            Translation :: get('From'),
            'id="creation_date_from" style="width:100px;"');
        $this->addGroup($creation_date_from, 'creation_date_from');
        $this->renderer->setGroupElementTemplate('{element}', 'creation_date_from');

        $creation_date_to = array();
        $creation_date_to[] = $this->createElement(
            'static',
            '',
            '',
            '<span style="display:inline-block; width:35px">' . Translation :: get('To') . '</span>');
        $creation_date_to[] = $this->createElement(
            'text',
            ContentObject :: PROPERTY_CREATION_DATE,
            Translation :: get('To'),
            'id="creation_date_to" style="width:100px;"');
        $this->addGroup($creation_date_to, 'creation_date_to');
        $this->renderer->setGroupElementTemplate('{element}', 'creation_date_to');
        $this->addElement('category');

        // modification date
        $this->addElement('category', Translation :: get('ModificationDate'));
        $modification_date_from = array();
        $modification_date_from[] = $this->createElement(
            'static',
            '',
            '',
            '<span style="display:inline-block;width:35px">' . Translation :: get('From') . '</span>');
        $modification_date_from[] = $this->createElement(
            'text',
            ContentObject :: PROPERTY_MODIFICATION_DATE,
            Translation :: get('From'),
            'id="modification_date_from" style="width:100px;"');
        $this->addGroup($modification_date_from, 'modification_date_from');
        $this->renderer->setGroupElementTemplate('{element}', 'modification_date_from');

        $modification_date_to = array();
        $modification_date_to[] = $this->createElement(
            'static',
            '',
            '',
            '<span style="display:inline-block; width:35px">' . Translation :: get('To') . '</span>');
        $modification_date_to[] = $this->createElement(
            'text',
            ContentObject :: PROPERTY_MODIFICATION_DATE,
            Translation :: get('To'),
            'id="modification_date_to" style="width:100px;"');
        $this->addGroup($modification_date_to, 'modification_date_to');
        $this->renderer->setGroupElementTemplate('{element}', 'modification_date_to');
        $this->addElement('category');

        // type
        $this->addElement('category', Translation :: get('Type'));
        $type_selector = new ContentObjectTypeSelector($this->manager, $this->get_allowed_content_object_types()/*,
                array(), $user_objects == 0*/);

        $select = $this->addElement(
            'select',
            ContentObject :: PROPERTY_TYPE,
            Translation :: get('Type'),
            array(),
            'class="full"');

        foreach ($type_selector->as_tree() as $key => $type)
        {
            $attributes = (is_integer($key) && $key != 0) ? array('disabled') : array();
            $select->addOption($type, $key, $attributes);
        }
        $this->addElement('category');

        // Extra parameters
        $type_filter = ContentObjectAdvancedFilter :: factory($this->get_type(), $this);

        if ($type_filter)
        {
            $type_filter->add_elements();
        }

        $this->addElement(
            'html',
            ResourceManager :: get_instance()->get_resource_html(
                Path :: namespace_to_full_path(RepositoryManager :: context(), true) . 'resources/javascript/search.js'));
        $this->addElement(
            'style_submit_button',
            'submit',
            Translation :: get('Search', array(), Utilities :: COMMON_LIBRARIES),
            array('class' => 'normal search'));
    }

    public function get_allowed_content_object_types()
    {
        $types = $this->manager->get_content_object_types(true, false);
        foreach ($types as $index => $type)
        {
            $registration = CommonDataManager :: get_registration(Utilities :: get_namespace_from_classname($type));
            if (! $registration || ! $registration->is_active())
            {
                unset($types[$index]);
            }
        }

        return $types;
    }

    public function get_type()
    {
        $session_filter = Session :: retrieve('advanced_filter');
        $html = array();
        if ($this->validate() || isset($session_filter))
        {
            if ($this->validate())
            {
                $values = $this->exportValues();
            }
            else
            {
                $values = unserialize(Session :: retrieve('advanced_filter'));
            }
            return $values[ContentObject :: PROPERTY_TYPE];
        }
        else
        {
            return false;
        }
    }

    public function get_filter_parameters()
    {
        $session_filter = Session :: retrieve('advanced_filter');
        $html = array();
        if ($this->validate() || isset($session_filter))
        {
            if ($this->validate())
            {
                $values = $this->exportValues();
            }
            else
            {
                $values = unserialize(Session :: retrieve('advanced_filter'));
            }

            $text = $values[self :: FILTER_TEXT];
            $type = $values[ContentObject :: PROPERTY_TYPE];

            $has_basic_parameters = ($text ||
                 ($values['creation_date_from']['created'] || $values['creation_date_to']['created']) ||
                 ($values['modification_date_from']['modified'] || $values['modification_date_to']['modified']) || (! is_numeric(
                    $type) && ! empty($type)));

            if ($has_basic_parameters)
            {
                $basic_html = array();

                if ($text)
                {
                    $basic_html[] = '<div class="parameter" id="parameter_general_text">' . $text . '</div>';
                }

                if ($values['creation_date_from']['created'] && $values['creation_date_to']['created'])
                {
                    $basic_html[] = '<div class="parameter" id="parameter_general_creation_dates">' . Translation :: get(
                        'CreatedBetween',
                        array(
                            'FROM' => $values['creation_date_from']['created'],
                            'TO' => $values['creation_date_to']['created'])) . '</div>';
                }
                else
                {
                    if ($values['creation_date_from']['created'])
                    {
                        $basic_html[] = '<div class="parameter" id="parameter_general_creation_date_from">' . Translation :: get(
                            'CreatedAfter',
                            array('FROM' => $values['creation_date_from']['created'])) . '</div>';
                    }
                    elseif ($values['creation_date_to']['created'])
                    {
                        $basic_html[] = '<div class="parameter" id="parameter_general_creation_date_to">' . Translation :: get(
                            'CreatedBefore',
                            array('TO' => $values['creation_date_to']['created'])) . '</div>';
                    }
                }

                if ($values['modification_date_from']['modified'] && $values['modification_date_to']['modified'])
                {
                    $basic_html[] = '<div class="parameter" id="parameter_general_modification_dates">' . Translation :: get(
                        'ModifiedBetween',
                        array(
                            'FROM' => $values['modification_date_from']['modified'],
                            'TO' => $values['modification_date_to']['modified'])) . '</div>';
                }
                else
                {
                    if ($values['modification_date_from']['modified'])
                    {
                        $basic_html[] = '<div class="parameter" id="parameter_general_modification_date_from">' . Translation :: get(
                            'ModifiedAfter',
                            array('FROM' => $values['modification_date_from']['modified'])) . '</div>';
                    }
                    elseif ($values['modification_date_to']['modified'])
                    {
                        $basic_html[] = '<div class="parameter" id="parameter_general_modification_date_to">' . Translation :: get(
                            'ModifiedBefore',
                            array('TO' => $values['modification_date_to']['modified'])) . '</div>';
                    }
                }

                if (! is_numeric($type) && ! empty($type))
                {
                    $basic_html[] = '<div class="parameter" id="parameter_general_type">' . Translation :: get(
                        'TypeName',
                        null,
                        Utilities :: get_namespace_from_classname($type)) . '</div>';
                }

                $basic_html = implode("\n", $basic_html);
            }

            // Extra parameters
            $type_filter = ContentObjectAdvancedFilter :: factory($this->get_type(), $this);

            if ($type_filter)
            {
                $extra_html = $type_filter->get_parameters();
            }

            if ($basic_html || $extra_html)
            {
                $html[] = '<div id="search_parameters"><h4>' . Translation :: get('SearchParameters') . '</h4>';
                $html[] = $basic_html;
                $html[] = $extra_html;
                $html[] = '<div class="parameter" id="parameter_general_all">' . Translation :: get(
                    'ClearAllParameters') . '</div>';
                $html[] = '<div class="clear"></div>';
                $html[] = '</div>';
            }
        }

        return implode("\n", $html);
    }

    public function get_filter_conditions()
    {
        $session_filter = Session :: retrieve('advanced_filter');
        if ($this->validate() || isset($session_filter))
        {
            if ($this->validate())
            {
                $values = $this->exportValues();
            }
            else
            {
                $values = unserialize(Session :: retrieve('advanced_filter'));
            }

            $filter = serialize($values);

            if ($this->validate())
            {
                Session :: register('advanced_filter', $filter);
            }

            $conditions = array();
            // Text
            $text = $values[self :: FILTER_TEXT];
            if ($text)
            {
                if ($this->get_type())
                {
                    $class = $this->get_type();
                    $searchable_property_names = $class :: get_searchable_property_names();
                }
                else
                {
                    $searchable_property_names = ContentObject :: get_searchable_property_names();
                }

                $text_conditions = array();
                $text_conditions[] = new PatternMatchCondition(ContentObject :: PROPERTY_TITLE, '*' . $text . '*');
                $text_conditions[] = new PatternMatchCondition(ContentObject :: PROPERTY_DESCRIPTION, '*' . $text . '*');

                foreach ($searchable_property_names as $searchable_property_name)
                {
                    $text_conditions[] = new PatternMatchCondition(
                        $searchable_property_name,
                        '*' . $text . '*',
                        DataManager :: get_alias_type_table(),
                        true);
                }

                $words = explode(' ', $text);
                if (count($words) > 1)
                {
                    foreach ($words as $word)
                    {
                        $text_conditions[] = new PatternMatchCondition(
                            ContentObject :: PROPERTY_TITLE,
                            '*' . $word . '*');
                        $text_conditions[] = new PatternMatchCondition(
                            ContentObject :: PROPERTY_DESCRIPTION,
                            '*' . $word . '*');

                        foreach ($searchable_property_names as $searchable_property_name)
                        {
                            $text_conditions[] = new PatternMatchCondition(
                                $searchable_property_name,
                                '*' . $word . '*',
                                DataManager :: get_alias_type_table(),
                                true);
                        }
                    }
                }
                $conditions[] = new OrCondition($text_conditions);
            }

            // Type
            $type = $values[ContentObject :: PROPERTY_TYPE];
            if (! is_numeric($type) && ! empty($type))
            {
                $conditions[] = new EqualityCondition(
                    ContentObject :: PROPERTY_TYPE,
                    $type,
                    ContentObject :: get_table_name());
            }

            // CreationDate
            if ($values['creation_date_from']['created'] && $values['creation_date_to']['created'])
            {
                $creation_date_conditions = array();
                $creation_date_conditions[] = new InequalityCondition(
                    ContentObject :: PROPERTY_CREATION_DATE,
                    InequalityCondition :: GREATER_THAN_OR_EQUAL,
                    strtotime($values['creation_date_from']['created']));
                $creation_date_conditions[] = new InequalityCondition(
                    ContentObject :: PROPERTY_CREATION_DATE,
                    InequalityCondition :: LESS_THAN_OR_EQUAL,
                    strtotime($values['creation_date_to']['created']));
                $conditions[] = new AndCondition($creation_date_conditions);
            }
            else
            {
                if ($values['creation_date_from']['created'])
                {
                    $conditions[] = new InequalityCondition(
                        ContentObject :: PROPERTY_CREATION_DATE,
                        InequalityCondition :: GREATER_THAN_OR_EQUAL,
                        strtotime($values['creation_date_from']['created']));
                }
                elseif ($values['creation_date_to']['created'])
                {
                    $conditions[] = new InequalityCondition(
                        ContentObject :: PROPERTY_CREATION_DATE,
                        InequalityCondition :: LESS_THAN_OR_EQUAL,
                        strtotime($values['creation_date_to']['created']));
                }
            }

            // ModificationDate
            if ($values['modification_date_from']['modified'] && $values['modification_date_to']['modified'])
            {
                $modification_date_conditions = array();
                $modification_date_conditions[] = new InequalityCondition(
                    ContentObject :: PROPERTY_CREATION_DATE,
                    InequalityCondition :: GREATER_THAN_OR_EQUAL,
                    strtotime($values['modification_date_from']['modified']));
                $modification_date_conditions[] = new InequalityCondition(
                    ContentObject :: PROPERTY_CREATION_DATE,
                    InequalityCondition :: LESS_THAN_OR_EQUAL,
                    strtotime($values['modification_date_to']['modified']));
                $conditions[] = new AndCondition($modification_date_conditions);
            }
            else
            {
                if ($values['modification_date_from']['modified'])
                {
                    $conditions[] = new InequalityCondition(
                        ContentObject :: PROPERTY_CREATION_DATE,
                        InequalityCondition :: GREATER_THAN_OR_EQUAL,
                        strtotime($values['modification_date_from']['modified']));
                }
                elseif ($values['modification_date_to']['modified'])
                {
                    $conditions[] = new InequalityCondition(
                        ContentObject :: PROPERTY_CREATION_DATE,
                        InequalityCondition :: LESS_THAN_OR_EQUAL,
                        strtotime($values['modification_date_to']['modified']));
                }
            }
            // Extra parameters conditions
            $type_filter = ContentObjectAdvancedFilter :: factory($this->get_type(), $this);
            if ($type_filter)
            {
                foreach ($type_filter->get_conditions() as $condition)
                {
                    $conditions[] = $condition;
                }
            }

            return new AndCondition($conditions);
        }
    }

    public function setDefaults($defaults = array ())
    {
        $values = unserialize(Session :: retrieve('advanced_filter'));
        $defaults[self :: FILTER_TEXT] = $values[self :: FILTER_TEXT];
        $defaults[ContentObject :: PROPERTY_TYPE] = $values[ContentObject :: PROPERTY_TYPE];
        $defaults['creation_date_from']['created'] = $values['creation_date_from']['created'];
        $defaults['creation_date_to']['created'] = $values['creation_date_to']['created'];
        $defaults['modification_date_from']['modified'] = $values['modification_date_from']['modified'];
        $defaults['modification_date_to']['modified'] = $values['modification_date_to']['modified'];
        parent :: setDefaults($defaults);
    }

    /**
     * Display the form
     */
    public function display()
    {
        $html = array();
        // $html [] = '<div style="text-align: right;">';
        $html[] = $this->renderer->toHtml();
        // $html [] = '</div>';
        return implode('', $html);
    }
}
