<?php
namespace repository;

use common\libraries\FormValidator;
use common\libraries\Translation;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;

/**
 * $Id: user_view_form.class.php 200 2009-11-13 12:30:04Z kariboe $
 * 
 * @package repository.lib.forms
 */
class UserViewForm extends FormValidator
{
    const TYPE_CREATE = 1;
    const TYPE_EDIT = 2;

    private $user_view;

    private $form_type;

    public function __construct($form_type, $user_view, $action)
    {
        parent :: __construct('user_views_settings', 'post', $action);
        
        $this->user_view = $user_view;
        
        $this->form_type = $form_type;
        if ($this->form_type == self :: TYPE_EDIT)
        {
            $this->build_editing_form();
        }
        elseif ($this->form_type == self :: TYPE_CREATE)
        {
            $this->build_creation_form();
        }
        
        $this->setDefaults();
    }

    public function build_basic_form()
    {
        $this->addElement('text', UserView :: PROPERTY_NAME, 
                Translation :: get('Name', null, Utilities :: COMMON_LIBRARIES), array("size" => "50"));
        $this->addRule(UserView :: PROPERTY_NAME, 
                Translation :: get('ThisFieldIsRequired', null, Utilities :: COMMON_LIBRARIES), 'required');
        $this->add_html_editor(UserView :: PROPERTY_DESCRIPTION, 
                Translation :: get('Description', null, Utilities :: COMMON_LIBRARIES), false);
        
        $registrations = DataManager :: get_registered_types();
        $hidden_types = DataManager :: get_active_helper_types();
        
        foreach ($registrations as $registration)
        {
            if (in_array($registration, $hidden_types))
                continue;
            $content_object_types[$registration] = Translation :: get('TypeName', null, 
                    Utilities :: get_namespace_from_classname($registration));
        }
        
        if ($this->form_type == self :: TYPE_EDIT)
        {
            
            $uvrlo = DataManager :: retrieves(UserViewRelContentObject :: class_name(), 
                    new EqualityCondition(UserViewRelContentObject :: PROPERTY_VIEW_ID, $this->get_user_view()->get_id()));
            while ($type = $uvrlo->next_result())
            {
                $content_object_types[$type->get_content_object_type()] = Translation :: get('TypeName', null, 
                        Utilities :: get_namespace_from_classname($type->get_content_object_type()));
                $defaults[] = $type->get_content_object_type();
            }
        }
        
        $this->addElement('multiselect', 'types', Translation :: get('SelectTypesToShow'), $content_object_types, 
                array('style' => 'width:300px; height: 300px'));
        
        $this->setDefaults(array('types' => $defaults));
        
        // $this->addElement('submit', 'user_view_settings', 'OK');
    }

    public function build_editing_form()
    {
        $user_view = $this->user_view;
        $this->build_basic_form();
        
        $this->addElement('hidden', UserView :: PROPERTY_ID);
        
        $buttons[] = $this->createElement('style_submit_button', 'submit', 
                Translation :: get('Update', null, Utilities :: COMMON_LIBRARIES), array('class' => 'positive update'));
        $buttons[] = $this->createElement('style_reset_button', 'reset', 
                Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array('class' => 'normal empty'));
        
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function build_creation_form()
    {
        $this->build_basic_form();
        
        $buttons[] = $this->createElement('style_submit_button', 'submit', 
                Translation :: get('Create', null, Utilities :: COMMON_LIBRARIES), array('class' => 'positive'));
        $buttons[] = $this->createElement('style_reset_button', 'reset', 
                Translation :: get('Reset', null, Utilities :: COMMON_LIBRARIES), array('class' => 'normal empty'));
        
        $this->addGroup($buttons, 'buttons', null, '&nbsp;', false);
    }

    public function update_user_view()
    {
        $user_view = $this->user_view;
        $values = $this->exportValues();
        
        $user_view->set_name($values[UserView :: PROPERTY_NAME]);
        $user_view->set_description($values[UserView :: PROPERTY_DESCRIPTION]);
        
        $condition = new EqualityCondition(UserViewRelContentObject :: PROPERTY_VIEW_ID, $user_view->get_id());
        
        $types = DataManager :: retrieves(UserViewRelContentObject :: class_name(), $condition);
        $existing_types = array();
        while ($type = $types->next_result())
        {
            $existing_types[] = $type->get_content_object_type();
        }
        
        $new_types = $values['types'];
        $to_add = array_diff($new_types, $existing_types);
        $to_delete = array_diff($existing_types, $new_types);
        
        foreach ($to_add as $type_to_add)
        {
            $user_view_type = new UserViewRelContentObject();
            $user_view_type->set_view_id($user_view->get_id());
            $user_view_type->set_content_object_type($type_to_add);
            $user_view_type->create();
        }
        
        $types->reset();
        while ($type = $types->next_result())
        {
            if (in_array($type->get_content_object_type(), $to_delete))
            {
                $type->delete();
            }
        }
        
        return $user_view->update();
    }

    public function create_user_view()
    {
        $user_view = $this->user_view;
        $values = $this->exportValues();
        
        $user_view->set_name($values[UserView :: PROPERTY_NAME]);
        $user_view->set_description($values[UserView :: PROPERTY_DESCRIPTION]);
        
        $value = $user_view->create($values['types']);
        
        return $value;
    }

    /**
     * Sets default values.
     * 
     * @param array $defaults Default values for this form's parameters.
     */
    public function setDefaults($defaults = array ())
    {
        $user_view = $this->user_view;
        $defaults[UserView :: PROPERTY_ID] = $user_view->get_id();
        $defaults[UserView :: PROPERTY_NAME] = $user_view->get_name();
        $defaults[UserView :: PROPERTY_DESCRIPTION] = $user_view->get_description();
        parent :: setDefaults($defaults);
    }

    public function get_user_view()
    {
        return $this->user_view;
    }
}
