<?php
namespace repository;

use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\Utilities;
use common\libraries\Toolbar;

/**
 * $Id: repository_browser_table_cell_renderer.class.php 204 2009-11-13 12:51:30Z kariboe $
 * 
 * @package repository.lib.repository_manager.component.browser
 */

/**
 * Cell rendere for the object browser table
 */
class RepositoryBrowserTableCellRenderer extends DefaultContentObjectTableCellRenderer
{

    /**
     * The repository browser component
     */
    private $browser;

    /**
     * Constructor
     * 
     * @param RepositoryManagerBrowserComponent $browser
     */
    public function __construct($browser)
    {
        parent :: __construct();
        $this->browser = $browser;
    }
    
    // Inherited
    public function render_cell($column, $content_object)
    {
        if ($column === RepositoryBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($content_object);
        }
        
        switch ($column->get_name())
        {
            case Theme :: get_common_image('action_category', 'png', Translation :: get('Type'), null, 
                    ToolbarItem :: DISPLAY_ICON) :
                $type = $content_object->get_type();
                return '<a href="' . Utilities :: htmlentities($this->browser->get_type_filter_url($type)) . '" title="' . htmlentities(
                        $content_object->get_type_string()) . '">' . parent :: render_cell($column, $content_object) . '</a>';
            case ContentObject :: PROPERTY_TITLE :
                $title = parent :: render_cell($column, $content_object);
                $title_short = Utilities :: truncate_string($title, 53, false);
                return '<a href="' . Utilities :: htmlentities(
                        $this->browser->get_content_object_viewing_url($content_object)) . '" title="' . $title . '">' . $title_short . '</a>';
        }
        return parent :: render_cell($column, $content_object);
    }

    /**
     * Gets the action links to display
     * 
     * @param ContentObject $content_object The object for which the action links should be returned
     * @return string A HTML representation of the action links
     */
    private function get_modification_links($content_object)
    {
        $toolbar = new Toolbar();
        $toolbar->add_items($this->browser->get_content_object_actions($content_object));
        return $toolbar->as_html();
    }
}
