<?php
namespace repository;

use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Application;
use common\libraries\DelegateComponent;
use common\libraries\Utilities;

/**
 * $Id: complex_builder.class.php 204 2009-11-13 12:51:30Z kariboe $
 *
 * @package repository.lib.repository_manager.component
 */

/**
 * Component to build complex content object items
 *
 * @author vanpouckesven
 */
class RepositoryManagerBuilderComponent extends RepositoryManager implements DelegateComponent
{
    const PARAM_POPUP = 'popup';

    private $content_object;

    public function display_header()
    {
        $is_popup = Request :: get(self :: PARAM_POPUP);
        if ($is_popup)
        {
            $this->small_header();
        }
        else
        {
            parent :: display_header();
        }
    }

    public function display_footer()
    {
        $is_popup = Request :: get(self :: PARAM_POPUP);
        if ($is_popup)
        {
            $this->small_footer();
        }
        else
        {
            parent :: display_footer();
        }
    }

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $this->set_parameter(self :: PARAM_POPUP, Request :: get(self :: PARAM_POPUP));
        $content_object_id = Request :: get(RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
        $this->content_object = $this->retrieve_content_object($content_object_id);

        // $type = Request :: get(RepositoryManager :: PARAM_TYPE);
        // $this->set_parameter(RepositoryManager :: PARAM_TYPE, $type);
        //
        // if($type)
        // {
        // ComplexBuilder :: launch($type, $this);
        // //$complex_builder = ComplexBuilder :: factory($this, $this->content_object->get_type());
        // //$complex_builder->run();
        // }

        if ($this->content_object)
        {
            BreadcrumbTrail :: get_instance()->add(
                new Breadcrumb(
                    $this->get_url(
                        array(
                            RepositoryManager :: PARAM_ACTION => RepositoryManager :: ACTION_BUILD_COMPLEX_CONTENT_OBJECT)),
                    Translation :: get(
                        'BuildContentObject',
                        array('CONTENT_OBJECT' => $this->content_object->get_title()))));
            ComplexBuilder :: launch($this->content_object->get_type(), $this);

            // $complex_builder = ComplexBuilder :: factory($this, $this->content_object->get_type());
            // $complex_builder->run();
        }
        else
        {
            $this->display_error_page(
                Translation :: get(
                    'NoObjectSelected',
                    array('OBJECT' => Translation :: get('ContentObject')),
                    Utilities :: COMMON_LIBRARIES));
        }
    }

    public function get_root_content_object()
    {
        return $this->content_object;
    }

    public function redirect_away_from_complex_builder($message, $error_message)
    {
        $this->redirect(
            $message,
            $error_message,
            array(Application :: PARAM_ACTION => RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS));
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('repository_complex_builder');
    }

    public function get_additional_parameters()
    {
        return array(RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
    }
}
