<?php
namespace repository;

use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Utilities;
use common\libraries\Display;
/**
 * $Id: content_object_copier.class.php 204 2009-11-13 12:51:30Z kariboe $
 * @package repository.lib.repository_manager.component
 */

class RepositoryManagerContentObjectCopierComponent extends RepositoryManager
{

    /**
     * Runs this component and displays its output.
     */
    function run()
    {
        $lo_ids = Request :: get(RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
        $rename = Request :: get(RepositoryManager :: PARAM_RENAME);
        
        if (! $lo_ids)
        {
            Display :: error_page(Translation :: get('NoObjectSelected', null, Utilities :: COMMON_LIBRARIES));
        }
        
        $target_user = Request :: get(RepositoryManager :: PARAM_TARGET_USER);
        
        if (! is_array($lo_ids))
        {
            $lo_ids = array($lo_ids);
        }
        
        if (count($lo_ids) == 0 || ! isset($target_user))
        {
            Display :: error_page(Translation :: get('ContentObjectAndTargetUserRequired'));
        }
        
        $content_object_copier = new ContentObjectCopier($target_user);
        
        $failures = 0;
        
        foreach ($lo_ids as $lo_id)
        {
            // retrieve object
            $lo = $this->retrieve_content_object($lo_id);
            
            // fail if no object exists
            if (! $lo)
            {
                $failures ++;
                continue;
            }
            
            // check for copy right
            if (! ($lo->get_owner_id() == $this->get_user_id() || $lo->get_owner_id() == 0 || RepositoryRights :: get_instance()->is_allowed_in_user_subtree(RepositoryRights :: COPY_RIGHT, $lo->get_id(), RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $lo->get_owner_id())))
            {
                $failures ++;
                continue;
            }
            
            /**
             * If it is copied to another user 's repo, that user doestn have the same categories
             */
            if ($target_user != $lo->get_owner_id())
            {
                $category_id = 0;
            }
            else
            {
                $category_id = $lo->get_parent_id();
            }
            
            // copy
            if (! $content_object_copier->copy_content_object($lo, $category_id, $rename))
            {
                $failures ++;
            }
        }
        
        $success = ($failures == 0);
        
        if (count($lo_ids) > 1)
        {
            $message = Translation :: get(($success ? 'ObjectsCopied' : 'ObjectsNotCopied'), array(
                    'OBJECTS' => Translation :: get('ContentObjects')), Utilities :: COMMON_LIBRARIES);
        }
        else
        {
            $message = Translation :: get(($success ? 'ObjectCopied' : 'ObjectNotCopied'), array(
                    'OBJECTS' => Translation :: get('ContentObject')), Utilities :: COMMON_LIBRARIES);
        }
        
        $this->redirect($message, ($success ? false : true), array(RepositoryManager :: PARAM_ACTION => null));
    
    }

    function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(
                RepositoryManager :: PARAM_ACTION => RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS)), Translation :: get('RepositoryManagerBrowserComponent')));
        $breadcrumbtrail->add_help('repository_content_object_copier');
    }

    function get_additional_parameters()
    {
        return array(RepositoryManager :: PARAM_CONTENT_OBJECT_ID, RepositoryManager :: PARAM_TARGET_USER);
    }

}
?>