<?php
namespace repository;

use repository\quota\Calculator;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\DelegateComponent;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Utilities;
use common\libraries\Application;
use common\libraries\ComplexContentObjectSupport;
use common\libraries\CommonDataManager;

/**
 * $Id: creator.class.php 204 2009-11-13 12:51:30Z kariboe $
 *
 * @package repository.lib.repository_manager.component
 */
/**
 * Repository manager component which gives the user the possibility to create a new content object in his repository.
 * When no type is passed to this component, the user will see a dropdown list in which a content object type can be
 * selected. Afterwards, the form to create the actual content object will be displayed.
 */
class RepositoryManagerCreatorComponent extends RepositoryManager implements DelegateComponent
{
    const TAB_MOST_USED = 'most_used';

    private $type;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        // $aform = new Form('form1');
        // $aform->add_element(new TextArea('title', 'Titel 1', 'Title in Form
        // 1'));

        // $group = new Fieldset('person', 'Person');

        // $name_group = new Grouping('name', 'Name', true);
        // $name_group->add_element(new Text('first', null, 'Hans'));
        // $name_group->add_element(new Text('last', null, 'De Bisschop'));

        // $test_group = new Fieldset('test', 'Test');
        // $test_group->add_element(new Text('alpha', null, 'Alpha'));
        // $test_group->add_element(new Text('beta', null, 'Beta'));
        // $name_group->add_element($test_group);

        // $another_group = new Grouping('another', 'Another', true);
        // $another_group->add_element(new Text('mcduck', null, 'Scrooge'));
        // $another_group->add_element(new Text('glomgold', null,
        // 'Flintheart'));
        // $test_group->add_element($another_group);

        // $group->add_element($name_group);

        // $aform->add_element($group);

        // $buttons = new Grouping('buttons', '', false);
        // $buttons->add_element(new StyleSubmitButton('submit', 'Submit'));
        // $buttons->add_element(new StyleResetbutton('reset', 'Reset'));

        // $aform->add_element($buttons);

        // $bform = new Form('form2');
        // $bform->add_element(new TextArea('title', 'Titel 2', 'Title in Form
        // 2'));

        // $mform = new MergedForm(array($aform, $bform));

        // $this->display_header();
        // echo $mform->render();
        // $this->display_footer();

        // exit();

        $type_selector = new ContentObjectTypeSelector($this, $this->get_allowed_content_object_types(), array());

        $this->type = ($type_selector->get_selection() ? $type_selector->get_selection() : Request :: get(
                ContentObjectTypeSelector :: PARAM_CONTENT_OBJECT_TYPE));

        if ($this->type)
        {
            BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb(
                            $this->get_url(
                                    array(
                                            RepositoryManager :: PARAM_ACTION => RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS)),
                            Translation :: get('CreateContentType',
                                    array(
                                            'OBJECTTYPE' => strtolower(
                                                    Translation :: get('TypeName', null,
                                                            Utilities :: get_namespace_from_classname($this->type))),
                                            'ICON' => Theme :: get_image('logo/16', 'png',
                                                    Translation :: get('TypeName', null,
                                                            Utilities :: get_namespace_from_classname($this->type)), null,
                                                    ToolbarItem :: DISPLAY_ICON, false,
                                                    Utilities :: get_namespace_from_classname($this->type))))));
            $category = Request :: get(RepositoryManager :: PARAM_CATEGORY_ID);

            $object = new $this->type();

            $object->set_owner_id($this->get_user_id());
            $object->set_parent_id($category);

            $content_object_form = ContentObjectForm :: factory(ContentObjectForm :: TYPE_CREATE, $object, 'create',
                    'post', $this->get_url(array(RepositoryManager :: PARAM_CONTENT_OBJECT_TYPE => $this->type)), null);

            if ($content_object_form->validate())
            {
                $object = $content_object_form->create_content_object();

                if (! $object)
                {
                    $this->redirect(
                            Translation :: get('ObjectNotCreated',
                                    array('OBJECT' => Translation :: get('ContentObject')),
                                    Utilities :: COMMON_LIBRARIES), true,
                            array(
                                    RepositoryManager :: PARAM_ACTION => RepositoryManager :: ACTION_CREATE_CONTENT_OBJECTS,
                                    'type' => $this->type));
                }

                if (! is_array($object) && $object instanceof ComplexContentObjectSupport)
                {
                    $parameters = array(
                            Application :: PARAM_ACTION => RepositoryManager :: ACTION_BUILD_COMPLEX_CONTENT_OBJECT,
                            RepositoryManager :: PARAM_CONTENT_OBJECT_ID => $object->get_id());
                    $filter = array('category');
                    $this->redirect(null, false, $parameters, $filter);
                }
                else
                {
                    if (is_array($object))
                    {
                        $parent = $object[0]->get_parent_id();
                        $type_name = $object[0]->get_type_name();
                    }
                    else
                    {
                        $parent = $object->get_parent_id();
                        $type_name = $object->get_type_name();
                    }

                    $parameters = array();
                    $parameters[Application :: PARAM_ACTION] = RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS;
                    $parameters[RepositoryManager :: PARAM_CATEGORY_ID] = $parent;

                    $this->redirect(
                            Translation :: get('ObjectCreated',
                                    array(
                                            'OBJECT' => Translation :: get('TypeName', null,
                                                    ContentObject :: get_content_object_type_namespace($type_name))),
                                    Utilities :: COMMON_LIBRARIES), false, $parameters);
                }
            }
            else
            {
                if (! Request :: get('publish'))
                {
                    $this->display_header();
                }
                else
                {
                    $this->display_header();
                }

                $content_object_form->display();
                $this->display_footer();
            }
        }
        else
        {
            if (Request :: get('publish'))
            {
                $this->display_header();
            }
            else
            {
                $this->display_header();
            }
            echo $type_selector->as_html();
            $this->display_footer();
        }
    }

    public function get_content_object_type_creation_url($type)
    {
        return $this->get_url(array(RepositoryManager :: PARAM_CONTENT_OBJECT_TYPE => $type));
    }

    public function get_allowed_content_object_types()
    {
        $types = $this->get_content_object_types(true, false);

        foreach ($types as $index => $type)
        {
            $registration = CommonDataManager :: get_registration(Utilities :: get_namespace_from_classname($type));
            if (! $registration || ! $registration->is_active())
            {
                unset($types[$index]);
            }
        }

        return $types;
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('repository_creator');
    }
}
