<?php
namespace repository;

use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\DelegateComponent;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Application;
use common\libraries\Utilities;

/**
 * $Id: editor.class.php 204 2009-11-13 12:51:30Z kariboe $
 * 
 * @package repository.lib.repository_manager.component
 */
/**
 * Repository manager component to edit an existing content object.
 */
class RepositoryManagerEditorComponent extends RepositoryManager implements DelegateComponent
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $id = Request :: get(RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
        if ($id)
        {
            $object = $this->retrieve_content_object($id);
            
            if (! $object)
            {
                $this->display_error_page(Translation :: get('NoObjectSelected', null, Utilities :: COMMON_LIBRARIES));
                exit();
            }
            
            BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb(null, 
                            Translation :: get('EditContentObject', 
                                    array('CONTENT_OBJECT' => $object->get_title(), 
                                            'ICON' => Theme :: get_image('logo/16', 'png', 
                                                    Translation :: get('TypeName', null, 
                                                            Utilities :: get_namespace_from_classname(
                                                                    $object->get_type())), null, 
                                                    ToolbarItem :: DISPLAY_ICON, false, 
                                                    Utilities :: get_namespace_from_classname($object->get_type()))), 
                                    RepositoryManager :: context())));
            
            if (! ($object->get_owner_id() == $this->get_user_id() || RepositoryRights :: get_instance()->is_allowed_in_user_subtree(
                    RepositoryRights :: COLLABORATE_RIGHT, $object->get_id(), 
                    RepositoryRights :: TYPE_USER_CONTENT_OBJECT, $object->get_owner_id())))
            {
                $this->not_allowed();
            }
            elseif (! $object->is_latest_version())
            {
                $parameters = array();
                $parameters[Application :: PARAM_ACTION] = RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS;
                $parameters[RepositoryManager :: PARAM_CATEGORY_ID] = $object->get_parent_id();
                
                $this->redirect(Translation :: get('EditNotAllowed'), true, $parameters);
            }
            
            if (! DataManager :: is_content_object_editable($object->get_id()))
            {
                $parameters = array();
                $parameters[Application :: PARAM_ACTION] = RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS;
                $parameters[RepositoryManager :: PARAM_CATEGORY_ID] = $object->get_parent_id();
                $this->redirect(Translation :: get('UpdateNotAllowed'), false, $parameters);
            }
            
            $form = ContentObjectForm :: factory(ContentObjectForm :: TYPE_EDIT, $object, 'edit', 'post', 
                    $this->get_url(array(RepositoryManager :: PARAM_CONTENT_OBJECT_ID => $id)));
            if ($form->validate())
            {
                
                $success = $form->update_content_object();
                $category_id = $object->get_parent_id();
                
                $parameters = array();
                $parameters[Application :: PARAM_ACTION] = RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS;
                $parameters[RepositoryManager :: PARAM_CATEGORY_ID] = $category_id;
                
                $this->redirect(
                        Translation :: get(
                                $success == ContentObjectForm :: RESULT_SUCCESS ? 'ObjectUpdated' : 'ObjectNotUpdated', 
                                array('OBJECT' => Translation :: get('ContentObject')), Utilities :: COMMON_LIBRARIES), 
                        ($success == ContentObjectForm :: RESULT_SUCCESS ? false : true), $parameters);
            }
            else
            {
                $this->display_header();
                $form->display();
                $this->display_footer();
            }
        }
        else
        {
            $this->display_error_page(
                    htmlentities(
                            Translation :: get('NoObjectSelected', 
                                    array('OBJECT' => Translation :: get('ContentObject')), 
                                    Utilities :: COMMON_LIBRARIES)));
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('repository_editor');
    }

    public function get_additional_parameters()
    {
        return array(RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
    }
}
