<?php
namespace repository;

use common\libraries\Utilities;
use common\libraries\Header;
use common\libraries\DelegateComponent;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\extensions\video_conferencing_manager\ConferenceRights;

class RepositoryManagerExternalInstanceComponent extends RepositoryManager implements DelegateComponent
{

    private $external_instance;

    public function run()
    {
        Header :: set_section('external_repository');
        $trail = BreadcrumbTrail :: get_instance();
        $trail->remove($trail->size() - 1);
        $trail->add(
                new Breadcrumb($this->get_url(), 
                        Translation :: get('ExternalInstance', null, ExternalInstanceManager :: get_namespace())));

        $external_instance_id = Request :: get(self :: PARAM_EXTERNAL_INSTANCE);
        $this->set_parameter(self :: PARAM_EXTERNAL_INSTANCE, $external_instance_id);
        $this->external_instance = \repository\DataManager :: retrieve_by_id(ExternalInstance :: class_name(), 
                $external_instance_id);
        
        if ($this->external_instance instanceof ExternalInstance && $this->external_instance->is_enabled())
        {
            $manager = Utilities :: get_namespace_parent(
                    Utilities :: get_namespace_parent($this->get_external_instance()->get_type())) . '\Manager';
            $manager :: launch($this);
        }
        else
        {
            $this->display_header();
            $this->display_error_message('NoSuchExternalInstanceManager');
            $this->display_footer();
        }
    }

    public function get_external_instance()
    {
        return $this->external_instance;
    }

    public function get_video_conferencing_rights()
    {
        $rights = ConferenceRights :: factory($this->get_external_instance()->get_type());
        
        $rights->set_moderator(true);
        
        return $rights;
    }
}
