<?php
namespace repository;

use common\libraries\Display;
use common\libraries\Utilities;
use common\libraries\Theme;
use common\libraries\Application;
use common\libraries\Path;
use common\libraries\Translation;
use common\libraries\Request;
use common\extensions\external_repository_manager\Manager;

class ExternalRepositoryComponent extends RepositoryManager
{

    private $external_instance;

    public function run()
    {
        $type = $this->get_type();
        $this->external_instance = \repository\DataManager :: retrieve_by_id(ExternalInstance :: class_name(), $type);
        $this->set_parameter(Manager :: PARAM_EXTERNAL_REPOSITORY, $type);

        Manager :: launch($this);
    }

    public function get_type()
    {
        return Request :: get(RepositoryManager :: PARAM_EXTERNAL_INSTANCE);
    }

    public function get_external_instance()
    {
        return $this->external_instance;
    }

    public function display_header()
    {
        $this->small_header();
    }

    public function display_footer()
    {
        $this->small_footer();
    }

    /**
     * Get a series of links for all external repository instances of one or more types
     *
     * @param $types array An array of external repository manager types
     * @param $auto_open unknown_type if there is only one instance, should it be opened automatically
     * @return string
     */
    public static function get_links($manager_types = array(), $types = array(), $auto_open = false)
    {
        $instances = \repository\DataManager :: retrieve_active_external_instances($manager_types, $types);

        if ($instances->size() == 0)
        {
            if (! is_array($types))
            {
                $types = array($types);
            }

            $type_names = array();
            foreach ($types as $type)
            {
                $type_names[] = Translation :: get('TypeName', null, Manager :: get_namespace($type));
            }
            $type_names = implode(', ', $type_names);

            if (count($types) > 1)
            {
                $translation = Translation :: get(
                    'NoExternalInstanceTypeManagersAvailable',
                    array('TYPES' => $type_names),
                    RepositoryManager :: APPLICATION_NAME);
            }
            else
            {
                $translation = Translation :: get(
                    'NoExternalInstanceTypeManagerAvailable',
                    array('TYPES' => $type_names),
                    RepositoryManager :: APPLICATION_NAME);
            }

            return Display :: warning_message($translation, true);
        }
        else
        {
            $html = array();
            $buttons = array();

            $available_instances = 0;

            while ($instance = $instances->next_result())
            {
                if (RepositoryRights :: get_instance()->is_allowed_in_external_instances_subtree(
                    RepositoryRights :: USE_RIGHT,
                    $instance->get_id()))
                {
                    $link = Path :: get(WEB_PATH) . 'index.php?' . Application :: PARAM_APPLICATION . '=' .
                         self :: APPLICATION_NAME . '&' . Application :: PARAM_ACTION . '=' .
                         self :: ACTION_EXTERNAL_REPOSITORY . '&' . RepositoryManager :: PARAM_EXTERNAL_INSTANCE . '=' .
                         $instance->get_id();
                    $image = Theme :: get_image_path($instance->get_type()) . 'logo/16.png';
                    $title = Translation :: get(
                        'BrowseObject',
                        array('OBJECT' => $instance->get_title()),
                        Utilities :: COMMON_LIBRARIES);
                    $buttons[] = '<a class="button normal_button upload_button" style="background-image: url(' .
                         htmlspecialchars($image) . ');" onclick="javascript:openPopup(\'' . htmlspecialchars($link) .
                         '\');"> ' . htmlspecialchars($title) . '</a>';
                    $available_instances ++;
                }
            }

            if ($available_instances == 0)
            {
                $translation = Translation :: get(
                    'NoExternalInstanceTypeManagerAvailable',
                    array('TYPES' => $type_names),
                    RepositoryManager :: APPLICATION_NAME);
                return Display :: warning_message($translation, true);
            }
            else
            {
                $html[] = '<div style="margin-bottom: 10px;">' . implode(' ', $buttons) . '</div>';

                if ($available_instances == 1 && $auto_open)
                {
                    $html[] = '<script type="text/javascript">';
                    $html[] = '$(document).ready(function ()';
                    $html[] = '{';
                    // htmlspecialchars converts & to &amp; -> need oher solution
                    // $html[] = ' openPopup(\'' . htmlspecialchars($link) . '\');';
                    $html[] = '	openPopup(\'' . $link . '\');';
                    $html[] = '});';
                    $html[] = '</script>';
                }

                return implode("\n", $html);
            }
        }
    }
}
