<?php
namespace repository;

use common\libraries\Theme;
use common\libraries\Path;
use common\libraries\Filesystem;
use common\libraries\Request;
use common\libraries\StringUtilities;
use common\libraries\Session;
use common\libraries\FileProperties;
use common\libraries\Translation;
use common\libraries\Breadcrumb;
use common\libraries\BreadcrumbTrail;
use common\libraries\Application;
use common\libraries\Utilities;

/**
 * $Id: importer.class.php 204 2009-11-13 12:51:30Z kariboe $
 *
 * @package repository.lib.repository_manager.component
 */
class RepositoryManagerImporterComponent extends RepositoryManager
{

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $type = Request :: get(self :: PARAM_IMPORT_TYPE);

        if ($type)
        {
            $import_form = ContentObjectImportForm :: factory(
                $type,
                $this,
                'post',
                $this->get_url(array(self :: PARAM_IMPORT_TYPE => $type)));

            if ($import_form->validate())
            {
                $values = $import_form->exportValues();
                $parent_id = $values[ContentObject :: PROPERTY_PARENT_ID];
                $new_category_name = $values[ContentObjectImportForm :: NEW_CATEGORY];

                if (! StringUtilities :: is_null_or_empty($new_category_name, true))
                {
                    $new_category = new RepositoryCategory();
                    $new_category->set_name($new_category_name);
                    $new_category->set_parent($parent_id);
                    $new_category->set_user_id($this->get_user_id());
                    $new_category->set_type(RepositoryCategory :: TYPE_NORMAL);
                    if (! $new_category->create())
                    {
                        throw new \Exception(Translation :: get('CategoryCreationFailed'));
                    }
                    else
                    {
                        $category_id = $new_category->get_id();
                    }
                }
                else
                {
                    $category_id = $parent_id;
                }

                if (isset($_FILES[ContentObjectImportForm :: IMPORT_FILE_NAME]))
                {
                    $file = FileProperties :: from_upload($_FILES[ContentObjectImportForm :: IMPORT_FILE_NAME]);
                }
                else
                {
                    $file = null;
                }

                $parameters = ImportParameters :: factory(
                    $import_form->exportValue(ContentObjectImportForm :: PROPERTY_TYPE),
                    $this->get_user_id(),
                    $category_id,
                    $file,
                    $values);
                $controller = ContentObjectImportController :: factory($parameters);
                $controller->run();

                $messages = $controller->get_messages_for_url();

                Session :: register(self :: PARAM_MESSAGES, $messages);

                $parameters = array(Application :: PARAM_ACTION => RepositoryManager :: ACTION_BROWSE_CONTENT_OBJECTS);

                $this->simple_redirect($parameters);
            }
            else
            {
                BreadcrumbTrail :: get_instance()->add(
                    new Breadcrumb(
                        $this->get_url(),
                        Translation :: get(
                            'ImportType',
                            array(
                                'TYPE' => Translation :: get(
                                    'ImportType' . Utilities :: underscores_to_camelcase($type))))));

                $this->display_header();
                $import_form->display();
                $this->display_footer();
            }
        }
        else
        {
            BreadcrumbTrail :: get_instance()->add(
                new Breadcrumb($this->get_url(), Translation :: get('ChooseImportFormat')));

            $this->display_header();

            $html = array();
            foreach ($this->get_types() as $type => $name)
            {

                $html[] = '<a href="' . $this->get_url(array(self :: PARAM_IMPORT_TYPE => $type)) . '">';
                $html[] = '<div class="create_block" style="background-image: url(' . Theme :: get_image_path() .
                     'import/' . $type . '.png);">';
                $html[] = $name;
                $html[] = '</div>';
                $html[] = '</a>';
            }
            echo implode("\n", $html);

            $this->display_footer();
        }
    }

    public function get_types()
    {
        $types_folder = Path :: get_repository_path() . 'lib/content_object_import/';
        $folders = Filesystem :: get_directory_content($types_folder, Filesystem :: LIST_DIRECTORIES, false);
        $folder_types = array();
        $sort_types = array();

        foreach ($folders as $folder)
        {
            $class = __NAMESPACE__ . '\\' . Utilities :: underscores_to_camelcase($folder) .
                 'ContentObjectImportController';

            if (class_exists($class))
            {
                if ($class :: is_available())
                {
                    $folder_types[$folder] = Translation :: get('ImportType' . Utilities :: underscores_to_camelcase($folder));
                    $sort_types[$folder] = strtolower(Translation :: get('ImportType' . Utilities :: underscores_to_camelcase($folder)));
                }
            }
        }

        asort($sort_types);

        $types = array();

        foreach($sort_types as $key => $value)
        {
            $types[$key] = $folder_types[$key];
        }

        return $types;
    }
}
