<?php
namespace repository;

use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\Theme;
use common\libraries\Utilities;

class LinkBrowserTableCellRenderer extends DefaultLinkTableCellRenderer
{
    /**
     * The repository browser component
     */
    private $browser;

    /**
     * Constructor
     * @param RepositoryManagerBrowserComponent $browser
     */
    public function __construct($browser, $type)
    {
        parent :: __construct($browser, $type);
        $this->browser = $browser;
    }

    // Inherited
    public function render_cell($column, $object)
    {
        if ($column === LinkBrowserTableColumnModel :: get_modification_column())
        {
            return $this->get_modification_links($object);
        }

        return parent :: render_cell($column, $object);
    }

    private function get_modification_links($object)
    {
        $toolbar = new Toolbar();

        $link_id = $this->render_id_cell($object);

        if ($this->type == LinkBrowserTable :: TYPE_INCLUDES)
        {
            return '&nbsp';
        }

        if ($this->type == LinkBrowserTable :: TYPE_INCLUDED_IN)
        {
            return '&nbsp';
        }

        if($this->browser->is_allowed_to_modify())
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_delete.png',
                    $this->browser->get_delete_link_url($this->type, $this->browser->get_object()->get_id(), $link_id),
                    ToolbarItem :: DISPLAY_ICON,
                    true
                )
            );
        }

        return $toolbar->as_html();

    }
}
