<?php
namespace repository;

use common\libraries\WebApplication;
use common\libraries\Path;
use common\libraries\DelegateComponent;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\BreadcrumbTrail;
use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\ResourceManager;
use common\libraries\ActionBarRenderer;
use common\libraries\ToolbarItem;
use common\libraries\Theme;
use common\libraries\ComplexContentObjectSupport;
use common\libraries\Breadcrumb;
use common\libraries\DynamicTabsRenderer;
use common\libraries\DynamicContentTab;
use common\libraries\PlatformSetting;

/**
 * $Id: viewer.class.php 204 2009-11-13 12:51:30Z kariboe $
 *
 * @package repository.lib.repository_manager.component
 */

/**
 * Repository manager component which can be used to view a learning object.
 */
class RepositoryManagerViewerComponent extends RepositoryManager implements ContentObjectDisplayContext,
    DelegateComponent
{

    private $action_bar;

    private $object;

    private $tabs;

    /**
     * caching variable that checks whether or not the current user can modify this object
     *
     * @var bool
     */
    private $allowed_to_modify;

    /**
     * Runs this component and displays its output.
     */
    public function run()
    {
        $id = Request :: get(RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
        if ($id)
        {
            $renderer_name = Utilities :: get_classname_from_object($this, true);
            $this->tabs = new DynamicTabsRenderer($renderer_name);

            $object = DataManager :: retrieve_content_object($id);
            if (! $object)
            {
                $this->display_error_page(Translation :: get('NoObjectSelected', null, Utilities :: COMMON_LIBRARIES));
                exit();
            }

            $this->object = $object;

            if (! ($object->get_owner_id() == $this->get_user_id() || $object->get_owner_id() == 0 || RepositoryRights :: get_instance()->is_allowed_in_user_subtree(
                RepositoryRights :: VIEW_RIGHT,
                $object->get_id(),
                RepositoryRights :: TYPE_USER_CONTENT_OBJECT,
                $object->get_owner_id())))
            {
                $this->not_allowed();
            }

            $is_owner = $object->get_owner_id() == $this->get_user_id();

            $this->allowed_to_modify = $is_owner || RepositoryRights :: get_instance()->is_allowed_in_user_subtree(
                RepositoryRights :: COLLABORATE_RIGHT,
                $object->get_id(),
                RepositoryRights :: TYPE_USER_CONTENT_OBJECT,
                $object->get_owner_id());

            $display = ContentObjectRenditionImplementation :: factory(
                $object,
                ContentObjectRendition :: FORMAT_HTML,
                ContentObjectRendition :: VIEW_FULL,
                $this);
            $trail = BreadcrumbTrail :: get_instance();
            BreadcrumbTrail :: get_instance()->add(
                new Breadcrumb(
                    null,
                    Translation :: get(
                        'ViewContentObject',
                        array(
                            'CONTENT_OBJECT' => $object->get_title(),
                            'ICON' => Theme :: get_image(
                                'logo/16',
                                'png',
                                Translation :: get(
                                    'TypeName',
                                    null,
                                    Utilities :: get_namespace_from_classname($object->get_type())),
                                null,
                                ToolbarItem :: DISPLAY_ICON,
                                false,
                                Utilities :: get_namespace_from_classname($object->get_type()))),
                        RepositoryManager :: context())));

            if ($object->get_state() == ContentObject :: STATE_RECYCLED)
            {
                $trail->add(new Breadcrumb($this->get_recycle_bin_url(), Translation :: get('RecycleBin')));
                $this->force_menu_url($this->get_recycle_bin_url());
            }
            // $trail->add(new Breadcrumb($this->get_url(), $object->get_title()
            // . ($object->is_latest_version() ? '' : ' (' . Translation ::
            // get('OldVersion') . ')')));

            $version_data = array();
            $versions = $object->get_content_object_versions();

            $publication_attr = array();

            foreach ($object->get_content_object_versions() as $version)
            {
                // If this learning object is published somewhere in an
                // application, these locations are listed here.
                $publications = $this->get_content_object_publication_attributes(
                    $version->get_id(),
                    WebApplication :: ATTRIBUTES_TYPE_OBJECT);
                $publication_attr = array_merge($publication_attr, $publications);
            }

            $this->action_bar = $this->get_action_bar($object);

            if (count($versions) >= 2)
            {
                $this->display_header();

                if ($this->action_bar)
                {
                    echo '<br />' . $this->action_bar->as_html();
                }

                echo $display->render();

                $version_parameters = array(
                    RepositoryManager :: PARAM_APPLICATION => RepositoryManager :: APPLICATION_NAME,
                    RepositoryManager :: PARAM_CONTENT_OBJECT_ID => $this->object->get_id(),
                    RepositoryManager :: PARAM_ACTION => RepositoryManager :: ACTION_COMPARE_CONTENT_OBJECTS);

                $version_browser = new RepositoryVersionBrowserTable(
                    $this,
                    $version_parameters,
                    new EqualityCondition(ContentObject :: PROPERTY_OBJECT_NUMBER, $object->get_object_number()));

                $version_tab_content = array();
                $version_tab_content[] = $version_browser->as_html();
                $version_tab_content[] = ResourceManager :: get_instance()->get_resource_html(
                    Path :: namespace_to_full_path(RepositoryManager :: context(), true) .
                         'resources/javascript/repository.js');
                $this->tabs->add_tab(
                    new DynamicContentTab(
                        'versions',
                        Translation :: get('Versions'),
                        Theme :: get_image_path() . 'place_mini_versions.png',
                        implode("\n", $version_tab_content)));
            }
            elseif (count($publication_attr) > 0)
            {
                $this->display_header();

                if ($this->action_bar)
                {
                    echo '<br />' . $this->action_bar->as_html();
                }

                echo $display->render();
            }
            else
            {
                $this->display_header();

                if ($this->action_bar)
                {
                    echo '<br />' . $this->action_bar->as_html();
                }

                echo $display->render();
            }
            $this->add_links_to_content_object_tabs($object);
            echo $this->tabs->render();
            $this->display_footer();
        }
        else
        {
            $this->display_error_page(
                htmlentities(
                    Translation :: get(
                        'NoObjectSelected',
                        array('OBJECT' => Translation :: get('ContentObject')),
                        Utilities :: COMMON_LIBRARIES)));
        }
    }

    private function get_action_bar($object)
    {
        $user = $this->get_user_info($this->get_user_id());
        $is_owner = $object->get_owner_id() == $this->get_user_id();

        if ($this->is_allowed_to_modify())
        {
            $action_bar = new ActionBarRenderer(ActionBarRenderer :: TYPE_HORIZONTAL);
            if ($object->is_latest_version())
            {
                $edit_url = $this->get_content_object_editing_url($object);
            }
            if (isset($edit_url))
            {
                $recycle_url = $this->get_content_object_recycling_url($object);
                $in_recycle_bin = false;
                if (isset($recycle_url) && $is_owner)
                {
                    $action_bar->add_common_action(
                        new ToolbarItem(
                            Translation :: get('Remove', null, Utilities :: COMMON_LIBRARIES),
                            Theme :: get_common_image_path() . 'action_recycle_bin.png',
                            $recycle_url,
                            ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                            Translation :: get('ConfirmRemove', null, Utilities :: COMMON_LIBRARIES)));
                }
                else
                {
                    if ($is_owner)
                    {
                        $delete_url = $this->get_content_object_deletion_url($object);
                        if (isset($delete_url))
                        {
                            $recycle_bin_button = new ToolbarItem(
                                Translation :: get('Delete', null, Utilities :: COMMON_LIBRARIES),
                                Theme :: get_common_image_path() . 'action_delete.png',
                                $delete_url,
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                                Translation :: get('ConfirmDelete', null, Utilities :: COMMON_LIBRARIES));
                            $in_recycle_bin = true;
                        }
                        else
                        {
                            $recycle_bin_button = new ToolbarItem(
                                Translation :: get('Remove', null, Utilities :: COMMON_LIBRARIES),
                                Theme :: get_common_image_path() . 'action_recycle_bin_na.png');
                        }
                    }
                }

                if (! $in_recycle_bin)
                {
                    $delete_link_url = $this->get_content_object_unlinker_url($object);

                    if (! DataManager :: content_object_deletion_allowed($object) &&
                         $object->get_state() != ContentObject :: STATE_RECYCLED && $is_owner)
                    {
                        $force_delete_button = new ToolbarItem(
                            Translation :: get('Unlink', null, Utilities :: COMMON_LIBRARIES),
                            Theme :: get_common_image_path() . 'action_unlink.png',
                            $delete_link_url,
                            ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                            true);
                    }

                    $edit_url = $this->get_content_object_editing_url($object);
                    if (isset($edit_url))
                    {
                        $action_bar->add_common_action(
                            new ToolbarItem(
                                Translation :: get('Edit', null, Utilities :: COMMON_LIBRARIES),
                                Theme :: get_common_image_path() . 'action_edit.png',
                                $edit_url,
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL));
                    }
                    else
                    {
                        $action_bar->add_common_action(
                            new ToolbarItem(
                                Translation :: get('EditNotAvailable', null, Utilities :: COMMON_LIBRARIES),
                                Theme :: get_common_image_path() . 'action_edit_na.png'));
                    }

                    if (isset($recycle_bin_button) && $is_owner)
                    {
                        $action_bar->add_common_action($recycle_bin_button);
                    }

                    if (isset($force_delete_button) && $is_owner)
                    {
                        $action_bar->add_common_action($force_delete_button);
                    }

                    if (DataManager :: get_number_of_categories($user->get_id()) > 1 && $is_owner)
                    {
                        $move_url = $this->get_content_object_moving_url($object);
                        $action_bar->add_common_action(
                            new ToolbarItem(
                                Translation :: get('Move', null, Utilities :: COMMON_LIBRARIES),
                                Theme :: get_common_image_path() . 'action_move.png',
                                $move_url,
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL));
                    }

                    if ($is_owner)
                    {
                        $hide_sharing = PlatformSetting :: get('hide_sharing', __NAMESPACE__) === 1 ? true : false;
                        if(!$hide_sharing)
                        {
                            $action_bar->add_common_action(
                                new ToolbarItem(
                                    Translation :: get('Share', null, Utilities :: COMMON_LIBRARIES),
                                    Theme :: get_common_image_path() . 'action_share.png',
                                    $this->get_share_content_objects_url($object->get_id()),
                                    ToolbarItem :: DISPLAY_ICON_AND_LABEL));
                        }
                    }

                    if ($object instanceof ComplexContentObjectSupport)
                    {
                        $clo_url = $this->get_browse_complex_content_object_url($object);
                        $action_bar->add_common_action(
                            new ToolbarItem(
                                Translation :: get('BuildComplexObject', null, Utilities :: COMMON_LIBRARIES),
                                Theme :: get_common_image_path() . 'action_build.png',
                                $clo_url,
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL));

                        $preview_url = $this->get_preview_complex_content_object_url($object);
                        $onclick = '" onclick="javascript:openPopup(\'' . $preview_url . '\'); return false;';
                        $action_bar->add_common_action(
                            new ToolbarItem(
                                Translation :: get('Preview', null, Utilities :: COMMON_LIBRARIES),
                                Theme :: get_common_image_path() . 'action_preview.png',
                                $preview_url,
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                                false,
                                $onclick,
                                '_blank'));
                    }
                }
                else
                {
                    if ($is_owner)
                    {
                        $restore_url = $this->get_content_object_restoring_url($object);
                        $action_bar->add_common_action(
                            new ToolbarItem(
                                Translation :: get('Restore', null, Utilities :: COMMON_LIBRARIES),
                                Theme :: get_common_image_path() . 'action_restore.png',
                                $restore_url,
                                ToolbarItem :: DISPLAY_ICON_AND_LABEL,
                                true));
                        if (isset($recycle_bin_button))
                        {
                            $action_bar->add_common_action($recycle_bin_button);
                        }
                    }
                }
            }

            // TODO implement settings structure to allow templates: at the
            // moment quick fix to disallow non-admins
            if ($user->is_platform_admin())
            {
                $action_bar->add_tool_action(
                    new ToolbarItem(
                        Translation :: get('CopyToTemplates'),
                        Theme :: get_common_image_path() . 'export_template.png',
                        $this->get_copy_content_object_url($object->get_id(), 1)));
            }

            return $action_bar;
        }
    }

    public function add_links_to_content_object_tabs($content_object)
    {
        $renderer_name = Utilities :: get_classname_from_object($this, true);
        $tabs = new DynamicTabsRenderer($renderer_name);
        $parameters = array(
            RepositoryManager :: PARAM_APPLICATION => RepositoryManager :: APPLICATION_NAME,
            RepositoryManager :: PARAM_CONTENT_OBJECT_ID => $this->object->get_id(),
            RepositoryManager :: PARAM_ACTION => RepositoryManager :: ACTION_VIEW_CONTENT_OBJECTS);

        // EXTERNAL INSTANCES
        if ($content_object->is_external())
        {
            $parameters[DynamicTabsRenderer :: PARAM_SELECTED_TAB] = 'external_instances';
            $browser = new ExternalLinkBrowserTable($this, $parameters);
            $this->tabs->add_tab(
                new DynamicContentTab(
                    'external_instances',
                    Translation :: get('ExternalInstances'),
                    Theme :: get_image_path() . 'place_mini_external_instance.png',
                    $browser->as_html()));
        }

        // LINKS | PUBLICATIONS
        if ($content_object->has_publications())
        {
            $parameters[DynamicTabsRenderer :: PARAM_SELECTED_TAB] = LinkBrowserTable :: TYPE_PUBLICATIONS;
            $browser = new LinkBrowserTable($this, $parameters, null, LinkBrowserTable :: TYPE_PUBLICATIONS);
            $this->tabs->add_tab(
                new DynamicContentTab(
                    LinkBrowserTable :: TYPE_PUBLICATIONS,
                    Translation :: get('Publications'),
                    Theme :: get_image_path() . 'place_mini_publications.png',
                    $browser->as_html()));
        }
        // EXPORT
        $parameters[DynamicTabsRenderer :: PARAM_SELECTED_TAB] = 'export';
        $this->tabs->add_tab(
            new DynamicContentTab(
                'export',
                Translation :: get('Export'),
                Theme :: get_image_path() . 'place_mini_export.png',
                $this->get_export_types()));

        // LINKS | PARENTS
        if ($content_object->has_parents())
        {
            $parameters[DynamicTabsRenderer :: PARAM_SELECTED_TAB] = LinkBrowserTable :: TYPE_PARENTS;
            $browser = new LinkBrowserTable($this, $parameters, null, LinkBrowserTable :: TYPE_PARENTS);
            $this->tabs->add_tab(
                new DynamicContentTab(
                    LinkBrowserTable :: TYPE_PARENTS,
                    Translation :: get('UsedIn'),
                    Theme :: get_image_path() . 'place_mini_parents.png',
                    $browser->as_html()));
        }

        // LINKS | CHILDREN
        if ($content_object->has_children())
        {
            $parameters[DynamicTabsRenderer :: PARAM_SELECTED_TAB] = LinkBrowserTable :: TYPE_CHILDREN;
            $browser = new LinkBrowserTable($this, $parameters, null, LinkBrowserTable :: TYPE_CHILDREN);
            $this->tabs->add_tab(
                new DynamicContentTab(
                    LinkBrowserTable :: TYPE_CHILDREN,
                    Translation :: get('Uses'),
                    Theme :: get_image_path() . 'place_mini_children.png',
                    $browser->as_html()));
        }

        // LINKS | ATTACHED TO
        if ($content_object->has_attachers())
        {
            $parameters[DynamicTabsRenderer :: PARAM_SELECTED_TAB] = LinkBrowserTable :: TYPE_ATTACHED_TO;
            $browser = new LinkBrowserTable($this, $parameters, null, LinkBrowserTable :: TYPE_ATTACHED_TO);
            $this->tabs->add_tab(
                new DynamicContentTab(
                    LinkBrowserTable :: TYPE_ATTACHED_TO,
                    Translation :: get('AttachedTo'),
                    Theme :: get_image_path() . 'place_mini_attached_to.png',
                    $browser->as_html()));
        }

        // LINKS | ATTACHES
        if ($content_object->has_attachments())
        {
            $parameters[DynamicTabsRenderer :: PARAM_SELECTED_TAB] = LinkBrowserTable :: TYPE_ATTACHES;
            $browser = new LinkBrowserTable($this, $parameters, null, LinkBrowserTable :: TYPE_ATTACHES);
            $this->tabs->add_tab(
                new DynamicContentTab(
                    LinkBrowserTable :: TYPE_ATTACHES,
                    Translation :: get('Attaches'),
                    Theme :: get_image_path() . 'place_mini_attaches.png',
                    $browser->as_html()));
        }

        // LINKS | INCLUDED IN
        if ($content_object->has_includers())
        {
            $parameters[DynamicTabsRenderer :: PARAM_SELECTED_TAB] = LinkBrowserTable :: TYPE_INCLUDED_IN;
            $browser = new LinkBrowserTable($this, $parameters, null, LinkBrowserTable :: TYPE_INCLUDED_IN);
            $this->tabs->add_tab(
                new DynamicContentTab(
                    LinkBrowserTable :: TYPE_INCLUDED_IN,
                    Translation :: get('IncludedIn'),
                    Theme :: get_image_path() . 'place_mini_included_in.png',
                    $browser->as_html()));
        }

        // LINKS | INCLUDES
        if ($content_object->has_includes())
        {
            $parameters[DynamicTabsRenderer :: PARAM_SELECTED_TAB] = LinkBrowserTable :: TYPE_INCLUDES;
            $browser = new LinkBrowserTable($this, $parameters, null, LinkBrowserTable :: TYPE_INCLUDES);
            $this->tabs->add_tab(
                new DynamicContentTab(
                    LinkBrowserTable :: TYPE_INCLUDES,
                    Translation :: get('Includes'),
                    Theme :: get_image_path() . 'place_mini_includes.png',
                    $browser->as_html()));
        }
    }

    /**
     * Returns whether or not a user can change the links
     *
     * @return s bool
     */
    public function is_allowed_to_modify()
    {
        return $this->allowed_to_modify;
    }

    public function get_object()
    {
        return $this->object;
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        // $breadcrumbtrail->add(new Breadcrumb($this->get_url(array(
        // RepositoryManager :: PARAM_ACTION => RepositoryManager ::
        // ACTION_BROWSE_CONTENT_OBJECTS)), Translation ::
        // get('RepositoryManagerBrowserComponent')));
        $breadcrumbtrail->add_help('repository_viewer');
    }

    public function get_additional_parameters()
    {
        return array(RepositoryManager :: PARAM_CONTENT_OBJECT_ID);
    }

    public function get_export_types()
    {
        $types = ContentObjectExportImplementation :: get_types_for_object(
            Utilities :: get_namespace_from_object($this->object));

        $html = array();
        foreach ($types as $type)
        {
            $link = $this->get_content_object_exporting_url($this->object, $type);
            $html[] = '<a href="' . $link . '">';
            $url = Theme :: get_image_system_path(Utilities :: get_namespace_from_object($this->object)) . 'export/' .
                 $type . '.png';
            if (file_exists($url))
            {
                $html[] = '<div class="create_block" style="background-image : url(' .
                     Theme :: get_image_path(Utilities :: get_namespace_from_object($this->object)) . 'export/' . $type .
                     '.png); ">' . Translation :: get(
                        'ExportType' . Utilities :: underscores_to_camelcase($type),
                        null,
                        Utilities :: get_namespace_from_object($this->object)) . '</div>';
            }
            else
            {
                $html[] = '<div class="create_block" style="background-image : url(' . Theme :: get_image_path(
                    __NAMESPACE__) . 'export/' . $type . '.png); ">' . Translation :: get(
                    'ExportType' . Utilities :: underscores_to_camelcase($type)) . '</div>';
            }
            $html[] = '</a>';
        }
        return implode("\n", $html);
    }
}
