<?php
namespace repository;

/**
 * The combination of options available for the FormValidatorHtmlEditor Should be implemented for each specific editor
 * to translate the generic option values
 *
 * @author Scaramanga
 */
use common\libraries\Registration;
use repository\content_object\template\Template;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Session;
use common\libraries\CommonDataManager;
use common\libraries\Utilities;

abstract class FormValidatorHtmlEditorTemplates
{

    /**
     *
     * @param $options Array
     */
    public function __construct()
    {
    }

    public function get_templates()
    {
        $conditions = array();
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_OWNER_ID, Session :: get_user_id());
        $conditions[] = new EqualityCondition(ContentObject :: PROPERTY_TYPE, Template :: get_type_name());
        $condition = new AndCondition($conditions);

        return \repository\DataManager :: retrieve_active_content_objects(ContentObject :: class_name(), $condition);
    }

    public function template_object_exists()
    {
        $registration = CommonDataManager :: get_registration('repository\content_object\template');

        if ($registration instanceof Registration && $registration->is_active())
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     *
     * @param $type String
     * @param $options Array
     * @return FormValidatorHtmlEditorOptions
     */
    public static function factory($type)
    {
        $file = dirname(__FILE__) . '/html_editor_templates/' . $type . '_html_editor_templates.class.php';
        $class = 'FormValidator' . Utilities :: underscores_to_camelcase($type) . 'HtmlEditorTemplates';

        if (file_exists($file))
        {
            return new $class();
        }
        else
        {
            return null;
        }
    }
}
