<?php
namespace repository;

use common\libraries\OrCondition;
use common\libraries\PatternMatchCondition;
use repository\content_object\document\Document;
use repository\content_object\file\File;

/**
 * $Id: content_object_table_data_provider.class.php 191 2009-11-13 11:50:28Z chellee $
 *
 * @package application.common.repo_viewer.component.content_object_table
 */
/**
 * This class represents a data provider for a publication candidate table
 */
class HandbookItemContentObjectTableDataProvider extends ContentObjectTableDataProvider
{

    /**
     * Constructor.
     *
     * @param int $owner The user id of the current active user.
     * @param array $types The possible types of learning objects which can be selected.
     * @param string $query The search query.
     */
    public function __construct($owner, $types, $query = null, $parent)
    {
        parent :: __construct($owner, $types, $query, $parent);
    }

    /*
     * Inherited
     */
    public function get_objects($offset, $count, $order_property = null)
    {
        $order_property = $this->get_order_property($order_property);

        if (! $this->get_parent()->is_shared_object_browser())
        {
            // var_dump('1' . $dm->retrieve_type_content_objects(HandbookItem :: get_type_name(),
            // $this->get_condition(), $order_property, $offset, $count));
            return \repository\DataManager :: retrieve_active_content_objects(
                ContentObject :: class_name(),
                $this->get_condition());
        }
        else
        {
            // var_dump('2' . $dm->retrieve_shared_type_content_objects(HandbookItem :: get_type_name(),
            // $this->get_condition(), $offset, $count, $order_property));
            return \repository\DataManager :: retrieve_shared_content_objects(
                $this->get_condition(),
                $offset,
                $count,
                $order_property);
        }
    }

    /*
     * Inherited
     */
    public function get_object_count()
    {
        if (! $this->get_parent()->is_shared_object_browser())
        {
            return \repository\DataManager :: count_active_content_objects(
                ContentObject :: class_name(),
                $this->get_condition());
        }
        else
        {
            return \repository\DataManager :: count_shared_content_objects($this->get_condition());
        }
    }

    public function get_type_conditions()
    {
        $handbook_item_types = array();
        $handbook_item_conditions = array();
        foreach ($handbook_item_types as $handbook_item_type)
        {
            if (ContentObject :: is_available('repository\content_object\document\Document'))
            {
                $handbook_item_conditions[] = new PatternMatchCondition(
                    File :: PROPERTY_FILENAME,
                    '*.' . $handbook_item_type,
                    Document :: get_type_name());
            }
            else
            {
                $handbook_item_conditions[] = new PatternMatchCondition(
                    File :: PROPERTY_FILENAME,
                    '*.' . $handbook_item_type,
                    File :: get_type_name());
            }
        }

        return new OrCondition($handbook_item_conditions);
    }
}
