<?php
namespace repository\viewer;

use common\libraries\BreadcrumbTrail;
use common\libraries\ToolbarItem;
use common\libraries\Breadcrumb;
use common\libraries\DelegateComponent;
use common\libraries\Request;
use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\Utilities;
use repository\RepositoryManager;
use repository\ContentObjectForm;
use repository\ContentObjectTypeSelector;

class CreatorComponent extends Manager implements DelegateComponent
{

    public function run($params = array())
    {
        $content_object_id = Request :: get(self :: PARAM_EDIT_ID);

        if ($content_object_id)
        {
            $this->get_editing_form($content_object_id);
        }
        else
        {
            $types = $this->get_types();
            $type_selector = new ContentObjectTypeSelector($this, $this->get_types());

            if (count($types) > 1)
            {
                $type = ($type_selector->get_selection() ? $type_selector->get_selection() : Request :: get(
                        ContentObjectTypeSelector :: PARAM_CONTENT_OBJECT_TYPE));

                if ($type)
                {
                    $this->set_parameter(ContentObjectTypeSelector :: PARAM_CONTENT_OBJECT_TYPE, $type);
                    $this->get_creation_form($type);
                }
                else
                {
                    $this->display_header();
                    echo $type_selector->as_html();
                    $this->display_footer();
                }
            }
            else
            {
                $this->get_creation_form($types[0]);
            }
        }
    }

    /**
     *
     * @return string
     */
    public function get_type()
    {
        $types = $this->get_types();

        if (count($types) > 1)
        {
            $type = Request :: post(self :: PARAM_CONTENT_OBJECT_TYPE);
            if (! $type)
            {
                $type = Request :: get(self :: PARAM_CONTENT_OBJECT_TYPE);
            }

            return $type;
        }
        else
        {
            return $types[0];
        }
    }

    /**
     * Gets the form to select a content object type.
     */
    protected function get_type_selector()
    {
        $type_selector = new ContentObjectTypeSelector($this, $this->get_types());

        if ($type_selector->get_selection())
        {
            $this->get_creation_form($type_selector->get_selection());
        }
        else
        {
            echo $type_selector->as_html();
        }
    }

    /**
     *
     * @param string $type
     * @return string
     */
    public function get_content_object_type_creation_url($type)
    {
        $object_type_parameters = $this->get_parameters();
        $object_type_parameters[self :: PARAM_CONTENT_OBJECT_TYPE] = $type;
        return $this->get_url($object_type_parameters);
    }

    /**
     *
     * @param string $type
     */
    protected function get_creation_form($type)
    {
        BreadcrumbTrail :: get_instance()->add(
                new Breadcrumb(null,
                        Translation :: get('CreateContentType',
                                array(
                                        'OBJECTTYPE' => strtolower(
                                                Translation :: get('TypeName', null,
                                                        Utilities :: get_namespace_from_classname($type))),
                                        'ICON' => Theme :: get_image('logo/16', 'png',
                                                Translation :: get('TypeName', null,
                                                        Utilities :: get_namespace_from_classname($type)), null,
                                                ToolbarItem :: DISPLAY_ICON, false,
                                                Utilities :: get_namespace_from_classname($type))),
                                RepositoryManager :: context())));

        $default_object = new $type();
        $default_object->set_owner_id($this->get_user_id());

        $form = ContentObjectForm :: factory(ContentObjectForm :: TYPE_CREATE, $default_object, 'create', 'post',
                $this->get_url(array_merge(array(self :: PARAM_CONTENT_OBJECT_TYPE => $type), $this->get_parameters())));

        $this->handle_form($form, ContentObjectForm :: TYPE_CREATE);
    }

    /**
     *
     * @param int $content_object_id
     */
    protected function get_editing_form($content_object_id)
    {
        $content_object = \repository\DataManager :: retrieve_content_object($content_object_id);

        BreadcrumbTrail :: get_instance()->add(
                new Breadcrumb(null,
                        Translation :: get('UpdateContentType',
                                array(
                                        'OBJECTTYPE' => strtolower(
                                                Translation :: get('TypeName', null, $content_object->get_type())),
                                        'ICON' => Theme :: get_image('logo/16', 'png',
                                                Translation :: get('TypeName', null, $content_object->get_type()), null,
                                                ToolbarItem :: DISPLAY_ICON, false, $content_object->get_type())),
                                RepositoryManager :: context())));

        $form = ContentObjectForm :: factory(ContentObjectForm :: TYPE_EDIT, $content_object, 'edit', 'post',
                $this->get_url(array_merge($this->get_parameters(), array(self :: PARAM_EDIT_ID => $content_object_id))));

        $this->handle_form($form, ContentObjectForm :: TYPE_EDIT);
    }

    /*
     * Handles the displaying and validation of a create/edit content object form
     */
    protected function handle_form($form, $type = ContentObjectForm :: TYPE_CREATE)
    {
        if ($form->validate())
        {
            if ($type == ContentObjectForm :: TYPE_EDIT)
            {
                $form->update_content_object();
                $content_object = $form->get_content_object();
            }
            else
            {
                $content_object = $form->create_content_object();
            }

            if (! $content_object)
            {
                $redirect_params = array_merge($this->get_parameters(),
                        array(self :: PARAM_ACTION => self :: ACTION_CREATOR));
                $this->redirect(Translation :: get('ContentObjectNotCreated'), true, $redirect_params);
            }

            if (is_array($content_object))
            {
                $content_object_ids = array();
                foreach ($content_object as $object)
                {
                    $content_object_ids[] = $object->get_id();
                }
            }
            else
            {
                $content_object_ids = $content_object->get_id();
            }

            $redirect_parameters = array_merge($this->get_parameters(),
                    array(self :: PARAM_ACTION => self :: ACTION_PUBLISHER, self :: PARAM_ID => $content_object_ids));

            $this->redirect(null, false, $redirect_parameters);
        }
        else
        {
            $this->display_header();
            echo $form->toHtml();
            $this->display_footer();
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('repo_viewer_viewer');
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_EDIT_ID, self :: PARAM_CONTENT_OBJECT_TYPE);
    }
}
