<?php
namespace repository\viewer;

use repository\ContentObject;
use repository\RepositoryRights;
use common\libraries\ComplexContentObjectSupport;
use common\libraries\Utilities;
use common\libraries\Theme;
use common\libraries\Translation;
use common\libraries\ToolbarItem;
use common\libraries\Toolbar;
use common\libraries\InCondition;
use common\libraries\Session;
use common\libraries\DelegateComponent;
use common\libraries\NewObjectTableSupport;

class ImportedSelecterComponent extends Manager implements NewObjectTableSupport, DelegateComponent
{

    public function run()
    {
        $table = new ImportTable($this);

        $this->display_header();
        echo $table->as_html();
        $this->display_footer();
    }

    /**
     *
     * @param string $object_table_class_name
     * @return \common\libraries\Condition
     */
    public function get_object_table_condition($object_table_class_name)
    {
        $content_object_ids = Session :: retrieve(self :: PARAM_IMPORTED_CONTENT_OBJECT_IDS);
        return new InCondition(ContentObject :: PROPERTY_ID, $content_object_ids);
    }

    /**
     *
     * @param \repository\ContentObject $content_object
     * @return \common\libraries\Toolbar
     */
    public function get_default_browser_actions($content_object)
    {
        $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

        if ($content_object->has_right(RepositoryRights :: USE_RIGHT, $this->get_user_id()))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Publish', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_publish.png',
                    $this->get_url(
                        array_merge(
                            $this->get_parameters(),
                            array(
                                self :: PARAM_ACTION => self :: ACTION_PUBLISHER,
                                self :: PARAM_ID => $content_object->get_id())),
                        false),
                    ToolbarItem :: DISPLAY_ICON));
        }

        if ($content_object->has_right(RepositoryRights :: VIEW_RIGHT, $this->get_user_id()))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Preview'),
                    Theme :: get_common_image_path() . 'action_browser.png',
                    $this->get_url(
                        array_merge(
                            $this->get_parameters(),
                            array(
                                self :: PARAM_ACTION => self :: ACTION_VIEWER,
                                self :: PARAM_ID => $content_object->get_id())),
                        false),
                    ToolbarItem :: DISPLAY_ICON));
        }

        if ($content_object->has_right(RepositoryRights :: COLLABORATE_RIGHT, $this->get_user_id()))
        {
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('EditAndPublish'),
                    Theme :: get_common_image_path() . 'action_editpublish.png',
                    $this->get_url(
                        array_merge(
                            $this->get_parameters(),
                            array(
                                self :: PARAM_ACTION => self :: ACTION_CREATOR,
                                self :: PARAM_EDIT_ID => $content_object->get_id())),
                        false),
                    ToolbarItem :: DISPLAY_ICON));
        }

        if ($content_object instanceof ComplexContentObjectSupport)
        {

            $preview_url = \repository\RepositoryManager :: get_preview_complex_content_object_url($content_object);
            $onclick = '" onclick="javascript:openPopup(\'' . $preview_url . '\'); return false;';
            $toolbar->add_item(
                new ToolbarItem(
                    Translation :: get('Preview', null, Utilities :: COMMON_LIBRARIES),
                    Theme :: get_common_image_path() . 'action_preview.png',
                    $preview_url,
                    ToolbarItem :: DISPLAY_ICON,
                    false,
                    $onclick,
                    '_blank'));
        }

        return $toolbar;
    }
}
