<?php
namespace repository\viewer;

use repository\RepositoryRights;
use repository\ContentObjectRendition;
use repository\ContentObjectRenditionImplementation;
use common\libraries\Toolbar;
use common\libraries\BreadcrumbTrail;
use common\libraries\Breadcrumb;
use common\libraries\Translation;
use common\libraries\Theme;
use common\libraries\Request;
use common\libraries\ToolbarItem;
use common\libraries\Utilities;

class ViewerComponent extends Manager
{

    public function run()
    {
        if (Request :: get(self :: PARAM_ID))
        {
            $content_object = \repository\DataManager :: retrieve_content_object(Request :: get(self :: PARAM_ID));
            $toolbar = new Toolbar(Toolbar :: TYPE_HORIZONTAL);

            if ($content_object->has_right(RepositoryRights :: USE_RIGHT, $this->get_user_id()))
            {
                $toolbar->add_item(
                        new ToolbarItem(Translation :: get('Publish', null, Utilities :: COMMON_LIBRARIES),
                                Theme :: get_common_image_path() . 'action_publish.png',
                                $this->get_url(
                                        array_merge($this->get_parameters(),
                                                array(self :: PARAM_ACTION => self :: ACTION_PUBLISHER,
                                                        self :: PARAM_ID => $content_object->get_id())), false)));
            }

            if ($content_object->has_right(RepositoryRights :: COLLABORATE_RIGHT, $this->get_user_id()))
            {
                $toolbar->add_item(
                        new ToolbarItem(Translation :: get('EditAndPublish'),
                                Theme :: get_common_image_path() . 'action_editpublish.png',
                                $this->get_url(
                                        array_merge($this->get_parameters(),
                                                array(self :: PARAM_ACTION => self :: ACTION_CREATOR,
                                                        self :: PARAM_EDIT_ID => $content_object->get_id())))));
            }

            $this->display_header();

            echo ContentObjectRenditionImplementation :: launch($content_object, ContentObjectRendition :: FORMAT_HTML,
                    ContentObjectRendition :: VIEW_FULL, $this);
            echo $toolbar->as_html();
            echo '<div class="clear"></div>';
            $this->display_footer();
        }
    }

    public function add_additional_breadcrumbs(BreadcrumbTrail $breadcrumbtrail)
    {
        $breadcrumbtrail->add_help('repo_viewer_viewer');
        $breadcrumbtrail->add(
                new Breadcrumb($this->get_url(array(self :: PARAM_ACTION => self :: ACTION_BROWSER)),
                        Translation :: get('BrowserComponent')));
    }

    public function get_additional_parameters()
    {
        return array(self :: PARAM_ID);
    }
}
