<?php
namespace rights;

use group\Group;
use group\GroupRightsTemplate;
use group\DataManager as GroupDataManager;

use user\User;
use user\UserRightsTemplate;
use user\UserDataManager;

use common\libraries\Utilities;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\DataClass;
use common\libraries\InCondition;

/**
 * $Id: rights_template.class.php 214 2009-11-13 13:57:37Z vanpouckesven $
 * @package rights.lib
 * @author Hans de Bisschop
 */

class RightsTemplate extends DataClass
{
    const CLASS_NAME = __CLASS__;
    const PROPERTY_NAME = 'name';
    const PROPERTY_TYPE = 'type';
    const PROPERTY_USER_ID = 'user_id';
    const PROPERTY_DESCRIPTION = 'description';

    /**
     * Get the default properties of all users.
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_NAME, self :: PROPERTY_TYPE, 
                self :: PROPERTY_USER_ID, self :: PROPERTY_DESCRIPTION));
    }

    /**
     * @return RightsDataManagerInterface
     */
    public function get_data_manager()
    {
        return RightsDataManager :: get_instance();
    }

    public function get_user_id()
    {
        return $this->get_default_property(self :: PROPERTY_USER_ID);
    }

    public function get_name()
    {
        return $this->get_default_property(self :: PROPERTY_NAME);
    }

    public function get_type()
    {
        return $this->get_default_property(self :: PROPERTY_TYPE);
    }

    public function get_description()
    {
        return $this->get_default_property(self :: PROPERTY_DESCRIPTION);
    }

    public function set_user_id($user_id)
    {
        $this->set_default_property(self :: PROPERTY_USER_ID, $user_id);
    }

    public function set_name($name)
    {
        $this->set_default_property(self :: PROPERTY_NAME, $name);
    }

    public function set_type($type)
    {
        $this->set_default_property(self :: PROPERTY_TYPE, $type);
    }

    public function set_description($description)
    {
        $this->set_default_property(self :: PROPERTY_DESCRIPTION, $description);
    }

    public function get_users($user_condition, $offset = null, $max_objects = null, $order_by = null)
    {
        $udm = UserDataManager :: get_instance();
        $condition = new EqualityCondition(UserRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID, $this->get_id());
        
        $user_rights_templates = $udm->retrieve_user_rights_templates($condition);
        $user_ids = array();
        
        while ($user_rights_template = $user_rights_templates->next_result())
        {
            $user_ids[] = $user_rights_template->get_user_id();
        }
        
        $groups = $this->get_groups();
        if (isset($groups))
        {
            $group_user_ids = array();
            while ($group = $groups->next_result())
            {
                $group_user_ids = $group->get_users(true, true);
                foreach ($group_user_ids as $id)
                {
                    $user_ids[] = $id;
                }
            }
        }
        
        if (count($user_ids) > 0)
        {
            $conditions = array();
            $conditions[] = new InCondition(User :: PROPERTY_ID, $user_ids);
            if (isset($user_condition))
            {
                $conditions[] = $user_condition;
            }
            $condition = new AndCondition($conditions);
            return $udm->retrieve_users($condition, $offset, $max_objects, $order_by);
        }
        else
        {
            return null;
        }
    }

    public function get_groups()
    {
        $condition = new EqualityCondition(GroupRightsTemplate :: PROPERTY_RIGHTS_TEMPLATE_ID, $this->get_id());
        
        $group_rights_templates = GroupDataManager :: retrieves(GroupRightsTemplate :: class_name(), $condition);
        $group_ids = array();
        
        while ($group_rights_template = $group_rights_templates->next_result())
        {
            $group_ids[] = $group_rights_template->get_group_id();
        }
        
        if (count($group_ids) > 0)
        {
            $condition = new InCondition(Group :: PROPERTY_ID, $group_ids);
            return GroupDataManager :: retrieves(Group :: class_name(), $condition);
        }
        else
        {
            return null;
        }
    }

    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }
}
