<?php
namespace tracking\upgrade\v4_0_0;

use common\libraries\Translation;
use common\libraries\package\Package;
use common\libraries\CommonDataManager;

/**
 *
 * @author Hans De Bisschop
 * @author Magali Gillard
 */
class Upgrader extends \common\libraries\package\Upgrader
{

    /**
     * Constructor
     */
    public function __construct()
    {
        parent :: __construct(\tracking\TrackingDataManager :: get_instance());
    }

    /*
     * (non-PHPdoc) @see \common\libraries\package\Upgrader::run()
     */
    public function run()
    {
        $data_manager = $this->get_data_manager();
        $upgrader = $this;

        $success = $data_manager->transactional(
            function ($c) use($data_manager, $upgrader)
            {
                $package = Package :: get('tracking');
                $registration = CommonDataManager :: get_registration('tracking');
                $registration->set_version($package->get_version());
                if (! $registration->update())
                {
                    return false;
                }

                $attributes = array('type' => 'text', 'length' => '255', 'notnull' => 1);

                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \tracking\TrackerRegistration :: get_table_name(),
                    \tracking\TrackerRegistration :: PROPERTY_TRACKER,
                    $attributes);

                if (! $success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Action :: TYPE_ERROR,
                        Translation :: get_instance()->get('StorageUnitAlterFailed', null, 'core\install') . ': <em>' .
                             \tracking\TrackerRegistration :: get_table_name() . ' >> ' .
                             \tracking\TrackerRegistration :: PROPERTY_TRACKER . '</em>');
                    return false;
                }

                $attributes = array(
                    'name' => \tracking\TrackerRegistration :: PROPERTY_CONTEXT,
                    'type' => 'text',
                    'length' => '255',
                    'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \tracking\TrackerRegistration :: get_table_name(),
                    'application',
                    $attributes);

                if (! $success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Action :: TYPE_ERROR,
                        Translation :: get_instance()->get('StorageUnitAlterFailed', null, 'core\install') . ': <em>' .
                             \tracking\TrackerRegistration :: get_table_name() . ' >> application</em>');
                    return false;
                }

                $attributes = array('type' => 'text', 'length' => '255', 'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \tracking\Event :: get_table_name(),
                    \tracking\Event :: PROPERTY_NAME,
                    $attributes);

                if (! $success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Action :: TYPE_ERROR,
                        Translation :: get_instance()->get('StorageUnitAlterFailed', null, 'core\install') . ': <em>' .
                             \tracking\Event :: get_table_name() . ' >> ' . \tracking\Event :: PROPERTY_NAME . '</em>');
                    return false;
                }

                $attributes = array(
                    'name' => \tracking\Event :: PROPERTY_CONTEXT,
                    'type' => 'text',
                    'length' => '255',
                    'notnull' => 1);
                $success = $data_manager->alter_storage_unit(
                    \common\libraries\DataManager :: ALTER_STORAGE_UNIT_CHANGE,
                    \tracking\Event :: get_table_name(),
                    'block',
                    $attributes);

                if (! $success)
                {
                    $upgrader->add_message(
                        \common\libraries\package\Action :: TYPE_ERROR,
                        Translation :: get_instance()->get('StorageUnitAlterFailed', null, 'core\install') . ': <em>' .
                             \tracking\Event :: get_table_name() . ' >> block</em>');

                    return false;
                }

                return true;
            });

        return $success;
    }
}
