<?php
namespace user\integration\reporting;

use common\libraries\Translation;
use common\libraries\EqualityCondition;
use common\libraries\AndCondition;
use common\libraries\Utilities;
use reporting\ReportingData;

class LoginDayBlock extends Block
{

    public function count_data()
    {
        $reporting_data = new ReportingData();

        $conditions = array();
        $conditions[] = new EqualityCondition(\user\integration\tracking\LoginLogout :: PROPERTY_TYPE, 'login');
        $user_id = $this->get_user_id();
        if (isset($user_id))
        {
            $conditions[] = new EqualityCondition(\user\integration\tracking\LoginLogout :: PROPERTY_USER_ID, $user_id);
        }
        $condition = new AndCondition($conditions);

        $login_logout_tracker = new \user\integration\tracking\LoginLogout();
        $data = $login_logout_tracker->retrieve_tracker_items($condition);

        $days = array();
        foreach ($data as $date)
        {
            $day = date('N', $date->get_date());
            if (array_key_exists($day, $days))
            {
                $days[$day] ++;
            }
            else
            {
                $days[$day] = 1;
            }
        }
        $new_days = array();

        $day_names = array(
            Translation :: get('MondayLong', null, Utilities :: COMMON_LIBRARIES),
            Translation :: get('TuesdayLong', null, Utilities :: COMMON_LIBRARIES),
            Translation :: get('WednesdayLong', null, Utilities :: COMMON_LIBRARIES),
            Translation :: get('ThursdayLong', null, Utilities :: COMMON_LIBRARIES),
            Translation :: get('FridayLong', null, Utilities :: COMMON_LIBRARIES),
            Translation :: get('SaturdayLong', null, Utilities :: COMMON_LIBRARIES),
            Translation :: get('SundayLong', null, Utilities :: COMMON_LIBRARIES));

        $reporting_data->set_categories($day_names);
        $reporting_data->set_rows(array(Translation :: get('Logins')));

        foreach ($day_names as $key => $name)
        {
            $reporting_data->add_data_category_row(
                $name,
                Translation :: get('Logins'),
                ($days[$key + 1] ? $days[$key + 1] : 0));
        }
        return $reporting_data;
    }

    public function is_sortable()
    {
        return true;
    }

    public function retrieve_data()
    {
        return $this->count_data();
    }

    public function get_application()
    {
        return \user\UserManager :: APPLICATION_NAME;
    }

    public function get_views()
    {
        return array(
            \reporting\viewer\HtmlBlockRendition :: VIEW_TABLE,
            \reporting\viewer\HtmlBlockRendition :: VIEW_STACKED_AREA,
            \reporting\viewer\HtmlBlockRendition :: VIEW_STACKED_BAR,
            \reporting\viewer\HtmlBlockRendition :: VIEW_RADAR,
            \reporting\viewer\HtmlBlockRendition :: VIEW_POLAR,
            \reporting\viewer\HtmlBlockRendition :: VIEW_3D_PIE,
            \reporting\viewer\HtmlBlockRendition :: VIEW_PIE,
            \reporting\viewer\HtmlBlockRendition :: VIEW_RING,
            \reporting\viewer\HtmlBlockRendition :: VIEW_BAR,
            \reporting\viewer\HtmlBlockRendition :: VIEW_LINE,
            \reporting\viewer\HtmlBlockRendition :: VIEW_AREA,
            \reporting\viewer\HtmlBlockRendition :: VIEW_CSV,
            \reporting\viewer\HtmlBlockRendition :: VIEW_XLSX,
            \reporting\viewer\HtmlBlockRendition :: VIEW_ODS,
            \reporting\viewer\HtmlBlockRendition :: VIEW_XML);
    }
}
