<?php
namespace user\integration\tracking;

use common\libraries\EqualityCondition;

class Country extends User
{

    public function validate_parameters(array $parameters = array())
    {
        $server = $parameters['server'];
        $hostname = $this->get_host($server['REMOTE_ADDR']);

        $country = $this->extract_country($hostname);

        $this->set_type(self :: TYPE_COUNTRY);
        $this->set_name($country);
    }

    public function empty_tracker($event)
    {
        $condition = new EqualityCondition(self :: PROPERTY_TYPE, self :: TYPE_COUNTRY);
        return $this->remove($condition);
    }

    public function export($start_date, $end_date, $event)
    {
        $condition = new EqualityCondition(self :: PROPERTY_TYPE, self :: TYPE_COUNTRY);
        return $this->get_data_manager()->retrieve_tracker_items($this->get_table_name(), $condition);
    }

    /**
     * Extracts the country code from the remote host
     *
     * @param Remote Host $remhost instance of $_SERVER['REMOTE_ADDR']
     * @return string country code
     */
    public function extract_country($remhost)
    {
        if ($remhost == "Unknown")
        {
            return $remhost;
        }

        // country code is the last value of remote host
        $explodedRemhost = explode(".", $remhost);
        $code = $explodedRemhost[sizeof($explodedRemhost) - 1];

        if ($code == 'localhost')
        {
            return "Unknown";
        }
        else
        {
            return $code;
        }
    }

    public function get_host($ip)
    {
        $ptr = implode(".", array_reverse(explode(".", $ip))) . ".in-addr.arpa";
        $host = dns_get_record($ptr, DNS_PTR);
        if ($host == null)
        {
            return $ip;
        }
        else
        {
            return $host[0]['target'];
        }
    }
}
