<?php
namespace user\integration\tracking;

use common\libraries\EqualityCondition;

class Provider extends User
{

    public function validate_parameters(array $parameters = array())
    {
        $server = $parameters['server'];
        $hostname = $this->get_host($server['REMOTE_ADDR']);
        $provider = $this->extract_provider($hostname);

        $this->set_type(self :: TYPE_PROVIDER);
        $this->set_name($provider);
    }

    public function empty_tracker($event)
    {
        $condition = new EqualityCondition(self :: PROPERTY_TYPE, self :: TYPE_PROVIDER);
        return $this->remove($condition);
    }

    public function export($start_date, $end_date, $event)
    {
        $condition = new EqualityCondition(self :: PROPERTY_TYPE, self :: TYPE_PROVIDER);
        return $this->get_data_manager()->retrieve_tracker_items($this->get_table_name(), $condition);
    }

    /**
     * Extracts a provider from a given hostname
     *
     * @param $remhost string The remote hostname
     * @return the provider
     */
    public function extract_provider($remhost)
    {
        if ($remhost == "Unknown")
        {
            return $remhost;
        }

        $explodedRemhost = explode(".", $remhost);
        $provider = $explodedRemhost[sizeof($explodedRemhost) - 2] . "." . $explodedRemhost[sizeof($explodedRemhost) - 1];

        if ($provider == "co.uk" || $provider == "co.jp")
        {
            return $explodedRemhost[sizeof($explodedRemhost) - 3] . $provider;
        }
        else
        {
            return $provider;
        }
    }

    public function get_host($ip)
    {
        $ptr = implode(".", array_reverse(explode(".", $ip))) . ".in-addr.arpa";
        $host = dns_get_record($ptr, DNS_PTR);
        if ($host == null)
        {
            return $ip;
        }
        else
        {
            return $host[0]['target'];
        }
    }
}
