<?php
namespace webservice;

use common\libraries\Utilities;
use common\libraries\DataClass;
/**
 * $Id: webservice_registration.class.php 208 2009-11-13 13:14:39Z vanpouckesven $
 * @package webservices.lib
 */

class WebserviceRegistration extends DataClass
{
    const CLASS_NAME = __CLASS__;

    const PROPERTY_NAME = 'name';
    const PROPERTY_DESCRIPTION = 'description';
    const PROPERTY_ACTIVE = 'active';
    const PROPERTY_CODE = 'code';
    const PROPERTY_CATEGORY = 'category_id';
    const PROPERTY_AUTHENTICATION = 'authentication';

    /**
     * Get the default properties of all webservices.
     * @return array The property names.
     */
    public static function get_default_property_names()
    {
        return parent :: get_default_property_names(array(self :: PROPERTY_NAME, self :: PROPERTY_DESCRIPTION,
                self :: PROPERTY_CATEGORY, self :: PROPERTY_ACTIVE, self :: PROPERTY_CODE,
                self :: PROPERTY_AUTHENTICATION));
    }

    /**
     * inherited
     */
    public function get_data_manager()
    {
        return WebserviceDataManager :: get_instance();
    }

    /**
     * Returns the code of this webservice.
     */
    public function get_code()
    {
        return $this->get_default_property(self :: PROPERTY_CODE);
    }

    /**
     * Returns the name of this webservice.
     * @return String The name
     */
    public function get_name()
    {
        return $this->get_default_property(self :: PROPERTY_NAME);
    }

    /**
     * Returns the description of this webservice.
     * @return String The description
     */
    public function get_description()
    {
        return $this->get_default_property(self :: PROPERTY_DESCRIPTION);
    }

    /**
     * Returns the category
     * @return int
     */
    public function get_category()
    {
        return $this->get_default_property(self :: PROPERTY_CATEGORY);
    }

    /**
     * Returns whether the webservice is active
     * @return boolean
     */
    public function get_active()
    {
        return $this->get_default_property(self :: PROPERTY_ACTIVE);
    }

    /**
     * Returns whether the webservice needs authentication
     * @return boolean
     */
    public function get_authentication()
    {
        return $this->get_default_property(self :: PROPERTY_AUTHENTICATION);
    }

    /**
     * Sets the category
     * @param int $category
     */
    public function set_category($category)
    {
        $this->set_default_property(self :: PROPERTY_CATEGORY, $category);
    }

    /**
     * Sets the code of this webservice.
     */
    public function set_code($code)
    {
        $this->set_default_property(self :: PROPERTY_CODE, $code);
    }

    /**
     * Sets the name of this webservice.
     * @param String $name the name.
     */
    public function set_name($name)
    {
        $this->set_default_property(self :: PROPERTY_NAME, $name);
    }

    /**
     * Sets the description of this webservice.
     * @param String $description the description.
     */
    public function set_description($description)
    {
        $this->set_default_property(self :: PROPERTY_DESCRIPTION, $description);
    }

    /**
     * Sets whether the webservice is active
     * @param boolen $active
     */
    public function set_active($active)
    {
        $this->set_default_property(self :: PROPERTY_ACTIVE, $active);
    }

    /**
     * Sets whether the webservice needs authentication
     * @param boolean $authentication
     */
    public function set_authentication($authentication)
    {
        $this->set_default_property(self :: PROPERTY_AUTHENTICATION, $authentication);
    }

    /**
     * Returns the table name
     * @return String
     */
    public static function get_table_name()
    {
        return Utilities :: get_classname_from_namespace(self :: CLASS_NAME, true);
    }

    /**
     * Creates the registration + a rights location in the database
     * @return boolean
     */
    public function create()
    {
        $wdm = WebserviceDataManager :: get_instance();
        return $wdm->create_webservice($this);
    }

}
